@@ -3,96 +3,16 @@
 # Use of this source code is governed by a BSD-style license that can be
 # found in the LICENSE file.
 
-import glob
-import json
-import os
 import re
 import subprocess
-import sys
 import unittest
 
 import PRESUBMIT
-
+from PRESUBMIT_test_mocks import MockChange, MockFile, MockAffectedFile
+from PRESUBMIT_test_mocks import MockInputApi, MockOutputApi
 
 _TEST_DATA_DIR = 'base/test/data/presubmit'
 
-
-class MockInputApi(object):
-  def __init__(self):
-    self.json = json
-    self.re = re
-    self.os_path = os.path
-    self.python_executable = sys.executable
-    self.subprocess = subprocess
-    self.files = []
-    self.is_committing = False
-
-  def AffectedFiles(self, file_filter=None):
-    return self.files
-
-  def PresubmitLocalPath(self):
-    return os.path.dirname(__file__)
-
-  def ReadFile(self, filename, mode='rU'):
-    for file_ in self.files:
-      if file_.LocalPath() == filename:
-        return '\n'.join(file_.NewContents())
-    # Otherwise, file is not in our mock API.
-    raise IOError, "No such file or directory: '%s'" % filename
-
-
-class MockOutputApi(object):
-  class PresubmitResult(object):
-    def __init__(self, message, items=None, long_text=''):
-      self.message = message
-      self.items = items
-      self.long_text = long_text
-
-  class PresubmitError(PresubmitResult):
-    def __init__(self, message, items, long_text=''):
-      MockOutputApi.PresubmitResult.__init__(self, message, items, long_text)
-      self.type = 'error'
-
-  class PresubmitPromptWarning(PresubmitResult):
-    def __init__(self, message, items, long_text=''):
-      MockOutputApi.PresubmitResult.__init__(self, message, items, long_text)
-      self.type = 'warning'
-
-  class PresubmitNotifyResult(PresubmitResult):
-    def __init__(self, message, items, long_text=''):
-      MockOutputApi.PresubmitResult.__init__(self, message, items, long_text)
-      self.type = 'notify'
-
-  class PresubmitPromptOrNotify(PresubmitResult):
-    def __init__(self, message, items, long_text=''):
-      MockOutputApi.PresubmitResult.__init__(self, message, items, long_text)
-      self.type = 'promptOrNotify'
-
-
-class MockFile(object):
-  def __init__(self, local_path, new_contents):
-    self._local_path = local_path
-    self._new_contents = new_contents
-    self._changed_contents = [(i + 1, l) for i, l in enumerate(new_contents)]
-
-  def ChangedContents(self):
-    return self._changed_contents
-
-  def NewContents(self):
-    return self._new_contents
-
-  def LocalPath(self):
-    return self._local_path
-
-
-class MockChange(object):
-  def __init__(self, changed_files):
-    self._changed_files = changed_files
-
-  def LocalPaths(self):
-    return self._changed_files
-
-
 class IncludeOrderTest(unittest.TestCase):
   def testSystemHeaderOrder(self):
     scope = [(1, '#include <csystem.h>'),
@@ -346,6 +266,83 @@ class VersionControlConflictsTest(unittest.TestCase):
     self.assertTrue('3' in errors[1])
     self.assertTrue('5' in errors[2])
 
+  def testIgnoresReadmes(self):
+    lines = ['A First Level Header',
+             '====================',
+             '',
+             'A Second Level Header',
+             '---------------------']
+    errors = PRESUBMIT._CheckForVersionControlConflictsInFile(
+        MockInputApi(), MockFile('some/polymer/README.md', lines))
+    self.assertEqual(0, len(errors))
+
+class UmaHistogramChangeMatchedOrNotTest(unittest.TestCase):
+  def testTypicalCorrectlyMatchedChange(self):
+    diff_cc = ['UMA_HISTOGRAM_BOOL("Bla.Foo.Dummy", true)']
+    diff_xml = ['<histogram name="Bla.Foo.Dummy"> </histogram>']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [
+      MockFile('some/path/foo.cc', diff_cc),
+      MockFile('tools/metrics/histograms/histograms.xml', diff_xml),
+    ]
+    warnings = PRESUBMIT._CheckUmaHistogramChanges(mock_input_api,
+                                                   MockOutputApi())
+    self.assertEqual(0, len(warnings))
+
+  def testTypicalNotMatchedChange(self):
+    diff_cc = ['UMA_HISTOGRAM_BOOL("Bla.Foo.Dummy", true)']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [MockFile('some/path/foo.cc', diff_cc)]
+    warnings = PRESUBMIT._CheckUmaHistogramChanges(mock_input_api,
+                                                   MockOutputApi())
+    self.assertEqual(1, len(warnings))
+    self.assertEqual('warning', warnings[0].type)
+
+  def testTypicalNotMatchedChangeViaSuffixes(self):
+    diff_cc = ['UMA_HISTOGRAM_BOOL("Bla.Foo.Dummy", true)']
+    diff_xml = ['<histogram_suffixes name="SuperHistogram">',
+                '  <suffix name="Dummy"/>',
+                '  <affected-histogram name="Snafu.Dummy"/>',
+                '</histogram>']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [
+      MockFile('some/path/foo.cc', diff_cc),
+      MockFile('tools/metrics/histograms/histograms.xml', diff_xml),
+    ]
+    warnings = PRESUBMIT._CheckUmaHistogramChanges(mock_input_api,
+                                                   MockOutputApi())
+    self.assertEqual(1, len(warnings))
+    self.assertEqual('warning', warnings[0].type)
+
+  def testTypicalCorrectlyMatchedChangeViaSuffixes(self):
+    diff_cc = ['UMA_HISTOGRAM_BOOL("Bla.Foo.Dummy", true)']
+    diff_xml = ['<histogram_suffixes name="SuperHistogram">',
+                '  <suffix name="Dummy"/>',
+                '  <affected-histogram name="Bla.Foo"/>',
+                '</histogram>']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [
+      MockFile('some/path/foo.cc', diff_cc),
+      MockFile('tools/metrics/histograms/histograms.xml', diff_xml),
+    ]
+    warnings = PRESUBMIT._CheckUmaHistogramChanges(mock_input_api,
+                                                   MockOutputApi())
+    self.assertEqual(0, len(warnings))
+
+  def testTypicalCorrectlyMatchedChangeViaSuffixesWithSeparator(self):
+    diff_cc = ['UMA_HISTOGRAM_BOOL("Snafu_Dummy", true)']
+    diff_xml = ['<histogram_suffixes name="SuperHistogram" separator="_">',
+                '  <suffix name="Dummy"/>',
+                '  <affected-histogram name="Snafu"/>',
+                '</histogram>']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [
+      MockFile('some/path/foo.cc', diff_cc),
+      MockFile('tools/metrics/histograms/histograms.xml', diff_xml),
+    ]
+    warnings = PRESUBMIT._CheckUmaHistogramChanges(mock_input_api,
+                                                   MockOutputApi())
+    self.assertEqual(0, len(warnings))
 
 class BadExtensionsTest(unittest.TestCase):
   def testBadRejFile(self):
@@ -384,13 +381,49 @@ class BadExtensionsTest(unittest.TestCase):
     results = PRESUBMIT._CheckPatchFiles(mock_input_api, MockOutputApi())
     self.assertEqual(0, len(results))
 
-  def testOnlyOwnersFiles(self):
-    mock_change = MockChange([
-      'some/path/OWNERS',
-      'A\Windows\Path\OWNERS',
-    ])
-    results = PRESUBMIT.GetPreferredTryMasters(None, mock_change)
-    self.assertEqual({}, results)
+
+class CheckSingletonInHeadersTest(unittest.TestCase):
+  def testSingletonInArbitraryHeader(self):
+    diff_singleton_h = ['base::subtle::AtomicWord '
+                        'base::Singleton<Type, Traits, DifferentiatingType>::']
+    diff_foo_h = ['// base::Singleton<Foo> in comment.',
+                  'friend class base::Singleton<Foo>']
+    diff_foo2_h = ['  //Foo* bar = base::Singleton<Foo>::get();']
+    diff_bad_h = ['Foo* foo = base::Singleton<Foo>::get();']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [MockAffectedFile('base/memory/singleton.h',
+                                     diff_singleton_h),
+                            MockAffectedFile('foo.h', diff_foo_h),
+                            MockAffectedFile('foo2.h', diff_foo2_h),
+                            MockAffectedFile('bad.h', diff_bad_h)]
+    warnings = PRESUBMIT._CheckSingletonInHeaders(mock_input_api,
+                                                  MockOutputApi())
+    self.assertEqual(1, len(warnings))
+    self.assertEqual(2, len(warnings[0].items))
+    self.assertEqual('error', warnings[0].type)
+    self.assertTrue('Found base::Singleton<T>' in warnings[0].message)
+
+  def testSingletonInCC(self):
+    diff_cc = ['Foo* foo = base::Singleton<Foo>::get();']
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [MockAffectedFile('some/path/foo.cc', diff_cc)]
+    warnings = PRESUBMIT._CheckSingletonInHeaders(mock_input_api,
+                                                  MockOutputApi())
+    self.assertEqual(0, len(warnings))
+
+
+class CheckNoDeprecatedCompiledResourcesGYPTest(unittest.TestCase):
+  def testNoDeprecatedCompiledResourcsGYP(self):
+    mock_input_api = MockInputApi()
+    mock_input_api.files = [MockFile('some/js/compiled_resources.gyp', [])]
+    errors = PRESUBMIT._CheckNoDeprecatedCompiledResourcesGYP(mock_input_api,
+                                                              MockOutputApi())
+    self.assertEquals(1, len(errors))
+
+    mock_input_api.files = [MockFile('some/js/compiled_resources2.gyp', [])]
+    errors = PRESUBMIT._CheckNoDeprecatedCompiledResourcesGYP(mock_input_api,
+                                                              MockOutputApi())
+    self.assertEquals(0, len(errors))
 
 
 class InvalidOSMacroNamesTest(unittest.TestCase):
@@ -443,7 +476,7 @@ class CheckAddedDepsHaveTetsApprovalsTest(unittest.TestCase):
       '"+chrome/plugin/chrome_content_plugin_client.h",',
       '"+chrome/utility/chrome_content_utility_client.h",',
       '"+chromeos/chromeos_paths.h",',
-      '"+components/crash",',
+      '"+components/crash/content",',
       '"+components/nacl/common",',
       '"+content/public/browser/render_process_host.h",',
       '"+jni/fooblat.h",',
@@ -462,7 +495,7 @@ class CheckAddedDepsHaveTetsApprovalsTest(unittest.TestCase):
       'chrome/plugin/chrome_content_plugin_client.h',
       'chrome/utility/chrome_content_utility_client.h',
       'chromeos/chromeos_paths.h',
-      'components/crash/DEPS',
+      'components/crash/content/DEPS',
       'components/nacl/common/DEPS',
       'content/public/browser/render_process_host.h',
       'policy/DEPS',
@@ -673,13 +706,29 @@ class IDLParsingTest(unittest.TestCase):
 class TryServerMasterTest(unittest.TestCase):
   def testTryServerMasters(self):
     bots = {
-        'tryserver.chromium.gpu': [
-            'mac_gpu',
-            'mac_gpu_triggered_tests',
-            'linux_gpu',
-            'linux_gpu_triggered_tests',
-            'win_gpu',
-            'win_gpu_triggered_tests',
+        'tryserver.chromium.android': [
+            'android_archive_rel_ng',
+            'android_arm64_dbg_recipe',
+            'android_blink_rel',
+            'android_chromium_variable',
+            'android_chromium_variable_archive',
+            'android_chromium_variable_arm64',
+            'android_chromium_variable_cast_shell',
+            'android_chromium_variable_clang',
+            'android_chromium_variable_gn',
+            'android_chromium_variable_nexus4',
+            'android_clang_dbg_recipe',
+            'android_compile_dbg',
+            'android_compile_mips_dbg',
+            'android_compile_rel',
+            'android_compile_x64_dbg',
+            'android_compile_x86_dbg',
+            'android_coverage',
+            'android_cronet_tester'
+            'android_swarming_rel',
+            'cast_shell_android',
+            'linux_android_dbg_ng',
+            'linux_android_rel_ng',
         ],
         'tryserver.chromium.mac': [
             'ios_dbg_simulator',
@@ -694,27 +743,10 @@ class TryServerMasterTest(unittest.TestCase):
             'mac_nacl_sdk',
             'mac_nacl_sdk_build',
             'mac_rel_naclmore',
-            'mac_valgrind',
             'mac_x64_rel',
             'mac_xcodebuild',
         ],
         'tryserver.chromium.linux': [
-            'android_aosp',
-            'android_chromium_gn_compile_dbg',
-            'android_chromium_gn_compile_rel',
-            'android_clang_dbg',
-            'android_dbg',
-            'android_dbg_recipe',
-            'android_dbg_triggered_tests',
-            'android_dbg_triggered_tests_recipe',
-            'android_fyi_dbg',
-            'android_fyi_dbg_triggered_tests',
-            'android_rel',
-            'android_rel_triggered_tests',
-            'android_x86_dbg',
-            'blink_android_compile_dbg',
-            'blink_android_compile_rel',
-            'blink_presubmit',
             'chromium_presubmit',
             'linux_arm_cross_compile',
             'linux_arm_tester',
@@ -773,5 +805,291 @@ class TryServerMasterTest(unittest.TestCase):
             bot, master, PRESUBMIT.GetTryServerMasterForBot(bot)))
 
 
+class UserMetricsActionTest(unittest.TestCase):
+  def testUserMetricsActionInActions(self):
+    input_api = MockInputApi()
+    file_with_user_action = 'file_with_user_action.cc'
+    contents_with_user_action = [
+      'base::UserMetricsAction("AboutChrome")'
+    ]
+
+    input_api.files = [MockFile(file_with_user_action,
+                                contents_with_user_action)]
+
+    self.assertEqual(
+      [], PRESUBMIT._CheckUserActionUpdate(input_api, MockOutputApi()))
+
+
+  def testUserMetricsActionNotAddedToActions(self):
+    input_api = MockInputApi()
+    file_with_user_action = 'file_with_user_action.cc'
+    contents_with_user_action = [
+      'base::UserMetricsAction("NotInActionsXml")'
+    ]
+
+    input_api.files = [MockFile(file_with_user_action,
+                                contents_with_user_action)]
+
+    output = PRESUBMIT._CheckUserActionUpdate(input_api, MockOutputApi())
+    self.assertEqual(
+      ('File %s line %d: %s is missing in '
+       'tools/metrics/actions/actions.xml. Please run '
+       'tools/metrics/actions/extract_actions.py to update.'
+       % (file_with_user_action, 1, 'NotInActionsXml')),
+      output[0].message)
+
+
+class PydepsNeedsUpdatingTest(unittest.TestCase):
+
+  class MockSubprocess(object):
+    CalledProcessError = subprocess.CalledProcessError
+
+  def setUp(self):
+    mock_all_pydeps = ['A.pydeps', 'B.pydeps']
+    self.old_ALL_PYDEPS_FILES = PRESUBMIT._ALL_PYDEPS_FILES
+    PRESUBMIT._ALL_PYDEPS_FILES = mock_all_pydeps
+    self.mock_input_api = MockInputApi()
+    self.mock_output_api = MockOutputApi()
+    self.mock_input_api.subprocess = PydepsNeedsUpdatingTest.MockSubprocess()
+    self.checker = PRESUBMIT.PydepsChecker(self.mock_input_api, mock_all_pydeps)
+    self.checker._file_cache = {
+        'A.pydeps': '# Generated by:\n# CMD A\nA.py\nC.py\n',
+        'B.pydeps': '# Generated by:\n# CMD B\nB.py\nC.py\n',
+    }
+
+  def tearDown(self):
+    PRESUBMIT._ALL_PYDEPS_FILES = self.old_ALL_PYDEPS_FILES
+
+  def _RunCheck(self):
+    return PRESUBMIT._CheckPydepsNeedsUpdating(self.mock_input_api,
+                                               self.mock_output_api,
+                                               checker_for_tests=self.checker)
+
+  def testAddedPydep(self):
+    self.mock_input_api.files = [
+      MockAffectedFile('new.pydeps', [], action='A'),
+    ]
+
+    results = self._RunCheck()
+    self.assertEqual(1, len(results))
+    self.assertTrue('PYDEPS_FILES' in str(results[0]))
+
+  def testRemovedPydep(self):
+    self.mock_input_api.files = [
+      MockAffectedFile(PRESUBMIT._ALL_PYDEPS_FILES[0], [], action='D'),
+    ]
+
+    results = self._RunCheck()
+    self.assertEqual(1, len(results))
+    self.assertTrue('PYDEPS_FILES' in str(results[0]))
+
+  def testRandomPyIgnored(self):
+    self.mock_input_api.files = [
+      MockAffectedFile('random.py', []),
+    ]
+
+    results = self._RunCheck()
+    self.assertEqual(0, len(results), 'Unexpected results: %r' % results)
+
+  def testRelevantPyNoChange(self):
+    self.mock_input_api.files = [
+      MockAffectedFile('A.py', []),
+    ]
+
+    def mock_check_output(cmd, shell=False):
+      self.assertEqual('CMD A --output ""', cmd)
+      return self.checker._file_cache['A.pydeps']
+
+    self.mock_input_api.subprocess.check_output = mock_check_output
+
+    results = self._RunCheck()
+    self.assertEqual(0, len(results), 'Unexpected results: %r' % results)
+
+  def testRelevantPyOneChange(self):
+    self.mock_input_api.files = [
+      MockAffectedFile('A.py', []),
+    ]
+
+    def mock_check_output(cmd, shell=False):
+      self.assertEqual('CMD A --output ""', cmd)
+      return 'changed data'
+
+    self.mock_input_api.subprocess.check_output = mock_check_output
+
+    results = self._RunCheck()
+    self.assertEqual(1, len(results))
+    self.assertTrue('File is stale' in str(results[0]))
+
+  def testRelevantPyTwoChanges(self):
+    self.mock_input_api.files = [
+      MockAffectedFile('C.py', []),
+    ]
+
+    def mock_check_output(cmd, shell=False):
+      return 'changed data'
+
+    self.mock_input_api.subprocess.check_output = mock_check_output
+
+    results = self._RunCheck()
+    self.assertEqual(2, len(results))
+    self.assertTrue('File is stale' in str(results[0]))
+    self.assertTrue('File is stale' in str(results[1]))
+
+
+class LogUsageTest(unittest.TestCase):
+
+  def testCheckAndroidCrLogUsage(self):
+    mock_input_api = MockInputApi()
+    mock_output_api = MockOutputApi()
+
+    mock_input_api.files = [
+      MockAffectedFile('RandomStuff.java', [
+        'random stuff'
+      ]),
+      MockAffectedFile('HasAndroidLog.java', [
+        'import android.util.Log;',
+        'some random stuff',
+        'Log.d("TAG", "foo");',
+      ]),
+      MockAffectedFile('HasExplicitUtilLog.java', [
+        'some random stuff',
+        'android.util.Log.d("TAG", "foo");',
+      ]),
+      MockAffectedFile('IsInBasePackage.java', [
+        'package org.chromium.base;',
+        'private static final String TAG = "cr_Foo";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('IsInBasePackageButImportsLog.java', [
+        'package org.chromium.base;',
+        'import android.util.Log;',
+        'private static final String TAG = "cr_Foo";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasBothLog.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "cr_Foo";',
+        'Log.d(TAG, "foo");',
+        'android.util.Log.d("TAG", "foo");',
+      ]),
+      MockAffectedFile('HasCorrectTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "cr_Foo";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasOldTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "cr.Foo";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasDottedTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "cr_foo.bar";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasNoTagDecl.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasIncorrectTagDecl.java', [
+        'import org.chromium.base.Log;',
+        'private static final String TAHG = "cr_Foo";',
+        'some random stuff',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasInlineTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "cr_Foo";',
+        'Log.d("TAG", "foo");',
+      ]),
+      MockAffectedFile('HasUnprefixedTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "rubbish";',
+        'Log.d(TAG, "foo");',
+      ]),
+      MockAffectedFile('HasTooLongTag.java', [
+        'import org.chromium.base.Log;',
+        'some random stuff',
+        'private static final String TAG = "21_charachers_long___";',
+        'Log.d(TAG, "foo");',
+      ]),
+    ]
+
+    msgs = PRESUBMIT._CheckAndroidCrLogUsage(
+        mock_input_api, mock_output_api)
+
+    self.assertEqual(5, len(msgs),
+                     'Expected %d items, found %d: %s' % (5, len(msgs), msgs))
+
+    # Declaration format
+    nb = len(msgs[0].items)
+    self.assertEqual(2, nb,
+                     'Expected %d items, found %d: %s' % (2, nb, msgs[0].items))
+    self.assertTrue('HasNoTagDecl.java' in msgs[0].items)
+    self.assertTrue('HasIncorrectTagDecl.java' in msgs[0].items)
+
+    # Tag length
+    nb = len(msgs[1].items)
+    self.assertEqual(1, nb,
+                     'Expected %d items, found %d: %s' % (1, nb, msgs[1].items))
+    self.assertTrue('HasTooLongTag.java' in msgs[1].items)
+
+    # Tag must be a variable named TAG
+    nb = len(msgs[2].items)
+    self.assertEqual(1, nb,
+                     'Expected %d items, found %d: %s' % (1, nb, msgs[2].items))
+    self.assertTrue('HasInlineTag.java:4' in msgs[2].items)
+
+    # Util Log usage
+    nb = len(msgs[3].items)
+    self.assertEqual(2, nb,
+                     'Expected %d items, found %d: %s' % (2, nb, msgs[3].items))
+    self.assertTrue('HasAndroidLog.java:3' in msgs[3].items)
+    self.assertTrue('IsInBasePackageButImportsLog.java:4' in msgs[3].items)
+
+    # Tag must not contain
+    nb = len(msgs[4].items)
+    self.assertEqual(2, nb,
+                     'Expected %d items, found %d: %s' % (2, nb, msgs[4].items))
+    self.assertTrue('HasDottedTag.java' in msgs[4].items)
+    self.assertTrue('HasOldTag.java' in msgs[4].items)
+
+class HardcodedGoogleHostsTest(unittest.TestCase):
+
+  def testWarnOnAssignedLiterals(self):
+    input_api = MockInputApi()
+    input_api.files = [
+      MockFile('content/file.cc',
+               ['char* host = "https://www.google.com";']),
+      MockFile('content/file.cc',
+               ['char* host = "https://www.googleapis.com";']),
+      MockFile('content/file.cc',
+               ['char* host = "https://clients1.google.com";']),
+    ]
+
+    warnings = PRESUBMIT._CheckHardcodedGoogleHostsInLowerLayers(
+      input_api, MockOutputApi())
+    self.assertEqual(1, len(warnings))
+    self.assertEqual(3, len(warnings[0].items))
+
+  def testAllowInComment(self):
+    input_api = MockInputApi()
+    input_api.files = [
+      MockFile('content/file.cc',
+               ['char* host = "https://www.aol.com"; // google.com'])
+    ]
+
+    warnings = PRESUBMIT._CheckHardcodedGoogleHostsInLowerLayers(
+      input_api, MockOutputApi())
+    self.assertEqual(0, len(warnings))
+
+
 if __name__ == '__main__':
   unittest.main()
