@@ -17,6 +17,10 @@
 #include "base/strings/sys_string_conversions.h"
 #include "build/build_config.h"
 
+#if !defined(OS_IOS)
+#import <AppKit/AppKit.h>
+#endif
+
 #if !defined(OS_IOS)
 extern "C" {
 CFTypeID SecACLGetTypeID();
@@ -165,7 +169,7 @@ FilePath GetAppBundlePath(const FilePath& exec_name) {
 
   // The first component may be "/" or "//", etc. Only append '/' if it doesn't
   // already end in '/'.
-  if (bundle_name[bundle_name.length() - 1] != '/')
+  if (bundle_name.back() != '/')
     bundle_name += '/';
 
   // Go through the remaining components.
@@ -316,7 +320,7 @@ NSFont* CFToNSCast(CTFontRef cf_val) {
   DCHECK(!cf_val ||
          CTFontGetTypeID() == CFGetTypeID(cf_val) ||
          (_CFIsObjC(CTFontGetTypeID(), cf_val) &&
-          [ns_val isKindOfClass:NSClassFromString(@"NSFont")]));
+          [ns_val isKindOfClass:[NSFont class]]));
   return ns_val;
 }
 
@@ -324,7 +328,7 @@ CTFontRef NSToCFCast(NSFont* ns_val) {
   CTFontRef cf_val = reinterpret_cast<CTFontRef>(ns_val);
   DCHECK(!cf_val ||
          CTFontGetTypeID() == CFGetTypeID(cf_val) ||
-         [ns_val isKindOfClass:NSClassFromString(@"NSFont")]);
+         [ns_val isKindOfClass:[NSFont class]]);
   return cf_val;
 }
 #endif
@@ -388,7 +392,7 @@ CFCast<CTFontRef>(const CFTypeRef& cf_val) {
     return NULL;
 
   id<NSObject> ns_val = reinterpret_cast<id>(const_cast<void*>(cf_val));
-  if ([ns_val isKindOfClass:NSClassFromString(@"NSFont")]) {
+  if ([ns_val isKindOfClass:[NSFont class]]) {
     return (CTFontRef)(cf_val);
   }
   return NULL;
