@@ -85,7 +85,7 @@ using TargetFunction = const void*(*)(WaitableEvent*, WaitableEvent*,
 // SignalAndWaitUntilSignaled() when coordinated with the main thread.
 class TargetThread : public PlatformThread::Delegate {
  public:
-  TargetThread(const StackConfiguration& stack_config);
+  explicit TargetThread(const StackConfiguration& stack_config);
 
   // PlatformThread::Delegate:
   void ThreadMain() override;
@@ -149,8 +149,12 @@ class TargetThread : public PlatformThread::Delegate {
 };
 
 TargetThread::TargetThread(const StackConfiguration& stack_config)
-    : thread_started_event_(false, false), finish_event_(false, false),
-      id_(0), stack_config_(stack_config) {}
+    : thread_started_event_(WaitableEvent::ResetPolicy::AUTOMATIC,
+                            WaitableEvent::InitialState::NOT_SIGNALED),
+      finish_event_(WaitableEvent::ResetPolicy::AUTOMATIC,
+                    WaitableEvent::InitialState::NOT_SIGNALED),
+      id_(0),
+      stack_config_(stack_config) {}
 
 void TargetThread::ThreadMain() {
   id_ = PlatformThread::CurrentId();
@@ -267,9 +271,8 @@ NativeLibrary LoadOtherLibrary() {
   const auto load = [](NativeLibrary* library) {
     FilePath other_library_path;
     ASSERT_TRUE(PathService::Get(DIR_EXE, &other_library_path));
-    other_library_path = other_library_path.Append(FilePath::FromUTF16Unsafe(
-        GetNativeLibraryName(ASCIIToUTF16(
-            "base_profiler_test_support_library"))));
+    other_library_path = other_library_path.AppendASCII(
+        GetNativeLibraryName("base_profiler_test_support_library"));
     NativeLibraryLoadError load_error;
     *library = LoadNativeLibrary(other_library_path, &load_error);
     ASSERT_TRUE(*library) << "error loading " << other_library_path.value()
@@ -351,9 +354,11 @@ void CaptureProfiles(const SamplingParams& params, TimeDelta profiler_wait_time,
                      CallStackProfiles* profiles) {
   profiles->clear();
 
-  WithTargetThread([&params, profiles, profiler_wait_time](
-      PlatformThreadId target_thread_id) {
-    WaitableEvent sampling_thread_completed(true, false);
+  WithTargetThread([&params, profiles,
+                    profiler_wait_time](PlatformThreadId target_thread_id) {
+    WaitableEvent sampling_thread_completed(
+        WaitableEvent::ResetPolicy::MANUAL,
+        WaitableEvent::InitialState::NOT_SIGNALED);
     const StackSamplingProfiler::CompletedCallback callback =
         Bind(&SaveProfilesAndSignalEvent, Unretained(profiles),
              Unretained(&sampling_thread_completed));
@@ -411,7 +416,7 @@ std::string FormatSampleForDiagnosticOutput(
     const Sample& sample,
     const std::vector<Module>& modules) {
   std::string output;
-  for (const Frame& frame: sample) {
+  for (const Frame& frame : sample) {
     output += StringPrintf(
         "0x%p %s\n", reinterpret_cast<const void*>(frame.instruction_pointer),
         modules[frame.module_index].filename.AsUTF8Unsafe().c_str());
@@ -465,13 +470,17 @@ void TestLibraryUnload(bool wait_until_unloaded) {
 
   target_thread.WaitForThreadStart();
 
-  WaitableEvent sampling_thread_completed(true, false);
+  WaitableEvent sampling_thread_completed(
+      WaitableEvent::ResetPolicy::MANUAL,
+      WaitableEvent::InitialState::NOT_SIGNALED);
   std::vector<CallStackProfile> profiles;
   const StackSamplingProfiler::CompletedCallback callback =
       Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
            Unretained(&sampling_thread_completed));
-  WaitableEvent stack_copied(true, false);
-  WaitableEvent start_stack_walk(true, false);
+  WaitableEvent stack_copied(WaitableEvent::ResetPolicy::MANUAL,
+                             WaitableEvent::InitialState::NOT_SIGNALED);
+  WaitableEvent start_stack_walk(WaitableEvent::ResetPolicy::MANUAL,
+                                 WaitableEvent::InitialState::NOT_SIGNALED);
   StackCopiedSignaler test_delegate(&stack_copied, &start_stack_walk,
                                     wait_until_unloaded);
   StackSamplingProfiler profiler(target_thread.id(), params, callback,
@@ -625,16 +634,19 @@ TEST(StackSamplingProfilerTest, MAYBE_Alloca) {
   params.samples_per_burst = 1;
 
   std::vector<CallStackProfile> profiles;
-  WithTargetThread([&params, &profiles](
-      PlatformThreadId target_thread_id) {
-    WaitableEvent sampling_thread_completed(true, false);
-    const StackSamplingProfiler::CompletedCallback callback =
-        Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
-             Unretained(&sampling_thread_completed));
-    StackSamplingProfiler profiler(target_thread_id, params, callback);
-    profiler.Start();
-    sampling_thread_completed.Wait();
-  }, StackConfiguration(StackConfiguration::WITH_ALLOCA));
+  WithTargetThread(
+      [&params, &profiles](PlatformThreadId target_thread_id) {
+        WaitableEvent sampling_thread_completed(
+            WaitableEvent::ResetPolicy::MANUAL,
+            WaitableEvent::InitialState::NOT_SIGNALED);
+        const StackSamplingProfiler::CompletedCallback callback =
+            Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
+                 Unretained(&sampling_thread_completed));
+        StackSamplingProfiler profiler(target_thread_id, params, callback);
+        profiler.Start();
+        sampling_thread_completed.Wait();
+      },
+      StackConfiguration(StackConfiguration::WITH_ALLOCA));
 
   // Look up the sample.
   ASSERT_EQ(1u, profiles.size());
@@ -686,7 +698,9 @@ TEST(StackSamplingProfilerTest, MAYBE_StartAndRunAsync) {
 
   CallStackProfiles profiles;
   WithTargetThread([&params, &profiles](PlatformThreadId target_thread_id) {
-    WaitableEvent sampling_thread_completed(false, false);
+    WaitableEvent sampling_thread_completed(
+        WaitableEvent::ResetPolicy::AUTOMATIC,
+        WaitableEvent::InitialState::NOT_SIGNALED);
     const StackSamplingProfiler::CompletedCallback callback =
         Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
              Unretained(&sampling_thread_completed));
@@ -790,7 +804,7 @@ TEST(StackSamplingProfilerTest, MAYBE_DestroyProfilerWhileProfiling) {
 
   CallStackProfiles profiles;
   WithTargetThread([&params, &profiles](PlatformThreadId target_thread_id) {
-    scoped_ptr<StackSamplingProfiler> profiler;
+    std::unique_ptr<StackSamplingProfiler> profiler;
     profiler.reset(new StackSamplingProfiler(
         target_thread_id, params, Bind(&SaveProfiles, Unretained(&profiles))));
     profiler->Start();
@@ -843,7 +857,9 @@ TEST(StackSamplingProfilerTest, MAYBE_ConcurrentProfiling) {
     ScopedVector<WaitableEvent> sampling_completed;
     ScopedVector<StackSamplingProfiler> profiler;
     for (int i = 0; i < 2; ++i) {
-      sampling_completed.push_back(new WaitableEvent(false, false));
+      sampling_completed.push_back(
+          new WaitableEvent(WaitableEvent::ResetPolicy::AUTOMATIC,
+                            WaitableEvent::InitialState::NOT_SIGNALED));
       const StackSamplingProfiler::CompletedCallback callback =
           Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles[i]),
                Unretained(sampling_completed[i]));
@@ -886,17 +902,20 @@ TEST(StackSamplingProfilerTest, MAYBE_OtherLibrary) {
   std::vector<CallStackProfile> profiles;
   {
     ScopedNativeLibrary other_library(LoadOtherLibrary());
-    WithTargetThread([&params, &profiles](
-        PlatformThreadId target_thread_id) {
-      WaitableEvent sampling_thread_completed(true, false);
-      const StackSamplingProfiler::CompletedCallback callback =
-          Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
-               Unretained(&sampling_thread_completed));
-      StackSamplingProfiler profiler(target_thread_id, params, callback);
-      profiler.Start();
-      sampling_thread_completed.Wait();
-    }, StackConfiguration(StackConfiguration::WITH_OTHER_LIBRARY,
-                          other_library.get()));
+    WithTargetThread(
+        [&params, &profiles](PlatformThreadId target_thread_id) {
+          WaitableEvent sampling_thread_completed(
+              WaitableEvent::ResetPolicy::MANUAL,
+              WaitableEvent::InitialState::NOT_SIGNALED);
+          const StackSamplingProfiler::CompletedCallback callback =
+              Bind(&SaveProfilesAndSignalEvent, Unretained(&profiles),
+                   Unretained(&sampling_thread_completed));
+          StackSamplingProfiler profiler(target_thread_id, params, callback);
+          profiler.Start();
+          sampling_thread_completed.Wait();
+        },
+        StackConfiguration(StackConfiguration::WITH_OTHER_LIBRARY,
+                           other_library.get()));
   }
 
   // Look up the sample.
