@@ -12,9 +12,11 @@ namespace trace_event {
 
 TEST(JavaHeapDumpProviderTest, JavaHeapDump) {
   auto jhdp = JavaHeapDumpProvider::GetInstance();
-  scoped_ptr<ProcessMemoryDump> pmd(new ProcessMemoryDump(nullptr));
+  MemoryDumpArgs dump_args = {MemoryDumpLevelOfDetail::DETAILED};
+  std::unique_ptr<ProcessMemoryDump> pmd(
+      new ProcessMemoryDump(nullptr, dump_args));
 
-  jhdp->OnMemoryDump(pmd.get());
+  jhdp->OnMemoryDump(dump_args, pmd.get());
 }
 
 }  // namespace trace_event
