@@ -8,6 +8,7 @@
 #include <cmath>
 
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "cc/animation/timing_function.h"
 #include "cc/base/time_util.h"
 #include "ui/gfx/animation/tween.h"
@@ -63,7 +64,8 @@ static base::TimeDelta SegmentDuration(const gfx::Vector2dF& delta,
                                            base::Time::kMicrosecondsPerSecond);
 }
 
-static scoped_ptr<TimingFunction> EaseOutWithInitialVelocity(double velocity) {
+static std::unique_ptr<TimingFunction> EaseOutWithInitialVelocity(
+    double velocity) {
   // Clamp velocity to a sane value.
   velocity = std::min(std::max(velocity, -100.0), 100.0);
 
@@ -75,17 +77,17 @@ static scoped_ptr<TimingFunction> EaseOutWithInitialVelocity(double velocity) {
 
 }  // namespace
 
-scoped_ptr<ScrollOffsetAnimationCurve> ScrollOffsetAnimationCurve::Create(
+std::unique_ptr<ScrollOffsetAnimationCurve> ScrollOffsetAnimationCurve::Create(
     const gfx::ScrollOffset& target_value,
-    scoped_ptr<TimingFunction> timing_function,
+    std::unique_ptr<TimingFunction> timing_function,
     DurationBehavior duration_behavior) {
-  return make_scoped_ptr(new ScrollOffsetAnimationCurve(
+  return base::WrapUnique(new ScrollOffsetAnimationCurve(
       target_value, std::move(timing_function), duration_behavior));
 }
 
 ScrollOffsetAnimationCurve::ScrollOffsetAnimationCurve(
     const gfx::ScrollOffset& target_value,
-    scoped_ptr<TimingFunction> timing_function,
+    std::unique_ptr<TimingFunction> timing_function,
     DurationBehavior duration_behavior)
     : target_value_(target_value),
       timing_function_(std::move(timing_function)),
@@ -106,6 +108,12 @@ bool ScrollOffsetAnimationCurve::HasSetInitialValue() const {
   return has_set_initial_value_;
 }
 
+void ScrollOffsetAnimationCurve::ApplyAdjustment(
+    const gfx::Vector2dF& adjustment) {
+  initial_value_ = ScrollOffsetWithDelta(initial_value_, adjustment);
+  target_value_ = ScrollOffsetWithDelta(target_value_, adjustment);
+}
+
 gfx::ScrollOffset ScrollOffsetAnimationCurve::GetValue(
     base::TimeDelta t) const {
   base::TimeDelta duration = total_animation_duration_ - last_retarget_;
@@ -133,16 +141,21 @@ AnimationCurve::CurveType ScrollOffsetAnimationCurve::Type() const {
   return SCROLL_OFFSET;
 }
 
-scoped_ptr<AnimationCurve> ScrollOffsetAnimationCurve::Clone() const {
-  scoped_ptr<TimingFunction> timing_function(
+std::unique_ptr<AnimationCurve> ScrollOffsetAnimationCurve::Clone() const {
+  return CloneToScrollOffsetAnimationCurve();
+}
+
+std::unique_ptr<ScrollOffsetAnimationCurve>
+ScrollOffsetAnimationCurve::CloneToScrollOffsetAnimationCurve() const {
+  std::unique_ptr<TimingFunction> timing_function(
       static_cast<TimingFunction*>(timing_function_->Clone().release()));
-  scoped_ptr<ScrollOffsetAnimationCurve> curve_clone =
+  std::unique_ptr<ScrollOffsetAnimationCurve> curve_clone =
       Create(target_value_, std::move(timing_function), duration_behavior_);
   curve_clone->initial_value_ = initial_value_;
   curve_clone->total_animation_duration_ = total_animation_duration_;
   curve_clone->last_retarget_ = last_retarget_;
   curve_clone->has_set_initial_value_ = has_set_initial_value_;
-  return std::move(curve_clone);
+  return curve_clone;
 }
 
 static double VelocityBasedDurationBound(gfx::Vector2dF old_delta,
@@ -175,6 +188,11 @@ static double VelocityBasedDurationBound(gfx::Vector2dF old_delta,
 void ScrollOffsetAnimationCurve::UpdateTarget(
     double t,
     const gfx::ScrollOffset& new_target) {
+  if (std::abs(MaximumDimension(target_value_.DeltaFrom(new_target))) <
+      kEpsilon) {
+    target_value_ = new_target;
+    return;
+  }
   gfx::ScrollOffset current_position =
       GetValue(base::TimeDelta::FromSecondsD(t));
   gfx::Vector2dF old_delta = target_value_.DeltaFrom(initial_value_);
