@@ -2,12 +2,14 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "cc/animation/transform_operations.h"
+
 #include <stddef.h>
 
 #include <limits>
+#include <vector>
 
-#include "base/memory/scoped_vector.h"
-#include "cc/animation/transform_operations.h"
+#include "base/memory/ptr_util.h"
 #include "cc/test/geometry_test_utils.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/gfx/animation/tween.h"
@@ -19,27 +21,28 @@ namespace cc {
 namespace {
 
 TEST(TransformOperationTest, TransformTypesAreUnique) {
-  ScopedVector<TransformOperations> transforms;
+  std::vector<std::unique_ptr<TransformOperations>> transforms;
 
-  TransformOperations* to_add = new TransformOperations();
+  std::unique_ptr<TransformOperations> to_add(
+      base::WrapUnique(new TransformOperations()));
   to_add->AppendTranslate(1, 0, 0);
-  transforms.push_back(to_add);
+  transforms.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendRotate(0, 0, 1, 2);
-  transforms.push_back(to_add);
+  transforms.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendScale(2, 2, 2);
-  transforms.push_back(to_add);
+  transforms.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendSkew(1, 0);
-  transforms.push_back(to_add);
+  transforms.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendPerspective(800);
-  transforms.push_back(to_add);
+  transforms.push_back(std::move(to_add));
 
   for (size_t i = 0; i < transforms.size(); ++i) {
     for (size_t j = 0; j < transforms.size(); ++j) {
@@ -90,54 +93,58 @@ TEST(TransformOperationTest, MatchTypesDifferentLength) {
   EXPECT_FALSE(translates.MatchesTypes(translates2));
 }
 
-void GetIdentityOperations(ScopedVector<TransformOperations>* operations) {
-  TransformOperations* to_add = new TransformOperations();
-  operations->push_back(to_add);
+std::vector<std::unique_ptr<TransformOperations>> GetIdentityOperations() {
+  std::vector<std::unique_ptr<TransformOperations>> operations;
+  std::unique_ptr<TransformOperations> to_add(
+      base::WrapUnique(new TransformOperations()));
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendTranslate(0, 0, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendTranslate(0, 0, 0);
   to_add->AppendTranslate(0, 0, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendScale(1, 1, 1);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendScale(1, 1, 1);
   to_add->AppendScale(1, 1, 1);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendSkew(0, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendSkew(0, 0);
   to_add->AppendSkew(0, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendRotate(0, 0, 1, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendRotate(0, 0, 1, 0);
   to_add->AppendRotate(0, 0, 1, 0);
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendMatrix(gfx::Transform());
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
 
-  to_add = new TransformOperations();
+  to_add = base::WrapUnique(new TransformOperations());
   to_add->AppendMatrix(gfx::Transform());
   to_add->AppendMatrix(gfx::Transform());
-  operations->push_back(to_add);
+  operations.push_back(std::move(to_add));
+
+  return operations;
 }
 
 TEST(TransformOperationTest, MatchTypesOrder) {
@@ -162,8 +169,8 @@ TEST(TransformOperationTest, MatchTypesOrder) {
 }
 
 TEST(TransformOperationTest, NoneAlwaysMatches) {
-  ScopedVector<TransformOperations> operations;
-  GetIdentityOperations(&operations);
+  std::vector<std::unique_ptr<TransformOperations>> operations =
+      GetIdentityOperations();
 
   TransformOperations none_operation;
   for (size_t i = 0; i < operations.size(); ++i)
@@ -498,8 +505,8 @@ TEST(TransformOperationTest, RotationToZeroDegSameAxes) {
 }
 
 TEST(TransformOperationTest, BlendRotationFromIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -532,8 +539,8 @@ TEST(TransformOperationTest, BlendRotationFromIdentity) {
 }
 
 TEST(TransformOperationTest, BlendTranslationFromIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -566,8 +573,8 @@ TEST(TransformOperationTest, BlendTranslationFromIdentity) {
 }
 
 TEST(TransformOperationTest, BlendScaleFromIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -631,8 +638,8 @@ TEST(TransformOperationTest, BlendSkewFromEmpty) {
 }
 
 TEST(TransformOperationTest, BlendPerspectiveFromIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -649,8 +656,8 @@ TEST(TransformOperationTest, BlendPerspectiveFromIdentity) {
 }
 
 TEST(TransformOperationTest, BlendRotationToIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -667,8 +674,8 @@ TEST(TransformOperationTest, BlendRotationToIdentity) {
 }
 
 TEST(TransformOperationTest, BlendTranslationToIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -685,8 +692,8 @@ TEST(TransformOperationTest, BlendTranslationToIdentity) {
 }
 
 TEST(TransformOperationTest, BlendScaleToIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
@@ -718,8 +725,8 @@ TEST(TransformOperationTest, BlendSkewToEmpty) {
 }
 
 TEST(TransformOperationTest, BlendPerspectiveToIdentity) {
-  ScopedVector<TransformOperations> identity_operations;
-  GetIdentityOperations(&identity_operations);
+  std::vector<std::unique_ptr<TransformOperations>> identity_operations =
+      GetIdentityOperations();
 
   for (size_t i = 0; i < identity_operations.size(); ++i) {
     TransformOperations operations;
