@@ -5,11 +5,11 @@
 #ifndef CC_BLINK_WEB_EXTERNAL_TEXTURE_LAYER_IMPL_H_
 #define CC_BLINK_WEB_EXTERNAL_TEXTURE_LAYER_IMPL_H_
 
+#include <memory>
 #include <vector>
 
 #include "base/bind.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "cc/blink/cc_blink_export.h"
 #include "cc/layers/texture_layer_client.h"
 #include "third_party/WebKit/public/platform/WebExternalTextureLayer.h"
@@ -50,7 +50,7 @@ class WebExternalTextureLayerImpl
   // TextureLayerClient implementation.
   bool PrepareTextureMailbox(
       cc::TextureMailbox* mailbox,
-      scoped_ptr<cc::SingleReleaseCallback>* release_callback,
+      std::unique_ptr<cc::SingleReleaseCallback>* release_callback,
       bool use_shared_memory) override;
 
  private:
@@ -64,8 +64,8 @@ class WebExternalTextureLayerImpl
   WebExternalBitmapImpl* AllocateBitmap();
 
   blink::WebExternalTextureLayerClient* client_;
-  scoped_ptr<WebLayerImpl> layer_;
-  std::vector<scoped_ptr<WebExternalBitmapImpl>> free_bitmaps_;
+  std::unique_ptr<WebLayerImpl> layer_;
+  std::vector<std::unique_ptr<WebExternalBitmapImpl>> free_bitmaps_;
 
   DISALLOW_COPY_AND_ASSIGN(WebExternalTextureLayerImpl);
 };
