@@ -5,8 +5,9 @@
 #ifndef CC_BLINK_WEB_SCROLLBAR_LAYER_IMPL_H_
 #define CC_BLINK_WEB_SCROLLBAR_LAYER_IMPL_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "cc/blink/cc_blink_export.h"
 #include "third_party/WebKit/public/platform/WebScrollbar.h"
 #include "third_party/WebKit/public/platform/WebScrollbarLayer.h"
@@ -38,7 +39,7 @@ class WebScrollbarLayerImpl : public blink::WebScrollbarLayer {
   void setScrollLayer(blink::WebLayer* layer) override;
 
  private:
-  scoped_ptr<WebLayerImpl> layer_;
+  std::unique_ptr<WebLayerImpl> layer_;
 
   DISALLOW_COPY_AND_ASSIGN(WebScrollbarLayerImpl);
 };
