@@ -8,13 +8,13 @@
 
 #include "base/auto_reset.h"
 #include "cc/base/math_util.h"
+#include "cc/input/main_thread_scrolling_reason.h"
 #include "cc/layers/painted_scrollbar_layer_impl.h"
 #include "cc/resources/ui_resource_bitmap.h"
 #include "cc/trees/draw_property_utils.h"
 #include "cc/trees/layer_tree_host.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "skia/ext/platform_canvas.h"
-#include "skia/ext/refptr.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkCanvas.h"
 #include "third_party/skia/include/core/SkSize.h"
@@ -23,25 +23,23 @@
 
 namespace cc {
 
-scoped_ptr<LayerImpl> PaintedScrollbarLayer::CreateLayerImpl(
+std::unique_ptr<LayerImpl> PaintedScrollbarLayer::CreateLayerImpl(
     LayerTreeImpl* tree_impl) {
   return PaintedScrollbarLayerImpl::Create(
       tree_impl, id(), scrollbar_->Orientation());
 }
 
 scoped_refptr<PaintedScrollbarLayer> PaintedScrollbarLayer::Create(
-    const LayerSettings& settings,
-    scoped_ptr<Scrollbar> scrollbar,
+    std::unique_ptr<Scrollbar> scrollbar,
     int scroll_layer_id) {
-  return make_scoped_refptr(new PaintedScrollbarLayer(
-      settings, std::move(scrollbar), scroll_layer_id));
+  return make_scoped_refptr(
+      new PaintedScrollbarLayer(std::move(scrollbar), scroll_layer_id));
 }
 
-PaintedScrollbarLayer::PaintedScrollbarLayer(const LayerSettings& settings,
-                                             scoped_ptr<Scrollbar> scrollbar,
-                                             int scroll_layer_id)
-    : Layer(settings),
-      scrollbar_(std::move(scrollbar)),
+PaintedScrollbarLayer::PaintedScrollbarLayer(
+    std::unique_ptr<Scrollbar> scrollbar,
+    int scroll_layer_id)
+    : scrollbar_(std::move(scrollbar)),
       scroll_layer_id_(scroll_layer_id),
       internal_contents_scale_(1.f),
       thumb_thickness_(scrollbar_->ThumbThickness()),
@@ -50,7 +48,8 @@ PaintedScrollbarLayer::PaintedScrollbarLayer(const LayerSettings& settings,
       has_thumb_(scrollbar_->HasThumb()),
       thumb_opacity_(scrollbar_->ThumbOpacity()) {
   if (!scrollbar_->IsOverlay())
-    SetShouldScrollOnMainThread(true);
+    AddMainThreadScrollingReasons(
+        MainThreadScrollingReason::kScrollbarScrolling);
 }
 
 PaintedScrollbarLayer::~PaintedScrollbarLayer() {}
@@ -71,6 +70,10 @@ bool PaintedScrollbarLayer::OpacityCanAnimateOnImplThread() const {
   return scrollbar_->IsOverlay();
 }
 
+bool PaintedScrollbarLayer::AlwaysUseActiveTreeOpacity() const {
+  return true;
+}
+
 ScrollbarOrientation PaintedScrollbarLayer::orientation() const {
   return scrollbar_->Orientation();
 }
@@ -191,7 +194,7 @@ void PaintedScrollbarLayer::UpdateInternalContentScale() {
           ->settings()
           .layer_transforms_should_scale_layer_contents) {
     gfx::Transform transform;
-    transform = DrawTransformFromPropertyTrees(
+    transform = draw_property_utils::ScreenSpaceTransform(
         this, layer_tree_host()->property_trees()->transform_tree);
 
     gfx::Vector2dF transform_scales =
