@@ -8,7 +8,7 @@
 #include "base/trace_event/trace_event.h"
 #include "cc/layers/content_layer_client.h"
 #include "cc/layers/picture_layer_impl.h"
-#include "cc/playback/display_list_recording_source.h"
+#include "cc/playback/recording_source.h"
 #include "cc/proto/cc_conversions.h"
 #include "cc/proto/gfx_conversions.h"
 #include "cc/proto/layer.pb.h"
@@ -19,34 +19,29 @@
 
 namespace cc {
 
-scoped_refptr<PictureLayer> PictureLayer::Create(const LayerSettings& settings,
-                                                 ContentLayerClient* client) {
-  return make_scoped_refptr(new PictureLayer(settings, client));
+scoped_refptr<PictureLayer> PictureLayer::Create(ContentLayerClient* client) {
+  return make_scoped_refptr(new PictureLayer(client));
 }
 
-PictureLayer::PictureLayer(const LayerSettings& settings,
-                           ContentLayerClient* client)
-    : Layer(settings),
-      client_(client),
+PictureLayer::PictureLayer(ContentLayerClient* client)
+    : client_(client),
       instrumentation_object_tracker_(id()),
       update_source_frame_number_(-1),
       is_mask_(false),
-      nearest_neighbor_(false) {
-}
+      nearest_neighbor_(false) {}
 
-PictureLayer::PictureLayer(const LayerSettings& settings,
-                           ContentLayerClient* client,
-                           scoped_ptr<DisplayListRecordingSource> source)
-    : PictureLayer(settings, client) {
+PictureLayer::PictureLayer(ContentLayerClient* client,
+                           std::unique_ptr<RecordingSource> source)
+    : PictureLayer(client) {
   recording_source_ = std::move(source);
 }
 
 PictureLayer::~PictureLayer() {
 }
 
-scoped_ptr<LayerImpl> PictureLayer::CreateLayerImpl(LayerTreeImpl* tree_impl) {
-  return PictureLayerImpl::Create(tree_impl, id(), is_mask_,
-                                  new LayerImpl::SyncedScrollOffset);
+std::unique_ptr<LayerImpl> PictureLayer::CreateLayerImpl(
+    LayerTreeImpl* tree_impl) {
+  return PictureLayerImpl::Create(tree_impl, id(), is_mask_);
 }
 
 void PictureLayer::PushPropertiesTo(LayerImpl* base_layer) {
@@ -61,7 +56,7 @@ void PictureLayer::PushPropertiesTo(LayerImpl* base_layer) {
 
   // Preserve lcd text settings from the current raster source.
   bool can_use_lcd_text = layer_impl->RasterSourceUsesLCDText();
-  scoped_refptr<DisplayListRasterSource> raster_source =
+  scoped_refptr<RasterSource> raster_source =
       recording_source_->CreateRasterSource(can_use_lcd_text);
   layer_impl->set_gpu_raster_max_texture_size(
       layer_tree_host()->device_viewport_size());
@@ -76,7 +71,7 @@ void PictureLayer::SetLayerTreeHost(LayerTreeHost* host) {
     return;
 
   if (!recording_source_)
-    recording_source_.reset(new DisplayListRecordingSource);
+    recording_source_.reset(new RecordingSource);
   recording_source_->SetSlowdownRasterScaleFactor(
       host->debug_state().slow_down_raster_scale_factor);
   // If we need to enable image decode tasks, then we have to generate the
@@ -97,7 +92,6 @@ bool PictureLayer::Update() {
   update_source_frame_number_ = layer_tree_host()->source_frame_number();
   bool updated = Layer::Update();
 
-  gfx::Rect update_rect = visible_layer_rect();
   gfx::Size layer_size = paint_properties().bounds;
 
   recording_source_->SetBackgroundColor(SafeOpaqueBackgroundColor());
@@ -116,9 +110,8 @@ bool PictureLayer::Update() {
   // for them.
   DCHECK(client_);
   updated |= recording_source_->UpdateAndExpandInvalidation(
-      client_, &last_updated_invalidation_, layer_size, update_rect,
-      update_source_frame_number_, DisplayListRecordingSource::RECORD_NORMALLY);
-  last_updated_visible_layer_rect_ = visible_layer_rect();
+      client_, &last_updated_invalidation_, layer_size,
+      update_source_frame_number_, RecordingSource::RECORD_NORMALLY);
 
   if (updated) {
     SetNeedsPushProperties();
@@ -135,22 +128,21 @@ void PictureLayer::SetIsMask(bool is_mask) {
   is_mask_ = is_mask;
 }
 
-skia::RefPtr<SkPicture> PictureLayer::GetPicture() const {
-  // We could either flatten the DisplayListRecordingSource into a single
+sk_sp<SkPicture> PictureLayer::GetPicture() const {
+  // We could either flatten the RecordingSource into a single
   // SkPicture, or paint a fresh one depending on what we intend to do with the
   // picture. For now we just paint a fresh one to get consistent results.
   if (!DrawsContent())
-    return skia::RefPtr<SkPicture>();
+    return nullptr;
 
   gfx::Size layer_size = bounds();
-  scoped_ptr<DisplayListRecordingSource> recording_source(
-      new DisplayListRecordingSource);
+  std::unique_ptr<RecordingSource> recording_source(new RecordingSource);
   Region recording_invalidation;
   recording_source->UpdateAndExpandInvalidation(
-      client_, &recording_invalidation, layer_size, gfx::Rect(layer_size),
-      update_source_frame_number_, DisplayListRecordingSource::RECORD_NORMALLY);
+      client_, &recording_invalidation, layer_size, update_source_frame_number_,
+      RecordingSource::RECORD_NORMALLY);
 
-  scoped_refptr<DisplayListRasterSource> raster_source =
+  scoped_refptr<RasterSource> raster_source =
       recording_source->CreateRasterSource(false);
 
   return raster_source->GetFlattenedPicture();
@@ -178,7 +170,7 @@ bool PictureLayer::HasDrawableContent() const {
 }
 
 void PictureLayer::SetTypeForProtoSerialization(proto::LayerNode* proto) const {
-  proto->set_type(proto::LayerType::PICTURE_LAYER);
+  proto->set_type(proto::LayerNode::PICTURE_LAYER);
 }
 
 void PictureLayer::LayerSpecificPropertiesToProto(
@@ -188,9 +180,21 @@ void PictureLayer::LayerSpecificPropertiesToProto(
 
   proto::PictureLayerProperties* picture = proto->mutable_picture();
   recording_source_->ToProtobuf(picture->mutable_recording_source());
+
+  // Add all SkPicture items to the picture cache.
+  const DisplayItemList* display_list = recording_source_->GetDisplayItemList();
+  if (display_list) {
+    for (auto it = display_list->begin(); it != display_list->end(); ++it) {
+      sk_sp<const SkPicture> picture = it->GetPicture();
+      // Only DrawingDisplayItems have SkPictures.
+      if (!picture)
+        continue;
+
+      layer_tree_host()->engine_picture_cache()->MarkUsed(picture.get());
+    }
+  }
+
   RegionToProto(last_updated_invalidation_, picture->mutable_invalidation());
-  RectToProto(last_updated_visible_layer_rect_,
-              picture->mutable_last_updated_visible_layer_rect());
   picture->set_is_mask(is_mask_);
   picture->set_nearest_neighbor(nearest_neighbor_);
 
@@ -203,12 +207,23 @@ void PictureLayer::FromLayerSpecificPropertiesProto(
     const proto::LayerProperties& proto) {
   Layer::FromLayerSpecificPropertiesProto(proto);
   const proto::PictureLayerProperties& picture = proto.picture();
-  recording_source_->FromProtobuf(picture.recording_source());
+  // If this is a new layer, ensure it has a recording source. During layer
+  // hierarchy deserialization, ::SetLayerTreeHost(...) is not called, but
+  // instead the member is set directly, so it needs to be set here explicitly.
+  if (!recording_source_)
+    recording_source_.reset(new RecordingSource);
+
+  std::vector<uint32_t> used_engine_picture_ids;
+  recording_source_->FromProtobuf(picture.recording_source(),
+                                  layer_tree_host()->client_picture_cache(),
+                                  &used_engine_picture_ids);
+
+  // Inform picture cache about which SkPictures are now in use.
+  for (uint32_t engine_picture_id : used_engine_picture_ids)
+    layer_tree_host()->client_picture_cache()->MarkUsed(engine_picture_id);
 
   Region new_invalidation = RegionFromProto(picture.invalidation());
   last_updated_invalidation_.Swap(&new_invalidation);
-  last_updated_visible_layer_rect_ =
-      ProtoToRect(picture.last_updated_visible_layer_rect());
   is_mask_ = picture.is_mask();
   nearest_neighbor_ = picture.nearest_neighbor();
 
