@@ -15,10 +15,9 @@ namespace cc {
 class CC_EXPORT SolidColorScrollbarLayer : public ScrollbarLayerInterface,
                                            public Layer {
  public:
-  scoped_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
+  std::unique_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
 
   static scoped_refptr<SolidColorScrollbarLayer> Create(
-      const LayerSettings& settings,
       ScrollbarOrientation orientation,
       int thumb_thickness,
       int track_start,
@@ -27,8 +26,10 @@ class CC_EXPORT SolidColorScrollbarLayer : public ScrollbarLayerInterface,
 
   // Layer overrides.
   bool OpacityCanAnimateOnImplThread() const override;
+  bool AlwaysUseActiveTreeOpacity() const override;
   ScrollbarLayerInterface* ToScrollbarLayer() override;
 
+  void SetOpacity(float opacity) override;
   void PushPropertiesTo(LayerImpl* layer) override;
 
   void SetNeedsDisplayRect(const gfx::Rect& rect) override;
@@ -40,15 +41,22 @@ class CC_EXPORT SolidColorScrollbarLayer : public ScrollbarLayerInterface,
   ScrollbarOrientation orientation() const override;
 
  protected:
-  SolidColorScrollbarLayer(const LayerSettings& settings,
-                           ScrollbarOrientation orientation,
+  SolidColorScrollbarLayer(ScrollbarOrientation orientation,
                            int thumb_thickness,
                            int track_start,
                            bool is_left_side_vertical_scrollbar,
                            int scroll_layer_id);
   ~SolidColorScrollbarLayer() override;
 
+  // Layer overrides for proto conversions.
+  void SetTypeForProtoSerialization(proto::LayerNode* proto) const override;
+  void LayerSpecificPropertiesToProto(proto::LayerProperties* proto) override;
+  void FromLayerSpecificPropertiesProto(
+      const proto::LayerProperties& proto) override;
+
  private:
+  friend class LayerSerializationTest;
+
   int scroll_layer_id_;
   ScrollbarOrientation orientation_;
   int thumb_thickness_;
