@@ -21,18 +21,18 @@ class VideoLayerImpl;
 // A Layer that contains a Video element.
 class CC_EXPORT VideoLayer : public Layer {
  public:
-  static scoped_refptr<VideoLayer> Create(const LayerSettings& settings,
-                                          VideoFrameProvider* provider,
+  static scoped_refptr<VideoLayer> Create(VideoFrameProvider* provider,
                                           media::VideoRotation video_rotation);
 
-  scoped_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
+  std::unique_ptr<LayerImpl> CreateLayerImpl(LayerTreeImpl* tree_impl) override;
 
   bool Update() override;
 
+  // Clears |provider_| to ensure it is not used after destruction.
+  void StopUsingProvider();
+
  private:
-  VideoLayer(const LayerSettings& settings,
-             VideoFrameProvider* provider,
-             media::VideoRotation video_rotation);
+  VideoLayer(VideoFrameProvider* provider, media::VideoRotation video_rotation);
   ~VideoLayer() override;
 
   // This pointer is only for passing to VideoLayerImpl's constructor. It should
