@@ -5,10 +5,10 @@
 #ifndef CC_OUTPUT_SOFTWARE_OUTPUT_DEVICE_H_
 #define CC_OUTPUT_SOFTWARE_OUTPUT_DEVICE_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "cc/base/cc_export.h"
-#include "skia/ext/refptr.h"
 #include "third_party/skia/include/core/SkSurface.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/geometry/size.h"
@@ -60,8 +60,8 @@ class CC_EXPORT SoftwareOutputDevice {
   gfx::Size viewport_pixel_size_;
   float scale_factor_;
   gfx::Rect damage_rect_;
-  skia::RefPtr<SkSurface> surface_;
-  scoped_ptr<gfx::VSyncProvider> vsync_provider_;
+  sk_sp<SkSurface> surface_;
+  std::unique_ptr<gfx::VSyncProvider> vsync_provider_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(SoftwareOutputDevice);
