@@ -8,6 +8,7 @@
 
 #include <vector>
 
+#include "base/memory/ptr_util.h"
 #include "cc/output/filter_operation.h"
 #include "cc/output/filter_operations.h"
 #include "cc/playback/clip_display_item.h"
@@ -19,8 +20,11 @@
 #include "cc/playback/float_clip_display_item.h"
 #include "cc/playback/transform_display_item.h"
 #include "cc/proto/display_item.pb.h"
+#include "cc/test/fake_client_picture_cache.h"
+#include "cc/test/fake_engine_picture_cache.h"
+#include "cc/test/fake_image_serialization_processor.h"
 #include "cc/test/skia_common.h"
-#include "skia/ext/refptr.h"
+#include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkCanvas.h"
@@ -45,48 +49,66 @@ void AppendFirstSerializationTestPicture(scoped_refptr<DisplayItemList> list,
                                          const gfx::Size& layer_size) {
   gfx::PointF offset(2.f, 3.f);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
+  sk_sp<SkCanvas> canvas;
 
   SkPaint red_paint;
   red_paint.setColor(SK_ColorRED);
 
-  canvas = skia::SharePtr(recorder.beginRecording(SkRect::MakeXYWH(
+  canvas = sk_ref_sp(recorder.beginRecording(SkRect::MakeXYWH(
       offset.x(), offset.y(), layer_size.width(), layer_size.height())));
   canvas->translate(offset.x(), offset.y());
   canvas->drawRectCoords(0.f, 0.f, 4.f, 4.f, red_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 }
 
 void AppendSecondSerializationTestPicture(scoped_refptr<DisplayItemList> list,
                                           const gfx::Size& layer_size) {
   gfx::PointF offset(2.f, 2.f);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
+  sk_sp<SkCanvas> canvas;
 
   SkPaint blue_paint;
   blue_paint.setColor(SK_ColorBLUE);
 
-  canvas = skia::SharePtr(recorder.beginRecording(SkRect::MakeXYWH(
+  canvas = sk_ref_sp(recorder.beginRecording(SkRect::MakeXYWH(
       offset.x(), offset.y(), layer_size.width(), layer_size.height())));
   canvas->translate(offset.x(), offset.y());
   canvas->drawRectCoords(3.f, 3.f, 7.f, 7.f, blue_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 }
 
 void ValidateDisplayItemListSerialization(const gfx::Size& layer_size,
                                           scoped_refptr<DisplayItemList> list) {
   list->Finalize();
 
+  std::unique_ptr<FakeImageSerializationProcessor>
+      fake_image_serialization_processor =
+          base::WrapUnique(new FakeImageSerializationProcessor);
+  std::unique_ptr<EnginePictureCache> fake_engine_picture_cache =
+      fake_image_serialization_processor->CreateEnginePictureCache();
+  FakeEnginePictureCache* fake_engine_picture_cache_ptr =
+      static_cast<FakeEnginePictureCache*>(fake_engine_picture_cache.get());
+  std::unique_ptr<ClientPictureCache> fake_client_picture_cache =
+      fake_image_serialization_processor->CreateClientPictureCache();
+
+  fake_engine_picture_cache_ptr->MarkAllSkPicturesAsUsed(list.get());
+
   // Serialize and deserialize the DisplayItemList.
   proto::DisplayItemList proto;
   list->ToProtobuf(&proto);
-  scoped_refptr<DisplayItemList> new_list =
-      DisplayItemList::CreateFromProto(proto);
 
-  EXPECT_TRUE(
-      AreDisplayListDrawingResultsSame(gfx::Rect(layer_size), list, new_list));
+  std::vector<uint32_t> actual_picture_ids;
+  scoped_refptr<DisplayItemList> new_list = DisplayItemList::CreateFromProto(
+      proto, fake_client_picture_cache.get(), &actual_picture_ids);
+
+  EXPECT_THAT(actual_picture_ids,
+              testing::UnorderedElementsAreArray(
+                  fake_engine_picture_cache_ptr->GetAllUsedPictureIds()));
+
+  EXPECT_TRUE(AreDisplayListDrawingResultsSame(gfx::Rect(layer_size),
+                                               list.get(), new_list.get()));
 }
 
 }  // namespace
@@ -138,7 +160,8 @@ TEST(DisplayItemListTest, SerializeClipItem) {
   gfx::Rect clip_rect(6, 6, 1, 1);
   std::vector<SkRRect> rrects;
   rrects.push_back(SkRRect::MakeOval(SkRect::MakeXYWH(5.f, 5.f, 4.f, 4.f)));
-  list->CreateAndAppendItem<ClipDisplayItem>(kVisualRect, clip_rect, rrects);
+  list->CreateAndAppendItem<ClipDisplayItem>(kVisualRect, clip_rect, rrects,
+                                             true);
 
   // Build the second DrawingDisplayItem.
   AppendSecondSerializationTestPicture(list, layer_size);
@@ -185,10 +208,10 @@ TEST(DisplayItemListTest, SerializeCompositingItem) {
   AppendFirstSerializationTestPicture(list, layer_size);
 
   // Build the CompositingDisplayItem.
-  skia::RefPtr<SkColorFilter> filter = skia::AdoptRef(
-      SkColorMatrixFilter::CreateLightingFilter(SK_ColorRED, SK_ColorGREEN));
   list->CreateAndAppendItem<CompositingDisplayItem>(
-      kVisualRect, 150, SkXfermode::Mode::kDst_Mode, nullptr, filter, false);
+      kVisualRect, 150, SkXfermode::Mode::kDst_Mode, nullptr,
+      SkColorMatrixFilter::MakeLightingFilter(SK_ColorRED, SK_ColorGREEN),
+      false);
 
   // Build the second DrawingDisplayItem.
   AppendSecondSerializationTestPicture(list, layer_size);
@@ -250,8 +273,7 @@ TEST(DisplayItemListTest, SerializeTransformItem) {
 TEST(DisplayItemListTest, SingleDrawingItem) {
   gfx::Rect layer_rect(100, 100);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
-  skia::RefPtr<SkPicture> picture;
+  sk_sp<SkCanvas> canvas;
   SkPaint blue_paint;
   blue_paint.setColor(SK_ColorBLUE);
   SkPaint red_paint;
@@ -263,14 +285,13 @@ TEST(DisplayItemListTest, SingleDrawingItem) {
 
   gfx::PointF offset(8.f, 9.f);
   gfx::RectF recording_rect(offset, gfx::SizeF(layer_rect.size()));
-  canvas = skia::SharePtr(
-      recorder.beginRecording(gfx::RectFToSkRect(recording_rect)));
+  canvas =
+      sk_ref_sp(recorder.beginRecording(gfx::RectFToSkRect(recording_rect)));
   canvas->translate(offset.x(), offset.y());
   canvas->drawRectCoords(0.f, 0.f, 60.f, 60.f, red_paint);
   canvas->drawRectCoords(50.f, 50.f, 75.f, 75.f, blue_paint);
-  picture = skia::AdoptRef(recorder.endRecordingAsPicture());
-  list->CreateAndAppendItem<DrawingDisplayItem>(kVisualRect,
-                                                std::move(picture));
+  list->CreateAndAppendItem<DrawingDisplayItem>(
+      kVisualRect, recorder.finishRecordingAsPicture());
   list->Finalize();
   DrawDisplayList(pixels, layer_rect, list);
 
@@ -294,7 +315,7 @@ TEST(DisplayItemListTest, SingleDrawingItem) {
 TEST(DisplayItemListTest, ClipItem) {
   gfx::Rect layer_rect(100, 100);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
+  sk_sp<SkCanvas> canvas;
   SkPaint blue_paint;
   blue_paint.setColor(SK_ColorBLUE);
   SkPaint red_paint;
@@ -307,26 +328,26 @@ TEST(DisplayItemListTest, ClipItem) {
 
   gfx::PointF first_offset(8.f, 9.f);
   gfx::RectF first_recording_rect(first_offset, gfx::SizeF(layer_rect.size()));
-  canvas = skia::SharePtr(
+  canvas = sk_ref_sp(
       recorder.beginRecording(gfx::RectFToSkRect(first_recording_rect)));
   canvas->translate(first_offset.x(), first_offset.y());
   canvas->drawRectCoords(0.f, 0.f, 60.f, 60.f, red_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 
   gfx::Rect clip_rect(60, 60, 10, 10);
   list->CreateAndAppendItem<ClipDisplayItem>(kVisualRect, clip_rect,
-                                             std::vector<SkRRect>());
+                                             std::vector<SkRRect>(), true);
 
   gfx::PointF second_offset(2.f, 3.f);
   gfx::RectF second_recording_rect(second_offset,
                                    gfx::SizeF(layer_rect.size()));
-  canvas = skia::SharePtr(
+  canvas = sk_ref_sp(
       recorder.beginRecording(gfx::RectFToSkRect(second_recording_rect)));
   canvas->translate(second_offset.x(), second_offset.y());
   canvas->drawRectCoords(50.f, 50.f, 75.f, 75.f, blue_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 
   list->CreateAndAppendItem<EndClipDisplayItem>(kVisualRect);
   list->Finalize();
@@ -354,7 +375,7 @@ TEST(DisplayItemListTest, ClipItem) {
 TEST(DisplayItemListTest, TransformItem) {
   gfx::Rect layer_rect(100, 100);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
+  sk_sp<SkCanvas> canvas;
   SkPaint blue_paint;
   blue_paint.setColor(SK_ColorBLUE);
   SkPaint red_paint;
@@ -367,12 +388,12 @@ TEST(DisplayItemListTest, TransformItem) {
 
   gfx::PointF first_offset(8.f, 9.f);
   gfx::RectF first_recording_rect(first_offset, gfx::SizeF(layer_rect.size()));
-  canvas = skia::SharePtr(
+  canvas = sk_ref_sp(
       recorder.beginRecording(gfx::RectFToSkRect(first_recording_rect)));
   canvas->translate(first_offset.x(), first_offset.y());
   canvas->drawRectCoords(0.f, 0.f, 60.f, 60.f, red_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 
   gfx::Transform transform;
   transform.Rotate(45.0);
@@ -381,12 +402,12 @@ TEST(DisplayItemListTest, TransformItem) {
   gfx::PointF second_offset(2.f, 3.f);
   gfx::RectF second_recording_rect(second_offset,
                                    gfx::SizeF(layer_rect.size()));
-  canvas = skia::SharePtr(
+  canvas = sk_ref_sp(
       recorder.beginRecording(gfx::RectFToSkRect(second_recording_rect)));
   canvas->translate(second_offset.x(), second_offset.y());
   canvas->drawRectCoords(50.f, 50.f, 75.f, 75.f, blue_paint);
   list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
+      kVisualRect, recorder.finishRecordingAsPicture());
 
   list->CreateAndAppendItem<EndTransformDisplayItem>(kVisualRect);
   list->Finalize();
@@ -420,12 +441,10 @@ TEST(DisplayItemListTest, FilterItem) {
   scoped_refptr<DisplayItemList> list =
       DisplayItemList::Create(layer_rect, settings);
 
-  skia::RefPtr<SkSurface> source_surface =
-      skia::AdoptRef(SkSurface::NewRasterN32Premul(50, 50));
+  sk_sp<SkSurface> source_surface = SkSurface::MakeRasterN32Premul(50, 50);
   SkCanvas* source_canvas = source_surface->getCanvas();
   source_canvas->clear(SkColorSetRGB(128, 128, 128));
-  skia::RefPtr<SkImage> source_image =
-      skia::AdoptRef(source_surface->newImageSnapshot());
+  sk_sp<SkImage> source_image = source_surface->makeImageSnapshot();
 
   // For most SkImageFilters, the |dst| bounds computed by computeFastBounds are
   // dependent on the provided |src| bounds. This means, for example, that
@@ -438,8 +457,7 @@ TEST(DisplayItemListTest, FilterItem) {
   // incorrect clipping of filter output. To test for this, we include an
   // SkImageSource filter in |filters|. Here, |src| is |filter_bounds|, defined
   // below.
-  skia::RefPtr<SkImageFilter> image_filter =
-      skia::AdoptRef(SkImageSource::Create(source_image.get()));
+  sk_sp<SkImageFilter> image_filter = SkImageSource::Make(source_image);
   filters.Append(FilterOperation::CreateReferenceFilter(image_filter));
   filters.Append(FilterOperation::CreateBrightnessFilter(0.5f));
   gfx::RectF filter_bounds(10.f, 10.f, 50.f, 50.f);
@@ -466,8 +484,7 @@ TEST(DisplayItemListTest, FilterItem) {
 TEST(DisplayItemListTest, CompactingItems) {
   gfx::Rect layer_rect(100, 100);
   SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
-  skia::RefPtr<SkPicture> picture;
+  sk_sp<SkCanvas> canvas;
   SkPaint blue_paint;
   blue_paint.setColor(SK_ColorBLUE);
   SkPaint red_paint;
@@ -482,12 +499,12 @@ TEST(DisplayItemListTest, CompactingItems) {
   scoped_refptr<DisplayItemList> list_without_caching =
       DisplayItemList::Create(layer_rect, no_caching_settings);
 
-  canvas = skia::SharePtr(
-      recorder.beginRecording(gfx::RectFToSkRect(recording_rect)));
+  canvas =
+      sk_ref_sp(recorder.beginRecording(gfx::RectFToSkRect(recording_rect)));
   canvas->translate(offset.x(), offset.y());
   canvas->drawRectCoords(0.f, 0.f, 60.f, 60.f, red_paint);
   canvas->drawRectCoords(50.f, 50.f, 75.f, 75.f, blue_paint);
-  picture = skia::AdoptRef(recorder.endRecordingAsPicture());
+  sk_sp<SkPicture> picture = recorder.finishRecordingAsPicture();
   list_without_caching->CreateAndAppendItem<DrawingDisplayItem>(kVisualRect,
                                                                 picture);
   list_without_caching->Finalize();
@@ -506,131 +523,6 @@ TEST(DisplayItemListTest, CompactingItems) {
   EXPECT_EQ(0, memcmp(pixels, expected_pixels, 4 * 100 * 100));
 }
 
-TEST(DisplayItemListTest, IsSuitableForGpuRasterizationWithCachedPicture) {
-  gfx::Rect layer_rect(1000, 1000);
-  SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
-
-  DisplayItemListSettings settings;
-  settings.use_cached_picture = true;
-  scoped_refptr<DisplayItemList> list =
-      DisplayItemList::Create(layer_rect, settings);
-  canvas =
-      skia::SharePtr(recorder.beginRecording(gfx::RectToSkRect(layer_rect)));
-
-  SkPath path;
-  path.moveTo(0, 0);
-  path.lineTo(0, 100);
-  path.lineTo(50, 50);
-  path.lineTo(100, 100);
-  path.lineTo(100, 0);
-  path.close();
-
-  SkPaint paint;
-  paint.setAntiAlias(true);
-  canvas->drawPath(path, paint);
-
-  skia::RefPtr<SkPicture> suitable_picture =
-      skia::AdoptRef(recorder.endRecordingAsPicture());
-  list->CreateAndAppendItem<DrawingDisplayItem>(kVisualRect, suitable_picture);
-  list->Finalize();
-
-  // A single DrawingDisplayItem with a large AA concave path shouldn't trigger
-  // a veto.
-  EXPECT_TRUE(list->IsSuitableForGpuRasterization());
-
-  // Now check the RasterIntoCanvas path.
-  list = DisplayItemList::Create(layer_rect, settings);
-  DrawingDisplayItem suitable_item(suitable_picture);
-  list->RasterIntoCanvas(suitable_item);
-  list->Finalize();
-  EXPECT_TRUE(list->IsSuitableForGpuRasterization());
-
-  list = DisplayItemList::Create(layer_rect, settings);
-  canvas =
-      skia::SharePtr(recorder.beginRecording(gfx::RectToSkRect(layer_rect)));
-  for (int i = 0; i < 10; ++i)
-    canvas->drawPath(path, paint);
-  skia::RefPtr<SkPicture> unsuitable_picture =
-      skia::AdoptRef(recorder.endRecordingAsPicture());
-  list->CreateAndAppendItem<DrawingDisplayItem>(kVisualRect,
-                                                unsuitable_picture);
-  list->Finalize();
-
-  // A single DrawingDisplayItem with several large AA concave paths should
-  // trigger a veto.
-  EXPECT_FALSE(list->IsSuitableForGpuRasterization());
-
-  // Now check the RasterIntoCanvas path.
-  list = DisplayItemList::Create(layer_rect, settings);
-  DrawingDisplayItem unsuitable_item(unsuitable_picture);
-  list->RasterIntoCanvas(unsuitable_item);
-  list->Finalize();
-  EXPECT_FALSE(list->IsSuitableForGpuRasterization());
-}
-
-TEST(DisplayItemListTest, IsSuitableForGpuRasterizationWithoutCachedPicture) {
-  gfx::Rect layer_rect(1000, 1000);
-  SkPictureRecorder recorder;
-  skia::RefPtr<SkCanvas> canvas;
-  skia::RefPtr<SkPicture> picture;
-
-  DisplayItemListSettings settings;
-  settings.use_cached_picture = false;
-  scoped_refptr<DisplayItemList> list =
-      DisplayItemList::Create(layer_rect, settings);
-  canvas =
-      skia::SharePtr(recorder.beginRecording(gfx::RectToSkRect(layer_rect)));
-
-  SkPath path;
-  path.moveTo(0, 0);
-  path.lineTo(0, 100);
-  path.lineTo(50, 50);
-  path.lineTo(100, 100);
-  path.lineTo(100, 0);
-  path.close();
-
-  SkPaint paint;
-  paint.setAntiAlias(true);
-  canvas->drawPath(path, paint);
-
-  list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
-  list->Finalize();
-
-  // A single DrawingDisplayItem with a large AA concave path shouldn't trigger
-  // a veto.
-  EXPECT_TRUE(list->IsSuitableForGpuRasterization());
-
-  list = DisplayItemList::Create(layer_rect, settings);
-  canvas =
-      skia::SharePtr(recorder.beginRecording(gfx::RectToSkRect(layer_rect)));
-  for (int i = 0; i < 10; ++i)
-    canvas->drawPath(path, paint);
-  list->CreateAndAppendItem<DrawingDisplayItem>(
-      kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
-  list->Finalize();
-
-  // A single DrawingDisplayItem with several large AA concave paths should
-  // trigger a veto.
-  EXPECT_FALSE(list->IsSuitableForGpuRasterization());
-
-  list = DisplayItemList::Create(layer_rect, settings);
-  for (int i = 0; i < 10; ++i) {
-    canvas =
-        skia::SharePtr(recorder.beginRecording(gfx::RectToSkRect(layer_rect)));
-    canvas->drawPath(path, paint);
-    list->CreateAndAppendItem<DrawingDisplayItem>(
-        kVisualRect, skia::AdoptRef(recorder.endRecordingAsPicture()));
-  }
-  list->Finalize();
-
-  // Without a cached picture, having several DrawingDisplayItems that each
-  // contain a single large AA concave will not trigger a veto, since each item
-  // is individually suitable for GPU rasterization.
-  EXPECT_TRUE(list->IsSuitableForGpuRasterization());
-}
-
 TEST(DisplayItemListTest, ApproximateMemoryUsage) {
   const int kNumCommandsInTestSkPicture = 1000;
   scoped_refptr<DisplayItemList> list;
@@ -644,8 +536,7 @@ TEST(DisplayItemListTest, ApproximateMemoryUsage) {
   SkCanvas* canvas = recorder.beginRecording(gfx::RectToSkRect(layer_rect));
   for (int i = 0; i < kNumCommandsInTestSkPicture; i++)
     canvas->drawPaint(blue_paint);
-  skia::RefPtr<SkPicture> picture =
-      skia::AdoptRef(recorder.endRecordingAsPicture());
+  sk_sp<SkPicture> picture = recorder.finishRecordingAsPicture();
   size_t picture_size = SkPictureUtils::ApproximateBytesUsed(picture.get());
   ASSERT_GE(picture_size, kNumCommandsInTestSkPicture * sizeof(blue_paint));
 
