@@ -13,7 +13,6 @@
 #include "cc/base/math_util.h"
 #include "cc/output/filter_operations.h"
 #include "cc/quads/debug_border_draw_quad.h"
-#include "cc/quads/io_surface_draw_quad.h"
 #include "cc/quads/largest_draw_quad.h"
 #include "cc/quads/picture_draw_quad.h"
 #include "cc/quads/render_pass.h"
@@ -24,7 +23,7 @@
 #include "cc/quads/texture_draw_quad.h"
 #include "cc/quads/tile_draw_quad.h"
 #include "cc/quads/yuv_video_draw_quad.h"
-#include "cc/test/fake_display_list_raster_source.h"
+#include "cc/test/fake_raster_source.h"
 #include "cc/test/geometry_test_utils.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/skia/include/effects/SkBlurImageFilter.h"
@@ -43,12 +42,11 @@ TEST(DrawQuadTest, CopySharedQuadState) {
   SkXfermode::Mode blend_mode = SkXfermode::kMultiply_Mode;
   int sorting_context_id = 65536;
 
-  scoped_ptr<SharedQuadState> state(new SharedQuadState);
+  std::unique_ptr<SharedQuadState> state(new SharedQuadState);
   state->SetAll(quad_transform, layer_bounds, visible_layer_rect, clip_rect,
                 is_clipped, opacity, blend_mode, sorting_context_id);
 
-  scoped_ptr<SharedQuadState> copy(new SharedQuadState);
-  copy->CopyFrom(state.get());
+  std::unique_ptr<SharedQuadState> copy(new SharedQuadState(*state));
   EXPECT_EQ(quad_transform, copy->quad_to_target_transform);
   EXPECT_EQ(visible_layer_rect, copy->visible_quad_layer_rect);
   EXPECT_EQ(opacity, copy->opacity);
@@ -85,11 +83,11 @@ void CompareDrawQuad(DrawQuad* quad,
 }
 
 #define CREATE_SHARED_STATE()                                              \
-  scoped_ptr<RenderPass> render_pass = RenderPass::Create();               \
+  std::unique_ptr<RenderPass> render_pass = RenderPass::Create();          \
   SharedQuadState* shared_state(CreateSharedQuadState(render_pass.get())); \
   SharedQuadState* copy_shared_state =                                     \
       render_pass->CreateAndAppendSharedQuadState();                       \
-  copy_shared_state->CopyFrom(shared_state);
+  *copy_shared_state = *shared_state;
 
 #define QUAD_DATA                              \
   gfx::Rect quad_rect(30, 40, 50, 60);         \
@@ -336,6 +334,31 @@ void CompareDrawQuad(DrawQuad* quad,
   }                                                                          \
   SETUP_AND_COPY_QUAD_NEW(Type, quad_new);
 
+#define CREATE_QUAD_11_ALL(Type, a, b, c, d, e, f, g, h, i, j, k)             \
+  Type* quad_all = render_pass->CreateAndAppendDrawQuad<Type>();              \
+  {                                                                           \
+    QUAD_DATA quad_all->SetAll(shared_state, quad_rect, quad_opaque_rect,     \
+                               quad_visible_rect, needs_blending, a, b, c, d, \
+                               e, f, g, h, i, j, k);                          \
+  }                                                                           \
+  SETUP_AND_COPY_QUAD_ALL(Type, quad_all);
+
+#define CREATE_QUAD_12_NEW(Type, a, b, c, d, e, f, g, h, i, j, k, l)         \
+  Type* quad_new = render_pass->CreateAndAppendDrawQuad<Type>();             \
+  {                                                                          \
+    QUAD_DATA quad_new->SetNew(shared_state, quad_rect, a, b, c, d, e, f, g, \
+                               h, i, j, k, l);                               \
+  }                                                                          \
+  SETUP_AND_COPY_QUAD_NEW(Type, quad_new);
+
+#define CREATE_QUAD_13_NEW(Type, a, b, c, d, e, f, g, h, i, j, k, l, m)      \
+  Type* quad_new = render_pass->CreateAndAppendDrawQuad<Type>();             \
+  {                                                                          \
+    QUAD_DATA quad_new->SetNew(shared_state, quad_rect, a, b, c, d, e, f, g, \
+                               h, i, j, k, l, m);                            \
+  }                                                                          \
+  SETUP_AND_COPY_QUAD_NEW(Type, quad_new);
+
 #define CREATE_QUAD_ALL_RP(Type, a, b, c, d, e, f, g, copy_a)    \
   Type* quad_all = render_pass->CreateAndAppendDrawQuad<Type>(); \
   {                                                              \
@@ -380,30 +403,6 @@ TEST(DrawQuadTest, CopyDebugBorderDrawQuad) {
   EXPECT_EQ(width, copy_quad->width);
 }
 
-TEST(DrawQuadTest, CopyIOSurfaceDrawQuad) {
-  gfx::Rect opaque_rect(33, 47, 10, 12);
-  gfx::Rect visible_rect(40, 50, 30, 20);
-  gfx::Size size(58, 95);
-  ResourceId resource_id = 72;
-  IOSurfaceDrawQuad::Orientation orientation = IOSurfaceDrawQuad::UNFLIPPED;
-  CREATE_SHARED_STATE();
-
-  CREATE_QUAD_5_NEW(IOSurfaceDrawQuad, opaque_rect, visible_rect, size,
-                    resource_id, orientation);
-  EXPECT_EQ(DrawQuad::IO_SURFACE_CONTENT, copy_quad->material);
-  EXPECT_EQ(visible_rect, copy_quad->visible_rect);
-  EXPECT_EQ(opaque_rect, copy_quad->opaque_rect);
-  EXPECT_EQ(size, copy_quad->io_surface_size);
-  EXPECT_EQ(resource_id, copy_quad->io_surface_resource_id());
-  EXPECT_EQ(orientation, copy_quad->orientation);
-
-  CREATE_QUAD_3_ALL(IOSurfaceDrawQuad, size, resource_id, orientation);
-  EXPECT_EQ(DrawQuad::IO_SURFACE_CONTENT, copy_quad->material);
-  EXPECT_EQ(size, copy_quad->io_surface_size);
-  EXPECT_EQ(resource_id, copy_quad->io_surface_resource_id());
-  EXPECT_EQ(orientation, copy_quad->orientation);
-}
-
 TEST(DrawQuadTest, CopyRenderPassDrawQuad) {
   gfx::Rect visible_rect(40, 50, 30, 20);
   RenderPassId render_pass_id(22, 64);
@@ -507,7 +506,7 @@ TEST(DrawQuadTest, CopyStreamVideoDrawQuad) {
 
 TEST(DrawQuadTest, CopySurfaceDrawQuad) {
   gfx::Rect visible_rect(40, 50, 30, 20);
-  SurfaceId surface_id(1234);
+  SurfaceId surface_id(0, 1234, 0);
   CREATE_SHARED_STATE();
 
   CREATE_QUAD_2_NEW(SurfaceDrawQuad, visible_rect, surface_id);
@@ -532,19 +531,13 @@ TEST(DrawQuadTest, CopyTextureDrawQuad) {
   const float vertex_opacity[] = { 1.0f, 1.0f, 1.0f, 1.0f };
   bool y_flipped = true;
   bool nearest_neighbor = true;
+  bool secure_output_only = true;
   CREATE_SHARED_STATE();
 
-  CREATE_QUAD_10_NEW(TextureDrawQuad,
-                     opaque_rect,
-                     visible_rect,
-                     resource_id,
-                     premultiplied_alpha,
-                     uv_top_left,
-                     uv_bottom_right,
-                     SK_ColorTRANSPARENT,
-                     vertex_opacity,
-                     y_flipped,
-                     nearest_neighbor);
+  CREATE_QUAD_11_NEW(TextureDrawQuad, opaque_rect, visible_rect, resource_id,
+                     premultiplied_alpha, uv_top_left, uv_bottom_right,
+                     SK_ColorTRANSPARENT, vertex_opacity, y_flipped,
+                     nearest_neighbor, secure_output_only);
   EXPECT_EQ(DrawQuad::TEXTURE_CONTENT, copy_quad->material);
   EXPECT_EQ(visible_rect, copy_quad->visible_rect);
   EXPECT_EQ(opaque_rect, copy_quad->opaque_rect);
@@ -555,11 +548,12 @@ TEST(DrawQuadTest, CopyTextureDrawQuad) {
   EXPECT_FLOAT_ARRAY_EQ(vertex_opacity, copy_quad->vertex_opacity, 4);
   EXPECT_EQ(y_flipped, copy_quad->y_flipped);
   EXPECT_EQ(nearest_neighbor, copy_quad->nearest_neighbor);
+  EXPECT_EQ(secure_output_only, copy_quad->secure_output_only);
 
-  CREATE_QUAD_9_ALL(TextureDrawQuad, resource_id, resource_size_in_pixels,
-                    premultiplied_alpha, uv_top_left, uv_bottom_right,
-                    SK_ColorTRANSPARENT, vertex_opacity, y_flipped,
-                    nearest_neighbor);
+  CREATE_QUAD_10_ALL(TextureDrawQuad, resource_id, resource_size_in_pixels,
+                     premultiplied_alpha, uv_top_left, uv_bottom_right,
+                     SK_ColorTRANSPARENT, vertex_opacity, y_flipped,
+                     nearest_neighbor, secure_output_only);
   EXPECT_EQ(DrawQuad::TEXTURE_CONTENT, copy_quad->material);
   EXPECT_EQ(resource_id, copy_quad->resource_id());
   EXPECT_EQ(resource_size_in_pixels, copy_quad->resource_size_in_pixels());
@@ -569,6 +563,7 @@ TEST(DrawQuadTest, CopyTextureDrawQuad) {
   EXPECT_FLOAT_ARRAY_EQ(vertex_opacity, copy_quad->vertex_opacity, 4);
   EXPECT_EQ(y_flipped, copy_quad->y_flipped);
   EXPECT_EQ(nearest_neighbor, copy_quad->nearest_neighbor);
+  EXPECT_EQ(secure_output_only, copy_quad->secure_output_only);
 }
 
 TEST(DrawQuadTest, CopyTileDrawQuad) {
@@ -623,13 +618,16 @@ TEST(DrawQuadTest, CopyYUVVideoDrawQuad) {
   ResourceId u_plane_resource_id = 532;
   ResourceId v_plane_resource_id = 4;
   ResourceId a_plane_resource_id = 63;
+  float resource_offset = 0.5f;
+  float resource_multiplier = 2.001f;
   YUVVideoDrawQuad::ColorSpace color_space = YUVVideoDrawQuad::JPEG;
   CREATE_SHARED_STATE();
 
-  CREATE_QUAD_11_NEW(YUVVideoDrawQuad, opaque_rect, visible_rect,
+  CREATE_QUAD_13_NEW(YUVVideoDrawQuad, opaque_rect, visible_rect,
                      ya_tex_coord_rect, uv_tex_coord_rect, ya_tex_size,
                      uv_tex_size, y_plane_resource_id, u_plane_resource_id,
-                     v_plane_resource_id, a_plane_resource_id, color_space);
+                     v_plane_resource_id, a_plane_resource_id, color_space,
+                     resource_offset, resource_multiplier);
   EXPECT_EQ(DrawQuad::YUV_VIDEO_CONTENT, copy_quad->material);
   EXPECT_EQ(opaque_rect, copy_quad->opaque_rect);
   EXPECT_EQ(visible_rect, copy_quad->visible_rect);
@@ -642,11 +640,14 @@ TEST(DrawQuadTest, CopyYUVVideoDrawQuad) {
   EXPECT_EQ(v_plane_resource_id, copy_quad->v_plane_resource_id());
   EXPECT_EQ(a_plane_resource_id, copy_quad->a_plane_resource_id());
   EXPECT_EQ(color_space, copy_quad->color_space);
+  EXPECT_EQ(resource_offset, copy_quad->resource_offset);
+  EXPECT_EQ(resource_multiplier, copy_quad->resource_multiplier);
 
-  CREATE_QUAD_9_ALL(YUVVideoDrawQuad, ya_tex_coord_rect, uv_tex_coord_rect,
-                    ya_tex_size, uv_tex_size, y_plane_resource_id,
-                    u_plane_resource_id, v_plane_resource_id,
-                    a_plane_resource_id, color_space);
+  CREATE_QUAD_11_ALL(YUVVideoDrawQuad, ya_tex_coord_rect, uv_tex_coord_rect,
+                     ya_tex_size, uv_tex_size, y_plane_resource_id,
+                     u_plane_resource_id, v_plane_resource_id,
+                     a_plane_resource_id, color_space, resource_offset,
+                     resource_multiplier);
   EXPECT_EQ(DrawQuad::YUV_VIDEO_CONTENT, copy_quad->material);
   EXPECT_EQ(ya_tex_coord_rect, copy_quad->ya_tex_coord_rect);
   EXPECT_EQ(uv_tex_coord_rect, copy_quad->uv_tex_coord_rect);
@@ -657,6 +658,8 @@ TEST(DrawQuadTest, CopyYUVVideoDrawQuad) {
   EXPECT_EQ(v_plane_resource_id, copy_quad->v_plane_resource_id());
   EXPECT_EQ(a_plane_resource_id, copy_quad->a_plane_resource_id());
   EXPECT_EQ(color_space, copy_quad->color_space);
+  EXPECT_EQ(resource_offset, copy_quad->resource_offset);
+  EXPECT_EQ(resource_multiplier, copy_quad->resource_multiplier);
 }
 
 TEST(DrawQuadTest, CopyPictureDrawQuad) {
@@ -668,8 +671,8 @@ TEST(DrawQuadTest, CopyPictureDrawQuad) {
   ResourceFormat texture_format = RGBA_8888;
   gfx::Rect content_rect(30, 40, 20, 30);
   float contents_scale = 3.141592f;
-  scoped_refptr<DisplayListRasterSource> raster_source =
-      FakeDisplayListRasterSource::CreateEmpty(gfx::Size(100, 100));
+  scoped_refptr<RasterSource> raster_source =
+      FakeRasterSource::CreateEmpty(gfx::Size(100, 100));
   CREATE_SHARED_STATE();
 
   CREATE_QUAD_9_NEW(PictureDrawQuad, opaque_rect, visible_rect, tex_coord_rect,
@@ -724,21 +727,6 @@ TEST_F(DrawQuadIteratorTest, DebugBorderDrawQuad) {
   EXPECT_EQ(0, IterateAndCount(quad_new));
 }
 
-TEST_F(DrawQuadIteratorTest, IOSurfaceDrawQuad) {
-  gfx::Rect opaque_rect(33, 47, 10, 12);
-  gfx::Rect visible_rect(40, 50, 30, 20);
-  gfx::Size size(58, 95);
-  ResourceId resource_id = 72;
-  IOSurfaceDrawQuad::Orientation orientation = IOSurfaceDrawQuad::UNFLIPPED;
-
-  CREATE_SHARED_STATE();
-  CREATE_QUAD_5_NEW(IOSurfaceDrawQuad, opaque_rect, visible_rect, size,
-                    resource_id, orientation);
-  EXPECT_EQ(resource_id, quad_new->io_surface_resource_id());
-  EXPECT_EQ(1, IterateAndCount(quad_new));
-  EXPECT_EQ(resource_id + 1, quad_new->io_surface_resource_id());
-}
-
 TEST_F(DrawQuadIteratorTest, RenderPassDrawQuad) {
   gfx::Rect visible_rect(40, 50, 30, 20);
   RenderPassId render_pass_id(22, 64);
@@ -807,7 +795,7 @@ TEST_F(DrawQuadIteratorTest, StreamVideoDrawQuad) {
 
 TEST_F(DrawQuadIteratorTest, SurfaceDrawQuad) {
   gfx::Rect visible_rect(40, 50, 30, 20);
-  SurfaceId surface_id(4321);
+  SurfaceId surface_id(0, 4321, 0);
 
   CREATE_SHARED_STATE();
   CREATE_QUAD_2_NEW(SurfaceDrawQuad, visible_rect, surface_id);
@@ -824,19 +812,13 @@ TEST_F(DrawQuadIteratorTest, TextureDrawQuad) {
   const float vertex_opacity[] = { 1.0f, 1.0f, 1.0f, 1.0f };
   bool y_flipped = true;
   bool nearest_neighbor = true;
+  bool secure_output_only = true;
 
   CREATE_SHARED_STATE();
-  CREATE_QUAD_10_NEW(TextureDrawQuad,
-                     opaque_rect,
-                     visible_rect,
-                     resource_id,
-                     premultiplied_alpha,
-                     uv_top_left,
-                     uv_bottom_right,
-                     SK_ColorTRANSPARENT,
-                     vertex_opacity,
-                     y_flipped,
-                     nearest_neighbor);
+  CREATE_QUAD_11_NEW(TextureDrawQuad, opaque_rect, visible_rect, resource_id,
+                     premultiplied_alpha, uv_top_left, uv_bottom_right,
+                     SK_ColorTRANSPARENT, vertex_opacity, y_flipped,
+                     nearest_neighbor, secure_output_only);
   EXPECT_EQ(resource_id, quad_new->resource_id());
   EXPECT_EQ(1, IterateAndCount(quad_new));
   EXPECT_EQ(resource_id + 1, quad_new->resource_id());
@@ -879,10 +861,11 @@ TEST_F(DrawQuadIteratorTest, YUVVideoDrawQuad) {
   YUVVideoDrawQuad::ColorSpace color_space = YUVVideoDrawQuad::JPEG;
 
   CREATE_SHARED_STATE();
-  CREATE_QUAD_11_NEW(YUVVideoDrawQuad, opaque_rect, visible_rect,
+  CREATE_QUAD_13_NEW(YUVVideoDrawQuad, opaque_rect, visible_rect,
                      ya_tex_coord_rect, uv_tex_coord_rect, ya_tex_size,
                      uv_tex_size, y_plane_resource_id, u_plane_resource_id,
-                     v_plane_resource_id, a_plane_resource_id, color_space);
+                     v_plane_resource_id, a_plane_resource_id, color_space, 0.0,
+                     1.0);
   EXPECT_EQ(DrawQuad::YUV_VIDEO_CONTENT, copy_quad->material);
   EXPECT_EQ(y_plane_resource_id, quad_new->y_plane_resource_id());
   EXPECT_EQ(u_plane_resource_id, quad_new->u_plane_resource_id());
@@ -906,8 +889,8 @@ TEST_F(DrawQuadIteratorTest, DISABLED_PictureDrawQuad) {
   ResourceFormat texture_format = RGBA_8888;
   gfx::Rect content_rect(30, 40, 20, 30);
   float contents_scale = 3.141592f;
-  scoped_refptr<DisplayListRasterSource> raster_source =
-      FakeDisplayListRasterSource::CreateEmpty(gfx::Size(100, 100));
+  scoped_refptr<RasterSource> raster_source =
+      FakeRasterSource::CreateEmpty(gfx::Size(100, 100));
 
   CREATE_SHARED_STATE();
   CREATE_QUAD_9_NEW(PictureDrawQuad, opaque_rect, visible_rect, tex_coord_rect,
@@ -924,9 +907,6 @@ TEST(DrawQuadTest, LargestQuadType) {
       case DrawQuad::DEBUG_BORDER:
         largest = std::max(largest, sizeof(DebugBorderDrawQuad));
         break;
-      case DrawQuad::IO_SURFACE_CONTENT:
-        largest = std::max(largest, sizeof(IOSurfaceDrawQuad));
-        break;
       case DrawQuad::PICTURE_CONTENT:
         largest = std::max(largest, sizeof(PictureDrawQuad));
         break;
@@ -968,9 +948,6 @@ TEST(DrawQuadTest, LargestQuadType) {
       case DrawQuad::DEBUG_BORDER:
         LOG(ERROR) << "DebugBorderDrawQuad " << sizeof(DebugBorderDrawQuad);
         break;
-      case DrawQuad::IO_SURFACE_CONTENT:
-        LOG(ERROR) << "IOSurfaceDrawQuad " << sizeof(IOSurfaceDrawQuad);
-        break;
       case DrawQuad::PICTURE_CONTENT:
         LOG(ERROR) << "PictureDrawQuad " << sizeof(PictureDrawQuad);
         break;
