@@ -8,22 +8,23 @@
 #include <stdint.h>
 
 #include "cc/base/cc_export.h"
+#include "cc/playback/raster_source.h"
 #include "ui/gfx/geometry/rect.h"
 
 namespace cc {
-class DisplayListRasterSource;
 
 class CC_EXPORT RasterBuffer {
  public:
   RasterBuffer();
   virtual ~RasterBuffer();
 
-  virtual void Playback(const DisplayListRasterSource* raster_source,
-                        const gfx::Rect& raster_full_rect,
-                        const gfx::Rect& raster_dirty_rect,
-                        uint64_t new_content_id,
-                        float scale,
-                        bool include_images) = 0;
+  virtual void Playback(
+      const RasterSource* raster_source,
+      const gfx::Rect& raster_full_rect,
+      const gfx::Rect& raster_dirty_rect,
+      uint64_t new_content_id,
+      float scale,
+      const RasterSource::PlaybackSettings& playback_settings) = 0;
 };
 
 }  // namespace cc
