@@ -5,10 +5,11 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <memory>
 #include <vector>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/time/time.h"
 #include "cc/base/completion_event.h"
 #include "cc/debug/lap_timer.h"
@@ -32,7 +33,7 @@ class PerfTaskImpl : public Task {
   // Overridden from Task:
   void RunOnWorkerThread() override {}
 
-  void Reset() { did_run_ = false; }
+  void Reset() { state().Reset(); }
 
  private:
   ~PerfTaskImpl() override {}
@@ -49,7 +50,7 @@ class TaskGraphRunnerPerfTest : public testing::Test {
 
   // Overridden from testing::Test:
   void SetUp() override {
-    task_graph_runner_ = make_scoped_ptr(new SynchronousTaskGraphRunner);
+    task_graph_runner_ = base::WrapUnique(new SynchronousTaskGraphRunner);
     namespace_token_ = task_graph_runner_->GetNamespaceToken();
   }
   void TearDown() override { task_graph_runner_ = nullptr; }
@@ -75,6 +76,10 @@ class TaskGraphRunnerPerfTest : public testing::Test {
       timer_.NextLap();
     } while (!timer_.HasTimeLimitExpired());
 
+    CancelTasks(leaf_tasks);
+    CancelTasks(tasks);
+    CancelTasks(top_level_tasks);
+
     perf_test::PrintResult("build_task_graph",
                            TestModifierString(),
                            test_name,
@@ -144,11 +149,15 @@ class TaskGraphRunnerPerfTest : public testing::Test {
     size_t count = 0;
     timer_.Reset();
     do {
+      size_t current_version = count % kNumVersions;
       graph.Reset();
-      BuildTaskGraph(top_level_tasks[count % kNumVersions],
-                     tasks[count % kNumVersions],
-                     leaf_tasks[count % kNumVersions],
-                     &graph);
+      // Reset tasks as we are not letting them execute, they get cancelled
+      // when next ScheduleTasks() happens.
+      ResetTasks(top_level_tasks[current_version]);
+      ResetTasks(tasks[current_version]);
+      ResetTasks(leaf_tasks[current_version]);
+      BuildTaskGraph(top_level_tasks[current_version], tasks[current_version],
+                     leaf_tasks[current_version], &graph);
       task_graph_runner_->ScheduleTasks(namespace_token_, &graph);
       CollectCompletedTasks(&completed_tasks);
       completed_tasks.clear();
@@ -187,14 +196,16 @@ class TaskGraphRunnerPerfTest : public testing::Test {
     timer_.Reset();
     do {
       graph.Reset();
+      // Tasks run have finished state. Reset them to be considered as new for
+      // scheduling again.
+      ResetTasks(top_level_tasks);
+      ResetTasks(tasks);
+      ResetTasks(leaf_tasks);
       BuildTaskGraph(top_level_tasks, tasks, leaf_tasks, &graph);
       task_graph_runner_->ScheduleTasks(namespace_token_, &graph);
       task_graph_runner_->RunUntilIdle();
       CollectCompletedTasks(&completed_tasks);
       completed_tasks.clear();
-      ResetTasks(&top_level_tasks);
-      ResetTasks(&tasks);
-      ResetTasks(&leaf_tasks);
       timer_.NextLap();
     } while (!timer_.HasTimeLimitExpired());
 
@@ -216,12 +227,14 @@ class TaskGraphRunnerPerfTest : public testing::Test {
       tasks->push_back(make_scoped_refptr(new PerfTaskImpl));
   }
 
-  void ResetTasks(PerfTaskImpl::Vector* tasks) {
-    for (PerfTaskImpl::Vector::iterator it = tasks->begin(); it != tasks->end();
-         ++it) {
-      PerfTaskImpl* task = it->get();
+  void CancelTasks(const PerfTaskImpl::Vector& tasks) {
+    for (auto& task : tasks)
+      task->state().DidCancel();
+  }
+
+  void ResetTasks(const PerfTaskImpl::Vector& tasks) {
+    for (auto& task : tasks)
       task->Reset();
-    }
   }
 
   void BuildTaskGraph(const PerfTaskImpl::Vector& top_level_tasks,
@@ -272,7 +285,7 @@ class TaskGraphRunnerPerfTest : public testing::Test {
 
   // Test uses SynchronousTaskGraphRunner, as this implementation introduces
   // minimal additional complexity over the TaskGraphWorkQueue helpers.
-  scoped_ptr<SynchronousTaskGraphRunner> task_graph_runner_;
+  std::unique_ptr<SynchronousTaskGraphRunner> task_graph_runner_;
   NamespaceToken namespace_token_;
   LapTimer timer_;
 };
