@@ -55,7 +55,6 @@ SchedulerStateMachine::SchedulerStateMachine(const SchedulerSettings& settings)
       critical_begin_main_frame_to_activate_is_fast_(true),
       main_thread_missed_last_deadline_(false),
       skip_next_begin_main_frame_to_reduce_latency_(false),
-      children_need_begin_frames_(false),
       defer_commits_(false),
       video_needs_begin_frames_(false),
       last_commit_had_no_updates_(false),
@@ -86,8 +85,6 @@ const char* SchedulerStateMachine::BeginImplFrameStateToString(
   switch (state) {
     case BEGIN_IMPL_FRAME_STATE_IDLE:
       return "BEGIN_IMPL_FRAME_STATE_IDLE";
-    case BEGIN_IMPL_FRAME_STATE_BEGIN_FRAME_STARTING:
-      return "BEGIN_IMPL_FRAME_STATE_BEGIN_FRAME_STARTING";
     case BEGIN_IMPL_FRAME_STATE_INSIDE_BEGIN_FRAME:
       return "BEGIN_IMPL_FRAME_STATE_INSIDE_BEGIN_FRAME";
     case BEGIN_IMPL_FRAME_STATE_INSIDE_DEADLINE:
@@ -189,12 +186,12 @@ const char* SchedulerStateMachine::ActionToString(Action action) {
   return "???";
 }
 
-scoped_refptr<base::trace_event::ConvertableToTraceFormat>
+std::unique_ptr<base::trace_event::ConvertableToTraceFormat>
 SchedulerStateMachine::AsValue() const {
-  scoped_refptr<base::trace_event::TracedValue> state =
-      new base::trace_event::TracedValue();
+  std::unique_ptr<base::trace_event::TracedValue> state(
+      new base::trace_event::TracedValue());
   AsValueInto(state.get());
-  return state;
+  return std::move(state);
 }
 
 void SchedulerStateMachine::AsValueInto(
@@ -256,7 +253,6 @@ void SchedulerStateMachine::AsValueInto(
                     main_thread_missed_last_deadline_);
   state->SetBoolean("skip_next_begin_main_frame_to_reduce_latency",
                     skip_next_begin_main_frame_to_reduce_latency_);
-  state->SetBoolean("children_need_begin_frames", children_need_begin_frames_);
   state->SetBoolean("video_needs_begin_frames", video_needs_begin_frames_);
   state->SetBoolean("defer_commits", defer_commits_);
   state->SetBoolean("last_commit_had_no_updates", last_commit_had_no_updates_);
@@ -315,8 +311,14 @@ bool SchedulerStateMachine::ShouldBeginOutputSurfaceCreation() const {
 
   // We only want to start output surface initialization after the
   // previous commit is complete.
-  if (begin_main_frame_state_ != BEGIN_MAIN_FRAME_STATE_IDLE)
+  // We make an exception if the embedder explicitly allows beginning output
+  // surface creation while the previous commit has not been aborted. This
+  // assumes that any state passed from the client during the commit will not be
+  // tied to the output surface.
+  if (begin_main_frame_state_ != BEGIN_MAIN_FRAME_STATE_IDLE &&
+      settings_.abort_commit_before_output_surface_creation) {
     return false;
+  }
 
   // Make sure the BeginImplFrame from any previous OutputSurfaces
   // are complete before creating the new OutputSurface.
@@ -409,18 +411,6 @@ bool SchedulerStateMachine::CouldSendBeginMainFrame() const {
   return true;
 }
 
-bool SchedulerStateMachine::SendingBeginMainFrameMightCauseDeadlock() const {
-  // NPAPI is the only case where the UI thread makes synchronous calls to the
-  // Renderer main thread. During that synchronous call, we may not get a
-  // SwapAck for the UI thread, which may prevent BeginMainFrame's from
-  // completing if there's enough back pressure. If the BeginMainFrame can't
-  // make progress, the Renderer can't service the UI thread's synchronous call
-  // and we have deadlock.
-  // This returns true if there's too much backpressure to finish a commit
-  // if we were to initiate a BeginMainFrame.
-  return has_pending_tree_ && active_tree_needs_first_draw_ && SwapThrottled();
-}
-
 bool SchedulerStateMachine::ShouldSendBeginMainFrame() const {
   if (!CouldSendBeginMainFrame())
     return false;
@@ -457,9 +447,6 @@ bool SchedulerStateMachine::ShouldSendBeginMainFrame() const {
 
   // We need a new commit for the forced redraw. This honors the
   // single commit per interval because the result will be swapped to screen.
-  // TODO(brianderson): Remove this or move it below the
-  // SendingBeginMainFrameMightCauseDeadlock check since  we want to avoid
-  // ever returning true from this method if we might cause deadlock.
   if (forced_redraw_state_ == FORCED_REDRAW_STATE_WAITING_FOR_COMMIT)
     return true;
 
@@ -467,10 +454,6 @@ bool SchedulerStateMachine::ShouldSendBeginMainFrame() const {
   if (!HasInitializedOutputSurface())
     return false;
 
-  // Make sure the BeginMainFrame can finish eventually if we start it.
-  if (SendingBeginMainFrameMightCauseDeadlock())
-    return false;
-
   if (!settings_.main_frame_while_swap_throttled_enabled) {
     // SwapAck throttle the BeginMainFrames unless we just swapped to
     // potentially improve impl-thread latency over main-thread throughput.
@@ -531,7 +514,7 @@ bool SchedulerStateMachine::ShouldInvalidateOutputSurface() const {
     return false;
 
   // Invalidations are only performed inside a BeginFrame.
-  if (begin_impl_frame_state_ != BEGIN_IMPL_FRAME_STATE_BEGIN_FRAME_STARTING)
+  if (begin_impl_frame_state_ != BEGIN_IMPL_FRAME_STATE_INSIDE_BEGIN_FRAME)
     return false;
 
   // TODO(sunnyps): needs_prepare_tiles_ is needed here because PrepareTiles is
@@ -576,7 +559,11 @@ void SchedulerStateMachine::WillSendBeginMainFrame() {
 }
 
 void SchedulerStateMachine::WillCommit(bool commit_has_no_updates) {
+  DCHECK(!has_pending_tree_ ||
+         (settings_.main_frame_before_activation_enabled &&
+          commit_has_no_updates));
   commit_count_++;
+  last_commit_had_no_updates_ = commit_has_no_updates;
 
   if (commit_has_no_updates || settings_.main_frame_before_activation_enabled) {
     begin_main_frame_state_ = BEGIN_MAIN_FRAME_STATE_IDLE;
@@ -584,11 +571,12 @@ void SchedulerStateMachine::WillCommit(bool commit_has_no_updates) {
     begin_main_frame_state_ = BEGIN_MAIN_FRAME_STATE_WAITING_FOR_ACTIVATION;
   }
 
-  // If the commit was aborted, then there is no pending tree.
-  has_pending_tree_ = !commit_has_no_updates;
-
-  wait_for_ready_to_draw_ =
-      !commit_has_no_updates && settings_.commit_to_active_tree;
+  if (!commit_has_no_updates) {
+    // Pending tree only exists if commit had updates.
+    has_pending_tree_ = true;
+    pending_tree_is_ready_for_activation_ = false;
+    wait_for_ready_to_draw_ = settings_.commit_to_active_tree;
+  }
 
   // Update state related to forced draws.
   if (forced_redraw_state_ == FORCED_REDRAW_STATE_WAITING_FOR_COMMIT) {
@@ -598,28 +586,11 @@ void SchedulerStateMachine::WillCommit(bool commit_has_no_updates) {
   }
 
   // Update the output surface state.
-  DCHECK_NE(output_surface_state_, OUTPUT_SURFACE_WAITING_FOR_FIRST_ACTIVATION);
   if (output_surface_state_ == OUTPUT_SURFACE_WAITING_FOR_FIRST_COMMIT) {
-    if (has_pending_tree_) {
-      output_surface_state_ = OUTPUT_SURFACE_WAITING_FOR_FIRST_ACTIVATION;
-    } else {
-      output_surface_state_ = OUTPUT_SURFACE_ACTIVE;
-    }
+    output_surface_state_ = has_pending_tree_
+                                ? OUTPUT_SURFACE_WAITING_FOR_FIRST_ACTIVATION
+                                : OUTPUT_SURFACE_ACTIVE;
   }
-
-  // Update state if there's no updates heading for the active tree, but we need
-  // to do a forced draw.
-  if (commit_has_no_updates &&
-      forced_redraw_state_ == FORCED_REDRAW_STATE_WAITING_FOR_DRAW) {
-    DCHECK(!has_pending_tree_);
-    needs_redraw_ = true;
-    active_tree_needs_first_draw_ = true;
-  }
-
-  // This post-commit work is common to both completed and aborted commits.
-  pending_tree_is_ready_for_activation_ = false;
-
-  last_commit_had_no_updates_ = commit_has_no_updates;
 }
 
 void SchedulerStateMachine::WillActivate() {
@@ -643,6 +614,13 @@ void SchedulerStateMachine::WillActivate() {
 }
 
 void SchedulerStateMachine::WillDrawInternal() {
+  // If a new active tree is pending after the one we are about to draw,
+  // the main thread is in a high latency mode.
+  // main_thread_missed_last_deadline_ is here in addition to
+  // OnBeginImplFrameIdle for cases where the scheduler aborts draws outside
+  // of the deadline.
+  main_thread_missed_last_deadline_ = CommitPending() || has_pending_tree_;
+
   // We need to reset needs_redraw_ before we draw since the
   // draw itself might request another draw.
   needs_redraw_ = false;
@@ -725,7 +703,11 @@ void SchedulerStateMachine::WillBeginOutputSurfaceCreation() {
   // The following DCHECKs make sure we are in the proper quiescent state.
   // The pipeline should be flushed entirely before we start output
   // surface creation to avoid complicated corner cases.
-  DCHECK_EQ(begin_main_frame_state_, BEGIN_MAIN_FRAME_STATE_IDLE);
+
+  // We allow output surface creation while the previous commit has not been
+  // aborted if the embedder explicitly allows it.
+  DCHECK(!settings_.abort_commit_before_output_surface_creation ||
+         begin_main_frame_state_ == BEGIN_MAIN_FRAME_STATE_IDLE);
   DCHECK(!has_pending_tree_);
   DCHECK(!active_tree_needs_first_draw_);
 }
@@ -749,10 +731,6 @@ void SchedulerStateMachine::SetSkipNextBeginMainFrameToReduceLatency() {
   skip_next_begin_main_frame_to_reduce_latency_ = true;
 }
 
-bool SchedulerStateMachine::BeginFrameRequiredForChildren() const {
-  return children_need_begin_frames_;
-}
-
 bool SchedulerStateMachine::BeginFrameNeededForVideo() const {
   return video_needs_begin_frames_;
 }
@@ -767,13 +745,8 @@ bool SchedulerStateMachine::BeginFrameNeeded() const {
   if (!visible_)
     return false;
 
-  return (BeginFrameRequiredForAction() || BeginFrameRequiredForChildren() ||
-          BeginFrameNeededForVideo() || ProactiveBeginFrameWanted());
-}
-
-void SchedulerStateMachine::SetChildrenNeedBeginFrames(
-    bool children_need_begin_frames) {
-  children_need_begin_frames_ = children_need_begin_frames;
+  return BeginFrameRequiredForAction() || BeginFrameNeededForVideo() ||
+         ProactiveBeginFrameWanted();
 }
 
 void SchedulerStateMachine::SetVideoNeedsBeginFrames(
@@ -844,7 +817,7 @@ bool SchedulerStateMachine::ProactiveBeginFrameWanted() const {
 }
 
 void SchedulerStateMachine::OnBeginImplFrame() {
-  begin_impl_frame_state_ = BEGIN_IMPL_FRAME_STATE_BEGIN_FRAME_STARTING;
+  begin_impl_frame_state_ = BEGIN_IMPL_FRAME_STATE_INSIDE_BEGIN_FRAME;
   current_frame_number_++;
 
   last_commit_had_no_updates_ = false;
@@ -861,10 +834,6 @@ void SchedulerStateMachine::OnBeginImplFrame() {
     prepare_tiles_funnel_--;
 }
 
-void SchedulerStateMachine::OnBeginImplFrameDeadlinePending() {
-  begin_impl_frame_state_ = BEGIN_IMPL_FRAME_STATE_INSIDE_BEGIN_FRAME;
-}
-
 void SchedulerStateMachine::OnBeginImplFrameDeadline() {
   begin_impl_frame_state_ = BEGIN_IMPL_FRAME_STATE_INSIDE_DEADLINE;
 
@@ -953,10 +922,6 @@ bool SchedulerStateMachine::ShouldTriggerBeginImplFrameDeadlineImmediately()
   return false;
 }
 
-bool SchedulerStateMachine::main_thread_missed_last_deadline() const {
-  return main_thread_missed_last_deadline_;
-}
-
 bool SchedulerStateMachine::SwapThrottled() const {
   return pending_swaps_ >= kMaxPendingSwaps;
 }
@@ -1003,7 +968,8 @@ void SchedulerStateMachine::SetNeedsPrepareTiles() {
   }
 }
 void SchedulerStateMachine::DidSwapBuffers() {
-  TRACE_EVENT_ASYNC_BEGIN0("cc", "Scheduler:pending_swaps", this);
+  TRACE_EVENT_ASYNC_BEGIN1("cc", "Scheduler:pending_swaps", this,
+                           "pending_frames", pending_swaps_);
   DCHECK_LT(pending_swaps_, kMaxPendingSwaps);
 
   pending_swaps_++;
@@ -1014,7 +980,8 @@ void SchedulerStateMachine::DidSwapBuffers() {
 }
 
 void SchedulerStateMachine::DidSwapBuffersComplete() {
-  TRACE_EVENT_ASYNC_END0("cc", "Scheduler:pending_swaps", this);
+  TRACE_EVENT_ASYNC_END1("cc", "Scheduler:pending_swaps", this,
+                         "pending_frames", pending_swaps_);
   pending_swaps_--;
 }
 
@@ -1066,6 +1033,11 @@ void SchedulerStateMachine::NotifyReadyToCommit() {
 
 void SchedulerStateMachine::BeginMainFrameAborted(CommitEarlyOutReason reason) {
   DCHECK_EQ(begin_main_frame_state_, BEGIN_MAIN_FRAME_STATE_STARTED);
+
+  // If the main thread aborted, it doesn't matter if the  main thread missed
+  // the last deadline since it didn't have an update anyway.
+  main_thread_missed_last_deadline_ = false;
+
   switch (reason) {
     case CommitEarlyOutReason::ABORTED_OUTPUT_SURFACE_LOST:
     case CommitEarlyOutReason::ABORTED_NOT_VISIBLE:
