@@ -9,9 +9,10 @@
 #include <map>
 
 #include "base/bind.h"
-#include "base/containers/hash_tables.h"
+#include "base/containers/adapters.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/stl_util.h"
 #include "base/trace_event/trace_event.h"
 #include "cc/base/math_util.h"
@@ -20,7 +21,9 @@
 #include "cc/quads/draw_quad.h"
 #include "cc/quads/render_pass_draw_quad.h"
 #include "cc/quads/shared_quad_state.h"
+#include "cc/quads/solid_color_draw_quad.h"
 #include "cc/quads/surface_draw_quad.h"
+#include "cc/quads/texture_draw_quad.h"
 #include "cc/surfaces/surface.h"
 #include "cc/surfaces/surface_factory.h"
 #include "cc/surfaces/surface_manager.h"
@@ -31,8 +34,9 @@ namespace {
 
 void MoveMatchingRequests(
     RenderPassId id,
-    std::multimap<RenderPassId, scoped_ptr<CopyOutputRequest>>* copy_requests,
-    std::vector<scoped_ptr<CopyOutputRequest>>* output_requests) {
+    std::multimap<RenderPassId, std::unique_ptr<CopyOutputRequest>>*
+        copy_requests,
+    std::vector<std::unique_ptr<CopyOutputRequest>>* output_requests) {
   auto request_range = copy_requests->equal_range(id);
   for (auto it = request_range.first; it != request_range.second; ++it) {
     DCHECK(it->second);
@@ -43,15 +47,14 @@ void MoveMatchingRequests(
 
 }  // namespace
 
-SurfaceAggregator::SurfaceAggregator(SurfaceAggregatorClient* client,
-                                     SurfaceManager* manager,
+SurfaceAggregator::SurfaceAggregator(SurfaceManager* manager,
                                      ResourceProvider* provider,
                                      bool aggregate_only_damaged)
-    : client_(client),
-      manager_(manager),
+    : manager_(manager),
       provider_(provider),
       next_render_pass_id_(1),
-      aggregate_only_damaged_(aggregate_only_damaged) {
+      aggregate_only_damaged_(aggregate_only_damaged),
+      weak_factory_(this) {
   DCHECK(manager_);
 }
 
@@ -107,7 +110,7 @@ class SurfaceAggregator::RenderPassIdAllocator {
   }
 
  private:
-  base::hash_map<RenderPassId, int> id_to_index_map_;
+  std::unordered_map<RenderPassId, int, RenderPassIdHash> id_to_index_map_;
   int* next_index_;
 
   DISALLOW_COPY_AND_ASSIGN(RenderPassIdAllocator);
@@ -122,11 +125,10 @@ static void UnrefHelper(base::WeakPtr<SurfaceFactory> surface_factory,
 
 RenderPassId SurfaceAggregator::RemapPassId(RenderPassId surface_local_pass_id,
                                             SurfaceId surface_id) {
-  RenderPassIdAllocator* allocator = render_pass_allocator_map_.get(surface_id);
-  if (!allocator) {
-    allocator = new RenderPassIdAllocator(&next_render_pass_id_);
-    render_pass_allocator_map_.set(surface_id, make_scoped_ptr(allocator));
-  }
+  std::unique_ptr<RenderPassIdAllocator>& allocator =
+      render_pass_allocator_map_[surface_id];
+  if (!allocator)
+    allocator.reset(new RenderPassIdAllocator(&next_render_pass_id_));
   allocator->AddKnownPass(surface_local_pass_id);
   return allocator->Remap(surface_local_pass_id);
 }
@@ -153,14 +155,22 @@ gfx::Rect SurfaceAggregator::DamageRectForSurface(
     const RenderPass& source,
     const gfx::Rect& full_rect) const {
   auto it = previous_contained_surfaces_.find(surface->surface_id());
-  if (it == previous_contained_surfaces_.end())
-    return full_rect;
+  if (it != previous_contained_surfaces_.end()) {
+    int previous_index = it->second;
+    if (previous_index == surface->frame_index())
+      return gfx::Rect();
+  }
+  SurfaceId previous_surface_id = surface->previous_frame_surface_id();
+
+  if (surface->surface_id() != previous_surface_id) {
+    it = previous_contained_surfaces_.find(previous_surface_id);
+  }
+  if (it != previous_contained_surfaces_.end()) {
+    int previous_index = it->second;
+    if (previous_index == surface->frame_index() - 1)
+      return source.damage_rect;
+  }
 
-  int previous_index = it->second;
-  if (previous_index == surface->frame_index())
-    return gfx::Rect();
-  if (previous_index == surface->frame_index() - 1)
-    return source.damage_rect;
   return full_rect;
 }
 
@@ -177,14 +187,12 @@ void SurfaceAggregator::HandleSurfaceQuad(
   Surface* surface = manager_->GetSurfaceForId(surface_id);
   if (!surface)
     return;
-  const CompositorFrame* frame = surface->GetEligibleFrame();
-  if (!frame)
-    return;
-  const DelegatedFrameData* frame_data = frame->delegated_frame_data.get();
+  const CompositorFrame& frame = surface->GetEligibleFrame();
+  const DelegatedFrameData* frame_data = frame.delegated_frame_data.get();
   if (!frame_data)
     return;
 
-  std::multimap<RenderPassId, scoped_ptr<CopyOutputRequest>> copy_requests;
+  std::multimap<RenderPassId, std::unique_ptr<CopyOutputRequest>> copy_requests;
   surface->TakeCopyOutputRequests(&copy_requests);
 
   const RenderPassList& render_pass_list = frame_data->render_pass_list;
@@ -212,11 +220,12 @@ void SurfaceAggregator::HandleSurfaceQuad(
 
     size_t sqs_size = source.shared_quad_state_list.size();
     size_t dq_size = source.quad_list.size();
-    scoped_ptr<RenderPass> copy_pass(RenderPass::Create(sqs_size, dq_size));
+    std::unique_ptr<RenderPass> copy_pass(
+        RenderPass::Create(sqs_size, dq_size));
 
     RenderPassId remapped_pass_id = RemapPassId(source.id, surface_id);
 
-    copy_pass->SetAll(remapped_pass_id, source.output_rect, gfx::Rect(),
+    copy_pass->SetAll(remapped_pass_id, source.output_rect, source.output_rect,
                       source.transform_to_root_target,
                       source.has_transparent_background);
 
@@ -235,6 +244,17 @@ void SurfaceAggregator::HandleSurfaceQuad(
                     child_to_parent_map, gfx::Transform(), ClipData(),
                     copy_pass.get(), surface_id);
 
+    if (!copy_request_passes_.count(remapped_pass_id) &&
+        !moved_pixel_passes_.count(remapped_pass_id)) {
+      gfx::Transform inverse_transform(gfx::Transform::kSkipInitialization);
+      if (copy_pass->transform_to_root_target.GetInverse(&inverse_transform)) {
+        gfx::Rect damage_rect_in_render_pass_space =
+            MathUtil::ProjectEnclosingClippedRect(inverse_transform,
+                                                  root_damage_rect_);
+        copy_pass->damage_rect.Intersect(damage_rect_in_render_pass_space);
+      }
+    }
+
     dest_pass_list_->push_back(std::move(copy_pass));
   }
 
@@ -295,7 +315,7 @@ SharedQuadState* SurfaceAggregator::CopySharedQuadState(
     RenderPass* dest_render_pass) {
   SharedQuadState* copy_shared_quad_state =
       dest_render_pass->CreateAndAppendSharedQuadState();
-  copy_shared_quad_state->CopyFrom(source_sqs);
+  *copy_shared_quad_state = *source_sqs;
   // target_transform contains any transformation that may exist
   // between the context that these quads are being copied from (i.e. the
   // surface's draw transform when aggregated from within a surface) to the
@@ -313,18 +333,22 @@ SharedQuadState* SurfaceAggregator::CopySharedQuadState(
   return copy_shared_quad_state;
 }
 
-static gfx::Rect CalculateQuadSpaceDamageRect(
+// Returns true if the damage rect is valid.
+static bool CalculateQuadSpaceDamageRect(
     const gfx::Transform& quad_to_target_transform,
     const gfx::Transform& target_to_root_transform,
-    const gfx::Rect& root_damage_rect) {
+    const gfx::Rect& root_damage_rect,
+    gfx::Rect* quad_space_damage_rect) {
   gfx::Transform quad_to_root_transform(target_to_root_transform,
                                         quad_to_target_transform);
   gfx::Transform inverse_transform(gfx::Transform::kSkipInitialization);
   bool inverse_valid = quad_to_root_transform.GetInverse(&inverse_transform);
-  DCHECK(inverse_valid);
+  if (!inverse_valid)
+    return false;
 
-  return MathUtil::ProjectEnclosingClippedRect(inverse_transform,
-                                               root_damage_rect);
+  *quad_space_damage_rect = MathUtil::ProjectEnclosingClippedRect(
+      inverse_transform, root_damage_rect);
+  return true;
 }
 
 void SurfaceAggregator::CopyQuadsToPass(
@@ -339,11 +363,14 @@ void SurfaceAggregator::CopyQuadsToPass(
   const SharedQuadState* dest_shared_quad_state = nullptr;
   // If the current frame has copy requests then aggregate the entire
   // thing, as otherwise parts of the copy requests may be ignored.
-  const bool ignore_undamaged = aggregate_only_damaged_ && !has_copy_requests_;
+  const bool ignore_undamaged = aggregate_only_damaged_ &&
+                                !has_copy_requests_ &&
+                                !moved_pixel_passes_.count(dest_pass->id);
   // Damage rect in the quad space of the current shared quad state.
   // TODO(jbauman): This rect may contain unnecessary area if
   // transform isn't axis-aligned.
   gfx::Rect damage_rect_in_quad_space;
+  bool damage_rect_in_quad_space_valid = false;
 
 #if DCHECK_IS_ON()
   // If quads have come in with SharedQuadState out of order, or when quads have
@@ -370,10 +397,11 @@ void SurfaceAggregator::CopyQuadsToPass(
         gfx::Transform quad_to_target_transform(
             target_transform,
             quad->shared_quad_state->quad_to_target_transform);
-        damage_rect_in_quad_space = CalculateQuadSpaceDamageRect(
+        damage_rect_in_quad_space_valid = CalculateQuadSpaceDamageRect(
             quad_to_target_transform, dest_pass->transform_to_root_target,
-            root_damage_rect_);
-        if (!damage_rect_in_quad_space.Intersects(quad->visible_rect))
+            root_damage_rect_, &damage_rect_in_quad_space);
+        if (damage_rect_in_quad_space_valid &&
+            !damage_rect_in_quad_space.Intersects(quad->visible_rect))
           continue;
       }
       HandleSurfaceQuad(surface_quad, target_transform, clip_rect, dest_pass);
@@ -383,14 +411,16 @@ void SurfaceAggregator::CopyQuadsToPass(
             quad->shared_quad_state, target_transform, clip_rect, dest_pass);
         last_copied_source_shared_quad_state = quad->shared_quad_state;
         if (aggregate_only_damaged_ && !has_copy_requests_) {
-          damage_rect_in_quad_space = CalculateQuadSpaceDamageRect(
+          damage_rect_in_quad_space_valid = CalculateQuadSpaceDamageRect(
               dest_shared_quad_state->quad_to_target_transform,
-              dest_pass->transform_to_root_target, root_damage_rect_);
+              dest_pass->transform_to_root_target, root_damage_rect_,
+              &damage_rect_in_quad_space);
         }
       }
 
       if (ignore_undamaged) {
-        if (!damage_rect_in_quad_space.Intersects(quad->visible_rect))
+        if (damage_rect_in_quad_space_valid &&
+            !damage_rect_in_quad_space.Intersects(quad->visible_rect))
           continue;
       }
 
@@ -404,6 +434,20 @@ void SurfaceAggregator::CopyQuadsToPass(
 
         dest_quad = dest_pass->CopyFromAndAppendRenderPassDrawQuad(
             pass_quad, dest_shared_quad_state, remapped_pass_id);
+      } else if (quad->material == DrawQuad::TEXTURE_CONTENT) {
+        const TextureDrawQuad* texture_quad =
+            TextureDrawQuad::MaterialCast(quad);
+        if (texture_quad->secure_output_only &&
+            (!output_is_secure_ || copy_request_passes_.count(dest_pass->id))) {
+          SolidColorDrawQuad* solid_color_quad =
+              dest_pass->CreateAndAppendDrawQuad<SolidColorDrawQuad>();
+          solid_color_quad->SetNew(dest_shared_quad_state, quad->rect,
+                                   quad->visible_rect, SK_ColorBLACK, false);
+          dest_quad = solid_color_quad;
+        } else {
+          dest_quad = dest_pass->CopyFromAndAppendDrawQuad(
+              quad, dest_shared_quad_state);
+        }
       } else {
         dest_quad =
             dest_pass->CopyFromAndAppendDrawQuad(quad, dest_shared_quad_state);
@@ -427,7 +471,7 @@ void SurfaceAggregator::CopyPasses(const DelegatedFrameData* frame_data,
                                    Surface* surface) {
   // The root surface is allowed to have copy output requests, so grab them
   // off its render passes.
-  std::multimap<RenderPassId, scoped_ptr<CopyOutputRequest>> copy_requests;
+  std::multimap<RenderPassId, std::unique_ptr<CopyOutputRequest>> copy_requests;
   surface->TakeCopyOutputRequests(&copy_requests);
 
   const RenderPassList& source_pass_list = frame_data->render_pass_list;
@@ -446,20 +490,31 @@ void SurfaceAggregator::CopyPasses(const DelegatedFrameData* frame_data,
 
     size_t sqs_size = source.shared_quad_state_list.size();
     size_t dq_size = source.quad_list.size();
-    scoped_ptr<RenderPass> copy_pass(RenderPass::Create(sqs_size, dq_size));
+    std::unique_ptr<RenderPass> copy_pass(
+        RenderPass::Create(sqs_size, dq_size));
 
     MoveMatchingRequests(source.id, &copy_requests, &copy_pass->copy_requests);
 
     RenderPassId remapped_pass_id =
         RemapPassId(source.id, surface->surface_id());
 
-    copy_pass->SetAll(remapped_pass_id, source.output_rect, gfx::Rect(),
+    copy_pass->SetAll(remapped_pass_id, source.output_rect, source.output_rect,
                       source.transform_to_root_target,
                       source.has_transparent_background);
 
     CopyQuadsToPass(source.quad_list, source.shared_quad_state_list,
                     child_to_parent_map, gfx::Transform(), ClipData(),
                     copy_pass.get(), surface->surface_id());
+    if (!copy_request_passes_.count(remapped_pass_id) &&
+        !moved_pixel_passes_.count(remapped_pass_id)) {
+      gfx::Transform inverse_transform(gfx::Transform::kSkipInitialization);
+      if (copy_pass->transform_to_root_target.GetInverse(&inverse_transform)) {
+        gfx::Rect damage_rect_in_render_pass_space =
+            MathUtil::ProjectEnclosingClippedRect(inverse_transform,
+                                                  root_damage_rect_);
+        copy_pass->damage_rect.Intersect(damage_rect_in_render_pass_space);
+      }
+    }
 
     dest_pass_list_->push_back(std::move(copy_pass));
   }
@@ -480,26 +535,23 @@ void SurfaceAggregator::ProcessAddedAndRemovedSurfaces() {
       Surface* surface_ptr = manager_->GetSurfaceForId(surface.first);
       if (surface_ptr) {
         surface_ptr->RunDrawCallbacks(SurfaceDrawStatus::DRAW_SKIPPED);
-        client_->RemoveSurface(surface_ptr);
       }
     }
   }
-
-  for (const auto& surface : contained_surfaces_) {
-    if (!previous_contained_surfaces_.count(surface.first)) {
-      // Notify client of added surface.
-      Surface* surface_ptr = manager_->GetSurfaceForId(surface.first);
-      if (surface_ptr)
-        client_->AddSurface(surface_ptr);
-    }
-  }
 }
 
 // Walk the Surface tree from surface_id. Validate the resources of the current
 // surface and its descendants, check if there are any copy requests, and
 // return the combined damage rect.
 gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
+                                         bool in_moved_pixel_pass,
+                                         RenderPassId parent_pass,
                                          PrewalkResult* result) {
+  // This is for debugging a possible use after free.
+  // TODO(jbauman): Remove this once we have enough information.
+  // http://crbug.com/560181
+  base::WeakPtr<SurfaceAggregator> debug_weak_this = weak_factory_.GetWeakPtr();
+
   if (referenced_surfaces_.count(surface_id))
     return gfx::Rect();
   Surface* surface = manager_->GetSurfaceForId(surface_id);
@@ -508,11 +560,9 @@ gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
     return gfx::Rect();
   }
   contained_surfaces_[surface_id] = surface->frame_index();
-  const CompositorFrame* surface_frame = surface->GetEligibleFrame();
-  if (!surface_frame)
-    return gfx::Rect();
+  const CompositorFrame& surface_frame = surface->GetEligibleFrame();
   const DelegatedFrameData* frame_data =
-      surface_frame->delegated_frame_data.get();
+      surface_frame.delegated_frame_data.get();
   if (!frame_data)
     return gfx::Rect();
   int child_id = 0;
@@ -523,29 +573,38 @@ gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
       surface->factory()->RefResources(frame_data->resource_list);
     provider_->ReceiveFromChild(child_id, frame_data->resource_list);
   }
+  CHECK(debug_weak_this.get());
 
   ResourceProvider::ResourceIdSet referenced_resources;
   size_t reserve_size = frame_data->resource_list.size();
-#if defined(COMPILER_MSVC)
   referenced_resources.reserve(reserve_size);
-#elif defined(COMPILER_GCC)
-  // Pre-standard hash-tables only implement resize, which behaves similarly
-  // to reserve for these keys. Resizing to 0 may also be broken (particularly
-  // on stlport).
-  // TODO(jbauman): Replace with reserve when C++11 is supported everywhere.
-  if (reserve_size)
-    referenced_resources.resize(reserve_size);
-#endif
 
   bool invalid_frame = false;
   ResourceProvider::ResourceIdMap empty_map;
   const ResourceProvider::ResourceIdMap& child_to_parent_map =
       provider_ ? provider_->GetChildToParentMap(child_id) : empty_map;
 
-  // Each pair in the vector is a child surface and the transform from its
-  // target to the root target of this surface.
-  std::vector<std::pair<SurfaceId, gfx::Transform>> child_surfaces;
-  for (const auto& render_pass : frame_data->render_pass_list) {
+  CHECK(debug_weak_this.get());
+  if (!frame_data->render_pass_list.empty()) {
+    RenderPassId remapped_pass_id =
+        RemapPassId(frame_data->render_pass_list.back()->id, surface_id);
+    if (in_moved_pixel_pass)
+      moved_pixel_passes_.insert(remapped_pass_id);
+    if (parent_pass.IsValid())
+      render_pass_dependencies_[parent_pass].insert(remapped_pass_id);
+  }
+
+  struct SurfaceInfo {
+    SurfaceId id;
+    bool has_moved_pixels;
+    RenderPassId parent_pass;
+    gfx::Transform target_to_surface_transform;
+  };
+  std::vector<SurfaceInfo> child_surfaces;
+
+  for (const auto& render_pass : base::Reversed(frame_data->render_pass_list)) {
+    RenderPassId remapped_pass_id = RemapPassId(render_pass->id, surface_id);
+    bool in_moved_pixel_pass = !!moved_pixel_passes_.count(remapped_pass_id);
     for (const auto& quad : render_pass->quad_list) {
       if (quad->material == DrawQuad::SURFACE_CONTENT) {
         const SurfaceDrawQuad* surface_quad =
@@ -553,8 +612,22 @@ gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
         gfx::Transform target_to_surface_transform(
             render_pass->transform_to_root_target,
             surface_quad->shared_quad_state->quad_to_target_transform);
-        child_surfaces.push_back(std::make_pair(surface_quad->surface_id,
-                                                target_to_surface_transform));
+        child_surfaces.push_back(
+            SurfaceInfo{surface_quad->surface_id, in_moved_pixel_pass,
+                        remapped_pass_id, target_to_surface_transform});
+      } else if (quad->material == DrawQuad::RENDER_PASS) {
+        const RenderPassDrawQuad* render_pass_quad =
+            RenderPassDrawQuad::MaterialCast(quad);
+        if (in_moved_pixel_pass ||
+            render_pass_quad->filters.HasFilterThatMovesPixels()) {
+          moved_pixel_passes_.insert(
+              RemapPassId(render_pass_quad->render_pass_id, surface_id));
+        }
+        if (render_pass_quad->background_filters.HasFilterThatMovesPixels()) {
+          in_moved_pixel_pass = true;
+        }
+        render_pass_dependencies_[remapped_pass_id].insert(
+            RemapPassId(render_pass_quad->render_pass_id, surface_id));
       }
 
       if (!provider_)
@@ -571,14 +644,18 @@ gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
 
   if (invalid_frame)
     return gfx::Rect();
+  CHECK(debug_weak_this.get());
   valid_surfaces_.insert(surface->surface_id());
 
   if (provider_)
     provider_->DeclareUsedResourcesFromChild(child_id, referenced_resources);
+  CHECK(debug_weak_this.get());
 
   gfx::Rect damage_rect;
+  gfx::Rect full_damage;
   if (!frame_data->render_pass_list.empty()) {
     RenderPass* last_pass = frame_data->render_pass_list.back().get();
+    full_damage = last_pass->output_rect;
     damage_rect =
         DamageRectForSurface(surface, *last_pass, last_pass->output_rect);
   }
@@ -588,23 +665,41 @@ gfx::Rect SurfaceAggregator::PrewalkTree(SurfaceId surface_id,
   SurfaceSet::iterator it =
       referenced_surfaces_.insert(surface->surface_id()).first;
   for (const auto& surface_info : child_surfaces) {
-    gfx::Rect surface_damage = PrewalkTree(surface_info.first, result);
-    damage_rect.Union(
-        MathUtil::MapEnclosingClippedRect(surface_info.second, surface_damage));
+    gfx::Rect surface_damage =
+        PrewalkTree(surface_info.id, surface_info.has_moved_pixels,
+                    surface_info.parent_pass, result);
+    if (surface_damage.IsEmpty())
+      continue;
+    if (surface_info.has_moved_pixels) {
+      // Areas outside the rect hit by target_to_surface_transform may be
+      // modified if there is a filter that moves pixels.
+      damage_rect = full_damage;
+      continue;
+    }
+
+    damage_rect.Union(MathUtil::MapEnclosingClippedRect(
+        surface_info.target_to_surface_transform, surface_damage));
   }
 
-  for (const auto& surface_id : surface_frame->metadata.referenced_surfaces) {
+  CHECK(debug_weak_this.get());
+  for (const auto& surface_id : surface_frame.metadata.referenced_surfaces) {
     if (!contained_surfaces_.count(surface_id)) {
       result->undrawn_surfaces.insert(surface_id);
-      PrewalkTree(surface_id, result);
+      PrewalkTree(surface_id, false, RenderPassId(), result);
     }
   }
 
+  CHECK(debug_weak_this.get());
   if (surface->factory())
     surface->factory()->WillDrawSurface(surface->surface_id(), damage_rect);
 
-  for (const auto& render_pass : frame_data->render_pass_list)
-    result->has_copy_requests |= !render_pass->copy_requests.empty();
+  CHECK(debug_weak_this.get());
+  for (const auto& render_pass : frame_data->render_pass_list) {
+    if (!render_pass->copy_requests.empty()) {
+      RenderPassId remapped_pass_id = RemapPassId(render_pass->id, surface_id);
+      copy_request_passes_.insert(remapped_pass_id);
+    }
+  }
 
   referenced_surfaces_.erase(it);
   return damage_rect;
@@ -624,19 +719,19 @@ void SurfaceAggregator::CopyUndrawnSurfaces(PrewalkResult* prewalk_result) {
     Surface* surface = manager_->GetSurfaceForId(surface_id);
     if (!surface)
       continue;
-    const CompositorFrame* surface_frame = surface->GetEligibleFrame();
-    if (!surface_frame)
+    const CompositorFrame& surface_frame = surface->GetEligibleFrame();
+    if (!surface_frame.delegated_frame_data)
       continue;
     bool surface_has_copy_requests = false;
     for (const auto& render_pass :
-         surface_frame->delegated_frame_data->render_pass_list) {
+         surface_frame.delegated_frame_data->render_pass_list) {
       surface_has_copy_requests |= !render_pass->copy_requests.empty();
     }
     if (!surface_has_copy_requests) {
       // Children are not necessarily included in undrawn_surfaces (because
       // they weren't referenced directly from a drawn surface), but may have
       // copy requests, so make sure to check them as well.
-      for (const auto& child_id : surface_frame->metadata.referenced_surfaces) {
+      for (const auto& child_id : surface_frame.metadata.referenced_surfaces) {
         // Don't iterate over the child Surface if it was already listed as a
         // child of a different Surface, or in the case where there's infinite
         // recursion.
@@ -647,44 +742,64 @@ void SurfaceAggregator::CopyUndrawnSurfaces(PrewalkResult* prewalk_result) {
       }
     } else {
       SurfaceSet::iterator it = referenced_surfaces_.insert(surface_id).first;
-      CopyPasses(surface_frame->delegated_frame_data.get(), surface);
+      CopyPasses(surface_frame.delegated_frame_data.get(), surface);
       referenced_surfaces_.erase(it);
     }
   }
 }
 
-scoped_ptr<CompositorFrame> SurfaceAggregator::Aggregate(SurfaceId surface_id) {
+void SurfaceAggregator::PropagateCopyRequestPasses() {
+  std::vector<RenderPassId> copy_requests_to_iterate(
+      copy_request_passes_.begin(), copy_request_passes_.end());
+  while (!copy_requests_to_iterate.empty()) {
+    RenderPassId first = copy_requests_to_iterate.back();
+    copy_requests_to_iterate.pop_back();
+    auto it = render_pass_dependencies_.find(first);
+    if (it == render_pass_dependencies_.end())
+      continue;
+    for (auto pass : it->second) {
+      if (copy_request_passes_.insert(pass).second) {
+        copy_requests_to_iterate.push_back(pass);
+      }
+    }
+  }
+}
+
+CompositorFrame SurfaceAggregator::Aggregate(SurfaceId surface_id) {
   Surface* surface = manager_->GetSurfaceForId(surface_id);
   DCHECK(surface);
   contained_surfaces_[surface_id] = surface->frame_index();
-  const CompositorFrame* root_surface_frame = surface->GetEligibleFrame();
-  if (!root_surface_frame)
-    return nullptr;
+  const CompositorFrame& root_surface_frame = surface->GetEligibleFrame();
+  if (!root_surface_frame.delegated_frame_data)
+    return CompositorFrame();
   TRACE_EVENT0("cc", "SurfaceAggregator::Aggregate");
 
-  scoped_ptr<CompositorFrame> frame(new CompositorFrame);
-  frame->delegated_frame_data = make_scoped_ptr(new DelegatedFrameData);
+  CompositorFrame frame;
+  frame.delegated_frame_data = base::WrapUnique(new DelegatedFrameData);
 
-  DCHECK(root_surface_frame->delegated_frame_data);
-
-  dest_resource_list_ = &frame->delegated_frame_data->resource_list;
-  dest_pass_list_ = &frame->delegated_frame_data->render_pass_list;
+  dest_resource_list_ = &frame.delegated_frame_data->resource_list;
+  dest_pass_list_ = &frame.delegated_frame_data->render_pass_list;
 
   valid_surfaces_.clear();
   PrewalkResult prewalk_result;
-  root_damage_rect_ = PrewalkTree(surface_id, &prewalk_result);
-  has_copy_requests_ = prewalk_result.has_copy_requests;
+  root_damage_rect_ =
+      PrewalkTree(surface_id, false, RenderPassId(), &prewalk_result);
+  PropagateCopyRequestPasses();
+  has_copy_requests_ = !copy_request_passes_.empty();
 
   CopyUndrawnSurfaces(&prewalk_result);
   SurfaceSet::iterator it = referenced_surfaces_.insert(surface_id).first;
-  CopyPasses(root_surface_frame->delegated_frame_data.get(), surface);
+  CopyPasses(root_surface_frame.delegated_frame_data.get(), surface);
   referenced_surfaces_.erase(it);
 
+  moved_pixel_passes_.clear();
+  copy_request_passes_.clear();
+  render_pass_dependencies_.clear();
+
   DCHECK(referenced_surfaces_.empty());
 
   if (dest_pass_list_->empty())
-    return nullptr;
-  dest_pass_list_->back()->damage_rect = root_damage_rect_;
+    return CompositorFrame();
 
   dest_pass_list_ = NULL;
   ProcessAddedAndRemovedSurfaces();
@@ -696,7 +811,7 @@ scoped_ptr<CompositorFrame> SurfaceAggregator::Aggregate(SurfaceId surface_id) {
        ++it) {
     Surface* surface = manager_->GetSurfaceForId(it->first);
     if (surface)
-      surface->TakeLatencyInfo(&frame->metadata.latency_info);
+      surface->TakeLatencyInfo(&frame.metadata.latency_info);
   }
 
   // TODO(jamesr): Aggregate all resource references into the returned frame's
