@@ -6,6 +6,7 @@
 
 #include <stdint.h>
 
+#include "base/rand_util.h"
 #include "cc/surfaces/surface_manager.h"
 
 namespace cc {
@@ -26,14 +27,10 @@ SurfaceIdAllocator::~SurfaceIdAllocator() {
 }
 
 SurfaceId SurfaceIdAllocator::GenerateId() {
-  SurfaceId id(static_cast<uint64_t>(id_namespace_) << 32 | next_id_);
+  uint64_t nonce = base::RandUint64();
+  SurfaceId id(id_namespace_, next_id_, nonce);
   next_id_++;
   return id;
 }
 
-// static
-uint32_t SurfaceIdAllocator::NamespaceForId(SurfaceId id) {
-  return id.id >> 32;
-}
-
 }  // namespace cc
