@@ -12,12 +12,12 @@ namespace {
 
 void AppendTilingSetRequiredQueues(
     const std::vector<PictureLayerImpl*>& layers,
-    std::vector<scoped_ptr<TilingSetRasterQueueRequired>>* queues) {
+    std::vector<std::unique_ptr<TilingSetRasterQueueRequired>>* queues) {
   for (auto* layer : layers) {
     if (!layer->HasValidTilePriorities())
       continue;
 
-    scoped_ptr<TilingSetRasterQueueRequired> tiling_set_queue(
+    std::unique_ptr<TilingSetRasterQueueRequired> tiling_set_queue(
         new TilingSetRasterQueueRequired(
             layer->picture_layer_tiling_set(),
             RasterTilePriorityQueueRequired::Type::REQUIRED_FOR_ACTIVATION));
@@ -51,7 +51,7 @@ void RasterTilePriorityQueueRequired::BuildRequiredForDraw(
     if (!layer->HasValidTilePriorities())
       continue;
 
-    scoped_ptr<TilingSetRasterQueueRequired> tiling_set_queue(
+    std::unique_ptr<TilingSetRasterQueueRequired> tiling_set_queue(
         new TilingSetRasterQueueRequired(layer->picture_layer_tiling_set(),
                                          Type::REQUIRED_FOR_DRAW));
     if (!tiling_set_queue->IsEmpty())
