@@ -2,13 +2,21 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
+
 #include "build/build_config.h"
 #include "cc/layers/content_layer_client.h"
 #include "cc/layers/picture_image_layer.h"
 #include "cc/layers/picture_layer.h"
 #include "cc/layers/solid_color_layer.h"
+#include "cc/playback/display_item_list_settings.h"
+#include "cc/playback/drawing_display_item.h"
 #include "cc/test/layer_tree_pixel_resource_test.h"
 #include "cc/test/pixel_comparator.h"
+#include "cc/test/solid_color_content_layer_client.h"
+#include "third_party/skia/include/core/SkImage.h"
+#include "third_party/skia/include/core/SkPictureRecorder.h"
+#include "third_party/skia/include/core/SkSurface.h"
 
 #if !defined(OS_ANDROID)
 
@@ -25,10 +33,16 @@ class MaskContentLayerClient : public ContentLayerClient {
   ~MaskContentLayerClient() override {}
 
   bool FillsBoundsCompletely() const override { return false; }
+  size_t GetApproximateUnsharedMemoryUsage() const override { return 0; }
+
+  gfx::Rect PaintableRegion() override { return gfx::Rect(bounds_); }
+
+  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
+      PaintingControlSetting picture_control) override {
+    SkPictureRecorder recorder;
+    sk_sp<SkCanvas> canvas = sk_ref_sp(
+        recorder.beginRecording(gfx::RectToSkRect(gfx::Rect(bounds_))));
 
-  void PaintContents(SkCanvas* canvas,
-                     const gfx::Rect& rect,
-                     PaintingControlSetting picture_control) override {
     SkPaint paint;
     paint.setStyle(SkPaint::kStroke_Style);
     paint.setStrokeWidth(SkIntToScalar(2));
@@ -44,13 +58,14 @@ class MaskContentLayerClient : public ContentLayerClient {
           paint);
       inset_rect.Inset(3, 3, 2, 2);
     }
-  }
 
-  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
-      const gfx::Rect& clip,
-      PaintingControlSetting picture_control) override {
-    NOTIMPLEMENTED();
-    return nullptr;
+    scoped_refptr<DisplayItemList> display_list =
+        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
+    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        PaintableRegion(), recorder.finishRecordingAsPicture());
+
+    display_list->Finalize();
+    return display_list;
   }
 
  private:
@@ -67,8 +82,7 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfLayer) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
@@ -84,20 +98,20 @@ TEST_P(LayerTreeHostMasksPixelTest, ImageMaskOfLayer) {
 
   gfx::Size mask_bounds(50, 50);
 
-  scoped_refptr<PictureImageLayer> mask =
-      PictureImageLayer::Create(layer_settings());
+  scoped_refptr<PictureImageLayer> mask = PictureImageLayer::Create();
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
   mask->SetBounds(mask_bounds);
 
-  SkBitmap bitmap;
-  bitmap.allocN32Pixels(200, 200);
-  SkCanvas canvas(bitmap);
-  canvas.scale(SkIntToScalar(4), SkIntToScalar(4));
+  sk_sp<SkSurface> surface = SkSurface::MakeRasterN32Premul(200, 200);
+  SkCanvas* canvas = surface->getCanvas();
+  canvas->scale(SkIntToScalar(4), SkIntToScalar(4));
   MaskContentLayerClient client(mask_bounds);
-  client.PaintContents(&canvas, gfx::Rect(mask_bounds),
-                       ContentLayerClient::PAINTING_BEHAVIOR_NORMAL);
-  mask->SetBitmap(bitmap);
+  scoped_refptr<DisplayItemList> mask_display_list =
+      client.PaintContentsToDisplayList(
+          ContentLayerClient::PAINTING_BEHAVIOR_NORMAL);
+  mask_display_list->Raster(canvas, nullptr);
+  mask->SetImage(surface->makeImageSnapshot());
 
   scoped_refptr<SolidColorLayer> green = CreateSolidColorLayerWithBorder(
       gfx::Rect(25, 25, 50, 50), kCSSGreen, 1, SK_ColorBLACK);
@@ -113,8 +127,8 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfClippedLayer) {
       gfx::Rect(100, 100), SK_ColorWHITE);
 
   // Clip to the top half of the green layer.
-  scoped_refptr<Layer> clip = Layer::Create(layer_settings());
-  clip->SetPosition(gfx::Point(0, 0));
+  scoped_refptr<Layer> clip = Layer::Create();
+  clip->SetPosition(gfx::PointF());
   clip->SetBounds(gfx::Size(100, 50));
   clip->SetMasksToBounds(true);
   background->AddChild(clip);
@@ -125,8 +139,7 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfClippedLayer) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
@@ -143,8 +156,7 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplica) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
@@ -157,9 +169,44 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplica) {
   gfx::Transform replica_transform;
   replica_transform.Rotate(-90.0);
 
-  scoped_refptr<Layer> replica = Layer::Create(layer_settings());
+  scoped_refptr<Layer> replica = Layer::Create();
   replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
-  replica->SetPosition(gfx::Point(50, 50));
+  replica->SetPosition(gfx::PointF(50.f, 50.f));
+  replica->SetTransform(replica_transform);
+  green->SetReplicaLayer(replica.get());
+
+  RunPixelResourceTest(
+      background, base::FilePath(FILE_PATH_LITERAL("mask_with_replica.png")));
+}
+
+// Test collapsing of render passes containing a single tile quad and
+// drawing them directly with masks and replicas.
+TEST_P(LayerTreeHostMasksPixelTest, ContentLayerMaskWithReplica) {
+  scoped_refptr<SolidColorLayer> background =
+      CreateSolidColorLayer(gfx::Rect(100, 100), SK_ColorWHITE);
+
+  gfx::Size mask_bounds(50, 50);
+  MaskContentLayerClient client(mask_bounds);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
+  mask->SetBounds(mask_bounds);
+  mask->SetIsDrawable(true);
+  mask->SetIsMask(true);
+
+  // Use a border to defeat solid color detection to force a tile quad.
+  SolidColorContentLayerClient green_client(kCSSGreen, mask_bounds, 1,
+                                            SK_ColorBLACK);
+  scoped_refptr<PictureLayer> green = PictureLayer::Create(&green_client);
+  green->SetBounds(mask_bounds);
+  green->SetIsDrawable(true);
+  background->AddChild(green);
+  green->SetMaskLayer(mask.get());
+
+  gfx::Transform replica_transform;
+  replica_transform.Rotate(-90.0);
+
+  scoped_refptr<Layer> replica = Layer::Create();
+  replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
+  replica->SetPosition(gfx::PointF(50.f, 50.f));
   replica->SetTransform(replica_transform);
   green->SetReplicaLayer(replica.get());
 
@@ -173,16 +220,15 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplicaOfClippedLayer) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
 
   // Clip to the bottom half of the green layer, and the left half of the
   // replica.
-  scoped_refptr<Layer> clip = Layer::Create(layer_settings());
-  clip->SetPosition(gfx::Point(0, 25));
+  scoped_refptr<Layer> clip = Layer::Create();
+  clip->SetPosition(gfx::PointF(0.f, 25.f));
   clip->SetBounds(gfx::Size(75, 75));
   clip->SetMasksToBounds(true);
   background->AddChild(clip);
@@ -195,9 +241,9 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskWithReplicaOfClippedLayer) {
   gfx::Transform replica_transform;
   replica_transform.Rotate(-90.0);
 
-  scoped_refptr<Layer> replica = Layer::Create(layer_settings());
+  scoped_refptr<Layer> replica = Layer::Create();
   replica->SetTransformOrigin(gfx::Point3F(25.f, 25.f, 0.f));
-  replica->SetPosition(gfx::Point(50, 50));
+  replica->SetPosition(gfx::PointF(50.f, 50.f));
   replica->SetTransform(replica_transform);
   green->SetReplicaLayer(replica.get());
 
@@ -212,8 +258,7 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplica) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
@@ -230,9 +275,9 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplica) {
   replica_transform.Rotate(180.0);
   replica_transform.Translate(50.0, 0.0);
 
-  scoped_refptr<Layer> replica = Layer::Create(layer_settings());
+  scoped_refptr<Layer> replica = Layer::Create();
   replica->SetTransformOrigin(gfx::Point3F(50.f, 50.f, 0.f));
-  replica->SetPosition(gfx::Point());
+  replica->SetPosition(gfx::PointF());
   replica->SetTransform(replica_transform);
   replica->SetMaskLayer(mask.get());
   green->SetReplicaLayer(replica.get());
@@ -247,15 +292,14 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplicaOfClippedLayer) {
 
   gfx::Size mask_bounds(50, 50);
   MaskContentLayerClient client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
 
   // Clip to the bottom 3/4 of the green layer, and the top 3/4 of the replica.
-  scoped_refptr<Layer> clip = Layer::Create(layer_settings());
-  clip->SetPosition(gfx::Point(0, 12));
+  scoped_refptr<Layer> clip = Layer::Create();
+  clip->SetPosition(gfx::PointF(0.f, 12.f));
   clip->SetBounds(gfx::Size(100, 75));
   clip->SetMasksToBounds(true);
   background->AddChild(clip);
@@ -272,9 +316,9 @@ TEST_P(LayerTreeHostMasksPixelTest, MaskOfReplicaOfClippedLayer) {
   replica_transform.Rotate(180.0);
   replica_transform.Translate(50.0, 0.0);
 
-  scoped_refptr<Layer> replica = Layer::Create(layer_settings());
+  scoped_refptr<Layer> replica = Layer::Create();
   replica->SetTransformOrigin(gfx::Point3F(50.f, 50.f, 0.f));
-  replica->SetPosition(gfx::Point());
+  replica->SetPosition(gfx::PointF());
   replica->SetTransform(replica_transform);
   replica->SetMaskLayer(mask.get());
   green->SetReplicaLayer(replica.get());
@@ -292,9 +336,14 @@ class CheckerContentLayerClient : public ContentLayerClient {
       : bounds_(bounds), color_(color), vertical_(vertical) {}
   ~CheckerContentLayerClient() override {}
   bool FillsBoundsCompletely() const override { return false; }
-  void PaintContents(SkCanvas* canvas,
-                     const gfx::Rect& rect,
-                     PaintingControlSetting picture_control) override {
+  size_t GetApproximateUnsharedMemoryUsage() const override { return 0; }
+  gfx::Rect PaintableRegion() override { return gfx::Rect(bounds_); }
+  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
+      PaintingControlSetting picture_control) override {
+    SkPictureRecorder recorder;
+    sk_sp<SkCanvas> canvas = sk_ref_sp(
+        recorder.beginRecording(gfx::RectToSkRect(gfx::Rect(bounds_))));
+
     SkPaint paint;
     paint.setStyle(SkPaint::kStroke_Style);
     paint.setStrokeWidth(SkIntToScalar(4));
@@ -309,12 +358,14 @@ class CheckerContentLayerClient : public ContentLayerClient {
         canvas->drawLine(0, i, bounds_.width(), i, paint);
       }
     }
-  }
-  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
-      const gfx::Rect& clip,
-      PaintingControlSetting picture_control) override {
-    NOTIMPLEMENTED();
-    return nullptr;
+
+    scoped_refptr<DisplayItemList> display_list =
+        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
+    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        PaintableRegion(), recorder.finishRecordingAsPicture());
+
+    display_list->Finalize();
+    return display_list;
   }
 
  private:
@@ -329,9 +380,14 @@ class CircleContentLayerClient : public ContentLayerClient {
       : bounds_(bounds) {}
   ~CircleContentLayerClient() override {}
   bool FillsBoundsCompletely() const override { return false; }
-  void PaintContents(SkCanvas* canvas,
-                     const gfx::Rect& rect,
-                     PaintingControlSetting picture_control) override {
+  size_t GetApproximateUnsharedMemoryUsage() const override { return 0; }
+  gfx::Rect PaintableRegion() override { return gfx::Rect(bounds_); }
+  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
+      PaintingControlSetting picture_control) override {
+    SkPictureRecorder recorder;
+    sk_sp<SkCanvas> canvas = sk_ref_sp(
+        recorder.beginRecording(gfx::RectToSkRect(gfx::Rect(bounds_))));
+
     SkPaint paint;
     paint.setStyle(SkPaint::kFill_Style);
     paint.setColor(SK_ColorWHITE);
@@ -340,12 +396,14 @@ class CircleContentLayerClient : public ContentLayerClient {
                        bounds_.height() / 2,
                        bounds_.width() / 4,
                        paint);
-  }
-  scoped_refptr<DisplayItemList> PaintContentsToDisplayList(
-      const gfx::Rect& clip,
-      PaintingControlSetting picture_control) override {
-    NOTIMPLEMENTED();
-    return nullptr;
+
+    scoped_refptr<DisplayItemList> display_list =
+        DisplayItemList::Create(PaintableRegion(), DisplayItemListSettings());
+    display_list->CreateAndAppendItem<DrawingDisplayItem>(
+        PaintableRegion(), recorder.finishRecordingAsPicture());
+
+    display_list->Finalize();
+    return display_list;
   }
 
  private:
@@ -355,19 +413,16 @@ class CircleContentLayerClient : public ContentLayerClient {
 using LayerTreeHostMasksForBackgroundFiltersPixelTest =
     ParameterizedPixelResourceTest;
 
-INSTANTIATE_TEST_CASE_P(
-    PixelResourceTest,
-    LayerTreeHostMasksForBackgroundFiltersPixelTest,
-    ::testing::Values(
-        // SOFTWARE, Background filters aren't implemented in software
-        GL_GPU_RASTER_2D_DRAW,
-        GL_ONE_COPY_2D_STAGING_2D_DRAW,
-        GL_ONE_COPY_RECT_STAGING_2D_DRAW,
-        GL_ONE_COPY_EXTERNAL_STAGING_2D_DRAW,
-        GL_ZERO_COPY_2D_DRAW,
-        GL_ZERO_COPY_RECT_DRAW,
-        GL_ZERO_COPY_EXTERNAL_DRAW,
-        GL_ASYNC_UPLOAD_2D_DRAW));
+INSTANTIATE_TEST_CASE_P(PixelResourceTest,
+                        LayerTreeHostMasksForBackgroundFiltersPixelTest,
+                        ::testing::Values(SOFTWARE,
+                                          GL_GPU_RASTER_2D_DRAW,
+                                          GL_ONE_COPY_2D_STAGING_2D_DRAW,
+                                          GL_ONE_COPY_RECT_STAGING_2D_DRAW,
+                                          GL_ONE_COPY_EXTERNAL_STAGING_2D_DRAW,
+                                          GL_ZERO_COPY_2D_DRAW,
+                                          GL_ZERO_COPY_RECT_DRAW,
+                                          GL_ZERO_COPY_EXTERNAL_DRAW));
 
 TEST_P(LayerTreeHostMasksForBackgroundFiltersPixelTest,
        MaskOfLayerWithBackgroundFilter) {
@@ -376,8 +431,7 @@ TEST_P(LayerTreeHostMasksForBackgroundFiltersPixelTest,
 
   gfx::Size picture_bounds(100, 100);
   CheckerContentLayerClient picture_client(picture_bounds, SK_ColorGREEN, true);
-  scoped_refptr<PictureLayer> picture =
-      PictureLayer::Create(layer_settings(), &picture_client);
+  scoped_refptr<PictureLayer> picture = PictureLayer::Create(&picture_client);
   picture->SetBounds(picture_bounds);
   picture->SetIsDrawable(true);
 
@@ -387,13 +441,12 @@ TEST_P(LayerTreeHostMasksForBackgroundFiltersPixelTest,
   background->AddChild(blur);
 
   FilterOperations filters;
-  filters.Append(FilterOperation::CreateBlurFilter(1.5f));
+  filters.Append(FilterOperation::CreateGrayscaleFilter(1.0));
   blur->SetBackgroundFilters(filters);
 
   gfx::Size mask_bounds(100, 100);
   CircleContentLayerClient mask_client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &mask_client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&mask_client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
@@ -426,14 +479,14 @@ TEST_P(LayerTreeHostMasksForBackgroundFiltersPixelTest,
   CheckerContentLayerClient picture_client_vertical(
       picture_bounds, SK_ColorGREEN, true);
   scoped_refptr<PictureLayer> picture_vertical =
-      PictureLayer::Create(layer_settings(), &picture_client_vertical);
+      PictureLayer::Create(&picture_client_vertical);
   picture_vertical->SetBounds(picture_bounds);
   picture_vertical->SetIsDrawable(true);
 
   CheckerContentLayerClient picture_client_horizontal(
       picture_bounds, SK_ColorMAGENTA, false);
   scoped_refptr<PictureLayer> picture_horizontal =
-      PictureLayer::Create(layer_settings(), &picture_client_horizontal);
+      PictureLayer::Create(&picture_client_horizontal);
   picture_horizontal->SetBounds(picture_bounds);
   picture_horizontal->SetIsDrawable(true);
   picture_horizontal->SetContentsOpaque(false);
@@ -444,8 +497,7 @@ TEST_P(LayerTreeHostMasksForBackgroundFiltersPixelTest,
 
   gfx::Size mask_bounds(128, 128);
   CircleContentLayerClient mask_client(mask_bounds);
-  scoped_refptr<PictureLayer> mask =
-      PictureLayer::Create(layer_settings(), &mask_client);
+  scoped_refptr<PictureLayer> mask = PictureLayer::Create(&mask_client);
   mask->SetBounds(mask_bounds);
   mask->SetIsDrawable(true);
   mask->SetIsMask(true);
