@@ -4,6 +4,7 @@
 
 #include "cc/trees/property_tree.h"
 
+#include "cc/input/main_thread_scrolling_reason.h"
 #include "cc/proto/property_tree.pb.h"
 #include "cc/test/geometry_test_utils.h"
 #include "cc/trees/draw_property_utils.h"
@@ -18,20 +19,13 @@ TEST(PropertyTreeSerializationTest, TransformNodeDataSerialization) {
   original.local.Translate3d(3.f, 1.f, 5.f);
   original.post_local.Translate3d(1.f, 8.f, 3.f);
   original.to_parent.Translate3d(3.2f, 2.f, 3.f);
-  original.to_target.Translate3d(2.6f, 2.f, 3.f);
-  original.from_target.Translate3d(4.3f, 2.f, 3.f);
-  original.to_screen.Translate3d(7.2f, 2.f, 4.5f);
-  original.from_screen.Translate3d(2.f, 2.f, 7.f);
-  original.target_id = 3;
-  original.content_target_id = 4;
   original.source_node_id = 5;
   original.needs_local_transform_update = false;
   original.is_invertible = false;
   original.ancestors_are_invertible = false;
-  original.is_animated = false;
-  original.to_screen_is_animated = false;
+  original.has_potential_animation = false;
+  original.to_screen_is_potentially_animated = false;
   original.has_only_translation_animations = false;
-  original.to_screen_has_scale_animation = false;
   original.flattens_inherited_transform = false;
   original.node_and_ancestors_are_flat = false;
   original.node_and_ancestors_have_only_integer_translation = false;
@@ -43,10 +37,6 @@ TEST(PropertyTreeSerializationTest, TransformNodeDataSerialization) {
   original.affected_by_outer_viewport_bounds_delta_y = false;
   original.in_subtree_of_page_scale_layer = false;
   original.post_local_scale_factor = 0.5f;
-  original.local_maximum_animation_target_scale = 0.6f;
-  original.local_starting_animation_scale = 0.7f;
-  original.combined_maximum_animation_target_scale = 0.8f;
-  original.combined_starting_animation_scale = 0.9f;
   original.sublayer_scale = gfx::Vector2dF(0.5f, 0.5f);
   original.scroll_offset = gfx::ScrollOffset(1.5f, 1.5f);
   original.scroll_snap = gfx::Vector2dF(0.4f, 0.4f);
@@ -76,21 +66,26 @@ TEST(PropertyTreeSerializationTest, TransformNodeSerialization) {
 }
 
 TEST(PropertyTreeSerializationTest, TransformTreeSerialization) {
-  TransformTree original;
+  PropertyTrees property_trees;
+  TransformTree& original = property_trees.transform_tree;
   TransformNode& root = *original.Node(0);
-  root.data.target_id = 3;
-  root.data.content_target_id = 4;
+  root.id = 0;
+  root.owner_id = 1;
+  original.SetTargetId(root.id, 3);
+  original.SetContentTargetId(root.id, 4);
   TransformNode second;
+  second.owner_id = 2;
   second.data.local.Translate3d(2.f, 2.f, 0.f);
   second.data.source_node_id = 0;
-  second.data.target_id = 0;
+  second.id = original.Insert(second, 0);
+  original.SetTargetId(second.id, 0);
   TransformNode third;
+  third.owner_id = 3;
   third.data.scrolls = true;
   third.data.source_node_id = 1;
-  third.data.target_id = 0;
+  third.id = original.Insert(third, 1);
+  original.SetTargetId(third.id, 0);
 
-  original.Insert(second, 0);
-  original.Insert(third, 1);
   original.set_needs_update(true);
 
   original.set_page_scale_factor(0.5f);
@@ -99,16 +94,18 @@ TEST(PropertyTreeSerializationTest, TransformTreeSerialization) {
       gfx::Transform(1.05f, 2.15f, 3.14f, 4.13f, 5.12f, 6.11f, 7.1f, 8.9f, 9.8f,
                      10.7f, 11.6f, 12.5f, 13.4f, 14.3f, 15.2f, 16.1f);
   original.SetDeviceTransformScaleFactor(transform);
-  original.SetInnerViewportBoundsDelta(gfx::Vector2dF(0.4f, 0.6f));
-  original.SetOuterViewportBoundsDelta(gfx::Vector2dF(0.5f, 0.7f));
   original.AddNodeAffectedByInnerViewportBoundsDelta(0);
   original.AddNodeAffectedByOuterViewportBoundsDelta(1);
 
   proto::PropertyTree proto;
   original.ToProtobuf(&proto);
   TransformTree result;
-  result.FromProtobuf(proto);
+  std::unordered_map<int, int> transform_id_to_index_map;
+  result.FromProtobuf(proto, &transform_id_to_index_map);
 
+  EXPECT_EQ(transform_id_to_index_map[1], 0);
+  EXPECT_EQ(transform_id_to_index_map[2], 1);
+  EXPECT_EQ(transform_id_to_index_map[3], 2);
   EXPECT_EQ(original, result);
 }
 
@@ -151,12 +148,15 @@ TEST(PropertyTreeSerializationTest, ClipNodeSerialization) {
 TEST(PropertyTreeSerializationTest, ClipTreeSerialization) {
   ClipTree original;
   ClipNode& root = *original.Node(0);
+  root.owner_id = 1;
   root.data.transform_id = 2;
   root.data.target_id = 1;
   ClipNode second;
+  second.owner_id = 2;
   second.data.transform_id = 4;
   second.data.applies_local_clip = true;
   ClipNode third;
+  third.owner_id = 3;
   third.data.target_id = 3;
   third.data.target_is_clipped = false;
 
@@ -167,8 +167,12 @@ TEST(PropertyTreeSerializationTest, ClipTreeSerialization) {
   proto::PropertyTree proto;
   original.ToProtobuf(&proto);
   ClipTree result;
-  result.FromProtobuf(proto);
+  std::unordered_map<int, int> clip_id_to_index_map;
+  result.FromProtobuf(proto, &clip_id_to_index_map);
 
+  EXPECT_EQ(clip_id_to_index_map[1], 0);
+  EXPECT_EQ(clip_id_to_index_map[2], 1);
+  EXPECT_EQ(clip_id_to_index_map[3], 2);
   EXPECT_EQ(original, result);
 }
 
@@ -179,6 +183,9 @@ TEST(PropertyTreeSerializationTest, EffectNodeDataSerialization) {
   original.has_render_surface = false;
   original.transform_id = 2;
   original.clip_id = 3;
+  original.mask_layer_id = 6;
+  original.replica_layer_id = 10;
+  original.replica_mask_layer_id = 9;
 
   proto::TreeNode proto;
   original.ToProtobuf(&proto);
@@ -205,35 +212,153 @@ TEST(PropertyTreeSerializationTest, EffectNodeSerialization) {
 TEST(PropertyTreeSerializationTest, EffectTreeSerialization) {
   EffectTree original;
   EffectNode& root = *original.Node(0);
+  root.owner_id = 5;
   root.data.transform_id = 2;
   root.data.clip_id = 1;
   EffectNode second;
+  second.owner_id = 6;
   second.data.transform_id = 4;
   second.data.opacity = true;
+  second.data.mask_layer_id = 32;
   EffectNode third;
+  third.owner_id = 7;
   third.data.clip_id = 3;
+  third.data.replica_layer_id = 44;
+  third.data.replica_mask_layer_id = 45;
   third.data.has_render_surface = false;
 
   original.Insert(second, 0);
   original.Insert(third, 1);
+  original.AddMaskOrReplicaLayerId(32);
+  original.AddMaskOrReplicaLayerId(44);
+  original.AddMaskOrReplicaLayerId(45);
   original.set_needs_update(true);
 
   proto::PropertyTree proto;
   original.ToProtobuf(&proto);
   EffectTree result;
+  std::unordered_map<int, int> effect_id_to_index_map;
+  result.FromProtobuf(proto, &effect_id_to_index_map);
+
+  EXPECT_EQ(effect_id_to_index_map[5], 0);
+  EXPECT_EQ(effect_id_to_index_map[6], 1);
+  EXPECT_EQ(effect_id_to_index_map[7], 2);
+  EXPECT_EQ(original, result);
+}
+
+TEST(PropertyTreeSerializationTest, ScrollNodeDataSerialization) {
+  ScrollNodeData original;
+  original.scrollable = true;
+  original.main_thread_scrolling_reasons =
+      MainThreadScrollingReason::kScrollbarScrolling;
+  original.contains_non_fast_scrollable_region = false;
+  original.scroll_clip_layer_bounds = gfx::Size(10, 10);
+  original.bounds = gfx::Size(15, 15);
+  original.max_scroll_offset_affected_by_page_scale = true;
+  original.is_inner_viewport_scroll_layer = true;
+  original.is_outer_viewport_scroll_layer = false;
+
+  proto::TreeNode proto;
+  original.ToProtobuf(&proto);
+  ScrollNodeData result;
   result.FromProtobuf(proto);
 
   EXPECT_EQ(original, result);
 }
 
+TEST(PropertyTreeSerializationTest, ScrollNodeSerialization) {
+  ScrollNode original;
+  original.id = 3;
+  original.parent_id = 2;
+  original.owner_id = 4;
+
+  proto::TreeNode proto;
+  original.ToProtobuf(&proto);
+  ScrollNode result;
+  result.FromProtobuf(proto);
+
+  EXPECT_EQ(original, result);
+}
+
+TEST(PropertyTreeSerializationTest, ScrollTreeSerialization) {
+  PropertyTrees property_trees;
+  property_trees.is_main_thread = true;
+  ScrollTree& original = property_trees.scroll_tree;
+  ScrollNode second;
+  second.owner_id = 10;
+  second.data.scrollable = true;
+  second.data.bounds = gfx::Size(15, 15);
+  ScrollNode third;
+  third.owner_id = 20;
+  third.data.contains_non_fast_scrollable_region = true;
+
+  original.Insert(second, 0);
+  original.Insert(third, 1);
+
+  original.set_currently_scrolling_node(1);
+  original.SetScrollOffset(1, gfx::ScrollOffset(1, 2));
+
+  proto::PropertyTree proto;
+  original.ToProtobuf(&proto);
+  ScrollTree result;
+  std::unordered_map<int, int> scroll_id_to_index_map;
+  result.FromProtobuf(proto, &scroll_id_to_index_map);
+
+  EXPECT_EQ(original, result);
+  EXPECT_EQ(scroll_id_to_index_map[10], 1);
+  EXPECT_EQ(scroll_id_to_index_map[20], 2);
+
+  original.clear();
+  original.set_currently_scrolling_node(0);
+  original.SetScrollOffset(2, gfx::ScrollOffset(1, 2));
+
+  proto::PropertyTree proto2;
+  original.ToProtobuf(&proto2);
+  result = ScrollTree();
+  scroll_id_to_index_map.clear();
+  result.FromProtobuf(proto2, &scroll_id_to_index_map);
+
+  EXPECT_EQ(original, result);
+}
+
 TEST(PropertyTreeSerializationTest, PropertyTrees) {
   PropertyTrees original;
-  original.transform_tree.Insert(TransformNode(), 0);
-  original.transform_tree.Insert(TransformNode(), 1);
-  original.clip_tree.Insert(ClipNode(), 0);
-  original.clip_tree.Insert(ClipNode(), 1);
-  original.effect_tree.Insert(EffectNode(), 0);
-  original.effect_tree.Insert(EffectNode(), 1);
+  TransformNode transform_node1 = TransformNode();
+  transform_node1.owner_id = 10;
+  transform_node1.id = original.transform_tree.Insert(transform_node1, 0);
+  TransformNode transform_node2 = TransformNode();
+  transform_node2.owner_id = 20;
+  transform_node2.id = original.transform_tree.Insert(transform_node2, 1);
+  original.transform_id_to_index_map[10] = 1;
+  original.transform_id_to_index_map[20] = 2;
+
+  ClipNode clip_node1 = ClipNode();
+  clip_node1.owner_id = 10;
+  clip_node1.id = original.clip_tree.Insert(clip_node1, 0);
+  ClipNode clip_node2 = ClipNode();
+  clip_node2.owner_id = 22;
+  clip_node2.id = original.clip_tree.Insert(clip_node2, 1);
+  original.clip_id_to_index_map[10] = 1;
+  original.clip_id_to_index_map[22] = 2;
+
+  EffectNode effect_node1 = EffectNode();
+  effect_node1.owner_id = 11;
+  effect_node1.id = original.effect_tree.Insert(effect_node1, 0);
+  EffectNode effect_node2 = EffectNode();
+  effect_node2.owner_id = 23;
+  effect_node2.id = original.effect_tree.Insert(effect_node2, 1);
+  original.effect_id_to_index_map[11] = 1;
+  original.effect_id_to_index_map[23] = 2;
+
+  ScrollNode scroll_node1 = ScrollNode();
+  scroll_node1.owner_id = 10;
+  scroll_node1.id = original.scroll_tree.Insert(scroll_node1, 0);
+  ScrollNode scroll_node2 = ScrollNode();
+  scroll_node2.owner_id = 20;
+  scroll_node2.id = original.scroll_tree.Insert(scroll_node2, 1);
+  original.scroll_id_to_index_map[10] = 1;
+  original.scroll_id_to_index_map[20] = 2;
+
   original.needs_rebuild = false;
   original.non_root_surfaces_enabled = false;
   original.sequence_number = 3;
@@ -258,30 +383,34 @@ class PropertyTreeTest : public testing::Test {
 
   virtual void StartTest() = 0;
 
-  TransformTree TransformTreeForTest(const TransformTree& transform_tree) {
-    if (!test_serialization_) {
-      return transform_tree;
-    }
+  void SetupTransformTreeForTest(TransformTree* transform_tree) {
+    if (!test_serialization_)
+      return;
+
     TransformTree new_tree;
     proto::PropertyTree proto;
-    transform_tree.ToProtobuf(&proto);
-    new_tree.FromProtobuf(proto);
+    transform_tree->ToProtobuf(&proto);
+    std::unordered_map<int, int> transform_id_to_index_map;
+    new_tree.FromProtobuf(proto, &transform_id_to_index_map);
+    EXPECT_EQ(*transform_tree, new_tree);
 
-    EXPECT_EQ(transform_tree, new_tree);
-    return new_tree;
+    PropertyTrees* property_trees = transform_tree->property_trees();
+    *transform_tree = new_tree;
+    transform_tree->SetPropertyTrees(property_trees);
   }
 
-  EffectTree EffectTreeForTest(const EffectTree& effect_tree) {
-    if (!test_serialization_) {
-      return effect_tree;
-    }
+  void SetupEffectTreeForTest(EffectTree* effect_tree) {
+    if (!test_serialization_)
+      return;
+
     EffectTree new_tree;
     proto::PropertyTree proto;
-    effect_tree.ToProtobuf(&proto);
-    new_tree.FromProtobuf(proto);
+    effect_tree->ToProtobuf(&proto);
+    std::unordered_map<int, int> effect_id_to_index_map;
+    new_tree.FromProtobuf(proto, &effect_id_to_index_map);
 
-    EXPECT_EQ(effect_tree, new_tree);
-    return new_tree;
+    EXPECT_EQ(*effect_tree, new_tree);
+    *effect_tree = new_tree;
   }
 
  private:
@@ -301,11 +430,13 @@ class PropertyTreeTest : public testing::Test {
 class PropertyTreeTestComputeTransformRoot : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
+    root.id = 0;
     root.data.local.Translate(2, 2);
-    root.data.target_id = 0;
-    tree = TransformTreeForTest(tree);
+    tree.SetTargetId(root.id, 0);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(0);
 
     gfx::Transform expected;
@@ -335,19 +466,20 @@ DIRECT_AND_SERIALIZED_PROPERTY_TREE_TEST_F(
 class PropertyTreeTestComputeTransformChild : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
     root.data.local.Translate(2, 2);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode child;
     child.data.local.Translate(3, 3);
-    child.data.target_id = 0;
     child.data.source_node_id = 0;
+    child.id = tree.Insert(child, 0);
+    tree.SetTargetId(child.id, 0);
 
-    tree.Insert(child, 0);
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(1);
 
     gfx::Transform expected;
@@ -387,26 +519,26 @@ DIRECT_AND_SERIALIZED_PROPERTY_TREE_TEST_F(
 class PropertyTreeTestComputeTransformSibling : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
     root.data.local.Translate(2, 2);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode child;
     child.data.local.Translate(3, 3);
     child.data.source_node_id = 0;
-    child.data.target_id = 0;
+    child.id = tree.Insert(child, 0);
+    tree.SetTargetId(child.id, 0);
 
     TransformNode sibling;
     sibling.data.local.Translate(7, 7);
     sibling.data.source_node_id = 0;
-    sibling.data.target_id = 0;
+    sibling.id = tree.Insert(sibling, 0);
+    tree.SetTargetId(sibling.id, 0);
 
-    tree.Insert(child, 0);
-    tree.Insert(sibling, 0);
-
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
 
     tree.UpdateTransforms(1);
     tree.UpdateTransforms(2);
@@ -444,32 +576,32 @@ class PropertyTreeTestComputeTransformSiblingSingularAncestor
     // transform, we cannot use screen space transforms to compute change of
     // basis
     // transforms between these nodes.
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
     root.data.local.Translate(2, 2);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode singular;
     singular.data.local.matrix().set(2, 2, 0.0);
     singular.data.source_node_id = 0;
-    singular.data.target_id = 0;
+    singular.id = tree.Insert(singular, 0);
+    tree.SetTargetId(singular.id, 0);
 
     TransformNode child;
     child.data.local.Translate(3, 3);
     child.data.source_node_id = 1;
-    child.data.target_id = 0;
+    child.id = tree.Insert(child, 1);
+    tree.SetTargetId(child.id, 0);
 
     TransformNode sibling;
     sibling.data.local.Translate(7, 7);
     sibling.data.source_node_id = 1;
-    sibling.data.target_id = 0;
+    sibling.id = tree.Insert(sibling, 1);
+    tree.SetTargetId(sibling.id, 0);
 
-    tree.Insert(singular, 0);
-    tree.Insert(child, 1);
-    tree.Insert(sibling, 1);
-
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
 
     tree.UpdateTransforms(1);
     tree.UpdateTransforms(2);
@@ -498,11 +630,12 @@ DIRECT_AND_SERIALIZED_PROPERTY_TREE_TEST_F(
 class PropertyTreeTestTransformsWithFlattening : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
 
     int grand_parent = tree.Insert(TransformNode(), 0);
-    tree.Node(grand_parent)->data.content_target_id = grand_parent;
-    tree.Node(grand_parent)->data.target_id = grand_parent;
+    tree.SetContentTargetId(grand_parent, grand_parent);
+    tree.SetTargetId(grand_parent, grand_parent);
     tree.Node(grand_parent)->data.source_node_id = 0;
 
     gfx::Transform rotation_about_x;
@@ -510,47 +643,45 @@ class PropertyTreeTestTransformsWithFlattening : public PropertyTreeTest {
 
     int parent = tree.Insert(TransformNode(), grand_parent);
     tree.Node(parent)->data.needs_sublayer_scale = true;
-    tree.Node(parent)->data.target_id = grand_parent;
-    tree.Node(parent)->data.content_target_id = parent;
+    tree.SetTargetId(parent, grand_parent);
+    tree.SetContentTargetId(parent, parent);
     tree.Node(parent)->data.source_node_id = grand_parent;
     tree.Node(parent)->data.local = rotation_about_x;
 
     int child = tree.Insert(TransformNode(), parent);
-    tree.Node(child)->data.target_id = parent;
-    tree.Node(child)->data.content_target_id = parent;
+    tree.SetTargetId(child, parent);
+    tree.SetContentTargetId(child, parent);
     tree.Node(child)->data.source_node_id = parent;
     tree.Node(child)->data.flattens_inherited_transform = true;
     tree.Node(child)->data.local = rotation_about_x;
 
     int grand_child = tree.Insert(TransformNode(), child);
-    tree.Node(grand_child)->data.target_id = parent;
-    tree.Node(grand_child)->data.content_target_id = parent;
+    tree.SetTargetId(grand_child, parent);
+    tree.SetContentTargetId(grand_child, parent);
     tree.Node(grand_child)->data.source_node_id = child;
     tree.Node(grand_child)->data.flattens_inherited_transform = true;
     tree.Node(grand_child)->data.local = rotation_about_x;
 
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     gfx::Transform flattened_rotation_about_x = rotation_about_x;
     flattened_rotation_about_x.FlattenTo2d();
 
-    EXPECT_TRANSFORMATION_MATRIX_EQ(rotation_about_x,
-                                    tree.Node(child)->data.to_target);
+    EXPECT_TRANSFORMATION_MATRIX_EQ(rotation_about_x, tree.ToTarget(child));
+
+    EXPECT_TRANSFORMATION_MATRIX_EQ(
+        flattened_rotation_about_x * rotation_about_x, tree.ToScreen(child));
 
     EXPECT_TRANSFORMATION_MATRIX_EQ(
         flattened_rotation_about_x * rotation_about_x,
-        tree.Node(child)->data.to_screen);
-
-    EXPECT_TRANSFORMATION_MATRIX_EQ(
-        flattened_rotation_about_x * rotation_about_x,
-        tree.Node(grand_child)->data.to_target);
+        tree.ToTarget(grand_child));
 
     EXPECT_TRANSFORMATION_MATRIX_EQ(flattened_rotation_about_x *
                                         flattened_rotation_about_x *
                                         rotation_about_x,
-                                    tree.Node(grand_child)->data.to_screen);
+                                    tree.ToScreen(grand_child));
 
     gfx::Transform grand_child_to_child;
     bool success =
@@ -561,15 +692,15 @@ class PropertyTreeTestTransformsWithFlattening : public PropertyTreeTest {
     // Remove flattening at grand_child, and recompute transforms.
     tree.Node(grand_child)->data.flattens_inherited_transform = false;
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     EXPECT_TRANSFORMATION_MATRIX_EQ(rotation_about_x * rotation_about_x,
-                                    tree.Node(grand_child)->data.to_target);
+                                    tree.ToTarget(grand_child));
 
     EXPECT_TRANSFORMATION_MATRIX_EQ(
         flattened_rotation_about_x * rotation_about_x * rotation_about_x,
-        tree.Node(grand_child)->data.to_screen);
+        tree.ToScreen(grand_child));
 
     success = tree.ComputeTransform(grand_child, child, &grand_child_to_child);
     EXPECT_TRUE(success);
@@ -583,19 +714,20 @@ DIRECT_AND_SERIALIZED_PROPERTY_TREE_TEST_F(
 class PropertyTreeTestMultiplicationOrder : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
     root.data.local.Translate(2, 2);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode child;
     child.data.local.Scale(2, 2);
-    child.data.target_id = 0;
     child.data.source_node_id = 0;
+    child.id = tree.Insert(child, 0);
+    tree.SetTargetId(child.id, 0);
 
-    tree.Insert(child, 0);
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(1);
 
     gfx::Transform expected;
@@ -624,18 +756,19 @@ class PropertyTreeTestComputeTransformWithUninvertibleTransform
     : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode child;
     child.data.local.Scale(0, 0);
-    child.data.target_id = 0;
     child.data.source_node_id = 0;
+    child.id = tree.Insert(child, 0);
+    tree.SetTargetId(child.id, 0);
 
-    tree.Insert(child, 0);
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(1);
 
     gfx::Transform expected;
@@ -662,40 +795,42 @@ class PropertyTreeTestComputeTransformWithSublayerScale
     : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
-    root.data.target_id = 0;
+    root.id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode grand_parent;
     grand_parent.data.local.Scale(2.f, 2.f);
-    grand_parent.data.target_id = 0;
     grand_parent.data.source_node_id = 0;
     grand_parent.data.needs_sublayer_scale = true;
     int grand_parent_id = tree.Insert(grand_parent, 0);
+    tree.SetTargetId(grand_parent_id, 0);
     tree.UpdateTransforms(grand_parent_id);
 
     TransformNode parent;
     parent.data.local.Translate(15.f, 15.f);
-    parent.data.target_id = grand_parent_id;
     parent.data.source_node_id = grand_parent_id;
     int parent_id = tree.Insert(parent, grand_parent_id);
+    tree.SetTargetId(parent_id, grand_parent_id);
     tree.UpdateTransforms(parent_id);
 
     TransformNode child;
     child.data.local.Scale(3.f, 3.f);
-    child.data.target_id = grand_parent_id;
     child.data.source_node_id = parent_id;
     int child_id = tree.Insert(child, parent_id);
+    tree.SetTargetId(child_id, grand_parent_id);
     tree.UpdateTransforms(child_id);
 
     TransformNode grand_child;
     grand_child.data.local.Scale(5.f, 5.f);
-    grand_child.data.target_id = grand_parent_id;
     grand_child.data.source_node_id = child_id;
     grand_child.data.needs_sublayer_scale = true;
     int grand_child_id = tree.Insert(grand_child, child_id);
-    tree = TransformTreeForTest(tree);
+    tree.SetTargetId(grand_child_id, grand_parent_id);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(grand_child_id);
 
     EXPECT_EQ(gfx::Vector2dF(2.f, 2.f),
@@ -779,35 +914,36 @@ class PropertyTreeTestComputeTransformToTargetWithZeroSublayerScale
     : public PropertyTreeTest {
  protected:
   void StartTest() override {
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
     TransformNode& root = *tree.Node(0);
-    root.data.target_id = 0;
+    tree.SetTargetId(root.id, 0);
     tree.UpdateTransforms(0);
 
     TransformNode grand_parent;
     grand_parent.data.local.Scale(2.f, 0.f);
-    grand_parent.data.target_id = 0;
     grand_parent.data.source_node_id = 0;
     grand_parent.data.needs_sublayer_scale = true;
     int grand_parent_id = tree.Insert(grand_parent, 0);
-    tree.Node(grand_parent_id)->data.content_target_id = grand_parent_id;
+    tree.SetTargetId(grand_parent_id, 0);
+    tree.SetContentTargetId(grand_parent_id, grand_parent_id);
     tree.UpdateTransforms(grand_parent_id);
 
     TransformNode parent;
     parent.data.local.Translate(1.f, 1.f);
-    parent.data.target_id = grand_parent_id;
-    parent.data.content_target_id = grand_parent_id;
     parent.data.source_node_id = grand_parent_id;
     int parent_id = tree.Insert(parent, grand_parent_id);
+    tree.SetTargetId(parent_id, grand_parent_id);
+    tree.SetContentTargetId(parent_id, grand_parent_id);
     tree.UpdateTransforms(parent_id);
 
     TransformNode child;
     child.data.local.Translate(3.f, 4.f);
-    child.data.target_id = grand_parent_id;
-    child.data.content_target_id = grand_parent_id;
     child.data.source_node_id = parent_id;
     int child_id = tree.Insert(child, parent_id);
-    tree = TransformTreeForTest(tree);
+    tree.SetTargetId(child_id, grand_parent_id);
+    tree.SetContentTargetId(child_id, grand_parent_id);
+    SetupTransformTreeForTest(&tree);
     tree.UpdateTransforms(child_id);
 
     gfx::Transform expected_transform;
@@ -822,9 +958,9 @@ class PropertyTreeTestComputeTransformToTargetWithZeroSublayerScale
     tree.Node(grand_parent_id)->data.local.Scale(0.f, 2.f);
     tree.Node(grand_parent_id)->data.needs_local_transform_update = true;
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
 
-    ComputeTransforms(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     success = tree.ComputeTransform(child_id, grand_parent_id, &transform);
     EXPECT_TRUE(success);
@@ -834,9 +970,9 @@ class PropertyTreeTestComputeTransformToTargetWithZeroSublayerScale
     tree.Node(grand_parent_id)->data.local.Scale(0.f, 0.f);
     tree.Node(grand_parent_id)->data.needs_local_transform_update = true;
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
+    SetupTransformTreeForTest(&tree);
 
-    ComputeTransforms(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     success = tree.ComputeTransform(child_id, grand_parent_id, &transform);
     EXPECT_TRUE(success);
@@ -854,11 +990,12 @@ class PropertyTreeTestFlatteningWhenDestinationHasOnlyFlatAncestors
     // This tests that flattening is performed correctly when
     // destination and its ancestors are flat, but there are 3d transforms
     // and flattening between the source and destination.
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
 
     int parent = tree.Insert(TransformNode(), 0);
-    tree.Node(parent)->data.content_target_id = parent;
-    tree.Node(parent)->data.target_id = parent;
+    tree.SetContentTargetId(parent, parent);
+    tree.SetTargetId(parent, parent);
     tree.Node(parent)->data.source_node_id = 0;
     tree.Node(parent)->data.local.Translate(2, 2);
 
@@ -866,20 +1003,20 @@ class PropertyTreeTestFlatteningWhenDestinationHasOnlyFlatAncestors
     rotation_about_x.RotateAboutXAxis(15);
 
     int child = tree.Insert(TransformNode(), parent);
-    tree.Node(child)->data.content_target_id = child;
-    tree.Node(child)->data.target_id = child;
+    tree.SetContentTargetId(child, child);
+    tree.SetTargetId(child, child);
     tree.Node(child)->data.source_node_id = parent;
     tree.Node(child)->data.local = rotation_about_x;
 
     int grand_child = tree.Insert(TransformNode(), child);
-    tree.Node(grand_child)->data.content_target_id = grand_child;
-    tree.Node(grand_child)->data.target_id = grand_child;
+    tree.SetContentTargetId(grand_child, grand_child);
+    tree.SetTargetId(grand_child, grand_child);
     tree.Node(grand_child)->data.source_node_id = child;
     tree.Node(grand_child)->data.flattens_inherited_transform = true;
 
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     gfx::Transform flattened_rotation_about_x = rotation_about_x;
     flattened_rotation_about_x.FlattenTo2d();
@@ -905,19 +1042,19 @@ class PropertyTreeTestScreenSpaceOpacityUpdateTest : public PropertyTreeTest {
 
     int parent = tree.Insert(EffectNode(), 0);
     int child = tree.Insert(EffectNode(), parent);
-    tree = EffectTreeForTest(tree);
+    SetupEffectTreeForTest(&tree);
 
     EXPECT_EQ(tree.Node(child)->data.screen_space_opacity, 1.f);
     tree.Node(parent)->data.opacity = 0.5f;
     tree.set_needs_update(true);
-    tree = EffectTreeForTest(tree);
-    ComputeEffects(&tree);
+    SetupEffectTreeForTest(&tree);
+    draw_property_utils::ComputeEffects(&tree);
     EXPECT_EQ(tree.Node(child)->data.screen_space_opacity, 0.5f);
 
     tree.Node(child)->data.opacity = 0.5f;
     tree.set_needs_update(true);
-    tree = EffectTreeForTest(tree);
-    ComputeEffects(&tree);
+    SetupEffectTreeForTest(&tree);
+    draw_property_utils::ComputeEffects(&tree);
     EXPECT_EQ(tree.Node(child)->data.screen_space_opacity, 0.25f);
   }
 };
@@ -930,18 +1067,19 @@ class PropertyTreeTestNonIntegerTranslationTest : public PropertyTreeTest {
   void StartTest() override {
     // This tests that when a node has non-integer translation, the information
     // is propagated to the subtree.
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
 
     int parent = tree.Insert(TransformNode(), 0);
-    tree.Node(parent)->data.target_id = parent;
+    tree.SetTargetId(parent, parent);
     tree.Node(parent)->data.local.Translate(1.5f, 1.5f);
 
     int child = tree.Insert(TransformNode(), parent);
-    tree.Node(child)->data.target_id = parent;
+    tree.SetTargetId(child, parent);
     tree.Node(child)->data.local.Translate(1, 1);
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
     EXPECT_FALSE(tree.Node(parent)
                      ->data.node_and_ancestors_have_only_integer_translation);
     EXPECT_FALSE(tree.Node(child)
@@ -950,18 +1088,18 @@ class PropertyTreeTestNonIntegerTranslationTest : public PropertyTreeTest {
     tree.Node(parent)->data.local.Translate(0.5f, 0.5f);
     tree.Node(child)->data.local.Translate(0.5f, 0.5f);
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
     EXPECT_TRUE(tree.Node(parent)
                     ->data.node_and_ancestors_have_only_integer_translation);
     EXPECT_FALSE(tree.Node(child)
                      ->data.node_and_ancestors_have_only_integer_translation);
 
     tree.Node(child)->data.local.Translate(0.5f, 0.5f);
-    tree.Node(child)->data.target_id = child;
+    tree.SetTargetId(child, child);
     tree.set_needs_update(true);
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
     EXPECT_TRUE(tree.Node(parent)
                     ->data.node_and_ancestors_have_only_integer_translation);
     EXPECT_TRUE(tree.Node(child)
@@ -977,36 +1115,37 @@ class PropertyTreeTestSingularTransformSnapTest : public PropertyTreeTest {
   void StartTest() override {
     // This tests that to_target transform is not snapped when it has a singular
     // transform.
-    TransformTree tree;
+    PropertyTrees property_trees;
+    TransformTree& tree = property_trees.transform_tree;
 
     int parent = tree.Insert(TransformNode(), 0);
-    tree.Node(parent)->data.target_id = parent;
+    tree.SetTargetId(parent, parent);
     tree.Node(parent)->data.scrolls = true;
 
     int child = tree.Insert(TransformNode(), parent);
     TransformNode* child_node = tree.Node(child);
-    child_node->data.target_id = parent;
+    tree.SetTargetId(child, parent);
     child_node->data.scrolls = true;
     child_node->data.local.Scale3d(6.0f, 6.0f, 0.0f);
     child_node->data.local.Translate(1.3f, 1.3f);
     tree.set_needs_update(true);
 
-    tree = TransformTreeForTest(tree);
-    ComputeTransforms(&tree);
+    SetupTransformTreeForTest(&tree);
+    draw_property_utils::ComputeTransforms(&tree);
 
     gfx::Transform from_target;
-    EXPECT_FALSE(child_node->data.to_target.GetInverse(&from_target));
+    EXPECT_FALSE(tree.ToTarget(child).GetInverse(&from_target));
     // The following checks are to ensure that snapping is skipped because of
     // singular transform (and not because of other reasons which also cause
     // snapping to be skipped).
     EXPECT_TRUE(child_node->data.scrolls);
-    EXPECT_TRUE(child_node->data.to_target.IsScaleOrTranslation());
-    EXPECT_FALSE(child_node->data.to_screen_is_animated);
+    EXPECT_TRUE(tree.ToTarget(child).IsScaleOrTranslation());
+    EXPECT_FALSE(child_node->data.to_screen_is_potentially_animated);
     EXPECT_FALSE(child_node->data.ancestors_are_invertible);
 
-    gfx::Transform rounded = child_node->data.to_target;
+    gfx::Transform rounded = tree.ToTarget(child);
     rounded.RoundTranslationComponents();
-    EXPECT_NE(child_node->data.to_target, rounded);
+    EXPECT_NE(tree.ToTarget(child), rounded);
   }
 };
 
