@@ -5,8 +5,10 @@
 #include "cc/trees/single_thread_proxy.h"
 
 #include "base/auto_reset.h"
+#include "base/memory/ptr_util.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/trace_event/trace_event.h"
+#include "cc/animation/animation_events.h"
 #include "cc/debug/benchmark_instrumentation.h"
 #include "cc/debug/devtools_instrumentation.h"
 #include "cc/output/context_provider.h"
@@ -14,35 +16,30 @@
 #include "cc/quads/draw_quad.h"
 #include "cc/scheduler/commit_earlyout_reason.h"
 #include "cc/scheduler/compositor_timing_history.h"
+#include "cc/scheduler/delay_based_time_source.h"
 #include "cc/scheduler/scheduler.h"
 #include "cc/trees/layer_tree_host.h"
+#include "cc/trees/layer_tree_host_common.h"
 #include "cc/trees/layer_tree_host_single_thread_client.h"
 #include "cc/trees/layer_tree_impl.h"
 #include "cc/trees/scoped_abort_remaining_swap_promises.h"
 
 namespace cc {
 
-scoped_ptr<Proxy> SingleThreadProxy::Create(
+std::unique_ptr<Proxy> SingleThreadProxy::Create(
     LayerTreeHost* layer_tree_host,
     LayerTreeHostSingleThreadClient* client,
-    scoped_refptr<base::SingleThreadTaskRunner> main_task_runner,
-    scoped_ptr<BeginFrameSource> external_begin_frame_source) {
-  return make_scoped_ptr(new SingleThreadProxy(
-                                 layer_tree_host,
-                                 client,
-                                 main_task_runner,
-                                 external_begin_frame_source.Pass()));
+    TaskRunnerProvider* task_runner_provider) {
+  return base::WrapUnique(
+      new SingleThreadProxy(layer_tree_host, client, task_runner_provider));
 }
 
-SingleThreadProxy::SingleThreadProxy(
-    LayerTreeHost* layer_tree_host,
-    LayerTreeHostSingleThreadClient* client,
-    scoped_refptr<base::SingleThreadTaskRunner> main_task_runner,
-    scoped_ptr<BeginFrameSource> external_begin_frame_source)
-    : Proxy(main_task_runner, NULL),
-      layer_tree_host_(layer_tree_host),
+SingleThreadProxy::SingleThreadProxy(LayerTreeHost* layer_tree_host,
+                                     LayerTreeHostSingleThreadClient* client,
+                                     TaskRunnerProvider* task_runner_provider)
+    : layer_tree_host_(layer_tree_host),
       client_(client),
-      external_begin_frame_source_(external_begin_frame_source.Pass()),
+      task_runner_provider_(task_runner_provider),
       next_frame_is_newly_committed_frame_(false),
 #if DCHECK_IS_ON()
       inside_impl_frame_(false),
@@ -55,50 +52,74 @@ SingleThreadProxy::SingleThreadProxy(
       output_surface_creation_requested_(false),
       weak_factory_(this) {
   TRACE_EVENT0("cc", "SingleThreadProxy::SingleThreadProxy");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_);
+  DCHECK(task_runner_provider_->IsMainThread());
   DCHECK(layer_tree_host);
-
-  if (layer_tree_host->settings().single_thread_proxy_scheduler &&
-      !scheduler_on_impl_thread_) {
-    SchedulerSettings scheduler_settings(
-        layer_tree_host->settings().ToSchedulerSettings());
-    scheduler_settings.commit_to_active_tree = CommitToActiveTree();
-
-    scoped_ptr<CompositorTimingHistory> compositor_timing_history(
-        new CompositorTimingHistory(
-            layer_tree_host->rendering_stats_instrumentation()));
-
-    scheduler_on_impl_thread_ = Scheduler::Create(
-        this, scheduler_settings, layer_tree_host_->id(),
-        MainThreadTaskRunner(), external_begin_frame_source_.get(),
-        compositor_timing_history.Pass());
-  }
 }
 
-void SingleThreadProxy::Start() {
-  DebugScopedSetImplThread impl(this);
+void SingleThreadProxy::Start(
+    std::unique_ptr<BeginFrameSource> external_begin_frame_source) {
+  DebugScopedSetImplThread impl(task_runner_provider_);
+  external_begin_frame_source_ = std::move(external_begin_frame_source);
+
+  if (layer_tree_host_->settings().single_thread_proxy_scheduler &&
+      !scheduler_on_impl_thread_) {
+    SchedulerSettings scheduler_settings(
+        layer_tree_host_->settings().ToSchedulerSettings());
+    scheduler_settings.commit_to_active_tree = CommitToActiveTree();
+
+    std::unique_ptr<CompositorTimingHistory> compositor_timing_history(
+        new CompositorTimingHistory(
+            scheduler_settings.using_synchronous_renderer_compositor,
+            CompositorTimingHistory::BROWSER_UMA,
+            layer_tree_host_->rendering_stats_instrumentation()));
+
+    BeginFrameSource* frame_source = nullptr;
+    if (!layer_tree_host_->settings().use_output_surface_begin_frame_source) {
+      frame_source = external_begin_frame_source_.get();
+      if (!scheduler_settings.throttle_frame_production) {
+        // Unthrottled source takes precedence over external sources.
+        unthrottled_begin_frame_source_.reset(new BackToBackBeginFrameSource(
+            base::MakeUnique<DelayBasedTimeSource>(
+                task_runner_provider_->MainThreadTaskRunner())));
+        frame_source = unthrottled_begin_frame_source_.get();
+      }
+      if (!frame_source) {
+        synthetic_begin_frame_source_.reset(new DelayBasedBeginFrameSource(
+            base::MakeUnique<DelayBasedTimeSource>(
+                task_runner_provider_->MainThreadTaskRunner())));
+        frame_source = synthetic_begin_frame_source_.get();
+      }
+    }
+
+    scheduler_on_impl_thread_ =
+        Scheduler::Create(this, scheduler_settings, layer_tree_host_->id(),
+                          task_runner_provider_->MainThreadTaskRunner(),
+                          frame_source, std::move(compositor_timing_history));
+  }
+
   layer_tree_host_impl_ = layer_tree_host_->CreateLayerTreeHostImpl(this);
 }
 
 SingleThreadProxy::~SingleThreadProxy() {
   TRACE_EVENT0("cc", "SingleThreadProxy::~SingleThreadProxy");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   // Make sure Stop() got called or never Started.
   DCHECK(!layer_tree_host_impl_);
 }
 
 void SingleThreadProxy::FinishAllRendering() {
   TRACE_EVENT0("cc", "SingleThreadProxy::FinishAllRendering");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   {
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetImplThread impl(task_runner_provider_);
     layer_tree_host_impl_->FinishAllRendering();
   }
 }
 
 bool SingleThreadProxy::IsStarted() const {
-  DCHECK(Proxy::IsMainThread());
-  return layer_tree_host_impl_;
+  DCHECK(task_runner_provider_->IsMainThread());
+  return !!layer_tree_host_impl_;
 }
 
 bool SingleThreadProxy::CommitToActiveTree() const {
@@ -107,37 +128,18 @@ bool SingleThreadProxy::CommitToActiveTree() const {
   return true;
 }
 
-void SingleThreadProxy::SetLayerTreeHostClientReady() {
-  TRACE_EVENT0("cc", "SingleThreadProxy::SetLayerTreeHostClientReady");
-  // Scheduling is controlled by the embedder in the single thread case, so
-  // nothing to do.
-  DCHECK(Proxy::IsMainThread());
-  DebugScopedSetImplThread impl(this);
-  if (scheduler_on_impl_thread_) {
-    scheduler_on_impl_thread_->SetCanStart();
-    scheduler_on_impl_thread_->SetVisible(layer_tree_host_impl_->visible());
-  }
-}
-
 void SingleThreadProxy::SetVisible(bool visible) {
   TRACE_EVENT1("cc", "SingleThreadProxy::SetVisible", "visible", visible);
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
+
   layer_tree_host_impl_->SetVisible(visible);
+
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->SetVisible(layer_tree_host_impl_->visible());
-  // Changing visibility could change ShouldComposite().
-}
-
-void SingleThreadProxy::SetThrottleFrameProduction(bool throttle) {
-  TRACE_EVENT1("cc", "SingleThreadProxy::SetThrottleFrameProduction",
-               "throttle", throttle);
-  DebugScopedSetImplThread impl(this);
-  if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetThrottleFrameProduction(throttle);
 }
 
 void SingleThreadProxy::RequestNewOutputSurface() {
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   DCHECK(layer_tree_host_->output_surface_lost());
   output_surface_creation_callback_.Cancel();
   if (output_surface_creation_requested_)
@@ -146,18 +148,26 @@ void SingleThreadProxy::RequestNewOutputSurface() {
   layer_tree_host_->RequestNewOutputSurface();
 }
 
-void SingleThreadProxy::SetOutputSurface(
-    scoped_ptr<OutputSurface> output_surface) {
-  DCHECK(Proxy::IsMainThread());
+void SingleThreadProxy::ReleaseOutputSurface() {
+  // |layer_tree_host_| should already be aware of this.
+  DCHECK(layer_tree_host_->output_surface_lost());
+
+  if (scheduler_on_impl_thread_)
+    scheduler_on_impl_thread_->DidLoseOutputSurface();
+  return layer_tree_host_impl_->ReleaseOutputSurface();
+}
+
+void SingleThreadProxy::SetOutputSurface(OutputSurface* output_surface) {
+  DCHECK(task_runner_provider_->IsMainThread());
   DCHECK(layer_tree_host_->output_surface_lost());
   DCHECK(output_surface_creation_requested_);
   renderer_capabilities_for_main_thread_ = RendererCapabilities();
 
   bool success;
   {
-    DebugScopedSetMainThreadBlocked main_thread_blocked(this);
-    DebugScopedSetImplThread impl(this);
-    success = layer_tree_host_impl_->InitializeRenderer(output_surface.Pass());
+    DebugScopedSetMainThreadBlocked main_thread_blocked(task_runner_provider_);
+    DebugScopedSetImplThread impl(task_runner_provider_);
+    success = layer_tree_host_impl_->InitializeRenderer(output_surface);
   }
 
   if (success) {
@@ -175,50 +185,32 @@ void SingleThreadProxy::SetOutputSurface(
 }
 
 const RendererCapabilities& SingleThreadProxy::GetRendererCapabilities() const {
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   DCHECK(!layer_tree_host_->output_surface_lost());
   return renderer_capabilities_for_main_thread_;
 }
 
 void SingleThreadProxy::SetNeedsAnimate() {
   TRACE_EVENT0("cc", "SingleThreadProxy::SetNeedsAnimate");
-  DCHECK(Proxy::IsMainThread());
-  client_->ScheduleAnimation();
+  DCHECK(task_runner_provider_->IsMainThread());
+  client_->RequestScheduleAnimation();
   if (animate_requested_)
     return;
   animate_requested_ = true;
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetNeedsCommit();
+    scheduler_on_impl_thread_->SetNeedsBeginMainFrame();
 }
 
 void SingleThreadProxy::SetNeedsUpdateLayers() {
   TRACE_EVENT0("cc", "SingleThreadProxy::SetNeedsUpdateLayers");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   SetNeedsCommit();
 }
 
-void SingleThreadProxy::DoAnimate() {
-  // Don't animate if there is no root layer.
-  // TODO(mithro): Both Animate and UpdateAnimationState already have a
-  // "!active_tree_->root_layer()" check?
-  if (!layer_tree_host_impl_->active_tree()->root_layer()) {
-    return;
-  }
-
-  layer_tree_host_impl_->Animate(
-      layer_tree_host_impl_->CurrentBeginFrameArgs().frame_time);
-
-  // If animations are not visible, update the animation state now as it
-  // won't happen in DoComposite.
-  if (!layer_tree_host_impl_->AnimationsAreVisible()) {
-    layer_tree_host_impl_->UpdateAnimationState(true);
-  }
-}
-
 void SingleThreadProxy::DoCommit() {
   TRACE_EVENT0("cc", "SingleThreadProxy::DoCommit");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
 
   // TODO(robliao): Remove ScopedTracker below once https://crbug.com/461509 is
   // fixed.
@@ -235,17 +227,16 @@ void SingleThreadProxy::DoCommit() {
     tracked_objects::ScopedTracker tracking_profile2(
         FROM_HERE_WITH_EXPLICIT_FUNCTION(
             "461509 SingleThreadProxy::DoCommit2"));
-    DebugScopedSetMainThreadBlocked main_thread_blocked(this);
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetMainThreadBlocked main_thread_blocked(task_runner_provider_);
+    DebugScopedSetImplThread impl(task_runner_provider_);
 
     // This CapturePostTasks should be destroyed before CommitComplete() is
     // called since that goes out to the embedder, and we want the embedder
     // to receive its callbacks before that.
     commit_blocking_task_runner_.reset(new BlockingTaskRunner::CapturePostTasks(
-        blocking_main_thread_task_runner()));
+        task_runner_provider_->blocking_main_thread_task_runner()));
 
     layer_tree_host_impl_->BeginCommit();
-    layer_tree_host_->BeginCommitOnImplThread(layer_tree_host_impl_.get());
 
     // TODO(robliao): Remove ScopedTracker below once https://crbug.com/461509
     // is fixed.
@@ -265,12 +256,17 @@ void SingleThreadProxy::DoCommit() {
 #if DCHECK_IS_ON()
     // In the single-threaded case, the scale and scroll deltas should never be
     // touched on the impl layer tree.
-    scoped_ptr<ScrollAndScaleSet> scroll_info =
+    std::unique_ptr<ScrollAndScaleSet> scroll_info =
         layer_tree_host_impl_->ProcessScrollDeltas();
     DCHECK(!scroll_info->scrolls.size());
     DCHECK_EQ(1.f, scroll_info->page_scale_delta);
 #endif
 
+    if (scheduler_on_impl_thread_)
+      scheduler_on_impl_thread_->DidCommit();
+
+    layer_tree_host_impl_->CommitComplete();
+
     // TODO(robliao): Remove ScopedTracker below once https://crbug.com/461509
     // is fixed.
     tracked_objects::ScopedTracker tracking_profile8(
@@ -286,18 +282,13 @@ void SingleThreadProxy::DoCommit() {
 }
 
 void SingleThreadProxy::CommitComplete() {
+  // Commit complete happens on the main side after activate to satisfy any
+  // SetNextCommitWaitsForActivation calls.
   DCHECK(!layer_tree_host_impl_->pending_tree())
       << "Activation is expected to have synchronously occurred by now.";
   DCHECK(commit_blocking_task_runner_);
 
-  if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->DidCommit();
-
-  // Notify commit complete on the impl side after activate to satisfy any
-  // SetNextCommitWaitsForActivation calls.
-  layer_tree_host_impl_->CommitComplete();
-
-  DebugScopedSetMainThread main(this);
+  DebugScopedSetMainThread main(task_runner_provider_);
   commit_blocking_task_runner_.reset();
   layer_tree_host_->CommitComplete();
   layer_tree_host_->DidBeginMainFrame();
@@ -306,31 +297,31 @@ void SingleThreadProxy::CommitComplete() {
 }
 
 void SingleThreadProxy::SetNeedsCommit() {
-  DCHECK(Proxy::IsMainThread());
-  client_->ScheduleComposite();
+  DCHECK(task_runner_provider_->IsMainThread());
+  client_->RequestScheduleComposite();
   if (commit_requested_)
     return;
   commit_requested_ = true;
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetNeedsCommit();
+    scheduler_on_impl_thread_->SetNeedsBeginMainFrame();
 }
 
 void SingleThreadProxy::SetNeedsRedraw(const gfx::Rect& damage_rect) {
   TRACE_EVENT0("cc", "SingleThreadProxy::SetNeedsRedraw");
-  DCHECK(Proxy::IsMainThread());
-  DebugScopedSetImplThread impl(this);
-  client_->ScheduleComposite();
+  DCHECK(task_runner_provider_->IsMainThread());
+  DebugScopedSetImplThread impl(task_runner_provider_);
+  client_->RequestScheduleComposite();
   SetNeedsRedrawRectOnImplThread(damage_rect);
 }
 
 void SingleThreadProxy::SetNextCommitWaitsForActivation() {
   // Activation always forced in commit, so nothing to do.
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
 }
 
 void SingleThreadProxy::SetDeferCommits(bool defer_commits) {
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   // Deferring commits only makes sense if there's a scheduler.
   if (!scheduler_on_impl_thread_)
     return;
@@ -347,12 +338,12 @@ void SingleThreadProxy::SetDeferCommits(bool defer_commits) {
 }
 
 bool SingleThreadProxy::CommitRequested() const {
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   return commit_requested_;
 }
 
 bool SingleThreadProxy::BeginMainFrameRequested() const {
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   // If there is no scheduler, then there can be no pending begin frame,
   // as all frames are all manually initiated by the embedder of cc.
   if (!scheduler_on_impl_thread_)
@@ -362,51 +353,63 @@ bool SingleThreadProxy::BeginMainFrameRequested() const {
 
 void SingleThreadProxy::Stop() {
   TRACE_EVENT0("cc", "SingleThreadProxy::stop");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
   {
-    DebugScopedSetMainThreadBlocked main_thread_blocked(this);
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetMainThreadBlocked main_thread_blocked(task_runner_provider_);
+    DebugScopedSetImplThread impl(task_runner_provider_);
+
+    // Take away the OutputSurface before destroying things so it doesn't try
+    // to call into its client mid-shutdown.
+    layer_tree_host_impl_->ReleaseOutputSurface();
 
     BlockingTaskRunner::CapturePostTasks blocked(
-        blocking_main_thread_task_runner());
+        task_runner_provider_->blocking_main_thread_task_runner());
     scheduler_on_impl_thread_ = nullptr;
     layer_tree_host_impl_ = nullptr;
   }
   layer_tree_host_ = NULL;
 }
 
+void SingleThreadProxy::SetMutator(std::unique_ptr<LayerTreeMutator> mutator) {
+  DCHECK(task_runner_provider_->IsMainThread());
+  DebugScopedSetImplThread impl(task_runner_provider_);
+  layer_tree_host_impl_->SetLayerTreeMutator(std::move(mutator));
+}
+
 void SingleThreadProxy::OnCanDrawStateChanged(bool can_draw) {
   TRACE_EVENT1(
       "cc", "SingleThreadProxy::OnCanDrawStateChanged", "can_draw", can_draw);
-  DCHECK(Proxy::IsImplThread());
+  DCHECK(task_runner_provider_->IsImplThread());
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->SetCanDraw(can_draw);
 }
 
 void SingleThreadProxy::NotifyReadyToActivate() {
   TRACE_EVENT0("cc", "SingleThreadProxy::NotifyReadyToActivate");
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->NotifyReadyToActivate();
 }
 
 void SingleThreadProxy::NotifyReadyToDraw() {
   TRACE_EVENT0("cc", "SingleThreadProxy::NotifyReadyToDraw");
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->NotifyReadyToDraw();
 }
 
 void SingleThreadProxy::SetNeedsRedrawOnImplThread() {
-  client_->ScheduleComposite();
+  client_->RequestScheduleComposite();
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->SetNeedsRedraw();
 }
 
-void SingleThreadProxy::SetNeedsAnimateOnImplThread() {
-  client_->ScheduleComposite();
+void SingleThreadProxy::SetNeedsOneBeginImplFrameOnImplThread() {
+  TRACE_EVENT0("cc",
+               "SingleThreadProxy::SetNeedsOneBeginImplFrameOnImplThread");
+  client_->RequestScheduleComposite();
   if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetNeedsAnimate();
+    scheduler_on_impl_thread_->SetNeedsOneBeginImplFrame();
 }
 
 void SingleThreadProxy::SetNeedsPrepareTilesOnImplThread() {
@@ -422,9 +425,9 @@ void SingleThreadProxy::SetNeedsRedrawRectOnImplThread(
 }
 
 void SingleThreadProxy::SetNeedsCommitOnImplThread() {
-  client_->ScheduleComposite();
+  client_->RequestScheduleComposite();
   if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetNeedsCommit();
+    scheduler_on_impl_thread_->SetNeedsBeginMainFrame();
 }
 
 void SingleThreadProxy::SetVideoNeedsBeginFrames(bool needs_begin_frames) {
@@ -436,22 +439,17 @@ void SingleThreadProxy::SetVideoNeedsBeginFrames(bool needs_begin_frames) {
 }
 
 void SingleThreadProxy::PostAnimationEventsToMainThreadOnImplThread(
-    scoped_ptr<AnimationEventsVector> events) {
+    std::unique_ptr<AnimationEvents> events) {
   TRACE_EVENT0(
       "cc", "SingleThreadProxy::PostAnimationEventsToMainThreadOnImplThread");
-  DCHECK(Proxy::IsImplThread());
-  DebugScopedSetMainThread main(this);
-  layer_tree_host_->SetAnimationEvents(events.Pass());
+  DCHECK(task_runner_provider_->IsImplThread());
+  DebugScopedSetMainThread main(task_runner_provider_);
+  layer_tree_host_->SetAnimationEvents(std::move(events));
 }
 
 bool SingleThreadProxy::IsInsideDraw() { return inside_draw_; }
 
 void SingleThreadProxy::DidActivateSyncTree() {
-  // This is required because NotifyReadyToActivate gets called immediately
-  // after commit since single thread commits directly to the active tree.
-  if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetWaitForReadyToDraw();
-
   // Synchronously call to CommitComplete. Resetting
   // |commit_blocking_task_runner| would make sure all tasks posted during
   // commit/activation before CommitComplete.
@@ -459,13 +457,13 @@ void SingleThreadProxy::DidActivateSyncTree() {
 }
 
 void SingleThreadProxy::WillPrepareTiles() {
-  DCHECK(Proxy::IsImplThread());
+  DCHECK(task_runner_provider_->IsImplThread());
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->WillPrepareTiles();
 }
 
 void SingleThreadProxy::DidPrepareTiles() {
-  DCHECK(Proxy::IsImplThread());
+  DCHECK(task_runner_provider_->IsImplThread());
   if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->DidPrepareTiles();
 }
@@ -475,7 +473,7 @@ void SingleThreadProxy::DidCompletePageScaleAnimationOnImplThread() {
 }
 
 void SingleThreadProxy::UpdateRendererCapabilitiesOnImplThread() {
-  DCHECK(IsImplThread());
+  DCHECK(task_runner_provider_->IsImplThread());
   renderer_capabilities_for_main_thread_ =
       layer_tree_host_impl_->GetRendererCapabilities().MainThreadCapabilities();
 }
@@ -483,7 +481,7 @@ void SingleThreadProxy::UpdateRendererCapabilitiesOnImplThread() {
 void SingleThreadProxy::DidLoseOutputSurfaceOnImplThread() {
   TRACE_EVENT0("cc", "SingleThreadProxy::DidLoseOutputSurfaceOnImplThread");
   {
-    DebugScopedSetMainThread main(this);
+    DebugScopedSetMainThread main(task_runner_provider_);
     // This must happen before we notify the scheduler as it may try to recreate
     // the output surface if already in BEGIN_IMPL_FRAME_STATE_IDLE.
     layer_tree_host_->DidLoseOutputSurface();
@@ -495,8 +493,18 @@ void SingleThreadProxy::DidLoseOutputSurfaceOnImplThread() {
 
 void SingleThreadProxy::CommitVSyncParameters(base::TimeTicks timebase,
                                               base::TimeDelta interval) {
+  if (synthetic_begin_frame_source_)
+    synthetic_begin_frame_source_->OnUpdateVSyncParameters(timebase, interval);
+}
+
+void SingleThreadProxy::SetBeginFrameSource(BeginFrameSource* source) {
+  DCHECK(layer_tree_host_->settings().single_thread_proxy_scheduler);
+  // TODO(enne): this overrides any preexisting begin frame source.  Those
+  // other sources will eventually be removed and this will be the only path.
+  if (!layer_tree_host_->settings().use_output_surface_begin_frame_source)
+    return;
   if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->CommitVSyncParameters(timebase, interval);
+    scheduler_on_impl_thread_->SetBeginFrameSource(source);
 }
 
 void SingleThreadProxy::SetEstimatedParentDrawTime(base::TimeDelta draw_time) {
@@ -504,11 +512,6 @@ void SingleThreadProxy::SetEstimatedParentDrawTime(base::TimeDelta draw_time) {
     scheduler_on_impl_thread_->SetEstimatedParentDrawTime(draw_time);
 }
 
-void SingleThreadProxy::SetMaxSwapsPendingOnImplThread(int max) {
-  if (scheduler_on_impl_thread_)
-    scheduler_on_impl_thread_->SetMaxSwapsPending(max);
-}
-
 void SingleThreadProxy::DidSwapBuffersOnImplThread() {
   TRACE_EVENT0("cc", "SingleThreadProxy::DidSwapBuffersOnImplThread");
   if (scheduler_on_impl_thread_)
@@ -524,33 +527,14 @@ void SingleThreadProxy::DidSwapBuffersCompleteOnImplThread() {
   layer_tree_host_->DidCompleteSwapBuffers();
 }
 
-void SingleThreadProxy::OnDrawForOutputSurface() {
+void SingleThreadProxy::OnDrawForOutputSurface(
+    bool resourceless_software_draw) {
   NOTREACHED() << "Implemented by ThreadProxy for synchronous compositor.";
 }
 
-void SingleThreadProxy::PostFrameTimingEventsOnImplThread(
-    scoped_ptr<FrameTimingTracker::CompositeTimingSet> composite_events,
-    scoped_ptr<FrameTimingTracker::MainFrameTimingSet> main_frame_events) {
-  layer_tree_host_->RecordFrameTimingEvents(composite_events.Pass(),
-                                            main_frame_events.Pass());
-}
-
-void SingleThreadProxy::LayoutAndUpdateLayers() {
-  if (layer_tree_host_->output_surface_lost()) {
-    RequestNewOutputSurface();
-    // RequestNewOutputSurface could have synchronously created an output
-    // surface, so check again before returning.
-    if (layer_tree_host_->output_surface_lost())
-      return;
-  }
-
-  layer_tree_host_->Layout();
-  layer_tree_host_->UpdateLayers();
-}
-
 void SingleThreadProxy::CompositeImmediately(base::TimeTicks frame_begin_time) {
   TRACE_EVENT0("cc,benchmark", "SingleThreadProxy::CompositeImmediately");
-  DCHECK(Proxy::IsMainThread());
+  DCHECK(task_runner_provider_->IsMainThread());
 #if DCHECK_IS_ON()
   DCHECK(!inside_impl_frame_);
 #endif
@@ -570,7 +554,7 @@ void SingleThreadProxy::CompositeImmediately(base::TimeTicks frame_begin_time) {
 
   // Start the impl frame.
   {
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetImplThread impl(task_runner_provider_);
     WillBeginImplFrame(begin_frame_args);
   }
 
@@ -588,14 +572,15 @@ void SingleThreadProxy::CompositeImmediately(base::TimeTicks frame_begin_time) {
 
   // Finish the impl frame.
   {
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetImplThread impl(task_runner_provider_);
     layer_tree_host_impl_->ActivateSyncTree();
     DCHECK(
         !layer_tree_host_impl_->active_tree()->needs_update_draw_properties());
     layer_tree_host_impl_->PrepareTiles();
     layer_tree_host_impl_->SynchronouslyInitializeAllTiles();
 
-    DoAnimate();
+    // TODO(danakj): Don't do this last... we prepared the wrong things. D:
+    layer_tree_host_impl_->Animate();
 
     LayerTreeHostImpl::FrameData frame;
     DoComposite(&frame);
@@ -609,22 +594,12 @@ void SingleThreadProxy::CompositeImmediately(base::TimeTicks frame_begin_time) {
   }
 }
 
-void SingleThreadProxy::ForceSerializeOnSwapBuffers() {
-  {
-    DebugScopedSetImplThread impl(this);
-    if (layer_tree_host_impl_->renderer()) {
-      DCHECK(!layer_tree_host_->output_surface_lost());
-      layer_tree_host_impl_->renderer()->DoNoOp();
-    }
-  }
-}
-
 bool SingleThreadProxy::SupportsImplScrolling() const {
   return false;
 }
 
 bool SingleThreadProxy::ShouldComposite() const {
-  DCHECK(Proxy::IsImplThread());
+  DCHECK(task_runner_provider_->IsImplThread());
   return layer_tree_host_impl_->visible() &&
          layer_tree_host_impl_->CanDraw();
 }
@@ -635,7 +610,7 @@ void SingleThreadProxy::ScheduleRequestNewOutputSurface() {
     output_surface_creation_callback_.Reset(
         base::Bind(&SingleThreadProxy::RequestNewOutputSurface,
                    weak_factory_.GetWeakPtr()));
-    MainThreadTaskRunner()->PostTask(
+    task_runner_provider_->MainThreadTaskRunner()->PostTask(
         FROM_HERE, output_surface_creation_callback_.callback());
   }
 }
@@ -647,7 +622,7 @@ DrawResult SingleThreadProxy::DoComposite(LayerTreeHostImpl::FrameData* frame) {
   DrawResult draw_result;
   bool draw_frame;
   {
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetImplThread impl(task_runner_provider_);
     base::AutoReset<bool> mark_inside(&inside_draw_, true);
 
     // TODO(robliao): Remove ScopedTracker below once https://crbug.com/461509
@@ -702,7 +677,7 @@ DrawResult SingleThreadProxy::DoComposite(LayerTreeHostImpl::FrameData* frame) {
   }
 
   if (draw_frame) {
-    DebugScopedSetImplThread impl(this);
+    DebugScopedSetImplThread impl(task_runner_provider_);
 
     // This CapturePostTasks should be destroyed before
     // DidCommitAndDrawFrame() is called since that goes out to the
@@ -712,10 +687,10 @@ DrawResult SingleThreadProxy::DoComposite(LayerTreeHostImpl::FrameData* frame) {
     // the DidCommitAndDrawFrame() must be post-tasked from the impl thread
     // there as the main thread is not blocked, so any posted tasks inside
     // the swap buffers will execute first.
-    DebugScopedSetMainThreadBlocked main_thread_blocked(this);
+    DebugScopedSetMainThreadBlocked main_thread_blocked(task_runner_provider_);
 
     BlockingTaskRunner::CapturePostTasks blocked(
-        blocking_main_thread_task_runner());
+        task_runner_provider_->blocking_main_thread_task_runner());
     // TODO(robliao): Remove ScopedTracker below once https://crbug.com/461509
     // is fixed.
     tracked_objects::ScopedTracker tracking_profile8(
@@ -735,7 +710,7 @@ DrawResult SingleThreadProxy::DoComposite(LayerTreeHostImpl::FrameData* frame) {
 
 void SingleThreadProxy::DidCommitAndDrawFrame() {
   if (next_frame_is_newly_committed_frame_) {
-    DebugScopedSetMainThread main(this);
+    DebugScopedSetMainThread main(task_runner_provider_);
     next_frame_is_newly_committed_frame_ = false;
     layer_tree_host_->DidCommitAndDrawFrame();
   }
@@ -749,18 +724,8 @@ bool SingleThreadProxy::MainFrameWillHappenForTesting() {
   return scheduler_on_impl_thread_->MainFrameForTestingWillHappen();
 }
 
-void SingleThreadProxy::SetChildrenNeedBeginFrames(
-    bool children_need_begin_frames) {
-  scheduler_on_impl_thread_->SetChildrenNeedBeginFrames(
-      children_need_begin_frames);
-}
-
-void SingleThreadProxy::SetAuthoritativeVSyncInterval(
-    const base::TimeDelta& interval) {
-  scheduler_on_impl_thread_->SetAuthoritativeVSyncInterval(interval);
-}
-
 void SingleThreadProxy::WillBeginImplFrame(const BeginFrameArgs& args) {
+  DebugScopedSetImplThread impl(task_runner_provider_);
 #if DCHECK_IS_ON()
   DCHECK(!inside_impl_frame_)
       << "WillBeginImplFrame called while already inside an impl frame!";
@@ -769,7 +734,8 @@ void SingleThreadProxy::WillBeginImplFrame(const BeginFrameArgs& args) {
   layer_tree_host_impl_->WillBeginImplFrame(args);
 }
 
-void SingleThreadProxy::ScheduledActionSendBeginMainFrame() {
+void SingleThreadProxy::ScheduledActionSendBeginMainFrame(
+    const BeginFrameArgs& begin_frame_args) {
   TRACE_EVENT0("cc", "SingleThreadProxy::ScheduledActionSendBeginMainFrame");
   // Although this proxy is single-threaded, it's problematic to synchronously
   // have BeginMainFrame happen after ScheduledActionSendBeginMainFrame.  This
@@ -782,10 +748,8 @@ void SingleThreadProxy::ScheduledActionSendBeginMainFrame() {
   DCHECK(inside_impl_frame_)
       << "BeginMainFrame should only be sent inside a BeginImplFrame";
 #endif
-  const BeginFrameArgs& begin_frame_args =
-      layer_tree_host_impl_->CurrentBeginFrameArgs();
 
-  MainThreadTaskRunner()->PostTask(
+  task_runner_provider_->MainThreadTaskRunner()->PostTask(
       FROM_HERE, base::Bind(&SingleThreadProxy::BeginMainFrame,
                             weak_factory_.GetWeakPtr(), begin_frame_args));
 }
@@ -795,6 +759,11 @@ void SingleThreadProxy::SendBeginMainFrameNotExpectedSoon() {
 }
 
 void SingleThreadProxy::BeginMainFrame(const BeginFrameArgs& begin_frame_args) {
+  if (scheduler_on_impl_thread_) {
+    scheduler_on_impl_thread_->NotifyBeginMainFrameStarted(
+        base::TimeTicks::Now());
+  }
+
   commit_requested_ = false;
   animate_requested_ = false;
 
@@ -838,7 +807,7 @@ void SingleThreadProxy::DoBeginMainFrame(
   layer_tree_host_->WillBeginMainFrame();
   layer_tree_host_->BeginMainFrame(begin_frame_args);
   layer_tree_host_->AnimateLayers(begin_frame_args.frame_time);
-  layer_tree_host_->Layout();
+  layer_tree_host_->RequestMainFrameUpdate();
 
   // New commits requested inside UpdateLayers should be respected.
   commit_requested_ = false;
@@ -848,15 +817,13 @@ void SingleThreadProxy::DoBeginMainFrame(
   // TODO(enne): SingleThreadProxy does not support cancelling commits yet,
   // search for CommitEarlyOutReason::FINISHED_NO_UPDATES inside
   // thread_proxy.cc
-  if (scheduler_on_impl_thread_) {
-    scheduler_on_impl_thread_->NotifyBeginMainFrameStarted();
+  if (scheduler_on_impl_thread_)
     scheduler_on_impl_thread_->NotifyReadyToCommit();
-  }
 }
 
 void SingleThreadProxy::BeginMainFrameAbortedOnImplThread(
     CommitEarlyOutReason reason) {
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   DCHECK(scheduler_on_impl_thread_->CommitPending());
   DCHECK(!layer_tree_host_impl_->pending_tree());
 
@@ -865,7 +832,7 @@ void SingleThreadProxy::BeginMainFrameAbortedOnImplThread(
 }
 
 DrawResult SingleThreadProxy::ScheduledActionDrawAndSwapIfPossible() {
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   LayerTreeHostImpl::FrameData frame;
   return DoComposite(&frame);
 }
@@ -876,29 +843,23 @@ DrawResult SingleThreadProxy::ScheduledActionDrawAndSwapForced() {
 }
 
 void SingleThreadProxy::ScheduledActionCommit() {
-  DebugScopedSetMainThread main(this);
+  DebugScopedSetMainThread main(task_runner_provider_);
   DoCommit();
 }
 
-void SingleThreadProxy::ScheduledActionAnimate() {
-  TRACE_EVENT0("cc", "ScheduledActionAnimate");
-  DebugScopedSetImplThread impl(this);
-  DoAnimate();
-}
-
 void SingleThreadProxy::ScheduledActionActivateSyncTree() {
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   layer_tree_host_impl_->ActivateSyncTree();
 }
 
 void SingleThreadProxy::ScheduledActionBeginOutputSurfaceCreation() {
-  DebugScopedSetMainThread main(this);
+  DebugScopedSetMainThread main(task_runner_provider_);
   DCHECK(scheduler_on_impl_thread_);
   // If possible, create the output surface in a post task.  Synchronously
   // creating the output surface makes tests more awkward since this differs
   // from the ThreadProxy behavior.  However, sometimes there is no
   // task runner.
-  if (Proxy::MainThreadTaskRunner()) {
+  if (task_runner_provider_->MainThreadTaskRunner()) {
     ScheduleRequestNewOutputSurface();
   } else {
     RequestNewOutputSurface();
@@ -907,7 +868,7 @@ void SingleThreadProxy::ScheduledActionBeginOutputSurfaceCreation() {
 
 void SingleThreadProxy::ScheduledActionPrepareTiles() {
   TRACE_EVENT0("cc", "SingleThreadProxy::ScheduledActionPrepareTiles");
-  DebugScopedSetImplThread impl(this);
+  DebugScopedSetImplThread impl(task_runner_provider_);
   layer_tree_host_impl_->PrepareTiles();
 }
 
@@ -915,6 +876,12 @@ void SingleThreadProxy::ScheduledActionInvalidateOutputSurface() {
   NOTREACHED();
 }
 
+void SingleThreadProxy::UpdateTopControlsState(TopControlsState constraints,
+                                               TopControlsState current,
+                                               bool animate) {
+  NOTREACHED() << "Top Controls are used only in threaded mode";
+}
+
 void SingleThreadProxy::DidFinishImplFrame() {
   layer_tree_host_impl_->DidFinishImplFrame();
 #if DCHECK_IS_ON()
@@ -924,8 +891,4 @@ void SingleThreadProxy::DidFinishImplFrame() {
 #endif
 }
 
-void SingleThreadProxy::SendBeginFramesToChildren(const BeginFrameArgs& args) {
-  layer_tree_host_->SendBeginFramesToChildren(args);
-}
-
 }  // namespace cc
