@@ -130,8 +130,7 @@ function PDFViewer(browserApi) {
 
   this.delayedScriptingMessages_ = [];
 
-  this.isPrintPreview_ = this.originalUrl_.indexOf(
-                             'chrome://print') == 0;
+  this.isPrintPreview_ = location.origin === 'chrome://print';
   // Parse open pdf parameters.
   this.paramsParser_ =
       new OpenPDFParamsParser(this.getNamedDestination_.bind(this));
@@ -237,6 +236,10 @@ function PDFViewer(browserApi) {
     this.viewport_.goToPage(e.detail.page);
   }.bind(this));
 
+  document.body.addEventListener('navigate', function(e) {
+    this.navigator_.navigate(e.detail.uri, e.detail.newtab);
+  }.bind(this));
+
   this.toolbarManager_ =
       new ToolbarManager(window, this.toolbar_, this.zoomToolbar_);
 
@@ -600,9 +603,6 @@ PDFViewer.prototype = {
             '&body=' + message.data.body;
         window.location.href = href;
         break;
-      case 'getAccessibilityJSONReply':
-        this.sendScriptingMessage_(message.data);
-        break;
       case 'getPassword':
         // If the password screen isn't up, put it up. Otherwise we're
         // responding to an incorrect password so deny it.
@@ -778,7 +778,6 @@ PDFViewer.prototype = {
     }
 
     switch (message.data.type.toString()) {
-      case 'getAccessibilityJSON':
       case 'getSelectedText':
       case 'print':
       case 'selectAll':
