@@ -184,14 +184,6 @@ cr.define('print_preview', function() {
     this.selectionOnly_ =
         new print_preview.ticket_items.SelectionOnly(this.documentInfo_);
 
-    /**
-     * Print friendly ticket item.
-     * @type {!print_preview.ticket_items.DistillPage}
-     * @private
-     */
-    this.distillPage_ = new print_preview.ticket_items.DistillPage(
-        this.documentInfo_);
-
     /**
      * Vendor ticket items.
      * @type {!print_preview.ticket_items.VendorItems}
@@ -275,10 +267,6 @@ cr.define('print_preview', function() {
       return this.headerFooter_;
     },
 
-    get distillPage() {
-      return this.distillPage_;
-    },
-
     get mediaSize() {
       return this.mediaSize_;
     },
