@@ -3,14 +3,11 @@
 // found in the LICENSE file.
 
 /**
- * @fileoverview 'cr-search-engines-list' is a component for showing a list of
- * search engines.
- *
- * @group Chrome Settings Elements
- * @element cr-search-engines-list
+ * @fileoverview 'settings-search-engines-list' is a component for showing a
+ * list of search engines.
  */
 Polymer({
-  is: 'cr-search-engines-list',
+  is: 'settings-search-engines-list',
 
   properties: {
     /** @type {!Array<!SearchEngine>} */
