@@ -9,8 +9,11 @@
  * Example:
  *
  *    <cr-settings></cr-settings>
- *
- * @group Chrome Settings Elements
- * @element cr-settings
  */
-Polymer({is: 'cr-settings'});
+Polymer({
+  is: 'cr-settings',
+
+  ready: function() {
+    this.$.ui.directionDelegate = new settings.DirectionDelegateImpl;
+  },
+});
