@@ -92,14 +92,12 @@ if (!chrome.embeddedSearch) {
       native function CheckIsUserSyncingHistory();
       native function DeleteMostVisitedItem();
       native function GetAppLauncherEnabled();
-      native function GetDispositionFromClick();
       native function GetMostVisitedItems();
       native function GetThemeBackgroundInfo();
       native function IsInputInProgress();
       native function LogEvent();
       native function LogMostVisitedImpression();
       native function LogMostVisitedNavigation();
-      native function NavigateContentWindow();
       native function UndoAllMostVisitedDeletions();
       native function UndoMostVisitedDeletion();
 
@@ -139,18 +137,6 @@ if (!chrome.embeddedSearch) {
         DeleteMostVisitedItem(restrictedId);
       };
 
-      this.getDispositionFromClick = function(middle_button,
-                                              alt_key,
-                                              ctrl_key,
-                                              meta_key,
-                                              shift_key) {
-        return GetDispositionFromClick(middle_button,
-                                       alt_key,
-                                       ctrl_key,
-                                       meta_key,
-                                       shift_key);
-      };
-
       this.checkIsUserSignedIntoChromeAs = function(identity) {
         CheckIsUserSignedInToChromeAs(identity);
       };
@@ -177,10 +163,6 @@ if (!chrome.embeddedSearch) {
         LogMostVisitedNavigation(position, provider);
       };
 
-      this.navigateContentWindow = function(destination, disposition) {
-        NavigateContentWindow(destination, disposition);
-      };
-
       this.undoAllMostVisitedDeletions = function() {
         UndoAllMostVisitedDeletions();
       };
