@@ -12,7 +12,7 @@ var CreateEvent = require('guestViewEvents').CreateEvent;
 var EventBindings = require('event_bindings');
 var GuestViewInternalNatives = requireNative('guest_view_internal');
 var idGeneratorNatives = requireNative('id_generator');
-var Utils = require('utils');
+var utils = require('utils');
 var WebViewImpl = require('webView').WebViewImpl;
 
 // This is the only "webViewInternal.onClicked" named event for this renderer.
@@ -37,41 +37,43 @@ function ContextMenusOnClickedEvent(webViewInstanceId,
                                     opt_argSchemas,
                                     opt_eventOptions) {
   var subEventName = GetUniqueSubEventName(opt_eventName);
-  EventBindings.Event.call(this,
-                           subEventName,
-                           opt_argSchemas,
-                           opt_eventOptions,
-                           webViewInstanceId);
+  $Function.call(EventBindings.Event,
+                 this,
+                 subEventName,
+                 opt_argSchemas,
+                 opt_eventOptions,
+                 webViewInstanceId);
 
   var view = GuestViewInternalNatives.GetViewFromID(webViewInstanceId);
   if (!view) {
     return;
   }
-  view.events.addScopedListener(ContextMenusEvent, $Function.bind(function() {
+  view.events.addScopedListener(ContextMenusEvent, function() {
     // Re-dispatch to subEvent's listeners.
     $Function.apply(this.dispatch, this, $Array.slice(arguments));
-  }, this), {instanceId: webViewInstanceId});
+  }.bind(this), {instanceId: webViewInstanceId});
 }
+$Object.setPrototypeOf(ContextMenusOnClickedEvent.prototype,
+                       EventBindings.Event.prototype);
 
-ContextMenusOnClickedEvent.prototype.__proto__ = EventBindings.Event.prototype;
-
+// This event is exposed as <webview>.contextMenus.onShow.
 function ContextMenusOnContextMenuEvent(webViewInstanceId,
                                         opt_eventName,
                                         opt_argSchemas,
                                         opt_eventOptions) {
   var subEventName = GetUniqueSubEventName(opt_eventName);
-  EventBindings.Event.call(this,
-                           subEventName,
-                           opt_argSchemas,
-                           opt_eventOptions,
-                           webViewInstanceId);
+  $Function.call(EventBindings.Event,
+                 this,
+                 subEventName,
+                 opt_argSchemas,
+                 opt_eventOptions,
+                 webViewInstanceId);
 
   var view = GuestViewInternalNatives.GetViewFromID(webViewInstanceId);
   if (!view) {
     return;
   }
-  view.events.addScopedListener(
-      ContextMenusHandlerEvent, $Function.bind(function(e) {
+  view.events.addScopedListener(ContextMenusHandlerEvent, function(e) {
     var defaultPrevented = false;
     var event = {
       'preventDefault': function() { defaultPrevented = true; }
@@ -89,11 +91,11 @@ function ContextMenusOnContextMenuEvent(webViewInstanceId,
           GetViewFromID(webViewInstanceId).guest.getId();
       ChromeWebView.showContextMenu(guestInstanceId, e.requestId, items);
     }
-  }, this), {instanceId: webViewInstanceId});
+  }.bind(this), {instanceId: webViewInstanceId});
 }
 
-ContextMenusOnContextMenuEvent.prototype.__proto__ =
-    EventBindings.Event.prototype;
+$Object.setPrototypeOf(ContextMenusOnContextMenuEvent.prototype,
+                       EventBindings.Event.prototype);
 
 // -----------------------------------------------------------------------------
 // WebViewContextMenusImpl object.
@@ -102,6 +104,7 @@ ContextMenusOnContextMenuEvent.prototype.__proto__ =
 function WebViewContextMenusImpl(viewInstanceId) {
   this.viewInstanceId_ = viewInstanceId;
 }
+$Object.setPrototypeOf(WebViewContextMenusImpl.prototype, null);
 
 WebViewContextMenusImpl.prototype.create = function() {
   var args = $Array.concat([this.viewInstanceId_], $Array.slice(arguments));
@@ -123,9 +126,17 @@ WebViewContextMenusImpl.prototype.update = function() {
   return $Function.apply(ChromeWebView.contextMenusUpdate, null, args);
 };
 
-var WebViewContextMenus = Utils.expose(
-    'WebViewContextMenus', WebViewContextMenusImpl,
-    { functions: ['create', 'remove', 'removeAll', 'update'] });
+function WebViewContextMenus() {
+  privates(WebViewContextMenus).constructPrivate(this, arguments);
+}
+utils.expose(WebViewContextMenus, WebViewContextMenusImpl, {
+  functions: [
+    'create',
+    'remove',
+    'removeAll',
+    'update',
+  ],
+});
 
 // -----------------------------------------------------------------------------
 
@@ -133,13 +144,13 @@ WebViewImpl.prototype.maybeSetupContextMenus = function() {
   if (!this.contextMenusOnContextMenuEvent_) {
     var eventName = 'chromeWebViewInternal.onContextMenuShow';
     var eventSchema =
-        Utils.lookup(ChromeWebViewSchema.events, 'name', 'onShow');
+        utils.lookup(ChromeWebViewSchema.events, 'name', 'onShow');
     var eventOptions = {supportsListeners: true};
     this.contextMenusOnContextMenuEvent_ = new ContextMenusOnContextMenuEvent(
         this.viewInstanceId, eventName, eventSchema, eventOptions);
   }
 
-  var createContextMenus = $Function.bind(function() {
+  var createContextMenus = function() {
     return this.weakWrapper(function() {
       if (this.contextMenus_) {
         return this.contextMenus_;
@@ -148,12 +159,12 @@ WebViewImpl.prototype.maybeSetupContextMenus = function() {
       this.contextMenus_ = new WebViewContextMenus(this.viewInstanceId);
 
       // Define 'onClicked' event property on |this.contextMenus_|.
-      var getOnClickedEvent = $Function.bind(function() {
+      var getOnClickedEvent = function() {
         return this.weakWrapper(function() {
           if (!this.contextMenusOnClickedEvent_) {
             var eventName = 'chromeWebViewInternal.onClicked';
             var eventSchema =
-                Utils.lookup(ChromeWebViewSchema.events, 'name', 'onClicked');
+                utils.lookup(ChromeWebViewSchema.events, 'name', 'onClicked');
             var eventOptions = {supportsListeners: true};
             var onClickedEvent = new ContextMenusOnClickedEvent(
                 this.viewInstanceId, eventName, eventSchema, eventOptions);
@@ -162,7 +173,7 @@ WebViewImpl.prototype.maybeSetupContextMenus = function() {
           }
           return this.contextMenusOnClickedEvent_;
         });
-      }, this);
+      }.bind(this);
       $Object.defineProperty(
           this.contextMenus_,
           'onClicked',
@@ -178,11 +189,9 @@ WebViewImpl.prototype.maybeSetupContextMenus = function() {
           });
       return this.contextMenus_;
     });
-  }, this);
+  }.bind(this);
 
   // Expose <webview>.contextMenus object.
-  // TODO(lazyboy): Add documentation for contextMenus:
-  // http://crbug.com/470979.
   $Object.defineProperty(
       this.element,
       'contextMenus',
