@@ -15,7 +15,8 @@
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/stl_util.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #ifndef TOOLKIT_QT
 #include "chrome/common/channel_info.h"
@@ -196,8 +197,7 @@ void SpellCheck::FillSuggestions(
 
     const base::string16& suggestion = suggestions_list[language][index];
     // Only add the suggestion if it's unique.
-    if (std::find(optional_suggestions->begin(), optional_suggestions->end(),
-                  suggestion) == optional_suggestions->end()) {
+    if (!ContainsValue(*optional_suggestions, suggestion)) {
         optional_suggestions->push_back(suggestion);
     }
     if (optional_suggestions->size() >=
