@@ -5,8 +5,9 @@
 #ifndef CHROMECAST_APP_CAST_MAIN_DELEGATE_H_
 #define CHROMECAST_APP_CAST_MAIN_DELEGATE_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "build/build_config.h"
 #include "chromecast/common/cast_content_client.h"
 #include "content/public/app/content_main_delegate.h"
@@ -23,6 +24,7 @@ namespace shell {
 
 class CastContentBrowserClient;
 class CastContentRendererClient;
+class CastContentUtilityClient;
 
 class CastMainDelegate : public content::ContentMainDelegate {
  public:
@@ -35,23 +37,24 @@ class CastMainDelegate : public content::ContentMainDelegate {
   int RunProcess(
       const std::string& process_type,
       const content::MainFunctionParams& main_function_params) override;
-  void ProcessExiting(const std::string& process_type) override;
 #if !defined(OS_ANDROID)
   void ZygoteForked() override;
 #endif  // !defined(OS_ANDROID)
   content::ContentBrowserClient* CreateContentBrowserClient() override;
   content::ContentRendererClient* CreateContentRendererClient() override;
+  content::ContentUtilityClient* CreateContentUtilityClient() override;
 
  private:
   void InitializeResourceBundle();
 
-  scoped_ptr<CastContentBrowserClient> browser_client_;
-  scoped_ptr<CastContentRendererClient> renderer_client_;
-  scoped_ptr<CastResourceDelegate> resource_delegate_;
+  std::unique_ptr<CastContentBrowserClient> browser_client_;
+  std::unique_ptr<CastContentRendererClient> renderer_client_;
+  std::unique_ptr<CastContentUtilityClient> utility_client_;
+  std::unique_ptr<CastResourceDelegate> resource_delegate_;
   CastContentClient content_client_;
 
 #if defined(OS_ANDROID)
-  scoped_ptr<content::BrowserMainRunner> browser_runner_;
+  std::unique_ptr<content::BrowserMainRunner> browser_runner_;
 #endif  // defined(OS_ANDROID)
 
   DISALLOW_COPY_AND_ASSIGN(CastMainDelegate);
