@@ -4,8 +4,11 @@
 
 #include "components/error_page/renderer/net_error_helper_core.h"
 
+#include <stddef.h>
+
 #include <set>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/bind.h"
@@ -16,18 +19,20 @@
 #include "base/json/json_writer.h"
 #include "base/location.h"
 #include "base/logging.h"
+#include "base/macros.h"
 #include "base/memory/scoped_vector.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/metrics/sparse_histogram.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/values.h"
+#include "build/build_config.h"
 #include "components/error_page/common/error_page_params.h"
+#include "components/url_formatter/url_formatter.h"
 #include "content/public/common/url_constants.h"
 #include "grit/components_strings.h"
 #include "net/base/escape.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_util.h"
 #include "third_party/WebKit/public/platform/WebString.h"
 #include "third_party/WebKit/public/platform/WebURLError.h"
 #include "ui/base/l10n/l10n_util.h"
@@ -38,26 +43,20 @@ namespace error_page {
 
 namespace {
 
-// Recorded when a user interacts with an error page, part of investigation into
-// http://crbug.com/500556.  Note that these are flags that can be or-ed
-// together.
-// TODO(mmenke): Remove when http://crbug.com/500556 is fixed.
-enum ErrorPageUnexpectedStateFlags {
-  // According to blink, the page reporting a button press is not an error page.
-  ERROR_PAGE_BUTTON_PRESS_FOR_NON_ERROR_PAGE_URL = 0x1,
-  // NetErrorHelperCore's state machine things the current page is not an error
-  // page.
-  ERROR_PAGE_BUTTON_PRESS_WITH_NO_ERROR_INFO     = 0x2,
+// |NetErrorNavigationCorrectionTypes| enum id for Web search query.
+// Other correction types uses the |kCorrectionResourceTable| array order.
+const int kWebSearchQueryUMAId = 100;
 
-  // Must be strictly greater than a value with all flags set.
-  ERROR_PAGE_BUTTON_PRESS_MAX                    = 0x4,
-};
+// Number of URL correction suggestions to display.
+const int kMaxUrlCorrectionsToDisplay = 1;
 
 struct CorrectionTypeToResourceTable {
   int resource_id;
   const char* correction_type;
 };
 
+// Note: Ordering should be the same as |NetErrorNavigationCorrectionTypes| enum
+// in histograms.xml.
 const CorrectionTypeToResourceTable kCorrectionResourceTable[] = {
   {IDS_ERRORPAGES_SUGGESTION_VISIT_GOOGLE_CACHE, "cachedPage"},
   // "reloadPage" is has special handling.
@@ -146,7 +145,7 @@ GURL SanitizeURL(const GURL& url) {
 
 // Sanitizes and formats a URL for upload to the error correction service.
 std::string PrepareUrlForUpload(const GURL& url) {
-  // TODO(yuusuke): Change to net::FormatUrl when Link Doctor becomes
+  // TODO(yuusuke): Change to url_formatter::FormatUrl when Link Doctor becomes
   // unicode-capable.
   std::string spec_to_send = SanitizeURL(url).spec();
 
@@ -196,7 +195,7 @@ std::string CreateRequestBody(
     const std::string& method,
     const std::string& error_param,
     const NetErrorHelperCore::NavigationCorrectionParams& correction_params,
-    scoped_ptr<base::DictionaryValue> params_dict) {
+    std::unique_ptr<base::DictionaryValue> params_dict) {
   // Set params common to all request types.
   params_dict->SetString("key", correction_params.api_key);
   params_dict->SetString("clientName", "chrome");
@@ -231,10 +230,10 @@ std::string CreateFixUrlRequestBody(
 
   // TODO(mmenke):  Investigate open sourcing the relevant protocol buffers and
   //                using those directly instead.
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
   params->SetString("urlQuery", PrepareUrlForUpload(error.unreachableURL));
   return CreateRequestBody("linkdoctor.fixurl.fixurl", error_param,
-                           correction_params, params.Pass());
+                           correction_params, std::move(params));
 }
 
 std::string CreateClickTrackingUrlRequestBody(
@@ -246,7 +245,7 @@ std::string CreateClickTrackingUrlRequestBody(
   bool result = ShouldUseFixUrlServiceForError(error, &error_param);
   DCHECK(result);
 
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
 
   params->SetString("originalUrlQuery",
                     PrepareUrlForUpload(error.unreachableURL));
@@ -259,48 +258,51 @@ std::string CreateClickTrackingUrlRequestBody(
   params->SetString("fingerprint", response.fingerprint);
 
   return CreateRequestBody("linkdoctor.fixurl.clicktracking", error_param,
-                           correction_params, params.Pass());
+                           correction_params, std::move(params));
 }
 
-base::string16 FormatURLForDisplay(const GURL& url, bool is_rtl,
-                                   const std::string accept_languages) {
+base::string16 FormatURLForDisplay(const GURL& url, bool is_rtl) {
   // Translate punycode into UTF8, unescape UTF8 URLs.
-  base::string16 url_for_display(net::FormatUrl(
-      url, accept_languages, net::kFormatUrlOmitNothing,
-      net::UnescapeRule::NORMAL, NULL, NULL, NULL));
+  base::string16 url_for_display(url_formatter::FormatUrl(
+      url, url_formatter::kFormatUrlOmitNothing,
+      net::UnescapeRule::NORMAL, nullptr, nullptr, nullptr));
   // URLs are always LTR.
   if (is_rtl)
     base::i18n::WrapStringWithLTRFormatting(&url_for_display);
   return url_for_display;
 }
 
-scoped_ptr<NavigationCorrectionResponse> ParseNavigationCorrectionResponse(
+std::unique_ptr<NavigationCorrectionResponse> ParseNavigationCorrectionResponse(
     const std::string raw_response) {
   // TODO(mmenke):  Open source related protocol buffers and use them directly.
-  scoped_ptr<base::Value> parsed = base::JSONReader::Read(raw_response);
-  scoped_ptr<NavigationCorrectionResponse> response(
+  std::unique_ptr<base::Value> parsed = base::JSONReader::Read(raw_response);
+  std::unique_ptr<NavigationCorrectionResponse> response(
       new NavigationCorrectionResponse());
   base::JSONValueConverter<NavigationCorrectionResponse> converter;
   if (!parsed || !converter.Convert(*parsed, response.get()))
     response.reset();
-  return response.Pass();
+  return response;
 }
 
-scoped_ptr<ErrorPageParams> CreateErrorPageParams(
+void LogCorrectionTypeShown(int type_id) {
+  UMA_HISTOGRAM_ENUMERATION(
+      "Net.ErrorPageCounts.NavigationCorrectionLinksShown", type_id,
+      kWebSearchQueryUMAId + 1);
+}
+
+std::unique_ptr<ErrorPageParams> CreateErrorPageParams(
     const NavigationCorrectionResponse& response,
     const blink::WebURLError& error,
     const NetErrorHelperCore::NavigationCorrectionParams& correction_params,
-    const std::string& accept_languages,
     bool is_rtl) {
   // Version of URL for display in suggestions.  It has to be sanitized first
   // because any received suggestions will be relative to the sanitized URL.
   base::string16 original_url_for_display =
-      FormatURLForDisplay(SanitizeURL(GURL(error.unreachableURL)), is_rtl,
-                          accept_languages);
+      FormatURLForDisplay(SanitizeURL(GURL(error.unreachableURL)), is_rtl);
 
-  scoped_ptr<ErrorPageParams> params(new ErrorPageParams());
+  std::unique_ptr<ErrorPageParams> params(new ErrorPageParams());
   params->override_suggestions.reset(new base::ListValue());
-  scoped_ptr<base::ListValue> parsed_corrections(new base::ListValue());
+  std::unique_ptr<base::ListValue> parsed_corrections(new base::ListValue());
   for (ScopedVector<NavigationCorrection>::const_iterator it =
            response.corrections.begin();
        it != response.corrections.end(); ++it) {
@@ -317,19 +319,24 @@ scoped_ptr<ErrorPageParams> CreateErrorPageParams(
     }
 
     if ((*it)->correction_type == "webSearchQuery") {
-      // If there are mutliple searches suggested, use the first suggestion.
+      // If there are multiple searches suggested, use the first suggestion.
       if (params->search_terms.empty()) {
         params->search_url = correction_params.search_url;
         params->search_terms = (*it)->url_correction;
         params->search_tracking_id = tracking_id;
+        LogCorrectionTypeShown(kWebSearchQueryUMAId);
       }
       continue;
     }
 
     // Allow reload page and web search query to be empty strings, but not
     // links.
-    if ((*it)->url_correction.empty())
+    if ((*it)->url_correction.empty() ||
+        (params->override_suggestions->GetSize() >=
+            kMaxUrlCorrectionsToDisplay)) {
       continue;
+    }
+
     size_t correction_index;
     for (correction_index = 0;
          correction_index < arraysize(kCorrectionResourceTable);
@@ -338,27 +345,28 @@ scoped_ptr<ErrorPageParams> CreateErrorPageParams(
               kCorrectionResourceTable[correction_index].correction_type) {
         continue;
       }
-      base::DictionaryValue* suggest = new base::DictionaryValue();
-      suggest->SetString("header",
+      std::unique_ptr<base::DictionaryValue> suggest(
+          new base::DictionaryValue());
+      suggest->SetString("summary",
           l10n_util::GetStringUTF16(
               kCorrectionResourceTable[correction_index].resource_id));
       suggest->SetString("urlCorrection", (*it)->url_correction);
       suggest->SetString(
           "urlCorrectionForDisplay",
-          FormatURLForDisplay(GURL((*it)->url_correction), is_rtl,
-                              accept_languages));
+          FormatURLForDisplay(GURL((*it)->url_correction), is_rtl));
       suggest->SetString("originalUrlForDisplay", original_url_for_display);
       suggest->SetInteger("trackingId", tracking_id);
       suggest->SetInteger("type", static_cast<int>(correction_index));
 
-      params->override_suggestions->Append(suggest);
+      params->override_suggestions->Append(std::move(suggest));
+      LogCorrectionTypeShown(static_cast<int>(correction_index));
       break;
     }
   }
 
   if (params->override_suggestions->empty() && !params->search_url.is_valid())
     params.reset();
-  return params.Pass();
+  return params;
 }
 
 void ReportAutoReloadSuccess(const blink::WebURLError& error, size_t count) {
@@ -381,25 +389,53 @@ void ReportAutoReloadFailure(const blink::WebURLError& error, size_t count) {
                        static_cast<base::HistogramBase::Sample>(count));
 }
 
+// Tracks navigation correction service usage in UMA to enable more in depth
+// analysis.
+void TrackClickUMA(std::string type_id) {
+  // Web search suggestion isn't in |kCorrectionResourceTable| array.
+  if (type_id == "webSearchQuery") {
+    UMA_HISTOGRAM_ENUMERATION(
+       "Net.ErrorPageCounts.NavigationCorrectionLinksUsed",
+       kWebSearchQueryUMAId, kWebSearchQueryUMAId + 1);
+    return;
+  }
+
+  size_t correction_index;
+  for (correction_index = 0;
+       correction_index < arraysize(kCorrectionResourceTable);
+       ++correction_index) {
+    if (kCorrectionResourceTable[correction_index].correction_type ==
+        type_id) {
+      UMA_HISTOGRAM_ENUMERATION(
+         "Net.ErrorPageCounts.NavigationCorrectionLinksUsed",
+         static_cast<int>(correction_index), kWebSearchQueryUMAId + 1);
+      break;
+    }
+  }
+}
+
 }  // namespace
 
 struct NetErrorHelperCore::ErrorPageInfo {
-  ErrorPageInfo(blink::WebURLError error, bool was_failed_post)
+  ErrorPageInfo(blink::WebURLError error,
+                bool was_failed_post,
+                bool was_ignoring_cache)
       : error(error),
         was_failed_post(was_failed_post),
+        was_ignoring_cache(was_ignoring_cache),
         needs_dns_updates(false),
         needs_load_navigation_corrections(false),
         reload_button_in_page(false),
         show_saved_copy_button_in_page(false),
-        show_cached_page_button_in_page(false),
         show_cached_copy_button_in_page(false),
+        show_offline_pages_button_in_page(false),
         is_finished_loading(false),
-        auto_reload_triggered(false) {
-  }
+        auto_reload_triggered(false) {}
 
   // Information about the failed page load.
   blink::WebURLError error;
   bool was_failed_post;
+  bool was_ignoring_cache;
 
   // Information about the status of the error page.
 
@@ -414,10 +450,10 @@ struct NetErrorHelperCore::ErrorPageInfo {
   // Navigation correction service paramers, which will be used in response to
   // certain types of network errors.  They are all stored here in case they
   // change over the course of displaying the error page.
-  scoped_ptr<NetErrorHelperCore::NavigationCorrectionParams>
+  std::unique_ptr<NetErrorHelperCore::NavigationCorrectionParams>
       navigation_correction_params;
 
-  scoped_ptr<NavigationCorrectionResponse> navigation_correction_response;
+  std::unique_ptr<NavigationCorrectionResponse> navigation_correction_response;
 
   // All the navigation corrections that have been clicked, for tracking
   // purposes.
@@ -426,8 +462,8 @@ struct NetErrorHelperCore::ErrorPageInfo {
   // Track if specific buttons are included in an error page, for statistics.
   bool reload_button_in_page;
   bool show_saved_copy_button_in_page;
-  bool show_cached_page_button_in_page;
   bool show_cached_copy_button_in_page;
+  bool show_offline_pages_button_in_page;
 
   // True if a page has completed loading, at which point it can receive
   // updates.
@@ -441,6 +477,9 @@ struct NetErrorHelperCore::ErrorPageInfo {
 NetErrorHelperCore::NavigationCorrectionParams::NavigationCorrectionParams() {
 }
 
+NetErrorHelperCore::NavigationCorrectionParams::NavigationCorrectionParams(
+    const NavigationCorrectionParams& other) = default;
+
 NetErrorHelperCore::NavigationCorrectionParams::~NavigationCorrectionParams() {
 }
 
@@ -470,7 +509,8 @@ NetErrorHelperCore::NetErrorHelperCore(Delegate* delegate,
                                        bool auto_reload_visible_only,
                                        bool is_visible)
     : delegate_(delegate),
-      last_probe_status_(chrome_common_net::DNS_PROBE_POSSIBLE),
+      last_probe_status_(DNS_PROBE_POSSIBLE),
+      can_show_network_diagnostics_dialog_(false),
       auto_reload_enabled_(auto_reload_enabled),
       auto_reload_visible_only_(auto_reload_visible_only),
       auto_reload_timer_(new base::Timer(false, false)),
@@ -481,6 +521,9 @@ NetErrorHelperCore::NetErrorHelperCore(Delegate* delegate,
       online_(true),
       visible_(is_visible),
       auto_reload_count_(0),
+#if defined(OS_ANDROID)
+      has_offline_pages_(false),
+#endif  // defined(OS_ANDROID)
       navigation_from_button_(NO_BUTTON) {
 }
 
@@ -569,10 +612,9 @@ void NetErrorHelperCore::OnCommitLoad(FrameType frame_type, const GURL& url) {
           pending_error_page_info_->error.unreachableURL) {
     DCHECK(navigation_from_button_ == RELOAD_BUTTON ||
            navigation_from_button_ == SHOW_SAVED_COPY_BUTTON);
-    chrome_common_net::RecordEvent(
-        navigation_from_button_ == RELOAD_BUTTON ?
-            chrome_common_net::NETWORK_ERROR_PAGE_RELOAD_BUTTON_ERROR :
-            chrome_common_net::NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_ERROR);
+    RecordEvent(navigation_from_button_ == RELOAD_BUTTON ?
+        NETWORK_ERROR_PAGE_RELOAD_BUTTON_ERROR :
+        NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_ERROR);
   }
   navigation_from_button_ = NO_BUTTON;
 
@@ -600,26 +642,22 @@ void NetErrorHelperCore::OnFinishLoad(FrameType frame_type) {
 
   committed_error_page_info_->is_finished_loading = true;
 
-  chrome_common_net::RecordEvent(chrome_common_net::NETWORK_ERROR_PAGE_SHOWN);
+  RecordEvent(NETWORK_ERROR_PAGE_SHOWN);
   if (committed_error_page_info_->reload_button_in_page) {
-    chrome_common_net::RecordEvent(
-        chrome_common_net::NETWORK_ERROR_PAGE_RELOAD_BUTTON_SHOWN);
+    RecordEvent(NETWORK_ERROR_PAGE_RELOAD_BUTTON_SHOWN);
   }
   if (committed_error_page_info_->show_saved_copy_button_in_page) {
-    chrome_common_net::RecordEvent(
-        chrome_common_net::NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_SHOWN);
+    RecordEvent(NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_SHOWN);
+  }
+  if (committed_error_page_info_->show_offline_pages_button_in_page) {
+    RecordEvent(NETWORK_ERROR_PAGE_SHOW_OFFLINE_PAGES_BUTTON_SHOWN);
   }
   if (committed_error_page_info_->reload_button_in_page &&
       committed_error_page_info_->show_saved_copy_button_in_page) {
-    chrome_common_net::RecordEvent(
-        chrome_common_net::NETWORK_ERROR_PAGE_BOTH_BUTTONS_SHOWN);
+    RecordEvent(NETWORK_ERROR_PAGE_BOTH_BUTTONS_SHOWN);
   }
   if (committed_error_page_info_->show_cached_copy_button_in_page) {
-    chrome_common_net::RecordEvent(
-        chrome_common_net::NETWORK_ERROR_PAGE_CACHED_COPY_BUTTON_SHOWN);
-  } else if (committed_error_page_info_->show_cached_page_button_in_page) {
-    chrome_common_net::RecordEvent(
-        chrome_common_net::NETWORK_ERROR_PAGE_CACHED_PAGE_BUTTON_SHOWN);
+    RecordEvent(NETWORK_ERROR_PAGE_CACHED_COPY_BUTTON_SHOWN);
   }
 
   delegate_->EnablePageHelperFunctions();
@@ -640,25 +678,26 @@ void NetErrorHelperCore::OnFinishLoad(FrameType frame_type) {
   }
 
   if (!committed_error_page_info_->needs_dns_updates ||
-      last_probe_status_ == chrome_common_net::DNS_PROBE_POSSIBLE) {
+      last_probe_status_ == DNS_PROBE_POSSIBLE) {
     return;
   }
   DVLOG(1) << "Error page finished loading; sending saved status.";
   UpdateErrorPage();
 }
 
-void NetErrorHelperCore::GetErrorHTML(
-    FrameType frame_type,
-    const blink::WebURLError& error,
-    bool is_failed_post,
-    std::string* error_html) {
+void NetErrorHelperCore::GetErrorHTML(FrameType frame_type,
+                                      const blink::WebURLError& error,
+                                      bool is_failed_post,
+                                      bool is_ignoring_cache,
+                                      std::string* error_html) {
   if (frame_type == MAIN_FRAME) {
     // If navigation corrections were needed before, that should have been
     // cancelled earlier by starting a new page load (Which has now failed).
     DCHECK(!committed_error_page_info_ ||
            !committed_error_page_info_->needs_load_navigation_corrections);
 
-    pending_error_page_info_.reset(new ErrorPageInfo(error, is_failed_post));
+    pending_error_page_info_.reset(
+        new ErrorPageInfo(error, is_failed_post, is_ignoring_cache));
     pending_error_page_info_->navigation_correction_params.reset(
         new NavigationCorrectionParams(navigation_correction_params_));
     GetErrorHtmlForMainFrame(pending_error_page_info_.get(), error_html);
@@ -667,19 +706,20 @@ void NetErrorHelperCore::GetErrorHTML(
     bool reload_button_in_page;
     bool show_saved_copy_button_in_page;
     bool show_cached_copy_button_in_page;
-    bool show_cached_page_button_in_page;
+    bool show_offline_pages_button_in_page;
 
     delegate_->GenerateLocalizedErrorPage(
-        error, is_failed_post, scoped_ptr<ErrorPageParams>(),
+        error, is_failed_post,
+        false /* No diagnostics dialogs allowed for subframes. */,
+        false /* No offline button provided in subframes */, nullptr,
         &reload_button_in_page, &show_saved_copy_button_in_page,
-        &show_cached_copy_button_in_page, &show_cached_page_button_in_page,
+        &show_cached_copy_button_in_page, &show_offline_pages_button_in_page,
         error_html);
   }
 }
 
-void NetErrorHelperCore::OnNetErrorInfo(
-    chrome_common_net::DnsProbeStatus status) {
-  DCHECK_NE(chrome_common_net::DNS_PROBE_POSSIBLE, status);
+void NetErrorHelperCore::OnNetErrorInfo(DnsProbeStatus status) {
+  DCHECK_NE(DNS_PROBE_POSSIBLE, status);
 
   last_probe_status_ = status;
 
@@ -692,6 +732,11 @@ void NetErrorHelperCore::OnNetErrorInfo(
   UpdateErrorPage();
 }
 
+void NetErrorHelperCore::OnSetCanShowNetworkDiagnosticsDialog(
+    bool can_show_network_diagnostics_dialog) {
+  can_show_network_diagnostics_dialog_ = can_show_network_diagnostics_dialog;
+}
+
 void NetErrorHelperCore::OnSetNavigationCorrectionInfo(
     const GURL& navigation_correction_url,
     const std::string& language,
@@ -705,6 +750,12 @@ void NetErrorHelperCore::OnSetNavigationCorrectionInfo(
   navigation_correction_params_.search_url = search_url;
 }
 
+void NetErrorHelperCore::OnSetHasOfflinePages(bool has_offline_pages) {
+#if defined(OS_ANDROID)
+  has_offline_pages_ = has_offline_pages;
+#endif  // defined(OS_ANDROID)
+}
+
 void NetErrorHelperCore::GetErrorHtmlForMainFrame(
     ErrorPageInfo* pending_error_page_info,
     std::string* error_html) {
@@ -724,33 +775,32 @@ void NetErrorHelperCore::GetErrorHtmlForMainFrame(
     // loading, a DNS probe status scheduled to be sent to it may be thrown
     // out, but since the new error page should trigger a new DNS probe, it
     // will just get the results for the next page load.
-    last_probe_status_ = chrome_common_net::DNS_PROBE_POSSIBLE;
+    last_probe_status_ = DNS_PROBE_POSSIBLE;
     pending_error_page_info->needs_dns_updates = true;
     error = GetUpdatedError(error);
   }
 
   delegate_->GenerateLocalizedErrorPage(
       error, pending_error_page_info->was_failed_post,
-      scoped_ptr<ErrorPageParams>(),
+      can_show_network_diagnostics_dialog_, HasOfflinePages(), nullptr,
       &pending_error_page_info->reload_button_in_page,
       &pending_error_page_info->show_saved_copy_button_in_page,
       &pending_error_page_info->show_cached_copy_button_in_page,
-      &pending_error_page_info->show_cached_page_button_in_page,
-      error_html);
+      &pending_error_page_info->show_offline_pages_button_in_page, error_html);
 }
 
 void NetErrorHelperCore::UpdateErrorPage() {
   DCHECK(committed_error_page_info_->needs_dns_updates);
   DCHECK(committed_error_page_info_->is_finished_loading);
-  DCHECK_NE(chrome_common_net::DNS_PROBE_POSSIBLE, last_probe_status_);
+  DCHECK_NE(DNS_PROBE_POSSIBLE, last_probe_status_);
 
   UMA_HISTOGRAM_ENUMERATION("DnsProbe.ErrorPageUpdateStatus",
                             last_probe_status_,
-                            chrome_common_net::DNS_PROBE_MAX);
+                            DNS_PROBE_MAX);
   // Every status other than DNS_PROBE_POSSIBLE and DNS_PROBE_STARTED is a
   // final status code.  Once one is reached, the page does not need further
   // updates.
-  if (last_probe_status_ != chrome_common_net::DNS_PROBE_STARTED)
+  if (last_probe_status_ != DNS_PROBE_STARTED)
     committed_error_page_info_->needs_dns_updates = false;
 
   // There is no need to worry about the button display statistics here because
@@ -758,12 +808,13 @@ void NetErrorHelperCore::UpdateErrorPage() {
   // by a DNS error update.
   delegate_->UpdateErrorPage(
       GetUpdatedError(committed_error_page_info_->error),
-      committed_error_page_info_->was_failed_post);
+      committed_error_page_info_->was_failed_post,
+      can_show_network_diagnostics_dialog_,
+      HasOfflinePages());
 }
 
 void NetErrorHelperCore::OnNavigationCorrectionsFetched(
     const std::string& corrections,
-    const std::string& accept_languages,
     bool is_rtl) {
   // Loading suggestions only starts when a blank error page finishes loading,
   // and is cancelled with a new load.
@@ -772,14 +823,15 @@ void NetErrorHelperCore::OnNavigationCorrectionsFetched(
   DCHECK(committed_error_page_info_->needs_load_navigation_corrections);
   DCHECK(committed_error_page_info_->navigation_correction_params);
 
-  pending_error_page_info_.reset(
-      new ErrorPageInfo(committed_error_page_info_->error,
-                        committed_error_page_info_->was_failed_post));
+  pending_error_page_info_.reset(new ErrorPageInfo(
+      committed_error_page_info_->error,
+      committed_error_page_info_->was_failed_post,
+      committed_error_page_info_->was_ignoring_cache));
   pending_error_page_info_->navigation_correction_response =
       ParseNavigationCorrectionResponse(corrections);
 
   std::string error_html;
-  scoped_ptr<ErrorPageParams> params;
+  std::unique_ptr<ErrorPageParams> params;
   if (pending_error_page_info_->navigation_correction_response) {
     // Copy navigation correction parameters used for the request, so tracking
     // requests can still be sent if the configuration changes.
@@ -789,16 +841,15 @@ void NetErrorHelperCore::OnNavigationCorrectionsFetched(
     params = CreateErrorPageParams(
           *pending_error_page_info_->navigation_correction_response,
           pending_error_page_info_->error,
-          *pending_error_page_info_->navigation_correction_params,
-          accept_languages, is_rtl);
+          *pending_error_page_info_->navigation_correction_params, is_rtl);
     delegate_->GenerateLocalizedErrorPage(
         pending_error_page_info_->error,
         pending_error_page_info_->was_failed_post,
-        params.Pass(),
-        &pending_error_page_info_->reload_button_in_page,
+        can_show_network_diagnostics_dialog_, HasOfflinePages(),
+        std::move(params), &pending_error_page_info_->reload_button_in_page,
         &pending_error_page_info_->show_saved_copy_button_in_page,
         &pending_error_page_info_->show_cached_copy_button_in_page,
-        &pending_error_page_info_->show_cached_page_button_in_page,
+        &pending_error_page_info_->show_offline_pages_button_in_page,
         &error_html);
   } else {
     // Since |navigation_correction_params| in |pending_error_page_info_| is
@@ -809,23 +860,20 @@ void NetErrorHelperCore::OnNavigationCorrectionsFetched(
   // TODO(mmenke):  Once the new API is in place, look into replacing this
   //                double page load by just updating the error page, like DNS
   //                probes do.
-  delegate_->LoadErrorPageInMainFrame(
-      error_html,
-      pending_error_page_info_->error.unreachableURL);
+  delegate_->LoadErrorPage(error_html,
+                           pending_error_page_info_->error.unreachableURL);
 }
 
 blink::WebURLError NetErrorHelperCore::GetUpdatedError(
     const blink::WebURLError& error) const {
   // If a probe didn't run or wasn't conclusive, restore the original error.
-  if (last_probe_status_ == chrome_common_net::DNS_PROBE_NOT_RUN ||
-      last_probe_status_ ==
-          chrome_common_net::DNS_PROBE_FINISHED_INCONCLUSIVE) {
+  if (last_probe_status_ == DNS_PROBE_NOT_RUN ||
+      last_probe_status_ == DNS_PROBE_FINISHED_INCONCLUSIVE) {
     return error;
   }
 
   blink::WebURLError updated_error;
-  updated_error.domain = blink::WebString::fromUTF8(
-      chrome_common_net::kDnsProbeErrorDomain);
+  updated_error.domain = blink::WebString::fromUTF8(kDnsProbeErrorDomain);
   updated_error.reason = last_probe_status_;
   updated_error.unreachableURL = error.unreachableURL;
   updated_error.staleCopyInCache = error.staleCopyInCache;
@@ -833,11 +881,11 @@ blink::WebURLError NetErrorHelperCore::GetUpdatedError(
   return updated_error;
 }
 
-void NetErrorHelperCore::Reload() {
+void NetErrorHelperCore::Reload(bool bypass_cache) {
   if (!committed_error_page_info_) {
     return;
   }
-  delegate_->ReloadPage();
+  delegate_->ReloadPage(bypass_cache);
 }
 
 bool NetErrorHelperCore::MaybeStartAutoReloadTimer() {
@@ -881,7 +929,7 @@ void NetErrorHelperCore::AutoReloadTimerFired() {
 
   auto_reload_count_++;
   auto_reload_in_flight_ = true;
-  Reload();
+  Reload(committed_error_page_info_->was_ignoring_cache);
 }
 
 void NetErrorHelperCore::PauseAutoReloadTimer() {
@@ -929,68 +977,47 @@ bool NetErrorHelperCore::ShouldSuppressErrorPage(FrameType frame_type,
   return true;
 }
 
-void NetErrorHelperCore::ExecuteButtonPress(bool is_error_page, Button button) {
-  // UMA to investigate http://crbug.com/500556.
-  // TODO(mmenke): Remove when the issue is fixed.
-  int button_press_info = 0;
-  if (!is_error_page)
-    button_press_info |= ERROR_PAGE_BUTTON_PRESS_FOR_NON_ERROR_PAGE_URL;
-  if (!committed_error_page_info_)
-    button_press_info |= ERROR_PAGE_BUTTON_PRESS_WITH_NO_ERROR_INFO;
-  UMA_HISTOGRAM_ENUMERATION("Net.ErrorPageButtonPressUnexpectedStates",
-                            button_press_info,
-                            ERROR_PAGE_BUTTON_PRESS_MAX);
-  if (button_press_info != 0) {
-    UMA_HISTOGRAM_ENUMERATION(
-        "Net.ErrorPageButtonPressedWhileInUnexpectedState",
-        button, BUTTON_MAX);
-  }
-
-  // Unclear why this happens.
-  // TODO(mmenke): Remove when http://crbug.com/500556 is fixed.
-  if (!committed_error_page_info_)
-    return;
+void NetErrorHelperCore::ExecuteButtonPress(Button button) {
+  // If there's no committed error page, should not be invoked.
+  DCHECK(committed_error_page_info_);
 
   switch (button) {
     case RELOAD_BUTTON:
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_PAGE_RELOAD_BUTTON_CLICKED);
+      RecordEvent(NETWORK_ERROR_PAGE_RELOAD_BUTTON_CLICKED);
       if (committed_error_page_info_->show_saved_copy_button_in_page) {
-        chrome_common_net::RecordEvent(
-            chrome_common_net::NETWORK_ERROR_PAGE_BOTH_BUTTONS_RELOAD_CLICKED);
+        RecordEvent(NETWORK_ERROR_PAGE_BOTH_BUTTONS_RELOAD_CLICKED);
       }
       navigation_from_button_ = RELOAD_BUTTON;
-      Reload();
+      Reload(false);
       return;
     case SHOW_SAVED_COPY_BUTTON:
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_CLICKED);
+      RecordEvent(NETWORK_ERROR_PAGE_SHOW_SAVED_COPY_BUTTON_CLICKED);
       navigation_from_button_ = SHOW_SAVED_COPY_BUTTON;
       if (committed_error_page_info_->reload_button_in_page) {
-        chrome_common_net::RecordEvent(chrome_common_net::
-            NETWORK_ERROR_PAGE_BOTH_BUTTONS_SHOWN_SAVED_COPY_CLICKED);
+        RecordEvent(NETWORK_ERROR_PAGE_BOTH_BUTTONS_SHOWN_SAVED_COPY_CLICKED);
       }
       delegate_->LoadPageFromCache(
           committed_error_page_info_->error.unreachableURL);
       return;
     case MORE_BUTTON:
       // Visual effects on page are handled in Javascript code.
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_PAGE_MORE_BUTTON_CLICKED);
+      RecordEvent(NETWORK_ERROR_PAGE_MORE_BUTTON_CLICKED);
       return;
     case EASTER_EGG:
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_EASTER_EGG_ACTIVATED);
+      RecordEvent(NETWORK_ERROR_EASTER_EGG_ACTIVATED);
       return;
     case SHOW_CACHED_COPY_BUTTON:
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_PAGE_CACHED_COPY_BUTTON_CLICKED);
+      RecordEvent(NETWORK_ERROR_PAGE_CACHED_COPY_BUTTON_CLICKED);
       return;
-    case SHOW_CACHED_PAGE_BUTTON:
-      chrome_common_net::RecordEvent(
-          chrome_common_net::NETWORK_ERROR_PAGE_CACHED_PAGE_BUTTON_CLICKED);
+    case DIAGNOSE_ERROR:
+      RecordEvent(NETWORK_ERROR_DIAGNOSE_BUTTON_CLICKED);
+      delegate_->DiagnoseError(
+          committed_error_page_info_->error.unreachableURL);
+      return;
+    case SHOW_OFFLINE_PAGES_BUTTON:
+      RecordEvent(NETWORK_ERROR_PAGE_SHOW_OFFLINE_PAGES_BUTTON_CLICKED);
+      delegate_->ShowOfflinePages();
       return;
-    case BUTTON_MAX:
     case NO_BUTTON:
       NOTREACHED();
       return;
@@ -1022,6 +1049,8 @@ void NetErrorHelperCore::TrackClick(int tracking_id) {
   if (committed_error_page_info_->clicked_corrections.count(tracking_id))
     return;
 
+  TrackClickUMA(response->corrections[tracking_id]->correction_type);
+
   committed_error_page_info_->clicked_corrections.insert(tracking_id);
   std::string request_body = CreateClickTrackingUrlRequestBody(
       committed_error_page_info_->error,
@@ -1033,4 +1062,12 @@ void NetErrorHelperCore::TrackClick(int tracking_id) {
       request_body);
 }
 
+bool NetErrorHelperCore::HasOfflinePages() const {
+#if defined(OS_ANDROID)
+  return has_offline_pages_;
+#else
+  return false;
+#endif  // defined(OS_ANDROID)
+}
+
 }  // namespace error_page
