@@ -17,14 +17,36 @@
 #include "base/process/process_handle.h"
 #endif  // defined(OS_ANDROID)
 
+namespace {
+
+#if defined(OS_ANDROID)
+bool SaveToFD(const printing::Metafile& metafile,
+              const base::FileDescriptor& fd) {
+  DCHECK_GT(metafile.GetDataSize(), 0U);
+
+  if (fd.fd < 0) {
+    DLOG(ERROR) << "Invalid file descriptor!";
+    return false;
+  }
+  base::File file(fd.fd);
+  bool result = metafile.SaveTo(&file);
+  DLOG_IF(ERROR, !result) << "Failed to save file with fd " << fd.fd;
+
+  if (!fd.auto_close)
+    file.TakePlatformFile();
+  return result;
+}
+#endif  // defined(OS_ANDROID)
+
+}  // namespace
+
 namespace printing {
 
 #if defined(ENABLE_BASIC_PRINTING)
 bool PrintWebViewHelper::PrintPagesNative(blink::WebFrame* frame,
                                           int page_count) {
-  PdfMetafileSkia metafile;
-  if (!metafile.Init())
-    return false;
+  PdfMetafileSkia metafile(PDF_SKIA_DOCUMENT_TYPE);
+  CHECK(metafile.Init());
 
   const PrintMsg_PrintPages_Params& params = *print_pages_params_;
   std::vector<int> printed_pages = GetPrintedPages(params, page_count);
@@ -51,7 +73,7 @@ bool PrintWebViewHelper::PrintPagesNative(blink::WebFrame* frame,
   Send(new PrintHostMsg_AllocateTempFileForPrinting(routing_id(),
                                                     &fd,
                                                     &sequence_number));
-  if (!metafile.SaveToFD(fd))
+  if (!SaveToFD(metafile, fd))
     return false;
 
   // Tell the browser we've finished writing the file.
