@@ -6,9 +6,11 @@
 
 #include <stddef.h>
 
+#include <memory>
+
 #include "base/bind.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/pending_task.h"
 #include "components/scheduler/base/task_queue_impl.h"
 #include "components/scheduler/base/virtual_time_domain.h"
@@ -28,6 +30,7 @@ class MockObserver : public TaskQueueSelector::Observer {
   virtual ~MockObserver() {}
 
   MOCK_METHOD1(OnTaskQueueEnabled, void(internal::TaskQueueImpl*));
+  MOCK_METHOD1(OnTriedToSelectBlockedWorkQueue, void(internal::WorkQueue*));
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockObserver);
@@ -35,8 +38,9 @@ class MockObserver : public TaskQueueSelector::Observer {
 
 class TaskQueueSelectorForTest : public TaskQueueSelector {
  public:
-  using TaskQueueSelector::ChooseOldestWithPriority;
   using TaskQueueSelector::SetImmediateStarvationCountForTest;
+  using TaskQueueSelector::PrioritizingSelector;
+  using TaskQueueSelector::enabled_selector_for_test;
 };
 
 class TaskQueueSelectorTest : public testing::Test {
@@ -45,18 +49,25 @@ class TaskQueueSelectorTest : public testing::Test {
       : test_closure_(base::Bind(&TaskQueueSelectorTest::TestFunction)) {}
   ~TaskQueueSelectorTest() override {}
 
+  TaskQueueSelectorForTest::PrioritizingSelector* enabled_selector() {
+    return selector_.enabled_selector_for_test();
+  }
+
+  WorkQueueSets* delayed_work_queue_sets() {
+    return enabled_selector()->delayed_work_queue_sets();
+  }
+  WorkQueueSets* immediate_work_queue_sets() {
+    return enabled_selector()->immediate_work_queue_sets();
+  }
+
   void PushTasks(const size_t queue_indices[], size_t num_tasks) {
     std::set<size_t> changed_queue_set;
     for (size_t i = 0; i < num_tasks; i++) {
       changed_queue_set.insert(queue_indices[i]);
-      task_queues_[queue_indices[i]]->immediate_work_queue()->PushTaskForTest(
+      task_queues_[queue_indices[i]]->immediate_work_queue()->Push(
           TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(), 0,
                               true, i));
     }
-    for (size_t queue_index : changed_queue_set) {
-      selector_.immediate_task_queue_sets()->OnPushQueue(
-          task_queues_[queue_index]->immediate_work_queue());
-    }
   }
 
   void PushTasksWithEnqueueOrder(const size_t queue_indices[],
@@ -65,14 +76,10 @@ class TaskQueueSelectorTest : public testing::Test {
     std::set<size_t> changed_queue_set;
     for (size_t i = 0; i < num_tasks; i++) {
       changed_queue_set.insert(queue_indices[i]);
-      task_queues_[queue_indices[i]]->immediate_work_queue()->PushTaskForTest(
+      task_queues_[queue_indices[i]]->immediate_work_queue()->Push(
           TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(), 0,
                               true, enqueue_orders[i]));
     }
-    for (size_t queue_index : changed_queue_set) {
-      selector_.immediate_task_queue_sets()->OnPushQueue(
-          task_queues_[queue_index]->immediate_work_queue());
-    }
   }
 
   std::vector<size_t> PopTasks() {
@@ -83,16 +90,26 @@ class TaskQueueSelectorTest : public testing::Test {
           queue_to_index_map_.find(chosen_work_queue->task_queue())->second;
       order.push_back(chosen_queue_index);
       chosen_work_queue->PopTaskForTest();
-      selector_.immediate_task_queue_sets()->OnPopQueue(chosen_work_queue);
+      immediate_work_queue_sets()->OnPopQueue(chosen_work_queue);
     }
     return order;
   }
 
   static void TestFunction() {}
 
+  void EnableQueue(TaskQueueImpl* queue) {
+    queue->SetQueueEnabled(true);
+    selector_.EnableQueue(queue);
+  }
+
+  void DisableQueue(TaskQueueImpl* queue) {
+    queue->SetQueueEnabled(false);
+    selector_.DisableQueue(queue);
+  }
+
  protected:
   void SetUp() final {
-    virtual_time_domain_ = make_scoped_ptr<VirtualTimeDomain>(
+    virtual_time_domain_ = base::WrapUnique<VirtualTimeDomain>(
         new VirtualTimeDomain(nullptr, base::TimeTicks()));
     for (size_t i = 0; i < kTaskQueueCount; i++) {
       scoped_refptr<TaskQueueImpl> task_queue = make_scoped_refptr(
@@ -102,15 +119,33 @@ class TaskQueueSelectorTest : public testing::Test {
       task_queues_.push_back(task_queue);
     }
     for (size_t i = 0; i < kTaskQueueCount; i++) {
-      EXPECT_TRUE(selector_.IsQueueEnabled(task_queues_[i].get())) << i;
+      EXPECT_EQ(TaskQueue::NORMAL_PRIORITY, task_queues_[i]->GetQueuePriority())
+          << i;
       queue_to_index_map_.insert(std::make_pair(task_queues_[i].get(), i));
     }
   }
 
+  void TearDown() final {
+    for (scoped_refptr<TaskQueueImpl>& task_queue : task_queues_) {
+      task_queue->UnregisterTaskQueue();
+      // Note since this test doesn't have a TaskQueueManager we need to
+      // manually remove |task_queue| from the |selector_|.  Normally
+      // UnregisterTaskQueue would do that.
+      selector_.RemoveQueue(task_queue.get());
+    }
+  }
+
+  scoped_refptr<TaskQueueImpl> NewTaskQueueWithBlockReporting() {
+    return make_scoped_refptr(new TaskQueueImpl(
+        nullptr, virtual_time_domain_.get(),
+        TaskQueue::Spec("test queue").SetShouldReportWhenExecutionBlocked(true),
+        "test", "test"));
+  }
+
   const size_t kTaskQueueCount = 5;
   base::Closure test_closure_;
   TaskQueueSelectorForTest selector_;
-  scoped_ptr<VirtualTimeDomain> virtual_time_domain_;
+  std::unique_ptr<VirtualTimeDomain> virtual_time_domain_;
   std::vector<scoped_refptr<TaskQueueImpl>> task_queues_;
   std::map<TaskQueueImpl*, size_t> queue_to_index_map_;
 };
@@ -142,24 +177,23 @@ TEST_F(TaskQueueSelectorTest, TestControlPriority) {
   PushTasks(queue_order, 5);
   selector_.SetQueuePriority(task_queues_[4].get(),
                              TaskQueue::CONTROL_PRIORITY);
-  EXPECT_TRUE(selector_.IsQueueEnabled(task_queues_[4].get()));
+  EXPECT_EQ(TaskQueue::CONTROL_PRIORITY, task_queues_[4]->GetQueuePriority());
   selector_.SetQueuePriority(task_queues_[2].get(), TaskQueue::HIGH_PRIORITY);
-  EXPECT_TRUE(selector_.IsQueueEnabled(task_queues_[2].get()));
+  EXPECT_EQ(TaskQueue::HIGH_PRIORITY, task_queues_[2]->GetQueuePriority());
   EXPECT_THAT(PopTasks(), testing::ElementsAre(4, 2, 0, 1, 3));
 }
 
-TEST_F(TaskQueueSelectorTest, TestObserverWithSetQueuePriority) {
-  selector_.SetQueuePriority(task_queues_[1].get(),
-                             TaskQueue::DISABLED_PRIORITY);
+TEST_F(TaskQueueSelectorTest, TestObserverWithEnabledQueue) {
+  DisableQueue(task_queues_[1].get());
   MockObserver mock_observer;
   selector_.SetTaskQueueSelectorObserver(&mock_observer);
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(1);
-  selector_.SetQueuePriority(task_queues_[1].get(), TaskQueue::NORMAL_PRIORITY);
+  EnableQueue(task_queues_[1].get());
 }
 
 TEST_F(TaskQueueSelectorTest,
-       TestObserverWithSetQueuePriorityAndQueueAlreadyEnabed) {
-  selector_.SetQueuePriority(task_queues_[1].get(), TaskQueue::NORMAL_PRIORITY);
+       TestObserverWithSetQueuePriorityAndQueueAlreadyEnabled) {
+  selector_.SetQueuePriority(task_queues_[1].get(), TaskQueue::HIGH_PRIORITY);
   MockObserver mock_observer;
   selector_.SetTaskQueueSelectorObserver(&mock_observer);
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(0);
@@ -172,22 +206,40 @@ TEST_F(TaskQueueSelectorTest, TestDisableEnable) {
 
   size_t queue_order[] = {0, 1, 2, 3, 4};
   PushTasks(queue_order, 5);
-  selector_.SetQueuePriority(task_queues_[2].get(),
-                             TaskQueue::DISABLED_PRIORITY);
-  EXPECT_FALSE(selector_.IsQueueEnabled(task_queues_[2].get()));
-  selector_.SetQueuePriority(task_queues_[4].get(),
-                             TaskQueue::DISABLED_PRIORITY);
-  EXPECT_FALSE(selector_.IsQueueEnabled(task_queues_[4].get()));
+  DisableQueue(task_queues_[2].get());
+  DisableQueue(task_queues_[4].get());
+  // Disabling a queue should not affect its priority.
+  EXPECT_EQ(TaskQueue::NORMAL_PRIORITY, task_queues_[2]->GetQueuePriority());
+  EXPECT_EQ(TaskQueue::NORMAL_PRIORITY, task_queues_[4]->GetQueuePriority());
   EXPECT_THAT(PopTasks(), testing::ElementsAre(0, 1, 3));
 
   EXPECT_CALL(mock_observer, OnTaskQueueEnabled(_)).Times(2);
+  EnableQueue(task_queues_[2].get());
   selector_.SetQueuePriority(task_queues_[2].get(),
                              TaskQueue::BEST_EFFORT_PRIORITY);
   EXPECT_THAT(PopTasks(), testing::ElementsAre(2));
-  selector_.SetQueuePriority(task_queues_[4].get(), TaskQueue::NORMAL_PRIORITY);
+  EnableQueue(task_queues_[4].get());
   EXPECT_THAT(PopTasks(), testing::ElementsAre(4));
 }
 
+TEST_F(TaskQueueSelectorTest, TestDisableChangePriorityThenEnable) {
+  EXPECT_TRUE(task_queues_[2]->delayed_work_queue()->Empty());
+  EXPECT_TRUE(task_queues_[2]->immediate_work_queue()->Empty());
+
+  DisableQueue(task_queues_[2].get());
+  selector_.SetQueuePriority(task_queues_[2].get(), TaskQueue::HIGH_PRIORITY);
+
+  size_t queue_order[] = {0, 1, 2, 3, 4};
+  PushTasks(queue_order, 5);
+
+  EXPECT_TRUE(task_queues_[2]->delayed_work_queue()->Empty());
+  EXPECT_FALSE(task_queues_[2]->immediate_work_queue()->Empty());
+  EnableQueue(task_queues_[2].get());
+
+  EXPECT_EQ(TaskQueue::HIGH_PRIORITY, task_queues_[2]->GetQueuePriority());
+  EXPECT_THAT(PopTasks(), testing::ElementsAre(2, 0, 1, 3, 4));
+}
+
 TEST_F(TaskQueueSelectorTest, TestEmptyQueues) {
   WorkQueue* chosen_work_queue = nullptr;
   EXPECT_FALSE(selector_.SelectWorkQueueToService(&chosen_work_queue));
@@ -195,9 +247,8 @@ TEST_F(TaskQueueSelectorTest, TestEmptyQueues) {
   // Test only disabled queues.
   size_t queue_order[] = {0};
   PushTasks(queue_order, 1);
-  selector_.SetQueuePriority(task_queues_[0].get(),
-                             TaskQueue::DISABLED_PRIORITY);
-  EXPECT_FALSE(selector_.IsQueueEnabled(task_queues_[0].get()));
+  task_queues_[0]->SetQueueEnabled(false);
+  selector_.DisableQueue(task_queues_[0].get());
   EXPECT_FALSE(selector_.SelectWorkQueueToService(&chosen_work_queue));
 }
 
@@ -249,7 +300,7 @@ TEST_F(TaskQueueSelectorTest, TestBestEffortGetsStarved) {
   PushTasks(queue_order, 2);
   selector_.SetQueuePriority(task_queues_[0].get(),
                              TaskQueue::BEST_EFFORT_PRIORITY);
-  selector_.SetQueuePriority(task_queues_[1].get(), TaskQueue::NORMAL_PRIORITY);
+  EXPECT_EQ(TaskQueue::NORMAL_PRIORITY, task_queues_[1]->GetQueuePriority());
   WorkQueue* chosen_work_queue = nullptr;
   for (int i = 0; i < 100; i++) {
     EXPECT_TRUE(selector_.SelectWorkQueueToService(&chosen_work_queue));
@@ -271,10 +322,6 @@ TEST_F(TaskQueueSelectorTest, TestBestEffortGetsStarved) {
   }
 }
 
-TEST_F(TaskQueueSelectorTest, DisabledPriorityIsPenultimate) {
-  EXPECT_EQ(TaskQueue::QUEUE_PRIORITY_COUNT, TaskQueue::DISABLED_PRIORITY + 1);
-}
-
 TEST_F(TaskQueueSelectorTest, EnabledWorkQueuesEmpty) {
   EXPECT_TRUE(selector_.EnabledWorkQueuesEmpty());
   size_t queue_order[] = {0, 1};
@@ -298,42 +345,99 @@ TEST_F(TaskQueueSelectorTest, EnabledWorkQueuesEmpty_ControlPriority) {
 TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_Empty) {
   WorkQueue* chosen_work_queue = nullptr;
   bool chose_delayed_over_immediate = false;
-  EXPECT_FALSE(selector_.ChooseOldestWithPriority(TaskQueue::NORMAL_PRIORITY,
-                                                  &chose_delayed_over_immediate,
-                                                  &chosen_work_queue));
+  EXPECT_FALSE(enabled_selector()->ChooseOldestWithPriority(
+      TaskQueue::NORMAL_PRIORITY, &chose_delayed_over_immediate,
+      &chosen_work_queue));
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
 TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_OnlyDelayed) {
-  task_queues_[0]->delayed_work_queue()->PushTaskForTest(TaskQueueImpl::Task(
+  task_queues_[0]->delayed_work_queue()->Push(TaskQueueImpl::Task(
       FROM_HERE, test_closure_, base::TimeTicks(), 0, true, 0));
-  selector_.delayed_task_queue_sets()->OnPushQueue(
-      task_queues_[0]->delayed_work_queue());
 
   WorkQueue* chosen_work_queue = nullptr;
   bool chose_delayed_over_immediate = false;
-  EXPECT_TRUE(selector_.ChooseOldestWithPriority(TaskQueue::NORMAL_PRIORITY,
-                                                 &chose_delayed_over_immediate,
-                                                 &chosen_work_queue));
+  EXPECT_TRUE(enabled_selector()->ChooseOldestWithPriority(
+      TaskQueue::NORMAL_PRIORITY, &chose_delayed_over_immediate,
+      &chosen_work_queue));
   EXPECT_EQ(chosen_work_queue, task_queues_[0]->delayed_work_queue());
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
 TEST_F(TaskQueueSelectorTest, ChooseOldestWithPriority_OnlyImmediate) {
-  task_queues_[0]->immediate_work_queue()->PushTaskForTest(TaskQueueImpl::Task(
+  task_queues_[0]->immediate_work_queue()->Push(TaskQueueImpl::Task(
       FROM_HERE, test_closure_, base::TimeTicks(), 0, true, 0));
-  selector_.immediate_task_queue_sets()->OnPushQueue(
-      task_queues_[0]->immediate_work_queue());
 
   WorkQueue* chosen_work_queue = nullptr;
   bool chose_delayed_over_immediate = false;
-  EXPECT_TRUE(selector_.ChooseOldestWithPriority(TaskQueue::NORMAL_PRIORITY,
-                                                 &chose_delayed_over_immediate,
-                                                 &chosen_work_queue));
+  EXPECT_TRUE(enabled_selector()->ChooseOldestWithPriority(
+      TaskQueue::NORMAL_PRIORITY, &chose_delayed_over_immediate,
+      &chosen_work_queue));
   EXPECT_EQ(chosen_work_queue, task_queues_[0]->immediate_work_queue());
   EXPECT_FALSE(chose_delayed_over_immediate);
 }
 
+TEST_F(TaskQueueSelectorTest, TestObserverWithOneBlockedQueue) {
+  TaskQueueSelectorForTest selector;
+  MockObserver mock_observer;
+  selector.SetTaskQueueSelectorObserver(&mock_observer);
+
+  scoped_refptr<TaskQueueImpl> task_queue(NewTaskQueueWithBlockReporting());
+  selector.AddQueue(task_queue.get());
+  task_queue->SetQueueEnabled(false);
+  selector.DisableQueue(task_queue.get());
+
+  task_queue->immediate_work_queue()->PushAndSetEnqueueOrder(
+      TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(), 0, true),
+      0);
+
+  WorkQueue* chosen_work_queue;
+  EXPECT_CALL(mock_observer, OnTriedToSelectBlockedWorkQueue(_)).Times(1);
+  EXPECT_FALSE(selector.SelectWorkQueueToService(&chosen_work_queue));
+
+  task_queue->UnregisterTaskQueue();
+  selector.RemoveQueue(task_queue.get());
+}
+
+TEST_F(TaskQueueSelectorTest, TestObserverWithTwoBlockedQueues) {
+  TaskQueueSelectorForTest selector;
+  MockObserver mock_observer;
+  selector.SetTaskQueueSelectorObserver(&mock_observer);
+
+  scoped_refptr<TaskQueueImpl> task_queue(NewTaskQueueWithBlockReporting());
+  scoped_refptr<TaskQueueImpl> task_queue2(NewTaskQueueWithBlockReporting());
+  selector.AddQueue(task_queue.get());
+  selector.AddQueue(task_queue2.get());
+  task_queue->SetQueueEnabled(false);
+  task_queue2->SetQueueEnabled(false);
+  selector.DisableQueue(task_queue.get());
+  selector.DisableQueue(task_queue2.get());
+  selector.SetQueuePriority(task_queue2.get(), TaskQueue::CONTROL_PRIORITY);
+
+  task_queue->immediate_work_queue()->PushAndSetEnqueueOrder(
+      TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(), 0, true),
+      0);
+  task_queue2->immediate_work_queue()->PushAndSetEnqueueOrder(
+      TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(), 0, true),
+      0);
+
+  // Should still only see one call to OnTriedToSelectBlockedWorkQueue.
+  WorkQueue* chosen_work_queue;
+  EXPECT_CALL(mock_observer, OnTriedToSelectBlockedWorkQueue(_)).Times(1);
+  EXPECT_FALSE(selector.SelectWorkQueueToService(&chosen_work_queue));
+  testing::Mock::VerifyAndClearExpectations(&mock_observer);
+
+  // Removing the second queue and selecting again should result in another
+  // notification.
+  task_queue->UnregisterTaskQueue();
+  selector.RemoveQueue(task_queue.get());
+  EXPECT_CALL(mock_observer, OnTriedToSelectBlockedWorkQueue(_)).Times(1);
+  EXPECT_FALSE(selector.SelectWorkQueueToService(&chosen_work_queue));
+
+  task_queue2->UnregisterTaskQueue();
+  selector.RemoveQueue(task_queue2.get());
+}
+
 struct ChooseOldestWithPriorityTestParam {
   int delayed_task_enqueue_order;
   int immediate_task_enqueue_order;
@@ -358,28 +462,24 @@ class ChooseOldestWithPriorityTest
       public testing::WithParamInterface<ChooseOldestWithPriorityTestParam> {};
 
 TEST_P(ChooseOldestWithPriorityTest, RoundRobinTest) {
-  task_queues_[0]->immediate_work_queue()->PushTaskForTest(
+  task_queues_[0]->immediate_work_queue()->Push(
       TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(),
                           GetParam().immediate_task_enqueue_order, true,
                           GetParam().immediate_task_enqueue_order));
-  selector_.immediate_task_queue_sets()->OnPushQueue(
-      task_queues_[0]->immediate_work_queue());
 
-  task_queues_[0]->delayed_work_queue()->PushTaskForTest(
+  task_queues_[0]->delayed_work_queue()->Push(
       TaskQueueImpl::Task(FROM_HERE, test_closure_, base::TimeTicks(),
                           GetParam().delayed_task_enqueue_order, true,
                           GetParam().delayed_task_enqueue_order));
-  selector_.delayed_task_queue_sets()->OnPushQueue(
-      task_queues_[0]->delayed_work_queue());
 
   selector_.SetImmediateStarvationCountForTest(
       GetParam().immediate_starvation_count);
 
   WorkQueue* chosen_work_queue = nullptr;
   bool chose_delayed_over_immediate = false;
-  EXPECT_TRUE(selector_.ChooseOldestWithPriority(TaskQueue::NORMAL_PRIORITY,
-                                                 &chose_delayed_over_immediate,
-                                                 &chosen_work_queue));
+  EXPECT_TRUE(enabled_selector()->ChooseOldestWithPriority(
+      TaskQueue::NORMAL_PRIORITY, &chose_delayed_over_immediate,
+      &chosen_work_queue));
   EXPECT_EQ(chosen_work_queue->task_queue(), task_queues_[0].get());
   EXPECT_STREQ(chosen_work_queue->name(), GetParam().expected_work_queue_name);
   EXPECT_EQ(chose_delayed_over_immediate,
