@@ -5,19 +5,30 @@
 #ifndef COMPONENTS_SCHEDULER_RENDERER_RENDERER_SCHEDULER_H_
 #define COMPONENTS_SCHEDULER_RENDERER_RENDERER_SCHEDULER_H_
 
+#include <memory>
+
 #include "base/macros.h"
 #include "base/message_loop/message_loop.h"
 #include "components/scheduler/child/child_scheduler.h"
 #include "components/scheduler/child/single_thread_idle_task_runner.h"
 #include "components/scheduler/renderer/render_widget_scheduling_state.h"
 #include "components/scheduler/scheduler_export.h"
+#include "third_party/WebKit/public/platform/WebScheduler.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "v8/include/v8.h"
+
+namespace base {
+namespace trace_event {
+class BlameContext;
+}
+}
 
 namespace cc {
 struct BeginFrameArgs;
 }
 
 namespace blink {
+class WebLocalFrame;
 class WebThread;
 }
 
@@ -27,33 +38,24 @@ class RenderWidgetSchedulingState;
 
 class SCHEDULER_EXPORT RendererScheduler : public ChildScheduler {
  public:
+  class SCHEDULER_EXPORT RAILModeObserver {
+   public:
+    virtual ~RAILModeObserver();
+    virtual void OnRAILModeChanged(v8::RAILMode rail_mode) = 0;
+  };
+
   ~RendererScheduler() override;
-  static scoped_ptr<RendererScheduler> Create();
+  static std::unique_ptr<RendererScheduler> Create();
 
   // Returns the compositor task runner.
-  virtual scoped_refptr<base::SingleThreadTaskRunner>
-  CompositorTaskRunner() = 0;
-
-  // Keep RendererScheduler::UseCaseToString in sync with this enum.
-  enum class UseCase {
-    NONE,
-    COMPOSITOR_GESTURE,
-    MAIN_THREAD_GESTURE,
-    SYNCHRONIZED_GESTURE,  // Both threads in the critical path.
-    TOUCHSTART,
-    LOADING,
-    // Must be the last entry.
-    USE_CASE_COUNT,
-    FIRST_USE_CASE = NONE,
-  };
-  static const char* UseCaseToString(UseCase use_case);
+  virtual scoped_refptr<TaskQueue> CompositorTaskRunner() = 0;
 
   // Creates a WebThread implementation for the renderer main thread.
-  virtual scoped_ptr<blink::WebThread> CreateMainThread() = 0;
+  virtual std::unique_ptr<blink::WebThread> CreateMainThread() = 0;
 
   // Returns the loading task runner.  This queue is intended for tasks related
   // to resource dispatch, foreground HTML parsing, etc...
-  virtual scoped_refptr<base::SingleThreadTaskRunner> LoadingTaskRunner() = 0;
+  virtual scoped_refptr<TaskQueue> LoadingTaskRunner() = 0;
 
   // Returns the timer task runner.  This queue is intended for DOM Timers.
   // TODO(alexclarke): Get rid of this default timer queue.
@@ -68,7 +70,7 @@ class SCHEDULER_EXPORT RendererScheduler : public ChildScheduler {
 
   // Returns a new RenderWidgetSchedulingState.  The signals from this will be
   // used to make scheduling decisions.
-  virtual scoped_ptr<RenderWidgetSchedulingState>
+  virtual std::unique_ptr<RenderWidgetSchedulingState>
   NewRenderWidgetSchedulingState() = 0;
 
   // Called to notify about the start of an extended period where no frames
@@ -120,14 +122,22 @@ class SCHEDULER_EXPORT RendererScheduler : public ChildScheduler {
   // Must be called on the main thread.
   virtual void OnRendererForegrounded() = 0;
 
-  // Tells the scheduler that a navigation task is pending. While any navigation
-  // tasks are pending, the scheduler will ensure that loading tasks are not
-  // blocked even if they are expensive. Must be called on the main thread.
-  virtual void AddPendingNavigation() = 0;
+  // Tells the scheduler that the render process should be suspended. This can
+  // only be done when the renderer is backgrounded. The renderer will be
+  // automatically resumed when foregrounded.
+  virtual void SuspendRenderer() = 0;
+
+  // Tells the scheduler that a navigation task is pending. While any main-frame
+  // navigation tasks are pending, the scheduler will ensure that loading tasks
+  // are not blocked even if they are expensive. Must be called on the main
+  // thread.
+  virtual void AddPendingNavigation(
+      blink::WebScheduler::NavigatingFrameType type) = 0;
 
   // Tells the scheduler that a navigation task is no longer pending.
   // Must be called on the main thread.
-  virtual void RemovePendingNavigation() = 0;
+  virtual void RemovePendingNavigation(
+      blink::WebScheduler::NavigatingFrameType type) = 0;
 
   // Tells the scheduler that a navigation has started.  The scheduler will
   // prioritize loading tasks for a short duration afterwards.
@@ -152,11 +162,17 @@ class SCHEDULER_EXPORT RendererScheduler : public ChildScheduler {
   // received via OnRendererBackgrounded. Defaults to disabled.
   virtual void SetTimerQueueSuspensionWhenBackgroundedEnabled(bool enabled) = 0;
 
-  // Returns a double which is the number of seconds since epoch (Jan 1, 1970).
-  virtual double CurrentTimeSeconds() const = 0;
+  // Sets the default blame context to which top level work should be
+  // attributed in this renderer. |blame_context| must outlive this scheduler.
+  virtual void SetTopLevelBlameContext(
+      base::trace_event::BlameContext* blame_context) = 0;
 
-  // Returns a microsecond resolution platform dependant time source.
-  virtual double MonotonicallyIncreasingTimeSeconds() const = 0;
+  // The renderer scheduler maintains an estimated RAIL mode[1]. This observer
+  // can be used to get notified when the mode changes. The observer will be
+  // called on the main thread and must outlive this class.
+  // [1]
+  // https://developers.google.com/web/tools/chrome-devtools/profile/evaluate-performance/rail
+  virtual void SetRAILModeObserver(RAILModeObserver* observer) = 0;
 
  protected:
   RendererScheduler();
