@@ -6,20 +6,22 @@
 
 #include <stddef.h>
 
+#include <memory>
+
 #include "base/containers/hash_tables.h"
 #include "base/environment.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
 #include "base/metrics/histogram_macros.h"
-#include "base/prefs/pref_registry_simple.h"
-#include "base/prefs/pref_service.h"
 #include "base/process/process_info.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/sys_info.h"
 #include "base/threading/platform_thread.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
+#include "components/prefs/pref_registry_simple.h"
+#include "components/prefs/pref_service.h"
 #include "components/startup_metric_utils/browser/pref_names.h"
 #include "components/version_info/version_info.h"
 
@@ -269,8 +271,8 @@ base::TimeTicks StartupTimeToTimeTicks(const base::Time& time) {
   // bumping the priority reduces the likelihood of a context switch interfering
   // with this computation.
 
-// platform_thread_mac.mm unfortunately doesn't properly support base's
-// thread priority APIs (crbug.com/554651).
+// Enabling this logic on OS X causes a significant performance regression.
+// https://crbug.com/601270
 #if !defined(OS_MACOSX)
   static bool statics_initialized = false;
 
@@ -289,7 +291,6 @@ base::TimeTicks StartupTimeToTimeTicks(const base::Time& time) {
   if (!statics_initialized) {
     base::PlatformThread::SetCurrentThreadPriority(previous_priority);
   }
-
   statics_initialized = true;
 #endif
 
@@ -350,7 +351,7 @@ const char kChromeMainTicksEnvVar[] = "CHROME_MAIN_TICKS";
 
 // Returns the time of main entry recorded from RecordExeMainEntryTime.
 base::TimeTicks ExeMainEntryPointTicks() {
-  scoped_ptr<base::Environment> env(base::Environment::Create());
+  std::unique_ptr<base::Environment> env(base::Environment::Create());
   std::string ticks_string;
   int64_t time_int = 0;
   if (env->GetVar(kChromeMainTicksEnvVar, &ticks_string) &&
@@ -537,7 +538,7 @@ void RecordMainEntryPointTime(const base::Time& time) {
 void RecordExeMainEntryPointTime(const base::Time& time) {
   const std::string exe_load_ticks =
       base::Int64ToString(StartupTimeToTimeTicks(time).ToInternalValue());
-  scoped_ptr<base::Environment> env(base::Environment::Create());
+  std::unique_ptr<base::Environment> env(base::Environment::Create());
   env->SetVar(kChromeMainTicksEnvVar, exe_load_ticks);
 }
 
