@@ -42,8 +42,8 @@ std::string Base64EncodeUrlSafe(const std::vector<uint8_t>& input) {
   return base64url_encoded;
 }
 
-scoped_ptr<base::DictionaryValue> CreatePublicKeyJwkDict() {
-  scoped_ptr<base::DictionaryValue> jwk(new base::DictionaryValue());
+std::unique_ptr<base::DictionaryValue> CreatePublicKeyJwkDict() {
+  std::unique_ptr<base::DictionaryValue> jwk(new base::DictionaryValue());
   jwk->SetString("kty", "RSA");
   jwk->SetString("n",
                  Base64EncodeUrlSafe(HexStringToBytes(kPublicKeyModulusHex)));
@@ -68,7 +68,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPkcs8WithRsaEncryption) {
 }
 
 TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithNoAlg) {
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
 
   blink::WebCryptoKey public_key;
   ASSERT_EQ(
@@ -80,7 +80,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithNoAlg) {
 }
 
 TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMatchingAlg) {
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
   jwk->SetString("alg", "RSA-OAEP");
 
   blink::WebCryptoKey public_key;
@@ -93,7 +93,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMatchingAlg) {
 }
 
 TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMismatchedAlgFails) {
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
   jwk->SetString("alg", "RSA-OAEP-512");
 
   blink::WebCryptoKey public_key;
@@ -106,7 +106,7 @@ TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMismatchedAlgFails) {
 }
 
 TEST_F(WebCryptoRsaOaepTest, ImportPublicJwkWithMismatchedTypeFails) {
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
   jwk->SetString("kty", "oct");
   jwk->SetString("alg", "RSA-OAEP");
 
@@ -131,7 +131,7 @@ TEST_F(WebCryptoRsaOaepTest, ExportPublicJwk) {
     const TestData& test_data = kTestData[i];
     SCOPED_TRACE(test_data.expected_jwk_alg);
 
-    scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+    std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
     jwk->SetString("alg", test_data.expected_jwk_alg);
 
     // Import the key in a known-good format
@@ -154,7 +154,7 @@ TEST_F(WebCryptoRsaOaepTest, ExportPublicJwk) {
 }
 
 TEST_F(WebCryptoRsaOaepTest, EncryptDecryptKnownAnswerTest) {
-  scoped_ptr<base::ListValue> tests;
+  std::unique_ptr<base::ListValue> tests;
   ASSERT_TRUE(ReadJsonTestFileToList("rsa_oaep.json", &tests));
 
   for (size_t test_index = 0; test_index < tests->GetSize(); ++test_index) {
@@ -206,7 +206,7 @@ TEST_F(WebCryptoRsaOaepTest, EncryptWithLargeMessageFails) {
   const blink::WebCryptoAlgorithmId kHash = blink::WebCryptoAlgorithmIdSha1;
   const size_t kHashSize = 20;
 
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
 
   blink::WebCryptoKey public_key;
   ASSERT_EQ(Status::Success(),
@@ -261,7 +261,7 @@ TEST_F(WebCryptoRsaOaepTest, EncryptWithLargeMessageFails) {
 TEST_F(WebCryptoRsaOaepTest, EncryptWithLargeDigestFails) {
   const blink::WebCryptoAlgorithmId kHash = blink::WebCryptoAlgorithmIdSha512;
 
-  scoped_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
+  std::unique_ptr<base::DictionaryValue> jwk(CreatePublicKeyJwkDict());
 
   blink::WebCryptoKey public_key;
   ASSERT_EQ(Status::Success(),
