@@ -7,38 +7,46 @@
 #include <stddef.h>
 #include <stdlib.h>
 #include <string.h>
+
+#include <memory>
 #include <string>
 #include <utility>
 
 #include "base/allocator/allocator_check.h"
 #include "base/allocator/allocator_extension.h"
 #include "base/at_exit.h"
+#include "base/base_switches.h"
 #include "base/command_line.h"
 #include "base/debug/debugger.h"
 #include "base/debug/stack_trace.h"
+#include "base/feature_list.h"
 #include "base/files/file_path.h"
 #include "base/i18n/icu_util.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
+#include "base/metrics/field_trial.h"
+#include "base/metrics/histogram_base.h"
 #include "base/metrics/statistics_recorder.h"
 #include "base/path_service.h"
 #include "base/process/launch.h"
 #include "base/process/memory.h"
 #include "base/process/process_handle.h"
-#include "base/profiler/alternate_timer.h"
 #include "base/profiler/scoped_tracker.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
-#include "components/tracing/trace_config_file.h"
-#include "components/tracing/trace_to_console.h"
-#include "components/tracing/tracing_switches.h"
+#include "components/tracing/browser/trace_config_file.h"
+#include "components/tracing/common/trace_to_console.h"
+#include "components/tracing/common/tracing_switches.h"
+#include "content/app/mojo/mojo_init.h"
 #include "content/browser/browser_main.h"
+#include "content/browser/gpu/gpu_process_host.h"
+#include "content/browser/renderer_host/render_process_host_impl.h"
+#include "content/browser/utility_process_host_impl.h"
 #include "content/common/set_process_title.h"
 #include "content/common/url_schemes.h"
 #include "content/gpu/in_process_gpu_thread.h"
@@ -51,6 +59,9 @@
 #include "content/public/common/content_switches.h"
 #include "content/public/common/main_function_params.h"
 #include "content/public/common/sandbox_init.h"
+#include "content/public/gpu/content_gpu_client.h"
+#include "content/public/renderer/content_renderer_client.h"
+#include "content/public/utility/content_utility_client.h"
 #include "content/renderer/in_process_renderer_thread.h"
 #include "content/utility/in_process_utility_thread.h"
 #include "ipc/ipc_descriptors.h"
@@ -64,20 +75,6 @@
 #include "gin/v8_initializer.h"
 #endif
 
-#if defined(USE_TCMALLOC)
-#include "third_party/tcmalloc/chromium/src/gperftools/malloc_extension.h"
-#endif
-
-#if !defined(OS_IOS)
-#include "content/app/mojo/mojo_init.h"
-#include "content/browser/gpu/gpu_process_host.h"
-#include "content/browser/renderer_host/render_process_host_impl.h"
-#include "content/browser/utility_process_host_impl.h"
-#include "content/public/plugin/content_plugin_client.h"
-#include "content/public/renderer/content_renderer_client.h"
-#include "content/public/utility/content_utility_client.h"
-#endif
-
 #if defined(OS_WIN)
 #include <malloc.h>
 #include <cstring>
@@ -87,15 +84,13 @@
 #if !defined(TOOLKIT_QT)
 #include "ui/base/win/atl_module.h"
 #endif
-#include "ui/gfx/win/dpi.h"
+#include "ui/display/win/dpi.h"
 #elif defined(OS_MACOSX)
 #include "base/mac/scoped_nsautorelease_pool.h"
-#if !defined(OS_IOS)
 #include "base/power_monitor/power_monitor_device_source.h"
 #include "content/app/mac/mac_init.h"
 #include "content/browser/mach_broker_mac.h"
 #include "content/common/sandbox_init_mac.h"
-#endif  // !OS_IOS
 #endif  // OS_WIN
 
 #if defined(OS_POSIX)
@@ -117,12 +112,6 @@
 #include "crypto/nss_util.h"
 #endif
 
-#if !defined(OS_MACOSX) && defined(USE_TCMALLOC)
-extern "C" {
-int tc_set_new_mode(int mode);
-}
-#endif
-
 namespace content {
 extern int GpuMain(const content::MainFunctionParams&);
 #if defined(ENABLE_PLUGINS)
@@ -134,29 +123,70 @@ extern int PpapiBrokerMain(const MainFunctionParams&);
 #endif
 extern int RendererMain(const content::MainFunctionParams&);
 extern int UtilityMain(const MainFunctionParams&);
+#if defined(OS_ANDROID)
+extern int DownloadMain(const MainFunctionParams&);
+#endif
 }  // namespace content
 
 namespace content {
 
+namespace {
+
+// This sets up two singletons responsible for managing field trials. The
+// |field_trial_list| singleton lives on the stack and must outlive the Run()
+// method of the process.
+void InitializeFieldTrialAndFeatureList(
+    std::unique_ptr<base::FieldTrialList>* field_trial_list) {
+  const base::CommandLine& command_line =
+      *base::CommandLine::ForCurrentProcess();
+
+  // Initialize statistical testing infrastructure.  We set the entropy
+  // provider to nullptr to disallow non-browser processes from creating
+  // their own one-time randomized trials; they should be created in the
+  // browser process.
+  field_trial_list->reset(new base::FieldTrialList(nullptr));
+
+  // Ensure any field trials in browser are reflected into the child
+  // process.
+  if (command_line.HasSwitch(switches::kForceFieldTrials)) {
+    bool result = base::FieldTrialList::CreateTrialsFromString(
+        command_line.GetSwitchValueASCII(switches::kForceFieldTrials),
+        std::set<std::string>());
+    DCHECK(result);
+  }
+
+  std::unique_ptr<base::FeatureList> feature_list(new base::FeatureList);
+  feature_list->InitializeFromCommandLine(
+      command_line.GetSwitchValueASCII(switches::kEnableFeatures),
+      command_line.GetSwitchValueASCII(switches::kDisableFeatures));
+  base::FeatureList::SetInstance(std::move(feature_list));
+}
+
+}  // namespace
+
 #if !defined(CHROME_MULTIPLE_DLL_CHILD)
 base::LazyInstance<ContentBrowserClient>
     g_empty_content_browser_client = LAZY_INSTANCE_INITIALIZER;
 #endif  //  !CHROME_MULTIPLE_DLL_CHILD
 
-#if !defined(OS_IOS) && !defined(CHROME_MULTIPLE_DLL_BROWSER)
-base::LazyInstance<ContentPluginClient>
-    g_empty_content_plugin_client = LAZY_INSTANCE_INITIALIZER;
+#if !defined(CHROME_MULTIPLE_DLL_BROWSER)
+base::LazyInstance<ContentGpuClient>
+    g_empty_content_gpu_client = LAZY_INSTANCE_INITIALIZER;
 base::LazyInstance<ContentRendererClient>
     g_empty_content_renderer_client = LAZY_INSTANCE_INITIALIZER;
 base::LazyInstance<ContentUtilityClient>
     g_empty_content_utility_client = LAZY_INSTANCE_INITIALIZER;
-#endif  // !OS_IOS && !CHROME_MULTIPLE_DLL_BROWSER
+#endif  // !CHROME_MULTIPLE_DLL_BROWSER
 
-#if defined(OS_WIN)
+#if defined(V8_USE_EXTERNAL_STARTUP_DATA) && defined(OS_ANDROID)
+#if defined __LP64__
+#define kV8SnapshotDataDescriptor kV8SnapshotDataDescriptor64
+#else
+#define kV8SnapshotDataDescriptor kV8SnapshotDataDescriptor32
+#endif
+#endif
 
-#endif  // defined(OS_WIN)
-
-#if defined(OS_POSIX) && !defined(OS_IOS)
+#if defined(OS_POSIX)
 
 // Setup signal-handling state: resanitize most signals, ignore SIGPIPE.
 void SetupSignalHandlers() {
@@ -180,7 +210,7 @@ void SetupSignalHandlers() {
   CHECK_NE(SIG_ERR, signal(SIGPIPE, SIG_IGN));
 }
 
-#endif  // OS_POSIX && !OS_IOS
+#endif  // OS_POSIX
 
 void CommonSubprocessInit(const std::string& process_type) {
 #if defined(OS_WIN)
@@ -206,7 +236,10 @@ void CommonSubprocessInit(const std::string& process_type) {
 #if !defined(OFFICIAL_BUILD)
   // Print stack traces to stderr when crashes occur. This opens up security
   // holes so it should never be enabled for official builds.
-  base::debug::EnableInProcessStackDumping();
+  if (!base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kDisableInProcessStackTraces)) {
+    base::debug::EnableInProcessStackDumping();
+  }
 #if defined(OS_WIN)
   base::RouteStdioToConsole(false);
   LoadLibraryA("dbghelp.dll");
@@ -228,17 +261,19 @@ class ContentClientInitializer {
     }
 #endif  // !CHROME_MULTIPLE_DLL_CHILD
 
-#if !defined(OS_IOS) && !defined(CHROME_MULTIPLE_DLL_BROWSER)
-    if (process_type == switches::kPluginProcess ||
-        process_type == switches::kPpapiPluginProcess) {
+#if !defined(CHROME_MULTIPLE_DLL_BROWSER)
+    base::CommandLine* cmd = base::CommandLine::ForCurrentProcess();
+    if (process_type == switches::kGpuProcess ||
+        cmd->HasSwitch(switches::kSingleProcess) ||
+        (process_type.empty() && cmd->HasSwitch(switches::kInProcessGPU))) {
       if (delegate)
-        content_client->plugin_ = delegate->CreateContentPluginClient();
-      if (!content_client->plugin_)
-        content_client->plugin_ = &g_empty_content_plugin_client.Get();
-      // Single process not supported in split dll mode.
-    } else if (process_type == switches::kRendererProcess ||
-               base::CommandLine::ForCurrentProcess()->HasSwitch(
-                   switches::kSingleProcess)) {
+        content_client->gpu_ = delegate->CreateContentGpuClient();
+      if (!content_client->gpu_)
+        content_client->gpu_ = &g_empty_content_gpu_client.Get();
+    }
+
+    if (process_type == switches::kRendererProcess ||
+        cmd->HasSwitch(switches::kSingleProcess)) {
       if (delegate)
         content_client->renderer_ = delegate->CreateContentRendererClient();
       if (!content_client->renderer_)
@@ -246,15 +281,14 @@ class ContentClientInitializer {
     }
 
     if (process_type == switches::kUtilityProcess ||
-        base::CommandLine::ForCurrentProcess()->HasSwitch(
-            switches::kSingleProcess)) {
+        cmd->HasSwitch(switches::kSingleProcess)) {
       if (delegate)
         content_client->utility_ = delegate->CreateContentUtilityClient();
       // TODO(scottmg): http://crbug.com/237249 Should be in _child.
       if (!content_client->utility_)
         content_client->utility_ = &g_empty_content_utility_client.Get();
     }
-#endif  // !OS_IOS && !CHROME_MULTIPLE_DLL_BROWSER
+#endif  // !CHROME_MULTIPLE_DLL_BROWSER
   }
 };
 
@@ -303,6 +337,9 @@ int RunZygote(const MainFunctionParams& main_function_params,
   MainFunctionParams main_params(command_line);
   main_params.zygote_child = true;
 
+  std::unique_ptr<base::FieldTrialList> field_trial_list;
+  InitializeFieldTrialAndFeatureList(&field_trial_list);
+
   for (size_t i = 0; i < arraysize(kMainFunctions); ++i) {
     if (process_type == kMainFunctions[i].name)
       return kMainFunctions[i].function(main_params);
@@ -316,7 +353,6 @@ int RunZygote(const MainFunctionParams& main_function_params,
 }
 #endif  // defined(OS_POSIX) && !defined(OS_MACOSX) && !defined(OS_ANDROID)
 
-#if !defined(OS_IOS)
 static void RegisterMainThreadFactories() {
 #if !defined(CHROME_MULTIPLE_DLL_BROWSER) && !defined(CHROME_MULTIPLE_DLL_CHILD)
   UtilityProcessHostImpl::RegisterUtilityMainThreadFactory(
@@ -351,15 +387,15 @@ int RunNamedProcessTypeMain(
 #endif
 #if !defined(CHROME_MULTIPLE_DLL_BROWSER)
 #if defined(ENABLE_PLUGINS)
-#if !defined(OS_LINUX)
-    { switches::kPluginProcess,      PluginMain },
-#endif
     { switches::kPpapiPluginProcess, PpapiPluginMain },
     { switches::kPpapiBrokerProcess, PpapiBrokerMain },
 #endif  // ENABLE_PLUGINS
     { switches::kUtilityProcess,     UtilityMain },
     { switches::kRendererProcess,    RendererMain },
     { switches::kGpuProcess,         GpuMain },
+#if defined(OS_ANDROID)
+    { switches::kDownloadProcess,    DownloadMain},
+#endif
 #endif  // !CHROME_MULTIPLE_DLL_BROWSER
   };
 
@@ -399,7 +435,6 @@ int RunNamedProcessTypeMain(
   NOTREACHED() << "Unknown process type: " << process_type;
   return 1;
 }
-#endif  // !OS_IOS
 
 class ContentMainRunnerImpl : public ContentMainRunner {
  public:
@@ -419,16 +454,6 @@ class ContentMainRunnerImpl : public ContentMainRunner {
       Shutdown();
   }
 
-#if defined(USE_TCMALLOC)
-  static bool GetNumericPropertyThunk(const char* name, size_t* value) {
-    return MallocExtension::instance()->GetNumericProperty(name, value);
-  }
-
-  static void ReleaseFreeMemoryThunk() {
-    MallocExtension::instance()->ReleaseFreeMemory();
-  }
-#endif
-
   int Initialize(const ContentMainParams& params) override {
     ui_task_ = params.ui_task;
 
@@ -449,35 +474,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     TRACE_EVENT0("startup,benchmark", "ContentMainRunnerImpl::Initialize");
 #endif  // OS_ANDROID
 
-    // NOTE(willchan): One might ask why these TCMalloc-related calls are done
-    // here rather than in process_util_linux.cc with the definition of
-    // EnableTerminationOnOutOfMemory().  That's because base shouldn't have a
-    // dependency on TCMalloc.  Really, we ought to have our allocator shim code
-    // implement this EnableTerminationOnOutOfMemory() function.  Whateverz.
-    // This works for now.
-#if !defined(OS_MACOSX) && defined(USE_TCMALLOC)
-    // For tcmalloc, we need to tell it to behave like new.
-    tc_set_new_mode(1);
-
-    // On windows, we've already set these thunks up in _heap_init()
-    base::allocator::SetGetNumericPropertyFunction(GetNumericPropertyThunk);
-    base::allocator::SetReleaseFreeMemoryFunction(ReleaseFreeMemoryThunk);
-
-    // Provide optional hook for monitoring allocation quantities on a
-    // per-thread basis.  Only set the hook if the environment indicates this
-    // needs to be enabled.
-    const char* profiling = getenv(tracked_objects::kAlternateProfilerTime);
-    if (profiling &&
-        (atoi(profiling) == tracked_objects::TIME_SOURCE_TYPE_TCMALLOC)) {
-      tracked_objects::SetAlternateTimeSource(
-          MallocExtension::GetBytesAllocatedOnCurrentThread,
-          tracked_objects::TIME_SOURCE_TYPE_TCMALLOC);
-    }
-#endif  // !OS_MACOSX && USE_TCMALLOC
-
-#if !defined(OS_IOS)
     base::GlobalDescriptors* g_fds = base::GlobalDescriptors::GetInstance();
-#endif
 
     // On Android,
     // - setlocale() is not supported.
@@ -485,7 +482,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     //   stack trace when crashing.
     // - The ipc_fd is passed through the Java service.
     // Thus, these are all disabled.
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
+#if !defined(OS_ANDROID)
     // Set C library locale to make sure CommandLine can parse argument values
     // in correct encoding.
     setlocale(LC_ALL, "");
@@ -499,7 +496,9 @@ class ContentMainRunnerImpl : public ContentMainRunner {
 
     g_fds->Set(kPrimaryIPCChannel,
                kPrimaryIPCChannel + base::GlobalDescriptors::kBaseDescriptor);
-#endif  // !OS_ANDROID && !OS_IOS
+    g_fds->Set(kMojoIPCChannel,
+               kMojoIPCChannel + base::GlobalDescriptors::kBaseDescriptor);
+#endif  // !OS_ANDROID
 
 #if defined(OS_LINUX) || defined(OS_OPENBSD)
     g_fds->Set(kCrashDumpSignal,
@@ -514,22 +513,18 @@ class ContentMainRunnerImpl : public ContentMainRunner {
 
     // The exit manager is in charge of calling the dtors of singleton objects.
     // On Android, AtExitManager is set up when library is loaded.
-    // On iOS, it's set up in main(), which can't call directly through to here.
     // A consequence of this is that you can't use the ctor/dtor-based
     // TRACE_EVENT methods on Linux or iOS builds till after we set this up.
-#if !defined(OS_ANDROID) && !defined(OS_IOS)
+#if !defined(OS_ANDROID)
     if (!ui_task_) {
       // When running browser tests, don't create a second AtExitManager as that
       // interfers with shutdown when objects created before ContentMain is
       // called are destructed when it returns.
       exit_manager_.reset(new base::AtExitManager);
     }
-#endif  // !OS_ANDROID && !OS_IOS
+#endif  // !OS_ANDROID
 
-    // Don't create this loop on iOS, since the outer loop is already handled
-    // and a loop that's destroyed in shutdown interleaves badly with the event
-    // loop pool on iOS.
-#if defined(OS_MACOSX) && !defined(OS_IOS)
+#if defined(OS_MACOSX)
 #if !defined(TOOLKIT_QT)
     // We need this pool for all the objects created before we get to the
     // event loop, but we don't want to leave them hanging around until the
@@ -565,9 +560,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     if (delegate_ && delegate_->ShouldEnableProfilerRecording())
       tracked_objects::ScopedTracker::Enable();
 
-#if !defined(OS_IOS)
     SetProcessTitleFromCommandLine(argv);
-#endif
 #endif  // !OS_ANDROID
 
     int exit_code = 0;
@@ -581,10 +574,8 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     std::string process_type =
         command_line.GetSwitchValueASCII(switches::kProcessType);
 
-#if !defined(OS_IOS)
     // Initialize mojo here so that services can be registered.
     InitializeMojo();
-#endif
 
 #if defined(OS_WIN)
     if (command_line.HasSwitch(switches::kDeviceScaleFactor)) {
@@ -592,7 +583,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
           switches::kDeviceScaleFactor);
       double scale_factor = 0;
       if (base::StringToDouble(scale_factor_string, &scale_factor))
-        gfx::SetDefaultDeviceScaleFactor(scale_factor);
+        display::win::SetDefaultDeviceScaleFactor(scale_factor);
     }
 #endif
 
@@ -650,7 +641,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     TRACE_EVENT0("startup,benchmark", "ContentMainRunnerImpl::Initialize");
 #endif  // !OS_ANDROID
 
-#if defined(OS_MACOSX) && !defined(OS_IOS)
+#if defined(OS_MACOSX)
     // We need to allocate the IO Ports before the Sandbox is initialized or
     // the first instance of PowerMonitor is created.
     // It's important not to allocate the ports for processes which don't
@@ -763,7 +754,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
 
 #if defined(OS_WIN)
     CHECK(InitializeSandbox(params.sandbox_info));
-#elif defined(OS_MACOSX) && !defined(OS_IOS)
+#elif defined(OS_MACOSX)
     if (process_type == switches::kRendererProcess ||
         process_type == switches::kPpapiPluginProcess ||
         (delegate_ && delegate_->DelaySandboxInitialization(process_type))) {
@@ -789,6 +780,14 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     std::string process_type =
         command_line.GetSwitchValueASCII(switches::kProcessType);
 
+    // Run this logic on all child processes. Zygotes will run this at a later
+    // point in time when the command line has been updated.
+    std::unique_ptr<base::FieldTrialList> field_trial_list;
+    if (!process_type.empty() && process_type != switches::kZygoteProcess)
+      InitializeFieldTrialAndFeatureList(&field_trial_list);
+
+    base::HistogramBase::EnableActivityReportHistogram(process_type);
+
     MainFunctionParams main_params(command_line);
     main_params.ui_task = ui_task_;
 #if defined(OS_WIN)
@@ -797,11 +796,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
     main_params.autorelease_pool = autorelease_pool_.get();
 #endif
 
-#if !defined(OS_IOS)
     return RunNamedProcessTypeMain(process_type, main_params, delegate_);
-#else
-    return 1;
-#endif
   }
 
   void Shutdown() override {
@@ -823,7 +818,7 @@ class ContentMainRunnerImpl : public ContentMainRunner {
 #endif  // _CRTDBG_MAP_ALLOC
 #endif  // OS_WIN
 
-#if defined(OS_MACOSX) && !defined(OS_IOS) && !defined(TOOLKIT_QT)
+#if defined(OS_MACOSX) && !defined(TOOLKIT_QT)
     autorelease_pool_.reset(NULL);
 #endif
 
@@ -849,11 +844,11 @@ class ContentMainRunnerImpl : public ContentMainRunner {
   // The delegate will outlive this object.
   ContentMainDelegate* delegate_;
 
-  scoped_ptr<base::AtExitManager> exit_manager_;
+  std::unique_ptr<base::AtExitManager> exit_manager_;
 #if defined(OS_WIN)
   sandbox::SandboxInterfaceInfo sandbox_info_;
 #elif defined(OS_MACOSX) && !defined(TOOLKIT_QT)
-  scoped_ptr<base::mac::ScopedNSAutoreleasePool> autorelease_pool_;
+  std::unique_ptr<base::mac::ScopedNSAutoreleasePool> autorelease_pool_;
 #endif
 
   base::Closure* ui_task_;
