@@ -4,7 +4,7 @@
 
 #include "content/app/mac/mac_init.h"
 
-#include <Cocoa/Cocoa.h>
+#import <Cocoa/Cocoa.h>
 
 #include "base/mac/mac_util.h"
 
@@ -12,7 +12,16 @@ namespace content {
 
 void InitializeMac() {
   [[NSUserDefaults standardUserDefaults] registerDefaults:@{
+      // Exceptions routed to -[NSApplication reportException:] should crash
+      // immediately, as opposed being swallowed or presenting UI that gives the
+      // user a choice in the matter.
+      @"NSApplicationCrashOnExceptions": @YES,
+
+      // Prevent Cocoa from turning command-line arguments into -[NSApplication
+      // application:openFile:], because they are handled directly. @"NO" looks
+      // like a mistake, but the value really is supposed to be a string.
       @"NSTreatUnknownArgumentsAsOpen": @"NO",
+
       // CoreAnimation has poor performance and CoreAnimation and
       // non-CoreAnimation exhibit window flickering when layers are not hosted
       // in the window server, which is the default when not not using the
