@@ -4,12 +4,13 @@
 
 #include "content/app/mojo/mojo_init.h"
 
+#include <memory>
+
+#include "base/command_line.h"
 #include "base/lazy_instance.h"
-#include "base/memory/scoped_ptr.h"
+#include "content/public/common/content_switches.h"
 #include "ipc/ipc_channel.h"
-#include "third_party/mojo/src/mojo/edk/embedder/configuration.h"
-#include "third_party/mojo/src/mojo/edk/embedder/embedder.h"
-#include "third_party/mojo/src/mojo/edk/embedder/simple_platform_support.h"
+#include "mojo/edk/embedder/embedder.h"
 
 namespace content {
 
@@ -18,10 +19,8 @@ namespace {
 class MojoInitializer {
  public:
   MojoInitializer() {
-    mojo::embedder::GetConfiguration()->max_message_num_bytes =
-        IPC::Channel::kMaximumMessageSize;
-    mojo::embedder::Init(scoped_ptr<mojo::embedder::PlatformSupport>(
-        new mojo::embedder::SimplePlatformSupport()));
+    mojo::edk::SetMaxMessageSize(IPC::Channel::kMaximumMessageSize);
+    mojo::edk::Init();
   }
 };
 
