@@ -2,6 +2,12 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "content/browser/appcache/appcache_url_request_job.h"
+
+#include <stdint.h>
+#include <string.h>
+
+#include <memory>
 #include <stack>
 #include <utility>
 
@@ -9,13 +15,16 @@
 #include "base/bind_helpers.h"
 #include "base/callback.h"
 #include "base/compiler_specific.h"
+#include "base/location.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/macros.h"
+#include "base/memory/weak_ptr.h"
 #include "base/pickle.h"
+#include "base/single_thread_task_runner.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/threading/thread.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "content/browser/appcache/appcache_response.h"
-#include "content/browser/appcache/appcache_url_request_job.h"
 #include "content/browser/appcache/mock_appcache_service.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
@@ -41,20 +50,32 @@ const char kHttpBasicBody[] = "Hello";
 const int kNumBlocks = 4;
 const int kBlockSize = 1024;
 
+// Used as an AppCacheURLRequestJob::OnPrepareToRestartCallback for requests
+// that aren't expected to be restarted.
+void ExpectNotRestarted() {
+  ADD_FAILURE() << "Request unexpectedly restarted";
+}
+
+// Used as an AppCacheURLRequestJob::OnPrepareToRestartCallback for requests
+// that are expected to be restarted. Allows tests to verify it was called by
+// setting |*value| to true when invoked.
+void SetIfCalled(bool* value) {
+  // Expected to be called only once.
+  EXPECT_FALSE(*value);
+  *value = true;
+}
+
 class MockURLRequestJobFactory : public net::URLRequestJobFactory {
  public:
-  MockURLRequestJobFactory() : job_(NULL) {
-  }
+  MockURLRequestJobFactory() {}
 
   ~MockURLRequestJobFactory() override { DCHECK(!job_); }
 
-  void SetJob(net::URLRequestJob* job) {
-    job_ = job;
+  void SetJob(std::unique_ptr<net::URLRequestJob> job) {
+    job_ = std::move(job);
   }
 
-  bool has_job() const {
-    return job_ != NULL;
-  }
+  bool has_job() const { return job_.get() != nullptr; }
 
   // net::URLRequestJobFactory implementation.
   net::URLRequestJob* MaybeCreateJobWithProtocolHandler(
@@ -62,9 +83,7 @@ class MockURLRequestJobFactory : public net::URLRequestJobFactory {
       net::URLRequest* request,
       net::NetworkDelegate* network_delegate) const override {
     if (job_) {
-      net::URLRequestJob* temp = job_;
-      job_ = NULL;
-      return temp;
+      return job_.release();
     } else {
       return new net::URLRequestErrorJob(request,
                                          network_delegate,
@@ -98,9 +117,12 @@ class MockURLRequestJobFactory : public net::URLRequestJobFactory {
   }
 
  private:
-  mutable net::URLRequestJob* job_;
+  // This is mutable because MaybeCreateJobWithProtocolHandler is const.
+  mutable std::unique_ptr<net::URLRequestJob> job_;
 };
 
+}  // namespace
+
 class AppCacheURLRequestJobTest : public testing::Test {
  public:
 
@@ -114,14 +136,14 @@ class AppCacheURLRequestJobTest : public testing::Test {
     }
 
     void OnResponseInfoLoaded(AppCacheResponseInfo* info,
-                              int64 response_id) override {
+                              int64_t response_id) override {
       loaded_info_ = info;
       loaded_info_id_ = response_id;
       test_->ScheduleNextTask();
     }
 
     scoped_refptr<AppCacheResponseInfo> loaded_info_;
-    int64 loaded_info_id_;
+    int64_t loaded_info_id_;
     AppCacheURLRequestJobTest* test_;
   };
 
@@ -216,8 +238,10 @@ class AppCacheURLRequestJobTest : public testing::Test {
 
   template <class Method>
   void RunTestOnIOThread(Method method) {
-    test_finished_event_ .reset(new base::WaitableEvent(false, false));
-    io_thread_->message_loop()->PostTask(
+    test_finished_event_.reset(new base::WaitableEvent(
+        base::WaitableEvent::ResetPolicy::AUTOMATIC,
+        base::WaitableEvent::InitialState::NOT_SIGNALED));
+    io_thread_->task_runner()->PostTask(
         FROM_HERE, base::Bind(&AppCacheURLRequestJobTest::MethodWrapper<Method>,
                               base::Unretained(this), method));
     test_finished_event_->Wait();
@@ -235,6 +259,8 @@ class AppCacheURLRequestJobTest : public testing::Test {
     reader_deletion_count_down_ = 0;
     writer_deletion_count_down_ = 0;
 
+    restart_callback_invoked_ = false;
+
     url_request_delegate_.reset(new MockURLRequestDelegate(this));
     job_factory_.reset(new MockURLRequestJobFactory());
     empty_context_.reset(new net::URLRequestContext());
@@ -267,10 +293,9 @@ class AppCacheURLRequestJobTest : public testing::Test {
     // We unwind the stack prior to finishing up to let stack
     // based objects get deleted.
     DCHECK(base::MessageLoop::current() == io_thread_->message_loop());
-    base::MessageLoop::current()->PostTask(
-        FROM_HERE,
-        base::Bind(&AppCacheURLRequestJobTest::TestFinishedUnwound,
-                   base::Unretained(this)));
+    base::ThreadTaskRunnerHandle::Get()->PostTask(
+        FROM_HERE, base::Bind(&AppCacheURLRequestJobTest::TestFinishedUnwound,
+                              base::Unretained(this)));
   }
 
   void TestFinishedUnwound() {
@@ -298,7 +323,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
     if (immediate)
       task.Run();
     else
-      base::MessageLoop::current()->PostTask(FROM_HERE, task);
+      base::ThreadTaskRunnerHandle::Get()->PostTask(FROM_HERE, task);
   }
 
   // Wrappers to call AppCacheResponseReader/Writer Read and Write methods
@@ -387,21 +412,22 @@ class AppCacheURLRequestJobTest : public testing::Test {
   }
 
   int GetHttpResponseInfoSize(const net::HttpResponseInfo* info) {
-    Pickle pickle;
+    base::Pickle pickle;
     return PickleHttpResonseInfo(&pickle, info);
   }
 
   bool CompareHttpResponseInfos(const net::HttpResponseInfo* info1,
                                 const net::HttpResponseInfo* info2) {
-    Pickle pickle1;
-    Pickle pickle2;
+    base::Pickle pickle1;
+    base::Pickle pickle2;
     PickleHttpResonseInfo(&pickle1, info1);
     PickleHttpResonseInfo(&pickle2, info2);
     return (pickle1.size() == pickle2.size()) &&
            (0 == memcmp(pickle1.data(), pickle2.data(), pickle1.size()));
   }
 
-  int PickleHttpResonseInfo(Pickle* pickle, const net::HttpResponseInfo* info) {
+  int PickleHttpResonseInfo(base::Pickle* pickle,
+                            const net::HttpResponseInfo* info) {
     const bool kSkipTransientHeaders = true;
     const bool kTruncated = false;
     info->Persist(pickle, kSkipTransientHeaders, kTruncated);
@@ -429,13 +455,14 @@ class AppCacheURLRequestJobTest : public testing::Test {
   // Basic -------------------------------------------------------------------
   void Basic() {
     AppCacheStorage* storage = service_->storage();
-    scoped_ptr<net::URLRequest> request(empty_context_->CreateRequest(
-        GURL("http://blah/"), net::DEFAULT_PRIORITY, NULL, NULL));
-    scoped_refptr<AppCacheURLRequestJob> job;
+    request_ = empty_context_->CreateRequest(GURL("http://blah/"),
+                                             net::DEFAULT_PRIORITY, nullptr);
 
     // Create an instance and see that it looks as expected.
 
-    job = new AppCacheURLRequestJob(request.get(), NULL, storage, NULL, false);
+    std::unique_ptr<AppCacheURLRequestJob> job(
+        new AppCacheURLRequestJob(request_.get(), nullptr, storage, nullptr,
+                                  false, base::Bind(&ExpectNotRestarted)));
     EXPECT_TRUE(job->is_waiting());
     EXPECT_FALSE(job->is_delivering_appcache_response());
     EXPECT_FALSE(job->is_delivering_network_response());
@@ -452,36 +479,38 @@ class AppCacheURLRequestJobTest : public testing::Test {
   // DeliveryOrders -----------------------------------------------------
   void DeliveryOrders() {
     AppCacheStorage* storage = service_->storage();
-    scoped_ptr<net::URLRequest> request(empty_context_->CreateRequest(
-        GURL("http://blah/"), net::DEFAULT_PRIORITY, NULL, NULL));
-    scoped_refptr<AppCacheURLRequestJob> job;
+    std::unique_ptr<net::URLRequest> request(empty_context_->CreateRequest(
+        GURL("http://blah/"), net::DEFAULT_PRIORITY, nullptr));
 
     // Create an instance, give it a delivery order and see that
     // it looks as expected.
 
-    job = new AppCacheURLRequestJob(request.get(), NULL, storage, NULL, false);
+    std::unique_ptr<AppCacheURLRequestJob> job(
+        new AppCacheURLRequestJob(request.get(), nullptr, storage, nullptr,
+                                  false, base::Bind(&ExpectNotRestarted)));
     job->DeliverErrorResponse();
     EXPECT_TRUE(job->is_delivering_error_response());
     EXPECT_FALSE(job->has_been_started());
 
-    job = new AppCacheURLRequestJob(request.get(), NULL, storage, NULL, false);
+    job.reset(new AppCacheURLRequestJob(request.get(), nullptr, storage,
+                                        nullptr, false,
+                                        base::Bind(&ExpectNotRestarted)));
     job->DeliverNetworkResponse();
     EXPECT_TRUE(job->is_delivering_network_response());
     EXPECT_FALSE(job->has_been_started());
 
-    job = new AppCacheURLRequestJob(request.get(), NULL, storage, NULL, false);
+    job.reset(new AppCacheURLRequestJob(request.get(), nullptr, storage,
+                                        nullptr, false,
+                                        base::Bind(&ExpectNotRestarted)));
     const GURL kManifestUrl("http://blah/");
-    const int64 kCacheId(1);
-    const int64 kGroupId(1);
+    const int64_t kCacheId(1);
     const AppCacheEntry kEntry(AppCacheEntry::EXPLICIT, 1);
-    job->DeliverAppCachedResponse(kManifestUrl, kCacheId, kGroupId,
-                                  kEntry, false);
+    job->DeliverAppCachedResponse(kManifestUrl, kCacheId, kEntry, false);
     EXPECT_FALSE(job->is_waiting());
     EXPECT_TRUE(job->is_delivering_appcache_response());
     EXPECT_FALSE(job->has_been_started());
     EXPECT_EQ(kManifestUrl, job->manifest_url());
     EXPECT_EQ(kCacheId, job->cache_id());
-    EXPECT_EQ(kGroupId, job->group_id());
     EXPECT_EQ(kEntry.types(), job->entry().types());
     EXPECT_EQ(kEntry.response_id(), job->entry().response_id());
 
@@ -499,17 +528,17 @@ class AppCacheURLRequestJobTest : public testing::Test {
     AppCacheStorage* storage = service_->storage();
     request_ = empty_context_->CreateRequest(GURL("http://blah/"),
                                              net::DEFAULT_PRIORITY,
-                                             url_request_delegate_.get(),
-                                             NULL);
+                                             url_request_delegate_.get());
 
-    // Setup to create an AppCacheURLRequestJob with orders to deliver
+    // Set up to create an AppCacheURLRequestJob with orders to deliver
     // a network response.
-    AppCacheURLRequestJob* mock_job = new AppCacheURLRequestJob(
-        request_.get(), NULL, storage, NULL, false);
-    job_factory_->SetJob(mock_job);
+    std::unique_ptr<AppCacheURLRequestJob> mock_job(new AppCacheURLRequestJob(
+        request_.get(), nullptr, storage, nullptr, false,
+        base::Bind(&SetIfCalled, &restart_callback_invoked_)));
     mock_job->DeliverNetworkResponse();
     EXPECT_TRUE(mock_job->is_delivering_network_response());
     EXPECT_FALSE(mock_job->has_been_started());
+    job_factory_->SetJob(std::move(mock_job));
 
     // Start the request.
     request_->Start();
@@ -522,6 +551,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
   void VerifyDeliverNetworkResponse() {
     EXPECT_EQ(request_->status().error(),
               net::ERR_INTERNET_DISCONNECTED);
+    EXPECT_TRUE(restart_callback_invoked_);
     TestFinished();
   }
 
@@ -536,17 +566,17 @@ class AppCacheURLRequestJobTest : public testing::Test {
     AppCacheStorage* storage = service_->storage();
     request_ = empty_context_->CreateRequest(GURL("http://blah/"),
                                              net::DEFAULT_PRIORITY,
-                                             url_request_delegate_.get(),
-                                             NULL);
+                                             url_request_delegate_.get());
 
     // Setup to create an AppCacheURLRequestJob with orders to deliver
     // a network response.
-    AppCacheURLRequestJob* mock_job = new AppCacheURLRequestJob(
-        request_.get(), NULL, storage, NULL, false);
-    job_factory_->SetJob(mock_job);
+    std::unique_ptr<AppCacheURLRequestJob> mock_job(
+        new AppCacheURLRequestJob(request_.get(), nullptr, storage, nullptr,
+                                  false, base::Bind(&ExpectNotRestarted)));
     mock_job->DeliverErrorResponse();
     EXPECT_TRUE(mock_job->is_delivering_error_response());
     EXPECT_FALSE(mock_job->has_been_started());
+    job_factory_->SetJob(std::move(mock_job));
 
     // Start the request.
     request_->Start();
@@ -578,7 +608,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
         base::Bind(&AppCacheURLRequestJobTest::RequestAppCachedResource,
                    base::Unretained(this), false));
 
-    writer_.reset(service_->storage()->CreateResponseWriter(GURL(), 0));
+    writer_.reset(service_->storage()->CreateResponseWriter(GURL()));
     written_response_id_ = writer_->response_id();
     WriteBasicResponse();
     // Continues async
@@ -588,35 +618,36 @@ class AppCacheURLRequestJobTest : public testing::Test {
     AppCacheStorage* storage = service_->storage();
     request_ = empty_context_->CreateRequest(GURL("http://blah/"),
                                              net::DEFAULT_PRIORITY,
-                                             url_request_delegate_.get(),
-                                             NULL);
+                                             url_request_delegate_.get());
 
     // Setup to create an AppCacheURLRequestJob with orders to deliver
     // a network response.
-    scoped_refptr<AppCacheURLRequestJob> job(new AppCacheURLRequestJob(
-        request_.get(), NULL, storage, NULL, false));
+    std::unique_ptr<AppCacheURLRequestJob> job(
+        new AppCacheURLRequestJob(request_.get(), NULL, storage, NULL, false,
+                                  base::Bind(&ExpectNotRestarted)));
 
     if (start_after_delivery_orders) {
       job->DeliverAppCachedResponse(
-          GURL(), 0, 111,
-          AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_),
-          false);
+          GURL(), 111,
+          AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_), false);
       EXPECT_TRUE(job->is_delivering_appcache_response());
     }
 
     // Start the request.
     EXPECT_FALSE(job->has_been_started());
-    job_factory_->SetJob(job.get());
+    base::WeakPtr<AppCacheURLRequestJob> weak_job = job->GetWeakPtr();
+    job_factory_->SetJob(std::move(job));
     request_->Start();
     EXPECT_FALSE(job_factory_->has_job());
-    EXPECT_TRUE(job->has_been_started());
+    ASSERT_TRUE(weak_job);
+    EXPECT_TRUE(weak_job->has_been_started());
 
     if (!start_after_delivery_orders) {
-      job->DeliverAppCachedResponse(
-          GURL(), 0, 111,
-          AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_),
-          false);
-      EXPECT_TRUE(job->is_delivering_appcache_response());
+      weak_job->DeliverAppCachedResponse(
+          GURL(), 111,
+          AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_), false);
+      ASSERT_TRUE(weak_job);
+      EXPECT_TRUE(weak_job->is_delivering_appcache_response());
     }
 
     // Completion is async.
@@ -650,7 +681,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
        &AppCacheURLRequestJobTest::RequestAppCachedResource,
        base::Unretained(this), true));
 
-    writer_.reset(service_->storage()->CreateResponseWriter(GURL(), 0));
+    writer_.reset(service_->storage()->CreateResponseWriter(GURL()));
     written_response_id_ = writer_->response_id();
     WriteLargeResponse();
     // Continues async
@@ -693,7 +724,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
        base::Unretained(this)));
     PushNextTask(base::Bind(
        &AppCacheURLRequestJobTest::MakeRangeRequest, base::Unretained(this)));
-    writer_.reset(service_->storage()->CreateResponseWriter(GURL(), 0));
+    writer_.reset(service_->storage()->CreateResponseWriter(GURL()));
     written_response_id_ = writer_->response_id();
     WriteBasicResponse();
     // Continues async
@@ -703,8 +734,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
     AppCacheStorage* storage = service_->storage();
     request_ = empty_context_->CreateRequest(GURL("http://blah/"),
                                              net::DEFAULT_PRIORITY,
-                                             url_request_delegate_.get(),
-                                             NULL);
+                                             url_request_delegate_.get());
 
     // Request a range, the 3 middle chars out of 'Hello'
     net::HttpRequestHeaders extra_headers;
@@ -712,20 +742,19 @@ class AppCacheURLRequestJobTest : public testing::Test {
     request_->SetExtraRequestHeaders(extra_headers);
 
     // Create job with orders to deliver an appcached entry.
-    scoped_refptr<AppCacheURLRequestJob> job(new AppCacheURLRequestJob(
-        request_.get(), NULL, storage, NULL, false));
+    std::unique_ptr<AppCacheURLRequestJob> job(
+        new AppCacheURLRequestJob(request_.get(), NULL, storage, NULL, false,
+                                  base::Bind(&ExpectNotRestarted)));
     job->DeliverAppCachedResponse(
-        GURL(), 0, 111,
-        AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_),
-        false);
+        GURL(), 111,
+        AppCacheEntry(AppCacheEntry::EXPLICIT, written_response_id_), false);
     EXPECT_TRUE(job->is_delivering_appcache_response());
 
     // Start the request.
     EXPECT_FALSE(job->has_been_started());
-    job_factory_->SetJob(job.get());
+    job_factory_->SetJob(std::move(job));
     request_->Start();
     EXPECT_FALSE(job_factory_->has_job());
-    EXPECT_TRUE(job->has_been_started());
     // Completion is async.
   }
 
@@ -739,7 +768,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
         url_request_delegate_->received_info_.headers.get();
     EXPECT_EQ(206, headers->response_code());
     EXPECT_EQ(3, headers->GetContentLength());
-    int64 range_start, range_end, object_size;
+    int64_t range_start, range_end, object_size;
     EXPECT_TRUE(
         headers->GetContentRange(&range_start, &range_end, &object_size));
     EXPECT_EQ(1, range_start);
@@ -762,7 +791,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
        &AppCacheURLRequestJobTest::RequestAppCachedResource,
        base::Unretained(this), true));
 
-    writer_.reset(service_->storage()->CreateResponseWriter(GURL(), 0));
+    writer_.reset(service_->storage()->CreateResponseWriter(GURL()));
     written_response_id_ = writer_->response_id();
     WriteLargeResponse();
 
@@ -791,7 +820,7 @@ class AppCacheURLRequestJobTest : public testing::Test {
        &AppCacheURLRequestJobTest::RequestAppCachedResource,
        base::Unretained(this), true));
 
-    writer_.reset(service_->storage()->CreateResponseWriter(GURL(), 0));
+    writer_.reset(service_->storage()->CreateResponseWriter(GURL()));
     written_response_id_ = writer_->response_id();
     WriteLargeResponse();
 
@@ -803,34 +832,36 @@ class AppCacheURLRequestJobTest : public testing::Test {
 
   // Data members --------------------------------------------------------
 
-  scoped_ptr<base::WaitableEvent> test_finished_event_;
-  scoped_ptr<MockStorageDelegate> storage_delegate_;
-  scoped_ptr<MockAppCacheService> service_;
+  std::unique_ptr<base::WaitableEvent> test_finished_event_;
+  std::unique_ptr<MockStorageDelegate> storage_delegate_;
+  std::unique_ptr<MockAppCacheService> service_;
   std::stack<std::pair<base::Closure, bool> > task_stack_;
 
-  scoped_ptr<AppCacheResponseReader> reader_;
+  std::unique_ptr<AppCacheResponseReader> reader_;
   scoped_refptr<HttpResponseInfoIOBuffer> read_info_buffer_;
   scoped_refptr<IOBuffer> read_buffer_;
   int expected_read_result_;
   int reader_deletion_count_down_;
 
-  int64 written_response_id_;
-  scoped_ptr<AppCacheResponseWriter> writer_;
+  int64_t written_response_id_;
+  std::unique_ptr<AppCacheResponseWriter> writer_;
   scoped_refptr<HttpResponseInfoIOBuffer> write_info_buffer_;
   scoped_refptr<IOBuffer> write_buffer_;
   int expected_write_result_;
   int writer_deletion_count_down_;
 
-  scoped_ptr<MockURLRequestJobFactory> job_factory_;
-  scoped_ptr<net::URLRequestContext> empty_context_;
-  scoped_ptr<net::URLRequest> request_;
-  scoped_ptr<MockURLRequestDelegate> url_request_delegate_;
+  bool restart_callback_invoked_;
 
-  static scoped_ptr<base::Thread> io_thread_;
+  std::unique_ptr<MockURLRequestJobFactory> job_factory_;
+  std::unique_ptr<net::URLRequestContext> empty_context_;
+  std::unique_ptr<net::URLRequest> request_;
+  std::unique_ptr<MockURLRequestDelegate> url_request_delegate_;
+
+  static std::unique_ptr<base::Thread> io_thread_;
 };
 
 // static
-scoped_ptr<base::Thread> AppCacheURLRequestJobTest::io_thread_;
+std::unique_ptr<base::Thread> AppCacheURLRequestJobTest::io_thread_;
 
 TEST_F(AppCacheURLRequestJobTest, Basic) {
   RunTestOnIOThread(&AppCacheURLRequestJobTest::Basic);
@@ -868,6 +899,4 @@ TEST_F(AppCacheURLRequestJobTest, CancelRequestWithIOPending) {
   RunTestOnIOThread(&AppCacheURLRequestJobTest::CancelRequestWithIOPending);
 }
 
-}  // namespace
-
 }  // namespace content
