@@ -18,22 +18,26 @@
 #ifndef CONTENT_BROWSER_BROWSER_PLUGIN_BROWSER_PLUGIN_GUEST_H_
 #define CONTENT_BROWSER_BROWSER_PLUGIN_BROWSER_PLUGIN_GUEST_H_
 
+#include <stdint.h>
+
 #include <map>
 #include <queue>
 
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/linked_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/values.h"
+#include "build/build_config.h"
 #include "content/common/edit_command.h"
 #include "content/common/input/input_event_ack_state.h"
 #include "content/public/browser/browser_plugin_guest_delegate.h"
 #include "content/public/browser/guest_host.h"
 #include "content/public/browser/readback_types.h"
 #include "content/public/browser/web_contents_observer.h"
+#include "third_party/WebKit/public/platform/WebDragOperation.h"
 #include "third_party/WebKit/public/platform/WebFocusType.h"
 #include "third_party/WebKit/public/web/WebCompositionUnderline.h"
-#include "third_party/WebKit/public/web/WebDragOperation.h"
 #include "third_party/WebKit/public/web/WebDragStatus.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
 #include "ui/base/ime/text_input_mode.h"
@@ -41,17 +45,14 @@
 #include "ui/gfx/geometry/rect.h"
 
 struct BrowserPluginHostMsg_Attach_Params;
-struct FrameHostMsg_CompositorFrameSwappedACK_Params;
-struct FrameHostMsg_ReclaimCompositorResources_Params;
-struct FrameMsg_CompositorFrameSwapped_Params;
 
-#if defined(OS_MACOSX)
+#if defined(OS_MACOSX) && defined(USE_EXTERNAL_POPUP_MENU)
 struct FrameHostMsg_ShowPopup_Params;
 #endif
 
 namespace cc {
 class CompositorFrame;
-struct SurfaceId;
+class SurfaceId;
 struct SurfaceSequence;
 }  // namespace cc
 
@@ -68,6 +69,7 @@ class RenderWidgetHostView;
 class RenderWidgetHostViewBase;
 class SiteInstance;
 struct DropData;
+struct TextInputState;
 
 // A browser plugin guest provides functionality for WebContents to operate in
 // the guest role and implements guest-specific overrides for ViewHostMsg_*
@@ -178,6 +180,7 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   // GuestHost implementation.
   int LoadURLWithParams(
       const NavigationController::LoadURLParams& load_params) override;
+  void GuestResizeDueToAutoResize(const gfx::Size& new_size) override;
   void SizeContents(const gfx::Size& new_size) override;
   void WillDestroy() override;
 
@@ -186,6 +189,17 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
 
   gfx::Point GetScreenCoordinates(const gfx::Point& relative_position) const;
 
+  // This method is called by the RenderWidgetHostViewGuest to inform the
+  // BrowserPlugin of the potential location of the context menu event (to
+  // come). The need for this (hack) is that the input events when passed on to
+  // the BrowserPlugin are modified by any CSS transforms applied on the plugin.
+  // Therefore, the coordinates of the context menu event with respect to the
+  // container window are modifed with the guest renderer process beiung unaware
+  // of the change. Then eventually, when the context menu event arrives at the
+  // browser, it contains the wrong coordinates (BUG=470087).
+  // TODO(ekaramad): Find a more fundamental solution and remove this later.
+  void SetContextMenuPosition(const gfx::Point& position);
+
   // Helper to send messages to embedder. If this guest is not yet attached,
   // then IPCs will be queued until attachment.
   void SendMessageToEmbedder(IPC::Message* msg);
@@ -224,26 +238,20 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
 
   void PointerLockPermissionResponse(bool allow);
 
-  // The next three functions are virtual for test purposes.
-  virtual void UpdateGuestSizeIfNecessary(const gfx::Size& frame_size,
-                                          float scale_factor);
-  virtual void SwapCompositorFrame(uint32 output_surface_id,
-                                   int host_process_id,
-                                   int host_routing_id,
-                                   scoped_ptr<cc::CompositorFrame> frame);
+  // The next function is virtual for test purposes.
   virtual void SetChildFrameSurface(const cc::SurfaceId& surface_id,
                                     const gfx::Size& frame_size,
                                     float scale_factor,
                                     const cc::SurfaceSequence& sequence);
 
-  void SetContentsOpaque(bool opaque);
-
   // Find the given |search_text| in the page. Returns true if the find request
   // is handled by this browser plugin guest.
-  bool Find(int request_id,
-            const base::string16& search_text,
-            const blink::WebFindOptions& options);
-  bool StopFinding(StopFindAction action);
+  bool HandleFindForEmbedder(int request_id,
+                             const base::string16& search_text,
+                             const blink::WebFindOptions& options);
+  bool HandleStopFindingForEmbedder(StopFindAction action);
+
+  void ResendEventToEmbedder(const blink::WebInputEvent& event);
 
  protected:
 
@@ -269,16 +277,11 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   void InitInternal(const BrowserPluginHostMsg_Attach_Params& params,
                     WebContentsImpl* owner_web_contents);
 
-  bool InAutoSizeBounds(const gfx::Size& size) const;
-
   void OnSatisfySequence(int instance_id, const cc::SurfaceSequence& sequence);
   void OnRequireSequence(int instance_id,
                          const cc::SurfaceId& id,
                          const cc::SurfaceSequence& sequence);
   // Message handlers for messages from embedder.
-  void OnCompositorFrameSwappedACK(
-      int instance_id,
-      const FrameHostMsg_CompositorFrameSwappedACK_Params& params);
   void OnDetach(int instance_id);
   // Handles drag events from the embedder.
   // When dragging, the drag events go to the embedder first, and if the drag
@@ -294,11 +297,6 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   void OnExecuteEditCommand(int instance_id,
                             const std::string& command);
 
-  // Returns compositor resources reclaimed in the embedder to the guest.
-  void OnReclaimCompositorResources(
-      int instance_id,
-      const FrameHostMsg_ReclaimCompositorResources_Params& params);
-
   void OnLockMouse(bool user_gesture,
                    bool last_unlocked_by_target,
                    bool privileged);
@@ -334,10 +332,7 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   void OnUnlockMouseAck(int instance_id);
   void OnUpdateGeometry(int instance_id, const gfx::Rect& view_rect);
 
-  void OnTextInputTypeChanged(ui::TextInputType type,
-                              ui::TextInputMode input_mode,
-                              bool can_compose_inline,
-                              int flags);
+  void OnTextInputStateChanged(const TextInputState& params);
   void OnImeSetComposition(
       int instance_id,
       const std::string& text,
@@ -361,7 +356,7 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
       blink::WebInputEvent::Type event_type,
       InputEventAckState ack_result);
   void OnHasTouchEventHandlers(bool accept);
-#if defined(OS_MACOSX)
+#if defined(OS_MACOSX) && defined(USE_EXTERNAL_POPUP_MENU)
   // On MacOS X popups are painted by the browser process. We handle them here
   // so that they are positioned correctly.
   void OnShowPopup(RenderFrameHost* render_frame_host,
@@ -377,6 +372,12 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   void OnWillAttachComplete(WebContentsImpl* embedder_web_contents,
                             const BrowserPluginHostMsg_Attach_Params& params);
 
+  // Returns identical message with current browser_plugin_instance_id() if
+  // the input was created with browser_plugin::kInstanceIdNone, else it returns
+  // the input message unmodified. If no current browser_plugin_instance_id()
+  // is set, or anything goes wrong, the input message is returned.
+  IPC::Message* UpdateInstanceIdIfNecessary(IPC::Message* msg) const;
+
   // Forwards all messages from the |pending_messages_| queue to the embedder.
   void SendQueuedMessages();
 
@@ -385,7 +386,7 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   // The last tooltip that was set with SetTooltipText().
   base::string16 current_tooltip_text_;
 
-  scoped_ptr<EmbedderVisibilityObserver> embedder_visibility_observer_;
+  std::unique_ptr<EmbedderVisibilityObserver> embedder_visibility_observer_;
   WebContentsImpl* owner_web_contents_;
 
   // Indicates whether this guest has been attached to a container.
@@ -423,10 +424,8 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   bool initialized_;
 
   // Text input type states.
-  ui::TextInputType last_text_input_type_;
-  ui::TextInputMode last_input_mode_;
-  int last_input_flags_;
-  bool last_can_compose_inline_;
+  // Using scoped_ptr to avoid including the header file: view_messages.h.
+  std::unique_ptr<const TextInputState> last_text_input_state_;
 
   // The is the routing ID for a swapped out RenderView for the guest
   // WebContents in the embedder's process.
@@ -438,17 +437,8 @@ class CONTENT_EXPORT BrowserPluginGuest : public GuestHost,
   // Whether or not our embedder has seen a DragSourceEndedAt() call.
   bool seen_embedder_drag_source_ended_at_;
 
-  // Indicates the URL dragged into the guest if any.
-  GURL dragged_url_;
-
-  // Guests generate frames and send a CompositorFrameSwapped (CFS) message
-  // indicating the next frame is ready to be positioned and composited.
-  // Subsequent frames are not generated until the IPC is ACKed. We would like
-  // to ensure that the guest generates frames on attachment so we directly ACK
-  // an unACKed CFS. ACKs could get lost between the time a guest is detached
-  // from a container and the time it is attached elsewhere. This mitigates this
-  // race by ensuring the guest is ACKed on attachment.
-  scoped_ptr<FrameMsg_CompositorFrameSwapped_Params> last_pending_frame_;
+  // Ignore the URL dragged into guest that is coming from guest.
+  bool ignore_dragged_url_;
 
   // This is a queue of messages that are destined to be sent to the embedder
   // once the guest is attached to a particular embedder.
