@@ -9,39 +9,56 @@
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/location.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/string_number_conversions.h"
 #include "cc/base/switches.h"
-#include "content/browser/compositor/browser_compositor_overlay_candidate_validator.h"
+#include "cc/output/output_surface_client.h"
+#include "cc/scheduler/begin_frame_source.h"
+#include "components/display_compositor/compositor_overlay_candidate_validator.h"
 #include "content/browser/compositor/reflector_impl.h"
 #include "content/common/gpu/client/context_provider_command_buffer.h"
 
 namespace content {
 
 BrowserCompositorOutputSurface::BrowserCompositorOutputSurface(
-    const scoped_refptr<cc::ContextProvider>& context_provider,
-    const scoped_refptr<cc::ContextProvider>& worker_context_provider,
-    const scoped_refptr<ui::CompositorVSyncManager>& vsync_manager,
-    scoped_ptr<BrowserCompositorOverlayCandidateValidator>
+    scoped_refptr<cc::ContextProvider> context_provider,
+    scoped_refptr<ui::CompositorVSyncManager> vsync_manager,
+    cc::SyntheticBeginFrameSource* begin_frame_source,
+    std::unique_ptr<display_compositor::CompositorOverlayCandidateValidator>
         overlay_candidate_validator)
-    : OutputSurface(context_provider, worker_context_provider),
-      vsync_manager_(vsync_manager),
+    : OutputSurface(std::move(context_provider), nullptr, nullptr),
+      vsync_manager_(std::move(vsync_manager)),
+      synthetic_begin_frame_source_(begin_frame_source),
       reflector_(nullptr),
       use_begin_frame_scheduling_(
-          base::CommandLine::ForCurrentProcess()
-              ->HasSwitch(cc::switches::kEnableBeginFrameScheduling)) {
+          base::CommandLine::ForCurrentProcess()->HasSwitch(
+              cc::switches::kEnableBeginFrameScheduling)) {
   overlay_candidate_validator_ = std::move(overlay_candidate_validator);
   Initialize();
 }
 
 BrowserCompositorOutputSurface::BrowserCompositorOutputSurface(
-    scoped_ptr<cc::SoftwareOutputDevice> software_device,
-    const scoped_refptr<ui::CompositorVSyncManager>& vsync_manager)
-    : OutputSurface(std::move(software_device)),
+    std::unique_ptr<cc::SoftwareOutputDevice> software_device,
+    const scoped_refptr<ui::CompositorVSyncManager>& vsync_manager,
+    cc::SyntheticBeginFrameSource* begin_frame_source)
+    : OutputSurface(nullptr, nullptr, std::move(software_device)),
       vsync_manager_(vsync_manager),
+      synthetic_begin_frame_source_(begin_frame_source),
       reflector_(nullptr),
       use_begin_frame_scheduling_(
-          base::CommandLine::ForCurrentProcess()
-              ->HasSwitch(cc::switches::kEnableBeginFrameScheduling)) {
+          base::CommandLine::ForCurrentProcess()->HasSwitch(
+              cc::switches::kEnableBeginFrameScheduling)) {
+  Initialize();
+}
+
+BrowserCompositorOutputSurface::BrowserCompositorOutputSurface(
+    const scoped_refptr<cc::VulkanContextProvider>& vulkan_context_provider,
+    const scoped_refptr<ui::CompositorVSyncManager>& vsync_manager,
+    cc::SyntheticBeginFrameSource* begin_frame_source)
+    : OutputSurface(std::move(vulkan_context_provider)),
+      vsync_manager_(vsync_manager),
+      synthetic_begin_frame_source_(begin_frame_source),
+      reflector_(nullptr) {
   Initialize();
 }
 
@@ -73,12 +90,22 @@ bool BrowserCompositorOutputSurface::BindToClient(
   return true;
 }
 
+void BrowserCompositorOutputSurface::UpdateVSyncParametersInternal(
+    base::TimeTicks timebase,
+    base::TimeDelta interval) {
+  if (interval.is_zero()) {
+    // TODO(brianderson): We should not be receiving 0 intervals.
+    interval = cc::BeginFrameArgs::DefaultInterval();
+  }
+  synthetic_begin_frame_source_->OnUpdateVSyncParameters(timebase, interval);
+}
+
 void BrowserCompositorOutputSurface::OnUpdateVSyncParameters(
     base::TimeTicks timebase,
     base::TimeDelta interval) {
   DCHECK(HasClient());
   DCHECK(!use_begin_frame_scheduling_);
-  CommitVSyncParameters(timebase, interval);
+  UpdateVSyncParametersInternal(timebase, interval);
 }
 
 void BrowserCompositorOutputSurface::OnUpdateVSyncParametersFromGpu(
@@ -86,7 +113,7 @@ void BrowserCompositorOutputSurface::OnUpdateVSyncParametersFromGpu(
     base::TimeDelta interval) {
   DCHECK(HasClient());
   if (use_begin_frame_scheduling_) {
-    CommitVSyncParameters(timebase, interval);
+    UpdateVSyncParametersInternal(timebase, interval);
     return;
   }
 
