@@ -5,6 +5,8 @@
 #ifndef CONTENT_BROWSER_DEVTOOLS_DEVTOOLS_MANAGER_H_
 #define CONTENT_BROWSER_DEVTOOLS_DEVTOOLS_MANAGER_H_
 
+#include <memory>
+
 #include "base/compiler_specific.h"
 #include "base/macros.h"
 #include "base/memory/singleton.h"
@@ -34,7 +36,7 @@ class CONTENT_EXPORT DevToolsManager {
  private:
   friend struct base::DefaultSingletonTraits<DevToolsManager>;
 
-  scoped_ptr<DevToolsManagerDelegate> delegate_;
+  std::unique_ptr<DevToolsManagerDelegate> delegate_;
   int attached_hosts_count_;
 
   DISALLOW_COPY_AND_ASSIGN(DevToolsManager);
