@@ -12,6 +12,7 @@
 #include "base/values.h"
 #include "build/build_config.h"
 #include "content/public/browser/devtools_agent_host.h"
+#include "content/public/browser/javascript_dialog_manager.h"
 #include "content/public/browser/render_view_host.h"
 #include "content/public/browser/web_contents.h"
 #include "content/public/common/url_constants.h"
@@ -34,6 +35,64 @@ const char kIdParam[] = "id";
 const char kMethodParam[] = "method";
 const char kParamsParam[] = "params";
 
+class TestJavaScriptDialogManager : public JavaScriptDialogManager,
+                                    public WebContentsDelegate {
+ public:
+  TestJavaScriptDialogManager() : handle_(false) {}
+  ~TestJavaScriptDialogManager() override {}
+
+  void Handle()
+  {
+    if (!callback_.is_null()) {
+      callback_.Run(true, base::string16());
+      callback_.Reset();
+    } else {
+      handle_ = true;
+    }
+  }
+
+  // WebContentsDelegate
+  JavaScriptDialogManager* GetJavaScriptDialogManager(
+      WebContents* source) override {
+    return this;
+  }
+
+  // JavaScriptDialogManager
+  void RunJavaScriptDialog(WebContents* web_contents,
+                           const GURL& origin_url,
+                           JavaScriptMessageType javascript_message_type,
+                           const base::string16& message_text,
+                           const base::string16& default_prompt_text,
+                           const DialogClosedCallback& callback,
+                           bool* did_suppress_message) override {
+    if (handle_) {
+      handle_ = false;
+      callback.Run(true, base::string16());
+    } else {
+      callback_ = callback;
+    }
+  };
+
+  void RunBeforeUnloadDialog(WebContents* web_contents,
+                             bool is_reload,
+                             const DialogClosedCallback& callback) override {}
+
+  bool HandleJavaScriptDialog(WebContents* web_contents,
+                              bool accept,
+                              const base::string16* prompt_override) override {
+    return true;
+  }
+
+  void CancelActiveAndPendingDialogs(WebContents* web_contents) override {}
+
+  void ResetDialogState(WebContents* web_contents) override {}
+
+ private:
+  DialogClosedCallback callback_;
+  bool handle_;
+  DISALLOW_COPY_AND_ASSIGN(TestJavaScriptDialogManager);
+};
+
 }
 
 class DevToolsProtocolTest : public ContentBrowserTest,
@@ -47,12 +106,12 @@ class DevToolsProtocolTest : public ContentBrowserTest,
 
  protected:
   void SendCommand(const std::string& method,
-                   scoped_ptr<base::DictionaryValue> params) {
+                   std::unique_ptr<base::DictionaryValue> params) {
     SendCommand(method, std::move(params), true);
   }
 
   void SendCommand(const std::string& method,
-                   scoped_ptr<base::DictionaryValue> params,
+                   std::unique_ptr<base::DictionaryValue> params,
                    bool wait) {
     in_dispatch_ = true;
     base::DictionaryValue command;
@@ -63,7 +122,7 @@ class DevToolsProtocolTest : public ContentBrowserTest,
 
     std::string json_command;
     base::JSONWriter::Write(command, &json_command);
-    agent_host_->DispatchProtocolMessage(json_command);
+    agent_host_->DispatchProtocolMessage(this, json_command);
     // Some messages are dispatched synchronously.
     // Only run loop if we are not finished yet.
     if (in_dispatch_ && wait) {
@@ -105,7 +164,7 @@ class DevToolsProtocolTest : public ContentBrowserTest,
 
   void TearDownOnMainThread() override {
     if (agent_host_) {
-      agent_host_->DetachClient();
+      agent_host_->DetachClient(this);
       agent_host_ = nullptr;
     }
   }
@@ -115,7 +174,7 @@ class DevToolsProtocolTest : public ContentBrowserTest,
     RunMessageLoop();
   }
 
-  scoped_ptr<base::DictionaryValue> result_;
+  std::unique_ptr<base::DictionaryValue> result_;
   scoped_refptr<DevToolsAgentHost> agent_host_;
   int last_sent_id_;
   std::vector<int> result_ids_;
@@ -124,8 +183,9 @@ class DevToolsProtocolTest : public ContentBrowserTest,
  private:
   void DispatchProtocolMessage(DevToolsAgentHost* agent_host,
                                const std::string& message) override {
-    scoped_ptr<base::DictionaryValue> root(static_cast<base::DictionaryValue*>(
-        base::JSONReader::Read(message).release()));
+    std::unique_ptr<base::DictionaryValue> root(
+        static_cast<base::DictionaryValue*>(
+            base::JSONReader::Read(message).release()));
     int id;
     if (root->GetInteger("id", &id)) {
       result_ids_.push_back(id);
@@ -163,7 +223,7 @@ class SyntheticKeyEventTest : public DevToolsProtocolTest {
                     int modifier,
                     int windowsKeyCode,
                     int nativeKeyCode) {
-    scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+    std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
     params->SetString("type", type);
     params->SetInteger("modifiers", modifier);
     params->SetInteger("windowsVirtualKeyCode", windowsKeyCode);
@@ -255,15 +315,14 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, DISABLED_SynthesizePinchGesture) {
 
   int old_width;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(window.innerWidth)", &old_width));
+      shell(), "domAutomationController.send(window.innerWidth)", &old_width));
 
   int old_height;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(window.innerHeight)", &old_height));
+      shell(), "domAutomationController.send(window.innerHeight)",
+      &old_height));
 
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
   params->SetInteger("x", old_width / 2);
   params->SetInteger("y", old_height / 2);
   params->SetDouble("scaleFactor", 2.0);
@@ -271,29 +330,28 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, DISABLED_SynthesizePinchGesture) {
 
   int new_width;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(window.innerWidth)", &new_width));
+      shell(), "domAutomationController.send(window.innerWidth)", &new_width));
   ASSERT_DOUBLE_EQ(2.0, static_cast<double>(old_width) / new_width);
 
   int new_height;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(window.innerHeight)", &new_height));
+      shell(), "domAutomationController.send(window.innerHeight)",
+      &new_height));
   ASSERT_DOUBLE_EQ(2.0, static_cast<double>(old_height) / new_height);
 }
 
-IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, SynthesizeScrollGesture) {
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, DISABLED_SynthesizeScrollGesture) {
   GURL test_url = GetTestUrl("devtools", "synthetic_gesture_tests.html");
   NavigateToURLBlockUntilNavigationsComplete(shell(), test_url, 1);
   Attach();
 
   int scroll_top;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(document.body.scrollTop)", &scroll_top));
+      shell(), "domAutomationController.send(document.body.scrollTop)",
+      &scroll_top));
   ASSERT_EQ(0, scroll_top);
 
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
   params->SetInteger("x", 0);
   params->SetInteger("y", 0);
   params->SetInteger("xDistance", 0);
@@ -301,23 +359,23 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, SynthesizeScrollGesture) {
   SendCommand("Input.synthesizeScrollGesture", std::move(params));
 
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(document.body.scrollTop)", &scroll_top));
+      shell(), "domAutomationController.send(document.body.scrollTop)",
+      &scroll_top));
   ASSERT_EQ(100, scroll_top);
 }
 
-IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, SynthesizeTapGesture) {
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, DISABLED_SynthesizeTapGesture) {
   GURL test_url = GetTestUrl("devtools", "synthetic_gesture_tests.html");
   NavigateToURLBlockUntilNavigationsComplete(shell(), test_url, 1);
   Attach();
 
   int scroll_top;
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(document.body.scrollTop)", &scroll_top));
+      shell(), "domAutomationController.send(document.body.scrollTop)",
+      &scroll_top));
   ASSERT_EQ(0, scroll_top);
 
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
   params->SetInteger("x", 16);
   params->SetInteger("y", 16);
   params->SetString("gestureSourceType", "touch");
@@ -328,8 +386,8 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, SynthesizeTapGesture) {
   // of the device that we're testing on, but in any case it should be greater
   // than 0.
   ASSERT_TRUE(content::ExecuteScriptAndExtractInt(
-      shell()->web_contents(),
-      "domAutomationController.send(document.body.scrollTop)", &scroll_top));
+      shell(), "domAutomationController.send(document.body.scrollTop)",
+      &scroll_top));
   ASSERT_GT(scroll_top, 0);
 }
 #endif  // defined(OS_ANDROID)
@@ -341,10 +399,12 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, NavigationPreservesMessages) {
   Attach();
   SendCommand("Page.enable", nullptr, false);
 
-  scoped_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
   test_url = GetTestUrl("devtools", "navigation.html");
   params->SetString("url", test_url.spec());
+  TestNavigationObserver navigation_observer(shell()->web_contents());
   SendCommand("Page.navigate", std::move(params), true);
+  navigation_observer.Wait();
 
   bool enough_results = result_ids_.size() >= 2u;
   EXPECT_TRUE(enough_results);
@@ -407,7 +467,7 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, CrossSitePauseInBeforeUnload) {
   SendCommand("Debugger.enable", nullptr);
 
   ASSERT_TRUE(content::ExecuteScript(
-      shell()->web_contents(),
+      shell(),
       "window.onbeforeunload = function() { debugger; return null; }"));
 
   shell()->LoadURL(
@@ -428,8 +488,7 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, InspectDuringFrameSwap) {
   NavigateToURLBlockUntilNavigationsComplete(shell(), test_url1, 1);
 
   ShellAddedObserver new_shell_observer;
-  EXPECT_TRUE(ExecuteScript(shell()->web_contents(),
-                            "window.open('about:blank','foo');"));
+  EXPECT_TRUE(ExecuteScript(shell(), "window.open('about:blank','foo');"));
   Shell* new_shell = new_shell_observer.GetShell();
   EXPECT_TRUE(new_shell->web_contents()->HasOpener());
 
@@ -452,7 +511,7 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, InspectDuringFrameSwap) {
   // should be fixed to support waiting on both WATCH_FOR_PROCESS_EXIT and
   // WATCH_FOR_HOST_DESTRUCTION, and then used here.
   bool success = false;
-  EXPECT_TRUE(ExecuteScriptAndExtractBool(shell()->web_contents(),
+  EXPECT_TRUE(ExecuteScriptAndExtractBool(shell(),
                                           "window.domAutomationController.send("
                                           "    !!window.open('', 'foo'));",
                                           &success));
@@ -467,7 +526,7 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, InspectDuringFrameSwap) {
   // Ensure that the A.com process is still alive by executing a script in the
   // original tab.
   success = false;
-  EXPECT_TRUE(ExecuteScriptAndExtractBool(shell()->web_contents(),
+  EXPECT_TRUE(ExecuteScriptAndExtractBool(shell(),
                                           "window.domAutomationController.send("
                                           "    !!window.open('', 'foo'));",
                                           &success));
@@ -507,4 +566,43 @@ IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, ReloadBlankPage) {
   // Should not crash at this point.
 }
 
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, EvaluateInBlankPage) {
+  NavigateToURLBlockUntilNavigationsComplete(shell(), GURL("about:blank"), 1);
+  Attach();
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  params->SetString("expression", "window");
+  SendCommand("Runtime.evaluate", std::move(params), true);
+  bool wasThrown = true;
+  EXPECT_TRUE(result_->GetBoolean("wasThrown", &wasThrown));
+  EXPECT_FALSE(wasThrown);
+}
+
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest,
+    EvaluateInBlankPageAfterNavigation) {
+  ASSERT_TRUE(embedded_test_server()->Start());
+  GURL test_url = embedded_test_server()->GetURL("/devtools/navigation.html");
+  NavigateToURLBlockUntilNavigationsComplete(shell(), test_url, 1);
+  Attach();
+  NavigateToURLBlockUntilNavigationsComplete(shell(), GURL("about:blank"), 1);
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  params->SetString("expression", "window");
+  SendCommand("Runtime.evaluate", std::move(params), true);
+  bool wasThrown = true;
+  EXPECT_TRUE(result_->GetBoolean("wasThrown", &wasThrown));
+  EXPECT_FALSE(wasThrown);
+}
+
+IN_PROC_BROWSER_TEST_F(DevToolsProtocolTest, JavaScriptDialogNotifications) {
+  NavigateToURLBlockUntilNavigationsComplete(shell(), GURL("about:blank"), 1);
+  Attach();
+  TestJavaScriptDialogManager dialog_manager;
+  shell()->web_contents()->SetDelegate(&dialog_manager);
+  SendCommand("Page.enable", nullptr, true);
+  std::unique_ptr<base::DictionaryValue> params(new base::DictionaryValue());
+  params->SetString("expression", "alert('alert')");
+  SendCommand("Runtime.evaluate", std::move(params), false);
+  WaitForNotification("Page.javascriptDialogOpening");
+  dialog_manager.Handle();
+}
+
 }  // namespace content
