@@ -5,6 +5,7 @@
 #ifndef CONTENT_BROWSER_FILEAPI_MOCK_URL_REQUEST_DELEGATE_H_
 #define CONTENT_BROWSER_FILEAPI_MOCK_URL_REQUEST_DELEGATE_H_
 
+#include "net/base/io_buffer.h"
 #include "net/url_request/url_request.h"
 
 namespace net {
@@ -22,6 +23,7 @@ class MockURLRequestDelegate : public net::URLRequest::Delegate {
   void OnResponseStarted(net::URLRequest* request) override;
   void OnReadCompleted(net::URLRequest* request, int bytes_read) override;
   const std::string& response_data() const { return response_data_; }
+  const net::IOBufferWithSize* metadata() const { return metadata_.get(); }
 
  private:
   void ReadSome(net::URLRequest* request);
@@ -30,6 +32,7 @@ class MockURLRequestDelegate : public net::URLRequest::Delegate {
 
   scoped_refptr<net::IOBuffer> io_buffer_;
   std::string response_data_;
+  scoped_refptr<net::IOBufferWithSize> metadata_;
 };
 
 }  // namespace content
