@@ -14,16 +14,14 @@ NavigationRequestInfo::NavigationRequestInfo(
     const url::Origin& request_initiator,
     bool is_main_frame,
     bool parent_is_main_frame,
-    int frame_tree_node_id,
-    scoped_refptr<ResourceRequestBody> request_body)
+    int frame_tree_node_id)
     : common_params(common_params),
       begin_params(begin_params),
       first_party_for_cookies(first_party_for_cookies),
       request_initiator(request_initiator),
       is_main_frame(is_main_frame),
       parent_is_main_frame(parent_is_main_frame),
-      frame_tree_node_id(frame_tree_node_id),
-      request_body(request_body) {}
+      frame_tree_node_id(frame_tree_node_id) {}
 
 NavigationRequestInfo::~NavigationRequestInfo() {}
 
