@@ -10,26 +10,27 @@
 #include "base/command_line.h"
 #include "base/containers/hash_tables.h"
 #include "base/debug/alias.h"
+#include "base/feature_list.h"
 #include "base/logging.h"
 #include "base/macros.h"
 #include "base/memory/shared_memory.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/time/time.h"
-#include "content/browser/devtools/devtools_netlog_observer.h"
 #include "content/browser/host_zoom_map_impl.h"
+#include "content/browser/loader/netlog_observer.h"
 #include "content/browser/loader/resource_buffer.h"
 #include "content/browser/loader/resource_dispatcher_host_impl.h"
 #include "content/browser/loader/resource_message_filter.h"
 #include "content/browser/loader/resource_request_info_impl.h"
-#include "content/browser/resource_context_impl.h"
 #include "content/common/resource_messages.h"
+#include "content/common/resource_request_completion_status.h"
 #include "content/common/view_messages.h"
 #include "content/public/browser/resource_dispatcher_host_delegate.h"
+#include "content/public/common/content_features.h"
 #include "content/public/common/resource_response.h"
 #include "net/base/io_buffer.h"
 #include "net/base/load_flags.h"
-#include "net/base/net_util.h"
 #include "net/log/net_log.h"
 #include "net/url_request/redirect_info.h"
 
@@ -45,6 +46,14 @@ static int kMaxAllocationSize = 1024 * 32;
 // The interval for calls to ReportUploadProgress.
 static int kUploadProgressIntervalMsec = 10;
 
+// Used when kOptimizeLoadingIPCForSmallResources is enabled.
+// Small resource typically issues two Read call: one for the content itself
+// and another for getting zero response to detect EOF.
+// Inline these two into the IPC message to avoid allocating an expensive
+// SharedMemory for small resources.
+const int kNumLeadingChunk = 2;
+const int kInlinedLeadingChunkSize = 2048;
+
 void GetNumericArg(const std::string& name, int* result) {
   const std::string& value =
       base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(name);
@@ -65,6 +74,112 @@ void InitializeResourceBufferConstants() {
 
 }  // namespace
 
+// Used when kOptimizeLoadingIPCForSmallResources is enabled.
+// The instance hooks the buffer allocation of AsyncResourceHandler, and
+// determine if we should use SharedMemory or should inline the data into
+// the IPC message.
+class AsyncResourceHandler::InliningHelper {
+ public:
+
+  InliningHelper() {}
+  ~InliningHelper() {}
+
+  void OnResponseReceived(const ResourceResponse& response) {
+    InliningStatus status = IsInliningApplicable(response);
+    UMA_HISTOGRAM_ENUMERATION(
+        "Net.ResourceLoader.InliningStatus",
+        static_cast<int>(status),
+        static_cast<int>(InliningStatus::INLINING_STATUS_COUNT));
+    inlining_applicable_ = status == InliningStatus::APPLICABLE;
+  }
+
+  // Returns true if InliningHelper allocates the buffer for inlining.
+  bool PrepareInlineBufferIfApplicable(scoped_refptr<net::IOBuffer>* buf,
+                                      int* buf_size) {
+    ++num_allocation_;
+
+    // If the server sends the resource in multiple small chunks,
+    // |num_allocation_| may exceed |kNumLeadingChunk|. Disable inlining and
+    // fall back to the regular resource loading path in that case.
+    if (!inlining_applicable_ ||
+        num_allocation_ > kNumLeadingChunk ||
+        !base::FeatureList::IsEnabled(
+            features::kOptimizeLoadingIPCForSmallResources)) {
+      return false;
+    }
+
+    leading_chunk_buffer_ = new net::IOBuffer(kInlinedLeadingChunkSize);
+    *buf = leading_chunk_buffer_;
+    *buf_size = kInlinedLeadingChunkSize;
+    return true;
+  }
+
+  // Returns true if the received data is sent to the consumer.
+  bool SendInlinedDataIfApplicable(int bytes_read,
+                                   int encoded_data_length,
+                                   IPC::Sender* sender,
+                                   int request_id) {
+    DCHECK(sender);
+    if (!leading_chunk_buffer_)
+      return false;
+
+    std::vector<char> data(
+        leading_chunk_buffer_->data(),
+        leading_chunk_buffer_->data() + bytes_read);
+    leading_chunk_buffer_ = nullptr;
+
+    sender->Send(new ResourceMsg_InlinedDataChunkReceived(
+        request_id, data, encoded_data_length));
+    return true;
+  }
+
+  void RecordHistogram(int64_t elapsed_time) {
+    if (inlining_applicable_) {
+      UMA_HISTOGRAM_CUSTOM_COUNTS(
+          "Net.ResourceLoader.ResponseStartToEnd.InliningApplicable",
+          elapsed_time, 1, 100000, 100);
+    }
+  }
+
+ private:
+  enum class InliningStatus : int {
+    APPLICABLE = 0,
+    EARLY_ALLOCATION = 1,
+    UNKNOWN_CONTENT_LENGTH = 2,
+    LARGE_CONTENT = 3,
+    HAS_TRANSFER_ENCODING = 4,
+    HAS_CONTENT_ENCODING = 5,
+    INLINING_STATUS_COUNT,
+  };
+
+  InliningStatus IsInliningApplicable(const ResourceResponse& response) {
+    // Disable if the leading chunk is already arrived.
+    if (num_allocation_)
+      return InliningStatus::EARLY_ALLOCATION;
+
+    // Disable if the content is known to be large.
+    if (response.head.content_length > kInlinedLeadingChunkSize)
+      return InliningStatus::LARGE_CONTENT;
+
+    // Disable if the length of the content is unknown.
+    if (response.head.content_length < 0)
+      return InliningStatus::UNKNOWN_CONTENT_LENGTH;
+
+    if (response.head.headers) {
+      if (response.head.headers->HasHeader("Transfer-Encoding"))
+        return InliningStatus::HAS_TRANSFER_ENCODING;
+      if (response.head.headers->HasHeader("Content-Encoding"))
+        return InliningStatus::HAS_CONTENT_ENCODING;
+    }
+
+    return InliningStatus::APPLICABLE;
+  }
+
+  int num_allocation_ = 0;
+  bool inlining_applicable_ = false;
+  scoped_refptr<net::IOBuffer> leading_chunk_buffer_;
+};
+
 class DependentIOBuffer : public net::WrappedIOBuffer {
  public:
   DependentIOBuffer(ResourceBuffer* backing, char* memory)
@@ -87,7 +202,8 @@ AsyncResourceHandler::AsyncResourceHandler(
       did_defer_(false),
       has_checked_for_sufficient_resources_(false),
       sent_received_response_msg_(false),
-      sent_first_data_msg_(false),
+      sent_data_buffer_msg_(false),
+      inlining_helper_(new InliningHelper),
       last_upload_position_(0),
       waiting_for_upload_progress_ack_(false),
       reported_transfer_size_(0) {
@@ -186,7 +302,7 @@ bool AsyncResourceHandler::OnRequestRedirected(
         redirect_info.new_url, request(), info->GetContext(), response);
   }
 
-  DevToolsNetLogObserver::PopulateResponseInfo(request(), response);
+  NetLogObserver::PopulateResponseInfo(request(), response);
   response->head.encoded_data_length = request()->GetTotalReceivedBytes();
   reported_transfer_size_ = 0;
   response->head.request_start = request()->creation_time();
@@ -207,6 +323,8 @@ bool AsyncResourceHandler::OnResponseStarted(ResourceResponse* response,
   // request commits, avoiding the possibility of e.g. zooming the old content
   // or of having to layout the new content twice.
 
+  response_started_ticks_ = base::TimeTicks::Now();
+
   progress_timer_.Stop();
   const ResourceRequestInfoImpl* info = GetRequestInfo();
   if (!info->filter())
@@ -225,7 +343,7 @@ bool AsyncResourceHandler::OnResponseStarted(ResourceResponse* response,
         request(), info->GetContext(), response, info->filter());
   }
 
-  DevToolsNetLogObserver::PopulateResponseInfo(request(), response);
+  NetLogObserver::PopulateResponseInfo(request(), response);
 
   const HostZoomMapImpl* host_zoom_map =
       static_cast<const HostZoomMapImpl*>(info->filter()->GetHostZoomMap());
@@ -264,6 +382,7 @@ bool AsyncResourceHandler::OnResponseStarted(ResourceResponse* response,
                                                                 copy));
   }
 
+  inlining_helper_->OnResponseReceived(*response);
   return true;
 }
 
@@ -289,6 +408,14 @@ bool AsyncResourceHandler::OnWillRead(scoped_refptr<net::IOBuffer>* buf,
                                       int min_size) {
   DCHECK_EQ(-1, min_size);
 
+  if (!CheckForSufficientResource())
+    return false;
+
+  // Return early if InliningHelper allocates the buffer, so that we should
+  // inline the data into the IPC message without allocating SharedMemory.
+  if (inlining_helper_->PrepareInlineBufferIfApplicable(buf, buf_size))
+    return true;
+
   if (!EnsureResourceBufferIsInitialized())
     return false;
 
@@ -312,31 +439,28 @@ bool AsyncResourceHandler::OnReadCompleted(int bytes_read, bool* defer) {
   if (!filter)
     return false;
 
+  int encoded_data_length = CalculateEncodedDataLengthToReport();
+
+  // Return early if InliningHelper handled the received data.
+  if (inlining_helper_->SendInlinedDataIfApplicable(
+          bytes_read, encoded_data_length, filter, GetRequestID()))
+    return true;
+
   buffer_->ShrinkLastAllocation(bytes_read);
 
-  if (!sent_first_data_msg_) {
-    base::SharedMemoryHandle handle;
-    int size;
-    if (!buffer_->ShareToProcess(filter->PeerHandle(), &handle, &size))
+  if (!sent_data_buffer_msg_) {
+    base::SharedMemoryHandle handle = base::SharedMemory::DuplicateHandle(
+        buffer_->GetSharedMemory().handle());
+    if (!base::SharedMemory::IsHandleValid(handle))
       return false;
-
-    // TODO(erikchen): Temporary debugging. http://crbug.com/527588.
-    CHECK_LE(size, kBufferSize);
     filter->Send(new ResourceMsg_SetDataBuffer(
-        GetRequestID(), handle, size, filter->peer_pid()));
-    sent_first_data_msg_ = true;
+        GetRequestID(), handle, buffer_->GetSharedMemory().mapped_size(),
+        filter->peer_pid()));
+    sent_data_buffer_msg_ = true;
   }
 
   int data_offset = buffer_->GetLastAllocationOffset();
 
-  int64_t current_transfer_size = request()->GetTotalReceivedBytes();
-  int encoded_data_length = current_transfer_size - reported_transfer_size_;
-  reported_transfer_size_ = current_transfer_size;
-
-  // TODO(erikchen): Temporary debugging. http://crbug.com/527588.
-  CHECK_LE(data_offset, kBufferSize);
-
-  filter->Send(new ResourceMsg_DataReceivedDebug(GetRequestID(), data_offset));
   filter->Send(new ResourceMsg_DataReceived(
       GetRequestID(), data_offset, bytes_read, encoded_data_length));
   ++pending_data_count_;
@@ -350,9 +474,7 @@ bool AsyncResourceHandler::OnReadCompleted(int bytes_read, bool* defer) {
 }
 
 void AsyncResourceHandler::OnDataDownloaded(int bytes_downloaded) {
-  int64_t current_transfer_size = request()->GetTotalReceivedBytes();
-  int encoded_data_length = current_transfer_size - reported_transfer_size_;
-  reported_transfer_size_ = current_transfer_size;
+  int encoded_data_length = CalculateEncodedDataLengthToReport();
 
   ResourceMessageFilter* filter = GetFilter();
   if (filter) {
@@ -402,7 +524,7 @@ void AsyncResourceHandler::OnResponseCompleted(
     error_code = net::ERR_FAILED;
   }
 
-  ResourceMsg_RequestCompleteData request_complete_data;
+  ResourceRequestCompletionStatus request_complete_data;
   request_complete_data.error_code = error_code;
   request_complete_data.was_ignored_by_handler = was_ignored_by_handler;
   request_complete_data.exists_in_cache = request()->response_info().was_cached;
@@ -412,20 +534,17 @@ void AsyncResourceHandler::OnResponseCompleted(
       request()->GetTotalReceivedBytes();
   info->filter()->Send(
       new ResourceMsg_RequestComplete(GetRequestID(), request_complete_data));
+
+  if (status.is_success())
+    RecordHistogram();
 }
 
 bool AsyncResourceHandler::EnsureResourceBufferIsInitialized() {
+  DCHECK(has_checked_for_sufficient_resources_);
+
   if (buffer_.get() && buffer_->IsInitialized())
     return true;
 
-  if (!has_checked_for_sufficient_resources_) {
-    has_checked_for_sufficient_resources_ = true;
-    if (!rdh_->HasSufficientResourcesForRequest(request())) {
-      controller()->CancelWithError(net::ERR_INSUFFICIENT_RESOURCES);
-      return false;
-    }
-  }
-
   buffer_ = new ResourceBuffer();
   return buffer_->Initialize(kBufferSize,
                              kMinAllocationSize,
@@ -444,4 +563,49 @@ void AsyncResourceHandler::OnDefer() {
   request()->LogBlockedBy("AsyncResourceHandler");
 }
 
+bool AsyncResourceHandler::CheckForSufficientResource() {
+  if (has_checked_for_sufficient_resources_)
+    return true;
+  has_checked_for_sufficient_resources_ = true;
+
+  if (rdh_->HasSufficientResourcesForRequest(request()))
+    return true;
+
+  controller()->CancelWithError(net::ERR_INSUFFICIENT_RESOURCES);
+  return false;
+}
+
+int AsyncResourceHandler::CalculateEncodedDataLengthToReport() {
+  int64_t current_transfer_size = request()->GetTotalReceivedBytes();
+  int encoded_data_length = current_transfer_size - reported_transfer_size_;
+  reported_transfer_size_ = current_transfer_size;
+  return encoded_data_length;
+}
+
+void AsyncResourceHandler::RecordHistogram() {
+  int64_t elapsed_time =
+      (base::TimeTicks::Now() - response_started_ticks_).InMicroseconds();
+  int64_t encoded_length = request()->GetTotalReceivedBytes();
+  if (encoded_length < 2 * 1024) {
+    // The resource was smaller than the smallest required buffer size.
+    UMA_HISTOGRAM_CUSTOM_COUNTS("Net.ResourceLoader.ResponseStartToEnd.LT_2kB",
+                                elapsed_time, 1, 100000, 100);
+  } else if (encoded_length < 32 * 1024) {
+    // The resource was smaller than single chunk.
+    UMA_HISTOGRAM_CUSTOM_COUNTS("Net.ResourceLoader.ResponseStartToEnd.LT_32kB",
+                                elapsed_time, 1, 100000, 100);
+  } else if (encoded_length < 512 * 1024) {
+    // The resource was smaller than single chunk.
+    UMA_HISTOGRAM_CUSTOM_COUNTS(
+        "Net.ResourceLoader.ResponseStartToEnd.LT_512kB",
+        elapsed_time, 1, 100000, 100);
+  } else {
+    UMA_HISTOGRAM_CUSTOM_COUNTS(
+        "Net.ResourceLoader.ResponseStartToEnd.Over_512kB",
+        elapsed_time, 1, 100000, 100);
+  }
+
+  inlining_helper_->RecordHistogram(elapsed_time);
+}
+
 }  // namespace content
