@@ -5,8 +5,9 @@
 #ifndef CONTENT_BROWSER_LOADER_NAVIGATION_URL_LOADER_FACTORY_H_
 #define CONTENT_BROWSER_LOADER_NAVIGATION_URL_LOADER_FACTORY_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "content/browser/loader/navigation_url_loader.h"
 
 namespace content {
@@ -15,10 +16,10 @@ namespace content {
 // the loader. This is intended for testing.
 class NavigationURLLoaderFactory {
  public:
-  virtual scoped_ptr<NavigationURLLoader> CreateLoader(
+  virtual std::unique_ptr<NavigationURLLoader> CreateLoader(
       BrowserContext* browser_context,
-      scoped_ptr<NavigationRequestInfo> request_info,
-      ServiceWorkerNavigationHandle* service_worker_handle,
+      std::unique_ptr<NavigationRequestInfo> request_info,
+      ServiceWorkerContextWrapper* service_worker_context_wrapper,
       NavigationURLLoaderDelegate* delegate) = 0;
 
  protected:
