@@ -4,11 +4,14 @@
 
 #include "content/browser/media/capture/web_contents_audio_input_stream.h"
 
+#include <stdint.h>
+
 #include <list>
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/location.h"
+#include "base/macros.h"
 #include "base/single_thread_task_runner.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/threading/thread.h"
@@ -30,9 +33,11 @@ using ::testing::NotNull;
 using ::testing::SaveArg;
 using ::testing::WithArgs;
 
+using media::AudioBus;
 using media::AudioInputStream;
 using media::AudioOutputStream;
 using media::AudioParameters;
+using media::AudioPushSink;
 using media::SineWaveAudioSource;
 using media::VirtualAudioInputStream;
 using media::VirtualAudioOutputStream;
@@ -116,12 +121,12 @@ class MockVirtualAudioInputStream : public VirtualAudioInputStream {
     ON_CALL(*this, GetAutomaticGainControl())
         .WillByDefault(
             Invoke(&real_, &VirtualAudioInputStream::GetAutomaticGainControl));
-    ON_CALL(*this, AddOutputStream(NotNull(), _))
+    ON_CALL(*this, AddInputProvider(NotNull(), _))
         .WillByDefault(
-            Invoke(&real_, &VirtualAudioInputStream::AddOutputStream));
-    ON_CALL(*this, RemoveOutputStream(NotNull(), _))
+            Invoke(&real_, &VirtualAudioInputStream::AddInputProvider));
+    ON_CALL(*this, RemoveInputProvider(NotNull(), _))
         .WillByDefault(
-            Invoke(&real_, &VirtualAudioInputStream::RemoveOutputStream));
+            Invoke(&real_, &VirtualAudioInputStream::RemoveInputProvider));
   }
 
   ~MockVirtualAudioInputStream() {
@@ -137,10 +142,12 @@ class MockVirtualAudioInputStream : public VirtualAudioInputStream {
   MOCK_METHOD0(GetVolume, double());
   MOCK_METHOD1(SetAutomaticGainControl, bool(bool));
   MOCK_METHOD0(GetAutomaticGainControl, bool());
-  MOCK_METHOD2(AddOutputStream, void(VirtualAudioOutputStream*,
-                                     const AudioParameters&));
-  MOCK_METHOD2(RemoveOutputStream, void(VirtualAudioOutputStream*,
-                                        const AudioParameters&));
+  MOCK_METHOD2(AddInputProvider,
+               void(media::AudioConverter::InputCallback*,
+                    const AudioParameters&));
+  MOCK_METHOD2(RemoveInputProvider,
+               void(media::AudioConverter::InputCallback*,
+                    const AudioParameters&));
 
  private:
   void OnRealStreamHasClosed(VirtualAudioInputStream* stream) {
@@ -162,7 +169,7 @@ class MockAudioInputCallback : public AudioInputStream::AudioInputCallback {
   MOCK_METHOD4(OnData,
                void(AudioInputStream* stream,
                     const media::AudioBus* src,
-                    uint32 hardware_delay_bytes,
+                    uint32_t hardware_delay_bytes,
                     double volume));
   MOCK_METHOD1(OnError, void(AudioInputStream* stream));
 
@@ -172,7 +179,7 @@ class MockAudioInputCallback : public AudioInputStream::AudioInputCallback {
 
 }  // namespace
 
-class WebContentsAudioInputStreamTest : public testing::Test {
+class WebContentsAudioInputStreamTest : public testing::TestWithParam<bool> {
  public:
   WebContentsAudioInputStreamTest()
       : thread_bundle_(new TestBrowserThreadBundle(
@@ -185,7 +192,8 @@ class WebContentsAudioInputStreamTest : public testing::Test {
         destination_(NULL),
         current_render_process_id_(kRenderProcessId),
         current_render_frame_id_(kRenderFrameId),
-        on_data_event_(false, false) {
+        on_data_event_(base::WaitableEvent::ResetPolicy::AUTOMATIC,
+                       base::WaitableEvent::InitialState::NOT_SIGNALED) {
     audio_thread_.Start();
   }
 
@@ -200,6 +208,12 @@ class WebContentsAudioInputStreamTest : public testing::Test {
     DCHECK(sources_.empty());
   }
 
+  // If this value is true, we are testing a WebContentsAudioInputStream
+  // instance, which requests duplicate audio.
+  // Otherwise, we are testing a WebContentsAudioInputStream instance, which
+  // requests diverting audio.
+  bool is_duplication() const { return GetParam(); }
+
   void Open() {
     mock_vais_ = new MockVirtualAudioInputStream(audio_thread_.task_runner());
     EXPECT_CALL(*mock_vais_, Open());
@@ -219,8 +233,8 @@ class WebContentsAudioInputStreamTest : public testing::Test {
 
     wcais_ = new WebContentsAudioInputStream(
         current_render_process_id_, current_render_frame_id_,
-        mock_mirroring_manager_.get(),
-        mock_tracker_, mock_vais_);
+        mock_mirroring_manager_.get(), mock_tracker_, mock_vais_,
+        is_duplication());
     wcais_->Open();
   }
 
@@ -267,36 +281,62 @@ class WebContentsAudioInputStreamTest : public testing::Test {
     // Note: WCAIS posts a task to invoke
     // MockAudioMirroringManager::StartMirroring() on the IO thread, which
     // causes our mock to set |destination_|.  Block until that has happened.
-    base::WaitableEvent done(false, false);
+    base::WaitableEvent done(base::WaitableEvent::ResetPolicy::AUTOMATIC,
+                             base::WaitableEvent::InitialState::NOT_SIGNALED);
     BrowserThread::PostTask(
         BrowserThread::IO, FROM_HERE, base::Bind(
             &base::WaitableEvent::Signal, base::Unretained(&done)));
     done.Wait();
     ASSERT_TRUE(destination_);
 
-    EXPECT_CALL(*mock_vais_, AddOutputStream(NotNull(), _))
+    EXPECT_CALL(*mock_vais_, AddInputProvider(NotNull(), _))
         .RetiresOnSaturation();
     // Later, when stream is closed:
-    EXPECT_CALL(*mock_vais_, RemoveOutputStream(NotNull(), _))
+    EXPECT_CALL(*mock_vais_, RemoveInputProvider(NotNull(), _))
         .RetiresOnSaturation();
 
     const AudioParameters& params = TestAudioParameters();
-    AudioOutputStream* const out = destination_->AddInput(params);
-    ASSERT_TRUE(out);
-    streams_.push_back(out);
-    EXPECT_TRUE(out->Open());
     SineWaveAudioSource* const source = new SineWaveAudioSource(
         params.channel_layout(), 200.0, params.sample_rate());
     sources_.push_back(source);
-    out->Start(source);
+    if (is_duplication()) {
+      media::AudioPushSink* out = destination_->AddPushInput(params);
+      ASSERT_TRUE(out);
+      sinks_.push_back(out);
+      std::unique_ptr<media::AudioBus> audio_data = AudioBus::Create(params);
+      base::TimeTicks now = base::TimeTicks::Now();
+      // 20 Audio buses are enough for all test cases.
+      const int kAudioBusesNumber = 20;
+      for (int i = 0; i < kAudioBusesNumber; i++) {
+        int frames = source->OnMoreData(audio_data.get(), 0, 0);
+        std::unique_ptr<media::AudioBus> copy = AudioBus::Create(params);
+        audio_data->CopyTo(copy.get());
+        out->OnData(std::move(copy), now);
+        now += base::TimeDelta::FromMillisecondsD(
+            frames * params.GetMicrosecondsPerFrame());
+      }
+    } else {
+      AudioOutputStream* const out = destination_->AddInput(params);
+      ASSERT_TRUE(out);
+      streams_.push_back(out);
+      EXPECT_TRUE(out->Open());
+      out->Start(source);
+    }
   }
 
   void RemoveOneInputInFIFOOrder() {
-    ASSERT_FALSE(streams_.empty());
-    AudioOutputStream* const out = streams_.front();
-    streams_.pop_front();
-    out->Stop();
-    out->Close();  // Self-deletes.
+    if (is_duplication()) {
+      ASSERT_FALSE(sinks_.empty());
+      AudioPushSink* const out = sinks_.front();
+      sinks_.pop_front();
+      out->Close();  // Self-deletes.
+    } else {
+      ASSERT_FALSE(streams_.empty());
+      AudioOutputStream* const out = streams_.front();
+      streams_.pop_front();
+      out->Stop();
+      out->Close();  // Self-deletes.
+    }
     ASSERT_TRUE(!sources_.empty());
     delete sources_.front();
     sources_.pop_front();
@@ -358,10 +398,10 @@ class WebContentsAudioInputStreamTest : public testing::Test {
     change_callback_.Run(render_process_id != -1 && render_frame_id != -1);
   }
 
-  scoped_ptr<TestBrowserThreadBundle> thread_bundle_;
+  std::unique_ptr<TestBrowserThreadBundle> thread_bundle_;
   base::Thread audio_thread_;
 
-  scoped_ptr<MockAudioMirroringManager> mock_mirroring_manager_;
+  std::unique_ptr<MockAudioMirroringManager> mock_mirroring_manager_;
   scoped_refptr<MockWebContentsTracker> mock_tracker_;
 
   MockVirtualAudioInputStream* mock_vais_;  // Owned by wcais_.
@@ -387,6 +427,7 @@ class WebContentsAudioInputStreamTest : public testing::Test {
   // Streams provided by calls to WebContentsAudioInputStream::AddInput().  Each
   // is started with a simulated source of audio data.
   std::list<AudioOutputStream*> streams_;
+  std::list<media::AudioPushSink*> sinks_;
   std::list<SineWaveAudioSource*> sources_;  // 1:1 with elements in streams_.
 
   base::WaitableEvent on_data_event_;
@@ -398,12 +439,12 @@ class WebContentsAudioInputStreamTest : public testing::Test {
   RunOnAudioThread(base::Bind(&WebContentsAudioInputStreamTest::method,  \
                               base::Unretained(this)))
 
-TEST_F(WebContentsAudioInputStreamTest, OpenedButNeverStarted) {
+TEST_P(WebContentsAudioInputStreamTest, OpenedButNeverStarted) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Close);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringNothing) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringNothing) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   WaitForData();
@@ -411,7 +452,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringNothing) {
   RUN_ON_AUDIO_THREAD(Close);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringOutputOutlivesSession) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringOutputOutlivesSession) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(AddAnotherInput);
@@ -421,7 +462,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringOutputOutlivesSession) {
   RUN_ON_AUDIO_THREAD(RemoveOneInputInFIFOOrder);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringOutputWithinSession) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringOutputWithinSession) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(AddAnotherInput);
@@ -431,7 +472,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringOutputWithinSession) {
   RUN_ON_AUDIO_THREAD(Close);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringNothingWithTargetChange) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringNothingWithTargetChange) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(ChangeMirroringTarget);
@@ -439,7 +480,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringNothingWithTargetChange) {
   RUN_ON_AUDIO_THREAD(Close);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringOneStreamAfterTargetChange) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringOneStreamAfterTargetChange) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(ChangeMirroringTarget);
@@ -450,7 +491,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringOneStreamAfterTargetChange) {
   RUN_ON_AUDIO_THREAD(RemoveOneInputInFIFOOrder);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringOneStreamWithTargetChange) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringOneStreamWithTargetChange) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(AddAnotherInput);
@@ -464,7 +505,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringOneStreamWithTargetChange) {
   RUN_ON_AUDIO_THREAD(RemoveOneInputInFIFOOrder);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringLostTarget) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringLostTarget) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(AddAnotherInput);
@@ -475,7 +516,7 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringLostTarget) {
   RUN_ON_AUDIO_THREAD(Close);
 }
 
-TEST_F(WebContentsAudioInputStreamTest, MirroringMultipleStreamsAndTargets) {
+TEST_P(WebContentsAudioInputStreamTest, MirroringMultipleStreamsAndTargets) {
   RUN_ON_AUDIO_THREAD(Open);
   RUN_ON_AUDIO_THREAD(Start);
   RUN_ON_AUDIO_THREAD(AddAnotherInput);
@@ -499,4 +540,6 @@ TEST_F(WebContentsAudioInputStreamTest, MirroringMultipleStreamsAndTargets) {
   RUN_ON_AUDIO_THREAD(Close);
 }
 
+INSTANTIATE_TEST_CASE_P(, WebContentsAudioInputStreamTest, ::testing::Bool());
+
 }  // namespace content
