@@ -55,7 +55,7 @@ class AudioDiscarder : public media::AudioOutputStream {
 
   // Calls FetchAudioData() at regular intervals and discards the data.
   media::FakeAudioWorker worker_;
-  scoped_ptr<media::AudioBus> audio_bus_;
+  std::unique_ptr<media::AudioBus> audio_bus_;
 
   DISALLOW_COPY_AND_ASSIGN(AudioDiscarder);
 };
@@ -104,7 +104,7 @@ class WebContentsAudioMuter::MuteDestination
       if (contents_containing_frame == web_contents_)
         matches.insert(*i);
     }
-    results_callback.Run(matches);
+    results_callback.Run(matches, false);
   }
 
   media::AudioOutputStream* AddInput(
@@ -112,6 +112,12 @@ class WebContentsAudioMuter::MuteDestination
     return new AudioDiscarder(params);
   }
 
+  media::AudioPushSink* AddPushInput(
+      const media::AudioParameters& params) override {
+    NOTREACHED();
+    return nullptr;
+  }
+
   WebContents* const web_contents_;
 
   DISALLOW_COPY_AND_ASSIGN(MuteDestination);
@@ -133,11 +139,10 @@ void WebContentsAudioMuter::StartMuting() {
     return;
   is_muting_ = true;
   BrowserThread::PostTask(
-      BrowserThread::IO,
-      FROM_HERE,
+      BrowserThread::IO, FROM_HERE,
       base::Bind(&AudioMirroringManager::StartMirroring,
                  base::Unretained(AudioMirroringManager::GetInstance()),
-                 destination_));
+                 base::RetainedRef(destination_)));
 }
 
 void WebContentsAudioMuter::StopMuting() {
@@ -146,11 +151,10 @@ void WebContentsAudioMuter::StopMuting() {
     return;
   is_muting_ = false;
   BrowserThread::PostTask(
-      BrowserThread::IO,
-      FROM_HERE,
+      BrowserThread::IO, FROM_HERE,
       base::Bind(&AudioMirroringManager::StopMirroring,
                  base::Unretained(AudioMirroringManager::GetInstance()),
-                 destination_));
+                 base::RetainedRef(destination_)));
 }
 
 }  // namespace content
