@@ -4,7 +4,8 @@
 
 #include "content/browser/net/view_blob_internals_job_factory.h"
 
-#include "base/memory/scoped_ptr.h"
+#include <memory>
+
 #include "base/strings/string_util.h"
 #include "content/public/common/url_constants.h"
 #include "storage/browser/blob/view_blob_internals_job.h"
