@@ -2,22 +2,19 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "content/browser/renderer_host/input/touch_emulator.h"
+
+#include <stddef.h>
+
+#include <memory>
 #include <vector>
 
-#include "base/basictypes.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "base/time/time.h"
-#include "content/browser/renderer_host/input/touch_emulator.h"
 #include "content/browser/renderer_host/input/touch_emulator_client.h"
 #include "content/common/input/web_input_event_traits.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
-#if defined(USE_AURA)
-#include "ui/aura/env.h"
-#include "ui/aura/test/test_screen.h"
-#endif
-
 using blink::WebGestureEvent;
 using blink::WebInputEvent;
 using blink::WebKeyboardEvent;
@@ -46,13 +43,7 @@ class TouchEmulatorTest : public testing::Test,
 
   // testing::Test
   void SetUp() override {
-#if defined(USE_AURA)
-    aura::Env::CreateInstance(true);
-    screen_.reset(aura::TestScreen::Create(gfx::Size()));
-    gfx::Screen::SetScreenInstance(gfx::SCREEN_TYPE_NATIVE, screen_.get());
-#endif
-
-    emulator_.reset(new TouchEmulator(this));
+    emulator_.reset(new TouchEmulator(this, 1.0f));
     emulator_->SetDoubleTapSupportForPageEnabled(false);
     emulator_->Enable(ui::GestureProviderConfigType::GENERIC_MOBILE);
   }
@@ -60,15 +51,10 @@ class TouchEmulatorTest : public testing::Test,
   void TearDown() override {
     emulator_->Disable();
     EXPECT_EQ("", ExpectedEvents());
-
-#if defined(USE_AURA)
-    aura::Env::DeleteInstance();
-    gfx::Screen::SetScreenInstance(gfx::SCREEN_TYPE_NATIVE, nullptr);
-    screen_.reset();
-#endif
   }
 
-  void ForwardGestureEvent(const blink::WebGestureEvent& event) override {
+  void ForwardEmulatedGestureEvent(
+      const blink::WebGestureEvent& event) override {
     forwarded_events_.push_back(event.type);
   }
 
@@ -77,8 +63,14 @@ class TouchEmulatorTest : public testing::Test,
     EXPECT_EQ(1U, event.touchesLength);
     EXPECT_EQ(last_mouse_x_, event.touches[0].position.x);
     EXPECT_EQ(last_mouse_y_, event.touches[0].position.y);
-    bool expected_cancelable = event.type != WebInputEvent::TouchCancel;
-    EXPECT_EQ(expected_cancelable, !!event.cancelable);
+    const int all_buttons = WebInputEvent::LeftButtonDown |
+        WebInputEvent::MiddleButtonDown | WebInputEvent::RightButtonDown;
+    EXPECT_EQ(0, event.modifiers & all_buttons);
+    WebInputEvent::DispatchType expected_dispatch_type =
+        event.type == WebInputEvent::TouchCancel
+            ? WebInputEvent::EventNonBlocking
+            : WebInputEvent::Blocking;
+    EXPECT_EQ(expected_dispatch_type, event.dispatchType);
     if (ack_touches_synchronously_) {
       emulator()->HandleTouchEventAck(
           event, INPUT_EVENT_ACK_STATE_NO_CONSUMER_EXISTS);
@@ -95,7 +87,8 @@ class TouchEmulatorTest : public testing::Test,
   }
 
   int modifiers() const {
-    return shift_pressed_ ? WebInputEvent::ShiftKey : 0;
+    return (shift_pressed_ ? WebInputEvent::ShiftKey : 0) |
+        (mouse_pressed_ ? WebInputEvent::LeftButtonDown : 0);
   }
 
   std::string ExpectedEvents() {
@@ -249,11 +242,8 @@ class TouchEmulatorTest : public testing::Test,
   void DisableSynchronousTouchAck() { ack_touches_synchronously_ = false; }
 
  private:
-  scoped_ptr<TouchEmulator> emulator_;
+  std::unique_ptr<TouchEmulator> emulator_;
   std::vector<WebInputEvent::Type> forwarded_events_;
-#if defined(USE_AURA)
-  scoped_ptr<gfx::Screen> screen_;
-#endif
   double last_event_time_seconds_;
   double event_time_delta_seconds_;
   bool shift_pressed_;
@@ -575,4 +565,8 @@ TEST_F(TouchEmulatorTest, CancelAfterDisableDoesNotCrash) {
   emulator()->CancelTouch();
 }
 
+TEST_F(TouchEmulatorTest, ConstructorWithHighDeviceScaleDoesNotCrash) {
+  TouchEmulator(this, 4.0f);
+}
+
 }  // namespace content
