@@ -6,9 +6,9 @@
 #include "build/build_config.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
+#include "ui/display/display.h"
+#include "ui/display/display_switches.h"
 #include "ui/events/event_constants.h"
-#include "ui/gfx/display.h"
-#include "ui/gfx/switches.h"
 
 #if defined(OS_WIN)
 #include "base/win/windows_version.h"
@@ -27,15 +27,15 @@ TEST(WebInputEventBuilderTest, TestMouseEventScale) {
   if (base::win::GetVersion() < base::win::VERSION_WIN7)
     return;
 
-  gfx::Display::ResetForceDeviceScaleFactorForTesting();
+  display::Display::ResetForceDeviceScaleFactorForTesting();
 
   base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
   command_line->AppendSwitchASCII(switches::kForceDeviceScaleFactor, "2");
 
   // Synthesize a mouse move with x = 300 and y = 200.
-  WebMouseEvent mouse_move =
-      WebMouseEventBuilder::Build(::GetDesktopWindow(), WM_MOUSEMOVE, 0,
-                                  MAKELPARAM(300, 200), 100);
+  WebMouseEvent mouse_move = WebMouseEventBuilder::Build(
+      ::GetDesktopWindow(), WM_MOUSEMOVE, 0, MAKELPARAM(300, 200), 100,
+      blink::WebPointerProperties::PointerType::Mouse);
 
   // The WebMouseEvent.x, WebMouseEvent.y, WebMouseEvent.windowX and
   // WebMouseEvent.windowY fields should be in pixels on return and hence
@@ -51,8 +51,11 @@ TEST(WebInputEventBuilderTest, TestMouseEventScale) {
   EXPECT_EQ(150, mouse_move.globalX);
   EXPECT_EQ(100, mouse_move.globalY);
 
+  EXPECT_EQ(blink::WebPointerProperties::PointerType::Mouse,
+            mouse_move.pointerType);
+
   command_line->AppendSwitchASCII(switches::kForceDeviceScaleFactor, "1");
-  gfx::Display::ResetForceDeviceScaleFactorForTesting();
+  display::Display::ResetForceDeviceScaleFactorForTesting();
 }
 #endif
 
