@@ -15,7 +15,8 @@
 #include "base/location.h"
 #include "base/run_loop.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/system_monitor/system_monitor.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "build/build_config.h"
 #include "content/browser/browser_thread_impl.h"
 #include "content/browser/renderer_host/media/audio_input_device_manager.h"
@@ -38,6 +39,8 @@
 #include "net/url_request/url_request_context.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
+#include "url/gurl.h"
+#include "url/origin.h"
 
 #if defined(OS_CHROMEOS)
 #include "chromeos/audio/cras_audio_handler.h"
@@ -60,10 +63,10 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
                                       public TestContentBrowserClient {
  public:
   MockMediaStreamDispatcherHost(
-      const ResourceContext::SaltCallback salt_callback,
+      const std::string& salt,
       const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
       MediaStreamManager* manager)
-      : MediaStreamDispatcherHost(kProcessId, salt_callback, manager),
+      : MediaStreamDispatcherHost(kProcessId, salt, manager, true),
         task_runner_(task_runner),
         current_ipc_(NULL) {}
 
@@ -81,7 +84,7 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
   void OnGenerateStream(int render_frame_id,
                         int page_request_id,
                         const StreamControls& controls,
-                        const GURL& security_origin,
+                        const url::Origin& security_origin,
                         const base::Closure& quit_closure) {
     quit_closures_.push(quit_closure);
     MediaStreamDispatcherHost::OnGenerateStream(
@@ -97,7 +100,7 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
                     int page_request_id,
                     const std::string& device_id,
                     MediaStreamType type,
-                    const GURL& security_origin,
+                    const url::Origin& security_origin,
                     const base::Closure& quit_closure) {
     quit_closures_.push(quit_closure);
     MediaStreamDispatcherHost::OnOpenDevice(
@@ -107,13 +110,27 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
   void OnEnumerateDevices(int render_frame_id,
                           int page_request_id,
                           MediaStreamType type,
-                          const GURL& security_origin,
+                          const url::Origin& security_origin,
                           const base::Closure& quit_closure) {
     quit_closures_.push(quit_closure);
     MediaStreamDispatcherHost::OnEnumerateDevices(
         render_frame_id, page_request_id, type, security_origin);
   }
 
+  void OnCancelEnumerateDevices(int render_frame_id, int page_request_id) {
+    MediaStreamDispatcherHost::OnCancelEnumerateDevices(render_frame_id,
+                                                        page_request_id);
+  }
+
+  void OnSubscribeToDeviceChangeNotifications(
+      int render_frame_id,
+      const url::Origin& security_origin,
+      const base::Closure& quit_closure) {
+    quit_closures_.push(quit_closure);
+    MediaStreamDispatcherHost::OnSubscribeToDeviceChangeNotifications(
+        render_frame_id, security_origin);
+  }
+
   std::string label_;
   StreamDeviceInfoArray audio_devices_;
   StreamDeviceInfoArray video_devices_;
@@ -130,8 +147,8 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
     CHECK(message);
     current_ipc_ = message;
 
-    // In this method we dispatch the messages to the according handlers as if
-    // we are the renderer.
+    // In this method we dispatch the messages to the corresponding handlers as
+    // if we are the renderer.
     bool handled = true;
     IPC_BEGIN_MESSAGE_MAP(MockMediaStreamDispatcherHost, *message)
       IPC_MESSAGE_HANDLER(MediaStreamMsg_StreamGenerated,
@@ -141,6 +158,7 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
       IPC_MESSAGE_HANDLER(MediaStreamMsg_DeviceStopped, OnDeviceStoppedInternal)
       IPC_MESSAGE_HANDLER(MediaStreamMsg_DeviceOpened, OnDeviceOpenedInternal)
       IPC_MESSAGE_HANDLER(MediaStreamMsg_DevicesEnumerated, OnDevicesEnumerated)
+      IPC_MESSAGE_HANDLER(MediaStreamMsg_DevicesChanged, OnDevicesChanged)
       IPC_MESSAGE_UNHANDLED(handled = false)
     IPC_END_MESSAGE_MAP()
     EXPECT_TRUE(handled);
@@ -178,7 +196,7 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
       task_runner_->PostTask(FROM_HERE, base::ResetAndReturn(&quit_closure));
     }
 
-    label_= "";
+    label_ = "";
   }
 
   void OnDeviceStoppedInternal(const std::string& label,
@@ -209,6 +227,12 @@ class MockMediaStreamDispatcherHost : public MediaStreamDispatcherHost,
     enumerated_devices_ = devices;
   }
 
+  void OnDevicesChanged() {
+    base::Closure quit_closure = quit_closures_.front();
+    quit_closures_.pop();
+    task_runner_->PostTask(FROM_HERE, base::ResetAndReturn(&quit_closure));
+  }
+
   const scoped_refptr<base::SingleThreadTaskRunner> task_runner_;
   IPC::Message* current_ipc_;
   std::queue<base::Closure> quit_closures_;
@@ -225,9 +249,9 @@ class MockMediaStreamUIProxy : public FakeMediaStreamUIProxy {
 class MediaStreamDispatcherHostTest : public testing::Test {
  public:
   MediaStreamDispatcherHostTest()
-      : old_browser_client_(NULL),
-        thread_bundle_(content::TestBrowserThreadBundle::IO_MAINLOOP),
-        origin_("https://test.com") {
+      : thread_bundle_(content::TestBrowserThreadBundle::IO_MAINLOOP),
+        old_browser_client_(NULL),
+        origin_(GURL("https://test.com")) {
     audio_manager_.reset(
         new media::MockAudioManager(base::ThreadTaskRunnerHandle::Get()));
     // Make sure we use fake devices to avoid long delays.
@@ -288,7 +312,7 @@ class MediaStreamDispatcherHostTest : public testing::Test {
       EXPECT_CALL(*stream_ui_, OnStarted(_, _));
     }
     media_stream_manager_->UseFakeUIForTests(
-      scoped_ptr<FakeMediaStreamUIProxy>(stream_ui_));
+        std::unique_ptr<FakeMediaStreamUIProxy>(stream_ui_));
   }
 
   void GenerateStreamAndWaitForResult(int render_frame_id,
@@ -349,6 +373,20 @@ class MediaStreamDispatcherHostTest : public testing::Test {
     ASSERT_FALSE(host_->enumerated_devices_.empty());
     EXPECT_FALSE(DoesContainRawIds(host_->enumerated_devices_));
     EXPECT_TRUE(DoesEveryDeviceMapToRawId(host_->enumerated_devices_, origin_));
+    // Enumeration requests must be cancelled manually.
+    host_->OnCancelEnumerateDevices(render_frame_id, page_request_id);
+  }
+
+  void SubscribeToDeviceChangeNotificationsAndWaitForNotification(
+      int render_frame_id) {
+    base::RunLoop run_loop;
+    host_->OnSubscribeToDeviceChangeNotifications(render_frame_id, origin_,
+                                                  run_loop.QuitClosure());
+    // Simulate a change in the set of devices.
+    video_capture_device_factory_->set_number_of_devices(5);
+    media_stream_manager_->OnDevicesChanged(
+        base::SystemMonitor::DEVTYPE_VIDEO_CAPTURE);
+    run_loop.Run();
   }
 
   bool DoesContainRawIds(const StreamDeviceInfoArray& devices) {
@@ -370,7 +408,7 @@ class MediaStreamDispatcherHostTest : public testing::Test {
   }
 
   bool DoesEveryDeviceMapToRawId(const StreamDeviceInfoArray& devices,
-                                 const GURL& origin) {
+                                 const url::Origin& origin) {
     for (size_t i = 0; i < devices.size(); ++i) {
       bool found_match = false;
       media::AudioDeviceNames::const_iterator audio_it =
@@ -422,16 +460,17 @@ class MediaStreamDispatcherHostTest : public testing::Test {
   }
 
   scoped_refptr<MockMediaStreamDispatcherHost> host_;
-  scoped_ptr<media::AudioManager> audio_manager_;
-  scoped_ptr<MediaStreamManager> media_stream_manager_;
+  std::unique_ptr<MediaStreamManager> media_stream_manager_;
+  content::TestBrowserThreadBundle thread_bundle_;
+  std::unique_ptr<media::AudioManager, media::AudioManagerDeleter>
+      audio_manager_;
   MockMediaStreamUIProxy* stream_ui_;
   ContentBrowserClient* old_browser_client_;
-  scoped_ptr<ContentClient> content_client_;
-  content::TestBrowserThreadBundle thread_bundle_;
+  std::unique_ptr<ContentClient> content_client_;
   content::TestBrowserContext browser_context_;
   media::AudioDeviceNames physical_audio_devices_;
   media::VideoCaptureDevice::Names physical_video_devices_;
-  GURL origin_;
+  url::Origin origin_;
   media::FakeVideoCaptureDeviceFactory* video_capture_device_factory_;
 };
 
@@ -846,7 +885,8 @@ TEST_F(MediaStreamDispatcherHostTest, CloseFromUI) {
   StreamControls controls(false, true);
 
   base::Closure close_callback;
-  scoped_ptr<MockMediaStreamUIProxy> stream_ui(new MockMediaStreamUIProxy());
+  std::unique_ptr<MockMediaStreamUIProxy> stream_ui(
+      new MockMediaStreamUIProxy());
   EXPECT_CALL(*stream_ui, OnStarted(_, _))
       .WillOnce(SaveArg<0>(&close_callback));
   media_stream_manager_->UseFakeUIForTests(std::move(stream_ui));
@@ -912,4 +952,12 @@ TEST_F(MediaStreamDispatcherHostTest, EnumerateVideoDevicesNoAccess) {
   EXPECT_TRUE(DoesNotContainLabels(host_->enumerated_devices_));
 }
 
+TEST_F(MediaStreamDispatcherHostTest, DeviceChangeNotification) {
+  SetupFakeUI(false);
+  // warm up the cache
+  EnumerateDevicesAndWaitForResult(kRenderId, kPageRequestId,
+                                   MEDIA_DEVICE_VIDEO_CAPTURE);
+  SubscribeToDeviceChangeNotificationsAndWaitForNotification(kRenderId);
+}
+
 };  // namespace content
