@@ -41,17 +41,18 @@
 #define CONTENT_BROWSER_RENDERER_HOST_MEDIA_VIDEO_CAPTURE_CONTROLLER_H_
 
 #include <list>
+#include <memory>
 
 #include "base/compiler_specific.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/process/process.h"
 #include "content/browser/renderer_host/media/video_capture_controller_event_handler.h"
 #include "content/common/content_export.h"
 #include "content/common/media/video_capture.h"
 #include "media/base/video_capture_types.h"
-#include "media/video/capture/video_capture_device.h"
+#include "media/capture/video/video_capture_device.h"
 
 namespace content {
 class VideoCaptureBufferPool;
@@ -69,10 +70,8 @@ class CONTENT_EXPORT VideoCaptureController {
   base::WeakPtr<VideoCaptureController> GetWeakPtrForIOThread();
 
   // Return a new VideoCaptureDeviceClient to forward capture events to this
-  // instance. Some device clients need to allocate resources for the given
-  // capture |format| and/or work on Capture Thread (|capture_task_runner|).
-  scoped_ptr<media::VideoCaptureDevice::Client> NewDeviceClient(
-      const scoped_refptr<base::SingleThreadTaskRunner>& capture_task_runner);
+  // instance.
+  std::unique_ptr<media::VideoCaptureDevice::Client> NewDeviceClient();
 
   // Start video capturing and try to use the resolution specified in |params|.
   // Buffers will be shared to the client as necessary. The client will continue
@@ -90,15 +89,21 @@ class CONTENT_EXPORT VideoCaptureController {
   int RemoveClient(VideoCaptureControllerID id,
                    VideoCaptureControllerEventHandler* event_handler);
 
-  // Pause or resume the video capture for specified client.
-  void PauseOrResumeClient(VideoCaptureControllerID id,
-                           VideoCaptureControllerEventHandler* event_handler,
-                           bool pause);
+  // Pause the video capture for specified client.
+  void PauseClient(VideoCaptureControllerID id,
+                   VideoCaptureControllerEventHandler* event_handler);
+  // Resume the video capture for specified client.
+  // Returns true if the client will be resumed.
+  bool ResumeClient(VideoCaptureControllerID id,
+                    VideoCaptureControllerEventHandler* event_handler);
 
   int GetClientCount() const;
 
-  // Return the number of clients that aren't paused.
-  int GetActiveClientCount() const;
+  // Return true if there is client that isn't paused.
+  bool HasActiveClient() const;
+
+  // Return true if there is client paused.
+  bool HasPausedClient() const;
 
   // API called directly by VideoCaptureManager in case the device is
   // prematurely closed.
@@ -106,14 +111,14 @@ class CONTENT_EXPORT VideoCaptureController {
 
   // Return a buffer with id |buffer_id| previously given in
   // VideoCaptureControllerEventHandler::OnBufferReady. In the case that the
-  // buffer was backed by a texture, |sync_point| will be waited on before
+  // buffer was backed by a texture, |sync_token| will be waited on before
   // destroying or recycling the texture, to synchronize with texture users in
   // the renderer process. If the consumer provided resource utilization
   // feedback, this will be passed here (-1.0 indicates no feedback).
   void ReturnBuffer(VideoCaptureControllerID id,
                     VideoCaptureControllerEventHandler* event_handler,
                     int buffer_id,
-                    uint32 sync_point,
+                    const gpu::SyncToken& sync_token,
                     double consumer_resource_utilization);
 
   const media::VideoCaptureFormat& GetVideoCaptureFormat() const;
@@ -121,18 +126,22 @@ class CONTENT_EXPORT VideoCaptureController {
   bool has_received_frames() const { return has_received_frames_; }
 
   // Worker functions on IO thread. Called by the VideoCaptureDeviceClient.
-  void DoIncomingCapturedVideoFrameOnIOThread(
-      scoped_ptr<media::VideoCaptureDevice::Client::Buffer> buffer,
-      const scoped_refptr<media::VideoFrame>& frame,
-      const base::TimeTicks& timestamp);
-  void DoErrorOnIOThread();
-  void DoLogOnIOThread(const std::string& message);
-  void DoBufferDestroyedOnIOThread(int buffer_id_to_drop);
+  virtual void DoIncomingCapturedVideoFrameOnIOThread(
+      std::unique_ptr<media::VideoCaptureDevice::Client::Buffer> buffer,
+      const scoped_refptr<media::VideoFrame>& frame);
+  virtual void DoErrorOnIOThread();
+  virtual void DoLogOnIOThread(const std::string& message);
+  virtual void DoBufferDestroyedOnIOThread(int buffer_id_to_drop);
 
  private:
   struct ControllerClient;
   typedef std::list<ControllerClient*> ControllerClients;
 
+  // Notify renderer that a new buffer has been created.
+  void DoNewBufferOnIOThread(ControllerClient* client,
+                             media::VideoCaptureDevice::Client::Buffer* buffer,
+                             const scoped_refptr<media::VideoFrame>& frame);
+
   // Find a client of |id| and |handler| in |clients|.
   ControllerClient* FindClient(VideoCaptureControllerID id,
                                VideoCaptureControllerEventHandler* handler,
