@@ -60,7 +60,7 @@ class FakeServerSocket : public net::ServerSocket {
     return net::OK;
   }
 
-  int Accept(scoped_ptr<net::StreamSocket>* socket,
+  int Accept(std::unique_ptr<net::StreamSocket>* socket,
              const net::CompletionCallback& callback) override {
     DCHECK(socket);
     if (!incoming_sockets_.empty()) {
@@ -79,7 +79,7 @@ class FakeServerSocket : public net::ServerSocket {
 
   net::IPEndPoint local_address_;
 
-  scoped_ptr<net::StreamSocket>* accept_socket_;
+  std::unique_ptr<net::StreamSocket>* accept_socket_;
   net::CompletionCallback accept_callback_;
 
   std::list<net::StreamSocket*> incoming_sockets_;
@@ -118,7 +118,7 @@ class P2PSocketHostTcpServerTest : public testing::Test {
 
   MockIPCSender sender_;
   FakeServerSocket* socket_;  // Owned by |socket_host_|.
-  scoped_ptr<P2PSocketHostTcpServer> socket_host_;
+  std::unique_ptr<P2PSocketHostTcpServer> socket_host_;
 };
 
 // Accept incoming connection.
@@ -134,7 +134,7 @@ TEST_F(P2PSocketHostTcpServerTest, Accept) {
 
   const int kAcceptedSocketId = 1;
 
-  scoped_ptr<P2PSocketHost> new_host(
+  std::unique_ptr<P2PSocketHost> new_host(
       socket_host_->AcceptIncomingTcpConnection(addr, kAcceptedSocketId));
   ASSERT_TRUE(new_host.get() != NULL);
   EXPECT_EQ(incoming, GetSocketFormTcpSocketHost(
@@ -162,12 +162,12 @@ TEST_F(P2PSocketHostTcpServerTest, Accept2) {
   const int kAcceptedSocketId1 = 1;
   const int kAcceptedSocketId2 = 2;
 
-  scoped_ptr<P2PSocketHost> new_host1(
+  std::unique_ptr<P2PSocketHost> new_host1(
       socket_host_->AcceptIncomingTcpConnection(addr1, kAcceptedSocketId1));
   ASSERT_TRUE(new_host1.get() != NULL);
   EXPECT_EQ(incoming1, GetSocketFormTcpSocketHost(
       reinterpret_cast<P2PSocketHostTcp*>(new_host1.get())));
-  scoped_ptr<P2PSocketHost> new_host2(
+  std::unique_ptr<P2PSocketHost> new_host2(
       socket_host_->AcceptIncomingTcpConnection(addr2, kAcceptedSocketId2));
   ASSERT_TRUE(new_host2.get() != NULL);
   EXPECT_EQ(incoming2, GetSocketFormTcpSocketHost(
