@@ -36,6 +36,10 @@ FrameTree* RenderViewHostDelegate::GetFrameTree() {
   return NULL;
 }
 
+double RenderViewHostDelegate::GetPendingPageZoomLevel() {
+  return 0.0;
+}
+
 bool RenderViewHostDelegate::IsNeverVisible() {
   return false;
 }
@@ -44,4 +48,8 @@ bool RenderViewHostDelegate::IsVirtualKeyboardRequested() {
   return false;
 }
 
+bool RenderViewHostDelegate::IsOverridingUserAgent() {
+  return false;
+}
+
 }  // namespace content
