@@ -4,10 +4,12 @@
 
 #include "content/browser/renderer_host/ui_events_helper.h"
 
+#include <stdint.h>
+
 #include "content/browser/renderer_host/input/web_input_event_util.h"
 #include "content/common/input/web_touch_event_traits.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
-#include "ui/events/blink/blink_event_util.h"
+#include "ui/events/base_event_utils.h"
 #include "ui/events/event.h"
 #include "ui/events/event_constants.h"
 
@@ -62,8 +64,8 @@ bool MakeUITouchEventsFromWebTouchEvents(
   }
 
   int flags = WebEventModifiersToEventFlags(touch.modifiers);
-  base::TimeDelta timestamp = base::TimeDelta::FromMicroseconds(
-      static_cast<int64>(touch.timeStampSeconds * 1000000));
+  base::TimeTicks timestamp =
+      ui::EventTimeStampFromSeconds(touch.timeStampSeconds);
   for (unsigned i = 0; i < touch.touchesLength; ++i) {
     const blink::WebTouchPoint& point = touch.touches[i];
     if (WebTouchPointStateToEventType(point.state) != type)
@@ -74,26 +76,15 @@ bool MakeUITouchEventsFromWebTouchEvents(
       location = point.position;
     else
       location = point.screenPosition;
-    ui::TouchEvent* uievent = new ui::TouchEvent(type,
-          location,
-          flags,
-          point.id,
-          timestamp,
-          point.radiusX,
-          point.radiusY,
-          point.rotationAngle,
-          point.force);
+    ui::TouchEvent* uievent = new ui::TouchEvent(
+        type, gfx::Point(), flags, point.id, timestamp, point.radiusX,
+        point.radiusY, point.rotationAngle, point.force);
+    uievent->set_location_f(location);
+    uievent->set_root_location_f(location);
     uievent->set_latency(touch_with_latency.latency);
     list->push_back(uievent);
   }
   return true;
 }
 
-blink::WebGestureEvent MakeWebGestureEventFromUIEvent(
-    const ui::GestureEvent& event) {
-  return ui::CreateWebGestureEvent(event.details(), event.time_stamp(),
-                                   event.location_f(), event.root_location_f(),
-                                   event.flags());
-}
-
 }  // namespace content
