@@ -6,9 +6,9 @@
 
 #include "build/build_config.h"
 #include "content/browser/renderer_host/input/web_input_event_util.h"
-#include "content/browser/renderer_host/ui_events_helper.h"
 #include "ui/aura/client/screen_position_client.h"
 #include "ui/aura/window.h"
+#include "ui/events/base_event_utils.h"
 #include "ui/events/blink/blink_event_util.h"
 #include "ui/events/event.h"
 #include "ui/events/event_utils.h"
@@ -51,28 +51,41 @@ blink::WebPointerProperties::PointerType EventPointerTypeToWebPointerType(
   return blink::WebPointerProperties::PointerType::Unknown;
 }
 
+// Creates a WebGestureEvent from a ui::GestureEvent. Note that it does not
+// populate the event coordinates (i.e. |x|, |y|, |globalX|, and |globalY|). So
+// the caller must populate these fields.
+blink::WebGestureEvent MakeWebGestureEventFromUIEvent(
+    const ui::GestureEvent& event) {
+  return ui::CreateWebGestureEvent(
+      event.details(), event.time_stamp(), event.location_f(),
+      event.root_location_f(), event.flags(), event.unique_touch_event_id());
+}
+
 }  // namespace
 
 #if defined(OS_WIN)
 blink::WebMouseEvent MakeUntranslatedWebMouseEventFromNativeEvent(
     const base::NativeEvent& native_event,
-    const base::TimeDelta& time_stamp);
+    const base::TimeTicks& time_stamp,
+    blink::WebPointerProperties::PointerType pointer_type);
 blink::WebMouseWheelEvent MakeUntranslatedWebMouseWheelEventFromNativeEvent(
     const base::NativeEvent& native_event,
-    const base::TimeDelta& time_stamp);
+    const base::TimeTicks& time_stamp,
+    blink::WebPointerProperties::PointerType pointer_type);
 blink::WebKeyboardEvent MakeWebKeyboardEventFromNativeEvent(
     const base::NativeEvent& native_event,
-    const base::TimeDelta& time_stamp);
+    const base::TimeTicks& time_stamp);
 blink::WebGestureEvent MakeWebGestureEventFromNativeEvent(
     const base::NativeEvent& native_event,
-    const base::TimeDelta& time_stamp);
+    const base::TimeTicks& time_stamp);
 #endif
 
 blink::WebKeyboardEvent MakeWebKeyboardEventFromAuraEvent(
     const ui::KeyEvent& event) {
   blink::WebKeyboardEvent webkit_event;
 
-  webkit_event.timeStampSeconds = event.time_stamp().InSecondsF();
+  webkit_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(event.time_stamp());
   webkit_event.modifiers = ui::EventFlagsToWebEventModifiers(event.flags()) |
                            DomCodeToWebInputEventModifiers(event.code());
 
@@ -115,7 +128,8 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEventFromAuraEvent(
   webkit_event.type = blink::WebInputEvent::MouseWheel;
   webkit_event.button = blink::WebMouseEvent::ButtonNone;
   webkit_event.modifiers = ui::EventFlagsToWebEventModifiers(event.flags());
-  webkit_event.timeStampSeconds = event.time_stamp().InSecondsF();
+  webkit_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(event.time_stamp());
   webkit_event.hasPreciseScrollingDeltas = true;
 
   float offset_ordinal_x = 0.f;
@@ -138,6 +152,9 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEventFromAuraEvent(
   webkit_event.wheelTicksY = webkit_event.deltaY / kPixelsPerTick;
   if (offset_ordinal_y != 0.f && webkit_event.deltaY != 0.f)
     webkit_event.accelerationRatioY = offset_ordinal_y / webkit_event.deltaY;
+
+  webkit_event.pointerType =
+      EventPointerTypeToWebPointerType(event.pointer_details().pointer_type);
   return webkit_event;
 }
 
@@ -163,7 +180,8 @@ blink::WebGestureEvent MakeWebGestureEventFromAuraEvent(
 
   webkit_event.sourceDevice = blink::WebGestureDeviceTouchpad;
   webkit_event.modifiers = ui::EventFlagsToWebEventModifiers(event.flags());
-  webkit_event.timeStampSeconds = event.time_stamp().InSecondsF();
+  webkit_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(event.time_stamp());
   return webkit_event;
 }
 
@@ -202,8 +220,10 @@ blink::WebMouseEvent MakeWebMouseEvent(const ui::MouseEvent& event) {
       // On Windows we have WM_ events comming from desktop and pure aura
       // events comming from metro mode.
       event.native_event().message && (event.type() != ui::ET_MOUSE_EXITED)
-          ? MakeUntranslatedWebMouseEventFromNativeEvent(event.native_event(),
-                                                         event.time_stamp())
+          ? MakeUntranslatedWebMouseEventFromNativeEvent(
+                event.native_event(), event.time_stamp(),
+                EventPointerTypeToWebPointerType(
+                    event.pointer_details().pointer_type))
           : MakeWebMouseEventFromAuraEvent(event);
 #else
       MakeWebMouseEventFromAuraEvent(event);
@@ -231,7 +251,9 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(
   blink::WebMouseWheelEvent webkit_event =
       event.native_event().message
           ? MakeUntranslatedWebMouseWheelEventFromNativeEvent(
-                event.native_event(), event.time_stamp())
+                event.native_event(), event.time_stamp(),
+                EventPointerTypeToWebPointerType(
+                    event.pointer_details().pointer_type))
           : MakeWebMouseWheelEventFromAuraEvent(event);
 #else
   blink::WebMouseWheelEvent webkit_event =
@@ -247,14 +269,6 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(
   webkit_event.globalX = screen_point.x();
   webkit_event.globalY = screen_point.y();
 
-  // Scroll events generated from the mouse wheel when the control key is held
-  // don't trigger scrolling. Instead, they may cause zooming.
-  bool from_mouse_wheel = !webkit_event.hasPreciseScrollingDeltas;
-  if ((webkit_event.modifiers & blink::WebInputEvent::ControlKey) &&
-      from_mouse_wheel) {
-    webkit_event.canScroll = false;
-  }
-
   return webkit_event;
 }
 
@@ -264,7 +278,9 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEvent(const ui::ScrollEvent& event) {
   blink::WebMouseWheelEvent webkit_event =
       event.native_event().message
           ? MakeUntranslatedWebMouseWheelEventFromNativeEvent(
-                event.native_event(), event.time_stamp())
+                event.native_event(), event.time_stamp(),
+                EventPointerTypeToWebPointerType(
+                    event.pointer_details().pointer_type))
           : MakeWebMouseWheelEventFromAuraEvent(event);
 #else
   blink::WebMouseWheelEvent webkit_event =
@@ -351,6 +367,8 @@ blink::WebGestureEvent MakeWebGestureEventFlingCancel() {
 
   // All other fields are ignored on a GestureFlingCancel event.
   gesture_event.type = blink::WebInputEvent::GestureFlingCancel;
+  gesture_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(ui::EventTimeForNow());
   gesture_event.sourceDevice = blink::WebGestureDeviceTouchpad;
   return gesture_event;
 }
@@ -360,8 +378,8 @@ blink::WebMouseEvent MakeWebMouseEventFromAuraEvent(
   blink::WebMouseEvent webkit_event;
 
   webkit_event.modifiers = ui::EventFlagsToWebEventModifiers(event.flags());
-  webkit_event.timeStampSeconds = event.time_stamp().InSecondsF();
-
+  webkit_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(event.time_stamp());
   webkit_event.button = blink::WebMouseEvent::ButtonNone;
   int button_flags = event.flags();
   if (event.type() == ui::ET_MOUSE_PRESSED ||
@@ -399,11 +417,11 @@ blink::WebMouseEvent MakeWebMouseEventFromAuraEvent(
       break;
   }
 
-  webkit_event.tiltX = roundf(event.pointer_details().tilt_x());
-  webkit_event.tiltY = roundf(event.pointer_details().tilt_y());
-  webkit_event.force = event.pointer_details().force();
+  webkit_event.tiltX = roundf(event.pointer_details().tilt_x);
+  webkit_event.tiltY = roundf(event.pointer_details().tilt_y);
+  webkit_event.force = event.pointer_details().force;
   webkit_event.pointerType =
-      EventPointerTypeToWebPointerType(event.pointer_details().pointer_type());
+      EventPointerTypeToWebPointerType(event.pointer_details().pointer_type);
 
   return webkit_event;
 }
@@ -415,7 +433,8 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEventFromAuraEvent(
   webkit_event.type = blink::WebInputEvent::MouseWheel;
   webkit_event.button = blink::WebMouseEvent::ButtonNone;
   webkit_event.modifiers = ui::EventFlagsToWebEventModifiers(event.flags());
-  webkit_event.timeStampSeconds = event.time_stamp().InSecondsF();
+  webkit_event.timeStampSeconds =
+      ui::EventTimeStampToSeconds(event.time_stamp());
 
   if ((event.flags() & ui::EF_SHIFT_DOWN) != 0 && event.x_offset() == 0) {
     webkit_event.deltaX = event.y_offset();
@@ -428,11 +447,11 @@ blink::WebMouseWheelEvent MakeWebMouseWheelEventFromAuraEvent(
   webkit_event.wheelTicksX = webkit_event.deltaX / kPixelsPerTick;
   webkit_event.wheelTicksY = webkit_event.deltaY / kPixelsPerTick;
 
-  webkit_event.tiltX = roundf(event.pointer_details().tilt_x());
-  webkit_event.tiltY = roundf(event.pointer_details().tilt_y());
-  webkit_event.force = event.pointer_details().force();
+  webkit_event.tiltX = roundf(event.pointer_details().tilt_x);
+  webkit_event.tiltY = roundf(event.pointer_details().tilt_y);
+  webkit_event.force = event.pointer_details().force;
   webkit_event.pointerType =
-      EventPointerTypeToWebPointerType(event.pointer_details().pointer_type());
+      EventPointerTypeToWebPointerType(event.pointer_details().pointer_type);
 
   return webkit_event;
 }
