@@ -5,6 +5,7 @@
 #ifndef CONTENT_BROWSER_SERVICE_WORKER_SERVICE_WORKER_REGISTER_JOB_H_
 #define CONTENT_BROWSER_SERVICE_WORKER_SERVICE_WORKER_REGISTER_JOB_H_
 
+#include <string>
 #include <vector>
 
 #include "base/macros.h"
@@ -34,7 +35,8 @@ class ServiceWorkerStorage;
 //  - waiting for older ServiceWorkerVersions to deactivate
 //  - designating the new version to be the 'active' version
 //  - updating storage
-class ServiceWorkerRegisterJob : public ServiceWorkerRegisterJobBase {
+class ServiceWorkerRegisterJob : public ServiceWorkerRegisterJobBase,
+                                 public EmbeddedWorkerInstance::Listener {
  public:
   typedef base::Callback<void(ServiceWorkerStatusCode status,
                               const std::string& status_message,
@@ -120,7 +122,11 @@ class ServiceWorkerRegisterJob : public ServiceWorkerRegisterJobBase {
   void OnStartWorkerFinished(ServiceWorkerStatusCode status);
   void OnStoreRegistrationComplete(ServiceWorkerStatusCode status);
   void InstallAndContinue();
-  void OnInstallFinished(ServiceWorkerStatusCode status);
+  void DispatchInstallEvent();
+  void OnInstallFinished(int request_id,
+                         blink::WebServiceWorkerEventResult result,
+                         bool has_fetch_handler);
+  void OnInstallFailed(ServiceWorkerStatusCode status);
   void Complete(ServiceWorkerStatusCode status);
   void Complete(ServiceWorkerStatusCode status,
                 const std::string& status_message);
@@ -133,12 +139,17 @@ class ServiceWorkerRegisterJob : public ServiceWorkerRegisterJobBase {
   void AddRegistrationToMatchingProviderHosts(
       ServiceWorkerRegistration* registration);
 
+  // EmbeddedWorkerInstance::Listener implementation:
+  void OnScriptLoaded() override;
+
+  void BumpLastUpdateCheckTimeIfNeeded();
+
   // The ServiceWorkerContextCore object should always outlive this.
   base::WeakPtr<ServiceWorkerContextCore> context_;
 
   RegistrationJobType job_type_;
   const GURL pattern_;
-  const GURL script_url_;
+  GURL script_url_;
   std::vector<RegistrationCallback> callbacks_;
   Phase phase_;
   Internal internal_;
