@@ -2,10 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "content/browser/time_zone_monitor.h"
-
 #import <Foundation/Foundation.h>
 
+#include "base/macros.h"
+#include "content/browser/time_zone_monitor.h"
+
 namespace content {
 
 class TimeZoneMonitorMac : public TimeZoneMonitor {
@@ -33,8 +34,8 @@ class TimeZoneMonitorMac : public TimeZoneMonitor {
 };
 
 // static
-scoped_ptr<TimeZoneMonitor> TimeZoneMonitor::Create() {
-  return scoped_ptr<TimeZoneMonitor>(new TimeZoneMonitorMac());
+std::unique_ptr<TimeZoneMonitor> TimeZoneMonitor::Create() {
+  return std::unique_ptr<TimeZoneMonitor>(new TimeZoneMonitorMac());
 }
 
 }  // namespace content
