@@ -9,7 +9,9 @@
 #include "base/strings/sys_string_conversions.h"
 #include "base/strings/utf_string_conversions.h"
 #include "components/mime_util/mime_util.h"
+#include "media/base/mime_util.h"
 #include "net/base/mime_util.h"
+#include "third_party/WebKit/public/platform/FilePathConversion.h"
 #include "third_party/WebKit/public/platform/WebString.h"
 
 using blink::WebString;
@@ -62,8 +64,7 @@ WebMimeRegistry::SupportsType
 // see TestShellWebMimeRegistryImpl.
 WebMimeRegistry::SupportsType SimpleWebMimeRegistryImpl::supportsMediaMIMEType(
     const WebString& mime_type,
-    const WebString& codecs,
-    const WebString& key_system) {
+    const WebString& codecs) {
   // Media features are only supported at the content/renderer/ layer.
   return IsNotSupported;
 }
@@ -72,7 +73,8 @@ bool SimpleWebMimeRegistryImpl::supportsMediaSourceMIMEType(
     const WebString& mime_type,
     const WebString& codecs) {
   // Media features are only supported at the content/renderer layer.
-  return false;
+  const std::string mime_type_ascii = ToASCIIOrEmpty(mime_type);
+  return media::IsSupportedMediaMimeType(mime_type_ascii);
 }
 
 WebMimeRegistry::SupportsType
@@ -87,7 +89,7 @@ WebString SimpleWebMimeRegistryImpl::mimeTypeForExtension(
     const WebString& file_extension) {
   std::string mime_type;
   net::GetMimeTypeFromExtension(
-      base::FilePath::FromUTF16Unsafe(file_extension).value(), &mime_type);
+      blink::WebStringToFilePath(file_extension).value(), &mime_type);
   return WebString::fromUTF8(mime_type);
 }
 
@@ -95,7 +97,7 @@ WebString SimpleWebMimeRegistryImpl::wellKnownMimeTypeForExtension(
     const WebString& file_extension) {
   std::string mime_type;
   net::GetWellKnownMimeTypeFromExtension(
-      base::FilePath::FromUTF16Unsafe(file_extension).value(), &mime_type);
+      blink::WebStringToFilePath(file_extension).value(), &mime_type);
   return WebString::fromUTF8(mime_type);
 }
 
