@@ -2,17 +2,19 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "content/child/v8_value_converter_impl.h"
+
 #include <stddef.h>
 #include <stdint.h>
 
 #include <cmath>
+#include <memory>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/stl_util.h"
 #include "base/test/values_test_util.h"
 #include "base/values.h"
-#include "content/child/v8_value_converter_impl.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "v8/include/v8.h"
 
@@ -163,10 +165,10 @@ class V8ValueConverterImplTest : public testing::Test {
   void TestWeirdType(const V8ValueConverterImpl& converter,
                      v8::Local<v8::Value> val,
                      base::Value::Type expected_type,
-                     scoped_ptr<base::Value> expected_value) {
+                     std::unique_ptr<base::Value> expected_value) {
     v8::Local<v8::Context> context =
         v8::Local<v8::Context>::New(isolate_, context_);
-    scoped_ptr<base::Value> raw(converter.FromV8Value(val, context));
+    std::unique_ptr<base::Value> raw(converter.FromV8Value(val, context));
 
     if (expected_value) {
       ASSERT_TRUE(raw.get());
@@ -178,9 +180,8 @@ class V8ValueConverterImplTest : public testing::Test {
 
     v8::Local<v8::Object> object(v8::Object::New(isolate_));
     object->Set(v8::String::NewFromUtf8(isolate_, "test"), val);
-    scoped_ptr<base::DictionaryValue> dictionary(
-        static_cast<base::DictionaryValue*>(
-            converter.FromV8Value(object, context)));
+    std::unique_ptr<base::DictionaryValue> dictionary(
+        base::DictionaryValue::From(converter.FromV8Value(object, context)));
     ASSERT_TRUE(dictionary.get());
 
     if (expected_value) {
@@ -194,8 +195,8 @@ class V8ValueConverterImplTest : public testing::Test {
 
     v8::Local<v8::Array> array(v8::Array::New(isolate_));
     array->Set(0, val);
-    scoped_ptr<base::ListValue> list(
-        static_cast<base::ListValue*>(converter.FromV8Value(array, context)));
+    std::unique_ptr<base::ListValue> list(
+        base::ListValue::From(converter.FromV8Value(array, context)));
     ASSERT_TRUE(list.get());
     if (expected_value) {
       base::Value* temp = NULL;
@@ -218,7 +219,7 @@ class V8ValueConverterImplTest : public testing::Test {
 };
 
 TEST_F(V8ValueConverterImplTest, BasicRoundTrip) {
-  scoped_ptr<base::Value> original_root = base::test::ParseJson(
+  std::unique_ptr<base::Value> original_root = base::test::ParseJson(
       "{ \n"
       "  \"null\": null, \n"
       "  \"true\": true, \n"
@@ -280,13 +281,14 @@ TEST_F(V8ValueConverterImplTest, BasicRoundTrip) {
   EXPECT_TRUE(v8_object->Get(v8::String::NewFromUtf8(isolate_, "empty-list"))
                   ->IsArray());
 
-  scoped_ptr<base::Value> new_root(converter.FromV8Value(v8_object, context));
+  std::unique_ptr<base::Value> new_root(
+      converter.FromV8Value(v8_object, context));
   EXPECT_NE(original_root.get(), new_root.get());
   EXPECT_TRUE(original_root->Equals(new_root.get()));
 }
 
 TEST_F(V8ValueConverterImplTest, KeysWithDots) {
-  scoped_ptr<base::Value> original =
+  std::unique_ptr<base::Value> original =
       base::test::ParseJson("{ \"foo.bar\": \"baz\" }");
 
   v8::HandleScope handle_scope(isolate_);
@@ -295,9 +297,8 @@ TEST_F(V8ValueConverterImplTest, KeysWithDots) {
   v8::Context::Scope context_scope(context);
 
   V8ValueConverterImpl converter;
-  scoped_ptr<base::Value> copy(
-      converter.FromV8Value(
-          converter.ToV8Value(original.get(), context), context));
+  std::unique_ptr<base::Value> copy(converter.FromV8Value(
+      converter.ToV8Value(original.get(), context), context));
 
   EXPECT_TRUE(original->Equals(copy.get()));
 }
@@ -307,6 +308,8 @@ TEST_F(V8ValueConverterImplTest, ObjectExceptions) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   // Set up objects to throw when reading or writing 'foo'.
   const char* source =
@@ -325,9 +328,8 @@ TEST_F(V8ValueConverterImplTest, ObjectExceptions) {
 
   // Converting from v8 value should replace the foo property with null.
   V8ValueConverterImpl converter;
-  scoped_ptr<base::DictionaryValue> converted(
-      static_cast<base::DictionaryValue*>(
-          converter.FromV8Value(object, context)));
+  std::unique_ptr<base::DictionaryValue> converted(
+      base::DictionaryValue::From(converter.FromV8Value(object, context)));
   EXPECT_TRUE(converted.get());
   // http://code.google.com/p/v8/issues/detail?id=1342
   // EXPECT_EQ(2u, converted->size());
@@ -350,6 +352,8 @@ TEST_F(V8ValueConverterImplTest, ArrayExceptions) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   const char* source = "(function() {"
       "var arr = [];"
@@ -368,8 +372,8 @@ TEST_F(V8ValueConverterImplTest, ArrayExceptions) {
 
   // Converting from v8 value should replace the first item with null.
   V8ValueConverterImpl converter;
-  scoped_ptr<base::ListValue> converted(static_cast<base::ListValue*>(
-      converter.FromV8Value(array, context)));
+  std::unique_ptr<base::ListValue> converted(
+      base::ListValue::From(converter.FromV8Value(array, context)));
   ASSERT_TRUE(converted.get());
   // http://code.google.com/p/v8/issues/detail?id=1342
   EXPECT_EQ(2u, converted->GetSize());
@@ -398,30 +402,23 @@ TEST_F(V8ValueConverterImplTest, WeirdTypes) {
       v8::String::NewFromUtf8(isolate_, "."), v8::RegExp::kNone));
 
   V8ValueConverterImpl converter;
-  TestWeirdType(converter,
-                v8::Undefined(isolate_),
+  TestWeirdType(converter, v8::Undefined(isolate_),
                 base::Value::TYPE_NULL,  // Arbitrary type, result is NULL.
-                scoped_ptr<base::Value>());
-  TestWeirdType(converter,
-                v8::Date::New(isolate_, 1000),
+                std::unique_ptr<base::Value>());
+  TestWeirdType(converter, v8::Date::New(isolate_, 1000),
                 base::Value::TYPE_DICTIONARY,
-                scoped_ptr<base::Value>(new base::DictionaryValue()));
-  TestWeirdType(converter,
-                regex,
-                base::Value::TYPE_DICTIONARY,
-                scoped_ptr<base::Value>(new base::DictionaryValue()));
+                std::unique_ptr<base::Value>(new base::DictionaryValue()));
+  TestWeirdType(converter, regex, base::Value::TYPE_DICTIONARY,
+                std::unique_ptr<base::Value>(new base::DictionaryValue()));
 
   converter.SetDateAllowed(true);
-  TestWeirdType(converter,
-                v8::Date::New(isolate_, 1000),
+  TestWeirdType(converter, v8::Date::New(isolate_, 1000),
                 base::Value::TYPE_DOUBLE,
-                scoped_ptr<base::Value>(new base::FundamentalValue(1.0)));
+                std::unique_ptr<base::Value>(new base::FundamentalValue(1.0)));
 
   converter.SetRegExpAllowed(true);
-  TestWeirdType(converter,
-                regex,
-                base::Value::TYPE_STRING,
-                scoped_ptr<base::Value>(new base::StringValue("/./")));
+  TestWeirdType(converter, regex, base::Value::TYPE_STRING,
+                std::unique_ptr<base::Value>(new base::StringValue("/./")));
 }
 
 TEST_F(V8ValueConverterImplTest, Prototype) {
@@ -429,6 +426,8 @@ TEST_F(V8ValueConverterImplTest, Prototype) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   const char* source = "(function() {"
       "Object.prototype.foo = 'foo';"
@@ -441,9 +440,8 @@ TEST_F(V8ValueConverterImplTest, Prototype) {
   ASSERT_FALSE(object.IsEmpty());
 
   V8ValueConverterImpl converter;
-  scoped_ptr<base::DictionaryValue> result(
-      static_cast<base::DictionaryValue*>(
-          converter.FromV8Value(object, context)));
+  std::unique_ptr<base::DictionaryValue> result(
+      base::DictionaryValue::From(converter.FromV8Value(object, context)));
   ASSERT_TRUE(result.get());
   EXPECT_EQ(0u, result->size());
 }
@@ -570,7 +568,7 @@ TEST_F(V8ValueConverterImplTest, ArrayPrototypeSetter) {
 
   // Try again, using an array without the index.
   base::ListValue one_item_list;
-  one_item_list.Append(new base::FundamentalValue(123456));
+  one_item_list.AppendInteger(123456);
   v8::Local<v8::Array> converted2 =
       converter.ToV8Value(&one_item_list, context).As<v8::Array>();
   EXPECT_FALSE(converted2.IsEmpty());
@@ -593,6 +591,8 @@ TEST_F(V8ValueConverterImplTest, StripNullFromObjects) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   const char* source = "(function() {"
       "return { foo: undefined, bar: null };"
@@ -606,9 +606,8 @@ TEST_F(V8ValueConverterImplTest, StripNullFromObjects) {
   V8ValueConverterImpl converter;
   converter.SetStripNullFromObjects(true);
 
-  scoped_ptr<base::DictionaryValue> result(
-      static_cast<base::DictionaryValue*>(
-          converter.FromV8Value(object, context)));
+  std::unique_ptr<base::DictionaryValue> result(
+      base::DictionaryValue::From(converter.FromV8Value(object, context)));
   ASSERT_TRUE(result.get());
   EXPECT_EQ(0u, result->size());
 }
@@ -627,9 +626,8 @@ TEST_F(V8ValueConverterImplTest, RecursiveObjects) {
               v8::String::NewFromUtf8(isolate_, "bar"));
   object->Set(v8::String::NewFromUtf8(isolate_, "obj"), object);
 
-  scoped_ptr<base::DictionaryValue> object_result(
-      static_cast<base::DictionaryValue*>(
-          converter.FromV8Value(object, context)));
+  std::unique_ptr<base::DictionaryValue> object_result(
+      base::DictionaryValue::From(converter.FromV8Value(object, context)));
   ASSERT_TRUE(object_result.get());
   EXPECT_EQ(2u, object_result->size());
   EXPECT_TRUE(IsNull(object_result.get(), "obj"));
@@ -639,8 +637,8 @@ TEST_F(V8ValueConverterImplTest, RecursiveObjects) {
   array->Set(0, v8::String::NewFromUtf8(isolate_, "1"));
   array->Set(1, array);
 
-  scoped_ptr<base::ListValue> list_result(
-      static_cast<base::ListValue*>(converter.FromV8Value(array, context)));
+  std::unique_ptr<base::ListValue> list_result(
+      base::ListValue::From(converter.FromV8Value(array, context)));
   ASSERT_TRUE(list_result.get());
   EXPECT_EQ(2u, list_result->GetSize());
   EXPECT_TRUE(IsNull(list_result.get(), 1));
@@ -651,6 +649,8 @@ TEST_F(V8ValueConverterImplTest, WeirdProperties) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   const char* source = "(function() {"
       "return {"
@@ -669,9 +669,9 @@ TEST_F(V8ValueConverterImplTest, WeirdProperties) {
   ASSERT_FALSE(object.IsEmpty());
 
   V8ValueConverterImpl converter;
-  scoped_ptr<base::Value> actual(converter.FromV8Value(object, context));
+  std::unique_ptr<base::Value> actual(converter.FromV8Value(object, context));
 
-  scoped_ptr<base::Value> expected = base::test::ParseJson(
+  std::unique_ptr<base::Value> expected = base::test::ParseJson(
       "{ \n"
       "  \"1\": \"foo\", \n"
       "  \"2\": \"bar\", \n"
@@ -689,6 +689,8 @@ TEST_F(V8ValueConverterImplTest, ArrayGetters) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   const char* source = "(function() {"
       "var a = [0];"
@@ -702,8 +704,8 @@ TEST_F(V8ValueConverterImplTest, ArrayGetters) {
   ASSERT_FALSE(array.IsEmpty());
 
   V8ValueConverterImpl converter;
-  scoped_ptr<base::ListValue> result(
-      static_cast<base::ListValue*>(converter.FromV8Value(array, context)));
+  std::unique_ptr<base::ListValue> result(
+      base::ListValue::From(converter.FromV8Value(array, context)));
   ASSERT_TRUE(result.get());
   EXPECT_EQ(2u, result->GetSize());
 }
@@ -713,6 +715,8 @@ TEST_F(V8ValueConverterImplTest, UndefinedValueBehavior) {
   v8::Local<v8::Context> context =
       v8::Local<v8::Context>::New(isolate_, context_);
   v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
 
   v8::Local<v8::Object> object;
   {
@@ -749,17 +753,18 @@ TEST_F(V8ValueConverterImplTest, UndefinedValueBehavior) {
 
   V8ValueConverterImpl converter;
 
-  scoped_ptr<base::Value> actual_object(
+  std::unique_ptr<base::Value> actual_object(
       converter.FromV8Value(object, context));
   EXPECT_TRUE(base::Value::Equals(
       base::test::ParseJson("{ \"bar\": null }").get(), actual_object.get()));
 
   // Everything is null because JSON stringification preserves array length.
-  scoped_ptr<base::Value> actual_array(converter.FromV8Value(array, context));
+  std::unique_ptr<base::Value> actual_array(
+      converter.FromV8Value(array, context));
   EXPECT_TRUE(base::Value::Equals(
       base::test::ParseJson("[ null, null, null ]").get(), actual_array.get()));
 
-  scoped_ptr<base::Value> actual_sparse_array(
+  std::unique_ptr<base::Value> actual_sparse_array(
       converter.FromV8Value(sparse_array, context));
   EXPECT_TRUE(
       base::Value::Equals(base::test::ParseJson("[ null, null, null ]").get(),
@@ -785,11 +790,12 @@ TEST_F(V8ValueConverterImplTest, ObjectsWithClashingIdentityHash) {
   root->Set(3, v8::Local<v8::Object>(v8::Array::New(isolate_, 0)));
 
   // The expected base::Value result.
-  scoped_ptr<base::Value> expected = base::test::ParseJson("[{},{},[],[]]");
+  std::unique_ptr<base::Value> expected =
+      base::test::ParseJson("[{},{},[],[]]");
   ASSERT_TRUE(expected.get());
 
   // The actual result.
-  scoped_ptr<base::Value> value(converter.FromV8Value(root, context));
+  std::unique_ptr<base::Value> value(converter.FromV8Value(root, context));
   ASSERT_TRUE(value.get());
 
   EXPECT_TRUE(expected->Equals(value.get()));
@@ -811,7 +817,7 @@ TEST_F(V8ValueConverterImplTest, DetectCycles) {
   expected_list.Append(base::Value::CreateNullValue());
 
   // The actual result.
-  scoped_ptr<base::Value> actual_list(
+  std::unique_ptr<base::Value> actual_list(
       converter.FromV8Value(recursive_array, context));
   ASSERT_TRUE(actual_list.get());
 
@@ -832,13 +838,81 @@ TEST_F(V8ValueConverterImplTest, DetectCycles) {
   expected_dictionary.Set(key, base::Value::CreateNullValue());
 
   // The actual result.
-  scoped_ptr<base::Value> actual_dictionary(
+  std::unique_ptr<base::Value> actual_dictionary(
       converter.FromV8Value(recursive_object, context));
   ASSERT_TRUE(actual_dictionary.get());
 
   EXPECT_TRUE(expected_dictionary.Equals(actual_dictionary.get()));
 }
 
+// Tests that reused object values with no cycles do not get nullified.
+TEST_F(V8ValueConverterImplTest, ReuseObjects) {
+  v8::HandleScope handle_scope(isolate_);
+  v8::Local<v8::Context> context =
+      v8::Local<v8::Context>::New(isolate_, context_);
+  v8::Context::Scope context_scope(context);
+  v8::MicrotasksScope microtasks(
+      isolate_, v8::MicrotasksScope::kDoNotRunMicrotasks);
+  V8ValueConverterImpl converter;
+
+  // Object with reused values in different keys.
+  {
+    const char* source = "(function() {"
+        "var objA = {key: 'another same value'};"
+        "var obj = {one: objA, two: objA};"
+        "return obj;"
+        "})();";
+    v8::Local<v8::Script> script(
+        v8::Script::Compile(v8::String::NewFromUtf8(isolate_, source)));
+    v8::Local<v8::Object> object = script->Run().As<v8::Object>();
+    ASSERT_FALSE(object.IsEmpty());
+
+    // The actual result.
+    std::unique_ptr<base::DictionaryValue> result(
+        base::DictionaryValue::From(converter.FromV8Value(object, context)));
+    ASSERT_TRUE(result.get());
+    EXPECT_EQ(2u, result->size());
+
+    {
+      base::DictionaryValue* one_dict = nullptr;
+      const char* key1 = "one";
+      ASSERT_TRUE(result->GetDictionary(key1, &one_dict));
+      EXPECT_EQ("another same value", GetString(one_dict, "key"));
+    }
+    {
+      base::DictionaryValue* two_dict = nullptr;
+      const char* key2 = "two";
+      ASSERT_TRUE(result->GetDictionary(key2, &two_dict));
+      EXPECT_EQ("another same value", GetString(two_dict, "key"));
+    }
+  }
+
+  // Array with reused values.
+  {
+    const char* source = "(function() {"
+        "var objA = {key: 'same value'};"
+        "var arr = [objA, objA];"
+        "return arr;"
+        "})();";
+    v8::Local<v8::Script> script(
+        v8::Script::Compile(v8::String::NewFromUtf8(isolate_, source)));
+    v8::Local<v8::Array> array = script->Run().As<v8::Array>();
+    ASSERT_FALSE(array.IsEmpty());
+
+    // The actual result.
+    std::unique_ptr<base::ListValue> list_result(
+        base::ListValue::From(converter.FromV8Value(array, context)));
+    ASSERT_TRUE(list_result.get());
+    ASSERT_EQ(2u, list_result->GetSize());
+    for (size_t i = 0; i < list_result->GetSize(); ++i) {
+      ASSERT_FALSE(IsNull(list_result.get(), i));
+      base::DictionaryValue* dict_value = nullptr;
+      ASSERT_TRUE(list_result->GetDictionary(0u, &dict_value));
+      EXPECT_EQ("same value", GetString(dict_value, "key"));
+    }
+  }
+}
+
 TEST_F(V8ValueConverterImplTest, MaxRecursionDepth) {
   v8::HandleScope handle_scope(isolate_);
   v8::Local<v8::Context> context =
@@ -859,7 +933,8 @@ TEST_F(V8ValueConverterImplTest, MaxRecursionDepth) {
   }
 
   V8ValueConverterImpl converter;
-  scoped_ptr<base::Value> value(converter.FromV8Value(deep_object, context));
+  std::unique_ptr<base::Value> value(
+      converter.FromV8Value(deep_object, context));
   ASSERT_TRUE(value);
 
   // Expected depth is kMaxRecursionDepth in v8_value_converter_impl.cc.
@@ -883,41 +958,41 @@ class V8ValueConverterOverridingStrategyForTesting
   V8ValueConverterOverridingStrategyForTesting()
       : reference_value_(NewReferenceValue()) {}
   bool FromV8Object(v8::Local<v8::Object> value,
-                    base::Value** out,
+                    std::unique_ptr<base::Value>* out,
                     v8::Isolate* isolate,
                     const FromV8ValueCallback& callback) const override {
     *out = NewReferenceValue();
     return true;
   }
   bool FromV8Array(v8::Local<v8::Array> value,
-                   base::Value** out,
+                   std::unique_ptr<base::Value>* out,
                    v8::Isolate* isolate,
                    const FromV8ValueCallback& callback) const override {
     *out = NewReferenceValue();
     return true;
   }
   bool FromV8ArrayBuffer(v8::Local<v8::Object> value,
-                         base::Value** out,
+                         std::unique_ptr<base::Value>* out,
                          v8::Isolate* isolate) const override {
     *out = NewReferenceValue();
     return true;
   }
   bool FromV8Number(v8::Local<v8::Number> value,
-                    base::Value** out) const override {
+                    std::unique_ptr<base::Value>* out) const override {
     *out = NewReferenceValue();
     return true;
   }
-  bool FromV8Undefined(base::Value** out) const override {
+  bool FromV8Undefined(std::unique_ptr<base::Value>* out) const override {
     *out = NewReferenceValue();
     return true;
   }
   base::Value* reference_value() const { return reference_value_.get(); }
 
  private:
-  static base::Value* NewReferenceValue() {
-    return new base::StringValue("strategy");
+  static std::unique_ptr<base::Value> NewReferenceValue() {
+    return base::MakeUnique<base::StringValue>("strategy");
   }
-  scoped_ptr<base::Value> reference_value_;
+  std::unique_ptr<base::Value> reference_value_;
 };
 
 TEST_F(V8ValueConverterImplTest, StrategyOverrides) {
@@ -931,19 +1006,21 @@ TEST_F(V8ValueConverterImplTest, StrategyOverrides) {
   converter.SetStrategy(&strategy);
 
   v8::Local<v8::Object> object(v8::Object::New(isolate_));
-  scoped_ptr<base::Value> object_value(converter.FromV8Value(object, context));
+  std::unique_ptr<base::Value> object_value(
+      converter.FromV8Value(object, context));
   ASSERT_TRUE(object_value);
   EXPECT_TRUE(
       base::Value::Equals(strategy.reference_value(), object_value.get()));
 
   v8::Local<v8::Array> array(v8::Array::New(isolate_));
-  scoped_ptr<base::Value> array_value(converter.FromV8Value(array, context));
+  std::unique_ptr<base::Value> array_value(
+      converter.FromV8Value(array, context));
   ASSERT_TRUE(array_value);
   EXPECT_TRUE(
       base::Value::Equals(strategy.reference_value(), array_value.get()));
 
   v8::Local<v8::ArrayBuffer> array_buffer(v8::ArrayBuffer::New(isolate_, 0));
-  scoped_ptr<base::Value> array_buffer_value(
+  std::unique_ptr<base::Value> array_buffer_value(
       converter.FromV8Value(array_buffer, context));
   ASSERT_TRUE(array_buffer_value);
   EXPECT_TRUE(base::Value::Equals(strategy.reference_value(),
@@ -951,20 +1028,21 @@ TEST_F(V8ValueConverterImplTest, StrategyOverrides) {
 
   v8::Local<v8::ArrayBufferView> array_buffer_view(
       v8::Uint8Array::New(array_buffer, 0, 0));
-  scoped_ptr<base::Value> array_buffer_view_value(
+  std::unique_ptr<base::Value> array_buffer_view_value(
       converter.FromV8Value(array_buffer_view, context));
   ASSERT_TRUE(array_buffer_view_value);
   EXPECT_TRUE(base::Value::Equals(strategy.reference_value(),
                                   array_buffer_view_value.get()));
 
   v8::Local<v8::Number> number(v8::Number::New(isolate_, 0.0));
-  scoped_ptr<base::Value> number_value(converter.FromV8Value(number, context));
+  std::unique_ptr<base::Value> number_value(
+      converter.FromV8Value(number, context));
   ASSERT_TRUE(number_value);
   EXPECT_TRUE(
       base::Value::Equals(strategy.reference_value(), number_value.get()));
 
   v8::Local<v8::Primitive> undefined(v8::Undefined(isolate_));
-  scoped_ptr<base::Value> undefined_value(
+  std::unique_ptr<base::Value> undefined_value(
       converter.FromV8Value(undefined, context));
   ASSERT_TRUE(undefined_value);
   EXPECT_TRUE(
@@ -975,27 +1053,29 @@ class V8ValueConverterBypassStrategyForTesting
     : public V8ValueConverter::Strategy {
  public:
   bool FromV8Object(v8::Local<v8::Object> value,
-                    base::Value** out,
+                    std::unique_ptr<base::Value>* out,
                     v8::Isolate* isolate,
                     const FromV8ValueCallback& callback) const override {
     return false;
   }
   bool FromV8Array(v8::Local<v8::Array> value,
-                   base::Value** out,
+                   std::unique_ptr<base::Value>* out,
                    v8::Isolate* isolate,
                    const FromV8ValueCallback& callback) const override {
     return false;
   }
   bool FromV8ArrayBuffer(v8::Local<v8::Object> value,
-                         base::Value** out,
+                         std::unique_ptr<base::Value>* out,
                          v8::Isolate* isolate) const override {
     return false;
   }
   bool FromV8Number(v8::Local<v8::Number> value,
-                    base::Value** out) const override {
+                    std::unique_ptr<base::Value>* out) const override {
+    return false;
+  }
+  bool FromV8Undefined(std::unique_ptr<base::Value>* out) const override {
     return false;
   }
-  bool FromV8Undefined(base::Value** out) const override { return false; }
 };
 
 // Verify that having a strategy that fallbacks to default behaviour
@@ -1011,16 +1091,20 @@ TEST_F(V8ValueConverterImplTest, StrategyBypass) {
   converter.SetStrategy(&strategy);
 
   v8::Local<v8::Object> object(v8::Object::New(isolate_));
-  scoped_ptr<base::Value> object_value(converter.FromV8Value(object, context));
+  std::unique_ptr<base::Value> object_value(
+      converter.FromV8Value(object, context));
   ASSERT_TRUE(object_value);
-  scoped_ptr<base::Value> reference_object_value(base::test::ParseJson("{}"));
+  std::unique_ptr<base::Value> reference_object_value(
+      base::test::ParseJson("{}"));
   EXPECT_TRUE(
       base::Value::Equals(reference_object_value.get(), object_value.get()));
 
   v8::Local<v8::Array> array(v8::Array::New(isolate_));
-  scoped_ptr<base::Value> array_value(converter.FromV8Value(array, context));
+  std::unique_ptr<base::Value> array_value(
+      converter.FromV8Value(array, context));
   ASSERT_TRUE(array_value);
-  scoped_ptr<base::Value> reference_array_value(base::test::ParseJson("[]"));
+  std::unique_ptr<base::Value> reference_array_value(
+      base::test::ParseJson("[]"));
   EXPECT_TRUE(
       base::Value::Equals(reference_array_value.get(), array_value.get()));
 
@@ -1028,14 +1112,16 @@ TEST_F(V8ValueConverterImplTest, StrategyBypass) {
   // this requires having blink to be initialized.
 
   v8::Local<v8::Number> number(v8::Number::New(isolate_, 0.0));
-  scoped_ptr<base::Value> number_value(converter.FromV8Value(number, context));
+  std::unique_ptr<base::Value> number_value(
+      converter.FromV8Value(number, context));
   ASSERT_TRUE(number_value);
-  scoped_ptr<base::Value> reference_number_value(base::test::ParseJson("0"));
+  std::unique_ptr<base::Value> reference_number_value(
+      base::test::ParseJson("0"));
   EXPECT_TRUE(
       base::Value::Equals(reference_number_value.get(), number_value.get()));
 
   v8::Local<v8::Primitive> undefined(v8::Undefined(isolate_));
-  scoped_ptr<base::Value> undefined_value(
+  std::unique_ptr<base::Value> undefined_value(
       converter.FromV8Value(undefined, context));
   EXPECT_FALSE(undefined_value);
 }
