@@ -4,15 +4,23 @@
 
 #include "content/child/web_url_loader_impl.h"
 
+#include <stdint.h>
 #include <string.h>
+
+#include <utility>
 #include <vector>
 
 #include "base/command_line.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/weak_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "base/single_thread_task_runner.h"
+#include "base/time/default_tick_clock.h"
 #include "base/time/time.h"
+#include "components/scheduler/child/scheduler_tqm_delegate_impl.h"
+#include "components/scheduler/child/web_task_runner_impl.h"
+#include "components/scheduler/child/worker_scheduler.h"
 #include "content/child/request_extra_data.h"
 #include "content/child/request_info.h"
 #include "content/child/resource_dispatcher.h"
@@ -20,6 +28,7 @@
 #include "content/public/child/request_peer.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/resource_response_info.h"
+#include "net/base/host_port_pair.h"
 #include "net/base/net_errors.h"
 #include "net/http/http_response_headers.h"
 #include "net/http/http_util.h"
@@ -46,27 +55,12 @@ const char kFtpDirListing[] =
     "drwxr-xr-x    3 ftp      ftp          4096 May 15 18:11 goat\n"
     "drwxr-xr-x    3 ftp      ftp          4096 May 15 18:11 hat";
 
-const char kMultipartResponseMimeType[] = "multipart/x-mixed-replace";
-const char kMultipartResponseHeaders[] =
-    "HTTP/1.0 200 Peachy\r\n"
-    "Content-Type: multipart/x-mixed-replace; boundary=boundary\r\n\r\n";
-// Simple multipart response.  Imporant details for the tests are that it
-// contains multiple chunks, and that it doesn't end with a boundary, so will
-// send data in OnResponseComplete.  Also, it will resolve to kTestData.
-const char kMultipartResponse[] =
-    "--boundary\n"
-    "Content-type: text/html\n\n"
-    "bl"
-    "--boundary\n"
-    "Content-type: text/html\n\n"
-    "ah!";
-
 class TestResourceDispatcher : public ResourceDispatcher {
  public:
   TestResourceDispatcher() :
       ResourceDispatcher(nullptr, nullptr),
-      peer_(NULL),
-      canceled_(false) {
+      canceled_(false),
+      defers_loading_(false) {
   }
 
   ~TestResourceDispatcher() override {}
@@ -74,11 +68,12 @@ class TestResourceDispatcher : public ResourceDispatcher {
   // TestDispatcher implementation:
 
   int StartAsync(const RequestInfo& request_info,
-                 ResourceRequestBody* request_body,
-                 RequestPeer* peer) override {
+                 ResourceRequestBodyImpl* request_body,
+                 std::unique_ptr<RequestPeer> peer) override {
     EXPECT_FALSE(peer_);
-    peer_ = peer;
+    peer_ = std::move(peer);
     url_ = request_info.url;
+    stream_url_ = request_info.resource_body_stream_url;
     return 1;
   }
 
@@ -87,27 +82,35 @@ class TestResourceDispatcher : public ResourceDispatcher {
     canceled_ = true;
   }
 
-  RequestPeer* peer() { return peer_; }
+  RequestPeer* peer() { return peer_.get(); }
 
   bool canceled() { return canceled_; }
 
   const GURL& url() { return url_; }
+  const GURL& stream_url() { return stream_url_; }
+
+  void SetDefersLoading(int request_id, bool value) override {
+    defers_loading_ = value;
+  }
+  bool defers_loading() const { return defers_loading_; }
 
  private:
-  RequestPeer* peer_;
+  std::unique_ptr<RequestPeer> peer_;
   bool canceled_;
+  bool defers_loading_;
   GURL url_;
+  GURL stream_url_;
 
   DISALLOW_COPY_AND_ASSIGN(TestResourceDispatcher);
 };
 
 class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
  public:
-  TestWebURLLoaderClient(
-      ResourceDispatcher* dispatcher,
-      scoped_refptr<base::SingleThreadTaskRunner> task_runner)
-      : loader_(new WebURLLoaderImpl(dispatcher, task_runner)),
-        expect_multipart_response_(false),
+  TestWebURLLoaderClient(ResourceDispatcher* dispatcher,
+                         scoped_refptr<scheduler::TaskQueue> task_runner)
+      : loader_(new WebURLLoaderImpl(
+            dispatcher,
+            base::WrapUnique(new scheduler::WebTaskRunnerImpl(task_runner)))),
         delete_on_receive_redirect_(false),
         delete_on_receive_response_(false),
         delete_on_receive_data_(false),
@@ -115,17 +118,22 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
         delete_on_fail_(false),
         did_receive_redirect_(false),
         did_receive_response_(false),
+        check_redirect_request_priority_(false),
         did_finish_(false) {}
 
   ~TestWebURLLoaderClient() override {}
 
   // blink::WebURLLoaderClient implementation:
-  void willSendRequest(
+  void willFollowRedirect(
       blink::WebURLLoader* loader,
       blink::WebURLRequest& newRequest,
       const blink::WebURLResponse& redirectResponse) override {
     EXPECT_TRUE(loader_);
     EXPECT_EQ(loader_.get(), loader);
+
+    if (check_redirect_request_priority_)
+      EXPECT_EQ(redirect_request_priority, newRequest.getPriority());
+
     // No test currently simulates mutiple redirects.
     EXPECT_FALSE(did_receive_redirect_);
     did_receive_redirect_ = true;
@@ -146,9 +154,7 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
       const blink::WebURLResponse& response) override {
     EXPECT_TRUE(loader_);
     EXPECT_EQ(loader_.get(), loader);
-
-    // Only multipart requests may receive multiple response headers.
-    EXPECT_TRUE(expect_multipart_response_ || !did_receive_response_);
+    EXPECT_FALSE(did_receive_response_);
 
     did_receive_response_ = true;
     response_ = response;
@@ -216,13 +222,15 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
     loader_.reset();
   }
 
-  void set_expect_multipart_response() { expect_multipart_response_ = true; }
-
   void set_delete_on_receive_redirect() { delete_on_receive_redirect_ = true; }
   void set_delete_on_receive_response() { delete_on_receive_response_ = true; }
   void set_delete_on_receive_data() { delete_on_receive_data_ = true; }
   void set_delete_on_finish() { delete_on_finish_ = true; }
   void set_delete_on_fail() { delete_on_fail_ = true; }
+  void set_redirect_request_priority(blink::WebURLRequest::Priority priority) {
+    check_redirect_request_priority_ = true;
+    redirect_request_priority = priority;
+  }
 
   bool did_receive_redirect() const { return did_receive_redirect_; }
   bool did_receive_response() const { return did_receive_response_; }
@@ -232,9 +240,7 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
   const blink::WebURLResponse& response() const { return response_; }
 
  private:
-  scoped_ptr<WebURLLoaderImpl> loader_;
-
-  bool expect_multipart_response_;
+  std::unique_ptr<WebURLLoaderImpl> loader_;
 
   bool delete_on_receive_redirect_;
   bool delete_on_receive_response_;
@@ -244,6 +250,8 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
 
   bool did_receive_redirect_;
   bool did_receive_response_;
+  bool check_redirect_request_priority_;
+  blink::WebURLRequest::Priority redirect_request_priority;
   std::string received_data_;
   bool did_finish_;
   blink::WebURLError error_;
@@ -255,7 +263,15 @@ class TestWebURLLoaderClient : public blink::WebURLLoaderClient {
 class WebURLLoaderImplTest : public testing::Test {
  public:
   explicit WebURLLoaderImplTest()
-      : client_(&dispatcher_, message_loop_.task_runner()) {}
+      : worker_scheduler_(scheduler::WorkerScheduler::Create(
+            scheduler::SchedulerTqmDelegateImpl::Create(
+                &message_loop_,
+                base::WrapUnique(new base::DefaultTickClock())))) {
+    worker_scheduler_->Init();
+    client_.reset(new TestWebURLLoaderClient(
+        &dispatcher_, worker_scheduler_->DefaultTaskRunner()));
+  }
+
   ~WebURLLoaderImplTest() override {}
 
   void DoStartAsyncRequest() {
@@ -266,6 +282,16 @@ class WebURLLoaderImplTest : public testing::Test {
     ASSERT_TRUE(peer());
   }
 
+  void DoStartAsyncRequestWithPriority(
+      blink::WebURLRequest::Priority priority) {
+    blink::WebURLRequest request;
+    request.initialize();
+    request.setPriority(priority);
+    request.setURL(GURL(kTestURL));
+    client()->loader()->loadAsynchronously(request, client());
+    ASSERT_TRUE(peer());
+  }
+
   void DoReceiveRedirect() {
     EXPECT_FALSE(client()->did_receive_redirect());
     net::RedirectInfo redirect_info;
@@ -287,7 +313,7 @@ class WebURLLoaderImplTest : public testing::Test {
   // Assumes it is called only once for a request.
   void DoReceiveData() {
     EXPECT_EQ("", client()->received_data());
-    peer()->OnReceivedData(make_scoped_ptr(new FixedReceivedData(
+    peer()->OnReceivedData(base::WrapUnique(new FixedReceivedData(
         kTestData, strlen(kTestData), strlen(kTestData))));
     EXPECT_EQ(kTestData, client()->received_data());
   }
@@ -302,23 +328,6 @@ class WebURLLoaderImplTest : public testing::Test {
     EXPECT_EQ("", client()->error().domain.utf8());
   }
 
-  void DoReceiveCompletedResponse() {
-    EXPECT_FALSE(client()->did_receive_response());
-    EXPECT_EQ("", client()->received_data());
-    EXPECT_FALSE(client()->did_finish());
-
-    peer()->OnReceivedCompletedResponse(
-        content::ResourceResponseInfo(),
-        make_scoped_ptr(new FixedReceivedData(kTestData, strlen(kTestData),
-                                              strlen(kTestData))),
-        net::OK, false, false, "", base::TimeTicks(), strlen(kTestData));
-
-    EXPECT_TRUE(client()->did_receive_response());
-    EXPECT_EQ(kTestData, client()->received_data());
-    EXPECT_EQ(net::OK, client()->error().reason);
-    EXPECT_EQ("", client()->error().domain.utf8());
-  }
-
   void DoFailRequest() {
     EXPECT_FALSE(client()->did_finish());
     peer()->OnCompletedRequest(net::ERR_FAILED, false, false, "",
@@ -337,40 +346,24 @@ class WebURLLoaderImplTest : public testing::Test {
   }
 
   void DoReceiveDataFtp() {
-    peer()->OnReceivedData(make_scoped_ptr(new FixedReceivedData(
+    peer()->OnReceivedData(base::WrapUnique(new FixedReceivedData(
         kFtpDirListing, strlen(kFtpDirListing), strlen(kFtpDirListing))));
     // The FTP delegate should modify the data the client sees.
     EXPECT_NE(kFtpDirListing, client()->received_data());
   }
 
-  void DoReceiveResponseMultipart() {
-    EXPECT_FALSE(client()->did_receive_response());
-    content::ResourceResponseInfo response_info;
-    response_info.headers = new net::HttpResponseHeaders(
-        net::HttpUtil::AssembleRawHeaders(kMultipartResponseHeaders,
-                                          strlen(kMultipartResponseHeaders)));
-    response_info.mime_type = kMultipartResponseMimeType;
-    peer()->OnReceivedResponse(response_info);
-    EXPECT_TRUE(client()->did_receive_response());
-  }
-
-  void DoReceiveDataMultipart() {
-    peer()->OnReceivedData(make_scoped_ptr(
-        new FixedReceivedData(kMultipartResponse, strlen(kMultipartResponse),
-                              strlen(kMultipartResponse))));
-    // Multipart delegate should modify the data the client sees.
-    EXPECT_NE(kMultipartResponse, client()->received_data());
-  }
-
-  TestWebURLLoaderClient* client() { return &client_; }
+  TestWebURLLoaderClient* client() { return client_.get(); }
   TestResourceDispatcher* dispatcher() { return &dispatcher_; }
   RequestPeer* peer() { return dispatcher()->peer(); }
   base::MessageLoop* message_loop() { return &message_loop_; }
 
  private:
   base::MessageLoop message_loop_;
+  // WorkerScheduler is needed because WebURLLoaderImpl needs a
+  // scheduler::TaskQueue.
+  std::unique_ptr<scheduler::WorkerScheduler> worker_scheduler_;
   TestResourceDispatcher dispatcher_;
-  TestWebURLLoaderClient client_;
+  std::unique_ptr<TestWebURLLoaderClient> client_;
 };
 
 TEST_F(WebURLLoaderImplTest, Success) {
@@ -392,6 +385,19 @@ TEST_F(WebURLLoaderImplTest, Redirect) {
   EXPECT_EQ(kTestData, client()->received_data());
 }
 
+TEST_F(WebURLLoaderImplTest, RedirectCorrectPriority) {
+  DoStartAsyncRequestWithPriority(
+      blink::WebURLRequest::Priority::PriorityVeryHigh);
+  client()->set_redirect_request_priority(
+      blink::WebURLRequest::Priority::PriorityVeryHigh);
+  DoReceiveRedirect();
+  DoReceiveResponse();
+  DoReceiveData();
+  DoCompleteRequest();
+  EXPECT_FALSE(dispatcher()->canceled());
+  EXPECT_EQ(kTestData, client()->received_data());
+}
+
 TEST_F(WebURLLoaderImplTest, Failure) {
   DoStartAsyncRequest();
   DoReceiveResponse();
@@ -400,13 +406,6 @@ TEST_F(WebURLLoaderImplTest, Failure) {
   EXPECT_FALSE(dispatcher()->canceled());
 }
 
-TEST_F(WebURLLoaderImplTest, ReceiveCompletedResponse) {
-  DoStartAsyncRequest();
-  DoReceiveCompletedResponse();
-  EXPECT_FALSE(dispatcher()->canceled());
-  EXPECT_EQ(kTestData, client()->received_data());
-}
-
 // The client may delete the WebURLLoader during any callback from the loader.
 // These tests make sure that doesn't result in a crash.
 TEST_F(WebURLLoaderImplTest, DeleteOnReceiveRedirect) {
@@ -542,6 +541,13 @@ TEST_F(WebURLLoaderImplTest, DataURLDefersLoading) {
   EXPECT_EQ("", client()->error().domain.utf8());
 }
 
+TEST_F(WebURLLoaderImplTest, DefersLoadingBeforeStart) {
+  client()->loader()->setDefersLoading(true);
+  EXPECT_FALSE(dispatcher()->defers_loading());
+  DoStartAsyncRequest();
+  EXPECT_TRUE(dispatcher()->defers_loading());
+}
+
 // FTP integration tests.  These are focused more on safe deletion than correct
 // parsing of FTP responses.
 
@@ -599,104 +605,34 @@ TEST_F(WebURLLoaderImplTest, FtpDeleteOnFail) {
   DoFailRequest();
 }
 
-// Multipart integration tests.  These are focused more on safe deletion than
-// correct parsing of Multipart responses.
-
-TEST_F(WebURLLoaderImplTest, Multipart) {
-  client()->set_expect_multipart_response();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  DoReceiveDataMultipart();
-  DoCompleteRequest();
-  EXPECT_EQ(kTestData, client()->received_data());
-  EXPECT_FALSE(dispatcher()->canceled());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteOnReceiveFirstResponse) {
-  client()->set_expect_multipart_response();
-  client()->set_delete_on_receive_response();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  EXPECT_EQ("", client()->received_data());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteOnReceiveSecondResponse) {
-  client()->set_expect_multipart_response();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  client()->set_delete_on_receive_response();
-  DoReceiveDataMultipart();
-  EXPECT_EQ("", client()->received_data());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteOnReceiveFirstData) {
-  client()->set_expect_multipart_response();
-  client()->set_delete_on_receive_data();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  DoReceiveDataMultipart();
-  EXPECT_EQ("bl", client()->received_data());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteOnReceiveMoreData) {
-  client()->set_expect_multipart_response();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  DoReceiveDataMultipart();
-  // For multipart responses, the delegate may send some data when notified
-  // of a request completing.
-  client()->set_delete_on_receive_data();
-  peer()->OnCompletedRequest(net::OK, false, false, "", base::TimeTicks(),
-                              strlen(kTestData));
-  EXPECT_FALSE(client()->did_finish());
-  EXPECT_EQ(kTestData, client()->received_data());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteFinish) {
-  client()->set_expect_multipart_response();
-  client()->set_delete_on_finish();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  DoReceiveDataMultipart();
-  DoCompleteRequest();
-  EXPECT_EQ(kTestData, client()->received_data());
-}
-
-TEST_F(WebURLLoaderImplTest, MultipartDeleteFail) {
-  client()->set_expect_multipart_response();
-  client()->set_delete_on_fail();
-  DoStartAsyncRequest();
-  DoReceiveResponseMultipart();
-  DoReceiveDataMultipart();
-  DoFailRequest();
-}
-
 // PlzNavigate: checks that the stream override parameters provided on
 // navigation commit are properly applied.
 TEST_F(WebURLLoaderImplTest, BrowserSideNavigationCommit) {
   // Initialize the request and the stream override.
+  const GURL kNavigationURL = GURL(kTestURL);
   const GURL kStreamURL = GURL("http://bar");
   const std::string kMimeType = "text/html";
   blink::WebURLRequest request;
   request.initialize();
-  request.setURL(GURL(kTestURL));
+  request.setURL(kNavigationURL);
   request.setFrameType(blink::WebURLRequest::FrameTypeTopLevel);
   request.setRequestContext(blink::WebURLRequest::RequestContextFrame);
-  scoped_ptr<StreamOverrideParameters> stream_override(
+  std::unique_ptr<StreamOverrideParameters> stream_override(
       new StreamOverrideParameters());
   stream_override->stream_url = kStreamURL;
   stream_override->response.mime_type = kMimeType;
   RequestExtraData* extra_data = new RequestExtraData();
-  extra_data->set_stream_override(stream_override.Pass());
+  extra_data->set_stream_override(std::move(stream_override));
   request.setExtraData(extra_data);
   base::CommandLine::ForCurrentProcess()->AppendSwitch(
       switches::kEnableBrowserSideNavigation);
 
   client()->loader()->loadAsynchronously(request, client());
 
-  // The stream url should have been requestead instead of the request url.
+  // The stream url should have been added to the RequestInfo.
   ASSERT_TRUE(peer());
-  EXPECT_EQ(kStreamURL, dispatcher()->url());
+  EXPECT_EQ(kNavigationURL, dispatcher()->url());
+  EXPECT_EQ(kStreamURL, dispatcher()->stream_url());
 
   EXPECT_FALSE(client()->did_receive_response());
   peer()->OnReceivedResponse(content::ResourceResponseInfo());
@@ -712,5 +648,32 @@ TEST_F(WebURLLoaderImplTest, BrowserSideNavigationCommit) {
   EXPECT_EQ(kTestData, client()->received_data());
 }
 
+TEST_F(WebURLLoaderImplTest, ResponseIPAddress) {
+  GURL url("http://example.test/");
+
+  struct TestCase {
+    const char* ip;
+    const char* expected;
+  } cases[] = {
+      {"127.0.0.1", "127.0.0.1"},
+      {"123.123.123.123", "123.123.123.123"},
+      {"::1", "[::1]"},
+      {"2001:0db8:85a3:0000:0000:8a2e:0370:7334",
+       "[2001:0db8:85a3:0000:0000:8a2e:0370:7334]"},
+      {"2001:db8:85a3:0:0:8a2e:370:7334", "[2001:db8:85a3:0:0:8a2e:370:7334]"},
+      {"2001:db8:85a3::8a2e:370:7334", "[2001:db8:85a3::8a2e:370:7334]"},
+      {"::ffff:192.0.2.128", "[::ffff:192.0.2.128]"}};
+
+  for (const auto& test : cases) {
+    SCOPED_TRACE(test.ip);
+    content::ResourceResponseInfo info;
+    info.socket_address = net::HostPortPair(test.ip, 443);
+    blink::WebURLResponse response;
+    response.initialize();
+    WebURLLoaderImpl::PopulateURLResponse(url, info, &response, true);
+    EXPECT_EQ(test.expected, response.remoteIPAddress().utf8());
+  };
+}
+
 }  // namespace
 }  // namespace content
