@@ -9,7 +9,7 @@
 #include "base/containers/mru_cache.h"
 #include "base/macros.h"
 #include "base/synchronization/lock.h"
-#include "skia/ext/refptr.h"
+#include "third_party/skia/include/core/SkRefCnt.h"
 #include "third_party/skia/include/core/SkStream.h"
 #include "third_party/skia/include/core/SkTypeface.h"
 #include "third_party/skia/include/ports/SkFontConfigInterface.h"
@@ -20,32 +20,28 @@
 
 class SkString;
 
-namespace BASE_HASH_NAMESPACE {
-template <>
-struct hash<SkFontConfigInterface::FontIdentity> {
-  std::size_t operator()(const SkFontConfigInterface::FontIdentity& sp) const;
-};
-}
-
 namespace content {
 
+struct SkFontConfigInterfaceFontIdentityHash {
+  std::size_t operator()(const SkFontConfigInterface::FontIdentity& sp) const;
+};
 
 // FontConfig implementation for Skia that proxies out of process to get out
-// of the sandbox. See http://code.google.com/p/chromium/wiki/LinuxSandboxIPC
+// of the sandbox. See https://chromium.googlesource.com/chromium/src/+/master/docs/linux_sandbox_ipc.md
 class FontConfigIPC : public SkFontConfigInterface {
  public:
   explicit FontConfigIPC(int fd);
   ~FontConfigIPC() override;
 
   bool matchFamilyName(const char familyName[],
-                       SkTypeface::Style requested,
+                       SkFontStyle requested,
                        FontIdentity* outFontIdentifier,
                        SkString* outFamilyName,
-                       SkTypeface::Style* outStyle) override;
+                       SkFontStyle* outStyle) override;
 
   // Returns a new SkTypeface instance or a ref'ed one from the cache. The
   // caller should adopt the pointer.
-  SkTypeface* createTypeface(const FontIdentity& identity) override
+  sk_sp<SkTypeface> makeTypeface(const FontIdentity& identity) override
       WARN_UNUSED_RESULT;
 
   enum Method {
@@ -74,7 +70,9 @@ class FontConfigIPC : public SkFontConfigInterface {
   // ttc-indices or styles but the same fontconfig interface id. Since the usage
   // frequency of ttc indices is very low, and style is not used by clients of
   // this API, this seems okay.
-  base::HashingMRUCache<FontIdentity, skia::RefPtr<SkTypeface>>
+  base::HashingMRUCache<FontIdentity,
+                        sk_sp<SkTypeface>,
+                        SkFontConfigInterfaceFontIdentityHash>
       mapped_typefaces_;
 
   DISALLOW_COPY_AND_ASSIGN(FontConfigIPC);
