@@ -8,6 +8,7 @@
 #include <stdint.h>
 
 #include "content/common/content_export.h"
+#include "sandbox/win/src/sandbox_types.h"
 #include "sandbox/win/src/security_level.h"
 
 namespace base {
@@ -24,19 +25,21 @@ namespace content {
 
 // Wrapper around sandbox::TargetPolicy::SetJobLevel that checks if the sandbox
 // should be let to run without a job object assigned.
-void SetJobLevel(const base::CommandLine& cmd_line,
-                 sandbox::JobLevel job_level,
-                 uint32_t ui_exceptions,
-                 sandbox::TargetPolicy* policy);
+sandbox::ResultCode SetJobLevel(const base::CommandLine& cmd_line,
+                                sandbox::JobLevel job_level,
+                                uint32_t ui_exceptions,
+                                sandbox::TargetPolicy* policy);
 
 // Closes handles that are opened at process creation and initialization.
-void AddBaseHandleClosePolicy(sandbox::TargetPolicy* policy);
+sandbox::ResultCode AddBaseHandleClosePolicy(sandbox::TargetPolicy* policy);
 
 // Add AppContainer policy for |sid| on supported OS.
-void AddAppContainerPolicy(sandbox::TargetPolicy* policy, const wchar_t* sid);
+sandbox::ResultCode AddAppContainerPolicy(sandbox::TargetPolicy* policy,
+                                          const wchar_t* sid);
 
 // Add the win32k lockdown policy on supported OS.
-bool AddWin32kLockdownPolicy(sandbox::TargetPolicy* policy);
+sandbox::ResultCode AddWin32kLockdownPolicy(sandbox::TargetPolicy* policy,
+                                            bool enable_opm);
 
 bool InitBrokerServices(sandbox::BrokerServices* broker_services);
 
