@@ -6,6 +6,7 @@
 
 #include "base/command_line.h"
 #include "base/lazy_instance.h"
+#include "content/public/common/browser_plugin_guest_mode.h"
 #include "content/public/common/browser_side_navigation_policy.h"
 #include "content/public/common/content_client.h"
 #include "content/public/common/content_switches.h"
@@ -14,9 +15,10 @@ namespace content {
 
 // static
 bool SiteIsolationPolicy::AreCrossProcessFramesPossible() {
-  return base::CommandLine::ForCurrentProcess()->HasSwitch(
-             switches::kSitePerProcess) ||
-         GetContentClient()->IsSupplementarySiteIsolationModeEnabled();
+  return UseDedicatedProcessesForAllSites() ||
+         IsTopDocumentIsolationEnabled() ||
+         GetContentClient()->IsSupplementarySiteIsolationModeEnabled() ||
+         BrowserPluginGuestMode::UseCrossProcessFramesForGuests();
 }
 
 // static
@@ -25,6 +27,16 @@ bool SiteIsolationPolicy::UseDedicatedProcessesForAllSites() {
       switches::kSitePerProcess);
 }
 
+// static
+bool SiteIsolationPolicy::IsTopDocumentIsolationEnabled() {
+  // --site-per-process trumps --top-document-isolation.
+  if (UseDedicatedProcessesForAllSites())
+    return false;
+
+  return base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kTopDocumentIsolation);
+}
+
 // static
 bool SiteIsolationPolicy::UseSubframeNavigationEntries() {
   // Enable the new navigation history behavior if any manner of site isolation
@@ -33,9 +45,4 @@ bool SiteIsolationPolicy::UseSubframeNavigationEntries() {
   return AreCrossProcessFramesPossible() || IsBrowserSideNavigationEnabled();
 }
 
-// static
-bool SiteIsolationPolicy::IsSwappedOutStateForbidden() {
-  return true;
-}
-
 }  // namespace content
