@@ -2,12 +2,11 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
 #include <stdlib.h>
 
-#if defined(OS_WIN)
-#include <dwmapi.h>
-#include <windows.h>
-#endif
+#include <memory>
+#include <utility>
 
 #include "base/lazy_instance.h"
 #include "base/message_loop/message_loop.h"
@@ -22,11 +21,7 @@
 #include "build/build_config.h"
 #include "content/child/child_process.h"
 #include "content/common/content_constants_internal.h"
-#include "content/common/gpu/gpu_config.h"
-#include "content/common/gpu/gpu_memory_buffer_factory.h"
-#include "content/common/gpu/gpu_messages.h"
-#include "content/common/gpu/media/gpu_video_decode_accelerator.h"
-#include "content/common/gpu/media/gpu_video_encode_accelerator.h"
+#include "content/common/gpu_host_messages.h"
 #include "content/common/sandbox_linux/sandbox_linux.h"
 #include "content/gpu/gpu_child_thread.h"
 #include "content/gpu/gpu_process.h"
@@ -38,21 +33,37 @@
 #include "gpu/config/gpu_info_collector.h"
 #include "gpu/config/gpu_switches.h"
 #include "gpu/config/gpu_util.h"
+#include "gpu/ipc/common/gpu_memory_buffer_support.h"
+#include "gpu/ipc/service/gpu_config.h"
+#include "gpu/ipc/service/gpu_memory_buffer_factory.h"
 #include "ui/events/platform/platform_event_source.h"
 #include "ui/gl/gl_context.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_surface.h"
 #include "ui/gl/gl_switches.h"
 #include "ui/gl/gpu_switching_manager.h"
+#include "ui/gl/init/gl_factory.h"
+
+#if defined(OS_WIN)
+#include <dwmapi.h>
+#include <windows.h>
+#endif
+
+#if defined(OS_ANDROID)
+#include "base/trace_event/memory_dump_manager.h"
+#include "components/tracing/common/graphics_memory_dump_provider_android.h"
+#endif
 
 #if defined(OS_WIN)
-#include "base/win/windows_version.h"
 #include "base/win/scoped_com_initializer.h"
+#include "base/win/windows_version.h"
+#include "media/gpu/dxva_video_decode_accelerator_win.h"
 #include "sandbox/win/src/sandbox.h"
 #endif
 
 #if defined(USE_X11)
-#include "ui/base/x/x11_util.h"
+#include "ui/base/x/x11_util.h"     // nogncheck
+#include "ui/gfx/x/x11_switches.h"  // nogncheck
 #endif
 
 #if defined(OS_LINUX)
@@ -65,7 +76,7 @@
 #endif
 
 #if defined(OS_CHROMEOS) && defined(ARCH_CPU_X86_FAMILY)
-#include "content/common/gpu/media/vaapi_wrapper.h"
+#include "media/gpu/vaapi_wrapper.h"
 #endif
 
 #if defined(SANITIZER_COVERAGE)
@@ -73,7 +84,15 @@
 #include <sanitizer/coverage_interface.h>
 #endif
 
+#if defined(CYGPROFILE_INSTRUMENTATION)
+const int kGpuTimeout = 30000;
+#elif defined(OS_WIN)
+// Use a slightly longer timeout on Windows due to prevalence of slow and
+// infected machines.
+const int kGpuTimeout = 15000;
+#else
 const int kGpuTimeout = 10000;
+#endif
 
 namespace content {
 
@@ -105,8 +124,8 @@ bool GpuProcessLogMessageHandler(int severity,
                                  const std::string& str) {
   std::string header = str.substr(0, message_start);
   std::string message = str.substr(message_start);
-  deferred_messages.Get().push(new GpuHostMsg_OnLogMessage(
-      severity, header, message));
+  deferred_messages.Get().push(
+      new GpuHostMsg_OnLogMessage(severity, header, message));
   return false;
 }
 
@@ -135,6 +154,14 @@ int GpuMain(const MainFunctionParams& parameters) {
       SEM_NOOPENFILEERRORBOX);
 #elif defined(USE_X11)
   ui::SetDefaultX11ErrorHandlers();
+
+#if !defined(OS_CHROMEOS)
+  DCHECK(base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kWindowDepth));
+  DCHECK(base::CommandLine::ForCurrentProcess()->HasSwitch(
+      switches::kX11VisualID));
+#endif
+
 #endif
 
   logging::SetLogMessageHandler(GpuProcessLogMessageHandler);
@@ -164,6 +191,7 @@ int GpuMain(const MainFunctionParams& parameters) {
 #if defined(OS_WIN)
   // Use a UI message loop because ANGLE and the desktop GL platform can
   // create child windows to render to.
+  base::MessagePumpForGpu::InitFactory();
 #if defined(TOOLKIT_QT)
   base::MessageLoop main_message_loop(base::MessageLoop::TYPE_DEFAULT);
 #else
@@ -173,15 +201,15 @@ int GpuMain(const MainFunctionParams& parameters) {
   // We need a UI loop so that we can grab the Expose events. See GLSurfaceGLX
   // and https://crbug.com/326995.
   base::MessageLoop main_message_loop(base::MessageLoop::TYPE_UI);
-  scoped_ptr<ui::PlatformEventSource> event_source =
+  std::unique_ptr<ui::PlatformEventSource> event_source =
       ui::PlatformEventSource::CreateDefault();
 #elif defined(OS_LINUX)
   base::MessageLoop main_message_loop(base::MessageLoop::TYPE_DEFAULT);
 #elif defined(OS_MACOSX)
   // This is necessary for CoreAnimation layers hosted in the GPU process to be
   // drawn. See http://crbug.com/312462.
-  scoped_ptr<base::MessagePump> pump(new base::MessagePumpCFRunLoop());
-  base::MessageLoop main_message_loop(pump.Pass());
+  std::unique_ptr<base::MessagePump> pump(new base::MessagePumpCFRunLoop());
+  base::MessageLoop main_message_loop(std::move(pump));
 #else
   base::MessageLoop main_message_loop(base::MessageLoop::TYPE_IO);
 #endif
@@ -227,9 +255,15 @@ int GpuMain(const MainFunctionParams& parameters) {
   // Get vendor_id, device_id, driver_version from browser process through
   // commandline switches.
   GetGpuInfoFromCommandLine(gpu_info, command_line);
+  gpu_info.in_process_gpu = false;
 
 #if defined(OS_CHROMEOS) && defined(ARCH_CPU_X86_FAMILY)
-  VaapiWrapper::PreSandboxInitialization();
+  media::VaapiWrapper::PreSandboxInitialization();
+#endif
+
+#if defined(OS_ANDROID) || defined(OS_CHROMEOS)
+  // Set thread priority before sandbox initialization.
+  base::PlatformThread::SetCurrentThreadPriority(base::ThreadPriority::DISPLAY);
 #endif
 
   // Warm up resources that don't need access to GPUInfo.
@@ -253,13 +287,13 @@ int GpuMain(const MainFunctionParams& parameters) {
     bool gl_already_initialized = false;
 #if defined(OS_MACOSX)
     if (!command_line.HasSwitch(switches::kNoSandbox)) {
-      // On Mac, if the sandbox is enabled, then GLSurface::InitializeOneOff()
+      // On Mac, if the sandbox is enabled, then gl::init::InitializeGLOneOff()
       // is called from the sandbox warmup code before getting here.
       gl_already_initialized = true;
     }
 #endif
     if (command_line.HasSwitch(switches::kInProcessGPU)) {
-      // With in-process GPU, GLSurface::InitializeOneOff() is called from
+      // With in-process GPU, gl::init::InitializeGLOneOff() is called from
       // GpuChildThread before getting here.
       gl_already_initialized = true;
     }
@@ -267,8 +301,8 @@ int GpuMain(const MainFunctionParams& parameters) {
     // Load and initialize the GL implementation and locate the GL entry points.
     bool gl_initialized =
         gl_already_initialized
-            ? gfx::GetGLImplementation() != gfx::kGLImplementationNone
-            : gfx::GLSurface::InitializeOneOff();
+            ? gl::GetGLImplementation() != gl::kGLImplementationNone
+            : gl::init::InitializeGLOneOff();
     if (gl_initialized) {
       // We need to collect GL strings (VENDOR, RENDERER) for blacklisting
       // purposes. However, on Mac we don't actually use them. As documented in
@@ -284,14 +318,18 @@ int GpuMain(const MainFunctionParams& parameters) {
       if (!CollectGraphicsInfo(gpu_info))
         dead_on_arrival = true;
 
-#if defined(OS_CHROMEOS) || defined(OS_ANDROID)
-      // Recompute gpu driver bug workarounds - this is specifically useful
-      // on systems where vendor_id/device_id aren't available.
+      // Recompute gpu driver bug workarounds.
+      // This is necessary on systems where vendor_id/device_id aren't available
+      // (Chrome OS, Android) or where workarounds may be dependent on GL_VENDOR
+      // and GL_RENDERER strings which are lazily computed (Linux).
       if (!command_line.HasSwitch(switches::kDisableGpuDriverBugWorkarounds)) {
+        // TODO: this can not affect disabled extensions, since they're already
+        // initialized in the bindings. This should be moved before bindings
+        // initialization. However, populating GPUInfo fully works only on
+        // Android. Other platforms would need the bindings to query GL strings.
         gpu::ApplyGpuDriverBugWorkarounds(
             gpu_info, const_cast<base::CommandLine*>(&command_line));
       }
-#endif
 
 #if defined(OS_LINUX)
       initialized_gl_context = true;
@@ -308,7 +346,7 @@ int GpuMain(const MainFunctionParams& parameters) {
       UMA_HISTOGRAM_TIMES("GPU.CollectContextGraphicsInfo",
                           collect_context_time);
     } else {  // gl_initialized
-      VLOG(1) << "gfx::GLSurface::InitializeOneOff failed";
+      VLOG(1) << "gl::init::InitializeGLOneOff failed";
       dead_on_arrival = true;
     }
 
@@ -327,7 +365,7 @@ int GpuMain(const MainFunctionParams& parameters) {
     // OSMesa is expected to run very slowly, so disable the watchdog in that
     // case.
     if (enable_watchdog &&
-        gfx::GetGLImplementation() == gfx::kGLImplementationOSMesaGL) {
+        gl::GetGLImplementation() == gl::kGLImplementationOSMesaGL) {
       watchdog_thread->Stop();
       watchdog_thread = NULL;
     }
@@ -345,22 +383,22 @@ int GpuMain(const MainFunctionParams& parameters) {
 #elif defined(OS_MACOSX)
     gpu_info.sandboxed = Sandbox::SandboxIsCurrentlyActive();
 #endif
-
-    gpu_info.video_decode_accelerator_supported_profiles =
-        content::GpuVideoDecodeAccelerator::GetSupportedProfiles();
-    gpu_info.video_encode_accelerator_supported_profiles =
-        content::GpuVideoEncodeAccelerator::GetSupportedProfiles();
   } else {
     dead_on_arrival = true;
   }
 
   logging::SetLogMessageHandler(NULL);
 
-  scoped_ptr<GpuMemoryBufferFactory> gpu_memory_buffer_factory =
-      GpuMemoryBufferFactory::Create(
-          GpuChildThread::GetGpuMemoryBufferFactoryType());
+  std::unique_ptr<gpu::GpuMemoryBufferFactory> gpu_memory_buffer_factory;
+  if (gpu::GetNativeGpuMemoryBufferType() != gfx::EMPTY_BUFFER)
+    gpu_memory_buffer_factory = gpu::GpuMemoryBufferFactory::CreateNativeType();
 
-  GpuProcess gpu_process;
+  base::ThreadPriority io_thread_priority = base::ThreadPriority::NORMAL;
+#if defined(OS_ANDROID) || defined(OS_CHROMEOS)
+  io_thread_priority = base::ThreadPriority::DISPLAY;
+#endif
+
+  GpuProcess gpu_process(io_thread_priority);
 
   GpuChildThread* child_thread = new GpuChildThread(
       watchdog_thread.get(), dead_on_arrival, gpu_info, deferred_messages.Get(),
@@ -375,6 +413,12 @@ int GpuMain(const MainFunctionParams& parameters) {
   if (watchdog_thread.get())
     watchdog_thread->AddPowerObserver();
 
+#if defined(OS_ANDROID)
+  base::trace_event::MemoryDumpManager::GetInstance()->RegisterDumpProvider(
+      tracing::GraphicsMemoryDumpProvider::GetInstance(), "AndroidGraphics",
+      nullptr);
+#endif
+
   {
     TRACE_EVENT0("gpu", "Run Message Loop");
     main_message_loop.Run();
@@ -404,6 +448,37 @@ void GetGpuInfoFromCommandLine(gpu::GPUInfo& gpu_info,
       command_line.GetSwitchValueASCII(switches::kGpuDriverVendor);
   gpu_info.driver_version =
       command_line.GetSwitchValueASCII(switches::kGpuDriverVersion);
+  gpu_info.driver_date =
+      command_line.GetSwitchValueASCII(switches::kGpuDriverDate);
+  gpu::ParseSecondaryGpuDevicesFromCommandLine(command_line, &gpu_info);
+
+  // Set active gpu device.
+  if (command_line.HasSwitch(switches::kGpuActiveVendorID) &&
+      command_line.HasSwitch(switches::kGpuActiveDeviceID)) {
+    uint32_t active_vendor_id = 0;
+    uint32_t active_device_id = 0;
+    success = base::HexStringToUInt(
+        command_line.GetSwitchValueASCII(switches::kGpuActiveVendorID),
+        &active_vendor_id);
+    DCHECK(success);
+    success = base::HexStringToUInt(
+        command_line.GetSwitchValueASCII(switches::kGpuActiveDeviceID),
+        &active_device_id);
+    DCHECK(success);
+    if (gpu_info.gpu.vendor_id == active_vendor_id &&
+        gpu_info.gpu.device_id == active_device_id) {
+      gpu_info.gpu.active = true;
+    } else {
+      for (size_t i = 0; i < gpu_info.secondary_gpus.size(); ++i) {
+        if (gpu_info.secondary_gpus[i].vendor_id == active_vendor_id &&
+            gpu_info.secondary_gpus[i].device_id == active_device_id) {
+          gpu_info.secondary_gpus[i].active = true;
+          break;
+        }
+      }
+    }
+  }
+
   GetContentClient()->SetGpuInfo(gpu_info);
 }
 
@@ -414,11 +489,17 @@ bool WarmUpSandbox(const base::CommandLine& command_line) {
     // platforms.
     (void) base::RandUint64();
   }
+
+#if defined(OS_WIN)
+  media::DXVAVideoDecodeAccelerator::PreSandboxInitialization();
+#endif
   return true;
 }
 
 #if !defined(OS_MACOSX)
 bool CollectGraphicsInfo(gpu::GPUInfo& gpu_info) {
+  TRACE_EVENT0("gpu,startup", "Collect Graphics Info");
+
   bool res = true;
   gpu::CollectInfoResult result = gpu::CollectContextGraphicsInfo(&gpu_info);
   switch (result) {
@@ -454,19 +535,19 @@ bool CanAccessNvidiaDeviceFile() {
 #endif
 
 void CreateDummyGlContext() {
-  scoped_refptr<gfx::GLSurface> surface(
-      gfx::GLSurface::CreateOffscreenGLSurface(gfx::Size()));
+  scoped_refptr<gl::GLSurface> surface(
+      gl::init::CreateOffscreenGLSurface(gfx::Size()));
   if (!surface.get()) {
-    DVLOG(1) << "gfx::GLSurface::CreateOffscreenGLSurface failed";
+    DVLOG(1) << "gl::init::CreateOffscreenGLSurface failed";
     return;
   }
 
   // On Linux, this is needed to make sure /dev/nvidiactl has
   // been opened and its descriptor cached.
-  scoped_refptr<gfx::GLContext> context(gfx::GLContext::CreateGLContext(
-      NULL, surface.get(), gfx::PreferDiscreteGpu));
+  scoped_refptr<gl::GLContext> context(
+      gl::init::CreateGLContext(NULL, surface.get(), gl::PreferDiscreteGpu));
   if (!context.get()) {
-    DVLOG(1) << "gfx::GLContext::CreateGLContext failed";
+    DVLOG(1) << "gl::init::CreateGLContext failed";
     return;
   }
 
@@ -474,7 +555,7 @@ void CreateDummyGlContext() {
   if (context->MakeCurrent(surface.get())) {
     context->ReleaseCurrent(surface.get());
   } else {
-    DVLOG(1)  << "gfx::GLContext::MakeCurrent failed";
+    DVLOG(1) << "gl::GLContext::MakeCurrent failed";
   }
 }
 
@@ -493,7 +574,7 @@ void WarmUpSandboxNvidia(const gpu::GPUInfo& gpu_info,
 bool StartSandboxLinux(const gpu::GPUInfo& gpu_info,
                        GpuWatchdogThread* watchdog_thread,
                        bool should_initialize_gl_context) {
-  TRACE_EVENT0("gpu", "Initialize sandbox");
+  TRACE_EVENT0("gpu,startup", "Initialize sandbox");
 
   bool res = false;
 
@@ -530,20 +611,13 @@ bool StartSandboxLinux(const gpu::GPUInfo& gpu_info,
 
 #if defined(OS_WIN)
 bool StartSandboxWindows(const sandbox::SandboxInterfaceInfo* sandbox_info) {
-  TRACE_EVENT0("gpu", "Lower token");
+  TRACE_EVENT0("gpu,startup", "Lower token");
 
   // For Windows, if the target_services interface is not zero, the process
   // is sandboxed and we must call LowerToken() before rendering untrusted
   // content.
   sandbox::TargetServices* target_services = sandbox_info->target_services;
   if (target_services) {
-#if defined(ADDRESS_SANITIZER)
-    // Bind and leak dbghelp.dll before the token is lowered, otherwise
-    // AddressSanitizer will crash when trying to symbolize a report.
-    if (!LoadLibraryA("dbghelp.dll"))
-      return false;
-#endif
-
     target_services->LowerToken();
     return true;
   }
