@@ -22,7 +22,6 @@ class FilePath;
 namespace content {
 
 class BrowserContext;
-class PluginProcessHost;
 class PluginServiceFilter;
 class ResourceContext;
 struct PepperPluginInfo;
@@ -54,9 +53,6 @@ class PluginService {
   // Must be called on the instance to finish initialization.
   virtual void Init() = 0;
 
-  // Starts watching for changes in the list of installed plugins.
-  virtual void StartWatchingPlugins() = 0;
-
   // Gets the plugin in the list of plugins that matches the given url and mime
   // type. Returns true if the data is frome a stale plugin list, false if it
   // is up to date. This can be called from any thread.
@@ -106,9 +102,6 @@ class PluginService {
   virtual void SetFilter(PluginServiceFilter* filter) = 0;
   virtual PluginServiceFilter* GetFilter() = 0;
 
-  // If the plugin with the given path is running, cleanly shuts it down.
-  virtual void ForcePluginShutdown(const base::FilePath& plugin_path) = 0;
-
   // Used to monitor plugin stability. An unstable plugin is one that has
   // crashed more than a set number of times in a set time period.
   virtual bool IsPluginUnstable(const base::FilePath& plugin_path) = 0;
@@ -117,14 +110,6 @@ class PluginService {
   // of whether they are already loaded.
   virtual void RefreshPlugins() = 0;
 
-  // Add/Remove an extra plugin to load when we actually do the loading.  Must
-  // be called before the plugins have been loaded.
-  virtual void AddExtraPluginPath(const base::FilePath& path) = 0;
-  virtual void RemoveExtraPluginPath(const base::FilePath& path) = 0;
-
-  // Same as above, but specifies a directory in which to search for plugins.
-  virtual void AddExtraPluginDir(const base::FilePath& path) = 0;
-
   // Register an internal plugin with the specified plugin information.
   // An internal plugin must be registered before it can
   // be loaded using PluginList::LoadPlugin().
@@ -140,27 +125,6 @@ class PluginService {
   // Gets a list of all the registered internal plugins.
   virtual void GetInternalPlugins(std::vector<WebPluginInfo>* plugins) = 0;
 
-  // Returns true iff NPAPI plugins are supported on the current platform.
-  // This can be called from any thread.
-  virtual bool NPAPIPluginsSupported() = 0;
-
-  // This is equivalent to specifying kDisablePluginsDiscovery, but is useful
-  // for unittests.
-  virtual void DisablePluginsDiscoveryForTesting() = 0;
-
-#if defined(OS_MACOSX)
-  // Called when the application is made active so that modal plugin windows can
-  // be made forward too.
-  virtual void AppActivated() = 0;
-#elif defined(OS_WIN)
-  // Returns the name and version of a plugin HWND. If the HWND isn't a valid
-  // plugin, returns false.
-  // This can be called from any thread.
-  virtual bool GetPluginInfoFromWindow(HWND window,
-                                       base::string16* plugin_name,
-                                       base::string16* plugin_version) = 0;
-#endif
-
   // Returns true iff PPAPI "dev channel" methods are supported.
   virtual bool PpapiDevChannelSupported(BrowserContext* browser_context,
                                         const GURL& document_url) = 0;
