@@ -4,8 +4,10 @@
 
 #include "content/public/browser/resource_dispatcher_host_delegate.h"
 
+#include "content/public/browser/navigation_data.h"
 #include "content/public/browser/resource_request_info.h"
 #include "content/public/browser/stream_info.h"
+#include "net/ssl/client_cert_store.h"
 
 namespace content {
 
@@ -30,7 +32,6 @@ void ResourceDispatcherHostDelegate::DownloadStarting(
     ResourceContext* resource_context,
     int child_id,
     int route_id,
-    int request_id,
     bool is_content_initiated,
     bool must_download,
     ScopedVector<ResourceThrottle>* throttles) {
@@ -49,7 +50,8 @@ bool ResourceDispatcherHostDelegate::HandleExternalProtocol(
     const ResourceRequestInfo::WebContentsGetter& web_contents_getter,
     bool is_main_frame,
     ui::PageTransition page_transition,
-    bool has_user_gesture) {
+    bool has_user_gesture,
+    ResourceContext* resource_context) {
   return true;
 }
 
@@ -70,8 +72,7 @@ bool ResourceDispatcherHostDelegate::ShouldInterceptResourceAsStream(
 
 void ResourceDispatcherHostDelegate::OnStreamCreated(
     net::URLRequest* request,
-    scoped_ptr<content::StreamInfo> stream) {
-}
+    std::unique_ptr<content::StreamInfo> stream) {}
 
 void ResourceDispatcherHostDelegate::OnResponseStarted(
     net::URLRequest* request,
@@ -97,7 +98,15 @@ bool ResourceDispatcherHostDelegate::ShouldEnableLoFiMode(
   return false;
 }
 
-ResourceDispatcherHostDelegate::ResourceDispatcherHostDelegate() {
+NavigationData* ResourceDispatcherHostDelegate::GetNavigationData(
+    net::URLRequest* request) const {
+  return nullptr;
+}
+
+std::unique_ptr<net::ClientCertStore>
+ResourceDispatcherHostDelegate::CreateClientCertStore(
+    ResourceContext* resource_context) {
+  return std::unique_ptr<net::ClientCertStore>();
 }
 
 ResourceDispatcherHostDelegate::~ResourceDispatcherHostDelegate() {
