@@ -5,10 +5,14 @@
 #ifndef CONTENT_PUBLIC_BROWSER_TRACING_DELEGATE_H_
 #define CONTENT_PUBLIC_BROWSER_TRACING_DELEGATE_H_
 
-#include "base/memory/scoped_ptr.h"
+#include <memory>
+#include <string>
+
+#include "base/callback.h"
 #include "content/common/content_export.h"
 
 namespace base {
+class DictionaryValue;
 class Time;
 }
 
@@ -17,9 +21,12 @@ class URLRequestContextGetter;
 }
 
 namespace content {
-struct BackgroundTracingConfig;
+class BackgroundTracingConfig;
 class TraceUploader;
 
+typedef base::Callback<bool(const std::string& metadata_name)>
+    MetadataFilterPredicate;
+
 // This can be implemented by the embedder to provide functionality for the
 // about://tracing WebUI.
 class CONTENT_EXPORT TracingDelegate {
@@ -27,7 +34,7 @@ class CONTENT_EXPORT TracingDelegate {
   virtual ~TracingDelegate() {}
 
   // Provide trace uploading functionality; see trace_uploader.h.
-  virtual scoped_ptr<TraceUploader> GetTraceUploader(
+  virtual std::unique_ptr<TraceUploader> GetTraceUploader(
       net::URLRequestContextGetter* request_context) = 0;
 
   // This can be used to veto a particular background tracing scenario.
@@ -38,6 +45,11 @@ class CONTENT_EXPORT TracingDelegate {
   virtual bool IsAllowedToEndBackgroundScenario(
       const content::BackgroundTracingConfig& config,
       bool requires_anonymized_data);
+
+  // Used to add any additional metadata to traces.
+  virtual void GenerateMetadataDict(base::DictionaryValue* metadata_dict) {}
+
+  virtual MetadataFilterPredicate GetMetadataFilterPredicate();
 };
 
 }  // namespace content
