@@ -153,10 +153,9 @@ JavaScriptDialogManager* WebContentsDelegate::GetJavaScriptDialogManager(
   return nullptr;
 }
 
-scoped_ptr<BluetoothChooser> WebContentsDelegate::RunBluetoothChooser(
-    WebContents* web_contents,
-    const BluetoothChooser::EventHandler& event_handler,
-    const GURL& origin) {
+std::unique_ptr<BluetoothChooser> WebContentsDelegate::RunBluetoothChooser(
+    RenderFrameHost* frame,
+    const BluetoothChooser::EventHandler& event_handler) {
   return nullptr;
 }
 
@@ -187,9 +186,8 @@ void WebContentsDelegate::RequestMediaAccessPermission(
     const MediaResponseCallback& callback) {
   LOG(ERROR) << "WebContentsDelegate::RequestMediaAccessPermission: "
              << "Not supported.";
-  callback.Run(MediaStreamDevices(),
-               MEDIA_DEVICE_NOT_SUPPORTED,
-               scoped_ptr<MediaStreamUI>());
+  callback.Run(MediaStreamDevices(), MEDIA_DEVICE_NOT_SUPPORTED,
+               std::unique_ptr<MediaStreamUI>());
 }
 
 bool WebContentsDelegate::CheckMediaAccessPermission(
@@ -259,4 +257,8 @@ void WebContentsDelegate::ShowCertificateViewerInDevTools(
     int cert_id) {
 }
 
+void WebContentsDelegate::RequestAppBannerFromDevTools(
+    content::WebContents* web_contents) {
+}
+
 }  // namespace content
