@@ -12,7 +12,6 @@
 #include "base/macros.h"
 #include "base/strings/stringprintf.h"
 #include "base/synchronization/cancellation_flag.h"
-#include "base/thread_task_runner_handle.h"
 #include "base/threading/platform_thread.h"
 #include "base/trace_event/trace_event.h"
 #include "base/trace_event/trace_event_argument.h"
@@ -106,7 +105,7 @@ class SampleRecord {
   void Collect(v8::Isolate* isolate,
                base::TimeTicks timestamp,
                const v8::RegisterState& state);
-  scoped_refptr<ConvertableToTraceFormat> ToTraceFormat() const;
+  std::unique_ptr<ConvertableToTraceFormat> ToTraceFormat() const;
 
  private:
   base::TimeTicks timestamp_;
@@ -128,8 +127,9 @@ void SampleRecord::Collect(v8::Isolate* isolate,
   vm_state_ = sample_info.vm_state;
 }
 
-scoped_refptr<ConvertableToTraceFormat> SampleRecord::ToTraceFormat() const {
-  scoped_refptr<TracedValue> data(new TracedValue());
+std::unique_ptr<ConvertableToTraceFormat> SampleRecord::ToTraceFormat() const {
+  std::unique_ptr<base::trace_event::TracedValue> data(
+      new base::trace_event::TracedValue());
   const char* vm_state = nullptr;
   switch (vm_state_) {
     case v8::StateTag::JS:
@@ -159,7 +159,7 @@ scoped_refptr<ConvertableToTraceFormat> SampleRecord::ToTraceFormat() const {
     data->AppendString(PtrToString(frames_[i]));
   }
   data->EndArray();
-  return data;
+  return std::move(data);
 }
 
 }  // namespace
@@ -169,7 +169,7 @@ class Sampler {
  public:
   ~Sampler();
 
-  static scoped_ptr<Sampler> CreateForCurrentThread();
+  static std::unique_ptr<Sampler> CreateForCurrentThread();
   static Sampler* GetInstance() { return tls_instance_.Pointer()->Get(); }
 
   // These methods are called from the sampling thread.
@@ -196,7 +196,7 @@ class Sampler {
 
   static void InstallJitCodeEventHandler(Isolate* isolate, void* data);
   static void HandleJitCodeEvent(const v8::JitCodeEvent* event);
-  static scoped_refptr<ConvertableToTraceFormat> JitCodeEventToTraceFormat(
+  static std::unique_ptr<ConvertableToTraceFormat> JitCodeEventToTraceFormat(
       const v8::JitCodeEvent* event);
 
   void InjectPendingEvents();
@@ -206,7 +206,7 @@ class Sampler {
 
   PlatformData platform_data_;
   Isolate* isolate_;
-  scoped_ptr<SamplingQueue> samples_data_;
+  std::unique_ptr<SamplingQueue> samples_data_;
   base::subtle::Atomic32 code_added_events_count_;
   base::subtle::Atomic32 samples_count_;
   int code_added_events_to_collect_for_test_;
@@ -236,8 +236,8 @@ Sampler::~Sampler() {
 }
 
 // static
-scoped_ptr<Sampler> Sampler::CreateForCurrentThread() {
-  return scoped_ptr<Sampler>(new Sampler());
+std::unique_ptr<Sampler> Sampler::CreateForCurrentThread() {
+  return std::unique_ptr<Sampler>(new Sampler());
 }
 
 void Sampler::Start() {
@@ -332,23 +332,23 @@ void Sampler::HandleJitCodeEvent(const v8::JitCodeEvent* event) {
     return;
   switch (event->type) {
     case v8::JitCodeEvent::CODE_ADDED:
-      TRACE_EVENT_INSTANT1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
-                           "JitCodeAdded", TRACE_EVENT_SCOPE_THREAD, "data",
-                           JitCodeEventToTraceFormat(event));
+      TRACE_EVENT_METADATA1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
+                            "JitCodeAdded", "data",
+                            JitCodeEventToTraceFormat(event));
       base::subtle::NoBarrier_AtomicIncrement(
           &sampler->code_added_events_count_, 1);
       break;
 
     case v8::JitCodeEvent::CODE_MOVED:
-      TRACE_EVENT_INSTANT1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
-                           "JitCodeMoved", TRACE_EVENT_SCOPE_THREAD, "data",
-                           JitCodeEventToTraceFormat(event));
+      TRACE_EVENT_METADATA1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
+                            "JitCodeMoved", "data",
+                            JitCodeEventToTraceFormat(event));
       break;
 
     case v8::JitCodeEvent::CODE_REMOVED:
-      TRACE_EVENT_INSTANT1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
-                           "JitCodeRemoved", TRACE_EVENT_SCOPE_THREAD, "data",
-                           JitCodeEventToTraceFormat(event));
+      TRACE_EVENT_METADATA1(TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"),
+                            "JitCodeRemoved", "data",
+                            JitCodeEventToTraceFormat(event));
       break;
 
     case v8::JitCodeEvent::CODE_ADD_LINE_POS_INFO:
@@ -359,33 +359,36 @@ void Sampler::HandleJitCodeEvent(const v8::JitCodeEvent* event) {
 }
 
 // static
-scoped_refptr<ConvertableToTraceFormat> Sampler::JitCodeEventToTraceFormat(
+std::unique_ptr<ConvertableToTraceFormat> Sampler::JitCodeEventToTraceFormat(
     const v8::JitCodeEvent* event) {
   switch (event->type) {
     case v8::JitCodeEvent::CODE_ADDED: {
-      scoped_refptr<TracedValue> data(new TracedValue());
+      std::unique_ptr<base::trace_event::TracedValue> data(
+          new base::trace_event::TracedValue());
       data->SetString("code_start", PtrToString(event->code_start));
       data->SetInteger("code_len", static_cast<unsigned>(event->code_len));
       data->SetString("name", std::string(event->name.str, event->name.len));
       if (!event->script.IsEmpty()) {
         data->SetInteger("script_id", event->script->GetId());
       }
-      return data;
+      return std::move(data);
     }
 
     case v8::JitCodeEvent::CODE_MOVED: {
-      scoped_refptr<TracedValue> data(new TracedValue());
+      std::unique_ptr<base::trace_event::TracedValue> data(
+          new base::trace_event::TracedValue());
       data->SetString("code_start", PtrToString(event->code_start));
       data->SetInteger("code_len", static_cast<unsigned>(event->code_len));
       data->SetString("new_code_start", PtrToString(event->new_code_start));
-      return data;
+      return std::move(data);
     }
 
     case v8::JitCodeEvent::CODE_REMOVED: {
-      scoped_refptr<TracedValue> data(new TracedValue());
+      std::unique_ptr<base::trace_event::TracedValue> data(
+          new base::trace_event::TracedValue());
       data->SetString("code_start", PtrToString(event->code_start));
       data->SetInteger("code_len", static_cast<unsigned>(event->code_len));
-      return data;
+      return std::move(data);
     }
 
     case v8::JitCodeEvent::CODE_ADD_LINE_POS_INFO:
@@ -576,29 +579,34 @@ void V8SamplingThread::Stop() {
 V8SamplingProfiler::V8SamplingProfiler(bool underTest)
     : sampling_thread_(nullptr),
       render_thread_sampler_(Sampler::CreateForCurrentThread()),
-      task_runner_(base::ThreadTaskRunnerHandle::Get()) {
+      weak_factory_(this) {
   DCHECK(underTest || RenderThreadImpl::current());
+  DCHECK(thread_checker_.CalledOnValidThread());
   // Force the "v8.cpu_profile*" categories to show up in the trace viewer.
   TraceLog::GetCategoryGroupEnabled(
       TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile"));
   TraceLog::GetCategoryGroupEnabled(
       TRACE_DISABLED_BY_DEFAULT("v8.cpu_profile.hires"));
-  TraceLog::GetInstance()->AddEnabledStateObserver(this);
+  TraceLog::GetInstance()->AddAsyncEnabledStateObserver(
+      weak_factory_.GetWeakPtr());
 }
 
 V8SamplingProfiler::~V8SamplingProfiler() {
-  TraceLog::GetInstance()->RemoveEnabledStateObserver(this);
+  DCHECK(thread_checker_.CalledOnValidThread());
+  TraceLog::GetInstance()->RemoveAsyncEnabledStateObserver(this);
   DCHECK(!sampling_thread_.get());
 }
 
 void V8SamplingProfiler::StartSamplingThread() {
   DCHECK(!sampling_thread_.get());
+  DCHECK(thread_checker_.CalledOnValidThread());
   sampling_thread_.reset(new V8SamplingThread(
       render_thread_sampler_.get(), waitable_event_for_testing_.get()));
   sampling_thread_->Start();
 }
 
 void V8SamplingProfiler::StopSamplingThread() {
+  DCHECK(thread_checker_.CalledOnValidThread());
   if (!sampling_thread_.get())
     return;
   sampling_thread_->Stop();
@@ -620,22 +628,20 @@ void V8SamplingProfiler::OnTraceLogEnabled() {
   if (record_mode == base::trace_event::TraceRecordMode::RECORD_CONTINUOUSLY)
     return;
 
-  task_runner_->PostTask(FROM_HERE,
-                         base::Bind(&V8SamplingProfiler::StartSamplingThread,
-                                    base::Unretained(this)));
+  StartSamplingThread();
 }
 
 void V8SamplingProfiler::OnTraceLogDisabled() {
-  task_runner_->PostTask(FROM_HERE,
-                         base::Bind(&V8SamplingProfiler::StopSamplingThread,
-                                    base::Unretained(this)));
+  StopSamplingThread();
 }
 
 void V8SamplingProfiler::EnableSamplingEventForTesting(int code_added_events,
                                                        int sample_events) {
   render_thread_sampler_->SetEventsToCollectForTest(code_added_events,
                                                     sample_events);
-  waitable_event_for_testing_.reset(new base::WaitableEvent(false, false));
+  waitable_event_for_testing_.reset(
+      new base::WaitableEvent(base::WaitableEvent::ResetPolicy::AUTOMATIC,
+                              base::WaitableEvent::InitialState::NOT_SIGNALED));
 }
 
 void V8SamplingProfiler::WaitSamplingEventForTesting() {
