@@ -7,16 +7,16 @@
 #include <utility>
 
 #include "base/bind.h"
+#include "base/location.h"
 #include "base/logging.h"
-#include "base/message_loop/message_loop.h"
+#include "base/single_thread_task_runner.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "content/child/image_decoder.h"
 #include "content/public/renderer/render_frame.h"
+#include "content/public/renderer/render_thread.h"
 #include "content/renderer/fetchers/multi_resolution_image_resource_fetcher.h"
-#include "mojo/common/url_type_converters.h"
-#include "mojo/converters/geometry/geometry_type_converters.h"
 #include "net/base/data_url.h"
 #include "skia/ext/image_operations.h"
-#include "skia/public/type_converters.h"
 #include "third_party/WebKit/public/platform/WebURLRequest.h"
 #include "third_party/WebKit/public/platform/WebVector.h"
 #include "third_party/WebKit/public/web/WebLocalFrame.h"
@@ -26,6 +26,7 @@
 #include "ui/gfx/skbitmap_operations.h"
 #include "url/url_constants.h"
 
+using blink::WebCachePolicy;
 using blink::WebFrame;
 using blink::WebVector;
 using blink::WebURL;
@@ -106,7 +107,11 @@ void FilterAndResizeImagesForMaximalSize(
     return;
   // Proportionally resize the minimal image to fit in a box of size
   // |max_image_size|.
-  images->push_back(ResizeImage(*min_image, max_image_size));
+  SkBitmap resized = ResizeImage(*min_image, max_image_size);
+  // Drop null or empty SkBitmap.
+  if (resized.drawsNothing())
+    return;
+  images->push_back(resized);
   original_image_sizes->push_back(
       gfx::Size(min_image->width(), min_image->height()));
 }
@@ -117,45 +122,51 @@ namespace content {
 
 ImageDownloaderImpl::ImageDownloaderImpl(
     RenderFrame* render_frame,
-    mojo::InterfaceRequest<image_downloader::ImageDownloader> request)
+    mojo::InterfaceRequest<content::mojom::ImageDownloader> request)
     : RenderFrameObserver(render_frame), binding_(this, std::move(request)) {
   DCHECK(render_frame);
+  RenderThread::Get()->AddObserver(this);
 }
 
 ImageDownloaderImpl::~ImageDownloaderImpl() {
+  RenderThread::Get()->RemoveObserver(this);
 }
 
 // static
 void ImageDownloaderImpl::CreateMojoService(
     RenderFrame* render_frame,
-    mojo::InterfaceRequest<image_downloader::ImageDownloader> request) {
+    mojo::InterfaceRequest<content::mojom::ImageDownloader> request) {
   DVLOG(1) << "ImageDownloaderImpl::CreateService";
   DCHECK(render_frame);
 
   new ImageDownloaderImpl(render_frame, std::move(request));
 }
 
-// ImageDownloader methods:
-void ImageDownloaderImpl::DownloadImage(
-    image_downloader::DownloadRequestPtr req,
-    const DownloadImageCallback& callback) {
-  const GURL image_url = req->url.To<GURL>();
-  bool is_favicon = req->is_favicon;
-  uint32_t max_image_size = req->max_bitmap_size;
-  bool bypass_cache = req->bypass_cache;
+// Ensure all loaders cleared before calling blink::shutdown.
+void ImageDownloaderImpl::OnRenderProcessShutdown() {
+  image_fetchers_.clear();
+}
 
+// ImageDownloader methods:
+void ImageDownloaderImpl::DownloadImage(const GURL& image_url,
+                                        bool is_favicon,
+                                        uint32_t max_bitmap_size,
+                                        bool bypass_cache,
+                                        const DownloadImageCallback& callback) {
   std::vector<SkBitmap> result_images;
   std::vector<gfx::Size> result_original_image_sizes;
 
   if (image_url.SchemeIs(url::kDataScheme)) {
     SkBitmap data_image = ImageFromDataUrl(image_url);
-    if (!data_image.empty()) {
-      result_images.push_back(ResizeImage(data_image, max_image_size));
+    SkBitmap resized = ResizeImage(data_image, max_bitmap_size);
+    // Drop null or empty SkBitmap.
+    if (!resized.drawsNothing()) {
+      result_images.push_back(resized);
       result_original_image_sizes.push_back(
           gfx::Size(data_image.width(), data_image.height()));
     }
   } else {
-    if (FetchImage(image_url, is_favicon, max_image_size, bypass_cache,
+    if (FetchImage(image_url, is_favicon, max_bitmap_size, bypass_cache,
                    callback)) {
       // Will complete asynchronously via ImageDownloaderImpl::DidFetchImage
       return;
@@ -177,8 +188,8 @@ bool ImageDownloaderImpl::FetchImage(const GURL& image_url,
   image_fetchers_.push_back(new MultiResolutionImageResourceFetcher(
       image_url, frame, 0, is_favicon ? WebURLRequest::RequestContextFavicon
                                       : WebURLRequest::RequestContextImage,
-      bypass_cache ? WebURLRequest::ReloadBypassingCache
-                   : WebURLRequest::UseProtocolCachePolicy,
+      bypass_cache ? WebCachePolicy::BypassingCache
+                   : WebCachePolicy::UseProtocolCachePolicy,
       base::Bind(&ImageDownloaderImpl::DidFetchImage, base::Unretained(this),
                  max_image_size, callback)));
   return true;
@@ -203,7 +214,7 @@ void ImageDownloaderImpl::DidFetchImage(
       std::find(image_fetchers_.begin(), image_fetchers_.end(), fetcher);
   if (iter != image_fetchers_.end()) {
     image_fetchers_.weak_erase(iter);
-    base::MessageLoop::current()->DeleteSoon(FROM_HERE, fetcher);
+    base::ThreadTaskRunnerHandle::Get()->DeleteSoon(FROM_HERE, fetcher);
   }
 }
 
@@ -212,15 +223,12 @@ void ImageDownloaderImpl::ReplyDownloadResult(
     const std::vector<SkBitmap>& result_images,
     const std::vector<gfx::Size>& result_original_image_sizes,
     const DownloadImageCallback& callback) {
-  image_downloader::DownloadResultPtr result =
-      image_downloader::DownloadResult::New();
+  callback.Run(http_status_code, mojo::Array<SkBitmap>::From(result_images),
+               result_original_image_sizes);
+}
 
-  result->http_status_code = http_status_code;
-  result->images = mojo::Array<skia::BitmapPtr>::From(result_images);
-  result->original_image_sizes =
-      mojo::Array<mojo::SizePtr>::From(result_original_image_sizes);
-
-  callback.Run(std::move(result));
+void ImageDownloaderImpl::OnDestruct() {
+  delete this;
 }
 
 }  // namespace content
