@@ -9,16 +9,17 @@
 
 #include "base/files/file.h"
 #include "base/macros.h"
+#include "base/threading/thread_checker.h"
 #include "content/common/content_export.h"
 #include "content/public/common/media_stream_request.h"
 #include "third_party/WebKit/public/platform/WebMediaConstraints.h"
-#include "third_party/libjingle/source/talk/app/webrtc/mediastreaminterface.h"
+#include "third_party/webrtc/api/mediastreaminterface.h"
+#include "third_party/webrtc/media/base/mediachannel.h"
 #include "third_party/webrtc/modules/audio_processing/include/audio_processing.h"
 
 namespace webrtc {
 
 class EchoCancellation;
-class MediaConstraintsInterface;
 class TypingDetection;
 
 }
@@ -28,7 +29,6 @@ namespace content {
 class RTCMediaConstraints;
 
 using webrtc::AudioProcessing;
-using webrtc::MediaConstraintsInterface;
 
 // A helper class to parse audio constraints from a blink::WebMediaConstraints
 // object.
@@ -48,11 +48,13 @@ class CONTENT_EXPORT MediaAudioConstraints {
   static const char kGoogTypingNoiseDetection[];
   static const char kGoogAudioMirroring[];
 
-  // Merge |constraints| with |kDefaultAudioConstraints|. For any key which
-  // exists in both, the value from |constraints| is maintained, including its
-  // mandatory/optional status. New values from |kDefaultAudioConstraints| will
-  // be added with optional status.
-  static void ApplyFixedAudioConstraints(RTCMediaConstraints* constraints);
+  // Merge |options| with |kDefaultAudioConstraints|. For any key which
+  // exists in both, the value from |options| is maintained.
+  // New values from |kDefaultAudioConstraints| will
+  // be added.
+  // TODO(hta): Switch to an interface without "cricket::" when webrtc has one.
+
+  static void ApplyFixedAudioConstraints(cricket::AudioOptions* options);
 
   // |effects| is the bitmasks telling whether certain platform
   // hardware audio effects are enabled, like hardware echo cancellation. If
@@ -62,17 +64,16 @@ class CONTENT_EXPORT MediaAudioConstraints {
                         int effects);
   virtual ~MediaAudioConstraints();
 
-  // Gets the property of the constraint named by |key| in |constraints_|.
-  // Returns the constraint's value if the key is found; otherwise returns the
-  // default value of the constraint.
-  // Note, for constraint of |kEchoCancellation| or |kGoogEchoCancellation|,
-  // clients should use GetEchoCancellationProperty().
-  bool GetProperty(const std::string& key) const;
-
-  // Gets the property of the constraint named by |key| in |constraints_| as a
-  // string. Returns the constraint's string value if the key is found;
-  // otherwise returns an empty string.
-  std::string GetPropertyAsString(const std::string& key) const;
+  bool GetGoogAudioMirroring() const;
+  bool GetGoogAutoGainControl() const;
+  bool GetGoogExperimentalEchoCancellation() const;
+  bool GetGoogTypingNoiseDetection() const;
+  bool GetGoogNoiseSuppression() const;
+  bool GetGoogExperimentalNoiseSuppression() const;
+  bool GetGoogBeamforming() const;
+  bool GetGoogHighpassFilter() const;
+  bool GetGoogExperimentalAutoGainControl() const;
+  std::string GetGoogArrayGeometry() const;
 
   // Gets the property of echo cancellation defined in |constraints_|. The
   // returned value depends on a combination of |effects_|, |kEchoCancellation|
@@ -83,12 +84,14 @@ class CONTENT_EXPORT MediaAudioConstraints {
   // Otherwise return false.
   bool IsValid() const;
 
+  // Exposed for testing.
+  bool default_audio_processing_constraint_value() const {
+    return default_audio_processing_constraint_value_;
+  }
+
  private:
   // Gets the default value of constraint named by |key| in |constraints|.
-  bool GetDefaultValueForConstraint(
-      const blink::WebMediaConstraints& constraints,
-      const std::string& key) const;
-
+  bool GetDefaultValueForConstraint(const std::string& key) const;
   const blink::WebMediaConstraints constraints_;
   const int effects_;
   bool default_audio_processing_constraint_value_;
@@ -101,14 +104,36 @@ class CONTENT_EXPORT EchoInformation {
   EchoInformation();
   virtual ~EchoInformation();
 
-  void UpdateAecDelayStats(webrtc::EchoCancellation* echo_cancellation);
+  // Updates stats, and reports delay metrics as UMA stats every 5 seconds.
+  // Must be called every time AudioProcessing::ProcessStream() is called.
+  void UpdateAecStats(webrtc::EchoCancellation* echo_cancellation);
+
+  // Reports AEC divergent filter metrics as UMA and resets the associated data.
+  void ReportAndResetAecDivergentFilterStats();
 
  private:
-  // Counter to track 5 seconds of processed 10 ms chunks in order to query a
-  // new metric from webrtc::EchoCancellation::GetEchoDelayMetrics().
-  int num_chunks_;
+  void UpdateAecDelayStats(webrtc::EchoCancellation* echo_cancellation);
+  void UpdateAecDivergentFilterStats(
+      webrtc::EchoCancellation* echo_cancellation);
+
+  // Counter to track 5 seconds of data in order to query a new metric from
+  // webrtc::EchoCancellation::GetEchoDelayMetrics().
+  int delay_stats_time_ms_;
   bool echo_frames_received_;
 
+  // Counter to track 1 second of data in order to query a new divergent filter
+  // fraction metric from webrtc::EchoCancellation::GetMetrics().
+  int divergent_filter_stats_time_ms_;
+
+  // Total number of times we queried for the divergent filter fraction metric.
+  int num_divergent_filter_fraction_;
+
+  // Number of non-zero divergent filter fraction metrics.
+  int num_non_zero_divergent_filter_fraction_;
+
+  // Ensures that this class is accessed on the same thread.
+  base::ThreadChecker thread_checker_;
+
   DISALLOW_COPY_AND_ASSIGN(EchoInformation);
 };
 
