@@ -15,16 +15,17 @@
 #include "base/path_service.h"
 #include "base/time/time.h"
 #include "build/build_config.h"
+#include "content/common/media/media_stream_options.h"
 #include "content/public/common/media_stream_request.h"
 #include "content/renderer/media/media_stream_audio_processor.h"
 #include "content/renderer/media/media_stream_audio_processor_options.h"
-#include "content/renderer/media/mock_media_constraint_factory.h"
-#include "media/audio/audio_parameters.h"
+#include "content/renderer/media/mock_constraint_factory.h"
 #include "media/base/audio_bus.h"
+#include "media/base/audio_parameters.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/WebKit/public/platform/WebMediaConstraints.h"
-#include "third_party/libjingle/source/talk/app/webrtc/mediastreaminterface.h"
+#include "third_party/webrtc/api/mediastreaminterface.h"
 
 using ::testing::_;
 using ::testing::AnyNumber;
@@ -91,17 +92,17 @@ class MediaStreamAudioProcessorTest : public ::testing::Test {
     const int packet_size =
         params.frames_per_buffer() * 2 * params.channels();
     const size_t length = packet_size * kNumberOfPacketsForTest;
-    scoped_ptr<char[]> capture_data(new char[length]);
+    std::unique_ptr<char[]> capture_data(new char[length]);
     ReadDataFromSpeechFile(capture_data.get(), length);
     const int16_t* data_ptr =
         reinterpret_cast<const int16_t*>(capture_data.get());
-    scoped_ptr<media::AudioBus> data_bus = media::AudioBus::Create(
-        params.channels(), params.frames_per_buffer());
+    std::unique_ptr<media::AudioBus> data_bus =
+        media::AudioBus::Create(params.channels(), params.frames_per_buffer());
 
     // |data_bus_playout| is used if the number of capture channels is larger
     // that max allowed playout channels. |data_bus_playout_to_use| points to
     // the AudioBus to use, either |data_bus| or |data_bus_playout|.
-    scoped_ptr<media::AudioBus> data_bus_playout;
+    std::unique_ptr<media::AudioBus> data_bus_playout;
     media::AudioBus* data_bus_playout_to_use = data_bus.get();
     if (params.channels() > kMaxNumberOfPlayoutDataChannels) {
       data_bus_playout =
@@ -122,7 +123,7 @@ class MediaStreamAudioProcessorTest : public ::testing::Test {
       // |audio_processor| does nothing when the audio processing is off in
       // the processor.
       webrtc::AudioProcessing* ap = audio_processor->audio_processing_.get();
-#if defined(OS_ANDROID) || defined(OS_IOS)
+#if defined(OS_ANDROID)
       const bool is_aec_enabled = ap && ap->echo_control_mobile()->is_enabled();
       // AEC should be turned off for mobiles.
       DCHECK(!ap || !ap->echo_cancellation()->is_enabled());
@@ -169,7 +170,7 @@ class MediaStreamAudioProcessorTest : public ::testing::Test {
     EXPECT_TRUE(audio_processing->echo_control_mobile()->routing_mode() ==
         webrtc::EchoControlMobile::kSpeakerphone);
     EXPECT_FALSE(audio_processing->echo_cancellation()->is_enabled());
-#elif !defined(OS_IOS)
+#else
     EXPECT_TRUE(audio_processing->echo_cancellation()->is_enabled());
     EXPECT_TRUE(audio_processing->echo_cancellation()->suppression_level() ==
         webrtc::EchoCancellation::kHighSuppression);
@@ -183,7 +184,7 @@ class MediaStreamAudioProcessorTest : public ::testing::Test {
         webrtc::NoiseSuppression::kHigh);
     EXPECT_TRUE(audio_processing->high_pass_filter()->is_enabled());
     EXPECT_TRUE(audio_processing->gain_control()->is_enabled());
-#if defined(OS_ANDROID) || defined(OS_IOS)
+#if defined(OS_ANDROID)
     EXPECT_TRUE(audio_processing->gain_control()->mode() ==
         webrtc::GainControl::kFixedDigital);
     EXPECT_FALSE(audio_processing->voice_detection()->is_enabled());
@@ -207,7 +208,8 @@ class MediaStreamAudioProcessorTest : public ::testing::Test {
 #define MAYBE_WithAudioProcessing WithAudioProcessing
 #endif
 TEST_F(MediaStreamAudioProcessorTest, MAYBE_WithAudioProcessing) {
-  MockMediaConstraintFactory constraint_factory;
+  base::MessageLoop message_loop;
+  MockConstraintFactory constraint_factory;
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
@@ -222,19 +224,20 @@ TEST_F(MediaStreamAudioProcessorTest, MAYBE_WithAudioProcessing) {
                              kAudioProcessingSampleRate,
                              kAudioProcessingNumberOfChannel,
                              kAudioProcessingSampleRate / 100);
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device| outlives
-  // |audio_processor|.
-  audio_processor = NULL;
+
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 TEST_F(MediaStreamAudioProcessorTest, VerifyTabCaptureWithoutAudioProcessing) {
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   // Create MediaStreamAudioProcessor instance for kMediaStreamSourceTab source.
-  MockMediaConstraintFactory tab_constraint_factory;
+  MockConstraintFactory tab_constraint_factory;
   const std::string tab_string = kMediaStreamSourceTab;
-  tab_constraint_factory.AddMandatory(kMediaStreamSource,
-                                      tab_string);
+  tab_constraint_factory.basic().mediaStreamSource.setExact(
+      blink::WebString::fromUTF8(tab_string));
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
       new rtc::RefCountedObject<MediaStreamAudioProcessor>(
           tab_constraint_factory.CreateWebMediaConstraints(),
@@ -249,23 +252,23 @@ TEST_F(MediaStreamAudioProcessorTest, VerifyTabCaptureWithoutAudioProcessing) {
 
   // Create MediaStreamAudioProcessor instance for kMediaStreamSourceSystem
   // source.
-  MockMediaConstraintFactory system_constraint_factory;
+  MockConstraintFactory system_constraint_factory;
   const std::string system_string = kMediaStreamSourceSystem;
-  system_constraint_factory.AddMandatory(kMediaStreamSource,
-                                         system_string);
+  system_constraint_factory.basic().mediaStreamSource.setExact(
+      blink::WebString::fromUTF8(system_string));
   audio_processor = new rtc::RefCountedObject<MediaStreamAudioProcessor>(
       system_constraint_factory.CreateWebMediaConstraints(),
       input_device_params_, webrtc_audio_device.get());
   EXPECT_FALSE(audio_processor->has_audio_processing());
 
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device| outlives
-  // |audio_processor|.
-  audio_processor = NULL;
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 TEST_F(MediaStreamAudioProcessorTest, TurnOffDefaultConstraints) {
   // Turn off the default constraints and pass it to MediaStreamAudioProcessor.
-  MockMediaConstraintFactory constraint_factory;
+  MockConstraintFactory constraint_factory;
   constraint_factory.DisableDefaultAudioConstraints();
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
@@ -280,49 +283,16 @@ TEST_F(MediaStreamAudioProcessorTest, TurnOffDefaultConstraints) {
                              params_.sample_rate(),
                              params_.channels(),
                              params_.sample_rate() / 100);
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device| outlives
-  // |audio_processor|.
-  audio_processor = NULL;
+
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 TEST_F(MediaStreamAudioProcessorTest, VerifyConstraints) {
-  static const char* kDefaultAudioConstraints[] = {
-    MediaAudioConstraints::kEchoCancellation,
-    MediaAudioConstraints::kGoogAudioMirroring,
-    MediaAudioConstraints::kGoogAutoGainControl,
-    MediaAudioConstraints::kGoogEchoCancellation,
-    MediaAudioConstraints::kGoogExperimentalEchoCancellation,
-    MediaAudioConstraints::kGoogExperimentalAutoGainControl,
-    MediaAudioConstraints::kGoogExperimentalNoiseSuppression,
-    MediaAudioConstraints::kGoogHighpassFilter,
-    MediaAudioConstraints::kGoogNoiseSuppression,
-    MediaAudioConstraints::kGoogTypingNoiseDetection,
-    kMediaStreamAudioHotword
-  };
-
-  // Verify mandatory constraints.
-  for (size_t i = 0; i < arraysize(kDefaultAudioConstraints); ++i) {
-    MockMediaConstraintFactory constraint_factory;
-    constraint_factory.AddMandatory(kDefaultAudioConstraints[i], false);
-    blink::WebMediaConstraints constraints =
-        constraint_factory.CreateWebMediaConstraints();
-    MediaAudioConstraints audio_constraints(constraints, 0);
-    EXPECT_FALSE(audio_constraints.GetProperty(kDefaultAudioConstraints[i]));
-  }
-
-  // Verify optional constraints.
-  for (size_t i = 0; i < arraysize(kDefaultAudioConstraints); ++i) {
-    MockMediaConstraintFactory constraint_factory;
-    constraint_factory.AddOptional(kDefaultAudioConstraints[i], false);
-    blink::WebMediaConstraints constraints =
-        constraint_factory.CreateWebMediaConstraints();
-    MediaAudioConstraints audio_constraints(constraints, 0);
-    EXPECT_FALSE(audio_constraints.GetProperty(kDefaultAudioConstraints[i]));
-  }
-
   {
-    // Verify echo cancellation is off when platform aec effect is on.
-    MockMediaConstraintFactory constraint_factory;
+    // Verify that echo cancellation is off when platform aec effect is on.
+    MockConstraintFactory constraint_factory;
     MediaAudioConstraints audio_constraints(
         constraint_factory.CreateWebMediaConstraints(),
         media::AudioParameters::ECHO_CANCELLER);
@@ -331,61 +301,82 @@ TEST_F(MediaStreamAudioProcessorTest, VerifyConstraints) {
 
   {
     // Verify |kEchoCancellation| overwrite |kGoogEchoCancellation|.
-    MockMediaConstraintFactory constraint_factory_1;
-    constraint_factory_1.AddOptional(MediaAudioConstraints::kEchoCancellation,
-                                   true);
-    constraint_factory_1.AddOptional(
-        MediaAudioConstraints::kGoogEchoCancellation, false);
+    MockConstraintFactory constraint_factory_1;
+    constraint_factory_1.AddAdvanced().echoCancellation.setExact(true);
+    constraint_factory_1.AddAdvanced().googEchoCancellation.setExact(false);
     blink::WebMediaConstraints constraints_1 =
         constraint_factory_1.CreateWebMediaConstraints();
     MediaAudioConstraints audio_constraints_1(constraints_1, 0);
     EXPECT_TRUE(audio_constraints_1.GetEchoCancellationProperty());
 
-    MockMediaConstraintFactory constraint_factory_2;
-    constraint_factory_2.AddOptional(MediaAudioConstraints::kEchoCancellation,
-                                     false);
-    constraint_factory_2.AddOptional(
-        MediaAudioConstraints::kGoogEchoCancellation, true);
+    MockConstraintFactory constraint_factory_2;
+    constraint_factory_2.AddAdvanced().echoCancellation.setExact(false);
+    constraint_factory_2.AddAdvanced().googEchoCancellation.setExact(true);
     blink::WebMediaConstraints constraints_2 =
         constraint_factory_2.CreateWebMediaConstraints();
     MediaAudioConstraints audio_constraints_2(constraints_2, 0);
     EXPECT_FALSE(audio_constraints_2.GetEchoCancellationProperty());
   }
-
   {
     // When |kEchoCancellation| is explicitly set to false, the default values
     // for all the constraints are false.
-    MockMediaConstraintFactory constraint_factory;
-    constraint_factory.AddOptional(MediaAudioConstraints::kEchoCancellation,
-                                   false);
+    MockConstraintFactory constraint_factory;
+    constraint_factory.AddAdvanced().echoCancellation.setExact(false);
     blink::WebMediaConstraints constraints =
         constraint_factory.CreateWebMediaConstraints();
     MediaAudioConstraints audio_constraints(constraints, 0);
-    for (size_t i = 0; i < arraysize(kDefaultAudioConstraints); ++i) {
-      EXPECT_FALSE(audio_constraints.GetProperty(kDefaultAudioConstraints[i]));
-    }
-  }
-
-  {
-    // |kMediaStreamAudioHotword| is always off by default.
-    MockMediaConstraintFactory constraint_factory;
-    MediaAudioConstraints audio_constraints(
-        constraint_factory.CreateWebMediaConstraints(), 0);
-    EXPECT_FALSE(audio_constraints.GetProperty(kMediaStreamAudioHotword));
   }
 }
 
-TEST_F(MediaStreamAudioProcessorTest, ValidateConstraints) {
-  MockMediaConstraintFactory constraint_factory;
-  const std::string dummy_constraint = "dummy";
-  constraint_factory.AddMandatory(dummy_constraint, true);
+TEST_F(MediaStreamAudioProcessorTest, ValidateBadConstraints) {
+  MockConstraintFactory constraint_factory;
+  // Add a constraint that is not valid for audio.
+  constraint_factory.basic().width.setExact(240);
   MediaAudioConstraints audio_constraints(
       constraint_factory.CreateWebMediaConstraints(), 0);
   EXPECT_FALSE(audio_constraints.IsValid());
 }
 
+TEST_F(MediaStreamAudioProcessorTest, ValidateGoodConstraints) {
+  MockConstraintFactory constraint_factory;
+  // Check that the renderToAssociatedSink constraint is considered valid.
+  constraint_factory.basic().renderToAssociatedSink.setExact(true);
+  MediaAudioConstraints audio_constraints(
+      constraint_factory.CreateWebMediaConstraints(), 0);
+  EXPECT_TRUE(audio_constraints.IsValid());
+}
+
+TEST_F(MediaStreamAudioProcessorTest, NoEchoTurnsOffProcessing) {
+  {
+    MockConstraintFactory constraint_factory;
+    MediaAudioConstraints audio_constraints(
+        constraint_factory.CreateWebMediaConstraints(), 0);
+    // The default value for echo cancellation is true, except when all
+    // audio processing has been turned off.
+    EXPECT_TRUE(audio_constraints.default_audio_processing_constraint_value());
+  }
+  // Turning off audio processing via a mandatory constraint.
+  {
+    MockConstraintFactory constraint_factory;
+    constraint_factory.basic().echoCancellation.setExact(false);
+    MediaAudioConstraints audio_constraints(
+        constraint_factory.CreateWebMediaConstraints(), 0);
+    // The default value for echo cancellation is true, except when all
+    // audio processing has been turned off.
+    EXPECT_FALSE(audio_constraints.default_audio_processing_constraint_value());
+  }
+  // Turning off audio processing via an optional constraint.
+  {
+    MockConstraintFactory constraint_factory;
+    constraint_factory.AddAdvanced().echoCancellation.setExact(false);
+    MediaAudioConstraints audio_constraints(
+        constraint_factory.CreateWebMediaConstraints(), 0);
+    EXPECT_FALSE(audio_constraints.default_audio_processing_constraint_value());
+  }
+}
+
 MediaAudioConstraints MakeMediaAudioConstraints(
-    const MockMediaConstraintFactory& constraint_factory) {
+    const MockConstraintFactory& constraint_factory) {
   return MediaAudioConstraints(constraint_factory.CreateWebMediaConstraints(),
                                AudioParameters::NO_EFFECTS);
 }
@@ -400,7 +391,7 @@ TEST_F(MediaStreamAudioProcessorTest, SelectsConstraintsArrayGeometryIfExists) {
 
   {
     // Both geometries empty.
-    MockMediaConstraintFactory constraint_factory;
+    MockConstraintFactory constraint_factory;
     MediaStreamDevice::AudioDeviceParameters input_params;
 
     const auto& actual_geometry = GetArrayGeometryPreferringConstraints(
@@ -409,7 +400,7 @@ TEST_F(MediaStreamAudioProcessorTest, SelectsConstraintsArrayGeometryIfExists) {
   }
   {
     // Constraints geometry empty.
-    MockMediaConstraintFactory constraint_factory;
+    MockConstraintFactory constraint_factory;
     MediaStreamDevice::AudioDeviceParameters input_params;
     input_params.mic_positions.push_back(media::Point(0, 0, 0));
     input_params.mic_positions.push_back(media::Point(0, 0.05f, 0));
@@ -420,9 +411,9 @@ TEST_F(MediaStreamAudioProcessorTest, SelectsConstraintsArrayGeometryIfExists) {
   }
   {
     // Input device geometry empty.
-    MockMediaConstraintFactory constraint_factory;
-    constraint_factory.AddOptional(MediaAudioConstraints::kGoogArrayGeometry,
-                                   std::string("-0.02 0 0 0.02 0 0"));
+    MockConstraintFactory constraint_factory;
+    constraint_factory.AddAdvanced().googArrayGeometry.setExact(
+        blink::WebString::fromUTF8("-0.02 0 0 0.02 0 0"));
     MediaStreamDevice::AudioDeviceParameters input_params;
 
     const auto& actual_geometry = GetArrayGeometryPreferringConstraints(
@@ -431,9 +422,9 @@ TEST_F(MediaStreamAudioProcessorTest, SelectsConstraintsArrayGeometryIfExists) {
   }
   {
     // Both geometries existing.
-    MockMediaConstraintFactory constraint_factory;
-    constraint_factory.AddOptional(MediaAudioConstraints::kGoogArrayGeometry,
-                                   std::string("-0.02 0 0 0.02 0 0"));
+    MockConstraintFactory constraint_factory;
+    constraint_factory.AddAdvanced().googArrayGeometry.setExact(
+        blink::WebString::fromUTF8("-0.02 0 0 0.02 0 0"));
     MediaStreamDevice::AudioDeviceParameters input_params;
     input_params.mic_positions.push_back(media::Point(0, 0, 0));
     input_params.mic_positions.push_back(media::Point(0, 0.05f, 0));
@@ -452,7 +443,8 @@ TEST_F(MediaStreamAudioProcessorTest, SelectsConstraintsArrayGeometryIfExists) {
 #define MAYBE_TestAllSampleRates TestAllSampleRates
 #endif
 TEST_F(MediaStreamAudioProcessorTest, MAYBE_TestAllSampleRates) {
-  MockMediaConstraintFactory constraint_factory;
+  base::MessageLoop message_loop;
+  MockConstraintFactory constraint_factory;
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
@@ -479,9 +471,9 @@ TEST_F(MediaStreamAudioProcessorTest, MAYBE_TestAllSampleRates) {
                                kAudioProcessingSampleRate / 100);
   }
 
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device|
-  // outlives |audio_processor|.
-  audio_processor = NULL;
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 // Test that if we have an AEC dump message filter created, we are getting it
@@ -493,7 +485,7 @@ TEST_F(MediaStreamAudioProcessorTest, GetAecDumpMessageFilter) {
       new AecDumpMessageFilter(message_loop.task_runner(),
                                message_loop.task_runner(), nullptr));
 
-  MockMediaConstraintFactory constraint_factory;
+  MockConstraintFactory constraint_factory;
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
@@ -503,17 +495,17 @@ TEST_F(MediaStreamAudioProcessorTest, GetAecDumpMessageFilter) {
 
   EXPECT_TRUE(audio_processor->aec_dump_message_filter_.get());
 
-  audio_processor = NULL;
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 TEST_F(MediaStreamAudioProcessorTest, TestStereoAudio) {
   // Set up the correct constraints to turn off the audio processing and turn
   // on the stereo channels mirroring.
-  MockMediaConstraintFactory constraint_factory;
-  constraint_factory.AddMandatory(MediaAudioConstraints::kEchoCancellation,
-                                  false);
-  constraint_factory.AddMandatory(MediaAudioConstraints::kGoogAudioMirroring,
-                                  true);
+  MockConstraintFactory constraint_factory;
+  constraint_factory.basic().echoCancellation.setExact(false);
+  constraint_factory.basic().googAudioMirroring.setExact(true);
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
@@ -525,17 +517,18 @@ TEST_F(MediaStreamAudioProcessorTest, TestStereoAudio) {
       media::AudioParameters::AUDIO_PCM_LOW_LATENCY,
       media::CHANNEL_LAYOUT_STEREO, 48000, 16, 480);
   audio_processor->OnCaptureFormatChanged(source_params);
-  EXPECT_EQ(audio_processor->OutputFormat().channels(), 2);
+  // There's no sense in continuing if this fails.
+  ASSERT_EQ(2, audio_processor->OutputFormat().channels());
 
   // Construct left and right channels, and assign different values to the
   // first data of the left channel and right channel.
   const int size = media::AudioBus::CalculateMemorySize(source_params);
-  scoped_ptr<float, base::AlignedFreeDeleter> left_channel(
+  std::unique_ptr<float, base::AlignedFreeDeleter> left_channel(
       static_cast<float*>(base::AlignedAlloc(size, 32)));
-  scoped_ptr<float, base::AlignedFreeDeleter> right_channel(
+  std::unique_ptr<float, base::AlignedFreeDeleter> right_channel(
       static_cast<float*>(base::AlignedAlloc(size, 32)));
-  scoped_ptr<media::AudioBus> wrapper = media::AudioBus::CreateWrapper(
-      source_params.channels());
+  std::unique_ptr<media::AudioBus> wrapper =
+      media::AudioBus::CreateWrapper(source_params.channels());
   wrapper->set_frames(source_params.frames_per_buffer());
   wrapper->SetChannelData(0, left_channel.get());
   wrapper->SetChannelData(1, right_channel.get());
@@ -562,9 +555,9 @@ TEST_F(MediaStreamAudioProcessorTest, TestStereoAudio) {
     EXPECT_EQ(pushed_capture_delay, capture_delay);
   }
 
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device| outlives
-  // |audio_processor|.
-  audio_processor = NULL;
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 // Disabled on android clang builds due to crbug.com/470499
@@ -573,11 +566,10 @@ TEST_F(MediaStreamAudioProcessorTest, TestStereoAudio) {
 #else
 #define MAYBE_TestWithKeyboardMicChannel TestWithKeyboardMicChannel
 #endif
-
 TEST_F(MediaStreamAudioProcessorTest, MAYBE_TestWithKeyboardMicChannel) {
-  MockMediaConstraintFactory constraint_factory;
-  constraint_factory.AddMandatory(
-      MediaAudioConstraints::kGoogExperimentalNoiseSuppression, true);
+  base::MessageLoop message_loop;
+  MockConstraintFactory constraint_factory;
+  constraint_factory.basic().googExperimentalNoiseSuppression.setExact(true);
   scoped_refptr<WebRtcAudioDeviceImpl> webrtc_audio_device(
       new WebRtcAudioDeviceImpl());
   scoped_refptr<MediaStreamAudioProcessor> audio_processor(
@@ -595,9 +587,10 @@ TEST_F(MediaStreamAudioProcessorTest, MAYBE_TestWithKeyboardMicChannel) {
                              kAudioProcessingSampleRate,
                              kAudioProcessingNumberOfChannel,
                              kAudioProcessingSampleRate / 100);
-  // Set |audio_processor| to NULL to make sure |webrtc_audio_device| outlives
-  // |audio_processor|.
-  audio_processor = NULL;
+
+  // Stop |audio_processor| so that it removes itself from
+  // |webrtc_audio_device| and clears its pointer to it.
+  audio_processor->Stop();
 }
 
 }  // namespace content
