@@ -8,9 +8,10 @@
 #include <stdint.h>
 #include <string.h>
 
+#include <memory>
+
 #include "base/i18n/char_iterator.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
@@ -270,10 +271,11 @@ void AppendTouchEvent(const WebInputEvent& event,
 WebTouchPoint CreateWebTouchPoint(const PP_TouchPoint& pp_pt,
                                   WebTouchPoint::State state) {
   WebTouchPoint pt;
+  pt.pointerType = blink::WebPointerProperties::PointerType::Touch;
   pt.id = pp_pt.id;
   pt.position.x = pp_pt.position.x;
   pt.position.y = pp_pt.position.y;
-  // TODO bug:http://code.google.com/p/chromium/issues/detail?id=93902
+  // TODO(crbug.com/93902): Add screen coordinate calculation.
   pt.screenPosition.x = 0;
   pt.screenPosition.y = 0;
   pt.force = pp_pt.pressure;
@@ -402,6 +404,8 @@ WebKeyboardEvent* BuildCharEvent(const InputEventData& event) {
 
 WebMouseEvent* BuildMouseEvent(const InputEventData& event) {
   WebMouseEvent* mouse_event = new WebMouseEvent();
+  mouse_event->pointerType = blink::WebPointerProperties::PointerType::Mouse;
+
   switch (event.event_type) {
     case PP_INPUTEVENT_TYPE_MOUSEDOWN:
       mouse_event->type = WebInputEvent::MouseDown;
@@ -529,9 +533,8 @@ void GetKeyCode(const std::string& char_text,
       base::string16 char_text16 = base::UTF8ToUTF16(char_text);
       DCHECK_EQ(char_text16.size(), 1U);
       vk_text = vk_code = char_text16[0];
-      *needs_shift_modifier =
-          (vk_code & 0xFF) >= 'A' && (vk_code & 0xFF) <= 'Z';
-      if ((vk_code & 0xFF) >= 'a' && (vk_code & 0xFF) <= 'z')
+      *needs_shift_modifier = base::IsAsciiUpper(vk_code & 0xFF);
+      if (base::IsAsciiLower(vk_code & 0xFF))
         vk_code -= 'a' - 'A';
       *generate_char = true;
     }
@@ -580,7 +583,7 @@ void CreateInputEventData(const WebInputEvent& event,
 }
 
 WebInputEvent* CreateWebInputEvent(const InputEventData& event) {
-  scoped_ptr<WebInputEvent> web_input_event;
+  std::unique_ptr<WebInputEvent> web_input_event;
   switch (event.event_type) {
     case PP_INPUTEVENT_TYPE_UNDEFINED:
       return NULL;
@@ -624,12 +627,12 @@ WebInputEvent* CreateWebInputEvent(const InputEventData& event) {
 
 // Generate a coherent sequence of input events to simulate a user event.
 // From src/components/test_runner/event_sender.cc.
-std::vector<scoped_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
+std::vector<std::unique_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
     const ppapi::InputEventData& event,
     int plugin_x,
     int plugin_y) {
-  std::vector<scoped_ptr<WebInputEvent>> events;
-  scoped_ptr<WebInputEvent> original_event(CreateWebInputEvent(event));
+  std::vector<std::unique_ptr<WebInputEvent>> events;
+  std::unique_ptr<WebInputEvent> original_event(CreateWebInputEvent(event));
 
   switch (event.event_type) {
     case PP_INPUTEVENT_TYPE_MOUSEDOWN:
@@ -680,8 +683,8 @@ std::vector<scoped_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
                  &generate_char);
 
       // Synthesize key down and key up events in all cases.
-      scoped_ptr<WebKeyboardEvent> key_down_event(new WebKeyboardEvent());
-      scoped_ptr<WebKeyboardEvent> key_up_event(new WebKeyboardEvent());
+      std::unique_ptr<WebKeyboardEvent> key_down_event(new WebKeyboardEvent());
+      std::unique_ptr<WebKeyboardEvent> key_up_event(new WebKeyboardEvent());
 
       key_down_event->type = WebInputEvent::RawKeyDown;
       key_down_event->windowsKeyCode = code;
@@ -699,7 +702,7 @@ std::vector<scoped_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
 
       *key_up_event = *web_char_event = *key_down_event;
 
-      events.push_back(make_scoped_ptr<WebInputEvent>(key_down_event.release()));
+      events.push_back(std::move(key_down_event));
 
       if (generate_char) {
         web_char_event->type = WebInputEvent::Char;
@@ -708,7 +711,7 @@ std::vector<scoped_ptr<WebInputEvent>> CreateSimulatedWebInputEvents(
       }
 
       key_up_event->type = WebInputEvent::KeyUp;
-      events.push_back(make_scoped_ptr<WebInputEvent>(key_up_event.release()));
+      events.push_back(std::move(key_up_event));
       break;
     }
 
@@ -739,7 +742,8 @@ PP_InputEvent_Class ClassifyInputEvent(WebInputEvent::Type type) {
     case WebInputEvent::TouchMove:
     case WebInputEvent::TouchStart:
       return PP_INPUTEVENT_CLASS_TOUCH;
-    case WebInputEvent::Undefined:
+    case WebInputEvent::TouchScrollStarted:
+      return PP_InputEvent_Class(0);
     default:
       CHECK(WebInputEvent::isGestureEventType(type));
       return PP_InputEvent_Class(0);
