@@ -5,9 +5,9 @@
 #ifndef CONTENT_RENDERER_PEPPER_EVENT_CONVERSION_H_
 #define CONTENT_RENDERER_PEPPER_EVENT_CONVERSION_H_
 
+#include <memory>
 #include <vector>
 
-#include "base/memory/scoped_ptr.h"
 #include "content/common/content_export.h"
 #include "ppapi/c/ppb_input_event.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
@@ -40,10 +40,10 @@ CONTENT_EXPORT blink::WebInputEvent* CreateWebInputEvent(
 // Creates an array of WebInputEvents to make the given event look like a user
 // input event on all platforms. |plugin_x| and |plugin_y| should be the
 // coordinates of a point within the plugin's area on the page.
-std::vector<scoped_ptr<blink::WebInputEvent>> CreateSimulatedWebInputEvents(
-    const ppapi::InputEventData& event,
-    int plugin_x,
-    int plugin_y);
+std::vector<std::unique_ptr<blink::WebInputEvent>>
+CreateSimulatedWebInputEvents(const ppapi::InputEventData& event,
+                              int plugin_x,
+                              int plugin_y);
 
 // Returns the PPAPI event class for the given WebKit event type. The given
 // type should not be "Undefined" since there's no corresponding PPAPI class.
