@@ -11,7 +11,7 @@
 #include "base/location.h"
 #include "base/logging.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
 #include "cc/resources/shared_bitmap.h"
@@ -36,6 +36,7 @@
 #include "ppapi/thunk/enter.h"
 #include "skia/ext/platform_canvas.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "third_party/skia/include/core/SkSwizzle.h"
 #include "ui/gfx/blit.h"
 #include "ui/gfx/geometry/point_conversions.h"
 #include "ui/gfx/geometry/rect.h"
@@ -89,21 +90,6 @@ bool ValidateAndConvertRect(const PP_Rect* rect,
   return true;
 }
 
-// Converts BGRA <-> RGBA.
-void ConvertBetweenBGRAandRGBA(const uint32_t* input,
-                               int pixel_length,
-                               uint32_t* output) {
-  for (int i = 0; i < pixel_length; i++) {
-    const unsigned char* pixel_in =
-        reinterpret_cast<const unsigned char*>(&input[i]);
-    unsigned char* pixel_out = reinterpret_cast<unsigned char*>(&output[i]);
-    pixel_out[0] = pixel_in[2];
-    pixel_out[1] = pixel_in[1];
-    pixel_out[2] = pixel_in[0];
-    pixel_out[3] = pixel_in[3];
-  }
-}
-
 // Converts ImageData from PP_IMAGEDATAFORMAT_BGRA_PREMUL to
 // PP_IMAGEDATAFORMAT_RGBA_PREMUL, or reverse. It's assumed that the
 // destination image is always mapped (so will have non-NULL data).
@@ -121,22 +107,22 @@ void ConvertImageData(PPB_ImageData_Impl* src_image,
   const SkBitmap* dest_bitmap = dest_image->GetMappedBitmap();
   if (src_rect.width() == src_image->width() &&
       dest_rect.width() == dest_image->width()) {
-    // Fast path if the full line needs to be converted.
-    ConvertBetweenBGRAandRGBA(
+    // Fast path if the full frame can be converted at once.
+    SkSwapRB(
+        dest_bitmap->getAddr32(static_cast<int>(dest_rect.fLeft),
+                               static_cast<int>(dest_rect.fTop)),
         src_bitmap->getAddr32(static_cast<int>(src_rect.fLeft),
                               static_cast<int>(src_rect.fTop)),
-        src_rect.width() * src_rect.height(),
-        dest_bitmap->getAddr32(static_cast<int>(dest_rect.fLeft),
-                               static_cast<int>(dest_rect.fTop)));
+        src_rect.width() * src_rect.height());
   } else {
     // Slow path where we convert line by line.
     for (int y = 0; y < src_rect.height(); y++) {
-      ConvertBetweenBGRAandRGBA(
+      SkSwapRB(
+          dest_bitmap->getAddr32(static_cast<int>(dest_rect.fLeft),
+                                 static_cast<int>(dest_rect.fTop + y)),
           src_bitmap->getAddr32(static_cast<int>(src_rect.fLeft),
                                 static_cast<int>(src_rect.fTop + y)),
-          src_rect.width(),
-          dest_bitmap->getAddr32(static_cast<int>(dest_rect.fLeft),
-                                 static_cast<int>(dest_rect.fTop + y)));
+          src_rect.width());
     }
   }
 }
@@ -144,7 +130,7 @@ void ConvertImageData(PPB_ImageData_Impl* src_image,
 }  // namespace
 
 struct PepperGraphics2DHost::QueuedOperation {
-  enum Type { PAINT, SCROLL, REPLACE, };
+  enum Type { PAINT, SCROLL, REPLACE, TRANSFORM };
 
   QueuedOperation(Type t)
       : type(t), paint_x(0), paint_y(0), scroll_dx(0), scroll_dy(0) {}
@@ -162,6 +148,10 @@ struct PepperGraphics2DHost::QueuedOperation {
 
   // Valid when type == REPLACE.
   scoped_refptr<PPB_ImageData_Impl> replace_image;
+
+  // Valid when type == TRANSFORM
+  float scale;
+  gfx::PointF translation;
 };
 
 // static
@@ -237,6 +227,8 @@ int32_t PepperGraphics2DHost::OnResourceMessageReceived(
                                         OnHostMsgFlush)
     PPAPI_DISPATCH_HOST_RESOURCE_CALL(PpapiHostMsg_Graphics2D_SetScale,
                                       OnHostMsgSetScale)
+    PPAPI_DISPATCH_HOST_RESOURCE_CALL(PpapiHostMsg_Graphics2D_SetLayerTransform,
+                                      OnHostMsgSetLayerTransform)
     PPAPI_DISPATCH_HOST_RESOURCE_CALL(PpapiHostMsg_Graphics2D_ReadImageData,
                                       OnHostMsgReadImageData)
   PPAPI_END_MESSAGE_MAP()
@@ -391,8 +383,6 @@ void PepperGraphics2DHost::ViewInitiatedPaint() {
   }
 }
 
-void PepperGraphics2DHost::SetScale(float scale) { scale_ = scale; }
-
 float PepperGraphics2DHost::GetScale() const { return scale_; }
 
 bool PepperGraphics2DHost::IsAlwaysOpaque() const { return is_always_opaque_; }
@@ -538,6 +528,21 @@ int32_t PepperGraphics2DHost::OnHostMsgSetScale(
   return PP_ERROR_BADARGUMENT;
 }
 
+int32_t PepperGraphics2DHost::OnHostMsgSetLayerTransform(
+            ppapi::host::HostMessageContext* context,
+            float scale,
+            const PP_FloatPoint& translation) {
+  if (scale < 0.0f)
+    return PP_ERROR_BADARGUMENT;
+
+  QueuedOperation operation(QueuedOperation::TRANSFORM);
+  operation.scale = scale;
+  operation.translation = gfx::PointF(translation.x, translation.y);
+  queued_operations_.push_back(operation);
+  return PP_OK;
+}
+
+
 int32_t PepperGraphics2DHost::OnHostMsgReadImageData(
     ppapi::host::HostMessageContext* context,
     PP_Resource image,
@@ -546,10 +551,11 @@ int32_t PepperGraphics2DHost::OnHostMsgReadImageData(
   return ReadImageData(image, &top_left) ? PP_OK : PP_ERROR_FAILED;
 }
 
-void PepperGraphics2DHost::ReleaseCallback(scoped_ptr<cc::SharedBitmap> bitmap,
-                                           const gfx::Size& bitmap_size,
-                                           const gpu::SyncToken& sync_token,
-                                           bool lost_resource) {
+void PepperGraphics2DHost::ReleaseCallback(
+    std::unique_ptr<cc::SharedBitmap> bitmap,
+    const gfx::Size& bitmap_size,
+    const gpu::SyncToken& sync_token,
+    bool lost_resource) {
   cached_bitmap_.reset();
   // Only keep around a cached bitmap if the plugin is currently drawing (has
   // need_flush_ack_ set).
@@ -560,12 +566,12 @@ void PepperGraphics2DHost::ReleaseCallback(scoped_ptr<cc::SharedBitmap> bitmap,
 
 bool PepperGraphics2DHost::PrepareTextureMailbox(
     cc::TextureMailbox* mailbox,
-    scoped_ptr<cc::SingleReleaseCallback>* release_callback) {
+    std::unique_ptr<cc::SingleReleaseCallback>* release_callback) {
   if (!texture_mailbox_modified_)
     return false;
   // TODO(jbauman): Send image_data_ through mailbox to avoid copy.
   gfx::Size pixel_image_size(image_data_->width(), image_data_->height());
-  scoped_ptr<cc::SharedBitmap> shared_bitmap;
+  std::unique_ptr<cc::SharedBitmap> shared_bitmap;
   if (cached_bitmap_) {
     if (cached_bitmap_size_ == pixel_image_size)
       shared_bitmap = std::move(cached_bitmap_);
@@ -603,10 +609,15 @@ int32_t PepperGraphics2DHost::Flush(PP_Resource* old_image_data) {
   bool done_replace_contents = false;
   bool no_update_visible = true;
   bool is_plugin_visible = true;
+
   for (size_t i = 0; i < queued_operations_.size(); i++) {
     QueuedOperation& operation = queued_operations_[i];
     gfx::Rect op_rect;
     switch (operation.type) {
+      case QueuedOperation::TRANSFORM:
+        ExecuteTransform(operation.scale, operation.translation);
+        no_update_visible = false;
+        break;
       case QueuedOperation::PAINT:
         ExecutePaintImageData(operation.paint_image.get(),
                               operation.paint_x,
@@ -640,11 +651,15 @@ int32_t PepperGraphics2DHost::Flush(PP_Resource* old_image_data) {
     // is visible to determine how to deal with the callback.
     if (bound_instance_ && !op_rect.IsEmpty()) {
       gfx::Point scroll_delta(operation.scroll_dx, operation.scroll_dy);
-      if (!ConvertToLogicalPixels(scale_,
-                                  &op_rect,
+      // In use-zoom-for-dsf mode, the viewport (thus cc) uses native
+      // pixels, so the damage and rects have to be scaled.
+      gfx::Rect op_rect_in_viewport = op_rect;
+      ConvertToLogicalPixels(scale_, &op_rect, nullptr);
+      if (!ConvertToLogicalPixels(scale_ / viewport_to_dip_scale_,
+                                  &op_rect_in_viewport,
                                   operation.type == QueuedOperation::SCROLL
                                       ? &scroll_delta
-                                      : NULL)) {
+                                      : nullptr)) {
         // Conversion requires falling back to InvalidateRect.
         operation.type = QueuedOperation::PAINT;
       }
@@ -662,10 +677,10 @@ int32_t PepperGraphics2DHost::Flush(PP_Resource* old_image_data) {
       // partially or completely off-screen.
       if (operation.type == QueuedOperation::SCROLL) {
         bound_instance_->ScrollRect(
-            scroll_delta.x(), scroll_delta.y(), op_rect);
+            scroll_delta.x(), scroll_delta.y(), op_rect_in_viewport);
       } else {
-        if (!op_rect.IsEmpty())
-          bound_instance_->InvalidateRect(op_rect);
+        if (!op_rect_in_viewport.IsEmpty())
+          bound_instance_->InvalidateRect(op_rect_in_viewport);
       }
       texture_mailbox_modified_ = true;
     }
@@ -692,6 +707,11 @@ int32_t PepperGraphics2DHost::Flush(PP_Resource* old_image_data) {
   return PP_OK_COMPLETIONPENDING;
 }
 
+void PepperGraphics2DHost::ExecuteTransform(const float& scale,
+                                            const gfx::PointF& translate) {
+  bound_instance_->SetGraphics2DTransform(scale, translate);
+}
+
 void PepperGraphics2DHost::ExecutePaintImageData(PPB_ImageData_Impl* image,
                                                  int x,
                                                  int y,
