@@ -11,6 +11,7 @@
 #include "ipc/ipc_listener.h"
 #include "ipc/ipc_sender.h"
 #include "third_party/WebKit/public/platform/WebFocusType.h"
+#include "third_party/WebKit/public/platform/WebInsecureRequestPolicy.h"
 #include "third_party/WebKit/public/web/WebRemoteFrame.h"
 #include "third_party/WebKit/public/web/WebRemoteFrameClient.h"
 #include "url/origin.h"
@@ -24,7 +25,7 @@ struct WebRect;
 }
 
 namespace cc {
-struct SurfaceId;
+class SurfaceId;
 struct SurfaceSequence;
 }
 
@@ -33,6 +34,8 @@ namespace content {
 class ChildFrameCompositingHelper;
 class RenderFrameImpl;
 class RenderViewImpl;
+class RenderWidget;
+struct ContentSecurityPolicyHeader;
 struct FrameReplicationState;
 
 // When a page's frames are rendered by multiple processes, each renderer has a
@@ -102,6 +105,10 @@ class CONTENT_EXPORT RenderFrameProxy
   // IPC::Sender
   bool Send(IPC::Message* msg) override;
 
+  // Out-of-process child frames receive a signal from RenderWidgetCompositor
+  // when a compositor frame will begin.
+  void WillBeginCompositorFrame();
+
   // Out-of-process child frames receive a signal from RenderWidgetCompositor
   // when a compositor frame has committed.
   void DidCommitCompositorFrame();
@@ -110,24 +117,20 @@ class CONTENT_EXPORT RenderFrameProxy
   // RenderFrameProxy's WebRemoteFrame.
   void SetReplicatedState(const FrameReplicationState& state);
 
-  // Navigating a top-level frame cross-process does not swap the WebLocalFrame
-  // for a WebRemoteFrame in the frame tree. In this case, this WebRemoteFrame
-  // is not attached to the frame tree and there is no blink::Frame associated
-  // with it, so it is not in state where most operations on it will succeed.
-  bool IsMainFrameDetachedFromTree() const;
-
   int routing_id() { return routing_id_; }
   RenderViewImpl* render_view() { return render_view_; }
   blink::WebRemoteFrame* web_frame() { return web_frame_; }
 
+  // Returns the widget used for the local frame root.
+  RenderWidget* render_widget() { return render_widget_; }
+
   // blink::WebRemoteFrameClient implementation:
   void frameDetached(DetachType type) override;
-  void postMessageEvent(blink::WebLocalFrame* sourceFrame,
-                        blink::WebRemoteFrame* targetFrame,
-                        blink::WebSecurityOrigin target,
-                        blink::WebDOMMessageEvent event) override;
-  void initializeChildFrame(const blink::WebRect& frame_rect,
-                            float scale_factor) override;
+  void forwardPostMessage(blink::WebLocalFrame* sourceFrame,
+                          blink::WebRemoteFrame* targetFrame,
+                          blink::WebSecurityOrigin target,
+                          blink::WebDOMMessageEvent event) override;
+  void initializeChildFrame(float scale_factor) override;
   void navigate(const blink::WebURLRequest& request,
                 bool should_replace_current_entry) override;
   void forwardInputEvent(const blink::WebInputEvent* event) override;
@@ -144,7 +147,9 @@ class CONTENT_EXPORT RenderFrameProxy
  private:
   RenderFrameProxy(int routing_id, int frame_routing_id);
 
-  void Init(blink::WebRemoteFrame* frame, RenderViewImpl* render_view);
+  void Init(blink::WebRemoteFrame* frame,
+            RenderViewImpl* render_view,
+            RenderWidget* render_widget);
 
   // IPC::Listener
   bool OnMessageReceived(const IPC::Message& msg) override;
@@ -161,11 +166,17 @@ class CONTENT_EXPORT RenderFrameProxy
   void OnDidStopLoading();
   void OnDidUpdateSandboxFlags(blink::WebSandboxFlags flags);
   void OnDispatchLoad();
-  void OnDidUpdateName(const std::string& name);
-  void OnEnforceStrictMixedContentChecking(bool should_enforce);
-  void OnDidUpdateOrigin(const url::Origin& origin);
+  void OnDidUpdateName(const std::string& name, const std::string& unique_name);
+  void OnAddContentSecurityPolicy(const ContentSecurityPolicyHeader& header);
+  void OnResetContentSecurityPolicy();
+  void OnEnforceInsecureRequestPolicy(blink::WebInsecureRequestPolicy policy);
+  void OnSetFrameOwnerProperties(
+      const blink::WebFrameOwnerProperties& properties);
+  void OnDidUpdateOrigin(const url::Origin& origin,
+                         bool is_potentially_trustworthy_unique_origin);
   void OnSetPageFocus(bool is_focused);
   void OnSetFocusedFrame();
+  void OnWillEnterFullscreen();
 
   // The routing ID by which this RenderFrameProxy is known.
   const int routing_id_;
@@ -179,6 +190,7 @@ class CONTENT_EXPORT RenderFrameProxy
   scoped_refptr<ChildFrameCompositingHelper> compositing_helper_;
 
   RenderViewImpl* render_view_;
+  RenderWidget* render_widget_;
 
   DISALLOW_COPY_AND_ASSIGN(RenderFrameProxy);
 };
