@@ -2,16 +2,30 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "content/common/view_messages.h"
-#include "content/public/test/render_widget_test.h"
+#include "content/common/resize_params.h"
+#include "content/public/test/render_view_test.h"
+#include "content/renderer/render_view_impl.h"
 #include "content/renderer/render_widget.h"
 
 namespace content {
 
+class RenderWidgetTest : public RenderViewTest {
+ protected:
+  RenderWidget* widget() { return view_->GetWidget(); }
+
+  void OnResize(const ResizeParams& params) {
+    widget()->OnResize(params);
+  }
+
+  bool next_paint_is_resize_ack() {
+    return widget()->next_paint_is_resize_ack();
+  }
+};
+
 TEST_F(RenderWidgetTest, OnResize) {
   // The initial bounds is empty, so setting it to the same thing should do
   // nothing.
-  ViewMsg_Resize_Params resize_params;
+  ResizeParams resize_params;
   resize_params.screen_info = blink::WebScreenInfo();
   resize_params.new_size = gfx::Size();
   resize_params.physical_backing_size = gfx::Size();
@@ -68,9 +82,8 @@ class RenderWidgetInitialSizeTest : public RenderWidgetTest {
       : RenderWidgetTest(), initial_size_(200, 100) {}
 
  protected:
-  scoped_ptr<ViewMsg_Resize_Params> InitialSizeParams() override {
-    scoped_ptr<ViewMsg_Resize_Params> initial_size_params(
-        new ViewMsg_Resize_Params());
+  std::unique_ptr<ResizeParams> InitialSizeParams() override {
+    std::unique_ptr<ResizeParams> initial_size_params(new ResizeParams());
     initial_size_params->new_size = initial_size_;
     initial_size_params->physical_backing_size = initial_size_;
     initial_size_params->needs_resize_ack = true;
