@@ -2,37 +2,39 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stddef.h>
+#include <utility>
+
 #include "base/base_switches.h"
 #include "base/command_line.h"
 #include "base/debug/debugger.h"
 #include "base/debug/leak_annotations.h"
-#include "base/debug/stack_trace.h"
 #include "base/i18n/rtl.h"
-#include "base/memory/ref_counted.h"
 #include "base/message_loop/message_loop.h"
-#include "base/metrics/field_trial.h"
-#include "base/metrics/histogram.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/metrics/statistics_recorder.h"
 #include "base/pending_task.h"
 #include "base/strings/string_util.h"
 #include "base/sys_info.h"
 #include "base/threading/platform_thread.h"
-#include "base/time/time.h"
 #include "base/timer/hi_res_timer_manager.h"
 #include "base/trace_event/trace_event.h"
+#include "build/build_config.h"
+#include "components/scheduler/renderer/renderer_scheduler.h"
 #include "content/child/child_process.h"
 #include "content/common/content_constants_internal.h"
+#include "content/common/mojo/mojo_shell_connection_impl.h"
 #include "content/public/common/content_switches.h"
 #include "content/public/common/main_function_params.h"
 #include "content/public/renderer/content_renderer_client.h"
 #include "content/renderer/render_process_impl.h"
 #include "content/renderer/render_thread_impl.h"
 #include "content/renderer/renderer_main_platform_delegate.h"
+#include "third_party/skia/include/core/SkGraphics.h"
 #include "ui/base/ui_base_switches.h"
 
 #if defined(OS_ANDROID)
 #include "base/android/library_loader/library_loader_hooks.h"
-#include "third_party/skia/include/core/SkGraphics.h"
 #endif  // OS_ANDROID
 
 #if defined(OS_MACOSX)
@@ -50,7 +52,11 @@
 #endif
 
 #if defined(ENABLE_WEBRTC)
-#include "third_party/libjingle/overrides/init_webrtc.h"
+#include "third_party/webrtc_overrides/init_webrtc.h"
+#endif
+
+#if defined(USE_OZONE)
+#include "ui/ozone/public/client_native_pixmap_factory.h"
 #endif
 
 namespace content {
@@ -66,36 +72,19 @@ static void HandleRendererErrorTestParameters(
     ChildProcess::WaitForDebugger("Renderer");
 }
 
-// This is a simplified version of the browser Jankometer, which measures
-// the processing time of tasks on the render thread.
-class RendererMessageLoopObserver : public base::MessageLoop::TaskObserver {
- public:
-  RendererMessageLoopObserver()
-      : process_times_(base::Histogram::FactoryGet(
-            "Chrome.ProcMsgL RenderThread",
-            1, 3600000, 50, base::Histogram::kUmaTargetedHistogramFlag)) {}
-  ~RendererMessageLoopObserver() override {}
-
-  void WillProcessTask(const base::PendingTask& pending_task) override {
-    begin_process_message_ = base::TimeTicks::Now();
-  }
-
-  void DidProcessTask(const base::PendingTask& pending_task) override {
-    if (!begin_process_message_.is_null())
-      process_times_->AddTime(base::TimeTicks::Now() - begin_process_message_);
-  }
-
- private:
-  base::TimeTicks begin_process_message_;
-  base::HistogramBase* const process_times_;
-  DISALLOW_COPY_AND_ASSIGN(RendererMessageLoopObserver);
-};
+#if defined(USE_OZONE)
+base::LazyInstance<std::unique_ptr<ui::ClientNativePixmapFactory>>
+    g_pixmap_factory = LAZY_INSTANCE_INITIALIZER;
+#endif
 
 }  // namespace
 
 // mainline routine for running as the Renderer process
 int RendererMain(const MainFunctionParams& parameters) {
-  TRACE_EVENT_BEGIN_ETW("RendererMain", 0, "");
+  // Don't use the TRACE_EVENT0 macro because the tracing infrastructure doesn't
+  // expect synchronous events around the main loop of a thread.
+  TRACE_EVENT_ASYNC_BEGIN0("startup", "RendererMain", 0);
+
   base::trace_event::TraceLog::GetInstance()->SetProcessName("Renderer");
   base::trace_event::TraceLog::GetInstance()->SetProcessSortIndex(
       kTraceEventRendererProcessSortIndex);
@@ -118,6 +107,7 @@ int RendererMain(const MainFunctionParams& parameters) {
   }
 #endif
 
+  SkGraphics::Init();
 #if defined(OS_ANDROID)
   const int kMB = 1024 * 1024;
   size_t font_cache_limit =
@@ -125,6 +115,11 @@ int RendererMain(const MainFunctionParams& parameters) {
   SkGraphics::SetFontCacheLimit(font_cache_limit);
 #endif
 
+#if defined(USE_OZONE)
+  g_pixmap_factory.Get() = ui::ClientNativePixmapFactory::Create();
+  ui::ClientNativePixmapFactory::SetInstance(g_pixmap_factory.Get().get());
+#endif
+
   // This function allows pausing execution using the --renderer-startup-dialog
   // flag allowing us to attach a debugger.
   // Do not move this function down since that would mean we can't easily debug
@@ -132,19 +127,17 @@ int RendererMain(const MainFunctionParams& parameters) {
   HandleRendererErrorTestParameters(parsed_command_line);
 
   RendererMainPlatformDelegate platform(parameters);
-  RendererMessageLoopObserver task_observer;
 #if defined(OS_MACOSX)
   // As long as scrollbars on Mac are painted with Cocoa, the message pump
   // needs to be backed by a Foundation-level loop to process NSTimers. See
   // http://crbug.com/306348#c24 for details.
-  scoped_ptr<base::MessagePump> pump(new base::MessagePumpNSRunLoop());
-  scoped_ptr<base::MessageLoop> main_message_loop(
-      new base::MessageLoop(pump.Pass()));
+  std::unique_ptr<base::MessagePump> pump(new base::MessagePumpNSRunLoop());
+  std::unique_ptr<base::MessageLoop> main_message_loop(
+      new base::MessageLoop(std::move(pump)));
 #else
   // The main message loop of the renderer services doesn't have IO or UI tasks.
-  scoped_ptr<base::MessageLoop> main_message_loop(new base::MessageLoop());
+  std::unique_ptr<base::MessageLoop> main_message_loop(new base::MessageLoop());
 #endif
-  main_message_loop->AddTaskObserver(&task_observer);
 
   base::PlatformThread::SetName("CrRendererMain");
 
@@ -154,22 +147,12 @@ int RendererMain(const MainFunctionParams& parameters) {
   base::StatisticsRecorder::Initialize();
 
 #if defined(OS_ANDROID)
-  // If we have a pending chromium android linker histogram, record it.
-  base::android::RecordChromiumAndroidLinkerRendererHistogram();
+  // If we have any pending LibraryLoader histograms, record them.
+  base::android::RecordLibraryLoaderRendererHistograms();
 #endif
 
-  // Initialize statistical testing infrastructure.  We set the entropy provider
-  // to NULL to disallow the renderer process from creating its own one-time
-  // randomized trials; they should be created in the browser process.
-  base::FieldTrialList field_trial_list(NULL);
-  // Ensure any field trials in browser are reflected into renderer.
-  if (parsed_command_line.HasSwitch(switches::kForceFieldTrials)) {
-    bool result = base::FieldTrialList::CreateTrialsFromString(
-        parsed_command_line.GetSwitchValueASCII(switches::kForceFieldTrials),
-        base::FieldTrialList::DONT_ACTIVATE_TRIALS,
-        std::set<std::string>());
-    DCHECK(result);
-  }
+  std::unique_ptr<scheduler::RendererScheduler> renderer_scheduler(
+      scheduler::RendererScheduler::Create());
 
   // PlatformInitialize uses FieldTrials, so this must happen later.
   platform.PlatformInitialize();
@@ -191,23 +174,18 @@ int RendererMain(const MainFunctionParams& parameters) {
     // TODO(markus): Check if it is OK to unconditionally move this
     // instruction down.
     RenderProcessImpl render_process;
-    new RenderThreadImpl(main_message_loop.Pass());
+    RenderThreadImpl::Create(std::move(main_message_loop),
+                             std::move(renderer_scheduler));
 #endif
     bool run_loop = true;
-    if (!no_sandbox) {
+    if (!no_sandbox)
       run_loop = platform.EnableSandbox();
-    } else {
-#ifndef NDEBUG
-      // For convenience, we print the stack traces for crashes.  When sandbox
-      // is enabled, the in-process stack dumping is enabled as part of the
-      // EnableSandbox() call.
-      base::debug::EnableInProcessStackDumping();
-#endif
-    }
 #if defined(OS_POSIX) && !defined(OS_MACOSX)
     RenderProcessImpl render_process;
-    new RenderThreadImpl(main_message_loop.Pass());
+    RenderThreadImpl::Create(std::move(main_message_loop),
+                             std::move(renderer_scheduler));
 #endif
+
     base::HighResolutionTimerManager hi_res_timer_manager;
 
     if (run_loop) {
@@ -215,10 +193,11 @@ int RendererMain(const MainFunctionParams& parameters) {
       if (pool)
         pool->Recycle();
 #endif
-      TRACE_EVENT_BEGIN_ETW("RendererMain.START_MSG_LOOP", 0, 0);
+      TRACE_EVENT_ASYNC_BEGIN0("toplevel", "RendererMain.START_MSG_LOOP", 0);
       base::MessageLoop::current()->Run();
-      TRACE_EVENT_END_ETW("RendererMain.START_MSG_LOOP", 0, 0);
+      TRACE_EVENT_ASYNC_END0("toplevel", "RendererMain.START_MSG_LOOP", 0);
     }
+
 #if defined(LEAK_SANITIZER)
     // Run leak detection before RenderProcessImpl goes out of scope. This helps
     // ignore shutdown-only leaks.
@@ -226,7 +205,7 @@ int RendererMain(const MainFunctionParams& parameters) {
 #endif
   }
   platform.PlatformUninitialize();
-  TRACE_EVENT_END_ETW("RendererMain", 0, "");
+  TRACE_EVENT_ASYNC_END0("startup", "RendererMain", 0);
   return 0;
 }
 
