@@ -10,25 +10,37 @@
 #include "content/common/content_export.h"
 #include "third_party/WebKit/public/platform/WebGraphicsContext3DProvider.h"
 
-namespace cc_blink {
-class ContextProviderWebContext;
+namespace gpu {
+namespace gles2 {
+class GLES2Interface;
+}
 }
 
 namespace content {
+class ContextProviderCommandBuffer;
 
 class CONTENT_EXPORT WebGraphicsContext3DProviderImpl
     : public NON_EXPORTED_BASE(blink::WebGraphicsContext3DProvider) {
  public:
   explicit WebGraphicsContext3DProviderImpl(
-      scoped_refptr<cc_blink::ContextProviderWebContext> provider);
+      scoped_refptr<ContextProviderCommandBuffer> provider);
   ~WebGraphicsContext3DProviderImpl() override;
 
   // WebGraphicsContext3DProvider implementation.
-  blink::WebGraphicsContext3D* context3d() override;
+  bool bindToCurrentThread() override;
+  gpu::gles2::GLES2Interface* contextGL() override;
   GrContext* grContext() override;
+  gpu::Capabilities getCapabilities() override;
+  void setLostContextCallback(blink::WebClosure) override;
+  void setErrorMessageCallback(
+      blink::WebFunction<void(const char*, int32_t)>) override;
+
+  ContextProviderCommandBuffer* context_provider() const {
+    return provider_.get();
+  }
 
  private:
-  scoped_refptr<cc_blink::ContextProviderWebContext> provider_;
+  scoped_refptr<ContextProviderCommandBuffer> provider_;
 };
 
 }  // namespace content
