@@ -9,46 +9,31 @@
 #include "content/public/utility/content_utility_client.h"
 #include "content/public/utility/utility_thread.h"
 #include "content/utility/utility_thread_impl.h"
-#include "mojo/shell/static_application_loader.h"
 
 #if defined(ENABLE_MOJO_MEDIA_IN_UTILITY_PROCESS)
-#include "media/mojo/services/mojo_media_application.h"
+#include "media/mojo/services/mojo_media_application_factory.h"  // nogncheck
 #endif
 
 namespace content {
 
-namespace {
-
-// Called when a static application terminates.
-void QuitProcess() {
-  UtilityThread::Get()->ReleaseProcessIfNeeded();
-}
-
-}  // namespace
-
 UtilityProcessControlImpl::UtilityProcessControlImpl() {}
 
 UtilityProcessControlImpl::~UtilityProcessControlImpl() {}
 
-void UtilityProcessControlImpl::RegisterApplicationLoaders(
-    URLToLoaderMap* url_to_loader_map) {
-  URLToLoaderMap& map_ref = *url_to_loader_map;
-
-  ContentUtilityClient::StaticMojoApplicationMap apps;
-  GetContentClient()->utility()->RegisterMojoApplications(&apps);
-
-  for (const auto& entry : apps) {
-    map_ref[entry.first] = new mojo::shell::StaticApplicationLoader(
-        entry.second, base::Bind(&QuitProcess));
-  }
+void UtilityProcessControlImpl::RegisterApplications(ApplicationMap* apps) {
+  GetContentClient()->utility()->RegisterMojoApplications(apps);
 
 #if defined(ENABLE_MOJO_MEDIA_IN_UTILITY_PROCESS)
-  map_ref[GURL("mojo:media")] = new mojo::shell::StaticApplicationLoader(
-      base::Bind(&media::MojoMediaApplication::CreateApp),
-      base::Bind(&QuitProcess));
+  MojoApplicationInfo app_info;
+  app_info.application_factory = base::Bind(&media::CreateMojoMediaApplication);
+  apps->insert(std::make_pair("mojo:media", app_info));
 #endif
 }
 
+void UtilityProcessControlImpl::OnApplicationQuit() {
+  UtilityThread::Get()->ReleaseProcessIfNeeded();
+}
+
 void UtilityProcessControlImpl::OnLoadFailed() {
   UtilityThreadImpl* utility_thread =
       static_cast<UtilityThreadImpl*>(UtilityThread::Get());
