@@ -5,6 +5,7 @@
 #ifndef CONTENT_UTILITY_WEBTHREAD_IMPL_FOR_UTILITY_THREAD_H_
 #define CONTENT_UTILITY_WEBTHREAD_IMPL_FOR_UTILITY_THREAD_H_
 
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
 #include "components/scheduler/child/webthread_base.h"
 
@@ -20,8 +21,8 @@ class WebThreadImplForUtilityThread : public scheduler::WebThreadBase {
   blink::PlatformThreadId threadId() const override;
 
   // WebThreadBase implementation.
-  base::SingleThreadTaskRunner* TaskRunner() const override;
-  scheduler::SingleThreadIdleTaskRunner* IdleTaskRunner() const override;
+  base::SingleThreadTaskRunner* GetTaskRunner() const override;
+  scheduler::SingleThreadIdleTaskRunner* GetIdleTaskRunner() const override;
 
  private:
   scoped_refptr<base::SingleThreadTaskRunner> task_runner_;
