@@ -8,7 +8,7 @@
 
 #include "base/logging.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "gin/public/gin_embedders.h"
 
 using v8::ArrayBuffer;
@@ -114,7 +114,7 @@ NamedPropertyInterceptor* PerIsolateData::GetNamedPropertyInterceptor(
 }
 
 void PerIsolateData::EnableIdleTasks(
-    scoped_ptr<V8IdleTaskRunner> idle_task_runner) {
+    std::unique_ptr<V8IdleTaskRunner> idle_task_runner) {
   idle_task_runner_ = std::move(idle_task_runner);
 }
 
