@@ -29,7 +29,7 @@ enum TrackingEventType {
   SIGN_OUT
 };
 
-std::string AccountKeyToObfuscatedId(const std::string email) {
+std::string AccountKeyToObfuscatedId(const std::string& email) {
   return "obfid-" + email;
 }
 
@@ -294,7 +294,7 @@ class IdentityAccountTrackerTest : public testing::Test {
 
   // Helpers to pass fake events to the tracker.
 
-  void NotifyLogin(const std::string account_key) {
+  void NotifyLogin(const std::string& account_key) {
     identity_provider()->LogIn(account_key);
   }
 
@@ -314,7 +314,7 @@ class IdentityAccountTrackerTest : public testing::Test {
         username, "access_token-" + username, base::Time::Max());
   }
 
-  std::string GetValidTokenInfoResponse(const std::string account_key) {
+  std::string GetValidTokenInfoResponse(const std::string& account_key) {
     return std::string("{ \"id\": \"") + AccountKeyToObfuscatedId(account_key) +
            "\" }";
   }
@@ -346,10 +346,10 @@ class IdentityAccountTrackerTest : public testing::Test {
 
   base::MessageLoopForIO message_loop_;  // net:: stuff needs IO message loop.
   net::TestURLFetcherFactory test_fetcher_factory_;
-  scoped_ptr<FakeOAuth2TokenService> fake_oauth2_token_service_;
-  scoped_ptr<FakeIdentityProvider> fake_identity_provider_;
+  std::unique_ptr<FakeOAuth2TokenService> fake_oauth2_token_service_;
+  std::unique_ptr<FakeIdentityProvider> fake_identity_provider_;
 
-  scoped_ptr<AccountTracker> account_tracker_;
+  std::unique_ptr<AccountTracker> account_tracker_;
   AccountTrackerObserver observer_;
 };
 
