@@ -7,7 +7,7 @@
 #include "base/bind.h"
 #include "base/location.h"
 #include "base/metrics/histogram.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "google_apis/gcm/monitoring/gcm_stats_recorder.h"
 #include "google_apis/gcm/protocol/checkin.pb.h"
 #include "net/base/load_flags.h"
@@ -91,6 +91,8 @@ CheckinRequest::RequestInfo::RequestInfo(
       settings_digest(settings_digest),
       chrome_build_proto(chrome_build_proto) {}
 
+CheckinRequest::RequestInfo::RequestInfo(const RequestInfo& other) = default;
+
 CheckinRequest::RequestInfo::~RequestInfo() {}
 
 CheckinRequest::CheckinRequest(
