@@ -5,11 +5,12 @@
 #include "google_apis/gcm/engine/connection_handler_impl.h"
 
 #include <stdint.h>
+
+#include <memory>
 #include <string>
 #include <utility>
 
 #include "base/bind.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/run_loop.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/test/test_timeouts.h"
@@ -19,6 +20,7 @@
 #include "google_apis/gcm/base/mcs_util.h"
 #include "google_apis/gcm/base/socket_stream.h"
 #include "google_apis/gcm/protocol/mcs.pb.h"
+#include "net/base/ip_address.h"
 #include "net/socket/socket_test_util.h"
 #include "net/socket/stream_socket.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -26,7 +28,7 @@
 namespace gcm {
 namespace {
 
-typedef scoped_ptr<google::protobuf::MessageLite> ScopedMessage;
+typedef std::unique_ptr<google::protobuf::MessageLite> ScopedMessage;
 typedef std::vector<net::MockRead> ReadList;
 typedef std::vector<net::MockWrite> WriteList;
 
@@ -170,28 +172,27 @@ class GCMConnectionHandlerImplTest : public testing::Test {
   // SocketStreams and their data provider.
   ReadList mock_reads_;
   WriteList mock_writes_;
-  scoped_ptr<net::StaticSocketDataProvider> data_provider_;
+  std::unique_ptr<net::StaticSocketDataProvider> data_provider_;
 
   // The connection handler being tested.
-  scoped_ptr<ConnectionHandlerImpl> connection_handler_;
+  std::unique_ptr<ConnectionHandlerImpl> connection_handler_;
 
   // The last connection error received.
   int last_error_;
 
   // net:: components.
-  scoped_ptr<net::StreamSocket> socket_;
+  std::unique_ptr<net::StreamSocket> socket_;
   net::MockClientSocketFactory socket_factory_;
   net::AddressList address_list_;
 
   base::MessageLoopForIO message_loop_;
-  scoped_ptr<base::RunLoop> run_loop_;
+  std::unique_ptr<base::RunLoop> run_loop_;
 };
 
 GCMConnectionHandlerImplTest::GCMConnectionHandlerImplTest()
   : last_error_(0) {
-  net::IPAddressNumber ip_number;
-  net::ParseIPLiteralToNumber("127.0.0.1", &ip_number);
-  address_list_ = net::AddressList::CreateFromIPAddress(ip_number, kMCSPort);
+  address_list_ = net::AddressList::CreateFromIPAddress(
+      net::IPAddress::IPv4Localhost(), kMCSPort);
 }
 
 GCMConnectionHandlerImplTest::~GCMConnectionHandlerImplTest() {
@@ -209,7 +210,7 @@ net::StreamSocket* GCMConnectionHandlerImplTest::BuildSocket(
   socket_factory_.AddSocketDataProvider(data_provider_.get());
 
   socket_ = socket_factory_.CreateTransportClientSocket(
-      address_list_, NULL, net::NetLog::Source());
+      address_list_, NULL, NULL, net::NetLog::Source());
   socket_->Connect(net::CompletionCallback());
 
   run_loop_.reset(new base::RunLoop());
