@@ -2,66 +2,39 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stdint.h>
 #include <map>
 #include <string>
+#include <utility>
 #include <vector>
 
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_tokenizer.h"
 #include "google_apis/gcm/engine/gcm_registration_request_handler.h"
+#include "google_apis/gcm/engine/gcm_request_test_base.h"
 #include "google_apis/gcm/engine/instance_id_get_token_request_handler.h"
 #include "google_apis/gcm/monitoring/fake_gcm_stats_recorder.h"
+#include "net/base/escape.h"
 #include "net/base/load_flags.h"
-#include "net/url_request/test_url_fetcher_factory.h"
+#include "net/base/net_errors.h"
 #include "net/url_request/url_request_status.h"
-#include "net/url_request/url_request_test_util.h"
-#include "testing/gtest/include/gtest/gtest.h"
 
 namespace gcm {
 
 namespace {
-const uint64 kAndroidId = 42UL;
+const uint64_t kAndroidId = 42UL;
 const char kAppId[] = "TestAppId";
 const char kDeveloperId[] = "Project1";
 const char kLoginHeader[] = "AidLogin";
 const char kRegistrationURL[] = "http://foo.bar/register";
-const uint64 kSecurityToken = 77UL;
+const uint64_t kSecurityToken = 77UL;
 const int kGCMVersion = 40;
 const char kInstanceId[] = "IID1";
 const char kScope[] = "GCM";
 
-// Backoff policy for testing registration request.
-const net::BackoffEntry::Policy kDefaultBackoffPolicy = {
-  // Number of initial errors (in sequence) to ignore before applying
-  // exponential back-off rules.
-  // Explicitly set to 2 to skip the delay on the first retry, as we are not
-  // trying to test the backoff itself, but rather the fact that retry happens.
-  2,
-
-  // Initial delay for exponential back-off in ms.
-  15000,  // 15 seconds.
-
-  // Factor by which the waiting time will be multiplied.
-  2,
-
-  // Fuzzing percentage. ex: 10% will spread requests randomly
-  // between 90%-100% of the calculated time.
-  0.5,  // 50%.
-
-  // Maximum amount of time we are willing to delay our request in ms.
-  1000 * 60 * 5, // 5 minutes.
-
-  // Time to keep an entry from being discarded even when it
-  // has no significant state, -1 to never discard.
-  -1,
-
-  // Don't use initial delay unless the last request was an error.
-  false,
-};
-
 }  // namespace
 
-class RegistrationRequestTest : public testing::Test {
+class RegistrationRequestTest : public GCMRequestTestBase {
  public:
   RegistrationRequestTest();
   ~RegistrationRequestTest() override;
@@ -69,9 +42,8 @@ class RegistrationRequestTest : public testing::Test {
   void RegistrationCallback(RegistrationRequest::Status status,
                             const std::string& registration_id);
 
-  void SetResponseStatusAndString(net::HttpStatusCode status_code,
-                                  const std::string& response_body);
-  void CompleteFetch();
+  void CompleteFetch() override;
+
   void set_max_retry_count(int max_retry_count) {
     max_retry_count_ = max_retry_count;
   }
@@ -82,19 +54,14 @@ class RegistrationRequestTest : public testing::Test {
   std::string registration_id_;
   bool callback_called_;
   std::map<std::string, std::string> extras_;
-  scoped_ptr<RegistrationRequest> request_;
-  base::MessageLoop message_loop_;
-  net::TestURLFetcherFactory url_fetcher_factory_;
-  scoped_refptr<net::TestURLRequestContextGetter> url_request_context_getter_;
+  std::unique_ptr<RegistrationRequest> request_;
   FakeGCMStatsRecorder recorder_;
 };
 
 RegistrationRequestTest::RegistrationRequestTest()
     : max_retry_count_(2),
       status_(RegistrationRequest::SUCCESS),
-      callback_called_(false),
-      url_request_context_getter_(new net::TestURLRequestContextGetter(
-          message_loop_.task_runner())) {}
+      callback_called_(false) {}
 
 RegistrationRequestTest::~RegistrationRequestTest() {}
 
@@ -106,23 +73,12 @@ void RegistrationRequestTest::RegistrationCallback(
   callback_called_ = true;
 }
 
-void RegistrationRequestTest::SetResponseStatusAndString(
-    net::HttpStatusCode status_code,
-    const std::string& response_body) {
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
-  ASSERT_TRUE(fetcher);
-  fetcher->set_response_code(status_code);
-  fetcher->SetResponseString(response_body);
-}
-
 void RegistrationRequestTest::CompleteFetch() {
   registration_id_.clear();
   status_ = RegistrationRequest::SUCCESS;
   callback_called_ = false;
 
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
-  ASSERT_TRUE(fetcher);
-  fetcher->delegate()->OnURLFetchComplete(fetcher);
+  GCMRequestTestBase::CompleteFetch();
 }
 
 class GCMRegistrationRequestTest : public RegistrationRequestTest {
@@ -142,19 +98,14 @@ GCMRegistrationRequestTest::~GCMRegistrationRequestTest() {
 void GCMRegistrationRequestTest::CreateRequest(const std::string& sender_ids) {
   RegistrationRequest::RequestInfo request_info(
       kAndroidId, kSecurityToken, kAppId);
-  scoped_ptr<GCMRegistrationRequestHandler> request_handler(
+  std::unique_ptr<GCMRegistrationRequestHandler> request_handler(
       new GCMRegistrationRequestHandler(sender_ids));
   request_.reset(new RegistrationRequest(
-      GURL(kRegistrationURL),
-      request_info,
-      request_handler.Pass(),
-      kDefaultBackoffPolicy,
+      GURL(kRegistrationURL), request_info, std::move(request_handler),
+      GetBackoffPolicy(),
       base::Bind(&RegistrationRequestTest::RegistrationCallback,
                  base::Unretained(this)),
-      max_retry_count_,
-      url_request_context_getter_.get(),
-      &recorder_,
-      sender_ids));
+      max_retry_count_, url_request_context_getter(), &recorder_, sender_ids));
 }
 
 TEST_F(GCMRegistrationRequestTest, RequestSuccessful) {
@@ -162,7 +113,7 @@ TEST_F(GCMRegistrationRequestTest, RequestSuccessful) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -175,7 +126,7 @@ TEST_F(GCMRegistrationRequestTest, RequestDataAndURL) {
   request_->Start();
 
   // Get data sent by request.
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
+  net::TestURLFetcher* fetcher = GetFetcher();
   ASSERT_TRUE(fetcher);
 
   EXPECT_EQ(GURL(kRegistrationURL), fetcher->GetOriginalURL());
@@ -218,7 +169,7 @@ TEST_F(GCMRegistrationRequestTest, RequestRegistrationWithMultipleSenderIds) {
   CreateRequest("sender1,sender2@gmail.com");
   request_->Start();
 
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
+  net::TestURLFetcher* fetcher = GetFetcher();
   ASSERT_TRUE(fetcher);
 
   // Verify data was formatted properly.
@@ -230,8 +181,10 @@ TEST_F(GCMRegistrationRequestTest, RequestRegistrationWithMultipleSenderIds) {
     continue;
 
   ASSERT_TRUE(data_tokenizer.GetNext());
-  std::string senders(net::UnescapeURLComponent(data_tokenizer.token(),
-      net::UnescapeRule::URL_SPECIAL_CHARS));
+  std::string senders(net::UnescapeURLComponent(
+      data_tokenizer.token(),
+      net::UnescapeRule::PATH_SEPARATORS |
+          net::UnescapeRule::URL_SPECIAL_CHARS_EXCEPT_PATH_SEPARATORS));
   base::StringTokenizer sender_tokenizer(senders, ",");
   ASSERT_TRUE(sender_tokenizer.GetNext());
   EXPECT_EQ("sender1", sender_tokenizer.token());
@@ -243,7 +196,7 @@ TEST_F(GCMRegistrationRequestTest, ResponseParsing) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -255,12 +208,12 @@ TEST_F(GCMRegistrationRequestTest, ResponseHttpStatusNotOK) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_UNAUTHORIZED, "token=2501");
+  SetResponse(net::HTTP_UNAUTHORIZED, "token=2501");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -272,18 +225,18 @@ TEST_F(GCMRegistrationRequestTest, ResponseMissingRegistrationId) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "");
+  SetResponse(net::HTTP_OK, "");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
-  SetResponseStatusAndString(net::HTTP_OK, "some error in response");
+  SetResponse(net::HTTP_OK, "some error in response");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
   // Ensuring a retry happened and succeeds.
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -295,13 +248,13 @@ TEST_F(GCMRegistrationRequestTest, ResponseDeviceRegistrationError) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "Error=PHONE_REGISTRATION_ERROR");
+  SetResponse(net::HTTP_OK, "Error=PHONE_REGISTRATION_ERROR");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
   // Ensuring a retry happened and succeeds.
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -313,14 +266,14 @@ TEST_F(GCMRegistrationRequestTest, ResponseAuthenticationError) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_UNAUTHORIZED,
+  SetResponse(net::HTTP_UNAUTHORIZED,
                              "Error=AUTHENTICATION_FAILED");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
   // Ensuring a retry happened and succeeds.
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -332,7 +285,7 @@ TEST_F(GCMRegistrationRequestTest, ResponseInvalidParameters) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "Error=INVALID_PARAMETERS");
+  SetResponse(net::HTTP_OK, "Error=INVALID_PARAMETERS");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -344,7 +297,7 @@ TEST_F(GCMRegistrationRequestTest, ResponseInvalidSender) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "Error=INVALID_SENDER");
+  SetResponse(net::HTTP_OK, "Error=INVALID_SENDER");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -356,7 +309,7 @@ TEST_F(GCMRegistrationRequestTest, ResponseInvalidSenderBadRequest) {
   CreateRequest("sender1");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_BAD_REQUEST, "Error=INVALID_SENDER");
+  SetResponse(net::HTTP_BAD_REQUEST, "Error=INVALID_SENDER");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -368,18 +321,18 @@ TEST_F(GCMRegistrationRequestTest, RequestNotSuccessful) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  net::URLRequestStatus request_status(net::URLRequestStatus::FAILED, 1);
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
+  SetResponse(net::HTTP_OK, "token=2501");
+
+  net::TestURLFetcher* fetcher = GetFetcher();
   ASSERT_TRUE(fetcher);
-  fetcher->set_status(request_status);
+  GetFetcher()->set_status(net::URLRequestStatus::FromError(net::ERR_FAILED));
 
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
   // Ensuring a retry happened and succeeded.
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -391,13 +344,13 @@ TEST_F(GCMRegistrationRequestTest, ResponseHttpNotOk) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
+  SetResponse(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
   // Ensuring a retry happened and succeeded.
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -410,7 +363,7 @@ TEST_F(GCMRegistrationRequestTest, MaximumAttemptsReachedWithZeroRetries) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
+  SetResponse(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -422,17 +375,17 @@ TEST_F(GCMRegistrationRequestTest, MaximumAttemptsReached) {
   CreateRequest("sender1,sender2");
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
+  SetResponse(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
-  SetResponseStatusAndString(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
+  SetResponse(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
-  SetResponseStatusAndString(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
+  SetResponse(net::HTTP_GATEWAY_TIMEOUT, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -464,19 +417,15 @@ void InstanceIDGetTokenRequestTest::CreateRequest(
     const std::map<std::string, std::string>& options) {
   RegistrationRequest::RequestInfo request_info(
       kAndroidId, kSecurityToken, kAppId);
-  scoped_ptr<InstanceIDGetTokenRequestHandler> request_handler(
-      new InstanceIDGetTokenRequestHandler(
-          instance_id, authorized_entity, scope, kGCMVersion, options));
+  std::unique_ptr<InstanceIDGetTokenRequestHandler> request_handler(
+      new InstanceIDGetTokenRequestHandler(instance_id, authorized_entity,
+                                           scope, kGCMVersion, options));
   request_.reset(new RegistrationRequest(
-      GURL(kRegistrationURL),
-      request_info,
-      request_handler.Pass(),
-      kDefaultBackoffPolicy,
+      GURL(kRegistrationURL), request_info, std::move(request_handler),
+      GetBackoffPolicy(),
       base::Bind(&RegistrationRequestTest::RegistrationCallback,
                  base::Unretained(this)),
-      max_retry_count_,
-      url_request_context_getter_.get(),
-      &recorder_,
+      max_retry_count_, url_request_context_getter(), &recorder_,
       authorized_entity));
 }
 
@@ -488,7 +437,7 @@ TEST_F(InstanceIDGetTokenRequestTest, RequestSuccessful) {
   CreateRequest(kInstanceId, kDeveloperId, kScope, options);
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
@@ -503,7 +452,7 @@ TEST_F(InstanceIDGetTokenRequestTest, RequestDataAndURL) {
   request_->Start();
 
   // Get data sent by request.
-  net::TestURLFetcher* fetcher = url_fetcher_factory_.GetFetcherByID(0);
+  net::TestURLFetcher* fetcher = GetFetcher();
   ASSERT_TRUE(fetcher);
 
   EXPECT_EQ(GURL(kRegistrationURL), fetcher->GetOriginalURL());
@@ -558,12 +507,12 @@ TEST_F(InstanceIDGetTokenRequestTest, ResponseHttpStatusNotOK) {
   CreateRequest(kInstanceId, kDeveloperId, kScope, options);
   request_->Start();
 
-  SetResponseStatusAndString(net::HTTP_UNAUTHORIZED, "token=2501");
+  SetResponse(net::HTTP_UNAUTHORIZED, "token=2501");
   CompleteFetch();
 
   EXPECT_FALSE(callback_called_);
 
-  SetResponseStatusAndString(net::HTTP_OK, "token=2501");
+  SetResponse(net::HTTP_OK, "token=2501");
   CompleteFetch();
 
   EXPECT_TRUE(callback_called_);
