@@ -9,11 +9,12 @@
 
 #include <stddef.h>
 
+#include <memory>
+
 #include "base/command_line.h"
 #include "base/environment.h"
 #include "base/lazy_instance.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/strings/stringize_macros.h"
 #include "google_apis/gaia/gaia_switches.h"
 
@@ -29,8 +30,8 @@
 #define GOOGLE_API_KEY DUMMY_API_TOKEN
 #endif
 
-#if !defined(GOOGLE_API_KEY_SAFESITES)
-#define GOOGLE_API_KEY_SAFESITES DUMMY_API_TOKEN
+#if !defined(GOOGLE_API_KEY_REMOTING)
+#define GOOGLE_API_KEY_REMOTING DUMMY_API_TOKEN
 #endif
 
 #if !defined(GOOGLE_CLIENT_ID_MAIN)
@@ -65,6 +66,13 @@
 #define GOOGLE_CLIENT_SECRET_REMOTING_HOST DUMMY_API_TOKEN
 #endif
 
+// This is really the API key for non-stable channels on Android. It's
+// named after the first feature that used it.
+// TODO(jkrcal,rogerta): Rename this to GOOGLE_API_KEY_ANDROID_NON_STABLE.
+#if !defined(GOOGLE_API_KEY_PHYSICAL_WEB_TEST)
+#define GOOGLE_API_KEY_PHYSICAL_WEB_TEST DUMMY_API_TOKEN
+#endif
+
 // These are used as shortcuts for developers and users providing
 // OAuth credentials via preprocessor defines or environment
 // variables.  If set, they will be used to replace any of the client
@@ -86,19 +94,26 @@ const char kAPIKeysDevelopersHowToURL[] =
 class APIKeyCache {
  public:
   APIKeyCache() {
-    scoped_ptr<base::Environment> environment(base::Environment::Create());
+    std::unique_ptr<base::Environment> environment(base::Environment::Create());
     base::CommandLine* command_line = base::CommandLine::ForCurrentProcess();
 
-    api_key_ = CalculateKeyValue(GOOGLE_API_KEY,
-                                 STRINGIZE_NO_EXPANSION(GOOGLE_API_KEY),
-                                 NULL,
-                                 std::string(),
-                                 environment.get(),
-                                 command_line);
+    api_key_ = CalculateKeyValue(
+        GOOGLE_API_KEY, STRINGIZE_NO_EXPANSION(GOOGLE_API_KEY), NULL,
+        std::string(), environment.get(), command_line);
 
-    api_key_safesites_ =
-        CalculateKeyValue(GOOGLE_API_KEY_SAFESITES,
-                          STRINGIZE_NO_EXPANSION(GOOGLE_API_KEY_SAFESITES),
+// A special non-stable key is at the moment defined only for Android Chrome.
+#if defined(OS_ANDROID)
+    api_key_non_stable_ = CalculateKeyValue(
+            GOOGLE_API_KEY_PHYSICAL_WEB_TEST,
+            STRINGIZE_NO_EXPANSION(GOOGLE_API_KEY_PHYSICAL_WEB_TEST), NULL,
+            std::string(), environment.get(), command_line);
+#else
+    api_key_non_stable_ = api_key_;
+#endif
+
+    api_key_remoting_ =
+        CalculateKeyValue(GOOGLE_API_KEY_REMOTING,
+                          STRINGIZE_NO_EXPANSION(GOOGLE_API_KEY_REMOTING),
                           NULL,
                           std::string(),
                           environment.get(),
@@ -187,7 +202,8 @@ class APIKeyCache {
   }
 
   std::string api_key() const { return api_key_; }
-  std::string api_key_safesites() const { return api_key_safesites_; }
+  std::string api_key_non_stable() const { return api_key_non_stable_; }
+  std::string api_key_remoting() const { return api_key_remoting_; }
 
   std::string GetClientID(OAuth2Client client) const {
     DCHECK_LT(client, CLIENT_NUM_ITEMS);
@@ -253,7 +269,8 @@ class APIKeyCache {
   }
 
   std::string api_key_;
-  std::string api_key_safesites_;
+  std::string api_key_non_stable_;
+  std::string api_key_remoting_;
   std::string client_ids_[CLIENT_NUM_ITEMS];
   std::string client_secrets_[CLIENT_NUM_ITEMS];
 };
@@ -280,8 +297,12 @@ std::string GetAPIKey() {
   return g_api_key_cache.Get().api_key();
 }
 
-std::string GetSafeSitesAPIKey() {
-  return g_api_key_cache.Get().api_key_safesites();
+std::string GetNonStableAPIKey() {
+  return g_api_key_cache.Get().api_key_non_stable();
+}
+
+std::string GetRemotingAPIKey() {
+  return g_api_key_cache.Get().api_key_remoting();
 }
 
 std::string GetOAuth2ClientID(OAuth2Client client) {
