@@ -46,6 +46,9 @@ ProgramInfoManager::Program::UniformInfo::UniformInfo(
   DCHECK(!(size > 1 && !is_array));
 }
 
+ProgramInfoManager::Program::UniformInfo::UniformInfo(
+    const UniformInfo& other) = default;
+
 ProgramInfoManager::Program::UniformInfo::~UniformInfo() {
 }
 
@@ -67,6 +70,9 @@ ProgramInfoManager::Program::UniformBlock::UniformBlock()
       referenced_by_fragment_shader(false) {
 }
 
+ProgramInfoManager::Program::UniformBlock::UniformBlock(
+    const UniformBlock& other) = default;
+
 ProgramInfoManager::Program::UniformBlock::~UniformBlock() {
 }
 
@@ -93,6 +99,8 @@ ProgramInfoManager::Program::Program()
       cached_es3_uniformsiv_(false) {
 }
 
+ProgramInfoManager::Program::Program(const Program& other) = default;
+
 ProgramInfoManager::Program::~Program() {
 }
 
