@@ -13,12 +13,13 @@
 
 #include "base/callback.h"
 #include "base/macros.h"
+#include "gpu/command_buffer/common/gles2_cmd_utils.h"
 #include "gpu/command_buffer/common/mailbox.h"
 #include "gpu/command_buffer/service/gles2_cmd_decoder.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "ui/gfx/geometry/size.h"
 
-namespace gfx {
+namespace gl {
 class GLContext;
 class GLSurface;
 }
@@ -43,22 +44,25 @@ class MockGLES2Decoder : public GLES2Decoder {
                               int num_entries,
                               int* entries_processed);
 
-  MOCK_METHOD6(Initialize,
-               bool(const scoped_refptr<gfx::GLSurface>& surface,
-                    const scoped_refptr<gfx::GLContext>& context,
+  MOCK_METHOD5(Initialize,
+               bool(const scoped_refptr<gl::GLSurface>& surface,
+                    const scoped_refptr<gl::GLContext>& context,
                     bool offscreen,
-                    const gfx::Size& size,
                     const DisallowedFeatures& disallowed_features,
-                    const std::vector<int32_t>& attribs));
+                    const ContextCreationAttribHelper& attrib_helper));
   MOCK_METHOD1(Destroy, void(bool have_context));
-  MOCK_METHOD1(SetSurface, void(const scoped_refptr<gfx::GLSurface>& surface));
-  MOCK_METHOD1(ProduceFrontBuffer, void(const Mailbox& mailbox));
+  MOCK_METHOD1(SetSurface, void(const scoped_refptr<gl::GLSurface>& surface));
+  MOCK_METHOD0(ReleaseSurface, void());
+  MOCK_METHOD1(TakeFrontBuffer, void(const Mailbox& mailbox));
+  MOCK_METHOD2(ReturnFrontBuffer, void(const Mailbox& mailbox, bool is_lost));
+  MOCK_METHOD0(GetSavedBackTextureCountForTest, size_t());
+  MOCK_METHOD0(GetCreatedBackTextureCountForTest, size_t());
   MOCK_METHOD1(ResizeOffscreenFrameBuffer, bool(const gfx::Size& size));
   MOCK_METHOD0(MakeCurrent, bool());
   MOCK_METHOD1(GetServiceIdForTesting, uint32_t(uint32_t client_id));
   MOCK_METHOD0(GetGLES2Util, GLES2Util*());
-  MOCK_METHOD0(GetGLSurface, gfx::GLSurface*());
-  MOCK_METHOD0(GetGLContext, gfx::GLContext*());
+  MOCK_METHOD0(GetGLSurface, gl::GLSurface*());
+  MOCK_METHOD0(GetGLContext, gl::GLContext*());
   MOCK_METHOD0(GetContextGroup, ContextGroup*());
   MOCK_METHOD0(GetContextState, const ContextState*());
   MOCK_METHOD0(GetCapabilities, Capabilities());
@@ -66,6 +70,8 @@ class MockGLES2Decoder : public GLES2Decoder {
   MOCK_METHOD1(ProcessPendingQueries, void(bool));
   MOCK_CONST_METHOD0(HasMoreIdleWork, bool());
   MOCK_METHOD0(PerformIdleWork, void());
+  MOCK_CONST_METHOD0(HasPollingWork, bool());
+  MOCK_METHOD0(PerformPollingWork, void());
   MOCK_METHOD1(RestoreState, void(const ContextState* prev_state));
   MOCK_CONST_METHOD0(RestoreActiveTexture, void());
   MOCK_CONST_METHOD1(
@@ -83,9 +89,10 @@ class MockGLES2Decoder : public GLES2Decoder {
   MOCK_CONST_METHOD0(ClearAllAttributes, void());
   MOCK_CONST_METHOD0(RestoreAllAttributes, void());
   MOCK_METHOD0(GetQueryManager, gpu::gles2::QueryManager*());
+  MOCK_METHOD0(
+      GetTransformFeedbackManager, gpu::gles2::TransformFeedbackManager*());
   MOCK_METHOD0(GetVertexArrayManager, gpu::gles2::VertexArrayManager*());
   MOCK_METHOD0(GetImageManager, gpu::gles2::ImageManager*());
-  MOCK_METHOD0(GetValuebufferManager, gpu::gles2::ValuebufferManager*());
   MOCK_METHOD1(
       SetResizeCallback, void(const base::Callback<void(gfx::Size, float)>&));
   MOCK_METHOD1(SetIgnoreCachedStateForTest, void(bool ignore));
@@ -113,17 +120,37 @@ class MockGLES2Decoder : public GLES2Decoder {
                     int y_offset,
                     int width,
                     int height));
+  MOCK_METHOD6(ClearCompressedTextureLevel,
+               bool(Texture* texture,
+                    unsigned target,
+                    int level,
+                    unsigned format,
+                    int width,
+                    int height));
+  MOCK_METHOD1(IsCompressedTextureFormat,
+               bool(unsigned format));
+  MOCK_METHOD8(ClearLevel3D,
+               bool(Texture* texture,
+                    unsigned target,
+                    int level,
+                    unsigned format,
+                    unsigned type,
+                    int width,
+                    int height,
+                    int depth));
   MOCK_METHOD0(GetErrorState, ErrorState *());
 
   MOCK_METHOD0(GetLogger, Logger*());
   MOCK_METHOD1(SetShaderCacheCallback,
                void(const ShaderCacheCallback& callback));
-  MOCK_METHOD1(SetWaitSyncPointCallback,
-               void(const WaitSyncPointCallback& callback));
   MOCK_METHOD1(SetFenceSyncReleaseCallback,
                void(const FenceSyncReleaseCallback& callback));
   MOCK_METHOD1(SetWaitFenceSyncCallback,
                void(const WaitFenceSyncCallback& callback));
+  MOCK_METHOD1(SetDescheduleUntilFinishedCallback,
+               void(const NoParamCallback& callback));
+  MOCK_METHOD1(SetRescheduleAfterFinishedCallback,
+               void(const NoParamCallback& callback));
   MOCK_METHOD1(WaitForReadPixels,
                void(base::Closure callback));
   MOCK_METHOD0(GetTextureUploadCount, uint32_t());
