@@ -16,7 +16,7 @@
 #include "ui/gl/gl_fence.h"
 #include "ui/gl/gl_implementation.h"
 
-#if !defined(OS_MACOSX) && !defined(TOOLKIT_QT)
+#if !defined(OS_MACOSX)
 #include "ui/gl/gl_fence_egl.h"
 #endif
 
@@ -27,8 +27,8 @@ namespace {
 
 base::LazyInstance<base::Lock> g_lock = LAZY_INSTANCE_INITIALIZER;
 
-#if !defined(OS_MACOSX) || defined(TOOLKIT_QT)
-typedef std::map<SyncToken, linked_ptr<gfx::GLFence>> SyncTokenToFenceMap;
+#if !defined(OS_MACOSX)
+typedef std::map<SyncToken, linked_ptr<gl::GLFence>> SyncTokenToFenceMap;
 base::LazyInstance<SyncTokenToFenceMap> g_sync_point_to_fence =
     LAZY_INSTANCE_INITIALIZER;
 base::LazyInstance<std::queue<SyncTokenToFenceMap::iterator>> g_sync_points =
@@ -36,9 +36,9 @@ base::LazyInstance<std::queue<SyncTokenToFenceMap::iterator>> g_sync_points =
 #endif
 
 void CreateFenceLocked(const SyncToken& sync_token) {
-#if !defined(OS_MACOSX) || defined(TOOLKIT_QT)
+#if !defined(OS_MACOSX)
   g_lock.Get().AssertAcquired();
-  if (gfx::GetGLImplementation() == gfx::kGLImplementationMockGL)
+  if (gl::GetGLImplementation() == gl::kGLImplementationMockGL)
     return;
 
   std::queue<SyncTokenToFenceMap::iterator>& sync_points = g_sync_points.Get();
@@ -49,13 +49,8 @@ void CreateFenceLocked(const SyncToken& sync_token) {
       sync_point_to_fence.erase(sync_points.front());
       sync_points.pop();
     }
-#if !defined(TOOLKIT_QT)
     // Need to use EGL fences since we are likely not in a single share group.
-    linked_ptr<gfx::GLFence> fence(make_linked_ptr(new gfx::GLFenceEGL));
-#else
-    // Need to use a fence that is actually supported, and we use the same share group.
-    linked_ptr<gfx::GLFence> fence(make_linked_ptr(gfx::GLFence::Create()));
-#endif
+    linked_ptr<gl::GLFence> fence(make_linked_ptr(new gl::GLFenceEGL));
     if (fence.get()) {
       std::pair<SyncTokenToFenceMap::iterator, bool> result =
           sync_point_to_fence.insert(std::make_pair(sync_token, fence));
@@ -68,7 +63,7 @@ void CreateFenceLocked(const SyncToken& sync_token) {
 }
 
 void AcquireFenceLocked(const SyncToken& sync_token) {
-#if !defined(OS_MACOSX) || defined(TOOLKIT_QT)
+#if !defined(OS_MACOSX)
   g_lock.Get().AssertAcquired();
   SyncTokenToFenceMap::iterator fence_it =
       g_sync_point_to_fence.Get().find(sync_token);
@@ -171,6 +166,9 @@ MailboxManagerSync::TextureGroupRef::TextureGroupRef(unsigned version,
     : version(version), group(group) {
 }
 
+MailboxManagerSync::TextureGroupRef::TextureGroupRef(
+    const TextureGroupRef& other) = default;
+
 MailboxManagerSync::TextureGroupRef::~TextureGroupRef() {
 }
 
@@ -294,8 +292,7 @@ void MailboxManagerSync::UpdateDefinitionLocked(
   if (definition.Matches(texture))
     return;
 
-  DCHECK(!image || image_buffer.get());
-  if (image && !image_buffer->IsClient(image)) {
+  if (image && (!image_buffer || !image_buffer->IsClient(image))) {
     LOG(ERROR) << "MailboxSync: Incompatible attachment";
     return;
   }
