@@ -4,6 +4,11 @@
 
 #include "gpu/config/gpu_control_list.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
+#include <utility>
+
 #include "base/cpu.h"
 #include "base/json/json_reader.h"
 #include "base/logging.h"
@@ -14,7 +19,7 @@
 #include "base/sys_info.h"
 #include "gpu/config/gpu_info.h"
 #include "gpu/config/gpu_util.h"
-#include "third_party/re2/re2/re2.h"
+#include "third_party/re2/src/re2/re2.h"
 
 namespace gpu {
 namespace {
@@ -25,7 +30,9 @@ bool ProcessVersionString(const std::string& version_string,
                           char splitter,
                           std::vector<std::string>* version) {
   DCHECK(version);
-  base::SplitString(version_string, splitter, version);
+  *version = base::SplitString(
+      version_string, std::string(1, splitter),
+      base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
   if (version->size() == 0)
     return false;
   // If the splitter is '-', we assume it's a date with format "mm-dd-yyyy";
@@ -377,7 +384,7 @@ GpuControlList::GpuControlListEntry::GetEntryFromValue(
   size_t dictionary_entry_count = 0;
 
   if (top_level) {
-    uint32 id;
+    uint32_t id;
     if (!value->GetInteger("id", reinterpret_cast<int*>(&id)) ||
         !entry->SetId(id)) {
       LOG(WARNING) << "Malformed id entry " << entry->id();
@@ -723,21 +730,50 @@ GpuControlList::GpuControlListEntry::GetEntryFromValue(
     dictionary_entry_count++;
   }
 
+  bool in_process_gpu;
+  if (value->GetBoolean("in_process_gpu", &in_process_gpu)) {
+    entry->SetInProcessGPUInfo(in_process_gpu);
+    dictionary_entry_count++;
+  }
+
   if (top_level) {
     const base::ListValue* feature_value = NULL;
     if (value->GetList("features", &feature_value)) {
       std::vector<std::string> feature_list;
+      std::vector<std::string> feature_exception_list;
       for (size_t i = 0; i < feature_value->GetSize(); ++i) {
         std::string feature;
+        const base::DictionaryValue* features_info_value = NULL;
         if (feature_value->GetString(i, &feature)) {
           feature_list.push_back(feature);
+        } else if (feature_value->GetDictionary(i, &features_info_value)) {
+          const base::ListValue* exception_list_value = NULL;
+          if (features_info_value->size() > 1) {
+            LOG(WARNING) << "Malformed feature entry " << entry->id();
+            return NULL;
+          }
+          if (features_info_value->GetList("exceptions",
+                                           &exception_list_value)) {
+            for (size_t i = 0; i < exception_list_value->GetSize(); ++i) {
+              std::string exception_feature;
+              if (exception_list_value->GetString(i, &exception_feature)) {
+                feature_exception_list.push_back(exception_feature);
+              } else {
+                LOG(WARNING) << "Malformed feature entry " << entry->id();
+                return NULL;
+              }
+            }
+          } else {
+            LOG(WARNING) << "Malformed feature entry " << entry->id();
+            return NULL;
+          }
         } else {
           LOG(WARNING) << "Malformed feature entry " << entry->id();
           return NULL;
         }
       }
-      if (!entry->SetFeatures(
-              feature_list, feature_map, supports_feature_type_all)) {
+      if (!entry->SetFeatures(feature_list, feature_exception_list, feature_map,
+                              supports_feature_type_all)) {
         LOG(WARNING) << "Malformed feature entry " << entry->id();
         return NULL;
       }
@@ -788,13 +824,13 @@ GpuControlList::GpuControlListEntry::GpuControlListEntry()
       disabled_(false),
       vendor_id_(0),
       multi_gpu_style_(kMultiGpuStyleNone),
-      multi_gpu_category_(kMultiGpuCategoryPrimary),
+      multi_gpu_category_(kMultiGpuCategoryActive),
       gl_type_(kGLTypeNone) {
 }
 
 GpuControlList::GpuControlListEntry::~GpuControlListEntry() { }
 
-bool GpuControlList::GpuControlListEntry::SetId(uint32 id) {
+bool GpuControlList::GpuControlListEntry::SetId(uint32_t id) {
   if (id != 0) {
     id_ = id;
     return true;
@@ -824,7 +860,7 @@ bool GpuControlList::GpuControlListEntry::SetVendorId(
 
 bool GpuControlList::GpuControlListEntry::AddDeviceId(
     const std::string& device_id_string) {
-  uint32 device_id = 0;
+  uint32_t device_id = 0;
   if (base::HexStringToUInt(device_id_string, &device_id) && device_id != 0) {
     device_id_list_.push_back(device_id);
     return true;
@@ -980,8 +1016,13 @@ void GpuControlList::GpuControlListEntry::SetDirectRenderingInfo(bool value) {
   direct_rendering_info_.reset(new BoolInfo(value));
 }
 
+void GpuControlList::GpuControlListEntry::SetInProcessGPUInfo(bool value) {
+  in_process_gpu_info_.reset(new BoolInfo(value));
+}
+
 bool GpuControlList::GpuControlListEntry::SetFeatures(
     const std::vector<std::string>& feature_strings,
+    const std::vector<std::string>& exception_strings,
     const FeatureMap& feature_map,
     bool supports_feature_type_all) {
   size_t size = feature_strings.size();
@@ -992,15 +1033,20 @@ bool GpuControlList::GpuControlListEntry::SetFeatures(
     int feature = 0;
     if (supports_feature_type_all && feature_strings[i] == "all") {
       for (FeatureMap::const_iterator iter = feature_map.begin();
-           iter != feature_map.end(); ++iter)
-        features_.insert(iter->second);
+           iter != feature_map.end(); ++iter) {
+        if (std::find(exception_strings.begin(), exception_strings.end(),
+                      iter->first) == exception_strings.end())
+          features_.insert(iter->second);
+      }
       continue;
     }
     if (!StringToFeature(feature_strings[i], &feature, feature_map)) {
       features_.clear();
       return false;
     }
-    features_.insert(feature);
+    if (std::find(exception_strings.begin(), exception_strings.end(),
+                  feature_strings[i]) == exception_strings.end())
+      features_.insert(feature);
   }
   return true;
 }
@@ -1018,8 +1064,8 @@ bool GpuControlList::GpuControlListEntry::GLVersionInfoMismatch(
   if (gl_version_info_.get() == NULL && gl_type_ == kGLTypeNone)
     return false;
 
-  std::vector<std::string> segments;
-  base::SplitString(gl_version, ' ', &segments);
+  std::vector<std::string> segments = base::SplitString(
+      gl_version, " ", base::TRIM_WHITESPACE, base::SPLIT_WANT_ALL);
   std::string number;
   GLType gl_type = kGLTypeNone;
   if (segments.size() > 2 &&
@@ -1029,7 +1075,8 @@ bool GpuControlList::GpuControlListEntry::GLVersionInfoMismatch(
 
     gl_type = kGLTypeGLES;
     if (segments.size() > 3 &&
-        base::StartsWithASCII(segments[3], "(ANGLE", false)) {
+        base::StartsWith(segments[3], "(ANGLE",
+                         base::CompareCase::INSENSITIVE_ASCII)) {
       gl_type = kGLTypeANGLE;
     }
   } else {
@@ -1133,12 +1180,14 @@ bool GpuControlList::GpuControlListEntry::Contains(
         candidates.push_back(gpu_info.gpu);
         break;
       case kMultiGpuCategoryActive:
-        if (gpu_info.gpu.active)
+        if (gpu_info.gpu.active || gpu_info.secondary_gpus.empty())
           candidates.push_back(gpu_info.gpu);
         for (size_t ii = 0; ii < gpu_info.secondary_gpus.size(); ++ii) {
           if (gpu_info.secondary_gpus[ii].active)
             candidates.push_back(gpu_info.secondary_gpus[ii]);
         }
+        if (candidates.empty())
+          candidates.push_back(gpu_info.gpu);
       default:
         break;
     }
@@ -1242,6 +1291,9 @@ bool GpuControlList::GpuControlListEntry::Contains(
   if (direct_rendering_info_.get() != NULL &&
       !direct_rendering_info_->Contains(gpu_info.direct_rendering))
     return false;
+  if (in_process_gpu_info_.get() != NULL &&
+      !in_process_gpu_info_->Contains(gpu_info.in_process_gpu))
+    return false;
   if (!cpu_brand_.empty()) {
     base::CPU cpu_info;
     if (StringMismatch(cpu_info.cpu_brand(), cpu_brand_))
@@ -1288,7 +1340,7 @@ GpuControlList::OsType GpuControlList::GpuControlListEntry::GetOsType() const {
   return os_info_->type();
 }
 
-uint32 GpuControlList::GpuControlListEntry::id() const {
+uint32_t GpuControlList::GpuControlListEntry::id() const {
   return id_;
 }
 
@@ -1342,7 +1394,7 @@ GpuControlList::~GpuControlList() {
 bool GpuControlList::LoadList(
     const std::string& json_context,
     GpuControlList::OsFilter os_filter) {
-  scoped_ptr<base::Value> root = base::JSONReader::Read(json_context);
+  std::unique_ptr<base::Value> root = base::JSONReader::Read(json_context);
   if (root.get() == NULL || !root->IsType(base::Value::TYPE_DICTIONARY))
     return false;
 
@@ -1365,7 +1417,7 @@ bool GpuControlList::LoadList(const base::DictionaryValue& parsed_json,
   if (!parsed_json.GetList("entries", &list))
     return false;
 
-  uint32 max_entry_id = 0;
+  uint32_t max_entry_id = 0;
   for (size_t i = 0; i < list->GetSize(); ++i) {
     const base::DictionaryValue* list_item = NULL;
     bool valid = list->GetDictionary(i, &list_item);
@@ -1448,8 +1500,8 @@ std::set<int> GpuControlList::MakeDecision(
   return features;
 }
 
-void GpuControlList::GetDecisionEntries(
-    std::vector<uint32>* entry_ids, bool disabled) const {
+void GpuControlList::GetDecisionEntries(std::vector<uint32_t>* entry_ids,
+                                        bool disabled) const {
   DCHECK(entry_ids);
   entry_ids->clear();
   for (size_t i = 0; i < active_entries_.size(); ++i) {
@@ -1480,18 +1532,18 @@ void GpuControlList::GetReasons(base::ListValue* problem_list,
     GpuControlListEntry* entry = active_entries_[i].get();
     if (entry->disabled())
       continue;
-    base::DictionaryValue* problem = new base::DictionaryValue();
+    std::unique_ptr<base::DictionaryValue> problem(new base::DictionaryValue());
 
     problem->SetString("description", entry->description());
 
     base::ListValue* cr_bugs = new base::ListValue();
     for (size_t j = 0; j < entry->cr_bugs().size(); ++j)
-      cr_bugs->Append(new base::FundamentalValue(entry->cr_bugs()[j]));
+      cr_bugs->AppendInteger(entry->cr_bugs()[j]);
     problem->Set("crBugs", cr_bugs);
 
     base::ListValue* webkit_bugs = new base::ListValue();
     for (size_t j = 0; j < entry->webkit_bugs().size(); ++j) {
-      webkit_bugs->Append(new base::FundamentalValue(entry->webkit_bugs()[j]));
+      webkit_bugs->AppendInteger(entry->webkit_bugs()[j]);
     }
     problem->Set("webkitBugs", webkit_bugs);
 
@@ -1502,7 +1554,7 @@ void GpuControlList::GetReasons(base::ListValue* problem_list,
     DCHECK(tag == "workarounds" || tag == "disabledFeatures");
     problem->SetString("tag", tag);
 
-    problem_list->Append(problem);
+    problem_list->Append(std::move(problem));
   }
 }
 
@@ -1510,7 +1562,7 @@ size_t GpuControlList::num_entries() const {
   return entries_.size();
 }
 
-uint32 GpuControlList::max_entry_id() const {
+uint32_t GpuControlList::max_entry_id() const {
   return max_entry_id_;
 }
 
