@@ -5,10 +5,10 @@
 #ifndef IPC_IPC_CHANNEL_FACTORY_H_
 #define IPC_IPC_CHANNEL_FACTORY_H_
 
+#include <memory>
 #include <string>
 #include <vector>
 
-#include "base/memory/scoped_ptr.h"
 #include "ipc/ipc_channel.h"
 
 namespace IPC {
@@ -20,12 +20,12 @@ class IPC_EXPORT ChannelFactory {
  public:
   // Creates a factory for "native" channel built through
   // IPC::Channel::Create().
-  static scoped_ptr<ChannelFactory> Create(const ChannelHandle& handle,
-                                           Channel::Mode mode);
+  static std::unique_ptr<ChannelFactory> Create(const ChannelHandle& handle,
+                                                Channel::Mode mode);
 
   virtual ~ChannelFactory() { }
   virtual std::string GetName() const = 0;
-  virtual scoped_ptr<Channel> BuildChannel(Listener* listener) = 0;
+  virtual std::unique_ptr<Channel> BuildChannel(Listener* listener) = 0;
 };
 
 }  // namespace IPC
