@@ -10,9 +10,11 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <string>
 
 #include "base/pickle.h"
+#include "base/run_loop.h"
 #include "base/strings/string16.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/threading/thread.h"
@@ -24,12 +26,7 @@ namespace {
 class IPCChannelTest : public IPCTestBase {
 };
 
-#if defined(OS_ANDROID)
-#define MAYBE_ChannelTest DISABLED_ChannelTest
-#else
-#define MAYBE_ChannelTest ChannelTest
-#endif
-TEST_F(IPCChannelTest, MAYBE_ChannelTest) {
+TEST_F(IPCChannelTest, ChannelTest) {
   Init("GenericClient");
 
   // Set up IPC channel and start client.
@@ -42,7 +39,7 @@ TEST_F(IPCChannelTest, MAYBE_ChannelTest) {
   IPC::TestChannelListener::SendOneMessage(sender(), "hello from parent");
 
   // Run message loop.
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 
   // Close the channel so the client's OnChannelError() gets fired.
   channel()->Close();
@@ -91,12 +88,7 @@ TEST_F(IPCChannelTest, ChannelTestExistingPipe) {
 }
 #endif  // defined (OS_WIN)
 
-#if defined(OS_ANDROID)
-#define MAYBE_ChannelProxyTest DISABLED_ChannelProxyTest
-#else
-#define MAYBE_ChannelProxyTest ChannelProxyTest
-#endif
-TEST_F(IPCChannelTest, MAYBE_ChannelProxyTest) {
+TEST_F(IPCChannelTest, ChannelProxyTest) {
   Init("GenericClient");
 
   base::Thread thread("ChannelProxyTestServer");
@@ -114,7 +106,7 @@ TEST_F(IPCChannelTest, MAYBE_ChannelProxyTest) {
   IPC::TestChannelListener::SendOneMessage(sender(), "hello from parent");
 
   // Run message loop.
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 
   EXPECT_TRUE(WaitForClientShutdown());
 
@@ -133,7 +125,7 @@ class ChannelListenerWithOnConnectedSend : public IPC::TestChannelListener {
   }
 };
 
-#if defined(OS_WIN) || defined(OS_ANDROID)
+#if defined(OS_WIN)
 // Acting flakey in Windows. http://crbug.com/129595
 #define MAYBE_SendMessageInChannelConnected DISABLED_SendMessageInChannelConnected
 #else
@@ -154,7 +146,7 @@ TEST_F(IPCChannelTest, MAYBE_SendMessageInChannelConnected) {
   IPC::TestChannelListener::SendOneMessage(sender(), "hello from parent");
 
   // Run message loop.
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 
   // Close the channel so the client's OnChannelError() gets fired.
   channel()->Close();
@@ -168,13 +160,13 @@ MULTIPROCESS_IPC_TEST_CLIENT_MAIN(GenericClient) {
   IPC::TestChannelListener listener;
 
   // Set up IPC channel.
-  scoped_ptr<IPC::Channel> channel(IPC::Channel::CreateClient(
+  std::unique_ptr<IPC::Channel> channel(IPC::Channel::CreateClient(
       IPCTestBase::GetChannelName("GenericClient"), &listener));
   CHECK(channel->Connect());
   listener.Init(channel.get());
   IPC::TestChannelListener::SendOneMessage(channel.get(), "hello from child");
 
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
   return 0;
 }
 
