@@ -8,12 +8,14 @@
 #include <stdint.h>
 
 #include <algorithm>
+#include <memory>
 #include <string>
 
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/pickle.h"
+#include "base/run_loop.h"
 #include "base/strings/stringprintf.h"
 #include "base/test/perf_time_logger.h"
 #include "base/test/test_io_thread.h"
@@ -224,9 +226,12 @@ class PerformanceChannelListener : public Listener {
   int count_down_;
   std::string payload_;
   EventTimeTracker latency_tracker_;
-  scoped_ptr<base::PerfTimeLogger> perf_logger_;
+  std::unique_ptr<base::PerfTimeLogger> perf_logger_;
 };
 
+IPCChannelPerfTestBase::IPCChannelPerfTestBase() = default;
+IPCChannelPerfTestBase::~IPCChannelPerfTestBase() = default;
+
 std::vector<PingPongTestParams>
 IPCChannelPerfTestBase::GetDefaultTestParams() {
   // Test several sizes. We use 12^N for message size, and limit the message
@@ -265,7 +270,7 @@ void IPCChannelPerfTestBase::RunTestChannelPingPong(
     sender()->Send(message);
 
     // Run message loop.
-    base::MessageLoop::current()->Run();
+    base::RunLoop().Run();
   }
 
   // Send quit message.
@@ -281,14 +286,13 @@ void IPCChannelPerfTestBase::RunTestChannelPingPong(
 
 void IPCChannelPerfTestBase::RunTestChannelProxyPingPong(
     const std::vector<PingPongTestParams>& params) {
+  io_thread_.reset(new base::TestIOThread(base::TestIOThread::kAutoStart));
   InitWithCustomMessageLoop("PerformanceClient",
-                            make_scoped_ptr(new base::MessageLoop()));
-
-  base::TestIOThread io_thread(base::TestIOThread::kAutoStart);
+                            base::WrapUnique(new base::MessageLoop()));
 
   // Set up IPC channel and start client.
   PerformanceChannelListener listener("ChannelProxy");
-  CreateChannelProxy(&listener, io_thread.task_runner());
+  CreateChannelProxy(&listener, io_thread_->task_runner());
   listener.Init(channel_proxy());
   ASSERT_TRUE(StartClient());
 
@@ -306,7 +310,7 @@ void IPCChannelPerfTestBase::RunTestChannelProxyPingPong(
     sender()->Send(message);
 
     // Run message loop.
-    base::MessageLoop::current()->Run();
+    base::RunLoop().Run();
   }
 
   // Send quit message.
@@ -318,6 +322,8 @@ void IPCChannelPerfTestBase::RunTestChannelProxyPingPong(
 
   EXPECT_TRUE(WaitForClientShutdown());
   DestroyChannelProxy();
+
+  io_thread_.reset();
 }
 
 
@@ -328,19 +334,18 @@ PingPongTestClient::PingPongTestClient()
 PingPongTestClient::~PingPongTestClient() {
 }
 
-scoped_ptr<Channel> PingPongTestClient::CreateChannel(
-    Listener* listener) {
+std::unique_ptr<Channel> PingPongTestClient::CreateChannel(Listener* listener) {
   return Channel::CreateClient(IPCTestBase::GetChannelName("PerformanceClient"),
                                listener);
 }
 
 int PingPongTestClient::RunMain() {
   LockThreadAffinity thread_locker(kSharedCore);
-  scoped_ptr<Channel> channel = CreateChannel(listener_.get());
+  std::unique_ptr<Channel> channel = CreateChannel(listener_.get());
   listener_->Init(channel.get());
   CHECK(channel->Connect());
 
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
   return 0;
 }
 
