@@ -5,10 +5,11 @@
 #include "base/bind.h"
 #include "base/macros.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/test/test_timeouts.h"
+#include "media/audio/audio_device_description.h"
 #include "media/audio/audio_input_controller.h"
-#include "media/audio/audio_manager_base.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
@@ -42,7 +43,7 @@ ACTION_P3(CheckCountAndPostQuitTask, count, limit, loop_or_proxy) {
 // Closes AudioOutputController synchronously.
 static void CloseAudioController(AudioInputController* controller) {
   controller->Close(base::MessageLoop::QuitWhenIdleClosure());
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 }
 
 class MockAudioInputControllerEventHandler
@@ -67,11 +68,20 @@ class MockAudioInputControllerEventHandler
 // Test fixture.
 class AudioInputControllerTest : public testing::Test {
  public:
-  AudioInputControllerTest() {}
-  ~AudioInputControllerTest() override {}
+  AudioInputControllerTest()
+      : audio_manager_(
+            AudioManager::CreateForTesting(message_loop_.task_runner())) {
+    // Flush the message loop to ensure that AudioManager is fully initialized.
+    base::RunLoop().RunUntilIdle();
+  }
+  ~AudioInputControllerTest() override {
+    audio_manager_.reset();
+    base::RunLoop().RunUntilIdle();
+  }
 
  protected:
   base::MessageLoop message_loop_;
+  ScopedAudioManagerPtr audio_manager_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(AudioInputControllerTest);
@@ -85,20 +95,16 @@ TEST_F(AudioInputControllerTest, CreateAndClose) {
   EXPECT_CALL(event_handler, OnCreated(NotNull()))
       .WillOnce(QuitMessageLoop(&message_loop_));
 
-  scoped_ptr<AudioManager> audio_manager(AudioManager::CreateForTesting());
   AudioParameters params(AudioParameters::AUDIO_FAKE, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
 
-  scoped_refptr<AudioInputController> controller =
-      AudioInputController::Create(audio_manager.get(),
-                                   &event_handler,
-                                   params,
-                                   AudioManagerBase::kDefaultDeviceId,
-                                   NULL);
+  scoped_refptr<AudioInputController> controller = AudioInputController::Create(
+      audio_manager_.get(), &event_handler, params,
+      AudioDeviceDescription::kDefaultDeviceId, NULL);
   ASSERT_TRUE(controller.get());
 
   // Wait for OnCreated() to fire.
-  message_loop_.Run();
+  base::RunLoop().Run();
 
   // Close the AudioInputController synchronously.
   CloseAudioController(controller.get());
@@ -123,24 +129,20 @@ TEST_F(AudioInputControllerTest, RecordAndClose) {
       .WillRepeatedly(CheckCountAndPostQuitTask(
           &count, 10, message_loop_.task_runner()));
 
-  scoped_ptr<AudioManager> audio_manager(AudioManager::CreateForTesting());
   AudioParameters params(AudioParameters::AUDIO_FAKE, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
 
   // Creating the AudioInputController should render an OnCreated() call.
-  scoped_refptr<AudioInputController> controller =
-      AudioInputController::Create(audio_manager.get(),
-                                   &event_handler,
-                                   params,
-                                   AudioManagerBase::kDefaultDeviceId,
-                                   NULL);
+  scoped_refptr<AudioInputController> controller = AudioInputController::Create(
+      audio_manager_.get(), &event_handler, params,
+      AudioDeviceDescription::kDefaultDeviceId, NULL);
   ASSERT_TRUE(controller.get());
 
   // Start recording and trigger one OnRecording() call.
   controller->Record();
 
   // Record and wait until ten OnData() callbacks are received.
-  message_loop_.Run();
+  base::RunLoop().Run();
 
   // Close the AudioInputController synchronously.
   CloseAudioController(controller.get());
@@ -176,29 +178,25 @@ TEST_F(AudioInputControllerTest, DISABLED_RecordAndError) {
       .Times(Exactly(1))
       .WillOnce(QuitMessageLoop(&message_loop_));
 
-  scoped_ptr<AudioManager> audio_manager(AudioManager::CreateForTesting());
   AudioParameters params(AudioParameters::AUDIO_FAKE, kChannelLayout,
                          kSampleRate, kBitsPerSample, kSamplesPerPacket);
 
   // Creating the AudioInputController should render an OnCreated() call.
-  scoped_refptr<AudioInputController> controller =
-      AudioInputController::Create(audio_manager.get(),
-                                   &event_handler,
-                                   params,
-                                   AudioManagerBase::kDefaultDeviceId,
-                                   NULL);
+  scoped_refptr<AudioInputController> controller = AudioInputController::Create(
+      audio_manager_.get(), &event_handler, params,
+      AudioDeviceDescription::kDefaultDeviceId, NULL);
   ASSERT_TRUE(controller.get());
 
   // Start recording and trigger one OnRecording() call.
   controller->Record();
 
   // Record and wait until ten OnData() callbacks are received.
-  message_loop_.Run();
+  base::RunLoop().Run();
 
   // Stop the stream and verify that OnError() is posted.
   AudioInputStream* stream = controller->stream_for_testing();
   stream->Stop();
-  message_loop_.Run();
+  base::RunLoop().Run();
 
   // Close the AudioInputController synchronously.
   CloseAudioController(controller.get());
@@ -213,18 +211,14 @@ TEST_F(AudioInputControllerTest, SamplesPerPacketTooLarge) {
   EXPECT_CALL(event_handler, OnCreated(NotNull()))
     .Times(Exactly(0));
 
-  scoped_ptr<AudioManager> audio_manager(AudioManager::CreateForTesting());
   AudioParameters params(AudioParameters::AUDIO_FAKE,
                          kChannelLayout,
                          kSampleRate,
                          kBitsPerSample,
                          kSamplesPerPacket * 1000);
-  scoped_refptr<AudioInputController> controller =
-      AudioInputController::Create(audio_manager.get(),
-                                   &event_handler,
-                                   params,
-                                   AudioManagerBase::kDefaultDeviceId,
-                                   NULL);
+  scoped_refptr<AudioInputController> controller = AudioInputController::Create(
+      audio_manager_.get(), &event_handler, params,
+      AudioDeviceDescription::kDefaultDeviceId, NULL);
   ASSERT_FALSE(controller.get());
 }
 
@@ -239,27 +233,23 @@ TEST_F(AudioInputControllerTest, CloseTwice) {
   EXPECT_CALL(event_handler, OnRecording(NotNull()))
       .Times(Exactly(1));
 
-  scoped_ptr<AudioManager> audio_manager(AudioManager::CreateForTesting());
   AudioParameters params(AudioParameters::AUDIO_FAKE,
                          kChannelLayout,
                          kSampleRate,
                          kBitsPerSample,
                          kSamplesPerPacket);
-  scoped_refptr<AudioInputController> controller =
-      AudioInputController::Create(audio_manager.get(),
-                                   &event_handler,
-                                   params,
-                                   AudioManagerBase::kDefaultDeviceId,
-                                   NULL);
+  scoped_refptr<AudioInputController> controller = AudioInputController::Create(
+      audio_manager_.get(), &event_handler, params,
+      AudioDeviceDescription::kDefaultDeviceId, NULL);
   ASSERT_TRUE(controller.get());
 
   controller->Record();
 
   controller->Close(base::MessageLoop::QuitWhenIdleClosure());
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 
   controller->Close(base::MessageLoop::QuitWhenIdleClosure());
-  base::MessageLoop::current()->Run();
+  base::RunLoop().Run();
 }
 
 }  // namespace media
