@@ -10,6 +10,7 @@
 #include "base/android/jni_android.h"
 #include "base/android/jni_string.h"
 #include "base/logging.h"
+#include "base/metrics/histogram_macros.h"
 #include "base/strings/string_util.h"
 #include "jni/MediaPlayerBridge_jni.h"
 #include "media/base/android/media_common_android.h"
@@ -23,6 +24,16 @@ using base::android::ScopedJavaLocalRef;
 
 namespace media {
 
+namespace {
+
+enum UMAExitStatus {
+  UMA_EXIT_SUCCESS = 0,
+  UMA_EXIT_ERROR,
+  UMA_EXIT_STATUS_MAX = UMA_EXIT_ERROR,
+};
+
+}  // namespace
+
 MediaPlayerBridge::MediaPlayerBridge(
     int player_id,
     const GURL& url,
@@ -32,11 +43,13 @@ MediaPlayerBridge::MediaPlayerBridge(
     MediaPlayerManager* manager,
     const OnDecoderResourcesReleasedCB& on_decoder_resources_released_cb,
     const GURL& frame_url,
-    bool allow_credentials)
+    bool allow_credentials,
+    int media_session_id)
     : MediaPlayerAndroid(player_id,
                          manager,
                          on_decoder_resources_released_cb,
-                         frame_url),
+                         frame_url,
+                         media_session_id),
       prepared_(false),
       pending_play_(false),
       should_seek_on_prepare_(false),
@@ -49,10 +62,11 @@ MediaPlayerBridge::MediaPlayerBridge(
       can_pause_(true),
       can_seek_forward_(true),
       can_seek_backward_(true),
-      volume_(-1.0),
       allow_credentials_(allow_credentials),
-      weak_factory_(this) {
-}
+      is_active_(false),
+      has_error_(false),
+      has_ever_started_(false),
+      weak_factory_(this) {}
 
 MediaPlayerBridge::~MediaPlayerBridge() {
   if (!j_media_player_bridge_.is_null()) {
@@ -61,6 +75,12 @@ MediaPlayerBridge::~MediaPlayerBridge() {
     Java_MediaPlayerBridge_destroy(env, j_media_player_bridge_.obj());
   }
   Release();
+
+  if (has_ever_started_) {
+    UMA_HISTOGRAM_ENUMERATION("Media.Android.MediaPlayerSuccess",
+                              has_error_ ? UMA_EXIT_ERROR : UMA_EXIT_SUCCESS,
+                              UMA_EXIT_STATUS_MAX + 1);
+  }
 }
 
 void MediaPlayerBridge::Initialize() {
@@ -100,8 +120,7 @@ void MediaPlayerBridge::CreateJavaMediaPlayerBridge() {
   j_media_player_bridge_.Reset(Java_MediaPlayerBridge_create(
       env, reinterpret_cast<intptr_t>(this)));
 
-  if (volume_ >= 0)
-    SetVolume(volume_);
+  UpdateEffectiveVolume();
 
   AttachListener(j_media_player_bridge_.obj());
 }
@@ -110,7 +129,7 @@ void MediaPlayerBridge::SetDuration(base::TimeDelta duration) {
   duration_ = duration;
 }
 
-void MediaPlayerBridge::SetVideoSurface(gfx::ScopedJavaSurface surface) {
+void MediaPlayerBridge::SetVideoSurface(gl::ScopedJavaSurface surface) {
   surface_ = std::move(surface);
 
   if (j_media_player_bridge_.is_null())
@@ -277,6 +296,17 @@ void MediaPlayerBridge::OnMediaMetadataExtracted(
 }
 
 void MediaPlayerBridge::Start() {
+  // A second Start() call after an error is considered another attempt for UMA
+  // and causes UMA reporting.
+  if (has_ever_started_ && has_error_) {
+    UMA_HISTOGRAM_ENUMERATION("Media.Android.MediaPlayerSuccess",
+                              UMA_EXIT_ERROR, UMA_EXIT_STATUS_MAX + 1);
+  }
+
+  has_ever_started_ = true;
+  has_error_ = false;
+  is_active_ = true;
+
   if (j_media_player_bridge_.is_null()) {
     pending_play_ = true;
     Prepare();
@@ -297,6 +327,8 @@ void MediaPlayerBridge::Pause(bool is_media_related_action) {
     else
       pending_play_ = false;
   }
+
+  is_active_ = false;
 }
 
 bool MediaPlayerBridge::IsPlaying() {
@@ -367,6 +399,8 @@ base::TimeDelta MediaPlayerBridge::GetDuration() {
 }
 
 void MediaPlayerBridge::Release() {
+  is_active_ = false;
+
   on_decoder_resources_released_cb_.Run(player_id());
   if (j_media_player_bridge_.is_null())
     return;
@@ -379,24 +413,23 @@ void MediaPlayerBridge::Release() {
 
   prepared_ = false;
   pending_play_ = false;
-  SetVideoSurface(gfx::ScopedJavaSurface());
+  SetVideoSurface(gl::ScopedJavaSurface());
   JNIEnv* env = base::android::AttachCurrentThread();
   Java_MediaPlayerBridge_release(env, j_media_player_bridge_.obj());
   j_media_player_bridge_.Reset();
   DetachListener();
 }
 
-void MediaPlayerBridge::SetVolume(double volume) {
+void MediaPlayerBridge::UpdateEffectiveVolumeInternal(double effective_volume) {
   if (j_media_player_bridge_.is_null()) {
-    volume_ = volume;
     return;
   }
 
   JNIEnv* env = base::android::AttachCurrentThread();
   CHECK(env);
 
-  Java_MediaPlayerBridge_setVolume(
-      env, j_media_player_bridge_.obj(), volume);
+  Java_MediaPlayerBridge_setVolume(env, j_media_player_bridge_.obj(),
+                                   effective_volume);
 }
 
 void MediaPlayerBridge::OnVideoSizeChanged(int width, int height) {
@@ -405,6 +438,22 @@ void MediaPlayerBridge::OnVideoSizeChanged(int width, int height) {
   MediaPlayerAndroid::OnVideoSizeChanged(width, height);
 }
 
+void MediaPlayerBridge::OnMediaError(int error_type) {
+  // Gather errors for UMA only in the active state.
+  // The MEDIA_ERROR_INVALID_CODE is reported by MediaPlayerListener.java in
+  // the situations that are considered normal, and is ignored by upper level.
+  if (is_active_ && error_type != MEDIA_ERROR_INVALID_CODE)
+    has_error_ = true;
+
+  // Do not propagate MEDIA_ERROR_SERVER_DIED. If it happens in the active state
+  // we want the playback to stall. It can be recovered by pressing the Play
+  // button again.
+  if (error_type == MEDIA_ERROR_SERVER_DIED)
+    error_type = MEDIA_ERROR_INVALID_CODE;
+
+  MediaPlayerAndroid::OnMediaError(error_type);
+}
+
 void MediaPlayerBridge::OnPlaybackComplete() {
   time_update_timer_.Stop();
   MediaPlayerAndroid::OnPlaybackComplete();
