@@ -15,9 +15,12 @@
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/callback_helpers.h"
+#include "base/location.h"
 #include "base/logging.h"
 #include "base/macros.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/trace_event/trace_event.h"
 #include "media/base/android/audio_decoder_job.h"
 #include "media/base/android/media_player_manager.h"
@@ -31,12 +34,14 @@ MediaSourcePlayer::MediaSourcePlayer(
     int player_id,
     MediaPlayerManager* manager,
     const OnDecoderResourcesReleasedCB& on_decoder_resources_released_cb,
-    scoped_ptr<DemuxerAndroid> demuxer,
-    const GURL& frame_url)
+    std::unique_ptr<DemuxerAndroid> demuxer,
+    const GURL& frame_url,
+    int media_session_id)
     : MediaPlayerAndroid(player_id,
                          manager,
                          on_decoder_resources_released_cb,
-                         frame_url),
+                         frame_url,
+                         media_session_id),
       demuxer_(std::move(demuxer)),
       pending_event_(NO_EVENT_PENDING),
       playing_(false),
@@ -74,13 +79,17 @@ MediaSourcePlayer::~MediaSourcePlayer() {
   Release();
   DCHECK_EQ(!cdm_, !cdm_registration_id_);
   if (cdm_) {
+    // Cancel previously registered callback (if any).
+    static_cast<MediaDrmBridge*>(cdm_.get())
+        ->SetMediaCryptoReadyCB(MediaDrmBridge::MediaCryptoReadyCB());
+
     static_cast<MediaDrmBridge*>(cdm_.get())
         ->UnregisterPlayer(cdm_registration_id_);
     cdm_registration_id_ = 0;
   }
 }
 
-void MediaSourcePlayer::SetVideoSurface(gfx::ScopedJavaSurface surface) {
+void MediaSourcePlayer::SetVideoSurface(gl::ScopedJavaSurface surface) {
   DVLOG(1) << __FUNCTION__;
   if (!video_decoder_job_->SetVideoSurface(std::move(surface)))
     return;
@@ -207,8 +216,8 @@ void MediaSourcePlayer::Release() {
   on_decoder_resources_released_cb_.Run(player_id());
 }
 
-void MediaSourcePlayer::SetVolume(double volume) {
-  audio_decoder_job_->SetVolume(volume);
+void MediaSourcePlayer::UpdateEffectiveVolumeInternal(double effective_volume) {
+  audio_decoder_job_->SetVolume(effective_volume);
 }
 
 bool MediaSourcePlayer::CanPause() {
@@ -728,7 +737,7 @@ void MediaSourcePlayer::StartStarvationCallback(
 
   decoder_starvation_callback_.Reset(
       base::Bind(&MediaSourcePlayer::OnDecoderStarved, weak_this_));
-  base::MessageLoop::current()->PostDelayedTask(
+  base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
       FROM_HERE, decoder_starvation_callback_.callback(), timeout);
 }
 
