@@ -5,12 +5,13 @@
 #include "media/base/android/media_source_player.h"
 
 #include <stdint.h>
+
 #include <string>
 #include <utility>
 
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/strings/stringprintf.h"
 #include "build/build_config.h"
 #include "media/base/android/audio_decoder_job.h"
@@ -162,11 +163,13 @@ class MediaSourcePlayerTest : public testing::Test {
   MediaSourcePlayerTest()
       : manager_(&message_loop_),
         demuxer_(new MockDemuxerAndroid(&message_loop_)),
-        player_(0, &manager_,
+        player_(0,
+                &manager_,
                 base::Bind(&MockMediaPlayerManager::OnDecorderResourcesReleased,
                            base::Unretained(&manager_)),
-                scoped_ptr<DemuxerAndroid>(demuxer_),
-                GURL()),
+                base::WrapUnique(demuxer_),
+                GURL(),
+                kDefaultMediaSessionId),
         decoder_callback_hook_executed_(false),
         surface_texture_a_is_next_(true) {}
 
@@ -702,17 +705,17 @@ class MediaSourcePlayerTest : public testing::Test {
   }
 
   void CreateNextTextureAndSetVideoSurface() {
-    gfx::SurfaceTexture* surface_texture;
+    gl::SurfaceTexture* surface_texture;
     if (surface_texture_a_is_next_) {
-      surface_texture_a_ = gfx::SurfaceTexture::Create(next_texture_id_++);
+      surface_texture_a_ = gl::SurfaceTexture::Create(next_texture_id_++);
       surface_texture = surface_texture_a_.get();
     } else {
-      surface_texture_b_ = gfx::SurfaceTexture::Create(next_texture_id_++);
+      surface_texture_b_ = gl::SurfaceTexture::Create(next_texture_id_++);
       surface_texture = surface_texture_b_.get();
     }
 
     surface_texture_a_is_next_ = !surface_texture_a_is_next_;
-    gfx::ScopedJavaSurface surface = gfx::ScopedJavaSurface(surface_texture);
+    gl::ScopedJavaSurface surface = gl::ScopedJavaSurface(surface_texture);
     player_.SetVideoSurface(std::move(surface));
   }
 
@@ -850,8 +853,8 @@ class MediaSourcePlayerTest : public testing::Test {
   // between two surface textures, only replacing the N-2 texture. Assumption is
   // that no more than N-1 texture is in use by decoder when
   // CreateNextTextureAndSetVideoSurface() is called.
-  scoped_refptr<gfx::SurfaceTexture> surface_texture_a_;
-  scoped_refptr<gfx::SurfaceTexture> surface_texture_b_;
+  scoped_refptr<gl::SurfaceTexture> surface_texture_a_;
+  scoped_refptr<gl::SurfaceTexture> surface_texture_b_;
   bool surface_texture_a_is_next_;
   int next_texture_id_;
 
@@ -887,7 +890,8 @@ TEST_F(MediaSourcePlayerTest, StartAudioDecoderWithInvalidConfig) {
 }
 
 TEST_F(MediaSourcePlayerTest, StartVideoCodecWithValidSurface) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test video codec will not be created until data is received.
   StartVideoDecoderJob();
@@ -913,9 +917,9 @@ TEST_F(MediaSourcePlayerTest, StartVideoCodecWithInvalidSurface) {
   SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
 
   // Test video codec will not be created when surface is invalid.
-  scoped_refptr<gfx::SurfaceTexture> surface_texture(
-      gfx::SurfaceTexture::Create(0));
-  gfx::ScopedJavaSurface surface(surface_texture.get());
+  scoped_refptr<gl::SurfaceTexture> surface_texture(
+      gl::SurfaceTexture::Create(0));
+  gl::ScopedJavaSurface surface(surface_texture.get());
   StartVideoDecoderJob();
 
   // Release the surface texture.
@@ -939,7 +943,8 @@ TEST_F(MediaSourcePlayerTest, ReadFromDemuxerAfterSeek) {
 }
 
 TEST_F(MediaSourcePlayerTest, SetSurfaceWhileSeeking) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test SetVideoSurface() will not cause an extra seek while the player is
   // waiting for demuxer to indicate seek is done.
@@ -970,7 +975,8 @@ TEST_F(MediaSourcePlayerTest, SetSurfaceWhileSeeking) {
 }
 
 TEST_F(MediaSourcePlayerTest, ChangeMultipleSurfaceWhileDecoding) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test MediaSourcePlayer can switch multiple surfaces during decoding.
   CreateNextTextureAndSetVideoSurface();
@@ -982,7 +988,7 @@ TEST_F(MediaSourcePlayerTest, ChangeMultipleSurfaceWhileDecoding) {
 
   // While the decoder is decoding, change multiple surfaces. Pass an empty
   // surface first.
-  gfx::ScopedJavaSurface empty_surface;
+  gl::ScopedJavaSurface empty_surface;
   player_.SetVideoSurface(std::move(empty_surface));
   // Next, pass a new non-empty surface.
   CreateNextTextureAndSetVideoSurface();
@@ -1011,7 +1017,8 @@ TEST_F(MediaSourcePlayerTest, ChangeMultipleSurfaceWhileDecoding) {
 }
 
 TEST_F(MediaSourcePlayerTest, SetEmptySurfaceAndStarveWhileDecoding) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test player pauses if an empty surface is passed.
   CreateNextTextureAndSetVideoSurface();
@@ -1022,7 +1029,7 @@ TEST_F(MediaSourcePlayerTest, SetEmptySurfaceAndStarveWhileDecoding) {
   player_.OnDemuxerDataAvailable(CreateReadFromDemuxerAckForVideo(false));
 
   // While the decoder is decoding, pass an empty surface.
-  gfx::ScopedJavaSurface empty_surface;
+  gl::ScopedJavaSurface empty_surface;
   player_.SetVideoSurface(std::move(empty_surface));
   // Let the player starve. However, it should not issue any new data request in
   // this case.
@@ -1046,7 +1053,8 @@ TEST_F(MediaSourcePlayerTest, SetEmptySurfaceAndStarveWhileDecoding) {
 }
 
 TEST_F(MediaSourcePlayerTest, ReleaseVideoDecoderResourcesWhileDecoding) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that if video decoder is released while decoding, the resources will
   // not be immediately released.
@@ -1096,7 +1104,8 @@ TEST_F(MediaSourcePlayerTest, AudioOnlyStartAfterSeekFinish) {
 }
 
 TEST_F(MediaSourcePlayerTest, VideoOnlyStartAfterSeekFinish) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test video decoder job will not start until pending seek event is handled.
   CreateNextTextureAndSetVideoSurface();
@@ -1158,7 +1167,8 @@ TEST_F(MediaSourcePlayerTest, StartImmediatelyAfterPause) {
 }
 
 TEST_F(MediaSourcePlayerTest, DecoderJobsCannotStartWithoutAudio) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that when Start() is called, video decoder job will wait for audio
   // decoder job before start decoding the data.
@@ -1211,7 +1221,8 @@ TEST_F(MediaSourcePlayerTest, StartTimeTicksResetAfterDecoderUnderruns) {
 }
 
 TEST_F(MediaSourcePlayerTest, V_SecondAccessUnitIsEOSAndResumePlayAfterSeek) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test MediaSourcePlayer can replay video after input EOS is reached.
   CreateNextTextureAndSetVideoSurface();
@@ -1238,7 +1249,8 @@ TEST_F(MediaSourcePlayerTest, A_FirstAccessUnitIsEOSAndResumePlayAfterSeek) {
 }
 
 TEST_F(MediaSourcePlayerTest, V_FirstAccessUnitAfterSeekIsEOS) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test decode of video EOS buffer, just after seeking, without any prior
   // decode (other than the simulated |kAborted| resulting from the seek
@@ -1261,7 +1273,8 @@ TEST_F(MediaSourcePlayerTest, A_FirstAccessUnitAfterSeekIsEOS) {
 }
 
 TEST_F(MediaSourcePlayerTest, AV_PlaybackCompletionAcrossConfigChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that if one stream (audio) has completed decode of EOS and the other
   // stream (video) processes config change, that subsequent video EOS completes
@@ -1286,7 +1299,8 @@ TEST_F(MediaSourcePlayerTest, AV_PlaybackCompletionAcrossConfigChange) {
 }
 
 TEST_F(MediaSourcePlayerTest, VA_PlaybackCompletionAcrossConfigChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that if one stream (video) has completed decode of EOS and the other
   // stream (audio) processes config change, that subsequent audio EOS completes
@@ -1311,7 +1325,8 @@ TEST_F(MediaSourcePlayerTest, VA_PlaybackCompletionAcrossConfigChange) {
 }
 
 TEST_F(MediaSourcePlayerTest, AV_NoPrefetchForFinishedVideoOnAudioStarvation) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that if one stream (video) has completed decode of EOS, prefetch
   // resulting from player starvation occurs only for the other stream (audio),
@@ -1348,7 +1363,8 @@ TEST_F(MediaSourcePlayerTest, AV_NoPrefetchForFinishedVideoOnAudioStarvation) {
 }
 
 TEST_F(MediaSourcePlayerTest, V_StarvationDuringEOSDecode) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that video-only playback completes without further data requested when
   // starvation occurs during EOS decode.
@@ -1381,7 +1397,8 @@ TEST_F(MediaSourcePlayerTest, A_StarvationDuringEOSDecode) {
 }
 
 TEST_F(MediaSourcePlayerTest, AV_SeekDuringEOSDecodePreventsCompletion) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that seek supercedes audio+video playback completion on simultaneous
   // audio and video EOS decode, if SeekTo() occurs during these EOS decodes.
@@ -1389,7 +1406,8 @@ TEST_F(MediaSourcePlayerTest, AV_SeekDuringEOSDecodePreventsCompletion) {
 }
 
 TEST_F(MediaSourcePlayerTest, AV_SeekDuringAudioEOSDecodePreventsCompletion) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that seek supercedes audio+video playback completion on simultaneous
   // audio EOS and video non-EOS decode, if SeekTo() occurs during these
@@ -1398,7 +1416,8 @@ TEST_F(MediaSourcePlayerTest, AV_SeekDuringAudioEOSDecodePreventsCompletion) {
 }
 
 TEST_F(MediaSourcePlayerTest, AV_SeekDuringVideoEOSDecodePreventsCompletion) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that seek supercedes audio+video playback completion on simultaneous
   // audio non-EOS and video EOS decode, if SeekTo() occurs during these
@@ -1407,7 +1426,8 @@ TEST_F(MediaSourcePlayerTest, AV_SeekDuringVideoEOSDecodePreventsCompletion) {
 }
 
 TEST_F(MediaSourcePlayerTest, V_SeekDuringEOSDecodePreventsCompletion) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that seek supercedes video-only playback completion on EOS decode, if
   // SeekTo() occurs during EOS decode.
@@ -1462,7 +1482,8 @@ TEST_F(MediaSourcePlayerTest, DemuxerDataArrivesAfterRelease) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_RegularSeekPendsBrowserSeekDone) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that a browser seek, once started, delays a newly arrived regular
   // SeekTo() request's demuxer seek until the browser seek is done.
@@ -1492,7 +1513,8 @@ TEST_F(MediaSourcePlayerTest, BrowserSeek_RegularSeekPendsBrowserSeekDone) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_InitialReleaseAndStart) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that no browser seek is requested if player Release() + Start() occurs
   // prior to receiving any data.
@@ -1517,7 +1539,8 @@ TEST_F(MediaSourcePlayerTest, BrowserSeek_InitialReleaseAndStart) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_MidStreamReleaseAndStart) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that one browser seek is requested if player Release() + Start(), with
   // video data received between Release() and Start().
@@ -1530,7 +1553,8 @@ TEST_F(MediaSourcePlayerTest, BrowserSeek_MidStreamReleaseAndStart) {
 }
 
 TEST_F(MediaSourcePlayerTest, NoBrowserSeekWithKeyFrameInCache) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that browser seek is not needed if a key frame is found in data
   // cache.
@@ -1570,7 +1594,8 @@ TEST_F(MediaSourcePlayerTest, PrerollAudioAfterSeek) {
 }
 
 TEST_F(MediaSourcePlayerTest, PrerollVideoAfterSeek) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test decoder job will preroll the media to the seek position.
   CreateNextTextureAndSetVideoSurface();
@@ -1714,7 +1739,8 @@ TEST_F(MediaSourcePlayerTest, PrerollContinuesAfterUnchangedConfigs) {
 }
 
 TEST_F(MediaSourcePlayerTest, AudioPrerollFinishesBeforeVideo) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that after audio finishes prerolling, it will wait for video to finish
   // prerolling before advancing together.
@@ -1766,7 +1792,8 @@ TEST_F(MediaSourcePlayerTest, AudioPrerollFinishesBeforeVideo) {
 }
 
 TEST_F(MediaSourcePlayerTest, SimultaneousAudioVideoConfigChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that the player allows simultaneous audio and video config change,
   // such as might occur during OnPrefetchDone() if next access unit for both
@@ -1807,7 +1834,8 @@ TEST_F(MediaSourcePlayerTest, SimultaneousAudioVideoConfigChange) {
 
 TEST_F(MediaSourcePlayerTest,
        SimultaneousAudioVideoConfigChangeWithAdaptivePlayback) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that the player allows simultaneous audio and video config change with
   // adaptive video playback enabled.
@@ -1891,7 +1919,8 @@ TEST_F(MediaSourcePlayerTest, DemuxerConfigRequestedIfInUnit1AfterPrefetch) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_PrerollAfterBrowserSeek) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test decoder job will preroll the media to the actual seek position
   // resulting from a browser seek.
@@ -1911,7 +1940,8 @@ TEST_F(MediaSourcePlayerTest, BrowserSeek_PrerollAfterBrowserSeek) {
 }
 
 TEST_F(MediaSourcePlayerTest, VideoDemuxerConfigChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that video config change notification results in creating a new
   // video codec without any browser seek.
@@ -1926,7 +1956,8 @@ TEST_F(MediaSourcePlayerTest, VideoDemuxerConfigChange) {
 }
 
 TEST_F(MediaSourcePlayerTest, VideoDemuxerConfigChangeWithAdaptivePlayback) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that if codec supports adaptive playback, no new codec should be
   // created beyond the one used to decode the prefetch media data prior to
@@ -1979,7 +2010,8 @@ TEST_F(MediaSourcePlayerTest, DecoderDrainInterruptedByRelease) {
 }
 
 TEST_F(MediaSourcePlayerTest, DecoderDrainInterruptedBySurfaceChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test if a video decoder is being drained while surface changes, draining
   // is canceled.
@@ -2001,7 +2033,8 @@ TEST_F(MediaSourcePlayerTest, DecoderDrainInterruptedBySurfaceChange) {
 
 TEST_F(MediaSourcePlayerTest,
        BrowserSeek_DecoderStarvationWhilePendingSurfaceChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test video decoder starvation while handling a pending surface change
   // should not cause any crashes.
@@ -2210,7 +2243,8 @@ TEST_F(MediaSourcePlayerTest, ConfigChangedThenReleaseThenStart) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_ThenReleaseThenDemuxerSeekDone) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that Release() after a browser seek's DemuxerSeek IPC request has been
   // sent behaves similar to a regular seek: if OnDemuxerSeekDone() occurs
@@ -2238,7 +2272,8 @@ TEST_F(MediaSourcePlayerTest, BrowserSeek_ThenReleaseThenDemuxerSeekDone) {
 }
 
 TEST_F(MediaSourcePlayerTest, BrowserSeek_ThenReleaseThenStart) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that Release() after a browser seek's DemuxerSeek IPC request has been
   // sent behaves similar to a regular seek: if OnDemuxerSeekDone() does not
@@ -2324,7 +2359,8 @@ TEST_F(MediaSourcePlayerTest, CurrentTimeKeepsIncreasingAfterConfigChange) {
 }
 
 TEST_F(MediaSourcePlayerTest, VideoMetadataChangeAfterConfigChange) {
-  SKIP_TEST_IF_MEDIA_CODEC_BRIDGE_IS_NOT_AVAILABLE();
+  // crbug.com/604602 and crbug.com/597836
+  SKIP_TEST_IF_VP8_DECODER_IS_NOT_SUPPORTED();
 
   // Test that after a config change, metadata change will be happen
   // after decoder is drained.
