@@ -4,30 +4,68 @@
 
 #include "media/base/audio_renderer_mixer.h"
 
+#include <cmath>
+
 #include "base/bind.h"
 #include "base/bind_helpers.h"
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
+#include "base/metrics/histogram_macros.h"
 
 namespace media {
 
 enum { kPauseDelaySeconds = 10 };
 
-AudioRendererMixer::AudioRendererMixer(
-    const AudioParameters& output_params,
-    const scoped_refptr<AudioRendererSink>& sink)
-    : audio_sink_(sink),
-      output_params_(output_params),
+// Tracks the maximum value of a counter and logs it into a UMA histogram upon
+// each increase of the maximum. NOT thread-safe, make sure it is used under
+// lock.
+class AudioRendererMixer::UMAMaxValueTracker {
+ public:
+  UMAMaxValueTracker(const UmaLogCallback& log_callback)
+      : log_callback_(log_callback), count_(0), max_count_(0) {}
+
+  ~UMAMaxValueTracker() {}
+
+  // Increments the counter, updates the maximum.
+  void Increment() {
+    ++count_;
+    if (max_count_ < count_) {
+      max_count_ = count_;
+      log_callback_.Run(max_count_);
+    }
+  }
+
+  // Decrements the counter.
+  void Decrement() {
+    DCHECK_GE(count_, 0);
+    --count_;
+  }
+
+ private:
+  const UmaLogCallback log_callback_;
+  int count_;
+  int max_count_;
+  DISALLOW_COPY_AND_ASSIGN(UMAMaxValueTracker);
+};
+
+AudioRendererMixer::AudioRendererMixer(const AudioParameters& output_params,
+                                       scoped_refptr<AudioRendererSink> sink,
+                                       const UmaLogCallback& log_callback)
+    : output_params_(output_params),
+      audio_sink_(std::move(sink)),
       master_converter_(output_params, output_params, true),
       pause_delay_(base::TimeDelta::FromSeconds(kPauseDelaySeconds)),
       last_play_time_(base::TimeTicks::Now()),
       // Initialize |playing_| to true since Start() results in an auto-play.
-      playing_(true) {
+      playing_(true),
+      input_count_tracker_(new UMAMaxValueTracker(log_callback)) {
+  DCHECK(audio_sink_);
   audio_sink_->Initialize(output_params, this);
   audio_sink_->Start();
 }
 
 AudioRendererMixer::~AudioRendererMixer() {
-  // AudioRendererSinks must be stopped before being destructed.
+  // AudioRendererSink must be stopped before mixer is destructed.
   audio_sink_->Stop();
 
   // Ensure that all mixer inputs have removed themselves prior to destruction.
@@ -54,7 +92,7 @@ void AudioRendererMixer::AddMixerInput(const AudioParameters& input_params,
     if (converter == converters_.end()) {
       std::pair<AudioConvertersMap::iterator, bool> result =
           converters_.insert(std::make_pair(
-              input_sample_rate, make_scoped_ptr(
+              input_sample_rate, base::WrapUnique(
                                      // We expect all InputCallbacks to be
                                      // capable of handling arbitrary buffer
                                      // size requests, disabling FIFO.
@@ -67,6 +105,8 @@ void AudioRendererMixer::AddMixerInput(const AudioParameters& input_params,
     }
     converter->second->AddInput(input);
   }
+
+  input_count_tracker_->Increment();
 }
 
 void AudioRendererMixer::RemoveMixerInput(
@@ -88,6 +128,8 @@ void AudioRendererMixer::RemoveMixerInput(
       converters_.erase(converter);
     }
   }
+
+  input_count_tracker_->Decrement();
 }
 
 void AudioRendererMixer::AddErrorCallback(const base::Closure& error_cb) {
@@ -110,14 +152,17 @@ void AudioRendererMixer::RemoveErrorCallback(const base::Closure& error_cb) {
   NOTREACHED();
 }
 
-OutputDevice* AudioRendererMixer::GetOutputDevice() {
+OutputDeviceInfo AudioRendererMixer::GetOutputDeviceInfo() {
   DVLOG(1) << __FUNCTION__;
-  base::AutoLock auto_lock(lock_);
-  return audio_sink_->GetOutputDevice();
+  return audio_sink_->GetOutputDeviceInfo();
+}
+
+bool AudioRendererMixer::CurrentThreadIsRenderingThread() {
+  return audio_sink_->CurrentThreadIsRenderingThread();
 }
 
 int AudioRendererMixer::Render(AudioBus* audio_bus,
-                               uint32_t audio_delay_milliseconds,
+                               uint32_t frames_delayed,
                                uint32_t frames_skipped) {
   base::AutoLock auto_lock(lock_);
 
@@ -132,8 +177,7 @@ int AudioRendererMixer::Render(AudioBus* audio_bus,
     playing_ = false;
   }
 
-  master_converter_.ConvertWithDelay(
-      base::TimeDelta::FromMilliseconds(audio_delay_milliseconds), audio_bus);
+  master_converter_.ConvertWithDelay(frames_delayed, audio_bus);
   return audio_bus->frames();
 }
 
