@@ -39,7 +39,7 @@ void ByteQueue::Push(const uint8_t* data, int size) {
     // Sanity check to make sure we didn't overflow.
     CHECK_GT(new_size, size_);
 
-    scoped_ptr<uint8_t[]> new_buffer(new uint8_t[new_size]);
+    std::unique_ptr<uint8_t[]> new_buffer(new uint8_t[new_size]);
 
     // Copy the data from the old buffer to the start of the new one.
     if (used_ > 0)
