@@ -4,6 +4,7 @@
 
 #include "media/base/cdm_callback_promise.h"
 
+#include "base/callback_helpers.h"
 #include "base/logging.h"
 
 namespace media {
@@ -19,12 +20,17 @@ CdmCallbackPromise<T...>::CdmCallbackPromise(
 
 template <typename... T>
 CdmCallbackPromise<T...>::~CdmCallbackPromise() {
+  if (IsPromiseSettled())
+    return;
+
+  DCHECK(!resolve_cb_.is_null() && !reject_cb_.is_null());
+  RejectPromiseOnDestruction();
 }
 
 template <typename... T>
 void CdmCallbackPromise<T...>::resolve(const T&... result) {
   MarkPromiseSettled();
-  resolve_cb_.Run(result...);
+  base::ResetAndReturn(&resolve_cb_).Run(result...);
 }
 
 template <typename... T>
@@ -32,7 +38,8 @@ void CdmCallbackPromise<T...>::reject(MediaKeys::Exception exception_code,
                                       uint32_t system_code,
                                       const std::string& error_message) {
   MarkPromiseSettled();
-  reject_cb_.Run(exception_code, system_code, error_message);
+  base::ResetAndReturn(&reject_cb_)
+      .Run(exception_code, system_code, error_message);
 }
 
 // Explicit template instantiation for the Promises needed.
