@@ -30,13 +30,12 @@ class FakeAudioRenderCallback
   // Renders a sine wave into the provided audio data buffer.  If |half_fill_|
   // is set, will only fill half the buffer.
   int Render(AudioBus* audio_bus,
-             uint32_t audio_delay_milliseconds,
+             uint32_t frames_delayed,
              uint32_t frames_skipped) override;
   MOCK_METHOD0(OnRenderError, void());
 
   // AudioTransform::ProvideAudioTransformInput implementation.
-  double ProvideInput(AudioBus* audio_bus,
-                      base::TimeDelta buffer_delay) override;
+  double ProvideInput(AudioBus* audio_bus, uint32_t frames_delayed) override;
 
   // Toggles only filling half the requested amount during Render().
   void set_half_fill(bool half_fill) { half_fill_ = half_fill; }
@@ -44,11 +43,9 @@ class FakeAudioRenderCallback
   // Reset the sine state to initial value.
   void reset() { x_ = 0; }
 
-  // Returns the last |audio_delay_milliseconds| provided to Render() or -1 if
+  // Returns the last |frames_delayed| provided to Render() or -1 if
   // no Render() call occurred.
-  int last_audio_delay_milliseconds() const {
-    return last_audio_delay_milliseconds_;
-  }
+  int last_frames_delayed() const { return last_frames_delayed_; }
 
   // Set volume information used by ProvideAudioTransformInput().
   void set_volume(double volume) { volume_ = volume; }
@@ -59,7 +56,7 @@ class FakeAudioRenderCallback
   bool half_fill_;
   double x_;
   double step_;
-  int last_audio_delay_milliseconds_;
+  int last_frames_delayed_;
   int last_channel_count_;
   double volume_;
 
