@@ -14,6 +14,7 @@
 
 #if defined(OS_ANDROID)
 #include "base/android/jni_android.h"
+#include "media/base/android/media_codec_util.h"
 #include "media/base/android/media_jni_registrar.h"
 #include "ui/gl/android/gl_jni_registrar.h"
 #endif
@@ -43,6 +44,9 @@ void TestSuiteNoAtExit::Initialize() {
   // Needed for surface texture support.
   ui::gl::android::RegisterJni(env);
   media::RegisterJni(env);
+
+  if (media::MediaCodecUtil::IsMediaCodecAvailable())
+    media::EnablePlatformDecoderSupport();
 #endif
 
   // Run this here instead of main() to ensure an AtExitManager is already
