@@ -5,6 +5,7 @@
 #include "media/base/text_renderer.h"
 
 #include <stddef.h>
+#include <memory>
 #include <utility>
 #include <vector>
 
@@ -13,6 +14,7 @@
 #include "base/macros.h"
 #include "base/memory/scoped_vector.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "media/base/audio_decoder_config.h"
 #include "media/base/decoder_buffer.h"
 #include "media/base/demuxer_stream.h"
@@ -67,7 +69,7 @@ class TextRendererTest : public testing::Test {
 
   void Destroy() {
     text_renderer_.reset();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void AddTextTrack(TextKind kind,
@@ -82,7 +84,7 @@ class TextRendererTest : public testing::Test {
 
     const TextTrackConfig config(kind, name, language, std::string());
     text_renderer_->AddTextStream(text_track_streams_.back(), config);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
 
     EXPECT_EQ(text_tracks_.size(), text_track_streams_.size());
     FakeTextTrack* const text_track = text_tracks_.back();
@@ -100,7 +102,7 @@ class TextRendererTest : public testing::Test {
     // here so we can inspect them.  They get removed from our cache when the
     // text renderer deallocates them.
     text_tracks_.push_back(new FakeTextTrack(destroy_cb, config));
-    scoped_ptr<TextTrack> text_track(text_tracks_.back());
+    std::unique_ptr<TextTrack> text_track(text_tracks_.back());
     done_cb.Run(std::move(text_track));
   }
 
@@ -124,7 +126,7 @@ class TextRendererTest : public testing::Test {
   void AbortPendingRead(unsigned idx) {
     FakeTextTrackStream* const stream = text_track_streams_[idx];
     stream->AbortPendingRead();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void AbortPendingReads() {
@@ -136,7 +138,7 @@ class TextRendererTest : public testing::Test {
   void SendEosNotification(unsigned idx) {
     FakeTextTrackStream* const stream = text_track_streams_[idx];
     stream->SendEosNotification();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void SendEosNotifications() {
@@ -164,7 +166,7 @@ class TextRendererTest : public testing::Test {
     }
 
     text_stream->SatisfyPendingRead(start, duration, id, content, settings);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void SendCues(bool expect_cue) {
@@ -184,7 +186,7 @@ class TextRendererTest : public testing::Test {
   void Pause() {
     text_renderer_->Pause(base::Bind(&TextRendererTest::OnPause,
                                      base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void Flush() {
@@ -210,7 +212,7 @@ class TextRendererTest : public testing::Test {
   typedef std::vector<FakeTextTrack*> TextTracks;
   TextTracks text_tracks_;
 
-  scoped_ptr<TextRenderer> text_renderer_;
+  std::unique_ptr<TextRenderer> text_renderer_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(TextRendererTest);
