@@ -5,8 +5,9 @@
 #ifndef MEDIA_BLINK_ACTIVE_LOADER_H_
 #define MEDIA_BLINK_ACTIVE_LOADER_H_
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "media/blink/media_blink_export.h"
 
 namespace blink {
@@ -22,7 +23,7 @@ class MEDIA_BLINK_EXPORT ActiveLoader {
  public:
   // Creates an ActiveLoader with the given loader. It is assumed that the
   // initial state of |loader| is loading and not deferred.
-  explicit ActiveLoader(scoped_ptr<blink::WebURLLoader> loader);
+  explicit ActiveLoader(std::unique_ptr<blink::WebURLLoader> loader);
   ~ActiveLoader();
 
   // Starts or stops deferring the resource load.
@@ -33,7 +34,7 @@ class MEDIA_BLINK_EXPORT ActiveLoader {
   friend class BufferedDataSourceTest;
   friend class MultibufferDataSourceTest;
 
-  scoped_ptr<blink::WebURLLoader> loader_;
+  std::unique_ptr<blink::WebURLLoader> loader_;
   bool deferred_;
 
   DISALLOW_IMPLICIT_CONSTRUCTORS(ActiveLoader);
