@@ -2,9 +2,13 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include <stdint.h>
+
 #include "base/bind.h"
+#include "base/macros.h"
 #include "base/message_loop/message_loop.h"
 #include "base/run_loop.h"
+#include "build/build_config.h"
 #include "media/base/media_log.h"
 #include "media/base/mock_filters.h"
 #include "media/base/test_helpers.h"
@@ -38,8 +42,8 @@ class MockBufferedDataSourceHost : public BufferedDataSourceHost {
   MockBufferedDataSourceHost() {}
   virtual ~MockBufferedDataSourceHost() {}
 
-  MOCK_METHOD1(SetTotalBytes, void(int64 total_bytes));
-  MOCK_METHOD2(AddBufferedByteRange, void(int64 start, int64 end));
+  MOCK_METHOD1(SetTotalBytes, void(int64_t total_bytes));
+  MOCK_METHOD2(AddBufferedByteRange, void(int64_t start, int64_t end));
 
  private:
   DISALLOW_COPY_AND_ASSIGN(MockBufferedDataSourceHost);
@@ -51,11 +55,12 @@ class MockBufferedDataSource : public BufferedDataSource {
  public:
   MockBufferedDataSource(
       const GURL& url,
+      BufferedResourceLoader::CORSMode cors_mode,
       const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
       WebLocalFrame* frame,
       BufferedDataSourceHost* host)
       : BufferedDataSource(url,
-                           BufferedResourceLoader::kUnspecified,
+                           cors_mode,
                            task_runner,
                            frame,
                            new media::MediaLog(),
@@ -66,9 +71,9 @@ class MockBufferedDataSource : public BufferedDataSource {
         loading_(false) {}
   virtual ~MockBufferedDataSource() {}
 
-  MOCK_METHOD2(CreateResourceLoader, BufferedResourceLoader*(int64, int64));
-  BufferedResourceLoader* CreateMockResourceLoader(int64 first_byte_position,
-                                                   int64 last_byte_position) {
+  MOCK_METHOD2(CreateResourceLoader, BufferedResourceLoader*(int64_t, int64_t));
+  BufferedResourceLoader* CreateMockResourceLoader(int64_t first_byte_position,
+                                                   int64_t last_byte_position) {
     CHECK(!loading_) << "Previous resource load wasn't cancelled";
 
     BufferedResourceLoader* loader =
@@ -83,7 +88,7 @@ class MockBufferedDataSource : public BufferedDataSource {
         .WillByDefault(Assign(&loading_, false));
 
     // |test_loader_| will be used when Start() is called.
-    loader->test_loader_ = scoped_ptr<WebURLLoader>(url_loader);
+    loader->test_loader_ = std::unique_ptr<WebURLLoader>(url_loader);
     return loader;
   }
 
@@ -102,18 +107,22 @@ class MockBufferedDataSource : public BufferedDataSource {
   DISALLOW_COPY_AND_ASSIGN(MockBufferedDataSource);
 };
 
-static const int64 kFileSize = 5000000;
-static const int64 kFarReadPosition = 4000000;
+static const int64_t kFileSize = 5000000;
+static const int64_t kFarReadPosition = 4000000;
 static const int kDataSize = 1024;
 
 static const char kHttpUrl[] = "http://localhost/foo.webm";
 static const char kFileUrl[] = "file:///tmp/bar.webm";
+static const char kHttpDifferentPathUrl[] = "http://localhost/bar.webm";
+static const char kHttpDifferentOriginUrl[] = "http://127.0.0.1/foo.webm";
 
 class BufferedDataSourceTest : public testing::Test {
  public:
   BufferedDataSourceTest()
-      : view_(WebView::create(NULL)),
-        frame_(WebLocalFrame::create(&client_)),
+      : view_(WebView::create(nullptr, blink::WebPageVisibilityStateVisible)),
+        frame_(
+            WebLocalFrame::create(blink::WebTreeScopeType::Document, &client_)),
+        bytes_received_(0),
         preload_(BufferedDataSource::AUTO) {
     view_->setMainFrame(frame_);
   }
@@ -125,13 +134,13 @@ class BufferedDataSourceTest : public testing::Test {
 
   MOCK_METHOD1(OnInitialize, void(bool));
 
-  void Initialize(const char* url, bool expected) {
+  void InitializeWithCORS(const char* url,
+                          bool expected,
+                          BufferedResourceLoader::CORSMode cors_mode) {
     GURL gurl(url);
-    data_source_.reset(
-        new MockBufferedDataSource(gurl,
-                                   message_loop_.message_loop_proxy(),
-                                   view_->mainFrame()->toWebLocalFrame(),
-                                   &host_));
+    data_source_.reset(new MockBufferedDataSource(
+        gurl, cors_mode, message_loop_.task_runner(),
+        view_->mainFrame()->toWebLocalFrame(), &host_));
     data_source_->SetPreload(preload_);
 
     response_generator_.reset(new TestResponseGenerator(gurl, kFileSize));
@@ -139,12 +148,16 @@ class BufferedDataSourceTest : public testing::Test {
     EXPECT_CALL(*this, OnInitialize(expected));
     data_source_->Initialize(base::Bind(&BufferedDataSourceTest::OnInitialize,
                                         base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
 
     bool is_http = gurl.SchemeIsHTTPOrHTTPS();
     EXPECT_EQ(data_source_->downloading(), is_http);
   }
 
+  void Initialize(const char* url, bool expected) {
+    InitializeWithCORS(url, expected, BufferedResourceLoader::kUnspecified);
+  }
+
   // Helper to initialize tests with a valid 200 response.
   void InitializeWith200Response() {
     Initialize(kHttpUrl, true);
@@ -177,52 +190,96 @@ class BufferedDataSourceTest : public testing::Test {
   void Stop() {
     if (data_source_->loading()) {
       loader()->didFail(url_loader(), response_generator_->GenerateError());
-      message_loop_.RunUntilIdle();
+      base::RunLoop().RunUntilIdle();
     }
 
     data_source_->Stop();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void ExpectCreateResourceLoader() {
     EXPECT_CALL(*data_source_, CreateResourceLoader(_, _))
         .WillOnce(Invoke(data_source_.get(),
                          &MockBufferedDataSource::CreateMockResourceLoader));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
+    bytes_received_ = 0;
   }
 
   void Respond(const WebURLResponse& response) {
     loader()->didReceiveResponse(url_loader(), response);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void ReceiveData(int size) {
-    scoped_ptr<char[]> data(new char[size]);
+    std::unique_ptr<char[]> data(new char[size]);
     memset(data.get(), 0xA5, size);  // Arbitrary non-zero value.
 
     loader()->didReceiveData(url_loader(), data.get(), size, size);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
+    bytes_received_ += size;
+    EXPECT_EQ(bytes_received_, data_source_->GetMemoryUsage());
   }
 
   void FinishLoading() {
     data_source_->set_loading(false);
     loader()->didFinishLoading(url_loader(), 0, -1);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   MOCK_METHOD1(ReadCallback, void(int size));
 
-  void ReadAt(int64 position) {
+  void ReadAt(int64_t position) {
     data_source_->Read(position, kDataSize, buffer_,
                        base::Bind(&BufferedDataSourceTest::ReadCallback,
                                   base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
+  }
+
+  void ExecuteMixedResponseSuccessTest(const WebURLResponse& response1,
+                                       const WebURLResponse& response2) {
+    EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+    EXPECT_CALL(host_, AddBufferedByteRange(kDataSize, kDataSize * 2 - 1));
+    EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+    EXPECT_CALL(*this, ReadCallback(kDataSize)).Times(2);
+
+    Respond(response1);
+    ReadAt(0);
+    ReceiveData(kDataSize);
+    EXPECT_TRUE(data_source_->loading());
+
+    ExpectCreateResourceLoader();
+    FinishLoading();
+    ReadAt(kDataSize);
+    Respond(response2);
+    ReceiveData(kDataSize);
+    FinishLoading();
+    Stop();
+  }
+
+  void ExecuteMixedResponseFailureTest(const WebURLResponse& response1,
+                                       const WebURLResponse& response2) {
+    EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+    EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+    EXPECT_CALL(*this, ReadCallback(kDataSize));
+    EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
+
+    Respond(response1);
+    ReadAt(0);
+    ReceiveData(kDataSize);
+    EXPECT_TRUE(data_source_->loading());
+
+    ExpectCreateResourceLoader();
+    FinishLoading();
+    ReadAt(kDataSize);
+    Respond(response2);
+    Stop();
   }
 
   // Accessors for private variables on |data_source_|.
   BufferedResourceLoader* loader() {
     return data_source_->loader_.get();
   }
+  ActiveLoader* active_loader() { return loader()->active_loader_.get(); }
   WebURLLoader* url_loader() {
     return loader()->active_loader_->loader_.get();
   }
@@ -233,27 +290,29 @@ class BufferedDataSourceTest : public testing::Test {
     return loader()->defer_strategy_;
   }
   int data_source_bitrate() { return data_source_->bitrate_; }
-  int data_source_playback_rate() { return data_source_->playback_rate_; }
+  double data_source_playback_rate() { return data_source_->playback_rate_; }
   int loader_bitrate() { return loader()->bitrate_; }
-  int loader_playback_rate() { return loader()->playback_rate_; }
+  double loader_playback_rate() { return loader()->playback_rate_; }
   bool is_local_source() { return data_source_->assume_fully_buffered(); }
   void set_might_be_reused_from_cache_in_future(bool value) {
     loader()->might_be_reused_from_cache_in_future_ = value;
   }
+  GURL url() { return data_source_->url_; }
 
-  scoped_ptr<MockBufferedDataSource> data_source_;
+  std::unique_ptr<MockBufferedDataSource> data_source_;
 
-  scoped_ptr<TestResponseGenerator> response_generator_;
+  std::unique_ptr<TestResponseGenerator> response_generator_;
   MockWebFrameClient client_;
   WebView* view_;
   WebLocalFrame* frame_;
 
   StrictMock<MockBufferedDataSourceHost> host_;
   base::MessageLoop message_loop_;
+  int bytes_received_;
 
  private:
   // Used for calling BufferedDataSource::Read().
-  uint8 buffer_[kDataSize];
+  uint8_t buffer_[kDataSize];
 
   BufferedDataSource::Preload preload_;
 
@@ -368,7 +427,7 @@ TEST_F(BufferedDataSourceTest, Http_AbortWhileReading) {
   // Abort!!!
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
   data_source_->Abort();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_FALSE(data_source_->loading());
   Stop();
@@ -383,7 +442,7 @@ TEST_F(BufferedDataSourceTest, File_AbortWhileReading) {
   // Abort!!!
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
   data_source_->Abort();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_FALSE(data_source_->loading());
   Stop();
@@ -432,6 +491,7 @@ TEST_F(BufferedDataSourceTest, Http_RetryOnError) {
           DoAll(InvokeWithoutArgs(&run_loop, &base::RunLoop::Quit),
                 Invoke(data_source_.get(),
                        &MockBufferedDataSource::CreateMockResourceLoader)));
+  bytes_received_ = 0;
   loader()->didFail(url_loader(), response_generator_->GenerateError());
   run_loop.Run();
   Respond(response_generator_->Generate206(kDataSize));
@@ -441,6 +501,173 @@ TEST_F(BufferedDataSourceTest, Http_RetryOnError) {
   Stop();
 }
 
+#if defined(OS_ANDROID)
+// If the initial response is a redirect, BDS saves it and uses it for future
+// requests.
+TEST_F(BufferedDataSourceTest, Http_InitialReponseRedirectsAreCached) {
+  Initialize(kHttpUrl, true);
+
+  WebURLResponse redirect =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  redirect.setURL(GURL(kHttpDifferentPathUrl));
+
+  EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+  Respond(redirect);
+  ASSERT_TRUE(url() == GURL(kHttpDifferentPathUrl));
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_RedirectsAfterTheInitialReponseAreNotCached) {
+  Initialize(kHttpUrl, true);
+
+  WebURLResponse response =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response.setURL(GURL(kHttpUrl));
+
+  EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+  EXPECT_CALL(host_, AddBufferedByteRange(kDataSize, kDataSize * 2 - 1));
+  EXPECT_CALL(*this, ReadCallback(kDataSize)).Times(2);
+
+  Respond(response);
+  ReadAt(0);
+  ReceiveData(kDataSize);
+
+  WebURLResponse redirect =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  redirect.setURL(GURL(kHttpDifferentPathUrl));
+
+  ExpectCreateResourceLoader();
+  FinishLoading();
+  ReadAt(kDataSize);
+  Respond(redirect);
+  // The redirect isn't cached.
+  ASSERT_TRUE(url() == GURL(kHttpUrl));
+  ReceiveData(kDataSize);
+  FinishLoading();
+  Stop();
+}
+
+TEST_F(BufferedDataSourceTest, Http_ServiceWorkerRedirectsAreNotCached) {
+  Initialize(kHttpUrl, true);
+
+  WebURLResponse redirect =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  redirect.setURL(GURL(kHttpDifferentPathUrl));
+  redirect.setWasFetchedViaServiceWorker(true);
+
+  EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+  Respond(redirect);
+  ASSERT_TRUE(url() == GURL(kHttpUrl));
+}
+#endif  // defined(OS_ANDROID)
+
+TEST_F(BufferedDataSourceTest, Http_PartialResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are same. So no error should
+  // occur.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_RedirectedToDifferentPathResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  response2.setURL(GURL(kHttpDifferentPathUrl));
+  // The origin URL of response1 and response2 are same. So no error should
+  // occur.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_RedirectedToDifferentOriginResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  response2.setURL(GURL(kHttpDifferentOriginUrl));
+  // The origin URL of response1 and response2 are different. So an error should
+  // occur.
+  ExecuteMixedResponseFailureTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_ServiceWorkerGeneratedResponseAndNormalResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // response1 is generated in a Service Worker but response2 is from a native
+  // server. So an error should occur.
+  ExecuteMixedResponseFailureTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_ServiceWorkerProxiedAndSameURLResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are same. So no error should
+  // occur.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_ServiceWorkerProxiedAndDifferentPathResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpDifferentPathUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are same. So no error should
+  // occur.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_ServiceWorkerProxiedAndDifferentOriginResponse) {
+  Initialize(kHttpUrl, true);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpDifferentOriginUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are different. So an error should
+  // occur.
+  ExecuteMixedResponseFailureTest(response1, response2);
+}
+
+TEST_F(BufferedDataSourceTest,
+       Http_MixedResponse_ServiceWorkerProxiedAndDifferentOriginResponseCORS) {
+  InitializeWithCORS(kHttpUrl, true, BufferedResourceLoader::kAnonymous);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpDifferentOriginUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are different, but a CORS check
+  // has been passed for each request, so expect success.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
 TEST_F(BufferedDataSourceTest, File_Retry) {
   InitializeWithFileResponse();
 
@@ -469,18 +696,11 @@ TEST_F(BufferedDataSourceTest, Http_TooManyRetries) {
   // Make sure there's a pending read -- we'll expect it to error.
   ReadAt(0);
 
-  // It'll try three times.
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->Generate206(0));
-
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->Generate206(0));
-
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->Generate206(0));
+  for (int i = 0; i < BufferedDataSource::kLoaderRetries; i++) {
+    ExpectCreateResourceLoader();
+    FinishLoading();
+    Respond(response_generator_->Generate206(0));
+  }
 
   // It'll error after this.
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
@@ -496,18 +716,11 @@ TEST_F(BufferedDataSourceTest, File_TooManyRetries) {
   // Make sure there's a pending read -- we'll expect it to error.
   ReadAt(0);
 
-  // It'll try three times.
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->GenerateFileResponse(0));
-
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->GenerateFileResponse(0));
-
-  ExpectCreateResourceLoader();
-  FinishLoading();
-  Respond(response_generator_->GenerateFileResponse(0));
+  for (int i = 0; i < BufferedDataSource::kLoaderRetries; i++) {
+    ExpectCreateResourceLoader();
+    FinishLoading();
+    Respond(response_generator_->GenerateFileResponse(0));
+  }
 
   // It'll error after this.
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
@@ -538,7 +751,7 @@ TEST_F(BufferedDataSourceTest, File_Successful) {
 TEST_F(BufferedDataSourceTest, StopDuringRead) {
   InitializeWith206Response();
 
-  uint8 buffer[256];
+  uint8_t buffer[256];
   data_source_->Read(0, arraysize(buffer), buffer, base::Bind(
       &BufferedDataSourceTest::ReadCallback, base::Unretained(this)));
 
@@ -548,7 +761,7 @@ TEST_F(BufferedDataSourceTest, StopDuringRead) {
     EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
     data_source_->Stop();
   }
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(BufferedDataSourceTest, DefaultValues) {
@@ -559,9 +772,9 @@ TEST_F(BufferedDataSourceTest, DefaultValues) {
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   EXPECT_EQ(0, data_source_bitrate());
-  EXPECT_EQ(0.0f, data_source_playback_rate());
+  EXPECT_EQ(0.0, data_source_playback_rate());
   EXPECT_EQ(0, loader_bitrate());
-  EXPECT_EQ(0.0f, loader_playback_rate());
+  EXPECT_EQ(0.0, loader_playback_rate());
 
   EXPECT_TRUE(data_source_->loading());
   Stop();
@@ -571,7 +784,7 @@ TEST_F(BufferedDataSourceTest, SetBitrate) {
   InitializeWith206Response();
 
   data_source_->SetBitrate(1234);
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_EQ(1234, data_source_bitrate());
   EXPECT_EQ(1234, loader_bitrate());
 
@@ -593,10 +806,10 @@ TEST_F(BufferedDataSourceTest, SetBitrate) {
 TEST_F(BufferedDataSourceTest, MediaPlaybackRateChanged) {
   InitializeWith206Response();
 
-  data_source_->MediaPlaybackRateChanged(2.0f);
-  message_loop_.RunUntilIdle();
-  EXPECT_EQ(2.0f, data_source_playback_rate());
-  EXPECT_EQ(2.0f, loader_playback_rate());
+  data_source_->MediaPlaybackRateChanged(2.0);
+  base::RunLoop().RunUntilIdle();
+  EXPECT_EQ(2.0, data_source_playback_rate());
+  EXPECT_EQ(2.0, loader_playback_rate());
 
   // Read so far ahead to cause the loader to get recreated.
   BufferedResourceLoader* old_loader = loader();
@@ -700,7 +913,8 @@ TEST_F(BufferedDataSourceTest, LocalResource_DeferStrategy) {
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
@@ -717,7 +931,8 @@ TEST_F(BufferedDataSourceTest, LocalResource_PreloadMetadata_DeferStrategy) {
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
@@ -734,7 +949,8 @@ TEST_F(BufferedDataSourceTest, ExternalResource_Reponse200_DeferStrategy) {
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
@@ -753,7 +969,8 @@ TEST_F(BufferedDataSourceTest,
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
@@ -770,13 +987,18 @@ TEST_F(BufferedDataSourceTest, ExternalResource_Reponse206_DeferStrategy) {
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
   set_might_be_reused_from_cache_in_future(true);
-  data_source_->MediaIsPaused();
+
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kNeverDefer, defer_strategy());
 
-  data_source_->MediaIsPlaying();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_NORMAL);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
+
   set_might_be_reused_from_cache_in_future(false);
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
@@ -794,17 +1016,100 @@ TEST_F(BufferedDataSourceTest,
 
   data_source_->MediaIsPlaying();
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
+
   set_might_be_reused_from_cache_in_future(true);
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kNeverDefer, defer_strategy());
 
-  data_source_->MediaIsPlaying();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_NORMAL);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
+
   set_might_be_reused_from_cache_in_future(false);
-  data_source_->MediaIsPaused();
+  data_source_->SetBufferingStrategy(
+      BufferedDataSource::BUFFERING_STRATEGY_AGGRESSIVE);
   EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
 
   Stop();
 }
 
+TEST_F(BufferedDataSourceTest, ExternalResource_Response206_VerifyDefer) {
+  set_preload(BufferedDataSource::METADATA);
+  InitializeWith206Response();
+
+  EXPECT_EQ(BufferedDataSource::METADATA, preload());
+  EXPECT_FALSE(is_local_source());
+  EXPECT_TRUE(loader()->range_supported());
+  EXPECT_EQ(BufferedResourceLoader::kReadThenDefer, defer_strategy());
+
+  // Read a bit from the beginning.
+  ReadAt(0);
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+  ReceiveData(kDataSize);
+
+  ASSERT_TRUE(active_loader());
+  EXPECT_TRUE(active_loader()->deferred());
+}
+
+TEST_F(BufferedDataSourceTest, ExternalResource_Response206_CancelAfterDefer) {
+  set_preload(BufferedDataSource::METADATA);
+  InitializeWith206Response();
+
+  EXPECT_EQ(BufferedDataSource::METADATA, preload());
+  EXPECT_FALSE(is_local_source());
+  EXPECT_TRUE(loader()->range_supported());
+  EXPECT_EQ(BufferedResourceLoader::kReadThenDefer, defer_strategy());
+
+  data_source_->OnBufferingHaveEnough(false);
+
+  ASSERT_TRUE(active_loader());
+
+  // Read a bit from the beginning.
+  ReadAt(0);
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+  ReceiveData(kDataSize);
+
+  EXPECT_FALSE(active_loader());
+}
+
+TEST_F(BufferedDataSourceTest, ExternalResource_Response206_CancelAfterPlay) {
+  set_preload(BufferedDataSource::METADATA);
+  InitializeWith206Response();
+
+  EXPECT_EQ(BufferedDataSource::METADATA, preload());
+  EXPECT_FALSE(is_local_source());
+  EXPECT_TRUE(loader()->range_supported());
+  EXPECT_EQ(BufferedResourceLoader::kReadThenDefer, defer_strategy());
+
+  // Marking the media as playing should prevent deferral. It also switches the
+  // data source into kCapacityDefer.
+  data_source_->MediaIsPlaying();
+  data_source_->OnBufferingHaveEnough(false);
+  EXPECT_EQ(BufferedResourceLoader::kCapacityDefer, defer_strategy());
+  ASSERT_TRUE(active_loader());
+
+  // Read a bit from the beginning and ensure deferral hasn't happened yet.
+  ReadAt(0);
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize - 1));
+  ReceiveData(kDataSize);
+  ASSERT_TRUE(active_loader());
+  ASSERT_FALSE(active_loader()->deferred());
+  data_source_->OnBufferingHaveEnough(true);
+
+  // Deliver data until capacity is reached and verify deferral.
+  int bytes_received = 0;
+  EXPECT_CALL(host_, AddBufferedByteRange(_, _)).Times(testing::AtLeast(1));
+  while (active_loader() && !active_loader()->deferred()) {
+    ReceiveData(kDataSize);
+    bytes_received += kDataSize;
+  }
+  EXPECT_GT(bytes_received, 0);
+  EXPECT_LT(bytes_received + kDataSize, kFileSize);
+  EXPECT_FALSE(active_loader());
+}
+
 }  // namespace media
