@@ -2,7 +2,10 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "key_system_config_selector.h"
+#include "media/blink/key_system_config_selector.h"
+
+#include <stddef.h>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/logging.h"
@@ -13,6 +16,7 @@
 #include "media/base/media_permission.h"
 #include "media/base/mime_util.h"
 #include "media/blink/webmediaplayer_util.h"
+#include "third_party/WebKit/public/platform/URLConversion.h"
 #include "third_party/WebKit/public/platform/WebMediaKeySystemConfiguration.h"
 #include "third_party/WebKit/public/platform/WebSecurityOrigin.h"
 #include "third_party/WebKit/public/platform/WebString.h"
@@ -273,29 +277,63 @@ KeySystemConfigSelector::KeySystemConfigSelector(
 KeySystemConfigSelector::~KeySystemConfigSelector() {
 }
 
+bool IsSupportedMediaFormat(const std::string& container_mime_type,
+                            const std::string& codecs,
+                            bool use_aes_decryptor) {
+  std::vector<std::string> codec_vector;
+  ParseCodecString(codecs, &codec_vector, false);
+  // AesDecryptor decrypts the stream in the demuxer before it reaches the
+  // decoder so check whether the media format is supported when clear.
+  SupportsType support_result =
+      use_aes_decryptor
+          ? IsSupportedMediaFormat(container_mime_type, codec_vector)
+          : IsSupportedEncryptedMediaFormat(container_mime_type, codec_vector);
+  switch (support_result) {
+    case IsSupported:
+      return true;
+    case MayBeSupported:
+      // If no codecs were specified, the best possible result is
+      // MayBeSupported, indicating support for the container.
+      return codec_vector.empty();
+    case IsNotSupported:
+      return false;
+  }
+  NOTREACHED();
+  return false;
+}
+
+// TODO(sandersd): Move contentType parsing from Blink to here so that invalid
+// parameters can be rejected. http://crbug.com/417561
 bool KeySystemConfigSelector::IsSupportedContentType(
     const std::string& key_system,
     EmeMediaType media_type,
     const std::string& container_mime_type,
     const std::string& codecs,
     KeySystemConfigSelector::ConfigState* config_state) {
-  // TODO(sandersd): Move contentType parsing from Blink to here so that invalid
-  // parameters can be rejected. http://crbug.com/417561
-  std::string container_lower = base::StringToLowerASCII(container_mime_type);
+  // From RFC6838: "Both top-level type and subtype names are case-insensitive."
+  std::string container_lower = base::ToLowerASCII(container_mime_type);
 
-  // Check that |container_mime_type| is supported by Chrome.
-  if (!media::IsSupportedMediaMimeType(container_lower))
-    return false;
+  // contentTypes must provide a codec string unless the container implies a
+  // particular codec. For EME, none of the currently supported containers
+  // imply a codec, so |codecs| must be provided.
+  if (codecs.empty()) {
+    // Since the spec didn't initially require this, add an exemption for
+    // some existing containers to give clients time to adapt.
+    // TODO(jrummell): Remove this exemption once the number of contentTypes
+    // without codecs drops low enough (UMA added in the blink code).
+    // http://crbug.com/605661.
+    if (container_lower != "audio/webm" && container_lower != "video/webm" &&
+        container_lower != "audio/mp4" && container_lower != "video/mp4") {
+      return false;
+    }
+  }
 
-  // Check that |codecs| are supported by Chrome. This is done primarily to
-  // validate extended codecs, but it also ensures that the CDM cannot support
-  // codecs that Chrome does not (which could complicate the robustness
-  // algorithm).
-  std::vector<std::string> codec_vector;
-  media::ParseCodecString(codecs, &codec_vector, false);
-  if (!codec_vector.empty() &&
-      (media::IsSupportedStrictMediaMimeType(container_lower, codec_vector) !=
-       media::IsSupported)) {
+  // Check that |container_mime_type| and |codecs| are supported by Chrome. This
+  // is done primarily to validate extended codecs, but it also ensures that the
+  // CDM cannot support codecs that Chrome does not (which could complicate the
+  // robustness algorithm).
+  if (!IsSupportedMediaFormat(container_lower, codecs,
+                              CanUseAesDecryptor(key_system))) {
     return false;
   }
 
@@ -303,7 +341,7 @@ bool KeySystemConfigSelector::IsSupportedContentType(
   // This check does not handle extended codecs, so extended codec information
   // is stripped (extended codec information was checked above).
   std::vector<std::string> stripped_codec_vector;
-  media::ParseCodecString(codecs, &stripped_codec_vector, true);
+  ParseCodecString(codecs, &stripped_codec_vector, true);
   EmeConfigRule codecs_rule = key_systems_->GetContentTypeConfigRule(
       key_system, media_type, container_lower, stripped_codec_vector);
   if (!config_state->IsRuleSupported(codecs_rule))
@@ -347,8 +385,10 @@ bool KeySystemConfigSelector::GetSupportedCapabilities(
     if (!base::IsStringASCII(capability.mimeType) ||
         !base::IsStringASCII(capability.codecs) ||
         !IsSupportedContentType(key_system, media_type,
-                                base::UTF16ToASCII(capability.mimeType),
-                                base::UTF16ToASCII(capability.codecs),
+                                base::UTF16ToASCII(
+                                    base::StringPiece16(capability.mimeType)),
+                                base::UTF16ToASCII(
+                                    base::StringPiece16(capability.codecs)),
                                 &proposed_config_state)) {
       continue;
     }
@@ -360,7 +400,8 @@ bool KeySystemConfigSelector::GetSupportedCapabilities(
       if (!base::IsStringASCII(capability.robustness))
         continue;
       EmeConfigRule robustness_rule = key_systems_->GetRobustnessConfigRule(
-          key_system, media_type, base::UTF16ToASCII(capability.robustness));
+          key_system, media_type, base::UTF16ToASCII(
+              base::StringPiece16(capability.robustness)));
       if (!proposed_config_state.IsRuleSupported(robustness_rule))
         continue;
       proposed_config_state.AddRule(robustness_rule);
@@ -546,6 +587,7 @@ KeySystemConfigSelector::GetSupportedConfiguration(
     }
 
     // 10.3. Add video capabilities to accumulated configuration.
+    accumulated_configuration->hasVideoCapabilities = true;
     accumulated_configuration->videoCapabilities = video_capabilities;
   }
 
@@ -564,6 +606,7 @@ KeySystemConfigSelector::GetSupportedConfiguration(
     }
 
     // 11.3. Add audio capabilities to accumulated configuration.
+    accumulated_configuration->hasAudioCapabilities = true;
     accumulated_configuration->audioCapabilities = audio_capabilities;
   }
 
@@ -688,7 +731,8 @@ void KeySystemConfigSelector::SelectConfig(
     return;
   }
 
-  std::string key_system_ascii = base::UTF16ToASCII(key_system);
+  std::string key_system_ascii =
+      base::UTF16ToASCII(base::StringPiece16(key_system));
   if (!key_systems_->IsSupportedKeySystem(key_system_ascii)) {
     not_supported_cb.Run("Unsupported keySystem");
     return;
@@ -696,18 +740,18 @@ void KeySystemConfigSelector::SelectConfig(
 
   // 7.2-7.4. Implemented by OnSelectConfig().
   // TODO(sandersd): This should be async, ideally not on the main thread.
-  scoped_ptr<SelectionRequest> request(new SelectionRequest());
+  std::unique_ptr<SelectionRequest> request(new SelectionRequest());
   request->key_system = key_system_ascii;
   request->candidate_configurations = candidate_configurations;
   request->security_origin = security_origin;
   request->are_secure_codecs_supported = are_secure_codecs_supported;
   request->succeeded_cb = succeeded_cb;
   request->not_supported_cb = not_supported_cb;
-  SelectConfigInternal(request.Pass());
+  SelectConfigInternal(std::move(request));
 }
 
 void KeySystemConfigSelector::SelectConfigInternal(
-    scoped_ptr<SelectionRequest> request) {
+    std::unique_ptr<SelectionRequest> request) {
   // Continued from requestMediaKeySystemAccess(), step 7.1, from
   // https://w3c.github.io/encrypted-media/#requestmediakeysystemaccess
   //
@@ -744,7 +788,8 @@ void KeySystemConfigSelector::SelectConfigInternal(
         {
           // Note: the GURL must not be constructed inline because
           // base::Passed(&request) sets |request| to null.
-          GURL security_origin(request->security_origin.toString());
+          GURL security_origin(
+              blink::WebStringToGURL(request->security_origin.toString()));
           media_permission_->RequestPermission(
               MediaPermission::PROTECTED_MEDIA_IDENTIFIER, security_origin,
               base::Bind(&KeySystemConfigSelector::OnPermissionResult,
@@ -772,11 +817,11 @@ void KeySystemConfigSelector::SelectConfigInternal(
 }
 
 void KeySystemConfigSelector::OnPermissionResult(
-    scoped_ptr<SelectionRequest> request,
+    std::unique_ptr<SelectionRequest> request,
     bool is_permission_granted) {
   request->was_permission_requested = true;
   request->is_permission_granted = is_permission_granted;
-  SelectConfigInternal(request.Pass());
+  SelectConfigInternal(std::move(request));
 }
 
 }  // namespace media
