@@ -65,7 +65,7 @@ class MultibufferDataSource::ReadOperation {
 
   // Runs |callback_| with the given |result|, deleting the operation
   // afterwards.
-  static void Run(scoped_ptr<ReadOperation> read_op, int result);
+  static void Run(std::unique_ptr<ReadOperation> read_op, int result);
 
   int64_t position() { return position_; }
   int size() { return size_; }
@@ -95,7 +95,7 @@ MultibufferDataSource::ReadOperation::~ReadOperation() {
 
 // static
 void MultibufferDataSource::ReadOperation::Run(
-    scoped_ptr<ReadOperation> read_op,
+    std::unique_ptr<ReadOperation> read_op,
     int result) {
   base::ResetAndReturn(&read_op->callback_).Run(result);
 }
@@ -113,6 +113,7 @@ MultibufferDataSource::MultibufferDataSource(
       total_bytes_(kPositionNotSpecified),
       streaming_(false),
       loading_(false),
+      failed_(false),
       render_task_runner_(task_runner),
       url_index_(url_index),
       frame_(frame),
@@ -185,17 +186,16 @@ void MultibufferDataSource::OnRedirect(
     const scoped_refptr<UrlData>& destination) {
   if (!destination) {
     // A failure occured.
+    failed_ = true;
     if (!init_cb_.is_null()) {
       render_task_runner_->PostTask(
           FROM_HERE,
           base::Bind(&MultibufferDataSource::StartCallback, weak_ptr_));
     } else {
-      {
-        base::AutoLock auto_lock(lock_);
-        StopInternal_Locked();
-      }
-      StopLoader();
+      base::AutoLock auto_lock(lock_);
+      StopInternal_Locked();
     }
+    StopLoader();
     return;
   }
   if (url_data_->url().GetOrigin() != destination->url().GetOrigin()) {
@@ -258,8 +258,7 @@ bool MultibufferDataSource::DidPassCORSAccessCheck() const {
   // If init_cb is set, we initialization is not finished yet.
   if (!init_cb_.is_null())
     return false;
-  // Loader will be false if there was a failure.
-  if (!reader_)
+  if (failed_)
     return false;
   return true;
 }
@@ -314,9 +313,10 @@ void MultibufferDataSource::SetBitrate(int bitrate) {
                             weak_factory_.GetWeakPtr(), bitrate));
 }
 
-void MultibufferDataSource::OnBufferingHaveEnough() {
+void MultibufferDataSource::OnBufferingHaveEnough(bool always_cancel) {
   DCHECK(render_task_runner_->BelongsToCurrentThread());
-  if (reader_ && preload_ == METADATA && !media_has_played_ && !IsStreaming()) {
+  if (reader_ && (always_cancel || (preload_ == METADATA &&
+                                    !media_has_played_ && !IsStreaming()))) {
     cancel_on_defer_ = true;
     if (!loading_)
       reader_.reset(nullptr);
@@ -329,6 +329,10 @@ int64_t MultibufferDataSource::GetMemoryUsage() const {
          << url_data_->multibuffer()->block_size_shift();
 }
 
+GURL MultibufferDataSource::GetUrlAfterRedirects() const {
+  return url_data_->url();
+}
+
 void MultibufferDataSource::Read(int64_t position,
                                  int size,
                                  uint8_t* data,
@@ -356,8 +360,13 @@ void MultibufferDataSource::Read(int64_t position,
 }
 
 bool MultibufferDataSource::GetSize(int64_t* size_out) {
-  *size_out = url_data_->length();
-  return *size_out != kPositionNotSpecified;
+  base::AutoLock auto_lock(lock_);
+  if (total_bytes_ != kPositionNotSpecified) {
+    *size_out = total_bytes_;
+    return true;
+  }
+  *size_out = 0;
+  return false;
 }
 
 bool MultibufferDataSource::IsStreaming() {
@@ -392,6 +401,16 @@ void MultibufferDataSource::ReadTask() {
     bytes_read =
         static_cast<int>(std::min<int64_t>(available, read_op_->size()));
     bytes_read = reader_->TryRead(read_op_->data(), bytes_read);
+
+    if (bytes_read == 0 && total_bytes_ == kPositionNotSpecified) {
+      // We've reached the end of the file and we didn't know the total size
+      // before. Update the total size so Read()s past the end of the file will
+      // fail like they would if we had known the file size at the beginning.
+      total_bytes_ = reader_->Tell();
+      if (total_bytes_ != kPositionNotSpecified)
+        host_->SetTotalBytes(total_bytes_);
+    }
+
     ReadOperation::Run(std::move(read_op_), bytes_read);
   } else {
     reader_->Wait(1, base::Bind(&MultibufferDataSource::ReadTask,
@@ -445,7 +464,10 @@ void MultibufferDataSource::StartCallback() {
                   url_data_->length() != kPositionNotSpecified);
 
   if (success) {
-    total_bytes_ = url_data_->length();
+    {
+      base::AutoLock auto_lock(lock_);
+      total_bytes_ = url_data_->length();
+    }
     streaming_ =
         !assume_fully_buffered() && (total_bytes_ == kPositionNotSpecified ||
                                      !url_data_->range_supported());
@@ -558,16 +580,25 @@ void MultibufferDataSource::UpdateBufferSizes() {
 
   int64_t preload = clamp(kTargetSecondsBufferedAhead * bytes_per_second,
                           kMinBufferPreload, kMaxBufferPreload);
+  int64_t preload_high = preload + kPreloadHighExtra;
+
+  // Assert that kMaxBufferSize is big enough that the subtraction on the next
+  // line cannot go negative.
+  static_assert(kMaxBufferSize > kMaxBufferPreload + kPreloadHighExtra,
+                "kMaxBufferSize too small to contain preload.");
   int64_t back_buffer = clamp(kTargetSecondsBufferedBehind * bytes_per_second,
-                              kMinBufferPreload, kMaxBufferPreload);
-  int64_t pin_forwards = kMaxBufferSize - back_buffer;
-  DCHECK_LE(preload_ + kPreloadHighExtra, pin_forwards);
-  reader_->SetMaxBuffer(back_buffer, pin_forwards);
+                              kMinBufferPreload, kMaxBufferSize - preload_high);
+  int64_t buffer_size =
+      std::min((kTargetSecondsBufferedAhead + kTargetSecondsBufferedBehind) *
+                   bytes_per_second,
+               kMaxBufferSize);
+  reader_->SetMaxBuffer(buffer_size);
+  reader_->SetPinRange(back_buffer, kMaxBufferPreload + kPreloadHighExtra);
 
   if (preload_ == METADATA) {
     reader_->SetPreload(0, 0);
   } else {
-    reader_->SetPreload(preload + kPreloadHighExtra, preload);
+    reader_->SetPreload(preload_high, preload);
   }
 }
 
