@@ -58,10 +58,16 @@ class TestMultiBufferDataProvider : public ResourceMultiBufferDataProvider {
     // Create a mock active loader.
     // Keep track of active loading state via loadAsynchronously() and cancel().
     NiceMock<MockWebURLLoader>* url_loader = new NiceMock<MockWebURLLoader>();
-    ON_CALL(*url_loader, cancel()).WillByDefault(Assign(&loading_, false));
+    ON_CALL(*url_loader, cancel())
+        .WillByDefault(Invoke([this]() {
+          // Check that we have not been destroyed first.
+          if (test_data_providers.find(this) != test_data_providers.end()) {
+            this->loading_ = false;
+          }
+        }));
     loading_ = true;
     active_loader_.reset(
-        new ActiveLoader(scoped_ptr<WebURLLoader>(url_loader)));
+        new ActiveLoader(std::unique_ptr<WebURLLoader>(url_loader)));
     if (!on_start_.is_null()) {
       on_start_.Run();
     }
@@ -82,12 +88,12 @@ class TestResourceMultiBuffer : public ResourceMultiBuffer {
   explicit TestResourceMultiBuffer(UrlData* url_data, int shift)
       : ResourceMultiBuffer(url_data, shift) {}
 
-  scoped_ptr<MultiBuffer::DataProvider> CreateWriter(
+  std::unique_ptr<MultiBuffer::DataProvider> CreateWriter(
       const BlockId& pos) override {
     TestMultiBufferDataProvider* ret =
         new TestMultiBufferDataProvider(url_data_, pos);
     ret->Start();
-    return scoped_ptr<MultiBuffer::DataProvider>(ret);
+    return std::unique_ptr<MultiBuffer::DataProvider>(ret);
   }
 
   // TODO: Make these global
@@ -138,7 +144,7 @@ class TestUrlData : public UrlData {
   ~TestUrlData() override {}
   const int block_shift_;
 
-  scoped_ptr<TestResourceMultiBuffer> test_multibuffer_;
+  std::unique_ptr<TestResourceMultiBuffer> test_multibuffer_;
 };
 
 class TestUrlIndex : public UrlIndex {
@@ -177,13 +183,14 @@ class MockMultibufferDataSource : public MultibufferDataSource {
  public:
   MockMultibufferDataSource(
       const GURL& url,
+      UrlData::CORSMode cors_mode,
       const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
       linked_ptr<UrlIndex> url_index,
       WebLocalFrame* frame,
       BufferedDataSourceHost* host)
       : MultibufferDataSource(
             url,
-            UrlData::CORS_UNSPECIFIED,
+            cors_mode,
             task_runner,
             url_index,
             frame,
@@ -201,9 +208,6 @@ class MockMultibufferDataSource : public MultibufferDataSource {
   // Whether the resource is downloading or deferred.
   bool downloading_;
 
-  // Whether the resource load has starting loading but yet to been cancelled.
-  bool loading_;
-
   DISALLOW_COPY_AND_ASSIGN(MockMultibufferDataSource);
 };
 
@@ -219,7 +223,7 @@ static const char kHttpDifferentOriginUrl[] = "http://127.0.0.1/foo.webm";
 class MultibufferDataSourceTest : public testing::Test {
  public:
   MultibufferDataSourceTest()
-      : view_(WebView::create(NULL)),
+      : view_(WebView::create(nullptr, blink::WebPageVisibilityStateVisible)),
         frame_(
             WebLocalFrame::create(blink::WebTreeScopeType::Document, &client_)),
         preload_(MultibufferDataSource::AUTO),
@@ -234,10 +238,12 @@ class MultibufferDataSourceTest : public testing::Test {
 
   MOCK_METHOD1(OnInitialize, void(bool));
 
-  void Initialize(const char* url, bool expected) {
+  void InitializeWithCORS(const char* url,
+                          bool expected,
+                          UrlData::CORSMode cors_mode) {
     GURL gurl(url);
     data_source_.reset(new MockMultibufferDataSource(
-        gurl, message_loop_.task_runner(), url_index_,
+        gurl, cors_mode, message_loop_.task_runner(), url_index_,
         view_->mainFrame()->toWebLocalFrame(), &host_));
     data_source_->SetPreload(preload_);
 
@@ -245,12 +251,16 @@ class MultibufferDataSourceTest : public testing::Test {
     EXPECT_CALL(*this, OnInitialize(expected));
     data_source_->Initialize(base::Bind(
         &MultibufferDataSourceTest::OnInitialize, base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
 
     // Not really loading until after OnInitialize is called.
     EXPECT_EQ(data_source_->downloading(), false);
   }
 
+  void Initialize(const char* url, bool expected) {
+    InitializeWithCORS(url, expected, UrlData::CORS_UNSPECIFIED);
+  }
+
   // Helper to initialize tests with a valid 200 response.
   void InitializeWith200Response() {
     Initialize(kHttpUrl, true);
@@ -291,11 +301,11 @@ class MultibufferDataSourceTest : public testing::Test {
     if (loading()) {
       data_provider()->didFail(url_loader(),
                                response_generator_->GenerateError());
-      message_loop_.RunUntilIdle();
+      base::RunLoop().RunUntilIdle();
     }
 
     data_source_->Stop();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void Respond(const WebURLResponse& response) {
@@ -303,18 +313,18 @@ class MultibufferDataSourceTest : public testing::Test {
     if (!active_loader())
       return;
     data_provider()->didReceiveResponse(url_loader(), response);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void ReceiveData(int size) {
     EXPECT_TRUE(url_loader());
     if (!url_loader())
       return;
-    scoped_ptr<char[]> data(new char[size]);
+    std::unique_ptr<char[]> data(new char[size]);
     memset(data.get(), 0xA5, size);  // Arbitrary non-zero value.
 
     data_provider()->didReceiveData(url_loader(), data.get(), size, size);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void FinishLoading() {
@@ -322,7 +332,13 @@ class MultibufferDataSourceTest : public testing::Test {
     if (!url_loader())
       return;
     data_provider()->didFinishLoading(url_loader(), 0, -1);
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
+  }
+
+  void FailLoading() {
+    data_provider()->didFail(url_loader(),
+                             response_generator_->GenerateError());
+    base::RunLoop().RunUntilIdle();
   }
 
   void Restart() {
@@ -339,7 +355,7 @@ class MultibufferDataSourceTest : public testing::Test {
     data_source_->Read(position, howmuch, buffer_,
                        base::Bind(&MultibufferDataSourceTest::ReadCallback,
                                   base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void ExecuteMixedResponseSuccessTest(const WebURLResponse& response1,
@@ -368,7 +384,11 @@ class MultibufferDataSourceTest : public testing::Test {
     EXPECT_CALL(host_, SetTotalBytes(kFileSize));
     EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
     EXPECT_CALL(*this, ReadCallback(kDataSize));
-    EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
+    // Stop() will also cause the readback to be called with kReadError, but
+    // we want to make sure it was called before Stop().
+    bool failed_ = false;
+    EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError))
+        .WillOnce(Assign(&failed_, true));
 
     Respond(response1);
     ReceiveData(kDataSize);
@@ -379,6 +399,7 @@ class MultibufferDataSourceTest : public testing::Test {
     Restart();
     ReadAt(kDataSize);
     Respond(response2);
+    EXPECT_TRUE(failed_);
     Stop();
   }
 
@@ -453,14 +474,14 @@ class MultibufferDataSourceTest : public testing::Test {
   WebView* view_;
   WebLocalFrame* frame_;
   MultibufferDataSource::Preload preload_;
+  base::MessageLoop message_loop_;
   linked_ptr<TestUrlIndex> url_index_;
 
-  scoped_ptr<MockMultibufferDataSource> data_source_;
+  std::unique_ptr<MockMultibufferDataSource> data_source_;
 
-  scoped_ptr<TestResponseGenerator> response_generator_;
+  std::unique_ptr<TestResponseGenerator> response_generator_;
 
   StrictMock<MockBufferedDataSourceHost> host_;
-  base::MessageLoop message_loop_;
 
   // Used for calling MultibufferDataSource::Read().
   uint8_t buffer_[kDataSize * 2];
@@ -506,6 +527,14 @@ TEST_F(MultibufferDataSourceTest, Range_NotSupported) {
   Stop();
 }
 
+TEST_F(MultibufferDataSourceTest, Range_NotSatisfiable) {
+  Initialize(kHttpUrl, true);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  Respond(response_generator_->GenerateResponse(416));
+  EXPECT_FALSE(loading());
+  Stop();
+}
+
 // Special carve-out for Apache versions that choose to return a 200 for
 // Range:0- ("because it's more efficient" than a 206)
 TEST_F(MultibufferDataSourceTest, Range_SupportedButReturned200) {
@@ -584,7 +613,7 @@ TEST_F(MultibufferDataSourceTest, Http_AbortWhileReading) {
   // Abort!!!
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
   data_source_->Abort();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_FALSE(loading());
   Stop();
@@ -599,7 +628,7 @@ TEST_F(MultibufferDataSourceTest, File_AbortWhileReading) {
   // Abort!!!
   EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
   data_source_->Abort();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_FALSE(loading());
   Stop();
@@ -769,6 +798,20 @@ TEST_F(MultibufferDataSourceTest,
   ExecuteMixedResponseFailureTest(response1, response2);
 }
 
+TEST_F(MultibufferDataSourceTest,
+       Http_MixedResponse_ServiceWorkerProxiedAndDifferentOriginResponseCORS) {
+  InitializeWithCORS(kHttpUrl, true, UrlData::CORS_ANONYMOUS);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpDifferentOriginUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+  // The origin URL of response1 and response2 are different, but a CORS check
+  // has been passed for each request, so expect success.
+  ExecuteMixedResponseSuccessTest(response1, response2);
+}
+
 TEST_F(MultibufferDataSourceTest, File_Retry) {
   InitializeWithFileResponse();
 
@@ -796,23 +839,19 @@ TEST_F(MultibufferDataSourceTest, Http_TooManyRetries) {
   // Make sure there's a pending read -- we'll expect it to error.
   ReadAt(kDataSize);
 
-  // It'll try three times.
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->Generate206(kDataSize));
-
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->Generate206(kDataSize));
-
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->Generate206(kDataSize));
-
-  // It'll error after this.
-  EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
-  FinishLoading();
+  for (int i = 0; i < ResourceMultiBufferDataProvider::kMaxRetries; i++) {
+    FailLoading();
+    data_provider()->Start();
+    Respond(response_generator_->Generate206(kDataSize));
+  }
 
+  // Stop() will also cause the readback to be called with kReadError, but
+  // we want to make sure it was called during FailLoading().
+  bool failed_ = false;
+  EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError))
+      .WillOnce(Assign(&failed_, true));
+  FailLoading();
+  EXPECT_TRUE(failed_);
   EXPECT_FALSE(loading());
   Stop();
 }
@@ -823,23 +862,19 @@ TEST_F(MultibufferDataSourceTest, File_TooManyRetries) {
   // Make sure there's a pending read -- we'll expect it to error.
   ReadAt(kDataSize);
 
-  // It'll try three times.
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->GenerateFileResponse(0));
-
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->GenerateFileResponse(0));
-
-  FinishLoading();
-  Restart();
-  Respond(response_generator_->GenerateFileResponse(0));
-
-  // It'll error after this.
-  EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
-  FinishLoading();
+  for (int i = 0; i < ResourceMultiBufferDataProvider::kMaxRetries; i++) {
+    FailLoading();
+    data_provider()->Start();
+    Respond(response_generator_->Generate206(kDataSize));
+  }
 
+  // Stop() will also cause the readback to be called with kReadError, but
+  // we want to make sure it was called during FailLoading().
+  bool failed_ = false;
+  EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError))
+      .WillOnce(Assign(&failed_, true));
+  FailLoading();
+  EXPECT_TRUE(failed_);
   EXPECT_FALSE(loading());
   Stop();
 }
@@ -878,7 +913,7 @@ TEST_F(MultibufferDataSourceTest, StopDuringRead) {
     EXPECT_CALL(*this, ReadCallback(media::DataSource::kReadError));
     data_source_->Stop();
   }
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(MultibufferDataSourceTest, DefaultValues) {
@@ -900,7 +935,7 @@ TEST_F(MultibufferDataSourceTest, SetBitrate) {
   InitializeWith206Response();
 
   data_source_->SetBitrate(1234);
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_EQ(1234, data_source_bitrate());
 
   // Read so far ahead to cause the loader to get recreated.
@@ -920,7 +955,7 @@ TEST_F(MultibufferDataSourceTest, MediaPlaybackRateChanged) {
   InitializeWith206Response();
 
   data_source_->MediaPlaybackRateChanged(2.0);
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_EQ(2.0, data_source_playback_rate());
 
   // Read so far ahead to cause the loader to get recreated.
@@ -944,8 +979,10 @@ TEST_F(MultibufferDataSourceTest, Http_Read) {
 
   ReadAt(kDataSize, kDataSize);
   EXPECT_CALL(*this, ReadCallback(kDataSize));
-  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
+  EXPECT_CALL(host_,
+              AddBufferedByteRange(kDataSize, kDataSize + kDataSize / 2));
   ReceiveData(kDataSize / 2);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
   ReceiveData(kDataSize / 2);
 
   EXPECT_TRUE(data_source_->downloading());
@@ -960,16 +997,18 @@ TEST_F(MultibufferDataSourceTest, Http_ShareData) {
 
   ReadAt(kDataSize, kDataSize);
   EXPECT_CALL(*this, ReadCallback(kDataSize));
-  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
+  EXPECT_CALL(host_,
+              AddBufferedByteRange(kDataSize, kDataSize + kDataSize / 2));
   ReceiveData(kDataSize / 2);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
   ReceiveData(kDataSize / 2);
 
   EXPECT_TRUE(data_source_->downloading());
 
   StrictMock<MockBufferedDataSourceHost> host2;
   MockMultibufferDataSource source2(
-      GURL(kHttpUrl), message_loop_.task_runner(), url_index_,
-      view_->mainFrame()->toWebLocalFrame(), &host2);
+      GURL(kHttpUrl), UrlData::CORS_UNSPECIFIED, message_loop_.task_runner(),
+      url_index_, view_->mainFrame()->toWebLocalFrame(), &host2);
   source2.SetPreload(preload_);
 
   EXPECT_CALL(*this, OnInitialize(true));
@@ -978,7 +1017,7 @@ TEST_F(MultibufferDataSourceTest, Http_ShareData) {
   EXPECT_CALL(host2, SetTotalBytes(response_generator_->content_length()));
   source2.Initialize(base::Bind(&MultibufferDataSourceTest::OnInitialize,
                                 base::Unretained(this)));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   // Always loading after initialize.
   EXPECT_EQ(source2.downloading(), true);
@@ -1208,7 +1247,7 @@ TEST_F(MultibufferDataSourceTest,
 
   ReadAt(kDataSize);
 
-  data_source_->OnBufferingHaveEnough();
+  data_source_->OnBufferingHaveEnough(false);
   ASSERT_TRUE(active_loader());
 
   EXPECT_CALL(*this, ReadCallback(kDataSize));
@@ -1218,4 +1257,202 @@ TEST_F(MultibufferDataSourceTest,
   EXPECT_FALSE(active_loader_allownull());
 }
 
+TEST_F(MultibufferDataSourceTest,
+       ExternalResource_Response206_CancelAfterPlay) {
+  set_preload(BufferedDataSource::METADATA);
+  InitializeWith206Response();
+
+  EXPECT_EQ(MultibufferDataSource::METADATA, preload());
+  EXPECT_FALSE(is_local_source());
+
+  EXPECT_TRUE(data_source_->range_supported());
+  CheckReadThenDefer();
+
+  ReadAt(kDataSize);
+
+  // Marking the media as playing should prevent deferral. It also tells the
+  // data source to start buffering beyond the initial load.
+  data_source_->MediaIsPlaying();
+  data_source_->OnBufferingHaveEnough(false);
+  CheckCapacityDefer();
+  ASSERT_TRUE(active_loader());
+
+  // Read a bit from the beginning and ensure deferral hasn't happened yet.
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
+  ReceiveData(kDataSize);
+  ASSERT_TRUE(active_loader());
+  data_source_->OnBufferingHaveEnough(true);
+  ASSERT_TRUE(active_loader());
+  ASSERT_FALSE(active_loader()->deferred());
+
+  // Deliver data until capacity is reached and verify deferral.
+  int bytes_received = 0;
+  EXPECT_CALL(host_, AddBufferedByteRange(_, _)).Times(testing::AtLeast(1));
+  while (active_loader_allownull() && !active_loader()->deferred()) {
+    ReceiveData(kDataSize);
+    bytes_received += kDataSize;
+  }
+  EXPECT_GT(bytes_received, 0);
+  EXPECT_LT(bytes_received + kDataSize, kFileSize);
+  EXPECT_FALSE(active_loader_allownull());
+}
+
+TEST_F(MultibufferDataSourceTest, SeekPastEOF) {
+  GURL gurl(kHttpUrl);
+  data_source_.reset(new MockMultibufferDataSource(
+      gurl, UrlData::CORS_UNSPECIFIED, message_loop_.task_runner(), url_index_,
+      view_->mainFrame()->toWebLocalFrame(), &host_));
+  data_source_->SetPreload(preload_);
+
+  response_generator_.reset(new TestResponseGenerator(gurl, kDataSize + 1));
+  EXPECT_CALL(*this, OnInitialize(true));
+  data_source_->Initialize(base::Bind(&MultibufferDataSourceTest::OnInitialize,
+                                      base::Unretained(this)));
+  base::RunLoop().RunUntilIdle();
+
+  // Not really loading until after OnInitialize is called.
+  EXPECT_EQ(data_source_->downloading(), false);
+
+  EXPECT_CALL(host_, SetTotalBytes(response_generator_->content_length()));
+  Respond(response_generator_->Generate206(0));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  ReceiveData(kDataSize);
+
+  // Read a bit from the beginning.
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  ReadAt(0);
+
+  EXPECT_CALL(host_, AddBufferedByteRange(kDataSize, kDataSize + 1));
+  ReceiveData(1);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 3));
+  FinishLoading();
+  EXPECT_CALL(*this, ReadCallback(0));
+
+  ReadAt(kDataSize + 5, kDataSize * 2);
+  Stop();
+}
+
+TEST_F(MultibufferDataSourceTest, Http_RetryThenRedirect) {
+  InitializeWith206Response();
+
+  // Read to advance our position.
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  ReadAt(0);
+
+  // Issue a pending read but trigger an error to force a retry.
+  EXPECT_CALL(*this, ReadCallback(kDataSize - 10));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
+  ReadAt(kDataSize + 10, kDataSize - 10);
+  base::RunLoop run_loop;
+  data_provider()->didFail(url_loader(), response_generator_->GenerateError());
+  data_provider()->RunOnStart(run_loop.QuitClosure());
+  run_loop.Run();
+
+  // Server responds with a redirect.
+  blink::WebURLRequest request((GURL(kHttpDifferentPathUrl)));
+  blink::WebURLResponse response((GURL(kHttpUrl)));
+  response.setHTTPStatusCode(307);
+  data_provider()->willFollowRedirect(url_loader(), request, response);
+  Respond(response_generator_->Generate206(kDataSize));
+  ReceiveData(kDataSize);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 3));
+  FinishLoading();
+  EXPECT_FALSE(loading());
+  Stop();
+}
+
+TEST_F(MultibufferDataSourceTest, Http_NotStreamingAfterRedirect) {
+  Initialize(kHttpUrl, true);
+
+  // Server responds with a redirect.
+  blink::WebURLRequest request((GURL(kHttpDifferentPathUrl)));
+  blink::WebURLResponse response((GURL(kHttpUrl)));
+  response.setHTTPStatusCode(307);
+  data_provider()->willFollowRedirect(url_loader(), request, response);
+
+  EXPECT_CALL(host_, SetTotalBytes(response_generator_->content_length()));
+  Respond(response_generator_->Generate206(0));
+
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  ReceiveData(kDataSize);
+
+  EXPECT_FALSE(data_source_->IsStreaming());
+
+  FinishLoading();
+  EXPECT_FALSE(loading());
+  Stop();
+}
+
+TEST_F(MultibufferDataSourceTest, Http_RangeNotSatisfiableAfterRedirect) {
+  Initialize(kHttpUrl, true);
+
+  // Server responds with a redirect.
+  blink::WebURLRequest request((GURL(kHttpDifferentPathUrl)));
+  blink::WebURLResponse response((GURL(kHttpUrl)));
+  response.setHTTPStatusCode(307);
+  data_provider()->willFollowRedirect(url_loader(), request, response);
+
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  Respond(response_generator_->GenerateResponse(416));
+  Stop();
+}
+
+TEST_F(MultibufferDataSourceTest, LengthKnownAtEOF) {
+  Initialize(kHttpUrl, true);
+  // Server responds without content-length.
+  WebURLResponse response = response_generator_->Generate200();
+  response.clearHTTPHeaderField(WebString::fromUTF8("Content-Length"));
+  response.setExpectedContentLength(kPositionNotSpecified);
+  Respond(response);
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  ReceiveData(kDataSize);
+  int64_t len;
+  EXPECT_FALSE(data_source_->GetSize(&len));
+  EXPECT_TRUE(data_source_->IsStreaming());
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+  ReadAt(0);
+
+  ReadAt(kDataSize);
+  EXPECT_CALL(host_, SetTotalBytes(kDataSize));
+  EXPECT_CALL(*this, ReadCallback(0));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize * 2));
+  FinishLoading();
+
+  // Done loading, now we should know the length.
+  EXPECT_TRUE(data_source_->GetSize(&len));
+  EXPECT_EQ(kDataSize, len);
+  Stop();
+}
+
+TEST_F(MultibufferDataSourceTest, DidPassCORSAccessTest) {
+  InitializeWithCORS(kHttpUrl, true, UrlData::CORS_ANONYMOUS);
+  set_preload(MultibufferDataSource::NONE);
+  WebURLResponse response1 =
+      response_generator_->GeneratePartial206(0, kDataSize - 1);
+  response1.setWasFetchedViaServiceWorker(true);
+  response1.setOriginalURLViaServiceWorker(GURL(kHttpDifferentOriginUrl));
+  WebURLResponse response2 =
+      response_generator_->GeneratePartial206(kDataSize, kDataSize * 2 - 1);
+
+  EXPECT_CALL(host_, SetTotalBytes(kFileSize));
+  EXPECT_CALL(host_, AddBufferedByteRange(0, kDataSize));
+  EXPECT_CALL(*this, ReadCallback(kDataSize));
+
+  EXPECT_FALSE(data_source_->DidPassCORSAccessCheck());
+  Respond(response1);
+  ReceiveData(kDataSize);
+  ReadAt(0);
+  EXPECT_TRUE(loading());
+  EXPECT_TRUE(data_source_->DidPassCORSAccessCheck());
+
+  FinishLoading();
+
+  // Verify that if reader_ is null, DidPassCORSAccessCheck still returns true.
+  data_source_->Stop();
+  base::RunLoop().RunUntilIdle();
+
+  EXPECT_TRUE(data_source_->DidPassCORSAccessCheck());
+}
+
 }  // namespace media
