@@ -12,22 +12,11 @@
 #include "base/metrics/histogram.h"
 #include "media/base/bind_to_current_loop.h"
 #include "media/base/media_client.h"
-#include "media/base/media_keys.h"
+#include "third_party/WebKit/public/platform/URLConversion.h"
 #include "third_party/WebKit/public/platform/WebMediaPlayerEncryptedMediaClient.h"
 
 namespace media {
 
-// Compile asserts shared by all platforms.
-
-#define STATIC_ASSERT_MATCHING_ENUM(name)                                    \
-  static_assert(static_cast<int>(blink::WebMediaPlayerEncryptedMediaClient:: \
-                                     MediaKeyErrorCode##name) ==             \
-                    static_cast<int>(MediaKeys::k##name##Error),             \
-                "mismatching enum values: " #name)
-STATIC_ASSERT_MATCHING_ENUM(Unknown);
-STATIC_ASSERT_MATCHING_ENUM(Client);
-#undef STATIC_ASSERT_MATCHING_ENUM
-
 blink::WebTimeRanges ConvertToWebTimeRanges(
     const Ranges<base::TimeDelta>& ranges) {
   blink::WebTimeRanges result(ranges.size());
@@ -40,19 +29,14 @@ blink::WebTimeRanges ConvertToWebTimeRanges(
 
 blink::WebMediaPlayer::NetworkState PipelineErrorToNetworkState(
     PipelineStatus error) {
-  DCHECK_NE(error, PIPELINE_OK);
-
   switch (error) {
     case PIPELINE_ERROR_NETWORK:
     case PIPELINE_ERROR_READ:
+    case CHUNK_DEMUXER_ERROR_EOS_STATUS_NETWORK_ERROR:
       return blink::WebMediaPlayer::NetworkStateNetworkError;
 
-    // TODO(vrk): Because OnPipelineInitialize() directly reports the
-    // NetworkStateFormatError instead of calling OnPipelineError(), I believe
-    // this block can be deleted. Should look into it! (crbug.com/126070)
     case PIPELINE_ERROR_INITIALIZATION_FAILED:
     case PIPELINE_ERROR_COULD_NOT_RENDER:
-    case PIPELINE_ERROR_URL_NOT_FOUND:
     case DEMUXER_ERROR_COULD_NOT_OPEN:
     case DEMUXER_ERROR_COULD_NOT_PARSE:
     case DEMUXER_ERROR_NO_SUPPORTED_STREAMS:
@@ -61,8 +45,11 @@ blink::WebMediaPlayer::NetworkState PipelineErrorToNetworkState(
 
     case PIPELINE_ERROR_DECODE:
     case PIPELINE_ERROR_ABORT:
-    case PIPELINE_ERROR_OPERATION_PENDING:
     case PIPELINE_ERROR_INVALID_STATE:
+    case CHUNK_DEMUXER_ERROR_APPEND_FAILED:
+    case CHUNK_DEMUXER_ERROR_EOS_STATUS_DECODE_ERROR:
+    case AUDIO_RENDERER_ERROR:
+    case AUDIO_RENDERER_ERROR_SPLICE_FAILED:
       return blink::WebMediaPlayer::NetworkStateDecodeError;
 
     case PIPELINE_OK:
@@ -120,11 +107,9 @@ std::string LoadTypeToString(blink::WebMediaPlayer::LoadType load_type) {
 
 }  // namespace
 
-// TODO(xhwang): Call this from WebMediaPlayerMS to report metrics for
-// MediaStream as well.
 void ReportMetrics(blink::WebMediaPlayer::LoadType load_type,
                    const GURL& url,
-                   const GURL& origin_url) {
+                   const blink::WebSecurityOrigin& security_origin) {
   // Report URL scheme, such as http, https, file, blob etc.
   UMA_HISTOGRAM_ENUMERATION("Media.URLScheme", URLScheme(url),
                             kMaxURLScheme + 1);
@@ -135,11 +120,39 @@ void ReportMetrics(blink::WebMediaPlayer::LoadType load_type,
 
   // Report the origin from where the media player is created.
   if (GetMediaClient()) {
+    GURL security_origin_url(
+        blink::WebStringToGURL(security_origin.toString()));
+
     GetMediaClient()->RecordRapporURL(
-        "Media.OriginUrl." + LoadTypeToString(load_type), origin_url);
+        "Media.OriginUrl." + LoadTypeToString(load_type), security_origin_url);
+
+    // For MSE, also report usage by secure/insecure origin.
+    if (load_type == blink::WebMediaPlayer::LoadTypeMediaSource) {
+      if (security_origin.isPotentiallyTrustworthy()) {
+        GetMediaClient()->RecordRapporURL("Media.OriginUrl.MSE.Secure",
+                                          security_origin_url);
+      } else {
+        GetMediaClient()->RecordRapporURL("Media.OriginUrl.MSE.Insecure",
+                                          security_origin_url);
+      }
+    }
   }
 }
 
+void ReportPipelineError(blink::WebMediaPlayer::LoadType load_type,
+                         const blink::WebSecurityOrigin& security_origin,
+                         PipelineStatus error) {
+  DCHECK_NE(PIPELINE_OK, error);
+
+  // Report the origin from where the media player is created.
+  if (!GetMediaClient())
+    return;
+
+  GetMediaClient()->RecordRapporURL(
+      "Media.OriginUrl." + LoadTypeToString(load_type) + ".PipelineError",
+      blink::WebStringToGURL(security_origin.toString()));
+}
+
 void RecordOriginOfHLSPlayback(const GURL& origin_url) {
   if (media::GetMediaClient())
     GetMediaClient()->RecordRapporURL("Media.OriginUrl.HLS", origin_url);
@@ -206,7 +219,7 @@ class SetSinkIdCallback {
  private:
   // Mutable is required so that Pass() can be called in the copy
   // constructor.
-  mutable scoped_ptr<blink::WebSetSinkIdCallbacks> web_callback_;
+  mutable std::unique_ptr<blink::WebSetSinkIdCallbacks> web_callback_;
 };
 
 void RunSetSinkIdCallback(const SetSinkIdCallback& callback,
@@ -236,7 +249,7 @@ void RunSetSinkIdCallback(const SetSinkIdCallback& callback,
 
 }  // namespace
 
-SwitchOutputDeviceCB ConvertToSwitchOutputDeviceCB(
+OutputDeviceStatusCB ConvertToOutputDeviceStatusCB(
     blink::WebSetSinkIdCallbacks* web_callbacks) {
   return media::BindToCurrentLoop(
       base::Bind(RunSetSinkIdCallback, SetSinkIdCallback(web_callbacks)));
