@@ -4,9 +4,10 @@
 #ifndef MEDIA_CAST_CAST_SENDER_IMPL_H_
 #define MEDIA_CAST_CAST_SENDER_IMPL_H_
 
+#include <memory>
+
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "media/cast/cast_environment.h"
 #include "media/cast/cast_sender.h"
 #include "media/cast/sender/audio_sender.h"
@@ -24,7 +25,7 @@ class VideoSender;
 class CastSenderImpl : public CastSender {
  public:
   CastSenderImpl(scoped_refptr<CastEnvironment> cast_environment,
-                 CastTransportSender* const transport_sender);
+                 CastTransport* const transport_sender);
 
   void InitializeAudio(const AudioSenderConfig& audio_config,
                        const StatusChangeCallback& status_change_cb) final;
@@ -43,20 +44,20 @@ class CastSenderImpl : public CastSender {
   scoped_refptr<VideoFrameInput> video_frame_input() final;
 
  private:
-  void ReceivedPacket(scoped_ptr<Packet> packet);
+  void ReceivedPacket(std::unique_ptr<Packet> packet);
   void OnAudioStatusChange(const StatusChangeCallback& status_change_cb,
                            OperationalStatus status);
   void OnVideoStatusChange(const StatusChangeCallback& status_change_cb,
                            OperationalStatus status);
 
-  scoped_ptr<AudioSender> audio_sender_;
-  scoped_ptr<VideoSender> video_sender_;
+  std::unique_ptr<AudioSender> audio_sender_;
+  std::unique_ptr<VideoSender> video_sender_;
   scoped_refptr<AudioFrameInput> audio_frame_input_;
   scoped_refptr<VideoFrameInput> video_frame_input_;
   scoped_refptr<CastEnvironment> cast_environment_;
   // The transport sender is owned by the owner of the CastSender, and should be
   // valid throughout the lifetime of the CastSender.
-  CastTransportSender* const transport_sender_;
+  CastTransport* const transport_sender_;
 
   // NOTE: Weak pointers must be invalidated before all other member variables.
   base::WeakPtrFactory<CastSenderImpl> weak_factory_;
