@@ -11,21 +11,27 @@
 #include "crypto/symmetric_key.h"
 #include "media/cast/net/cast_transport_defines.h"
 
+namespace media {
+namespace cast {
+
 namespace {
 
 // Crypto.
 const size_t kAesBlockSize = 16;
 const size_t kAesKeySize = 16;
 
-std::string GetAesNonce(uint32_t frame_id, const std::string& iv_mask) {
+std::string GetAesNonce(FrameId frame_id, const std::string& iv_mask) {
+  DCHECK(!frame_id.is_null());
+
   std::string aes_nonce(kAesBlockSize, 0);
 
   // Serializing frame_id in big-endian order (aes_nonce[8] is the most
   // significant byte of frame_id).
-  aes_nonce[11] = frame_id & 0xff;
-  aes_nonce[10] = (frame_id >> 8) & 0xff;
-  aes_nonce[9] = (frame_id >> 16) & 0xff;
-  aes_nonce[8] = (frame_id >> 24) & 0xff;
+  const uint32_t truncated_id = frame_id.lower_32_bits();
+  aes_nonce[11] = truncated_id & 0xff;
+  aes_nonce[10] = (truncated_id >> 8) & 0xff;
+  aes_nonce[9] = (truncated_id >> 16) & 0xff;
+  aes_nonce[8] = (truncated_id >> 24) & 0xff;
 
   for (size_t i = 0; i < kAesBlockSize; ++i) {
     aes_nonce[i] ^= iv_mask[i];
@@ -35,9 +41,6 @@ std::string GetAesNonce(uint32_t frame_id, const std::string& iv_mask) {
 
 }  // namespace
 
-namespace media {
-namespace cast {
-
 TransportEncryptionHandler::TransportEncryptionHandler()
     : key_(), encryptor_(), iv_mask_(), is_activated_(false) {}
 
@@ -48,8 +51,7 @@ bool TransportEncryptionHandler::Initialize(const std::string& aes_key,
   is_activated_ = false;
   if (aes_iv_mask.size() == kAesKeySize && aes_key.size() == kAesKeySize) {
     iv_mask_ = aes_iv_mask;
-    key_.reset(
-        crypto::SymmetricKey::Import(crypto::SymmetricKey::AES, aes_key));
+    key_ = crypto::SymmetricKey::Import(crypto::SymmetricKey::AES, aes_key);
     encryptor_.reset(new crypto::Encryptor());
     encryptor_->Init(key_.get(), crypto::Encryptor::CTR, std::string());
     is_activated_ = true;
@@ -63,7 +65,7 @@ bool TransportEncryptionHandler::Initialize(const std::string& aes_key,
   return true;
 }
 
-bool TransportEncryptionHandler::Encrypt(uint32_t frame_id,
+bool TransportEncryptionHandler::Encrypt(FrameId frame_id,
                                          const base::StringPiece& data,
                                          std::string* encrypted_data) {
   if (!is_activated_)
@@ -79,7 +81,7 @@ bool TransportEncryptionHandler::Encrypt(uint32_t frame_id,
   return true;
 }
 
-bool TransportEncryptionHandler::Decrypt(uint32_t frame_id,
+bool TransportEncryptionHandler::Decrypt(FrameId frame_id,
                                          const base::StringPiece& ciphertext,
                                          std::string* plaintext) {
   if (!is_activated_) {
