@@ -55,8 +55,8 @@ TEST_F(FramerTest, AlwaysStartWithKey) {
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_TRUE(complete);
   EXPECT_FALSE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-  rtp_header_.frame_id = 1;
-  rtp_header_.reference_frame_id = 1;
+  rtp_header_.frame_id = FrameId::first() + 1;
+  rtp_header_.reference_frame_id = FrameId::first() + 1;
   rtp_header_.is_key_frame = true;
   complete = framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
@@ -65,8 +65,8 @@ TEST_F(FramerTest, AlwaysStartWithKey) {
   EXPECT_TRUE(next_frame);
   EXPECT_TRUE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(1u, frame.frame_id);
-  EXPECT_EQ(1u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 1, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 1, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 }
 
@@ -79,6 +79,8 @@ TEST_F(FramerTest, CompleteFrame) {
 
   // Start with a complete key frame.
   rtp_header_.is_key_frame = true;
+  rtp_header_.frame_id = FrameId::first();
+  rtp_header_.reference_frame_id = FrameId::first();
   complete = framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_TRUE(complete);
@@ -86,8 +88,8 @@ TEST_F(FramerTest, CompleteFrame) {
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(0u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first(), frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
   // Incomplete delta.
@@ -119,6 +121,8 @@ TEST_F(FramerTest, DuplicatePackets) {
 
   // Start with an incomplete key frame.
   rtp_header_.is_key_frame = true;
+  rtp_header_.frame_id = FrameId::first();
+  rtp_header_.reference_frame_id = FrameId::first();
   rtp_header_.max_packet_id = 1;
   duplicate = true;
   complete = framer_.InsertPacket(
@@ -145,7 +149,7 @@ TEST_F(FramerTest, DuplicatePackets) {
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
   EXPECT_FALSE(multiple);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
 
   // Add same packet again in complete key frame.
   duplicate = false;
@@ -155,9 +159,9 @@ TEST_F(FramerTest, DuplicatePackets) {
   EXPECT_TRUE(duplicate);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(0u, frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.frame_id);
   EXPECT_FALSE(multiple);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
   // Incomplete delta frame.
@@ -189,8 +193,8 @@ TEST_F(FramerTest, DuplicatePackets) {
   EXPECT_FALSE(duplicate);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(1u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 1, frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   EXPECT_FALSE(multiple);
 
   // Add same packet again in complete delta frame.
@@ -201,8 +205,8 @@ TEST_F(FramerTest, DuplicatePackets) {
   EXPECT_TRUE(duplicate);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(1u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 1, frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   EXPECT_FALSE(multiple);
 }
 
@@ -215,6 +219,8 @@ TEST_F(FramerTest, ContinuousSequence) {
 
   // Start with a complete key frame.
   rtp_header_.is_key_frame = true;
+  rtp_header_.frame_id = FrameId::first();
+  rtp_header_.reference_frame_id = FrameId::first();
   complete = framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_TRUE(complete);
@@ -222,12 +228,12 @@ TEST_F(FramerTest, ContinuousSequence) {
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(0u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first(), frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
   // Complete - not continuous.
-  rtp_header_.frame_id = 2;
+  rtp_header_.frame_id = FrameId::first() + 2;
   rtp_header_.reference_frame_id = rtp_header_.frame_id - 1;
   rtp_header_.is_key_frame = false;
   complete = framer_.InsertPacket(
@@ -236,73 +242,20 @@ TEST_F(FramerTest, ContinuousSequence) {
   EXPECT_FALSE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
 }
 
-TEST_F(FramerTest, Wrap) {
-  // Insert key frame, frame_id = 255 (will jump to that)
-  EncodedFrame frame;
-  bool next_frame = false;
-  bool multiple = true;
-  bool duplicate = false;
-
-  // Start with a complete key frame.
-  rtp_header_.is_key_frame = true;
-  rtp_header_.frame_id = 255;
-  rtp_header_.reference_frame_id = 255;
-  framer_.InsertPacket(
-      &payload_[0], payload_.size(), rtp_header_, &duplicate);
-  EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-  EXPECT_TRUE(next_frame);
-  EXPECT_FALSE(multiple);
-  EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(255u, frame.frame_id);
-  EXPECT_EQ(255u, frame.referenced_frame_id);
-  framer_.ReleaseFrame(frame.frame_id);
-
-  // Insert wrapped delta frame - should be continuous.
-  rtp_header_.is_key_frame = false;
-  rtp_header_.frame_id = 256;
-  framer_.InsertPacket(
-      &payload_[0], payload_.size(), rtp_header_, &duplicate);
-  EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-  EXPECT_TRUE(next_frame);
-  EXPECT_FALSE(multiple);
-  EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(256u, frame.frame_id);
-  EXPECT_EQ(255u, frame.referenced_frame_id);
-  framer_.ReleaseFrame(frame.frame_id);
-}
-
-TEST_F(FramerTest, Reset) {
-  EncodedFrame frame;
-  bool next_frame = false;
-  bool complete = false;
-  bool multiple = true;
-  bool duplicate = false;
-
-  // Start with a complete key frame.
-  rtp_header_.is_key_frame = true;
-  complete = framer_.InsertPacket(
-      &payload_[0], payload_.size(), rtp_header_, &duplicate);
-  EXPECT_TRUE(complete);
-  framer_.Reset();
-  EXPECT_FALSE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-}
-
-TEST_F(FramerTest, RequireKeyAfterReset) {
+TEST_F(FramerTest, RequireKeyFrameForFirstFrame) {
   EncodedFrame frame;
   bool next_frame = false;
   bool multiple = false;
   bool duplicate = false;
 
-  framer_.Reset();
-
   // Start with a complete key frame.
   rtp_header_.is_key_frame = false;
-  rtp_header_.frame_id = 0;
+  rtp_header_.frame_id = FrameId::first();
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_FALSE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-  rtp_header_.frame_id = 1;
-  rtp_header_.reference_frame_id = 1;
+  rtp_header_.frame_id = FrameId::first() + 1;
+  rtp_header_.reference_frame_id = FrameId::first() + 1;
   rtp_header_.is_key_frame = true;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
@@ -318,7 +271,8 @@ TEST_F(FramerTest, BasicNonLastReferenceId) {
   bool duplicate = false;
 
   rtp_header_.is_key_frame = true;
-  rtp_header_.frame_id = 0;
+  rtp_header_.frame_id = FrameId::first();
+  rtp_header_.reference_frame_id = FrameId::first();
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
@@ -327,8 +281,8 @@ TEST_F(FramerTest, BasicNonLastReferenceId) {
   framer_.ReleaseFrame(frame.frame_id);
 
   rtp_header_.is_key_frame = false;
-  rtp_header_.reference_frame_id = 0;
-  rtp_header_.frame_id = 5;
+  rtp_header_.frame_id = FrameId::first() + 5;
+  rtp_header_.reference_frame_id = FrameId::first();
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
@@ -345,51 +299,52 @@ TEST_F(FramerTest, InOrderReferenceFrameSelection) {
   bool duplicate = false;
 
   rtp_header_.is_key_frame = true;
-  rtp_header_.frame_id = 0;
+  rtp_header_.frame_id = FrameId::first();
+  rtp_header_.reference_frame_id = FrameId::first();
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   rtp_header_.is_key_frame = false;
-  rtp_header_.frame_id = 1;
+  rtp_header_.frame_id = FrameId::first() + 1;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
   // Insert frame #2 partially.
-  rtp_header_.frame_id = 2;
+  rtp_header_.frame_id = FrameId::first() + 2;
   rtp_header_.max_packet_id = 1;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
-  rtp_header_.frame_id = 4;
+  rtp_header_.frame_id = FrameId::first() + 4;
   rtp_header_.max_packet_id = 0;
-  rtp_header_.reference_frame_id = 0;
+  rtp_header_.reference_frame_id = FrameId::first();
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(0u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first(), frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   EXPECT_FALSE(multiple);
   framer_.ReleaseFrame(frame.frame_id);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_TRUE(next_frame);
   EXPECT_TRUE(multiple);
   EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(1u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 1, frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_FALSE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(4u, frame.frame_id);
-  EXPECT_EQ(0u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 4, frame.frame_id);
+  EXPECT_EQ(FrameId::first(), frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
   // Insert remaining packet of frame #2 - should no be continuous.
-  rtp_header_.frame_id = 2;
+  rtp_header_.frame_id = FrameId::first() + 2;
   rtp_header_.packet_id = 1;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_FALSE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
-  rtp_header_.frame_id = 5;
+  rtp_header_.frame_id = FrameId::first() + 5;
   rtp_header_.reference_frame_id = rtp_header_.frame_id - 1;
   rtp_header_.packet_id = 0;
   rtp_header_.max_packet_id = 0;
@@ -399,20 +354,19 @@ TEST_F(FramerTest, InOrderReferenceFrameSelection) {
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::DEPENDENT, frame.dependency);
-  EXPECT_EQ(5u, frame.frame_id);
-  EXPECT_EQ(4u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 5, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 4, frame.referenced_frame_id);
 }
 
-TEST_F(FramerTest, AudioWrap) {
-  // All audio frames are marked as key frames.
+TEST_F(FramerTest, ReleasesAllReceivedKeyFramesInContinuousSequence) {
   EncodedFrame frame;
   bool next_frame = false;
   bool multiple = false;
   bool duplicate = false;
 
   rtp_header_.is_key_frame = true;
-  rtp_header_.frame_id = 254;
-  rtp_header_.reference_frame_id = 254;
+  rtp_header_.frame_id = FrameId::first() + 254;
+  rtp_header_.reference_frame_id = FrameId::first() + 254;
 
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
@@ -420,18 +374,18 @@ TEST_F(FramerTest, AudioWrap) {
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(254u, frame.frame_id);
-  EXPECT_EQ(254u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 254, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 254, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
-  rtp_header_.frame_id = 255;
-  rtp_header_.reference_frame_id = 255;
+  rtp_header_.frame_id = FrameId::first() + 255;
+  rtp_header_.reference_frame_id = FrameId::first() + 255;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
   // Insert wrapped frame - should be continuous.
-  rtp_header_.frame_id = 256;
-  rtp_header_.reference_frame_id = 256;
+  rtp_header_.frame_id = FrameId::first() + 256;
+  rtp_header_.reference_frame_id = FrameId::first() + 256;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
@@ -439,65 +393,63 @@ TEST_F(FramerTest, AudioWrap) {
   EXPECT_TRUE(next_frame);
   EXPECT_TRUE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(255u, frame.frame_id);
-  EXPECT_EQ(255u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 255, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 255, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(256u, frame.frame_id);
-  EXPECT_EQ(256u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 256, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 256, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 }
 
-TEST_F(FramerTest, AudioWrapWithMissingFrame) {
-  // All audio frames are marked as key frames.
+TEST_F(FramerTest, SkipsMissingFramesWhenLaterKeyFramesAreAvailable) {
   EncodedFrame frame;
   bool next_frame = false;
   bool multiple = true;
   bool duplicate = false;
 
-  // Insert and get first packet.
+  // Insert and get first frame's packet.
   rtp_header_.is_key_frame = true;
-  rtp_header_.frame_id = 253;
-  rtp_header_.reference_frame_id = 253;
+  rtp_header_.frame_id = FrameId::first() + 253;
+  rtp_header_.reference_frame_id = FrameId::first() + 253;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(253u, frame.frame_id);
-  EXPECT_EQ(253u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 253, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 253, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 
-  // Insert third and fourth packets.
-  rtp_header_.frame_id = 255;
-  rtp_header_.reference_frame_id = 255;
+  // Insert third and fourth frames' packet.
+  rtp_header_.frame_id = FrameId::first() + 255;
+  rtp_header_.reference_frame_id = FrameId::first() + 255;
   framer_.InsertPacket(
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
-  rtp_header_.frame_id = 256;
-  rtp_header_.reference_frame_id = 256;
+  rtp_header_.frame_id = FrameId::first() + 256;
+  rtp_header_.reference_frame_id = FrameId::first() + 256;
   framer_.InsertPacket(
-
       &payload_[0], payload_.size(), rtp_header_, &duplicate);
 
-  // Get third and fourth packets.
+  // Get third and fourth frame.
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_FALSE(next_frame);
   EXPECT_TRUE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(255u, frame.frame_id);
-  EXPECT_EQ(255u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 255, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 255, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
   EXPECT_TRUE(framer_.GetEncodedFrame(&frame, &next_frame, &multiple));
   EXPECT_TRUE(next_frame);
   EXPECT_FALSE(multiple);
   EXPECT_EQ(EncodedFrame::KEY, frame.dependency);
-  EXPECT_EQ(256u, frame.frame_id);
-  EXPECT_EQ(256u, frame.referenced_frame_id);
+  EXPECT_EQ(FrameId::first() + 256, frame.frame_id);
+  EXPECT_EQ(FrameId::first() + 256, frame.referenced_frame_id);
   framer_.ReleaseFrame(frame.frame_id);
 }
 
