@@ -7,13 +7,14 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/test/simple_test_tick_clock.h"
+#include "media/base/fake_single_thread_task_runner.h"
 #include "media/cast/net/pacing/paced_sender.h"
 #include "media/cast/net/rtp/packet_storage.h"
 #include "media/cast/net/rtp/rtp_parser.h"
-#include "media/cast/test/fake_single_thread_task_runner.h"
 #include "testing/gmock/include/gmock/gmock.h"
 
 namespace media {
@@ -28,7 +29,7 @@ static const int kSsrc = 0x12345;
 static const unsigned int kFrameSize = 5000;
 }
 
-class TestRtpPacketTransport : public PacketSender {
+class TestRtpPacketTransport : public PacketTransport {
  public:
   explicit TestRtpPacketTransport(RtpPacketizerConfig config)
       : config_(config),
@@ -36,7 +37,7 @@ class TestRtpPacketTransport : public PacketSender {
         packets_sent_(0),
         expected_number_of_packets_(0),
         expected_packet_id_(0),
-        expected_frame_id_(0) {}
+        expected_frame_id_(FrameId::first() + 1) {}
 
   void VerifyRtpHeader(const RtpCastHeader& rtp_header) {
     VerifyCommonRtpHeader(rtp_header);
@@ -57,7 +58,7 @@ class TestRtpPacketTransport : public PacketSender {
     EXPECT_EQ(expected_packet_id_, rtp_header.packet_id);
     EXPECT_EQ(expected_number_of_packets_ - 1, rtp_header.max_packet_id);
     EXPECT_TRUE(rtp_header.is_reference);
-    EXPECT_EQ(expected_frame_id_ - 1u, rtp_header.reference_frame_id);
+    EXPECT_EQ(expected_frame_id_ - 1, rtp_header.reference_frame_id);
     if (rtp_header.packet_id != 0) {
       EXPECT_EQ(rtp_header.num_extensions, 0)
           << "Extensions only allowed on first packet of a frame";
@@ -80,6 +81,11 @@ class TestRtpPacketTransport : public PacketSender {
 
   int64_t GetBytesSent() final { return 0; }
 
+  void StartReceiving(
+      const PacketReceiverCallbackWithStatus& packet_receiver) final {}
+
+  void StopReceiving() final {}
+
   size_t number_of_packets_received() const { return packets_sent_; }
 
   void set_expected_number_of_packets(size_t expected_number_of_packets) {
@@ -97,7 +103,7 @@ class TestRtpPacketTransport : public PacketSender {
   size_t expected_number_of_packets_;
   // Assuming packets arrive in sequence.
   int expected_packet_id_;
-  uint32_t expected_frame_id_;
+  FrameId expected_frame_id_;
   RtpTimeTicks expected_rtp_timestamp_;
 
  private:
@@ -107,7 +113,7 @@ class TestRtpPacketTransport : public PacketSender {
 class RtpPacketizerTest : public ::testing::Test {
  protected:
   RtpPacketizerTest()
-      : task_runner_(new test::FakeSingleThreadTaskRunner(&testing_clock_)) {
+      : task_runner_(new FakeSingleThreadTaskRunner(&testing_clock_)) {
     config_.sequence_number = kSeqNum;
     config_.ssrc = kSsrc;
     config_.payload_type = kPayload;
@@ -120,7 +126,7 @@ class RtpPacketizerTest : public ::testing::Test {
     rtp_packetizer_.reset(new RtpPacketizer(
         pacer_.get(), &packet_storage_, config_));
     video_frame_.dependency = EncodedFrame::DEPENDENT;
-    video_frame_.frame_id = 0;
+    video_frame_.frame_id = FrameId::first() + 1;
     video_frame_.referenced_frame_id = video_frame_.frame_id - 1;
     video_frame_.data.assign(kFrameSize, 123);
     video_frame_.rtp_timestamp = RtpTimeTicks().Expand(UINT32_C(0x0055aa11));
@@ -135,13 +141,13 @@ class RtpPacketizerTest : public ::testing::Test {
   }
 
   base::SimpleTestTickClock testing_clock_;
-  scoped_refptr<test::FakeSingleThreadTaskRunner> task_runner_;
+  scoped_refptr<FakeSingleThreadTaskRunner> task_runner_;
   EncodedFrame video_frame_;
   PacketStorage packet_storage_;
   RtpPacketizerConfig config_;
-  scoped_ptr<TestRtpPacketTransport> transport_;
-  scoped_ptr<PacedSender> pacer_;
-  scoped_ptr<RtpPacketizer> rtp_packetizer_;
+  std::unique_ptr<TestRtpPacketTransport> transport_;
+  std::unique_ptr<PacedSender> pacer_;
+  std::unique_ptr<RtpPacketizer> rtp_packetizer_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(RtpPacketizerTest);
