@@ -11,6 +11,7 @@
 #include "base/macros.h"
 #include "base/message_loop/message_loop.h"
 #include "base/power_monitor/power_monitor.h"
+#include "base/run_loop.h"
 #include "base/test/launcher/unit_test_launcher.h"
 #include "base/test/power_monitor_test_base.h"
 #include "base/test/simple_test_tick_clock.h"
@@ -87,8 +88,8 @@ class MetadataRecorder : public base::RefCountedThreadSafe<MetadataRecorder> {
 
   int count_frames_delivered() const { return count_frames_delivered_; }
 
-  void PushExpectation(uint32_t expected_frame_id,
-                       uint32_t expected_last_referenced_frame_id,
+  void PushExpectation(FrameId expected_frame_id,
+                       FrameId expected_last_referenced_frame_id,
                        RtpTimeTicks expected_rtp_timestamp,
                        const base::TimeTicks& expected_reference_time) {
     expectations_.push(Expectation{expected_frame_id,
@@ -97,7 +98,8 @@ class MetadataRecorder : public base::RefCountedThreadSafe<MetadataRecorder> {
                                    expected_reference_time});
   }
 
-  void CompareFrameWithExpected(scoped_ptr<SenderEncodedFrame> encoded_frame) {
+  void CompareFrameWithExpected(
+      std::unique_ptr<SenderEncodedFrame> encoded_frame) {
     ASSERT_LT(0u, expectations_.size());
     auto e = expectations_.front();
     expectations_.pop();
@@ -123,8 +125,8 @@ class MetadataRecorder : public base::RefCountedThreadSafe<MetadataRecorder> {
   int count_frames_delivered_;
 
   struct Expectation {
-    uint32_t expected_frame_id;
-    uint32_t expected_last_referenced_frame_id;
+    FrameId expected_frame_id;
+    FrameId expected_last_referenced_frame_id;
     RtpTimeTicks expected_rtp_timestamp;
     base::TimeTicks expected_reference_time;
   };
@@ -140,11 +142,11 @@ class EndToEndFrameChecker
       : decoder_(), count_frames_checked_(0) {
     bool decoder_init_result;
     decoder_.Initialize(
-        config, false, media::SetCdmReadyCB(),
+        config, false, nullptr,
         base::Bind(&SaveDecoderInitResult, &decoder_init_result),
         base::Bind(&EndToEndFrameChecker::CompareFrameWithExpected,
                    base::Unretained(this)));
-    base::MessageLoop::current()->RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     EXPECT_TRUE(decoder_init_result);
   }
 
@@ -152,7 +154,7 @@ class EndToEndFrameChecker
     expectations_.push(frame);
   }
 
-  void EncodeDone(scoped_ptr<SenderEncodedFrame> encoded_frame) {
+  void EncodeDone(std::unique_ptr<SenderEncodedFrame> encoded_frame) {
     auto buffer = DecoderBuffer::CopyFrom(encoded_frame->bytes(),
                                           encoded_frame->data.size());
     decoder_.Decode(buffer, base::Bind(&EndToEndFrameChecker::DecodeDone,
@@ -167,9 +169,7 @@ class EndToEndFrameChecker
     ++count_frames_checked_;
   }
 
-  void DecodeDone(VideoDecoder::Status status) {
-    EXPECT_EQ(VideoDecoder::kOk, status);
-  }
+  void DecodeDone(DecodeStatus status) { EXPECT_EQ(DecodeStatus::OK, status); }
 
   int count_frames_checked() const { return count_frames_checked_; }
 
@@ -200,18 +200,17 @@ void CreateFrameAndMemsetPlane(VideoFrameFactory* const video_frame_factory) {
 }
 
 void NoopFrameEncodedCallback(
-    scoped_ptr<media::cast::SenderEncodedFrame> /*encoded_frame*/) {
-}
+    std::unique_ptr<media::cast::SenderEncodedFrame> /*encoded_frame*/) {}
 
 class TestPowerSource : public base::PowerMonitorSource {
  public:
   void GenerateSuspendEvent() {
     ProcessPowerEvent(SUSPEND_EVENT);
-    base::MessageLoop::current()->RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
   void GenerateResumeEvent() {
     ProcessPowerEvent(RESUME_EVENT);
-    base::MessageLoop::current()->RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
  private:
@@ -227,22 +226,22 @@ class H264VideoToolboxEncoderTest : public ::testing::Test {
     clock_->Advance(base::TimeTicks::Now() - base::TimeTicks());
 
     power_source_ = new TestPowerSource();
-    power_monitor_.reset(
-        new base::PowerMonitor(scoped_ptr<TestPowerSource>(power_source_)));
+    power_monitor_.reset(new base::PowerMonitor(
+        std::unique_ptr<TestPowerSource>(power_source_)));
 
     cast_environment_ = new CastEnvironment(
-        scoped_ptr<base::TickClock>(clock_), message_loop_.task_runner(),
+        std::unique_ptr<base::TickClock>(clock_), message_loop_.task_runner(),
         message_loop_.task_runner(), message_loop_.task_runner());
     encoder_.reset(new H264VideoToolboxEncoder(
         cast_environment_, video_sender_config_,
         base::Bind(&SaveOperationalStatus, &operational_status_)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     EXPECT_EQ(STATUS_INITIALIZED, operational_status_);
   }
 
   void TearDown() final {
     encoder_.reset();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     power_monitor_.reset();
   }
 
@@ -270,10 +269,10 @@ class H264VideoToolboxEncoderTest : public ::testing::Test {
   base::SimpleTestTickClock* clock_;  // Owned by CastEnvironment.
   base::MessageLoop message_loop_;
   scoped_refptr<CastEnvironment> cast_environment_;
-  scoped_ptr<VideoEncoder> encoder_;
+  std::unique_ptr<VideoEncoder> encoder_;
   OperationalStatus operational_status_;
   TestPowerSource* power_source_;  // Owned by the power monitor.
-  scoped_ptr<base::PowerMonitor> power_monitor_;
+  std::unique_ptr<base::PowerMonitor> power_monitor_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(H264VideoToolboxEncoderTest);
@@ -289,12 +288,14 @@ TEST_F(H264VideoToolboxEncoderTest, CheckFrameMetadataSequence) {
       &MetadataRecorder::CompareFrameWithExpected, metadata_recorder.get());
 
   metadata_recorder->PushExpectation(
-      0, 0, RtpTimeTicks::FromTimeDelta(frame_->timestamp(), kVideoFrequency),
+      FrameId::first(), FrameId::first(),
+      RtpTimeTicks::FromTimeDelta(frame_->timestamp(), kVideoFrequency),
       clock_->NowTicks());
   EXPECT_TRUE(encoder_->EncodeVideoFrame(frame_, clock_->NowTicks(), cb));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
-  for (uint32_t frame_id = 1; frame_id < 10; ++frame_id) {
+  for (FrameId frame_id = FrameId::first() + 1;
+       frame_id < FrameId::first() + 10; ++frame_id) {
     AdvanceClockAndVideoFrameTimestamp();
     metadata_recorder->PushExpectation(
         frame_id, frame_id - 1,
@@ -304,7 +305,7 @@ TEST_F(H264VideoToolboxEncoderTest, CheckFrameMetadataSequence) {
   }
 
   encoder_.reset();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
 
   EXPECT_EQ(10, metadata_recorder->count_frames_delivered());
 }
@@ -314,12 +315,13 @@ TEST_F(H264VideoToolboxEncoderTest, CheckFramesAreDecodable) {
   VideoDecoderConfig config(kCodecH264, H264PROFILE_MAIN, frame_->format(),
                             COLOR_SPACE_UNSPECIFIED, frame_->coded_size(),
                             frame_->visible_rect(), frame_->natural_size(),
-                            EmptyExtraData(), false);
+                            EmptyExtraData(), Unencrypted());
   scoped_refptr<EndToEndFrameChecker> checker(new EndToEndFrameChecker(config));
 
   VideoEncoder::FrameEncodedCallback cb =
       base::Bind(&EndToEndFrameChecker::EncodeDone, checker.get());
-  for (uint32_t frame_id = 0; frame_id < 6; ++frame_id) {
+  for (FrameId frame_id = FrameId::first(); frame_id < FrameId::first() + 6;
+       ++frame_id) {
     checker->PushExpectation(frame_);
     EXPECT_TRUE(encoder_->EncodeVideoFrame(frame_, clock_->NowTicks(), cb));
     AdvanceClockAndVideoFrameTimestamp();
@@ -341,7 +343,7 @@ TEST_F(H264VideoToolboxEncoderTest, CheckVideoFrameFactory) {
   // request a frame again.
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   CreateFrameAndMemsetPlane(video_frame_factory.get());
 }
 
@@ -377,7 +379,7 @@ TEST_F(H264VideoToolboxEncoderTest, CheckPowerMonitoringVideoFrameFactory) {
   // request a frame again.
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   CreateFrameAndMemsetPlane(video_frame_factory.get());
 
   // After a power suspension, the factory should not produce frames.
@@ -385,7 +387,7 @@ TEST_F(H264VideoToolboxEncoderTest, CheckPowerMonitoringVideoFrameFactory) {
 
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
 
@@ -406,7 +408,7 @@ TEST_F(H264VideoToolboxEncoderTest,
 
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   ASSERT_FALSE(video_frame_factory->MaybeCreateFrame(
       gfx::Size(kVideoWidth, kVideoHeight), base::TimeDelta()));
 
