@@ -21,7 +21,7 @@ SizeAdaptableVideoEncoderBase::SizeAdaptableVideoEncoderBase(
       video_config_(video_config),
       status_change_cb_(status_change_cb),
       frames_in_encoder_(0),
-      last_frame_id_(kFirstFrameId - 1),
+      next_frame_id_(FrameId::first()),
       weak_factory_(this) {
   cast_environment_->PostTask(
       CastEnvironment::MAIN,
@@ -79,8 +79,8 @@ void SizeAdaptableVideoEncoderBase::GenerateKeyFrame() {
     encoder_->GenerateKeyFrame();
 }
 
-scoped_ptr<VideoFrameFactory>
-    SizeAdaptableVideoEncoderBase::CreateVideoFrameFactory() {
+std::unique_ptr<VideoFrameFactory>
+SizeAdaptableVideoEncoderBase::CreateVideoFrameFactory() {
   DCHECK(cast_environment_->CurrentlyOn(CastEnvironment::MAIN));
   return nullptr;
 }
@@ -152,11 +152,13 @@ void SizeAdaptableVideoEncoderBase::OnEncoderStatusChange(
 
 void SizeAdaptableVideoEncoderBase::OnEncodedVideoFrame(
     const FrameEncodedCallback& frame_encoded_callback,
-    scoped_ptr<SenderEncodedFrame> encoded_frame) {
+    std::unique_ptr<SenderEncodedFrame> encoded_frame) {
   DCHECK(cast_environment_->CurrentlyOn(CastEnvironment::MAIN));
   --frames_in_encoder_;
   DCHECK_GE(frames_in_encoder_, 0);
-  last_frame_id_ = encoded_frame->frame_id;
+
+  if (encoded_frame)
+    next_frame_id_ = encoded_frame->frame_id + 1;
   frame_encoded_callback.Run(std::move(encoded_frame));
 }
 
