@@ -10,9 +10,8 @@
 #include "base/logging.h"
 #include "base/strings/stringprintf.h"
 #include "media/base/video_frame.h"
-#include "media/cast/cast_defines.h"
 #include "media/cast/constants.h"
-#include "third_party/libvpx_new/source/libvpx/vpx/vp8cx.h"
+#include "third_party/libvpx/source/libvpx/vpx/vp8cx.h"
 
 namespace media {
 namespace cast {
@@ -32,16 +31,16 @@ const int kRestartFramePeriods = 3;
 // The smaller, the shorter of the time averaging window.
 const int kEncodingSpeedAccHalfLife = 120000;  // 0.12 second.
 
-// The target deadline utilization signal. This is a trade-off between quality
+// The target encoder utilization signal. This is a trade-off between quality
 // and less CPU usage. The range of this value is [0, 1]. Higher the value,
 // better the quality and higher the CPU usage.
 //
 // For machines with more than two encoding threads.
-const double kHiTargetDeadlineUtilization = 0.7;
+const double kHiTargetEncoderUtilization = 0.7;
 // For machines with two encoding threads.
-const double kMidTargetDeadlineUtilization = 0.6;
+const double kMidTargetEncoderUtilization = 0.6;
 // For machines with single encoding thread.
-const double kLoTargetDeadlineUtilization = 0.5;
+const double kLoTargetEncoderUtilization = 0.5;
 
 // This is the equivalent change on encoding speed for the change on each
 // quantizer step.
@@ -66,15 +65,15 @@ bool HasSufficientFeedback(
 
 Vp8Encoder::Vp8Encoder(const VideoSenderConfig& video_config)
     : cast_config_(video_config),
-      target_deadline_utilization_(
+      target_encoder_utilization_(
           video_config.number_of_encode_threads > 2
-              ? kHiTargetDeadlineUtilization
+              ? kHiTargetEncoderUtilization
               : (video_config.number_of_encode_threads > 1
-                     ? kMidTargetDeadlineUtilization
-                     : kLoTargetDeadlineUtilization)),
+                     ? kMidTargetEncoderUtilization
+                     : kLoTargetEncoderUtilization)),
       key_frame_requested_(true),
       bitrate_kbit_(cast_config_.start_bitrate / 1000),
-      last_encoded_frame_id_(kFirstFrameId - 1),
+      next_frame_id_(FrameId::first()),
       has_seen_zero_length_encoded_frame_(false),
       encoding_speed_acc_(
           base::TimeDelta::FromMicroseconds(kEncodingSpeedAccHalfLife)),
@@ -193,7 +192,7 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
   DCHECK(thread_checker_.CalledOnValidThread());
   DCHECK(encoded_frame);
 
-  // Note: This is used to compute the |deadline_utilization| and so it uses the
+  // Note: This is used to compute the |encoder_utilization| and so it uses the
   // real-world clock instead of the CastEnvironment clock, the latter of which
   // might be simulated.
   const base::TimeTicks start_time = base::TimeTicks::Now();
@@ -262,7 +261,7 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
       << "BUG: Invalid arguments passed to vpx_codec_encode().";
 
   // Pull data from the encoder, populating a new EncodedFrame.
-  encoded_frame->frame_id = ++last_encoded_frame_id_;
+  encoded_frame->frame_id = next_frame_id_++;
   const vpx_codec_cx_pkt_t* pkt = NULL;
   vpx_codec_iter_t iter = NULL;
   while ((pkt = vpx_codec_get_cx_data(&encoder_, &iter)) != NULL) {
@@ -277,7 +276,7 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
       // Frame dependencies could theoretically be relaxed by looking for the
       // VPX_FRAME_IS_DROPPABLE flag, but in recent testing (Oct 2014), this
       // flag never seems to be set.
-      encoded_frame->referenced_frame_id = last_encoded_frame_id_ - 1;
+      encoded_frame->referenced_frame_id = encoded_frame->frame_id - 1;
     }
     encoded_frame->rtp_timestamp =
         RtpTimeTicks::FromTimeDelta(video_frame->timestamp(), kVideoFrequency);
@@ -299,7 +298,8 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
     const std::string details = base::StringPrintf(
         "SV/%c,id=%" PRIu32 ",rtp=%" PRIu32 ",br=%d,kfr=%c",
         encoded_frame->dependency == EncodedFrame::KEY ? 'K' : 'D',
-        encoded_frame->frame_id, encoded_frame->rtp_timestamp.lower_32_bits(),
+        encoded_frame->frame_id.lower_32_bits(),
+        encoded_frame->rtp_timestamp.lower_32_bits(),
         static_cast<int>(config_.rc_target_bitrate),
         key_frame_requested_ ? 'Y' : 'N');
     base::debug::SetCrashKeyValue(kZeroEncodeDetails, details);
@@ -308,10 +308,10 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
     base::debug::ClearCrashKey(kZeroEncodeDetails);
   }
 
-  // Compute deadline utilization as the real-world time elapsed divided by the
+  // Compute encoder utilization as the real-world time elapsed divided by the
   // frame duration.
   const base::TimeDelta processing_time = base::TimeTicks::Now() - start_time;
-  encoded_frame->deadline_utilization =
+  encoded_frame->encoder_utilization =
       processing_time.InSecondsF() / predicted_frame_duration.InSecondsF();
 
   // Compute lossy utilization.  The VP8 encoder took an estimated guess at what
@@ -336,7 +336,7 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
 
   DVLOG(2) << "VP8 encoded frame_id " << encoded_frame->frame_id
            << ", sized: " << encoded_frame->data.size()
-           << ", deadline_utilization: " << encoded_frame->deadline_utilization
+           << ", encoder_utilization: " << encoded_frame->encoder_utilization
            << ", lossy_utilization: " << encoded_frame->lossy_utilization
            << " (quantizer chosen by the encoder was " << quantizer << ')';
 
@@ -353,8 +353,8 @@ void Vp8Encoder::Encode(const scoped_refptr<media::VideoFrame>& video_frame,
         kEquivalentEncodingSpeedStepPerQpStep *
             std::max(0, quantizer - cast_config_.min_qp);
     double adjusted_encoding_speed = actual_encoding_speed *
-                                     encoded_frame->deadline_utilization /
-                                     target_deadline_utilization_;
+                                     encoded_frame->encoder_utilization /
+                                     target_encoder_utilization_;
     encoding_speed_acc_.Update(adjusted_encoding_speed,
                                video_frame->timestamp());
   }
