@@ -17,9 +17,9 @@
 #include "media/base/audio_decoder_config.h"
 #include "media/base/audio_timestamp_helper.h"
 #include "media/base/bind_to_current_loop.h"
+#include "media/base/cdm_context.h"
 #include "media/base/decoder_buffer.h"
 #include "media/base/media_log.h"
-#include "media/base/pipeline.h"
 #include "media/base/timestamp_constants.h"
 
 namespace media {
@@ -50,7 +50,7 @@ std::string DecryptingAudioDecoder::GetDisplayName() const {
 }
 
 void DecryptingAudioDecoder::Initialize(const AudioDecoderConfig& config,
-                                        const SetCdmReadyCB& set_cdm_ready_cb,
+                                        CdmContext* cdm_context,
                                         const InitCB& init_cb,
                                         const OutputCB& output_cb) {
   DVLOG(2) << "Initialize()";
@@ -62,6 +62,8 @@ void DecryptingAudioDecoder::Initialize(const AudioDecoderConfig& config,
   init_cb_ = BindToCurrentLoop(init_cb);
   output_cb_ = BindToCurrentLoop(output_cb);
 
+  // TODO(xhwang): We should be able to DCHECK config.IsValidConfig() and
+  // config.is_encrypted().
   if (!config.IsValidConfig()) {
     DLOG(ERROR) << "Invalid audio stream config.";
     base::ResetAndReturn(&init_cb_).Run(false);
@@ -77,16 +79,19 @@ void DecryptingAudioDecoder::Initialize(const AudioDecoderConfig& config,
   config_ = config;
 
   if (state_ == kUninitialized) {
-    DCHECK(!set_cdm_ready_cb.is_null());
-    state_ = kDecryptorRequested;
-    set_cdm_ready_cb_ = set_cdm_ready_cb;
-    set_cdm_ready_cb_.Run(BindToCurrentLoop(
-        base::Bind(&DecryptingAudioDecoder::SetCdm, weak_this_)));
-    return;
+    DCHECK(cdm_context);
+    if (!cdm_context->GetDecryptor()) {
+      MEDIA_LOG(DEBUG, media_log_) << GetDisplayName() << ": no decryptor";
+      base::ResetAndReturn(&init_cb_).Run(false);
+      return;
+    }
+
+    decryptor_ = cdm_context->GetDecryptor();
+  } else {
+    // Reinitialization (i.e. upon a config change)
+    decryptor_->DeinitializeDecoder(Decryptor::kAudio);
   }
 
-  // Reinitialization (i.e. upon a config change)
-  decryptor_->DeinitializeDecoder(Decryptor::kAudio);
   InitializeDecoder();
 }
 
@@ -103,7 +108,7 @@ void DecryptingAudioDecoder::Decode(const scoped_refptr<DecoderBuffer>& buffer,
   // Return empty (end-of-stream) frames if decoding has finished.
   if (state_ == kDecodeFinished) {
     output_cb_.Run(AudioBuffer::CreateEOSBuffer());
-    base::ResetAndReturn(&decode_cb_).Run(kOk);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::OK);
     return;
   }
 
@@ -145,7 +150,7 @@ void DecryptingAudioDecoder::Reset(const base::Closure& closure) {
   if (state_ == kWaitingForKey) {
     DCHECK(!decode_cb_.is_null());
     pending_buffer_to_decode_ = NULL;
-    base::ResetAndReturn(&decode_cb_).Run(kAborted);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::ABORTED);
   }
 
   DCHECK(decode_cb_.is_null());
@@ -163,41 +168,15 @@ DecryptingAudioDecoder::~DecryptingAudioDecoder() {
     decryptor_->DeinitializeDecoder(Decryptor::kAudio);
     decryptor_ = NULL;
   }
-  if (!set_cdm_ready_cb_.is_null())
-    base::ResetAndReturn(&set_cdm_ready_cb_).Run(CdmReadyCB());
   pending_buffer_to_decode_ = NULL;
   if (!init_cb_.is_null())
     base::ResetAndReturn(&init_cb_).Run(false);
   if (!decode_cb_.is_null())
-    base::ResetAndReturn(&decode_cb_).Run(kAborted);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::ABORTED);
   if (!reset_cb_.is_null())
     base::ResetAndReturn(&reset_cb_).Run();
 }
 
-void DecryptingAudioDecoder::SetCdm(CdmContext* cdm_context,
-                                    const CdmAttachedCB& cdm_attached_cb) {
-  DVLOG(2) << __FUNCTION__;
-  DCHECK(task_runner_->BelongsToCurrentThread());
-  DCHECK_EQ(state_, kDecryptorRequested) << state_;
-  DCHECK(!init_cb_.is_null());
-  DCHECK(!set_cdm_ready_cb_.is_null());
-
-  set_cdm_ready_cb_.Reset();
-
-  if (!cdm_context || !cdm_context->GetDecryptor()) {
-    MEDIA_LOG(DEBUG, media_log_) << GetDisplayName() << ": no decryptor set";
-    base::ResetAndReturn(&init_cb_).Run(false);
-    state_ = kError;
-    cdm_attached_cb.Run(false);
-    return;
-  }
-
-  decryptor_ = cdm_context->GetDecryptor();
-
-  InitializeDecoder();
-  cdm_attached_cb.Run(true);
-}
-
 void DecryptingAudioDecoder::InitializeDecoder() {
   state_ = kPendingDecoderInit;
   decryptor_->InitializeAudioDecoder(
@@ -269,7 +248,7 @@ void DecryptingAudioDecoder::DeliverFrame(
   pending_buffer_to_decode_ = NULL;
 
   if (!reset_cb_.is_null()) {
-    base::ResetAndReturn(&decode_cb_).Run(kAborted);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::ABORTED);
     DoReset();
     return;
   }
@@ -280,7 +259,7 @@ void DecryptingAudioDecoder::DeliverFrame(
     DVLOG(2) << "DeliverFrame() - kError";
     MEDIA_LOG(ERROR, media_log_) << GetDisplayName() << ": decode error";
     state_ = kDecodeFinished; // TODO add kError state
-    base::ResetAndReturn(&decode_cb_).Run(kDecodeError);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::DECODE_ERROR);
     return;
   }
 
@@ -307,7 +286,7 @@ void DecryptingAudioDecoder::DeliverFrame(
     DVLOG(2) << "DeliverFrame() - kNeedMoreData";
     state_ = scoped_pending_buffer_to_decode->end_of_stream() ? kDecodeFinished
                                                               : kIdle;
-    base::ResetAndReturn(&decode_cb_).Run(kOk);
+    base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::OK);
     return;
   }
 
@@ -324,7 +303,7 @@ void DecryptingAudioDecoder::DeliverFrame(
   }
 
   state_ = kIdle;
-  base::ResetAndReturn(&decode_cb_).Run(kOk);
+  base::ResetAndReturn(&decode_cb_).Run(DecodeStatus::OK);
 }
 
 void DecryptingAudioDecoder::OnKeyAdded() {
