@@ -6,12 +6,21 @@
 
 #include "base/callback_helpers.h"
 #include "base/stl_util.h"
+#include "media/base/media_track.h"
+#include "media/base/media_tracks.h"
 #include "media/filters/chunk_demuxer.h"
 #include "media/filters/frame_processor.h"
 #include "media/filters/source_buffer_stream.h"
 
 namespace media {
 
+enum {
+  // Limits the number of MEDIA_LOG() calls warning the user that a muxed stream
+  // media segment is missing a block from at least one of the audio or video
+  // tracks.
+  kMaxMissingTrackInSegmentLogs = 10,
+};
+
 static TimeDelta EndTimestamp(const StreamParser::BufferQueue& queue) {
   return queue.back()->timestamp() + queue.back()->duration();
 }
@@ -77,19 +86,21 @@ Ranges<TimeDelta> MediaSourceState::ComputeRangesIntersection(
 }
 
 MediaSourceState::MediaSourceState(
-    scoped_ptr<StreamParser> stream_parser,
-    scoped_ptr<FrameProcessor> frame_processor,
+    std::unique_ptr<StreamParser> stream_parser,
+    std::unique_ptr<FrameProcessor> frame_processor,
     const CreateDemuxerStreamCB& create_demuxer_stream_cb,
     const scoped_refptr<MediaLog>& media_log)
     : create_demuxer_stream_cb_(create_demuxer_stream_cb),
       timestamp_offset_during_append_(NULL),
-      new_media_segment_(false),
       parsing_media_segment_(false),
+      media_segment_contained_audio_frame_(false),
+      media_segment_contained_video_frame_(false),
       stream_parser_(stream_parser.release()),
       audio_(NULL),
       video_(NULL),
       frame_processor_(frame_processor.release()),
       media_log_(media_log),
+      state_(UNINITIALIZED),
       auto_update_timestamp_offset_(false) {
   DCHECK(!create_demuxer_stream_cb_.is_null());
   DCHECK(frame_processor_);
@@ -107,9 +118,11 @@ void MediaSourceState::Init(
     bool allow_video,
     const StreamParser::EncryptedMediaInitDataCB& encrypted_media_init_data_cb,
     const NewTextTrackCB& new_text_track_cb) {
+  DCHECK_EQ(state_, UNINITIALIZED);
   new_text_track_cb_ = new_text_track_cb;
   init_cb_ = init_cb;
 
+  state_ = PENDING_PARSER_CONFIG;
   stream_parser_->Init(
       base::Bind(&MediaSourceState::OnSourceInitDone, base::Unretained(this)),
       base::Bind(&MediaSourceState::OnNewConfigs, base::Unretained(this),
@@ -135,21 +148,24 @@ void MediaSourceState::SetGroupStartTimestampIfInSequenceMode(
   frame_processor_->SetGroupStartTimestampIfInSequenceMode(timestamp_offset);
 }
 
-bool MediaSourceState::Append(
-    const uint8_t* data,
-    size_t length,
-    TimeDelta append_window_start,
-    TimeDelta append_window_end,
-    TimeDelta* timestamp_offset,
-    const InitSegmentReceivedCB& init_segment_received_cb) {
+void MediaSourceState::SetTracksWatcher(
+    const Demuxer::MediaTracksUpdatedCB& tracks_updated_cb) {
+  DCHECK(init_segment_received_cb_.is_null());
+  DCHECK(!tracks_updated_cb.is_null());
+  init_segment_received_cb_ = tracks_updated_cb;
+}
+
+bool MediaSourceState::Append(const uint8_t* data,
+                              size_t length,
+                              TimeDelta append_window_start,
+                              TimeDelta append_window_end,
+                              TimeDelta* timestamp_offset) {
+  append_in_progress_ = true;
   DCHECK(timestamp_offset);
   DCHECK(!timestamp_offset_during_append_);
-  DCHECK(!init_segment_received_cb.is_null());
-  DCHECK(init_segment_received_cb_.is_null());
   append_window_start_during_append_ = append_window_start;
   append_window_end_during_append_ = append_window_end;
   timestamp_offset_during_append_ = timestamp_offset;
-  init_segment_received_cb_ = init_segment_received_cb;
 
   // TODO(wolenetz/acolwell): Curry and pass a NewBuffersCB here bound with
   // append window and timestamp offset pointer. See http://crbug.com/351454.
@@ -162,7 +178,7 @@ bool MediaSourceState::Append(
         << " append_window_end=" << append_window_end.InSecondsF();
   }
   timestamp_offset_during_append_ = NULL;
-  init_segment_received_cb_.Reset();
+  append_in_progress_ = false;
   return result;
 }
 
@@ -180,6 +196,8 @@ void MediaSourceState::ResetParserState(TimeDelta append_window_start,
 
   frame_processor_->Reset();
   parsing_media_segment_ = false;
+  media_segment_contained_audio_frame_ = false;
+  media_segment_contained_video_frame_ = false;
 }
 
 void MediaSourceState::Remove(TimeDelta start,
@@ -293,6 +311,23 @@ Ranges<TimeDelta> MediaSourceState::GetBufferedRanges(TimeDelta duration,
   return ComputeRangesIntersection(ranges_list, ended);
 }
 
+TimeDelta MediaSourceState::GetHighestPresentationTimestamp() const {
+  TimeDelta max_pts;
+
+  if (audio_)
+    max_pts = std::max(max_pts, audio_->GetHighestPresentationTimestamp());
+
+  if (video_)
+    max_pts = std::max(max_pts, video_->GetHighestPresentationTimestamp());
+
+  for (TextStreamMap::const_iterator itr = text_stream_map_.begin();
+       itr != text_stream_map_.end(); ++itr) {
+    max_pts = std::max(max_pts, itr->second->GetHighestPresentationTimestamp());
+  }
+
+  return max_pts;
+}
+
 TimeDelta MediaSourceState::GetMaxBufferedDuration() const {
   TimeDelta max_duration;
 
@@ -458,13 +493,49 @@ bool MediaSourceState::IsSeekWaitingForData() const {
 bool MediaSourceState::OnNewConfigs(
     bool allow_audio,
     bool allow_video,
-    const AudioDecoderConfig& audio_config,
-    const VideoDecoderConfig& video_config,
+    std::unique_ptr<MediaTracks> tracks,
     const StreamParser::TextTrackConfigMap& text_configs) {
+  DCHECK_GE(state_, PENDING_PARSER_CONFIG);
+  DCHECK(tracks.get());
+
+  MediaTrack* audio_track = nullptr;
+  MediaTrack* video_track = nullptr;
+  AudioDecoderConfig audio_config;
+  VideoDecoderConfig video_config;
+  for (const auto& track : tracks->tracks()) {
+    const auto& track_id = track->bytestream_track_id();
+
+    if (track->type() == MediaTrack::Audio) {
+      if (audio_track) {
+        MEDIA_LOG(ERROR, media_log_)
+            << "Error: more than one audio track is currently not supported.";
+        return false;
+      }
+      audio_track = track.get();
+      audio_config = tracks->getAudioConfig(track_id);
+      DCHECK(audio_config.IsValidConfig());
+    } else if (track->type() == MediaTrack::Video) {
+      if (video_track) {
+        MEDIA_LOG(ERROR, media_log_)
+            << "Error: more than one video track is currently not supported.";
+        return false;
+      }
+      video_track = track.get();
+      video_config = tracks->getVideoConfig(track_id);
+      DCHECK(video_config.IsValidConfig());
+    } else {
+      MEDIA_LOG(ERROR, media_log_) << "Error: unsupported media track type "
+                                   << track->type();
+      return false;
+    }
+  }
+
   DVLOG(1) << "OnNewConfigs(" << allow_audio << ", " << allow_video << ", "
            << audio_config.IsValidConfig() << ", "
            << video_config.IsValidConfig() << ")";
-  DCHECK(!init_segment_received_cb_.is_null());
+  // MSE spec allows new configs to be emitted only during Append, but not
+  // during Flush or parser reset operations.
+  CHECK(append_in_progress_);
 
   if (!audio_config.IsValidConfig() && !video_config.IsValidConfig()) {
     DVLOG(1) << "OnNewConfigs() : Audio & video config are not valid!";
@@ -626,23 +697,52 @@ bool MediaSourceState::OnNewConfigs(
 
   frame_processor_->SetAllTrackBuffersNeedRandomAccessPoint();
 
+  if (audio_track) {
+    DCHECK(audio_);
+    audio_track->set_id(audio_->media_track_id());
+  }
+  if (video_track) {
+    DCHECK(video_);
+    video_track->set_id(video_->media_track_id());
+  }
+
   DVLOG(1) << "OnNewConfigs() : " << (success ? "success" : "failed");
-  if (success)
-    init_segment_received_cb_.Run();
+  if (success) {
+    if (state_ == PENDING_PARSER_CONFIG)
+      state_ = PENDING_PARSER_INIT;
+    DCHECK(!init_segment_received_cb_.is_null());
+    init_segment_received_cb_.Run(std::move(tracks));
+  }
 
   return success;
 }
 
 void MediaSourceState::OnNewMediaSegment() {
   DVLOG(2) << "OnNewMediaSegment()";
+  DCHECK_EQ(state_, PARSER_INITIALIZED);
   parsing_media_segment_ = true;
-  new_media_segment_ = true;
+  media_segment_contained_audio_frame_ = false;
+  media_segment_contained_video_frame_ = false;
 }
 
 void MediaSourceState::OnEndOfMediaSegment() {
   DVLOG(2) << "OnEndOfMediaSegment()";
+  DCHECK_EQ(state_, PARSER_INITIALIZED);
   parsing_media_segment_ = false;
-  new_media_segment_ = false;
+
+  const bool missing_audio = audio_ && !media_segment_contained_audio_frame_;
+  const bool missing_video = video_ && !media_segment_contained_video_frame_;
+  if (!missing_audio && !missing_video)
+    return;
+
+  LIMITED_MEDIA_LOG(DEBUG, media_log_, num_missing_track_logs_,
+                    kMaxMissingTrackInSegmentLogs)
+      << "Media segment did not contain any "
+      << (missing_audio && missing_video ? "audio or video"
+                                         : missing_audio ? "audio" : "video")
+      << " coded frames, mismatching initialization segment. Therefore, MSE "
+         "coded frame processing may not interoperably detect discontinuities "
+         "in appended media.";
 }
 
 bool MediaSourceState::OnNewBuffers(
@@ -650,9 +750,13 @@ bool MediaSourceState::OnNewBuffers(
     const StreamParser::BufferQueue& video_buffers,
     const StreamParser::TextBufferQueueMap& text_map) {
   DVLOG(2) << "OnNewBuffers()";
+  DCHECK_EQ(state_, PARSER_INITIALIZED);
   DCHECK(timestamp_offset_during_append_);
   DCHECK(parsing_media_segment_);
 
+  media_segment_contained_audio_frame_ |= !audio_buffers.empty();
+  media_segment_contained_video_frame_ |= !video_buffers.empty();
+
   const TimeDelta timestamp_offset_before_processing =
       *timestamp_offset_during_append_;
 
@@ -672,10 +776,10 @@ bool MediaSourceState::OnNewBuffers(
     }
   }
 
-  if (!frame_processor_->ProcessFrames(
-          audio_buffers, video_buffers, text_map,
-          append_window_start_during_append_, append_window_end_during_append_,
-          &new_media_segment_, timestamp_offset_during_append_)) {
+  if (!frame_processor_->ProcessFrames(audio_buffers, video_buffers, text_map,
+                                       append_window_start_during_append_,
+                                       append_window_end_during_append_,
+                                       timestamp_offset_during_append_)) {
     return false;
   }
 
@@ -690,6 +794,8 @@ bool MediaSourceState::OnNewBuffers(
 
 void MediaSourceState::OnSourceInitDone(
     const StreamParser::InitParameters& params) {
+  DCHECK_EQ(state_, PENDING_PARSER_INIT);
+  state_ = PARSER_INITIALIZED;
   auto_update_timestamp_offset_ = params.auto_update_timestamp_offset;
   base::ResetAndReturn(&init_cb_).Run(params);
 }
