@@ -8,6 +8,7 @@
 #include "base/callback_helpers.h"
 #include "base/macros.h"
 #include "base/message_loop/message_loop.h"
+#include "base/run_loop.h"
 #include "media/base/fake_demuxer_stream.h"
 #include "media/base/gmock_callback_support.h"
 #include "media/base/mock_filters.h"
@@ -30,22 +31,6 @@ using ::testing::StrictMock;
 static const int kNumConfigs = 4;
 static const int kNumBuffersInOneConfig = 5;
 
-// Use anonymous namespace here to prevent the actions to be defined multiple
-// times across multiple test files. Sadly we can't use static for them.
-namespace {
-
-ACTION_P3(ExecuteCallbackWithVerifier, cdm_context, done_cb, verifier) {
-  // verifier must be called first since |done_cb| call will invoke it as well.
-  verifier->RecordACalled();
-  arg0.Run(cdm_context, done_cb);
-}
-
-ACTION_P(ReportCallback, verifier) {
-  verifier->RecordBCalled();
-}
-
-}  // namespace
-
 namespace media {
 
 struct VideoFrameStreamTestParams {
@@ -125,7 +110,7 @@ class VideoFrameStreamTest
     decoder2_ = NULL;
     decoder3_ = NULL;
     video_frame_stream_.reset();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
 
     DCHECK(!pending_initialize_);
     DCHECK(!pending_read_);
@@ -134,8 +119,6 @@ class VideoFrameStreamTest
   }
 
   MOCK_METHOD1(OnNewSpliceBuffer, void(base::TimeDelta));
-  MOCK_METHOD1(SetCdmReadyCallback, void(const media::CdmReadyCB&));
-  MOCK_METHOD1(CdmSet, void(bool));
   MOCK_METHOD0(OnWaitingForDecryptionKey, void(void));
 
   void OnStatistics(const PipelineStatistics& statistics) {
@@ -165,12 +148,11 @@ class VideoFrameStreamTest
     video_frame_stream_->Initialize(
         demuxer_stream_.get(), base::Bind(&VideoFrameStreamTest::OnInitialized,
                                           base::Unretained(this)),
-        base::Bind(&VideoFrameStreamTest::SetCdmReadyCallback,
-                   base::Unretained(this)),
+        cdm_context_.get(),
         base::Bind(&VideoFrameStreamTest::OnStatistics, base::Unretained(this)),
         base::Bind(&VideoFrameStreamTest::OnWaitingForDecryptionKey,
                    base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   // Fake Decrypt() function used by DecryptingDemuxerStream. It does nothing
@@ -207,12 +189,6 @@ class VideoFrameStreamTest
     pending_read_ = false;
   }
 
-  void FrameReadyHoldDemuxer(VideoFrameStream::Status status,
-                             const scoped_refptr<VideoFrame>& frame) {
-    FrameReady(status, frame);
-
-  }
-
   void OnReset() {
     DCHECK(!pending_read_);
     DCHECK(pending_reset_);
@@ -224,7 +200,7 @@ class VideoFrameStreamTest
     pending_read_ = true;
     video_frame_stream_->Read(base::Bind(
         &VideoFrameStreamTest::FrameReady, base::Unretained(this)));
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void ReadUntilPending() {
@@ -233,22 +209,25 @@ class VideoFrameStreamTest
     } while (!pending_read_);
   }
 
-  void ReadAllFrames() {
+  void ReadAllFrames(int expected_decoded_frames) {
     do {
       ReadOneFrame();
     } while (frame_read_.get() &&
              !frame_read_->metadata()->IsTrue(
                  VideoFrameMetadata::END_OF_STREAM));
 
-    const int total_num_frames = kNumConfigs * kNumBuffersInOneConfig;
-    DCHECK_EQ(num_decoded_frames_, total_num_frames);
+    DCHECK_EQ(expected_decoded_frames, num_decoded_frames_);
+  }
+
+  void ReadAllFrames() {
+    // No frames should have been dropped.
+    ReadAllFrames(kNumConfigs * kNumBuffersInOneConfig);
   }
 
   enum PendingState {
     NOT_PENDING,
     DEMUXER_READ_NORMAL,
     DEMUXER_READ_CONFIG_CHANGE,
-    SET_DECRYPTOR,
     DECRYPTOR_NO_KEY,
     DECODER_INIT,
     DECODER_REINIT,
@@ -256,15 +235,6 @@ class VideoFrameStreamTest
     DECODER_RESET
   };
 
-  void ExpectCdmNotification() {
-    EXPECT_CALL(*this, SetCdmReadyCallback(_))
-        .WillRepeatedly(ExecuteCallbackWithVerifier(
-            cdm_context_.get(),
-            base::Bind(&VideoFrameStreamTest::CdmSet, base::Unretained(this)),
-            &verifier_));
-    EXPECT_CALL(*this, CdmSet(true)).WillRepeatedly(ReportCallback(&verifier_));
-  }
-
   void EnterPendingState(PendingState state) {
     EnterPendingState(state, decoder1_);
   }
@@ -282,23 +252,14 @@ class VideoFrameStreamTest
         ReadUntilPending();
         break;
 
-      case SET_DECRYPTOR:
-        // Hold CdmReadyCB.
-        EXPECT_CALL(*this, SetCdmReadyCallback(_)).Times(2);
-        // Initialize will fail because no decryptor is available.
-        InitializeVideoFrameStream();
-        break;
-
       case DECRYPTOR_NO_KEY:
         if (GetParam().is_encrypted)
           EXPECT_CALL(*this, OnWaitingForDecryptionKey());
-        ExpectCdmNotification();
         has_no_key_ = true;
         ReadOneFrame();
         break;
 
       case DECODER_INIT:
-        ExpectCdmNotification();
         decoder->HoldNextInit();
         InitializeVideoFrameStream();
         break;
@@ -318,7 +279,7 @@ class VideoFrameStreamTest
         pending_reset_ = true;
         video_frame_stream_->Reset(base::Bind(&VideoFrameStreamTest::OnReset,
                                               base::Unretained(this)));
-        message_loop_.RunUntilIdle();
+        base::RunLoop().RunUntilIdle();
         break;
 
       case NOT_PENDING:
@@ -339,9 +300,8 @@ class VideoFrameStreamTest
         demuxer_stream_->SatisfyRead();
         break;
 
-      // These two cases are only interesting to test during
-      // VideoFrameStream destruction.  There's no need to satisfy a callback.
-      case SET_DECRYPTOR:
+      // This is only interesting to test during VideoFrameStream destruction.
+      // There's no need to satisfy a callback.
       case DECRYPTOR_NO_KEY:
         NOTREACHED();
         break;
@@ -367,7 +327,7 @@ class VideoFrameStreamTest
         break;
     }
 
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   void Initialize() {
@@ -392,13 +352,14 @@ class VideoFrameStreamTest
 
   base::MessageLoop message_loop_;
 
-  scoped_ptr<VideoFrameStream> video_frame_stream_;
-  scoped_ptr<FakeDemuxerStream> demuxer_stream_;
-  scoped_ptr<StrictMock<MockCdmContext>> cdm_context_;
+  std::unique_ptr<VideoFrameStream> video_frame_stream_;
+  std::unique_ptr<FakeDemuxerStream> demuxer_stream_;
+  std::unique_ptr<StrictMock<MockCdmContext>> cdm_context_;
 
   // Use NiceMock since we don't care about most of calls on the decryptor,
   // e.g. RegisterNewKeyCB().
-  scoped_ptr<NiceMock<MockDecryptor>> decryptor_;
+  std::unique_ptr<NiceMock<MockDecryptor>> decryptor_;
+
   // Three decoders are needed to test that decoder fallback can occur more than
   // once on a config change. They are owned by |video_frame_stream_|.
   FakeVideoDecoder* decoder1_;
@@ -418,8 +379,6 @@ class VideoFrameStreamTest
   // Decryptor has no key to decrypt a frame.
   bool has_no_key_;
 
-  CallbackPairChecker verifier_;
-
  private:
   DISALLOW_COPY_AND_ASSIGN(VideoFrameStreamTest);
 };
@@ -450,7 +409,7 @@ TEST_P(VideoFrameStreamTest, Initialization) {
   Initialize();
 }
 
-TEST_P(VideoFrameStreamTest, DecoderInitializationFails) {
+TEST_P(VideoFrameStreamTest, AllDecoderInitializationFails) {
   decoder1_->SimulateFailureToInit();
   decoder2_->SimulateFailureToInit();
   decoder3_->SimulateFailureToInit();
@@ -458,6 +417,13 @@ TEST_P(VideoFrameStreamTest, DecoderInitializationFails) {
   EXPECT_FALSE(is_initialized_);
 }
 
+TEST_P(VideoFrameStreamTest, PartialDecoderInitializationFails) {
+  decoder1_->SimulateFailureToInit();
+  decoder2_->SimulateFailureToInit();
+  Initialize();
+  EXPECT_TRUE(is_initialized_);
+}
+
 TEST_P(VideoFrameStreamTest, ReadOneFrame) {
   Initialize();
   Read();
@@ -489,7 +455,7 @@ TEST_P(VideoFrameStreamTest, Read_BlockedDemuxer) {
   while (pending_read_) {
     ++demuxed_buffers;
     demuxer_stream_->SatisfyReadAndHoldNext();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   EXPECT_EQ(std::min(GetParam().decoding_delay + 1, kNumBuffersInOneConfig + 1),
@@ -501,7 +467,7 @@ TEST_P(VideoFrameStreamTest, Read_BlockedDemuxer) {
   ReadUntilPending();
 
   demuxer_stream_->SatisfyRead();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(pending_read_);
 }
 
@@ -523,12 +489,12 @@ TEST_P(VideoFrameStreamTest, Read_BlockedDemuxerAndDecoder) {
   while (pending_read_) {
     ++demuxed_buffers;
     demuxer_stream_->SatisfyReadAndHoldNext();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
 
     // Always keep one decode request pending.
     if (demuxed_buffers > 1) {
       decoder1_->SatisfySingleDecode();
-      message_loop_.RunUntilIdle();
+      base::RunLoop().RunUntilIdle();
     }
   }
 
@@ -538,7 +504,7 @@ TEST_P(VideoFrameStreamTest, Read_BlockedDemuxerAndDecoder) {
   // Unblocking one decode request should unblock read even when demuxer is
   // still blocked.
   decoder1_->SatisfySingleDecode();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(pending_read_);
 
   // Stream should still be blocked on the demuxer after unblocking the decoder.
@@ -552,7 +518,7 @@ TEST_P(VideoFrameStreamTest, Read_BlockedDemuxerAndDecoder) {
 
   // Unblocking the demuxer will unblock the stream.
   demuxer_stream_->SatisfyRead();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   EXPECT_FALSE(pending_read_);
 }
 
@@ -573,7 +539,7 @@ TEST_P(VideoFrameStreamTest, Read_DuringEndOfStreamDecode) {
       ReadOneFrame();
       while (pending_read_) {
         decoder1_->SatisfySingleDecode();
-        message_loop_.RunUntilIdle();
+        base::RunLoop().RunUntilIdle();
       }
     }
   }
@@ -584,7 +550,7 @@ TEST_P(VideoFrameStreamTest, Read_DuringEndOfStreamDecode) {
 
   // Satisfy decoding of the end of stream buffer. The read should complete.
   decoder1_->SatisfySingleDecode();
-  message_loop_.RunUntilIdle();
+  base::RunLoop().RunUntilIdle();
   ASSERT_FALSE(pending_read_);
   EXPECT_EQ(last_read_status_, VideoFrameStream::OK);
 
@@ -703,15 +669,6 @@ TEST_P(VideoFrameStreamTest, Reset_DuringNoKeyRead) {
 TEST_P(VideoFrameStreamTest, Destroy_BeforeInitialization) {
 }
 
-TEST_P(VideoFrameStreamTest, Destroy_DuringSetDecryptor) {
-  if (!GetParam().is_encrypted) {
-    DVLOG(1) << "SetDecryptor test only runs when the stream is encrytped.";
-    return;
-  }
-
-  EnterPendingState(SET_DECRYPTOR);
-}
-
 TEST_P(VideoFrameStreamTest, Destroy_DuringInitialization) {
   EnterPendingState(DECODER_INIT);
 }
@@ -796,18 +753,316 @@ TEST_P(VideoFrameStreamTest, Destroy_AfterRead_AfterReset) {
   Reset();
 }
 
-TEST_P(VideoFrameStreamTest, DecoderErrorWhenReading) {
+TEST_P(VideoFrameStreamTest, FallbackDecoder_SelectedOnInitialDecodeError) {
   Initialize();
-  EnterPendingState(DECODER_DECODE);
   decoder1_->SimulateError();
-  message_loop_.RunUntilIdle();
+  ReadOneFrame();
+
+  // |video_frame_stream_| should have fallen back to |decoder2_|.
   ASSERT_FALSE(pending_read_);
+  ASSERT_EQ(VideoFrameStream::OK, last_read_status_);
+
+  // Can't check |decoder1_| right now, it might have been destroyed already.
+  ASSERT_GT(decoder2_->total_bytes_decoded(), 0);
+
+  // Verify no frame was dropped.
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_EndOfStreamReachedBeforeFallback) {
+  // Only consider cases where there is a decoder delay. For test simplicity,
+  // omit the parallel case.
+  if (GetParam().decoding_delay == 0 || GetParam().parallel_decoding > 1)
+    return;
+
+  Initialize();
+  decoder1_->HoldDecode();
+  ReadOneFrame();
+
+  // One buffer should have already pulled from the demuxer stream. Set the next
+  // one to be an EOS.
+  demuxer_stream_->SeekToEndOfStream();
+
+  decoder1_->SatisfySingleDecode();
+  base::RunLoop().RunUntilIdle();
+
+  // |video_frame_stream_| should not have emited a frame.
+  EXPECT_TRUE(pending_read_);
+
+  // Pending buffers should contain a regular buffer and an EOS buffer.
+  EXPECT_EQ(video_frame_stream_->get_pending_buffers_size_for_testing(), 2);
+
+  decoder1_->SimulateError();
+  base::RunLoop().RunUntilIdle();
+
+  //  A frame should have been emited
+  EXPECT_FALSE(pending_read_);
+  EXPECT_EQ(last_read_status_, VideoFrameStream::OK);
+  EXPECT_FALSE(
+      frame_read_->metadata()->IsTrue(VideoFrameMetadata::END_OF_STREAM));
+  EXPECT_GT(decoder2_->total_bytes_decoded(), 0);
+
+  ReadOneFrame();
+
+  EXPECT_FALSE(pending_read_);
+  EXPECT_EQ(0, video_frame_stream_->get_fallback_buffers_size_for_testing());
+  EXPECT_TRUE(
+      frame_read_->metadata()->IsTrue(VideoFrameMetadata::END_OF_STREAM));
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_DoesReinitializeStompPendingRead) {
+  // Test only the case where there is no decoding delay and parallel decoding.
+  if (GetParam().decoding_delay != 0 || GetParam().parallel_decoding <= 1)
+    return;
+
+  Initialize();
+  decoder1_->HoldDecode();
+
+  // Queue one read, defer the second.
+  frame_read_ = nullptr;
+  pending_read_ = true;
+  video_frame_stream_->Read(
+      base::Bind(&VideoFrameStreamTest::FrameReady, base::Unretained(this)));
+  demuxer_stream_->HoldNextRead();
+
+  // Force an error to occur on the first decode, but ensure it isn't propagated
+  // until after the next read has been started.
+  decoder1_->SimulateError();
+  decoder2_->HoldDecode();
+
+  // Complete the fallback to the second decoder with the read still pending.
+  base::RunLoop().RunUntilIdle();
+
+  // Can't check |decoder1_| right now, it might have been destroyed already.
+  // Verify that there was nothing decoded until we kicked the decoder.
+  EXPECT_EQ(decoder2_->total_bytes_decoded(), 0);
+  decoder2_->SatisfyDecode();
+  const int first_decoded_bytes = decoder2_->total_bytes_decoded();
+  ASSERT_GT(first_decoded_bytes, 0);
+
+  // Satisfy the previously pending read and ensure it is decoded.
+  demuxer_stream_->SatisfyRead();
+  base::RunLoop().RunUntilIdle();
+  ASSERT_GT(decoder2_->total_bytes_decoded(), first_decoded_bytes);
+}
+
+TEST_P(VideoFrameStreamTest,
+       FallbackDecoder_SelectedOnInitialDecodeError_Twice) {
+  Initialize();
+  decoder1_->SimulateError();
+  decoder2_->HoldNextInit();
+  ReadOneFrame();
+
+  decoder2_->SatisfyInit();
+  decoder2_->SimulateError();
+  base::RunLoop().RunUntilIdle();
+
+  // |video_frame_stream_| should have fallen back to |decoder3_|.
+  ASSERT_FALSE(pending_read_);
+  ASSERT_EQ(VideoFrameStream::OK, last_read_status_);
+
+  // Can't check |decoder1_| or |decoder2_| right now, they might have been
+  // destroyed already.
+  ASSERT_GT(decoder3_->total_bytes_decoded(), 0);
+
+  // Verify no frame was dropped.
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_ConfigChangeClearsPendingBuffers) {
+  // Test case is only interesting if the decoder can receive a config change
+  // before returning its first frame.
+  if (GetParam().decoding_delay < kNumBuffersInOneConfig)
+    return;
+
+  Initialize();
+  EnterPendingState(DEMUXER_READ_CONFIG_CHANGE);
+  ASSERT_GT(video_frame_stream_->get_pending_buffers_size_for_testing(), 0);
+
+  SatisfyPendingCallback(DEMUXER_READ_CONFIG_CHANGE);
+  ASSERT_EQ(video_frame_stream_->get_pending_buffers_size_for_testing(), 0);
+  EXPECT_FALSE(pending_read_);
+
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_ErrorDuringConfigChangeFlushing) {
+  // Test case is only interesting if the decoder can receive a config change
+  // before returning its first frame.
+  if (GetParam().decoding_delay < kNumBuffersInOneConfig)
+    return;
+
+  Initialize();
+  EnterPendingState(DEMUXER_READ_CONFIG_CHANGE);
+  EXPECT_GT(video_frame_stream_->get_pending_buffers_size_for_testing(), 0);
+
+  decoder1_->HoldDecode();
+  SatisfyPendingCallback(DEMUXER_READ_CONFIG_CHANGE);
+
+  // The flush request should have been sent and held.
+  EXPECT_EQ(video_frame_stream_->get_pending_buffers_size_for_testing(), 0);
+  EXPECT_TRUE(pending_read_);
+
+  // Triggering an error here will cause the frames in |decoder1_| to be lost.
+  // There are no pending buffers buffers to give to give to |decoder2_| due to
+  // crbug.com/603713.
+  decoder1_->SimulateError();
+  base::RunLoop().RunUntilIdle();
+
+  // We want to make sure that |decoder2_| can decode the rest of the frames
+  // in the demuxer stream.
+  ReadAllFrames(kNumBuffersInOneConfig * (kNumConfigs - 1));
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_PendingBuffersIsFilledAndCleared) {
+  // Test applies only when there is a decoder delay, and the decoder will not
+  // receive a config change before outputing its first frame. Parallel decoding
+  // is also disabled in this test case, for readability and simplicity of the
+  // unit test.
+  if (GetParam().decoding_delay == 0 ||
+      GetParam().decoding_delay > kNumBuffersInOneConfig ||
+      GetParam().parallel_decoding > 1) {
+    return;
+  }
+  Initialize();
+
+  // Block on demuxer read and decoder decode so we can step through.
+  demuxer_stream_->HoldNextRead();
+  decoder1_->HoldDecode();
+  ReadOneFrame();
+
+  int demuxer_reads_satisfied = 0;
+  // Send back and requests buffers until the next one would fill the decoder
+  // delay.
+  while (demuxer_reads_satisfied < GetParam().decoding_delay - 1) {
+    // Send a buffer back.
+    demuxer_stream_->SatisfyReadAndHoldNext();
+    base::RunLoop().RunUntilIdle();
+    ++demuxer_reads_satisfied;
+
+    // Decode one buffer.
+    decoder1_->SatisfySingleDecode();
+    base::RunLoop().RunUntilIdle();
+    EXPECT_TRUE(pending_read_);
+    EXPECT_EQ(demuxer_reads_satisfied,
+              video_frame_stream_->get_pending_buffers_size_for_testing());
+    // No fallback buffers should be queued up yet.
+    EXPECT_EQ(0, video_frame_stream_->get_fallback_buffers_size_for_testing());
+  }
+
+  // Hold the init before triggering the error, to verify internal state.
+  demuxer_stream_->SatisfyReadAndHoldNext();
+  ++demuxer_reads_satisfied;
+  decoder2_->HoldNextInit();
+  decoder1_->SimulateError();
+  base::RunLoop().RunUntilIdle();
+
+  EXPECT_TRUE(pending_read_);
+  EXPECT_EQ(demuxer_reads_satisfied,
+            video_frame_stream_->get_pending_buffers_size_for_testing());
+
+  decoder2_->SatisfyInit();
+  decoder2_->HoldDecode();
+  base::RunLoop().RunUntilIdle();
+
+  // Make sure the pending buffers have been transfered to fallback buffers.
+  // One call to Decode() during the initialization process, so we expect one
+  // buffer to already have been consumed from the fallback buffers.
+  // Pending buffers should never go down (unless we encounter a config change)
+  EXPECT_EQ(demuxer_reads_satisfied - 1,
+            video_frame_stream_->get_fallback_buffers_size_for_testing());
+  EXPECT_EQ(demuxer_reads_satisfied,
+            video_frame_stream_->get_pending_buffers_size_for_testing());
+
+  decoder2_->SatisfyDecode();
+  base::RunLoop().RunUntilIdle();
+
+  // Make sure all buffers consumed by |decoder2_| have come from the fallback.
+  // Pending buffers should not have been cleared yet.
+  EXPECT_EQ(0, video_frame_stream_->get_fallback_buffers_size_for_testing());
+  EXPECT_EQ(demuxer_reads_satisfied,
+            video_frame_stream_->get_pending_buffers_size_for_testing());
+  EXPECT_EQ(video_frame_stream_->get_previous_decoder_for_testing(), decoder1_);
+  EXPECT_TRUE(pending_read_);
+
+  // Give the decoder one more buffer, enough to release a frame.
+  demuxer_stream_->SatisfyReadAndHoldNext();
+  base::RunLoop().RunUntilIdle();
+
+  // New buffers should not have been added after the frame was released.
+  EXPECT_EQ(video_frame_stream_->get_pending_buffers_size_for_testing(), 0);
+  EXPECT_FALSE(pending_read_);
+
+  demuxer_stream_->SatisfyRead();
+
+  // Confirm no frames were dropped.
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_SelectedOnDecodeThenInitErrors) {
+  Initialize();
+  decoder1_->SimulateError();
+  decoder2_->SimulateFailureToInit();
+  ReadOneFrame();
+
+  // |video_frame_stream_| should have fallen back to |decoder3_|
+  ASSERT_FALSE(pending_read_);
+  ASSERT_EQ(VideoFrameStream::OK, last_read_status_);
+
+  // Can't check |decoder1_| or |decoder2_| right now, they might have been
+  // destroyed already.
+  ASSERT_GT(decoder3_->total_bytes_decoded(), 0);
+
+  // Verify no frame was dropped.
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest, FallbackDecoder_SelectedOnInitThenDecodeErrors) {
+  decoder1_->SimulateFailureToInit();
+  decoder2_->HoldDecode();
+  Initialize();
+  ReadOneFrame();
+  decoder2_->SimulateError();
+  base::RunLoop().RunUntilIdle();
+
+  // |video_frame_stream_| should have fallen back to |decoder3_|
+  ASSERT_FALSE(pending_read_);
+  ASSERT_EQ(VideoFrameStream::OK, last_read_status_);
+
+  // Can't check |decoder1_| or |decoder2_| right now, they might have been
+  // destroyed already.
+  ASSERT_GT(decoder3_->total_bytes_decoded(), 0);
+
+  // Verify no frame was dropped.
+  ReadAllFrames();
+}
+
+TEST_P(VideoFrameStreamTest,
+       FallbackDecoder_NotSelectedOnMidstreamDecodeError) {
+  Initialize();
+  ReadOneFrame();
+
+  // Successfully received a frame.
+  EXPECT_FALSE(pending_read_);
+  ASSERT_GT(decoder1_->total_bytes_decoded(), 0);
+
+  decoder1_->SimulateError();
+
+  // The error must surface from Read() as DECODE_ERROR.
+  while (last_read_status_ == VideoFrameStream::OK) {
+    ReadOneFrame();
+    base::RunLoop().RunUntilIdle();
+    EXPECT_FALSE(pending_read_);
+  }
+
+  // Verify the error was surfaced, rather than falling back to |decoder2_|.
+  EXPECT_FALSE(pending_read_);
+  ASSERT_EQ(decoder2_->total_bytes_decoded(), 0);
   ASSERT_EQ(VideoFrameStream::DECODE_ERROR, last_read_status_);
 }
 
 TEST_P(VideoFrameStreamTest, DecoderErrorWhenNotReading) {
   Initialize();
-
   decoder1_->HoldDecode();
   ReadOneFrame();
   EXPECT_TRUE(pending_read_);
@@ -815,7 +1070,7 @@ TEST_P(VideoFrameStreamTest, DecoderErrorWhenNotReading) {
   // Satisfy decode requests until we get the first frame out.
   while (pending_read_) {
     decoder1_->SatisfySingleDecode();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
   }
 
   // Trigger an error in the decoding.
@@ -824,7 +1079,7 @@ TEST_P(VideoFrameStreamTest, DecoderErrorWhenNotReading) {
   // The error must surface from Read() as DECODE_ERROR.
   while (last_read_status_ == VideoFrameStream::OK) {
     ReadOneFrame();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     EXPECT_FALSE(pending_read_);
   }
   EXPECT_EQ(VideoFrameStream::DECODE_ERROR, last_read_status_);
@@ -833,8 +1088,13 @@ TEST_P(VideoFrameStreamTest, DecoderErrorWhenNotReading) {
 TEST_P(VideoFrameStreamTest, FallbackDecoderSelectedOnFailureToReinitialize) {
   Initialize();
   decoder1_->SimulateFailureToInit();
+  // Holding decode, because large decoder delays might cause us to get rid of
+  // |previous_decoder_| before we are in a pending state again.
+  decoder2_->HoldDecode();
   ReadUntilDecoderReinitialized(decoder1_);
   ASSERT_TRUE(video_frame_stream_->get_previous_decoder_for_testing());
+  decoder2_->SatisfyDecode();
+  base::RunLoop().RunUntilIdle();
   ReadAllFrames();
   ASSERT_FALSE(video_frame_stream_->get_previous_decoder_for_testing());
 }
@@ -859,7 +1119,7 @@ TEST_P(VideoFrameStreamTest, DecodeErrorAfterFallbackDecoderSelectionFails) {
   // The error will surface from Read() as DECODE_ERROR.
   while (last_read_status_ == VideoFrameStream::OK) {
     ReadOneFrame();
-    message_loop_.RunUntilIdle();
+    base::RunLoop().RunUntilIdle();
     EXPECT_FALSE(pending_read_);
   }
   EXPECT_EQ(VideoFrameStream::DECODE_ERROR, last_read_status_);
