@@ -2,11 +2,15 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "media/formats/mpeg/mpeg1_audio_stream_parser.h"
+
 #include <stdint.h>
 
+#include <memory>
+
+#include "base/memory/ptr_util.h"
 #include "media/base/test_data_util.h"
 #include "media/formats/common/stream_parser_test_base.h"
-#include "media/formats/mpeg/mpeg1_audio_stream_parser.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace media {
@@ -15,7 +19,7 @@ class MPEG1AudioStreamParserTest
     : public StreamParserTestBase, public testing::Test {
  public:
   MPEG1AudioStreamParserTest()
-      : StreamParserTestBase(make_scoped_ptr(new MPEG1AudioStreamParser())) {}
+      : StreamParserTestBase(base::WrapUnique(new MPEG1AudioStreamParser())) {}
 };
 
 // Test parsing with small prime sized chunks to smoke out "power of
