@@ -2,11 +2,14 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
+#include "media/formats/webm/webm_cluster_parser.h"
+
 #include <stddef.h>
 #include <stdint.h>
 
 #include <algorithm>
 #include <cstdlib>
+#include <memory>
 #include <string>
 #include <vector>
 
@@ -20,7 +23,6 @@
 #include "media/base/timestamp_constants.h"
 #include "media/formats/webm/cluster_builder.h"
 #include "media/formats/webm/opus_packet_builder.h"
-#include "media/formats/webm/webm_cluster_parser.h"
 #include "media/formats/webm/webm_constants.h"
 #include "testing/gmock/include/gmock/gmock.h"
 #include "testing/gtest/include/gtest/gtest.h"
@@ -101,17 +103,19 @@ struct BlockInfo {
   // Default data will be used if no data given.
   const uint8_t* data;
   int data_length;
+
+  bool is_key_frame;
 };
 
 const BlockInfo kDefaultBlockInfo[] = {
-    {kAudioTrackNum, 0, 23, true, NULL, 0},
-    {kAudioTrackNum, 23, 23, true, NULL, 0},
+    {kAudioTrackNum, 0, 23, true, NULL, 0, true},
+    {kAudioTrackNum, 23, 23, true, NULL, 0, true},
     // Assumes not using DefaultDuration
-    {kVideoTrackNum, 33, 34, true, NULL, 0},
-    {kAudioTrackNum, 46, 23, true, NULL, 0},
-    {kVideoTrackNum, 67, 33, false, NULL, 0},
-    {kAudioTrackNum, 69, 23, false, NULL, 0},
-    {kVideoTrackNum, 100, 33, false, NULL, 0},
+    {kVideoTrackNum, 33, 34, true, NULL, 0, true},
+    {kAudioTrackNum, 46, 23, true, NULL, 0, false},
+    {kVideoTrackNum, 67, 33, false, NULL, 0, true},
+    {kAudioTrackNum, 69, 23, false, NULL, 0, false},
+    {kVideoTrackNum, 100, 33, false, NULL, 0, false},
 };
 
 const uint8_t kEncryptedFrame[] = {
@@ -121,9 +125,9 @@ const uint8_t kEncryptedFrame[] = {
     // IV
     0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08};
 
-scoped_ptr<Cluster> CreateCluster(int timecode,
-                                  const BlockInfo* block_info,
-                                  int block_count) {
+std::unique_ptr<Cluster> CreateCluster(int timecode,
+                                       const BlockInfo* block_info,
+                                       int block_count) {
   ClusterBuilder cb;
   cb.SetClusterTimecode(0);
 
@@ -142,20 +146,22 @@ scoped_ptr<Cluster> CreateCluster(int timecode,
 
     if (block_info[i].use_simple_block) {
       CHECK_GE(block_info[i].duration, 0);
-      cb.AddSimpleBlock(block_info[i].track_num, block_info[i].timestamp, 0,
-                        data, data_length);
+      cb.AddSimpleBlock(block_info[i].track_num, block_info[i].timestamp,
+                        block_info[i].is_key_frame ? 0x80 : 0x00, data,
+                        data_length);
       continue;
     }
 
     if (block_info[i].duration < 0) {
-      cb.AddBlockGroupWithoutBlockDuration(block_info[i].track_num,
-                                           block_info[i].timestamp, 0, data,
-                                           data_length);
+      cb.AddBlockGroupWithoutBlockDuration(
+          block_info[i].track_num, block_info[i].timestamp, 0,
+          block_info[i].is_key_frame, data, data_length);
       continue;
     }
 
     cb.AddBlockGroup(block_info[i].track_num, block_info[i].timestamp,
-                     block_info[i].duration, 0, data, data_length);
+                     block_info[i].duration, 0, block_info[i].is_key_frame,
+                     data, data_length);
   }
 
   return cb.Finish();
@@ -163,7 +169,7 @@ scoped_ptr<Cluster> CreateCluster(int timecode,
 
 // Creates a Cluster with one encrypted Block. |bytes_to_write| is number of
 // bytes of the encrypted frame to write.
-scoped_ptr<Cluster> CreateEncryptedCluster(int bytes_to_write) {
+std::unique_ptr<Cluster> CreateEncryptedCluster(int bytes_to_write) {
   CHECK_GT(bytes_to_write, 0);
   CHECK_LE(bytes_to_write, static_cast<int>(sizeof(kEncryptedFrame)));
 
@@ -225,12 +231,13 @@ bool VerifyBuffers(const WebMClusterParser::BufferQueue& audio_buffers,
               buffer->duration().InMillisecondsF());
     EXPECT_EQ(expected_type, buffer->type());
     EXPECT_EQ(block_info[i].track_num, buffer->track_id());
+    EXPECT_EQ(block_info[i].is_key_frame, buffer->is_key_frame());
   }
 
   return true;
 }
 
-bool VerifyBuffers(const scoped_ptr<WebMClusterParser>& parser,
+bool VerifyBuffers(const std::unique_ptr<WebMClusterParser>& parser,
                    const BlockInfo* block_info,
                    int block_count) {
   const WebMClusterParser::TextBufferQueueMap& text_map =
@@ -249,7 +256,7 @@ bool VerifyBuffers(const scoped_ptr<WebMClusterParser>& parser,
                        block_count);
 }
 
-bool VerifyTextBuffers(const scoped_ptr<WebMClusterParser>& parser,
+bool VerifyTextBuffers(const std::unique_ptr<WebMClusterParser>& parser,
                        const BlockInfo* block_info_ptr,
                        int block_count,
                        int text_track_num,
@@ -371,7 +378,7 @@ class WebMClusterParserTest : public testing::Test {
   }
 
   scoped_refptr<StrictMock<MockMediaLog>> media_log_;
-  scoped_ptr<WebMClusterParser> parser_;
+  std::unique_ptr<WebMClusterParser> parser_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(WebMClusterParserTest);
@@ -400,15 +407,19 @@ TEST_F(WebMClusterParserTest, HeldBackBufferHoldsBackAllTracks) {
   const int kExpectedVideoEstimationInMs = 33;
 
   const BlockInfo kBlockInfo[] = {
-      {kVideoTrackNum, 0, 33, true, NULL, 0},
-      {kAudioTrackNum, 0, 23, false, NULL, 0},
-      {kTextTrackNum, 10, 42, false, NULL, 0},
-      {kAudioTrackNum, 23, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kVideoTrackNum, 33, 33, true, NULL, 0},
-      {kAudioTrackNum, 36, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kVideoTrackNum, 66, kExpectedVideoEstimationInMs, true, NULL, 0},
-      {kAudioTrackNum, 70, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kAudioTrackNum, 83, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
+      {kVideoTrackNum, 0, 33, true, NULL, 0, false},
+      {kAudioTrackNum, 0, 23, false, NULL, 0, false},
+      {kTextTrackNum, 10, 42, false, NULL, 0, true},
+      {kAudioTrackNum, 23, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kVideoTrackNum, 33, 33, true, NULL, 0, false},
+      {kAudioTrackNum, 36, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kVideoTrackNum, 66, kExpectedVideoEstimationInMs, true, NULL, 0, false},
+      {kAudioTrackNum, 70, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kAudioTrackNum, 83, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
   };
 
   const int kExpectedBuffersOnPartialCluster[] = {
@@ -439,8 +450,8 @@ TEST_F(WebMClusterParserTest, HeldBackBufferHoldsBackAllTracks) {
     // parse of all but one byte should deterministically parse all but the
     // last full block. Don't |exceed block_count| blocks though.
     int blocks_in_cluster = std::min(i + 2, block_count);
-    scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo,
-                                              blocks_in_cluster));
+    std::unique_ptr<Cluster> cluster(
+        CreateCluster(0, kBlockInfo, blocks_in_cluster));
     // Parse all but the last byte unless we need to parse the full cluster.
     bool parse_full_cluster = i == (block_count - 1);
 
@@ -471,7 +482,8 @@ TEST_F(WebMClusterParserTest, Reset) {
   InSequence s;
 
   int block_count = arraysize(kDefaultBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kDefaultBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(
+      CreateCluster(0, kDefaultBlockInfo, block_count));
 
   // Send slightly less than the full cluster so all but the last block is
   // parsed.
@@ -490,7 +502,8 @@ TEST_F(WebMClusterParserTest, Reset) {
 
 TEST_F(WebMClusterParserTest, ParseClusterWithSingleCall) {
   int block_count = arraysize(kDefaultBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kDefaultBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(
+      CreateCluster(0, kDefaultBlockInfo, block_count));
 
   int result = parser_->Parse(cluster->data(), cluster->size());
   EXPECT_EQ(cluster->size(), result);
@@ -499,7 +512,8 @@ TEST_F(WebMClusterParserTest, ParseClusterWithSingleCall) {
 
 TEST_F(WebMClusterParserTest, ParseClusterWithMultipleCalls) {
   int block_count = arraysize(kDefaultBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kDefaultBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(
+      CreateCluster(0, kDefaultBlockInfo, block_count));
 
   WebMClusterParser::BufferQueue audio_buffers;
   WebMClusterParser::BufferQueue video_buffers;
@@ -542,8 +556,8 @@ TEST_F(WebMClusterParserTest, ParseClusterWithMultipleCalls) {
 // one of these scenarios.
 TEST_F(WebMClusterParserTest, ParseBlockGroup) {
   const BlockInfo kBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, false, NULL, 0},
-      {kVideoTrackNum, 33, 34, false, NULL, 0},
+      {kAudioTrackNum, 0, 23, false, NULL, 0, true},
+      {kVideoTrackNum, 33, 34, false, NULL, 0, true},
   };
   int block_count = arraysize(kBlockInfo);
 
@@ -568,14 +582,14 @@ TEST_F(WebMClusterParserTest, ParseBlockGroup) {
 
 TEST_F(WebMClusterParserTest, ParseSimpleBlockAndBlockGroupMixture) {
   const BlockInfo kBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 23, false, NULL, 0},
-      {kVideoTrackNum, 33, 34, true, NULL, 0},
-      {kAudioTrackNum, 46, 23, false, NULL, 0},
-      {kVideoTrackNum, 67, 33, false, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 23, false, NULL, 0, false},
+      {kVideoTrackNum, 33, 34, true, NULL, 0, false},
+      {kAudioTrackNum, 46, 23, false, NULL, 0, false},
+      {kVideoTrackNum, 67, 33, false, NULL, 0, false},
   };
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
 
   int result = parser_->Parse(cluster->data(), cluster->size());
   EXPECT_EQ(cluster->size(), result);
@@ -589,25 +603,25 @@ TEST_F(WebMClusterParserTest, IgnoredTracks) {
   parser_.reset(CreateParserWithIgnoredTracks(ignored_tracks));
 
   const BlockInfo kInputBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 23, true, NULL, 0},
-      {kVideoTrackNum, 33, 34, true, NULL, 0},
-      {kTextTrackNum, 33, 99, true, NULL, 0},
-      {kAudioTrackNum, 46, 23, true, NULL, 0},
-      {kVideoTrackNum, 67, 34, true, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 33, 34, true, NULL, 0, false},
+      {kTextTrackNum, 33, 99, true, NULL, 0, false},
+      {kAudioTrackNum, 46, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 67, 34, true, NULL, 0, false},
   };
   int input_block_count = arraysize(kInputBlockInfo);
 
   const BlockInfo kOutputBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 23, true, NULL, 0},
-      {kVideoTrackNum, 33, 34, true, NULL, 0},
-      {kAudioTrackNum, 46, 23, true, NULL, 0},
-      {kVideoTrackNum, 67, 34, true, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 33, 34, true, NULL, 0, false},
+      {kAudioTrackNum, 46, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 67, 34, true, NULL, 0, false},
   };
   int output_block_count = arraysize(kOutputBlockInfo);
 
-  scoped_ptr<Cluster> cluster(
+  std::unique_ptr<Cluster> cluster(
       CreateCluster(0, kInputBlockInfo, input_block_count));
 
   EXPECT_MEDIA_LOG(WebMSimpleBlockDurationEstimated(23));
@@ -628,17 +642,17 @@ TEST_F(WebMClusterParserTest, ParseTextTracks) {
       kNoTimestamp(), kNoTimestamp(), text_tracks));
 
   const BlockInfo kInputBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 23, true, NULL, 0},
-      {kVideoTrackNum, 33, 34, true, NULL, 0},
-      {kTextTrackNum, 33, 42, false, NULL, 0},
-      {kAudioTrackNum, 46, 23, true, NULL, 0},
-      {kTextTrackNum, 55, 44, false, NULL, 0},
-      {kVideoTrackNum, 67, 34, true, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 33, 34, true, NULL, 0, false},
+      {kTextTrackNum, 33, 42, false, NULL, 0, true},
+      {kAudioTrackNum, 46, 23, true, NULL, 0, false},
+      {kTextTrackNum, 55, 44, false, NULL, 0, true},
+      {kVideoTrackNum, 67, 34, true, NULL, 0, false},
   };
   int input_block_count = arraysize(kInputBlockInfo);
 
-  scoped_ptr<Cluster> cluster(
+  std::unique_ptr<Cluster> cluster(
       CreateCluster(0, kInputBlockInfo, input_block_count));
 
   EXPECT_MEDIA_LOG(WebMSimpleBlockDurationEstimated(23));
@@ -663,7 +677,7 @@ TEST_F(WebMClusterParserTest, TextTracksSimpleBlock) {
   };
   int input_block_count = arraysize(kInputBlockInfo);
 
-  scoped_ptr<Cluster> cluster(
+  std::unique_ptr<Cluster> cluster(
       CreateCluster(0, kInputBlockInfo, input_block_count));
 
   int result = parser_->Parse(cluster->data(), cluster->size());
@@ -688,18 +702,18 @@ TEST_F(WebMClusterParserTest, ParseMultipleTextTracks) {
       kNoTimestamp(), kNoTimestamp(), text_tracks));
 
   const BlockInfo kInputBlockInfo[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 23, true, NULL, 0},
-      {kVideoTrackNum, 33, 34, true, NULL, 0},
-      {kSubtitleTextTrackNum, 33, 42, false, NULL, 0},
-      {kAudioTrackNum, 46, 23, true, NULL, 0},
-      {kCaptionTextTrackNum, 55, 44, false, NULL, 0},
-      {kVideoTrackNum, 67, 34, true, NULL, 0},
-      {kSubtitleTextTrackNum, 67, 33, false, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 33, 34, true, NULL, 0, false},
+      {kSubtitleTextTrackNum, 33, 42, false, NULL, 0, false},
+      {kAudioTrackNum, 46, 23, true, NULL, 0, false},
+      {kCaptionTextTrackNum, 55, 44, false, NULL, 0, false},
+      {kVideoTrackNum, 67, 34, true, NULL, 0, false},
+      {kSubtitleTextTrackNum, 67, 33, false, NULL, 0, false},
   };
   int input_block_count = arraysize(kInputBlockInfo);
 
-  scoped_ptr<Cluster> cluster(
+  std::unique_ptr<Cluster> cluster(
       CreateCluster(0, kInputBlockInfo, input_block_count));
 
   EXPECT_MEDIA_LOG(WebMSimpleBlockDurationEstimated(23));
@@ -722,7 +736,8 @@ TEST_F(WebMClusterParserTest, ParseMultipleTextTracks) {
 }
 
 TEST_F(WebMClusterParserTest, ParseEncryptedBlock) {
-  scoped_ptr<Cluster> cluster(CreateEncryptedCluster(sizeof(kEncryptedFrame)));
+  std::unique_ptr<Cluster> cluster(
+      CreateEncryptedCluster(sizeof(kEncryptedFrame)));
 
   parser_.reset(CreateParserWithKeyIdsAndAudioCodec(
       std::string(), "video_key_id", kUnknownAudioCodec));
@@ -739,7 +754,7 @@ TEST_F(WebMClusterParserTest, ParseEncryptedBlock) {
 }
 
 TEST_F(WebMClusterParserTest, ParseBadEncryptedBlock) {
-  scoped_ptr<Cluster> cluster(
+  std::unique_ptr<Cluster> cluster(
       CreateEncryptedCluster(sizeof(kEncryptedFrame) - 1));
 
   parser_.reset(CreateParserWithKeyIdsAndAudioCodec(
@@ -777,10 +792,10 @@ TEST_F(WebMClusterParserTest, ParseInvalidTextBlockGroupWithoutDuration) {
       kNoTimestamp(), kNoTimestamp(), text_tracks));
 
   const BlockInfo kBlockInfo[] = {
-    { kTextTrackNum,  33, -42, false },
+      {kTextTrackNum, 33, -42, false, NULL, 0, false},
   };
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
   int result = parser_->Parse(cluster->data(), cluster->size());
   EXPECT_LT(result, 0);
 }
@@ -793,17 +808,24 @@ TEST_F(WebMClusterParserTest, ParseWithDefaultDurationsSimpleBlocks) {
   EXPECT_LT(kTestVideoFrameDefaultDurationInMs, 33);
 
   const BlockInfo kBlockInfo[] = {
-      {kAudioTrackNum, 0, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kAudioTrackNum, 23, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kVideoTrackNum, 33, kTestVideoFrameDefaultDurationInMs, true, NULL, 0},
-      {kAudioTrackNum, 46, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kVideoTrackNum, 67, kTestVideoFrameDefaultDurationInMs, true, NULL, 0},
-      {kAudioTrackNum, 69, kTestAudioFrameDefaultDurationInMs, true, NULL, 0},
-      {kVideoTrackNum, 100, kTestVideoFrameDefaultDurationInMs, true, NULL, 0},
+      {kAudioTrackNum, 0, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kAudioTrackNum, 23, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kVideoTrackNum, 33, kTestVideoFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kAudioTrackNum, 46, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kVideoTrackNum, 67, kTestVideoFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kAudioTrackNum, 69, kTestAudioFrameDefaultDurationInMs, true, NULL, 0,
+       false},
+      {kVideoTrackNum, 100, kTestVideoFrameDefaultDurationInMs, true, NULL, 0,
+       false},
   };
 
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
 
   // Send slightly less than the full cluster so all but the last block is
   // parsed. Though all the blocks are simple blocks, none should be held aside
@@ -835,17 +857,18 @@ TEST_F(WebMClusterParserTest, ParseWithoutAnyDurationsSimpleBlocks) {
   const int kExpectedAudioEstimationInMs = 22;
   const int kExpectedVideoEstimationInMs = 34;
   const BlockInfo kBlockInfo1[] = {
-      {kAudioTrackNum, 0, 23, true, NULL, 0},
-      {kAudioTrackNum, 23, 22, true, NULL, 0},
-      {kVideoTrackNum, 33, 33, true, NULL, 0},
-      {kAudioTrackNum, 45, 23, true, NULL, 0},
-      {kVideoTrackNum, 66, 34, true, NULL, 0},
-      {kAudioTrackNum, 68, kExpectedAudioEstimationInMs, true, NULL, 0},
-      {kVideoTrackNum, 100, kExpectedVideoEstimationInMs, true, NULL, 0},
+      {kAudioTrackNum, 0, 23, true, NULL, 0, false},
+      {kAudioTrackNum, 23, 22, true, NULL, 0, false},
+      {kVideoTrackNum, 33, 33, true, NULL, 0, false},
+      {kAudioTrackNum, 45, 23, true, NULL, 0, false},
+      {kVideoTrackNum, 66, 34, true, NULL, 0, false},
+      {kAudioTrackNum, 68, kExpectedAudioEstimationInMs, true, NULL, 0, false},
+      {kVideoTrackNum, 100, kExpectedVideoEstimationInMs, true, NULL, 0, false},
   };
 
   int block_count1 = arraysize(kBlockInfo1);
-  scoped_ptr<Cluster> cluster1(CreateCluster(0, kBlockInfo1, block_count1));
+  std::unique_ptr<Cluster> cluster1(
+      CreateCluster(0, kBlockInfo1, block_count1));
 
   // Send slightly less than the first full cluster so all but the last video
   // block is parsed. Verify the last fully parsed audio and video buffer are
@@ -874,13 +897,14 @@ TEST_F(WebMClusterParserTest, ParseWithoutAnyDurationsSimpleBlocks) {
   // each track.
   const BlockInfo kBlockInfo2[] = {
       // Estimate carries over across clusters
-      {kAudioTrackNum, 200, kExpectedAudioEstimationInMs, true, NULL, 0},
+      {kAudioTrackNum, 200, kExpectedAudioEstimationInMs, true, NULL, 0, false},
       // Estimate carries over across clusters
-      {kVideoTrackNum, 201, kExpectedVideoEstimationInMs, true, NULL, 0},
+      {kVideoTrackNum, 201, kExpectedVideoEstimationInMs, true, NULL, 0, false},
   };
 
   int block_count2 = arraysize(kBlockInfo2);
-  scoped_ptr<Cluster> cluster2(CreateCluster(0, kBlockInfo2, block_count2));
+  std::unique_ptr<Cluster> cluster2(
+      CreateCluster(0, kBlockInfo2, block_count2));
   EXPECT_MEDIA_LOG(
       WebMSimpleBlockDurationEstimated(kExpectedAudioEstimationInMs));
   EXPECT_MEDIA_LOG(
@@ -903,17 +927,20 @@ TEST_F(WebMClusterParserTest, ParseWithoutAnyDurationsBlockGroups) {
   const int kExpectedAudioEstimationInMs = 22;
   const int kExpectedVideoEstimationInMs = 34;
   const BlockInfo kBlockInfo1[] = {
-      {kAudioTrackNum, 0, -23, false, NULL, 0},
-      {kAudioTrackNum, 23, -22, false, NULL, 0},
-      {kVideoTrackNum, 33, -33, false, NULL, 0},
-      {kAudioTrackNum, 45, -23, false, NULL, 0},
-      {kVideoTrackNum, 66, -34, false, NULL, 0},
-      {kAudioTrackNum, 68, -kExpectedAudioEstimationInMs, false, NULL, 0},
-      {kVideoTrackNum, 100, -kExpectedVideoEstimationInMs, false, NULL, 0},
+      {kAudioTrackNum, 0, -23, false, NULL, 0, false},
+      {kAudioTrackNum, 23, -22, false, NULL, 0, false},
+      {kVideoTrackNum, 33, -33, false, NULL, 0, false},
+      {kAudioTrackNum, 45, -23, false, NULL, 0, false},
+      {kVideoTrackNum, 66, -34, false, NULL, 0, false},
+      {kAudioTrackNum, 68, -kExpectedAudioEstimationInMs, false, NULL, 0,
+       false},
+      {kVideoTrackNum, 100, -kExpectedVideoEstimationInMs, false, NULL, 0,
+       false},
   };
 
   int block_count1 = arraysize(kBlockInfo1);
-  scoped_ptr<Cluster> cluster1(CreateCluster(0, kBlockInfo1, block_count1));
+  std::unique_ptr<Cluster> cluster1(
+      CreateCluster(0, kBlockInfo1, block_count1));
 
   // Send slightly less than the first full cluster so all but the last video
   // block is parsed. Verify the last fully parsed audio and video buffer are
@@ -941,12 +968,15 @@ TEST_F(WebMClusterParserTest, ParseWithoutAnyDurationsBlockGroups) {
   // Verify that the estimated frame duration is tracked across clusters for
   // each track.
   const BlockInfo kBlockInfo2[] = {
-      {kAudioTrackNum, 200, -kExpectedAudioEstimationInMs, false, NULL, 0},
-      {kVideoTrackNum, 201, -kExpectedVideoEstimationInMs, false, NULL, 0},
+      {kAudioTrackNum, 200, -kExpectedAudioEstimationInMs, false, NULL, 0,
+       false},
+      {kVideoTrackNum, 201, -kExpectedVideoEstimationInMs, false, NULL, 0,
+       false},
   };
 
   int block_count2 = arraysize(kBlockInfo2);
-  scoped_ptr<Cluster> cluster2(CreateCluster(0, kBlockInfo2, block_count2));
+  std::unique_ptr<Cluster> cluster2(
+      CreateCluster(0, kBlockInfo2, block_count2));
   EXPECT_MEDIA_LOG(
       WebMSimpleBlockDurationEstimated(kExpectedAudioEstimationInMs));
   EXPECT_MEDIA_LOG(
@@ -966,22 +996,24 @@ TEST_F(WebMClusterParserTest,
   EXPECT_LT(kTestVideoFrameDefaultDurationInMs, 33);
 
   const BlockInfo kBlockInfo[] = {
-      {kAudioTrackNum, 0, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0},
-      {kAudioTrackNum, 23, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0},
-      {kVideoTrackNum, 33, -kTestVideoFrameDefaultDurationInMs, false, NULL, 0},
-      {kAudioTrackNum, 46, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0},
-      {kVideoTrackNum, 67, -kTestVideoFrameDefaultDurationInMs, false, NULL, 0},
-      {kAudioTrackNum, 69, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0},
-      {kVideoTrackNum,
-       100,
-       -kTestVideoFrameDefaultDurationInMs,
-       false,
-       NULL,
-       0},
+      {kAudioTrackNum, 0, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kAudioTrackNum, 23, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kVideoTrackNum, 33, -kTestVideoFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kAudioTrackNum, 46, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kVideoTrackNum, 67, -kTestVideoFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kAudioTrackNum, 69, -kTestAudioFrameDefaultDurationInMs, false, NULL, 0,
+       false},
+      {kVideoTrackNum, 100, -kTestVideoFrameDefaultDurationInMs, false, NULL, 0,
+       false},
   };
 
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
 
   // Send slightly less than the full cluster so all but the last block is
   // parsed. None should be held aside for duration estimation prior to end of
@@ -1016,7 +1048,7 @@ TEST_F(WebMClusterParserTest,
   };
 
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
   EXPECT_MEDIA_LOG(WebMSimpleBlockDurationEstimated(
       WebMClusterParser::kDefaultAudioBufferDurationInMs));
   EXPECT_MEDIA_LOG(WebMSimpleBlockDurationEstimated(
@@ -1036,7 +1068,7 @@ TEST_F(WebMClusterParserTest,
   };
 
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
   int result = parser_->Parse(cluster->data(), cluster->size());
   EXPECT_EQ(cluster->size(), result);
   ASSERT_TRUE(VerifyBuffers(parser_, kBlockInfo, block_count));
@@ -1059,7 +1091,7 @@ TEST_F(WebMClusterParserTest, ReadOpusDurationsSimpleBlockAtEndOfCluster) {
                                      packet_ptr->size()}};
 
     int block_count = arraysize(kBlockInfo);
-    scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+    std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
     int duration_ms = packet_ptr->duration_ms();  // Casts from double.
     if (duration_ms > 120) {
       EXPECT_MEDIA_LOG(OpusPacketDurationTooHigh(duration_ms));
@@ -1105,7 +1137,8 @@ TEST_F(WebMClusterParserTest, PreferOpusDurationsOverBlockDurations) {
                                 packet_ptr->size()}};
 
     int block_count = arraysize(block_infos);
-    scoped_ptr<Cluster> cluster(CreateCluster(0, block_infos, block_count));
+    std::unique_ptr<Cluster> cluster(
+        CreateCluster(0, block_infos, block_count));
     int result = parser_->Parse(cluster->data(), cluster->size());
     EXPECT_EQ(cluster->size(), result);
 
@@ -1145,7 +1178,7 @@ TEST_F(WebMClusterParserTest, DontReadEncodedDurationWhenEncrypted) {
                                    arraysize(kEncryptedFrame)}};
 
   int block_count = arraysize(kBlockInfo);
-  scoped_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
+  std::unique_ptr<Cluster> cluster(CreateCluster(0, kBlockInfo, block_count));
   int result = parser_->Parse(cluster->data(), cluster->size());
   EXPECT_EQ(cluster->size(), result);
 
