@@ -7,19 +7,19 @@
 
 #include <stdint.h>
 
+#include "base/callback.h"
 #include "base/macros.h"
 #include "media/base/cdm_promise.h"
+#include "media/mojo/common/mojo_type_trait.h"
 #include "media/mojo/interfaces/content_decryption_module.mojom.h"
-#include "media/mojo/services/mojo_type_trait.h"
-#include "mojo/public/cpp/bindings/callback.h"
 
 namespace media {
 
-// media::CdmPromiseTemplate implementations backed by mojo::Callbacks.
+// media::CdmPromiseTemplate implementations backed by base::Callbacks.
 template <typename... T>
 class MojoCdmPromise : public CdmPromiseTemplate<T...> {
  public:
-  typedef mojo::Callback<void(interfaces::CdmPromiseResultPtr,
+  typedef base::Callback<void(mojom::CdmPromiseResultPtr,
                               typename MojoTypeTrait<T>::MojoType...)>
       CallbackType;
 
@@ -33,7 +33,9 @@ class MojoCdmPromise : public CdmPromiseTemplate<T...> {
               const std::string& error_message) final;
 
  private:
-  using media::CdmPromiseTemplate<T...>::MarkPromiseSettled;
+  using CdmPromiseTemplate<T...>::IsPromiseSettled;
+  using CdmPromiseTemplate<T...>::MarkPromiseSettled;
+  using CdmPromiseTemplate<T...>::RejectPromiseOnDestruction;
 
   CallbackType callback_;
 };
