@@ -9,7 +9,7 @@
 namespace media {
 
 MojoProvisionFetcher::MojoProvisionFetcher(
-    interfaces::ProvisionFetcherPtr provision_fetcher_ptr)
+    mojom::ProvisionFetcherPtr provision_fetcher_ptr)
     : provision_fetcher_ptr_(std::move(provision_fetcher_ptr)),
       weak_factory_(this) {
   DVLOG(1) << __FUNCTION__;
@@ -30,7 +30,7 @@ void MojoProvisionFetcher::Retrieve(const std::string& default_url,
 
 void MojoProvisionFetcher::OnResponse(const ResponseCB& response_cb,
                                       bool success,
-                                      const std::string& response) {
+                                      mojo::String response) {
   response_cb.Run(success, response);
 }
 
