@@ -12,54 +12,42 @@
 #include "base/bind.h"
 #include "base/callback.h"
 #include "base/callback_helpers.h"
+#include "base/command_line.h"
 #include "base/logging.h"
-#include "base/metrics/histogram.h"
+#include "base/power_monitor/power_monitor.h"
 #include "base/single_thread_task_runner.h"
 #include "base/time/default_tick_clock.h"
 #include "build/build_config.h"
 #include "media/base/audio_buffer.h"
 #include "media/base/audio_buffer_converter.h"
-#include "media/base/audio_hardware_config.h"
+#include "media/base/audio_latency.h"
 #include "media/base/audio_splicer.h"
 #include "media/base/bind_to_current_loop.h"
 #include "media/base/demuxer_stream.h"
 #include "media/base/media_log.h"
+#include "media/base/media_switches.h"
+#include "media/base/renderer_client.h"
 #include "media/base/timestamp_constants.h"
 #include "media/filters/audio_clock.h"
 #include "media/filters/decrypting_demuxer_stream.h"
 
 namespace media {
 
-namespace {
-
-enum AudioRendererEvent {
-  INITIALIZED,
-  RENDER_ERROR,
-  RENDER_EVENT_MAX = RENDER_ERROR,
-};
-
-void HistogramRendererEvent(AudioRendererEvent event) {
-  UMA_HISTOGRAM_ENUMERATION(
-      "Media.AudioRendererEvents", event, RENDER_EVENT_MAX + 1);
-}
-
-}  // namespace
-
 AudioRendererImpl::AudioRendererImpl(
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
     media::AudioRendererSink* sink,
     ScopedVector<AudioDecoder> decoders,
-    const AudioHardwareConfig& hardware_config,
     const scoped_refptr<MediaLog>& media_log)
     : task_runner_(task_runner),
       expecting_config_changes_(false),
       sink_(sink),
       audio_buffer_stream_(
           new AudioBufferStream(task_runner, std::move(decoders), media_log)),
-      hardware_config_(hardware_config),
       media_log_(media_log),
+      client_(nullptr),
       tick_clock_(new base::DefaultTickClock()),
       last_audio_memory_usage_(0),
+      last_decoded_sample_rate_(0),
       playback_rate_(0.0),
       state_(kUninitialized),
       buffering_state_(BUFFERING_HAVE_NOTHING),
@@ -68,16 +56,47 @@ AudioRendererImpl::AudioRendererImpl(
       pending_read_(false),
       received_end_of_stream_(false),
       rendered_end_of_stream_(false),
+      is_suspending_(false),
       weak_factory_(this) {
   audio_buffer_stream_->set_splice_observer(base::Bind(
       &AudioRendererImpl::OnNewSpliceBuffer, weak_factory_.GetWeakPtr()));
   audio_buffer_stream_->set_config_change_observer(base::Bind(
       &AudioRendererImpl::OnConfigChange, weak_factory_.GetWeakPtr()));
+
+// Suspend and resume work differently on Android and are handled at a higher
+// level than here. OnSuspend() notifications will be delivered a few seconds
+// after an application is backgrounded, even if it should still be playing.
+// See http://crbug.com/623066 for more details.
+#if !defined(OS_ANDROID)
+  // Tests may not have a power monitor.
+  base::PowerMonitor* monitor = base::PowerMonitor::Get();
+  if (!monitor)
+    return;
+
+  // PowerObserver's must be added and removed from the same thread, but we
+  // won't remove the observer until we're destructed on |task_runner_| so we
+  // must post it here if we're on the wrong thread.
+  if (task_runner_->BelongsToCurrentThread()) {
+    monitor->AddObserver(this);
+  } else {
+    // Safe to post this without a WeakPtr because this class must be destructed
+    // on the same thread and construction has not completed yet.
+    task_runner_->PostTask(FROM_HERE,
+                           base::Bind(&base::PowerMonitor::AddObserver,
+                                      base::Unretained(monitor), this));
+  }
+#endif
+  // Do not add anything below this line since the above actions are only safe
+  // as the last lines of the constructor.
 }
 
 AudioRendererImpl::~AudioRendererImpl() {
   DVLOG(1) << __FUNCTION__;
   DCHECK(task_runner_->BelongsToCurrentThread());
+#if !defined(OS_ANDROID)
+  if (base::PowerMonitor::Get())
+    base::PowerMonitor::Get()->RemoveObserver(this);
+#endif
 
   // If Render() is in progress, this call will wait for Render() to finish.
   // After this call, the |sink_| will not call back into |this| anymore.
@@ -158,20 +177,37 @@ void AudioRendererImpl::SetMediaTime(base::TimeDelta time) {
   ended_timestamp_ = kInfiniteDuration();
   last_render_time_ = stop_rendering_time_ = base::TimeTicks();
   first_packet_timestamp_ = kNoTimestamp();
+  last_media_timestamp_ = base::TimeDelta();
   audio_clock_.reset(new AudioClock(time, audio_parameters_.sample_rate()));
 }
 
 base::TimeDelta AudioRendererImpl::CurrentMediaTime() {
-  // In practice the Render() method is called with a high enough frequency
-  // that returning only the front timestamp is good enough and also prevents
-  // returning values that go backwards in time.
-  base::TimeDelta current_media_time;
-  {
-    base::AutoLock auto_lock(lock_);
-    current_media_time = audio_clock_->front_timestamp();
+  base::AutoLock auto_lock(lock_);
+
+  // Return the current time based on the known extents of the rendered audio
+  // data plus an estimate based on the last time those values were calculated.
+  base::TimeDelta current_media_time = audio_clock_->front_timestamp();
+  if (!last_render_time_.is_null()) {
+    current_media_time +=
+        (tick_clock_->NowTicks() - last_render_time_) * playback_rate_;
+    if (current_media_time > audio_clock_->back_timestamp())
+      current_media_time = audio_clock_->back_timestamp();
+  }
+
+  // Clamp current media time to the last reported value, this prevents higher
+  // level clients from seeing time go backwards based on inaccurate or spurious
+  // delay values reported to the AudioClock.
+  //
+  // It is expected that such events are transient and will be recovered as
+  // rendering continues over time.
+  if (current_media_time < last_media_timestamp_) {
+    DVLOG(2) << __FUNCTION__ << ": " << last_media_timestamp_
+             << " (clamped), actual: " << current_media_time;
+    return last_media_timestamp_;
   }
 
   DVLOG(2) << __FUNCTION__ << ": " << current_media_time;
+  last_media_timestamp_ = current_media_time;
   return current_media_time;
 }
 
@@ -185,7 +221,7 @@ bool AudioRendererImpl::GetWallClockTimes(
   const double playback_rate = playback_rate_ ? playback_rate_ : 1.0;
   const bool is_time_moving = sink_playing_ && playback_rate_ &&
                               !last_render_time_.is_null() &&
-                              stop_rendering_time_.is_null();
+                              stop_rendering_time_.is_null() && !is_suspending_;
 
   // Pre-compute the time until playback of the audio buffer extents, since
   // these values are frequently used below.
@@ -307,39 +343,28 @@ void AudioRendererImpl::StartPlaying() {
   AttemptRead_Locked();
 }
 
-void AudioRendererImpl::Initialize(
-    DemuxerStream* stream,
-    const PipelineStatusCB& init_cb,
-    const SetCdmReadyCB& set_cdm_ready_cb,
-    const StatisticsCB& statistics_cb,
-    const BufferingStateCB& buffering_state_cb,
-    const base::Closure& ended_cb,
-    const PipelineStatusCB& error_cb,
-    const base::Closure& waiting_for_decryption_key_cb) {
+void AudioRendererImpl::Initialize(DemuxerStream* stream,
+                                   CdmContext* cdm_context,
+                                   RendererClient* client,
+                                   const PipelineStatusCB& init_cb) {
   DVLOG(1) << __FUNCTION__;
   DCHECK(task_runner_->BelongsToCurrentThread());
+  DCHECK(client);
   DCHECK(stream);
   DCHECK_EQ(stream->type(), DemuxerStream::AUDIO);
   DCHECK(!init_cb.is_null());
-  DCHECK(!statistics_cb.is_null());
-  DCHECK(!buffering_state_cb.is_null());
-  DCHECK(!ended_cb.is_null());
-  DCHECK(!error_cb.is_null());
   DCHECK_EQ(kUninitialized, state_);
   DCHECK(sink_.get());
 
   state_ = kInitializing;
+  client_ = client;
 
   // Always post |init_cb_| because |this| could be destroyed if initialization
   // failed.
   init_cb_ = BindToCurrentLoop(init_cb);
 
-  buffering_state_cb_ = buffering_state_cb;
-  ended_cb_ = ended_cb;
-  error_cb_ = error_cb;
-  statistics_cb_ = statistics_cb;
-
-  const AudioParameters& hw_params = hardware_config_.GetOutputConfig();
+  const AudioParameters& hw_params =
+      sink_->GetOutputDeviceInfo().output_params();
   expecting_config_changes_ = stream->SupportsConfigChanges();
   if (!expecting_config_changes_ || !hw_params.IsValid() ||
       hw_params.format() == AudioParameters::AUDIO_FAKE) {
@@ -354,24 +379,71 @@ void AudioRendererImpl::Initialize(
         buffer_size);
     buffer_converter_.reset();
   } else {
-    audio_parameters_.Reset(
-        hw_params.format(),
-        // Always use the source's channel layout to avoid premature downmixing
-        // (http://crbug.com/379288), platform specific issues around channel
-        // layouts (http://crbug.com/266674), and unnecessary upmixing overhead.
-        stream->audio_decoder_config().channel_layout(),
-#if defined(OS_CHROMEOS) || defined(OS_ANDROID)
-        // On ChromeOS and Android let the OS level resampler handle resampling
-        // unless the initial sample rate is too low; this allows support for
-        // sample rate adaptations where necessary.
-        stream->audio_decoder_config().samples_per_second() < 44100
-            ? hw_params.sample_rate()
-            : stream->audio_decoder_config().samples_per_second(),
-#else
-        hw_params.sample_rate(),
+    // To allow for seamless sample rate adaptations (i.e. changes from say
+    // 16kHz to 48kHz), always resample to the hardware rate.
+    int sample_rate = hw_params.sample_rate();
+    int preferred_buffer_size = hw_params.frames_per_buffer();
+
+#if defined(OS_CHROMEOS)
+    // On ChromeOS let the OS level resampler handle resampling unless the
+    // initial sample rate is too low; this allows support for sample rate
+    // adaptations where necessary.
+    if (stream->audio_decoder_config().samples_per_second() >= 44100) {
+      sample_rate = stream->audio_decoder_config().samples_per_second();
+      preferred_buffer_size = 0;  // No preference.
+    }
 #endif
-        hw_params.bits_per_sample(),
-        hardware_config_.GetHighLatencyBufferSize());
+
+    int stream_channel_count = ChannelLayoutToChannelCount(
+        stream->audio_decoder_config().channel_layout());
+
+    bool try_supported_channel_layouts = false;
+#if defined(OS_WIN)
+    try_supported_channel_layouts =
+        base::CommandLine::ForCurrentProcess()->HasSwitch(
+            switches::kTrySupportedChannelLayouts);
+#endif
+
+    // We don't know how to up-mix for DISCRETE layouts (fancy multichannel
+    // hardware with non-standard speaker arrangement). Instead, pretend the
+    // hardware layout is stereo and let the OS take care of further up-mixing
+    // to the discrete layout (http://crbug.com/266674). Additionally, pretend
+    // hardware is stereo whenever kTrySupportedChannelLayouts is set. This flag
+    // is for savvy users who want stereo content to output in all surround
+    // speakers. Using the actual layout (likely 5.1 or higher) will mean our
+    // mixer will attempt to up-mix stereo source streams to just the left/right
+    // speaker of the 5.1 setup, nulling out the other channels
+    // (http://crbug.com/177872).
+    ChannelLayout hw_channel_layout =
+        hw_params.channel_layout() == CHANNEL_LAYOUT_DISCRETE ||
+                try_supported_channel_layouts
+            ? CHANNEL_LAYOUT_STEREO
+            : hw_params.channel_layout();
+    int hw_channel_count = ChannelLayoutToChannelCount(hw_channel_layout);
+
+    // The layout we pass to |audio_parameters_| will be used for the lifetime
+    // of this audio renderer, regardless of changes to hardware and/or stream
+    // properties. Below we choose the max of stream layout vs. hardware layout
+    // to leave room for changes to the hardware and/or stream (i.e. avoid
+    // premature down-mixing - http://crbug.com/379288).
+    // If stream_channels < hw_channels:
+    //   Taking max means we up-mix to hardware layout. If stream later changes
+    //   to have more channels, we aren't locked into down-mixing to the
+    //   initial stream layout.
+    // If stream_channels > hw_channels:
+    //   We choose to output stream's layout, meaning mixing is a no-op for the
+    //   renderer. Browser-side will down-mix to the hardware config. If the
+    //   hardware later changes to equal stream channels, browser-side will stop
+    //   down-mixing and use the data from all stream channels.
+    ChannelLayout renderer_channel_layout =
+        hw_channel_count > stream_channel_count
+            ? hw_channel_layout
+            : stream->audio_decoder_config().channel_layout();
+
+    audio_parameters_.Reset(hw_params.format(), renderer_channel_layout,
+                            sample_rate, hw_params.bits_per_sample(),
+                            media::AudioLatency::GetHighLatencyBufferSize(
+                                sample_rate, preferred_buffer_size));
   }
 
   audio_clock_.reset(
@@ -380,7 +452,10 @@ void AudioRendererImpl::Initialize(
   audio_buffer_stream_->Initialize(
       stream, base::Bind(&AudioRendererImpl::OnAudioBufferStreamInitialized,
                          weak_factory_.GetWeakPtr()),
-      set_cdm_ready_cb, statistics_cb, waiting_for_decryption_key_cb);
+      cdm_context, base::Bind(&AudioRendererImpl::OnStatisticsUpdate,
+                              weak_factory_.GetWeakPtr()),
+      base::Bind(&AudioRendererImpl::OnWaitingForDecryptionKey,
+                 weak_factory_.GetWeakPtr()));
 }
 
 void AudioRendererImpl::OnAudioBufferStreamInitialized(bool success) {
@@ -414,8 +489,6 @@ void AudioRendererImpl::OnAudioBufferStreamInitialized(bool success) {
 
   ChangeState_Locked(kFlushed);
 
-  HistogramRendererEvent(INITIALIZED);
-
   {
     base::AutoUnlock auto_unlock(lock_);
     sink_->Initialize(audio_parameters_, this);
@@ -429,12 +502,47 @@ void AudioRendererImpl::OnAudioBufferStreamInitialized(bool success) {
   base::ResetAndReturn(&init_cb_).Run(PIPELINE_OK);
 }
 
+void AudioRendererImpl::OnPlaybackError(PipelineStatus error) {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnError(error);
+}
+
+void AudioRendererImpl::OnPlaybackEnded() {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnEnded();
+}
+
+void AudioRendererImpl::OnStatisticsUpdate(const PipelineStatistics& stats) {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnStatisticsUpdate(stats);
+}
+
+void AudioRendererImpl::OnBufferingStateChange(BufferingState state) {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnBufferingStateChange(state);
+}
+
+void AudioRendererImpl::OnWaitingForDecryptionKey() {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnWaitingForDecryptionKey();
+}
+
 void AudioRendererImpl::SetVolume(float volume) {
   DCHECK(task_runner_->BelongsToCurrentThread());
   DCHECK(sink_.get());
   sink_->SetVolume(volume);
 }
 
+void AudioRendererImpl::OnSuspend() {
+  base::AutoLock auto_lock(lock_);
+  is_suspending_ = true;
+}
+
+void AudioRendererImpl::OnResume() {
+  base::AutoLock auto_lock(lock_);
+  is_suspending_ = false;
+}
+
 void AudioRendererImpl::DecodedAudioReady(
     AudioBufferStream::Status status,
     const scoped_refptr<AudioBuffer>& buffer) {
@@ -449,12 +557,12 @@ void AudioRendererImpl::DecodedAudioReady(
 
   if (status == AudioBufferStream::ABORTED ||
       status == AudioBufferStream::DEMUXER_READ_ABORTED) {
-    HandleAbortedReadOrDecodeError(false);
+    HandleAbortedReadOrDecodeError(PIPELINE_OK);
     return;
   }
 
   if (status == AudioBufferStream::DECODE_ERROR) {
-    HandleAbortedReadOrDecodeError(true);
+    HandleAbortedReadOrDecodeError(PIPELINE_ERROR_DECODE);
     return;
   }
 
@@ -468,17 +576,47 @@ void AudioRendererImpl::DecodedAudioReady(
   }
 
   if (expecting_config_changes_) {
+    if (last_decoded_sample_rate_ &&
+        buffer->sample_rate() != last_decoded_sample_rate_) {
+      DVLOG(1) << __FUNCTION__ << " Updating audio sample_rate."
+               << " ts:" << buffer->timestamp().InMicroseconds()
+               << " old:" << last_decoded_sample_rate_
+               << " new:" << buffer->sample_rate();
+      OnConfigChange();
+    }
+    last_decoded_sample_rate_ = buffer->sample_rate();
+
     DCHECK(buffer_converter_);
     buffer_converter_->AddInput(buffer);
     while (buffer_converter_->HasNextBuffer()) {
       if (!splicer_->AddInput(buffer_converter_->GetNextBuffer())) {
-        HandleAbortedReadOrDecodeError(true);
+        HandleAbortedReadOrDecodeError(AUDIO_RENDERER_ERROR_SPLICE_FAILED);
         return;
       }
     }
   } else {
+    // TODO(chcunningham, tguilbert): Figure out if we want to support implicit
+    // config changes during src=. Doing so requires resampling each individual
+    // stream which is inefficient when there are many tags in a page.
+    //
+    // Check if the buffer we received matches the expected configuration.
+    // Note: We explicitly do not check channel layout here to avoid breaking
+    // weird behavior with multichannel wav files: http://crbug.com/600538.
+    if (!buffer->end_of_stream() &&
+        (buffer->sample_rate() != audio_parameters_.sample_rate() ||
+         buffer->channel_count() != audio_parameters_.channels())) {
+      MEDIA_LOG(ERROR, media_log_)
+          << "Unsupported midstream configuration change!"
+          << " Sample Rate: " << buffer->sample_rate() << " vs "
+          << audio_parameters_.sample_rate()
+          << ", Channels: " << buffer->channel_count() << " vs "
+          << audio_parameters_.channels();
+      HandleAbortedReadOrDecodeError(PIPELINE_ERROR_DECODE);
+      return;
+    }
+
     if (!splicer_->AddInput(buffer)) {
-      HandleAbortedReadOrDecodeError(true);
+      HandleAbortedReadOrDecodeError(AUDIO_RENDERER_ERROR_SPLICE_FAILED);
       return;
     }
   }
@@ -533,7 +671,9 @@ bool AudioRendererImpl::HandleSplicerBuffer_Locked(
   PipelineStatistics stats;
   stats.audio_memory_usage = memory_usage - last_audio_memory_usage_;
   last_audio_memory_usage_ = memory_usage;
-  task_runner_->PostTask(FROM_HERE, base::Bind(statistics_cb_, stats));
+  task_runner_->PostTask(FROM_HERE,
+                         base::Bind(&AudioRendererImpl::OnStatisticsUpdate,
+                                    weak_factory_.GetWeakPtr(), stats));
 
   switch (state_) {
     case kUninitialized:
@@ -628,13 +768,13 @@ bool AudioRendererImpl::IsBeforeStartTime(
 }
 
 int AudioRendererImpl::Render(AudioBus* audio_bus,
-                              uint32_t audio_delay_milliseconds,
+                              uint32_t frames_delayed,
                               uint32_t frames_skipped) {
-  const int requested_frames = audio_bus->frames();
-  base::TimeDelta playback_delay = base::TimeDelta::FromMilliseconds(
-      audio_delay_milliseconds);
-  const int delay_frames = static_cast<int>(playback_delay.InSecondsF() *
-                                            audio_parameters_.sample_rate());
+  const int frames_requested = audio_bus->frames();
+  DVLOG(4) << __FUNCTION__ << " frames_delayed:" << frames_delayed
+           << " frames_skipped:" << frames_skipped
+           << " frames_requested:" << frames_requested;
+
   int frames_written = 0;
   {
     base::AutoLock auto_lock(lock_);
@@ -642,27 +782,27 @@ int AudioRendererImpl::Render(AudioBus* audio_bus,
 
     if (!stop_rendering_time_.is_null()) {
       audio_clock_->CompensateForSuspendedWrites(
-          last_render_time_ - stop_rendering_time_, delay_frames);
+          last_render_time_ - stop_rendering_time_, frames_delayed);
       stop_rendering_time_ = base::TimeTicks();
     }
 
     // Ensure Stop() hasn't destroyed our |algorithm_| on the pipeline thread.
     if (!algorithm_) {
-      audio_clock_->WroteAudio(
-          0, requested_frames, delay_frames, playback_rate_);
+      audio_clock_->WroteAudio(0, frames_requested, frames_delayed,
+                               playback_rate_);
       return 0;
     }
 
-    if (playback_rate_ == 0) {
-      audio_clock_->WroteAudio(
-          0, requested_frames, delay_frames, playback_rate_);
+    if (playback_rate_ == 0 || is_suspending_) {
+      audio_clock_->WroteAudio(0, frames_requested, frames_delayed,
+                               playback_rate_);
       return 0;
     }
 
     // Mute audio by returning 0 when not playing.
     if (state_ != kPlaying) {
-      audio_clock_->WroteAudio(
-          0, requested_frames, delay_frames, playback_rate_);
+      audio_clock_->WroteAudio(0, frames_requested, frames_delayed,
+                               playback_rate_);
       return 0;
     }
 
@@ -673,24 +813,28 @@ int AudioRendererImpl::Render(AudioBus* audio_bus,
       CHECK_GE(first_packet_timestamp_, base::TimeDelta());
       const base::TimeDelta play_delay =
           first_packet_timestamp_ - audio_clock_->back_timestamp();
-      CHECK_LT(play_delay.InSeconds(), 1000)
-          << "first_packet_timestamp_ = " << first_packet_timestamp_
-          << ", audio_clock_->back_timestamp() = "
-          << audio_clock_->back_timestamp();
       if (play_delay > base::TimeDelta()) {
         DCHECK_EQ(frames_written, 0);
-        frames_written =
-            std::min(static_cast<int>(play_delay.InSecondsF() *
-                                      audio_parameters_.sample_rate()),
-                     requested_frames);
+
+        // Don't multiply |play_delay| out since it can be a huge value on
+        // poorly encoded media and multiplying by the sample rate could cause
+        // the value to overflow.
+        if (play_delay.InSecondsF() > static_cast<double>(frames_requested) /
+                                          audio_parameters_.sample_rate()) {
+          frames_written = frames_requested;
+        } else {
+          frames_written =
+              play_delay.InSecondsF() * audio_parameters_.sample_rate();
+        }
+
         audio_bus->ZeroFramesPartial(0, frames_written);
       }
 
       // If there's any space left, actually render the audio; this is where the
       // aural magic happens.
-      if (frames_written < requested_frames) {
+      if (frames_written < frames_requested) {
         frames_written += algorithm_->FillBuffer(
-            audio_bus, frames_written, requested_frames - frames_written,
+            audio_bus, frames_written, frames_requested - frames_written,
             playback_rate_);
       }
     }
@@ -721,7 +865,7 @@ int AudioRendererImpl::Render(AudioBus* audio_bus,
       if (received_end_of_stream_) {
         if (ended_timestamp_ == kInfiniteDuration())
           ended_timestamp_ = audio_clock_->back_timestamp();
-        frames_after_end_of_stream = requested_frames;
+        frames_after_end_of_stream = frames_requested;
       } else if (state_ == kPlaying &&
                  buffering_state_ != BUFFERING_HAVE_NOTHING) {
         algorithm_->IncreaseQueueCapacity();
@@ -730,9 +874,7 @@ int AudioRendererImpl::Render(AudioBus* audio_bus,
     }
 
     audio_clock_->WroteAudio(frames_written + frames_after_end_of_stream,
-                             requested_frames,
-                             delay_frames,
-                             playback_rate_);
+                             frames_requested, frames_delayed, playback_rate_);
 
     if (CanRead_Locked()) {
       task_runner_->PostTask(FROM_HERE,
@@ -743,33 +885,29 @@ int AudioRendererImpl::Render(AudioBus* audio_bus,
     if (audio_clock_->front_timestamp() >= ended_timestamp_ &&
         !rendered_end_of_stream_) {
       rendered_end_of_stream_ = true;
-      task_runner_->PostTask(FROM_HERE, ended_cb_);
+      task_runner_->PostTask(FROM_HERE,
+                             base::Bind(&AudioRendererImpl::OnPlaybackEnded,
+                                        weak_factory_.GetWeakPtr()));
     }
   }
 
-  DCHECK_LE(frames_written, requested_frames);
+  DCHECK_LE(frames_written, frames_requested);
   return frames_written;
 }
 
 void AudioRendererImpl::OnRenderError() {
-  // UMA data tells us this happens ~0.01% of the time. Trigger an error instead
-  // of trying to gracefully fall back to a fake sink. It's very likely
-  // OnRenderError() should be removed and the audio stack handle errors without
-  // notifying clients. See http://crbug.com/234708 for details.
-  HistogramRendererEvent(RENDER_ERROR);
-
   MEDIA_LOG(ERROR, media_log_) << "audio render error";
 
   // Post to |task_runner_| as this is called on the audio callback thread.
-  task_runner_->PostTask(FROM_HERE,
-                         base::Bind(error_cb_, PIPELINE_ERROR_DECODE));
+  task_runner_->PostTask(
+      FROM_HERE, base::Bind(&AudioRendererImpl::OnPlaybackError,
+                            weak_factory_.GetWeakPtr(), AUDIO_RENDERER_ERROR));
 }
 
-void AudioRendererImpl::HandleAbortedReadOrDecodeError(bool is_decode_error) {
+void AudioRendererImpl::HandleAbortedReadOrDecodeError(PipelineStatus status) {
   DCHECK(task_runner_->BelongsToCurrentThread());
   lock_.AssertAcquired();
 
-  PipelineStatus status = is_decode_error ? PIPELINE_ERROR_DECODE : PIPELINE_OK;
   switch (state_) {
     case kUninitialized:
     case kInitializing:
@@ -782,16 +920,19 @@ void AudioRendererImpl::HandleAbortedReadOrDecodeError(bool is_decode_error) {
         return;
       }
 
-      MEDIA_LOG(ERROR, media_log_) << "audio decode error during flushing";
-      error_cb_.Run(status);
+      MEDIA_LOG(ERROR, media_log_) << "audio error during flushing, status: "
+                                   << MediaLog::PipelineStatusToString(status);
+      client_->OnError(status);
       base::ResetAndReturn(&flush_cb_).Run();
       return;
 
     case kFlushed:
     case kPlaying:
       if (status != PIPELINE_OK) {
-        MEDIA_LOG(ERROR, media_log_) << "audio decode error during playing";
-        error_cb_.Run(status);
+        MEDIA_LOG(ERROR, media_log_)
+            << "audio error during playing, status: "
+            << MediaLog::PipelineStatusToString(status);
+        client_->OnError(status);
       }
       return;
   }
@@ -827,8 +968,9 @@ void AudioRendererImpl::SetBufferingState_Locked(
   lock_.AssertAcquired();
   buffering_state_ = buffering_state;
 
-  task_runner_->PostTask(FROM_HERE,
-                         base::Bind(buffering_state_cb_, buffering_state_));
+  task_runner_->PostTask(
+      FROM_HERE, base::Bind(&AudioRendererImpl::OnBufferingStateChange,
+                            weak_factory_.GetWeakPtr(), buffering_state_));
 }
 
 }  // namespace media
