@@ -14,11 +14,14 @@
 #include "base/location.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
+#include "media/base/audio_decoder_config.h"
 #include "media/base/audio_renderer.h"
 #include "media/base/bind_to_current_loop.h"
 #include "media/base/demuxer_stream_provider.h"
 #include "media/base/media_switches.h"
+#include "media/base/renderer_client.h"
 #include "media/base/time_source.h"
+#include "media/base/video_decoder_config.h"
 #include "media/base/video_renderer.h"
 #include "media/base/wall_clock_time_source.h"
 
@@ -27,10 +30,42 @@ namespace media {
 // See |video_underflow_threshold_|.
 static const int kDefaultVideoUnderflowThresholdMs = 3000;
 
+class RendererImpl::RendererClientInternal : public RendererClient {
+ public:
+  RendererClientInternal(DemuxerStream::Type type, RendererImpl* renderer)
+      : type_(type), renderer_(renderer) {
+    DCHECK((type_ == DemuxerStream::AUDIO) || (type_ == DemuxerStream::VIDEO));
+  }
+
+  void OnError(PipelineStatus error) override { renderer_->OnError(error); }
+  void OnEnded() override { renderer_->OnRendererEnded(type_); }
+  void OnStatisticsUpdate(const PipelineStatistics& stats) override {
+    renderer_->OnStatisticsUpdate(stats);
+  }
+  void OnBufferingStateChange(BufferingState state) override {
+    renderer_->OnBufferingStateChange(type_, state);
+  }
+  void OnWaitingForDecryptionKey() override {
+    renderer_->OnWaitingForDecryptionKey();
+  }
+  void OnVideoNaturalSizeChange(const gfx::Size& size) override {
+    DCHECK(type_ == DemuxerStream::VIDEO);
+    renderer_->OnVideoNaturalSizeChange(size);
+  }
+  void OnVideoOpacityChange(bool opaque) override {
+    DCHECK(type_ == DemuxerStream::VIDEO);
+    renderer_->OnVideoOpacityChange(opaque);
+  }
+
+ private:
+  DemuxerStream::Type type_;
+  RendererImpl* renderer_;
+};
+
 RendererImpl::RendererImpl(
     const scoped_refptr<base::SingleThreadTaskRunner>& task_runner,
-    scoped_ptr<AudioRenderer> audio_renderer,
-    scoped_ptr<VideoRenderer> video_renderer)
+    std::unique_ptr<AudioRenderer> audio_renderer,
+    std::unique_ptr<VideoRenderer> video_renderer)
     : state_(STATE_UNINITIALIZED),
       task_runner_(task_runner),
       audio_renderer_(std::move(audio_renderer)),
@@ -72,38 +107,32 @@ RendererImpl::~RendererImpl() {
   video_renderer_.reset();
   audio_renderer_.reset();
 
-  if (!init_cb_.is_null())
-    base::ResetAndReturn(&init_cb_).Run(PIPELINE_ERROR_ABORT);
-  else if (!flush_cb_.is_null())
+  if (!init_cb_.is_null()) {
+    FinishInitialization(PIPELINE_ERROR_ABORT);
+  } else if (!flush_cb_.is_null()) {
     base::ResetAndReturn(&flush_cb_).Run();
+  }
 }
 
-void RendererImpl::Initialize(
-    DemuxerStreamProvider* demuxer_stream_provider,
-    const PipelineStatusCB& init_cb,
-    const StatisticsCB& statistics_cb,
-    const BufferingStateCB& buffering_state_cb,
-    const base::Closure& ended_cb,
-    const PipelineStatusCB& error_cb,
-    const base::Closure& waiting_for_decryption_key_cb) {
+void RendererImpl::Initialize(DemuxerStreamProvider* demuxer_stream_provider,
+                              RendererClient* client,
+                              const PipelineStatusCB& init_cb) {
   DVLOG(1) << __FUNCTION__;
   DCHECK(task_runner_->BelongsToCurrentThread());
   DCHECK_EQ(state_, STATE_UNINITIALIZED);
   DCHECK(!init_cb.is_null());
-  DCHECK(!statistics_cb.is_null());
-  DCHECK(!buffering_state_cb.is_null());
-  DCHECK(!ended_cb.is_null());
-  DCHECK(!error_cb.is_null());
+  DCHECK(client);
   DCHECK(demuxer_stream_provider->GetStream(DemuxerStream::AUDIO) ||
          demuxer_stream_provider->GetStream(DemuxerStream::VIDEO));
 
+  client_ = client;
   demuxer_stream_provider_ = demuxer_stream_provider;
-  statistics_cb_ = statistics_cb;
-  buffering_state_cb_ = buffering_state_cb;
-  ended_cb_ = ended_cb;
-  error_cb_ = error_cb;
   init_cb_ = init_cb;
-  waiting_for_decryption_key_cb_ = waiting_for_decryption_key_cb;
+
+  if (HasEncryptedStream() && !cdm_context_) {
+    state_ = STATE_INIT_PENDING_CDM;
+    return;
+  }
 
   state_ = STATE_INITIALIZING;
   InitializeAudioRenderer();
@@ -123,12 +152,17 @@ void RendererImpl::SetCdm(CdmContext* cdm_context,
 
   cdm_context_ = cdm_context;
 
-  if (cdm_ready_cb_.is_null()) {
+  if (state_ != STATE_INIT_PENDING_CDM) {
     cdm_attached_cb.Run(true);
     return;
   }
 
-  base::ResetAndReturn(&cdm_ready_cb_).Run(cdm_context, cdm_attached_cb);
+  DCHECK(!init_cb_.is_null());
+  state_ = STATE_INITIALIZING;
+  // |cdm_attached_cb| will be fired after initialization finishes.
+  pending_cdm_attached_cb_ = cdm_attached_cb;
+
+  InitializeAudioRenderer();
 }
 
 void RendererImpl::Flush(const base::Closure& flush_cb) {
@@ -254,25 +288,27 @@ bool RendererImpl::GetWallClockTimes(
   return time_source_->GetWallClockTimes(media_timestamps, wall_clock_times);
 }
 
-void RendererImpl::SetCdmReadyCallback(const CdmReadyCB& cdm_ready_cb) {
-  // Cancels the previous CDM request.
-  if (cdm_ready_cb.is_null()) {
-    if (!cdm_ready_cb_.is_null()) {
-      base::ResetAndReturn(&cdm_ready_cb_)
-          .Run(nullptr, base::Bind(IgnoreCdmAttached));
-    }
-    return;
-  }
+bool RendererImpl::HasEncryptedStream() {
+  DemuxerStream* audio_stream =
+      demuxer_stream_provider_->GetStream(DemuxerStream::AUDIO);
+  if (audio_stream && audio_stream->audio_decoder_config().is_encrypted())
+    return true;
 
-  // We initialize audio and video decoders in sequence.
-  DCHECK(cdm_ready_cb_.is_null());
+  DemuxerStream* video_stream =
+      demuxer_stream_provider_->GetStream(DemuxerStream::VIDEO);
+  if (video_stream && video_stream->video_decoder_config().is_encrypted())
+    return true;
 
-  if (cdm_context_) {
-    cdm_ready_cb.Run(cdm_context_, base::Bind(IgnoreCdmAttached));
-    return;
-  }
+  return false;
+}
 
-  cdm_ready_cb_ = cdm_ready_cb;
+void RendererImpl::FinishInitialization(PipelineStatus status) {
+  DCHECK(!init_cb_.is_null());
+
+  if (!pending_cdm_attached_cb_.is_null())
+    base::ResetAndReturn(&pending_cdm_attached_cb_).Run(status == PIPELINE_OK);
+
+  base::ResetAndReturn(&init_cb_).Run(status);
 }
 
 void RendererImpl::InitializeAudioRenderer() {
@@ -290,17 +326,13 @@ void RendererImpl::InitializeAudioRenderer() {
     return;
   }
 
+  audio_renderer_client_.reset(
+      new RendererClientInternal(DemuxerStream::AUDIO, this));
   // Note: After the initialization of a renderer, error events from it may
   // happen at any time and all future calls must guard against STATE_ERROR.
   audio_renderer_->Initialize(
-      demuxer_stream_provider_->GetStream(DemuxerStream::AUDIO), done_cb,
-      base::Bind(&RendererImpl::SetCdmReadyCallback, weak_this_),
-      base::Bind(&RendererImpl::OnUpdateStatistics, weak_this_),
-      base::Bind(&RendererImpl::OnBufferingStateChanged, weak_this_,
-                 &audio_buffering_state_),
-      base::Bind(&RendererImpl::OnAudioRendererEnded, weak_this_),
-      base::Bind(&RendererImpl::OnError, weak_this_),
-      waiting_for_decryption_key_cb_);
+      demuxer_stream_provider_->GetStream(DemuxerStream::AUDIO), cdm_context_,
+      audio_renderer_client_.get(), done_cb);
 }
 
 void RendererImpl::OnAudioRendererInitializeDone(PipelineStatus status) {
@@ -316,7 +348,7 @@ void RendererImpl::OnAudioRendererInitializeDone(PipelineStatus status) {
   }
 
   if (status != PIPELINE_OK) {
-    base::ResetAndReturn(&init_cb_).Run(status);
+    FinishInitialization(status);
     return;
   }
 
@@ -339,16 +371,13 @@ void RendererImpl::InitializeVideoRenderer() {
     return;
   }
 
+  video_renderer_client_.reset(
+      new RendererClientInternal(DemuxerStream::VIDEO, this));
   video_renderer_->Initialize(
-      demuxer_stream_provider_->GetStream(DemuxerStream::VIDEO), done_cb,
-      base::Bind(&RendererImpl::SetCdmReadyCallback, weak_this_),
-      base::Bind(&RendererImpl::OnUpdateStatistics, weak_this_),
-      base::Bind(&RendererImpl::OnBufferingStateChanged, weak_this_,
-                 &video_buffering_state_),
-      base::Bind(&RendererImpl::OnVideoRendererEnded, weak_this_),
-      base::Bind(&RendererImpl::OnError, weak_this_),
+      demuxer_stream_provider_->GetStream(DemuxerStream::VIDEO), cdm_context_,
+      video_renderer_client_.get(),
       base::Bind(&RendererImpl::GetWallClockTimes, base::Unretained(this)),
-      waiting_for_decryption_key_cb_);
+      done_cb);
 }
 
 void RendererImpl::OnVideoRendererInitializeDone(PipelineStatus status) {
@@ -367,7 +396,7 @@ void RendererImpl::OnVideoRendererInitializeDone(PipelineStatus status) {
   DCHECK(!init_cb_.is_null());
 
   if (status != PIPELINE_OK) {
-    base::ResetAndReturn(&init_cb_).Run(status);
+    FinishInitialization(status);
     return;
   }
 
@@ -381,7 +410,8 @@ void RendererImpl::OnVideoRendererInitializeDone(PipelineStatus status) {
   state_ = STATE_PLAYING;
   DCHECK(time_source_);
   DCHECK(audio_renderer_ || video_renderer_);
-  base::ResetAndReturn(&init_cb_).Run(PIPELINE_OK);
+
+  FinishInitialization(PIPELINE_OK);
 }
 
 void RendererImpl::FlushAudioRenderer() {
@@ -453,16 +483,21 @@ void RendererImpl::OnVideoRendererFlushDone() {
   base::ResetAndReturn(&flush_cb_).Run();
 }
 
-void RendererImpl::OnUpdateStatistics(const PipelineStatistics& stats) {
+void RendererImpl::OnStatisticsUpdate(const PipelineStatistics& stats) {
   DCHECK(task_runner_->BelongsToCurrentThread());
-  statistics_cb_.Run(stats);
+  client_->OnStatisticsUpdate(stats);
 }
 
-void RendererImpl::OnBufferingStateChanged(BufferingState* buffering_state,
-                                           BufferingState new_buffering_state) {
-  const bool is_audio = buffering_state == &audio_buffering_state_;
+void RendererImpl::OnBufferingStateChange(DemuxerStream::Type type,
+                                          BufferingState new_buffering_state) {
+  DCHECK((type == DemuxerStream::AUDIO) || (type == DemuxerStream::VIDEO));
+  BufferingState* buffering_state = type == DemuxerStream::AUDIO
+                                        ? &audio_buffering_state_
+                                        : &video_buffering_state_;
+
   DVLOG(1) << __FUNCTION__ << "(" << *buffering_state << ", "
-           << new_buffering_state << ") " << (is_audio ? "audio" : "video");
+           << new_buffering_state << ") "
+           << (type == DemuxerStream::AUDIO ? "audio" : "video");
   DCHECK(task_runner_->BelongsToCurrentThread());
 
   bool was_waiting_for_enough_data = WaitingForEnoughData();
@@ -470,14 +505,15 @@ void RendererImpl::OnBufferingStateChanged(BufferingState* buffering_state,
   // When audio is present and has enough data, defer video underflow callbacks
   // for some time to avoid unnecessary glitches in audio; see
   // http://crbug.com/144683#c53.
-  if (audio_renderer_ && !is_audio && state_ == STATE_PLAYING) {
+  if (audio_renderer_ && type == DemuxerStream::VIDEO &&
+      state_ == STATE_PLAYING) {
     if (video_buffering_state_ == BUFFERING_HAVE_ENOUGH &&
         audio_buffering_state_ == BUFFERING_HAVE_ENOUGH &&
         new_buffering_state == BUFFERING_HAVE_NOTHING &&
         deferred_underflow_cb_.IsCancelled()) {
-      deferred_underflow_cb_.Reset(base::Bind(
-          &RendererImpl::OnBufferingStateChanged, weak_factory_.GetWeakPtr(),
-          buffering_state, new_buffering_state));
+      deferred_underflow_cb_.Reset(
+          base::Bind(&RendererImpl::OnBufferingStateChange,
+                     weak_factory_.GetWeakPtr(), type, new_buffering_state));
       task_runner_->PostDelayedTask(FROM_HERE,
                                     deferred_underflow_cb_.callback(),
                                     video_underflow_threshold_);
@@ -485,7 +521,8 @@ void RendererImpl::OnBufferingStateChanged(BufferingState* buffering_state,
     }
 
     deferred_underflow_cb_.Cancel();
-  } else if (!deferred_underflow_cb_.IsCancelled() && is_audio &&
+  } else if (!deferred_underflow_cb_.IsCancelled() &&
+             type == DemuxerStream::AUDIO &&
              new_buffering_state == BUFFERING_HAVE_NOTHING) {
     // If audio underflows while we have a deferred video underflow in progress
     // we want to mark video as underflowed immediately and cancel the deferral.
@@ -514,7 +551,7 @@ void RendererImpl::OnBufferingStateChanged(BufferingState* buffering_state,
   // Renderer prerolled.
   if (was_waiting_for_enough_data && !WaitingForEnoughData()) {
     StartPlayback();
-    buffering_state_cb_.Run(BUFFERING_HAVE_ENOUGH);
+    client_->OnBufferingStateChange(BUFFERING_HAVE_ENOUGH);
     return;
   }
 }
@@ -545,6 +582,7 @@ void RendererImpl::PausePlayback() {
       break;
 
     case STATE_UNINITIALIZED:
+    case STATE_INIT_PENDING_CDM:
     case STATE_INITIALIZING:
       NOTREACHED() << "Invalid state: " << state_;
       break;
@@ -573,28 +611,21 @@ void RendererImpl::StartPlayback() {
     video_renderer_->OnTimeStateChanged(true);
 }
 
-void RendererImpl::OnAudioRendererEnded() {
+void RendererImpl::OnRendererEnded(DemuxerStream::Type type) {
   DVLOG(1) << __FUNCTION__;
   DCHECK(task_runner_->BelongsToCurrentThread());
+  DCHECK((type == DemuxerStream::AUDIO) || (type == DemuxerStream::VIDEO));
 
   if (state_ != STATE_PLAYING)
     return;
 
-  DCHECK(!audio_ended_);
-  audio_ended_ = true;
-
-  RunEndedCallbackIfNeeded();
-}
-
-void RendererImpl::OnVideoRendererEnded() {
-  DVLOG(1) << __FUNCTION__;
-  DCHECK(task_runner_->BelongsToCurrentThread());
-
-  if (state_ != STATE_PLAYING)
-    return;
-
-  DCHECK(!video_ended_);
-  video_ended_ = true;
+  if (type == DemuxerStream::AUDIO) {
+    DCHECK(!audio_ended_);
+    audio_ended_ = true;
+  } else {
+    DCHECK(!video_ended_);
+    video_ended_ = true;
+  }
 
   RunEndedCallbackIfNeeded();
 }
@@ -622,7 +653,7 @@ void RendererImpl::RunEndedCallbackIfNeeded() {
   if (time_ticking_)
     PausePlayback();
 
-  ended_cb_.Run();
+  client_->OnEnded();
 }
 
 void RendererImpl::OnError(PipelineStatus error) {
@@ -637,16 +668,33 @@ void RendererImpl::OnError(PipelineStatus error) {
   const State old_state = state_;
   state_ = STATE_ERROR;
 
-  if (old_state == STATE_INITIALIZING) {
-    base::ResetAndReturn(&init_cb_).Run(error);
+  if (!init_cb_.is_null()) {
+    DCHECK(old_state == STATE_INITIALIZING ||
+           old_state == STATE_INIT_PENDING_CDM);
+    FinishInitialization(error);
     return;
   }
 
   // After OnError() returns, the pipeline may destroy |this|.
-  base::ResetAndReturn(&error_cb_).Run(error);
+  client_->OnError(error);
 
   if (!flush_cb_.is_null())
     base::ResetAndReturn(&flush_cb_).Run();
 }
 
+void RendererImpl::OnWaitingForDecryptionKey() {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnWaitingForDecryptionKey();
+}
+
+void RendererImpl::OnVideoNaturalSizeChange(const gfx::Size& size) {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnVideoNaturalSizeChange(size);
+}
+
+void RendererImpl::OnVideoOpacityChange(bool opaque) {
+  DCHECK(task_runner_->BelongsToCurrentThread());
+  client_->OnVideoOpacityChange(opaque);
+}
+
 }  // namespace media
