@@ -4,6 +4,7 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <vector>
 
 #include "base/bind.h"
@@ -29,30 +30,27 @@ namespace media {
 
 const int64_t kStartPlayingTimeInMs = 100;
 
-ACTION_P2(SetBufferingState, cb, buffering_state) {
-  cb->Run(buffering_state);
+ACTION_P2(SetBufferingState, renderer_client, buffering_state) {
+  (*renderer_client)->OnBufferingStateChange(buffering_state);
 }
 
-ACTION_P2(AudioError, cb, error) {
-  cb->Run(error);
+ACTION_P2(SetError, renderer_client, error) {
+  (*renderer_client)->OnError(error);
 }
 
 class RendererImplTest : public ::testing::Test {
  public:
   // Used for setting expectations on pipeline callbacks. Using a StrictMock
   // also lets us test for missing callbacks.
-  class CallbackHelper {
+  class CallbackHelper : public MockRendererClient {
    public:
     CallbackHelper() {}
     virtual ~CallbackHelper() {}
 
+    // Completion callbacks.
     MOCK_METHOD1(OnInitialize, void(PipelineStatus));
     MOCK_METHOD0(OnFlushed, void());
-    MOCK_METHOD0(OnEnded, void());
-    MOCK_METHOD1(OnError, void(PipelineStatus));
-    MOCK_METHOD1(OnUpdateStatistics, void(const PipelineStatistics&));
-    MOCK_METHOD1(OnBufferingStateChange, void(BufferingState));
-    MOCK_METHOD0(OnWaitingForDecryptionKey, void());
+    MOCK_METHOD1(OnCdmAttached, void(bool));
 
    private:
     DISALLOW_COPY_AND_ASSIGN(CallbackHelper);
@@ -64,49 +62,51 @@ class RendererImplTest : public ::testing::Test {
         audio_renderer_(new StrictMock<MockAudioRenderer>()),
         renderer_impl_(
             new RendererImpl(message_loop_.task_runner(),
-                             scoped_ptr<AudioRenderer>(audio_renderer_),
-                             scoped_ptr<VideoRenderer>(video_renderer_))) {
-    // SetDemuxerExpectations() adds overriding expectations for expected
-    // non-NULL streams.
+                             std::unique_ptr<AudioRenderer>(audio_renderer_),
+                             std::unique_ptr<VideoRenderer>(video_renderer_))),
+        cdm_context_(new StrictMock<MockCdmContext>()),
+        video_renderer_client_(nullptr),
+        audio_renderer_client_(nullptr),
+        initialization_status_(PIPELINE_OK) {
+    // CreateAudioStream() and CreateVideoStream() overrides expectations for
+    // expected non-NULL streams.
     DemuxerStream* null_pointer = NULL;
     EXPECT_CALL(*demuxer_, GetStream(_))
         .WillRepeatedly(Return(null_pointer));
   }
 
-  virtual ~RendererImplTest() {
+  virtual ~RendererImplTest() { Destroy(); }
+
+ protected:
+  void Destroy() {
     renderer_impl_.reset();
     base::RunLoop().RunUntilIdle();
   }
 
- protected:
-  typedef std::vector<MockDemuxerStream*> MockDemuxerStreamVector;
-
-  scoped_ptr<StrictMock<MockDemuxerStream> > CreateStream(
+  std::unique_ptr<StrictMock<MockDemuxerStream>> CreateStream(
       DemuxerStream::Type type) {
-    scoped_ptr<StrictMock<MockDemuxerStream> > stream(
+    std::unique_ptr<StrictMock<MockDemuxerStream>> stream(
         new StrictMock<MockDemuxerStream>(type));
     return stream;
   }
 
   // Sets up expectations to allow the audio renderer to initialize.
   void SetAudioRendererInitializeExpectations(PipelineStatus status) {
-    EXPECT_CALL(*audio_renderer_,
-                Initialize(audio_stream_.get(), _, _, _, _, _, _, _))
-        .WillOnce(DoAll(SaveArg<4>(&audio_buffering_state_cb_),
-                        SaveArg<5>(&audio_ended_cb_),
-                        SaveArg<6>(&audio_error_cb_), RunCallback<1>(status)));
+    EXPECT_CALL(*audio_renderer_, Initialize(audio_stream_.get(), _, _, _))
+        .WillOnce(
+            DoAll(SaveArg<2>(&audio_renderer_client_), RunCallback<3>(status)));
   }
 
   // Sets up expectations to allow the video renderer to initialize.
   void SetVideoRendererInitializeExpectations(PipelineStatus status) {
-    EXPECT_CALL(*video_renderer_,
-                Initialize(video_stream_.get(), _, _, _, _, _, _, _, _))
-        .WillOnce(DoAll(SaveArg<4>(&video_buffering_state_cb_),
-                        SaveArg<5>(&video_ended_cb_), RunCallback<1>(status)));
+    EXPECT_CALL(*video_renderer_, Initialize(video_stream_.get(), _, _, _, _))
+        .WillOnce(
+            DoAll(SaveArg<2>(&video_renderer_client_), RunCallback<4>(status)));
   }
 
   void InitializeAndExpect(PipelineStatus start_status) {
-    EXPECT_CALL(callbacks_, OnInitialize(start_status));
+    EXPECT_CALL(callbacks_, OnInitialize(start_status))
+        .WillOnce(SaveArg<0>(&initialization_status_));
     EXPECT_CALL(callbacks_, OnWaitingForDecryptionKey()).Times(0);
 
     if (start_status == PIPELINE_OK && audio_stream_) {
@@ -116,36 +116,29 @@ class RendererImplTest : public ::testing::Test {
       renderer_impl_->set_time_source_for_testing(&time_source_);
     }
 
-    renderer_impl_->Initialize(
-        demuxer_.get(),
-        base::Bind(&CallbackHelper::OnInitialize,
-                   base::Unretained(&callbacks_)),
-        base::Bind(&CallbackHelper::OnUpdateStatistics,
-                   base::Unretained(&callbacks_)),
-        base::Bind(&CallbackHelper::OnBufferingStateChange,
-                   base::Unretained(&callbacks_)),
-        base::Bind(&CallbackHelper::OnEnded, base::Unretained(&callbacks_)),
-        base::Bind(&CallbackHelper::OnError, base::Unretained(&callbacks_)),
-        base::Bind(&CallbackHelper::OnWaitingForDecryptionKey,
-                   base::Unretained(&callbacks_)));
+    renderer_impl_->Initialize(demuxer_.get(), &callbacks_,
+                               base::Bind(&CallbackHelper::OnInitialize,
+                                          base::Unretained(&callbacks_)));
     base::RunLoop().RunUntilIdle();
   }
 
   void CreateAudioStream() {
     audio_stream_ = CreateStream(DemuxerStream::AUDIO);
-    streams_.push_back(audio_stream_.get());
     EXPECT_CALL(*demuxer_, GetStream(DemuxerStream::AUDIO))
         .WillRepeatedly(Return(audio_stream_.get()));
   }
 
-  void CreateVideoStream() {
+  void CreateVideoStream(bool is_encrypted = false) {
     video_stream_ = CreateStream(DemuxerStream::VIDEO);
-    video_stream_->set_video_decoder_config(video_decoder_config_);
-    streams_.push_back(video_stream_.get());
+    video_stream_->set_video_decoder_config(
+        is_encrypted ? TestVideoConfig::NormalEncrypted()
+                     : TestVideoConfig::Normal());
     EXPECT_CALL(*demuxer_, GetStream(DemuxerStream::VIDEO))
         .WillRepeatedly(Return(video_stream_.get()));
   }
 
+  void CreateEncryptedVideoStream() { CreateVideoStream(true); }
+
   void CreateAudioAndVideoStream() {
     CreateAudioStream();
     CreateVideoStream();
@@ -181,13 +174,13 @@ class RendererImplTest : public ::testing::Test {
 
     if (audio_stream_) {
       EXPECT_CALL(*audio_renderer_, StartPlaying())
-          .WillOnce(SetBufferingState(&audio_buffering_state_cb_,
+          .WillOnce(SetBufferingState(&audio_renderer_client_,
                                       BUFFERING_HAVE_ENOUGH));
     }
 
     if (video_stream_) {
       EXPECT_CALL(*video_renderer_, StartPlayingFrom(start_time))
-          .WillOnce(SetBufferingState(&video_buffering_state_cb_,
+          .WillOnce(SetBufferingState(&video_renderer_client_,
                                       BUFFERING_HAVE_ENOUGH));
     }
 
@@ -201,14 +194,14 @@ class RendererImplTest : public ::testing::Test {
 
     if (audio_stream_) {
       EXPECT_CALL(*audio_renderer_, Flush(_))
-          .WillOnce(DoAll(SetBufferingState(&audio_buffering_state_cb_,
+          .WillOnce(DoAll(SetBufferingState(&audio_renderer_client_,
                                             BUFFERING_HAVE_NOTHING),
                           RunClosure<0>()));
     }
 
     if (video_stream_) {
       EXPECT_CALL(*video_renderer_, Flush(_))
-          .WillOnce(DoAll(SetBufferingState(&video_buffering_state_cb_,
+          .WillOnce(DoAll(SetBufferingState(&video_renderer_client_,
                                             BUFFERING_HAVE_NOTHING),
                           RunClosure<0>()));
     }
@@ -248,33 +241,88 @@ class RendererImplTest : public ::testing::Test {
     return IsMediaTimeAdvancing(1.0);
   }
 
+  void SetCdmAndExpect(bool expected_result) {
+    EXPECT_CALL(callbacks_, OnCdmAttached(expected_result));
+    renderer_impl_->SetCdm(cdm_context_.get(),
+                           base::Bind(&CallbackHelper::OnCdmAttached,
+                                      base::Unretained(&callbacks_)));
+    base::RunLoop().RunUntilIdle();
+  }
+
   // Fixture members.
   base::MessageLoop message_loop_;
   StrictMock<CallbackHelper> callbacks_;
   base::SimpleTestTickClock test_tick_clock_;
 
-  scoped_ptr<StrictMock<MockDemuxer> > demuxer_;
+  std::unique_ptr<StrictMock<MockDemuxer>> demuxer_;
   StrictMock<MockVideoRenderer>* video_renderer_;
   StrictMock<MockAudioRenderer>* audio_renderer_;
-  scoped_ptr<RendererImpl> renderer_impl_;
+  std::unique_ptr<RendererImpl> renderer_impl_;
+  std::unique_ptr<StrictMock<MockCdmContext>> cdm_context_;
 
   StrictMock<MockTimeSource> time_source_;
-  scoped_ptr<StrictMock<MockDemuxerStream> > audio_stream_;
-  scoped_ptr<StrictMock<MockDemuxerStream> > video_stream_;
-  MockDemuxerStreamVector streams_;
-  BufferingStateCB audio_buffering_state_cb_;
-  BufferingStateCB video_buffering_state_cb_;
-  base::Closure audio_ended_cb_;
-  base::Closure video_ended_cb_;
-  PipelineStatusCB audio_error_cb_;
+  std::unique_ptr<StrictMock<MockDemuxerStream>> audio_stream_;
+  std::unique_ptr<StrictMock<MockDemuxerStream>> video_stream_;
+  RendererClient* video_renderer_client_;
+  RendererClient* audio_renderer_client_;
   VideoDecoderConfig video_decoder_config_;
+  PipelineStatus initialization_status_;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(RendererImplTest);
 };
 
-TEST_F(RendererImplTest, DestroyBeforeInitialize) {
-  // |renderer_impl_| will be destroyed in the dtor.
+TEST_F(RendererImplTest, Destroy_BeforeInitialize) {
+  Destroy();
+}
+
+TEST_F(RendererImplTest, Destroy_PendingInitialize) {
+  CreateAudioAndVideoStream();
+
+  SetAudioRendererInitializeExpectations(PIPELINE_OK);
+  // Not returning the video initialization callback.
+  EXPECT_CALL(*video_renderer_, Initialize(video_stream_.get(), _, _, _, _));
+
+  InitializeAndExpect(PIPELINE_ERROR_ABORT);
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  Destroy();
+}
+
+TEST_F(RendererImplTest, Destroy_PendingInitializeWithoutCdm) {
+  CreateAudioStream();
+  CreateEncryptedVideoStream();
+
+  // Audio is clear and video is encrypted. Initialization will not start
+  // because no CDM is set. So neither AudioRenderer::Initialize() nor
+  // VideoRenderer::Initialize() should not be called. The InitCB will be
+  // aborted when |renderer_impl_| is destructed.
+  InitializeAndExpect(PIPELINE_ERROR_ABORT);
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  Destroy();
+}
+
+TEST_F(RendererImplTest, Destroy_PendingInitializeAfterSetCdm) {
+  CreateAudioStream();
+  CreateEncryptedVideoStream();
+
+  // Audio is clear and video is encrypted. Initialization will not start
+  // because no CDM is set.
+  InitializeAndExpect(PIPELINE_ERROR_ABORT);
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  SetAudioRendererInitializeExpectations(PIPELINE_OK);
+  // Not returning the video initialization callback. So initialization will
+  // be pending.
+  EXPECT_CALL(*video_renderer_, Initialize(video_stream_.get(), _, _, _, _));
+
+  // SetCdm() will trigger the initialization to start. But it will not complete
+  // because the |video_renderer_| is not returning the initialization callback.
+  SetCdmAndExpect(false);
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  Destroy();
 }
 
 TEST_F(RendererImplTest, InitializeWithAudio) {
@@ -315,6 +363,54 @@ TEST_F(RendererImplTest, InitializeWithAudioVideo_VideoRendererFailed) {
   InitializeAndExpect(PIPELINE_ERROR_INITIALIZATION_FAILED);
 }
 
+TEST_F(RendererImplTest, SetCdmBeforeInitialize) {
+  // CDM will be successfully attached immediately if set before RendererImpl
+  // initialization, regardless of the later initialization result.
+  SetCdmAndExpect(true);
+}
+
+TEST_F(RendererImplTest, SetCdmAfterInitialize_ClearStream) {
+  InitializeWithAudioAndVideo();
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  // CDM will be successfully attached immediately since initialization is
+  // completed.
+  SetCdmAndExpect(true);
+}
+
+TEST_F(RendererImplTest, SetCdmAfterInitialize_EncryptedStream_Success) {
+  CreateAudioStream();
+  CreateEncryptedVideoStream();
+
+  SetAudioRendererInitializeExpectations(PIPELINE_OK);
+  SetVideoRendererInitializeExpectations(PIPELINE_OK);
+  InitializeAndExpect(PIPELINE_OK);
+  // Initialization is pending until CDM is set.
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  SetCdmAndExpect(true);
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+}
+
+TEST_F(RendererImplTest, SetCdmAfterInitialize_EncryptedStream_Failure) {
+  CreateAudioStream();
+  CreateEncryptedVideoStream();
+
+  SetAudioRendererInitializeExpectations(PIPELINE_OK);
+  SetVideoRendererInitializeExpectations(PIPELINE_ERROR_INITIALIZATION_FAILED);
+  InitializeAndExpect(PIPELINE_ERROR_INITIALIZATION_FAILED);
+  // Initialization is pending until CDM is set.
+  EXPECT_EQ(PIPELINE_OK, initialization_status_);
+
+  SetCdmAndExpect(false);
+  EXPECT_EQ(PIPELINE_ERROR_INITIALIZATION_FAILED, initialization_status_);
+}
+
+TEST_F(RendererImplTest, SetCdmMultipleTimes) {
+  SetCdmAndExpect(true);
+  SetCdmAndExpect(false);  // Do not support switching CDM.
+}
+
 TEST_F(RendererImplTest, StartPlayingFrom) {
   InitializeWithAudioAndVideo();
   Play();
@@ -372,7 +468,7 @@ TEST_F(RendererImplTest, FlushAfterUnderflow) {
 
   // Simulate underflow.
   EXPECT_CALL(time_source_, StopTicking());
-  audio_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  audio_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
 
   // Flush while underflowed. We shouldn't call StopTicking() again.
   Flush(true);
@@ -397,7 +493,7 @@ TEST_F(RendererImplTest, AudioStreamEnded) {
   EXPECT_CALL(time_source_, StopTicking());
   EXPECT_CALL(callbacks_, OnEnded());
 
-  audio_ended_cb_.Run();
+  audio_renderer_client_->OnEnded();
   base::RunLoop().RunUntilIdle();
 }
 
@@ -408,7 +504,7 @@ TEST_F(RendererImplTest, VideoStreamEnded) {
   EXPECT_CALL(time_source_, StopTicking());
   EXPECT_CALL(callbacks_, OnEnded());
 
-  video_ended_cb_.Run();
+  video_renderer_client_->OnEnded();
   base::RunLoop().RunUntilIdle();
 }
 
@@ -417,20 +513,20 @@ TEST_F(RendererImplTest, AudioVideoStreamsEnded) {
   Play();
 
   // OnEnded() is called only when all streams have finished.
-  audio_ended_cb_.Run();
+  audio_renderer_client_->OnEnded();
   base::RunLoop().RunUntilIdle();
 
   EXPECT_CALL(time_source_, StopTicking());
   EXPECT_CALL(callbacks_, OnEnded());
 
-  video_ended_cb_.Run();
+  video_renderer_client_->OnEnded();
   base::RunLoop().RunUntilIdle();
 }
 
 TEST_F(RendererImplTest, ErrorAfterInitialize) {
   InitializeWithAudio();
   EXPECT_CALL(callbacks_, OnError(PIPELINE_ERROR_DECODE));
-  audio_error_cb_.Run(PIPELINE_ERROR_DECODE);
+  audio_renderer_client_->OnError(PIPELINE_ERROR_DECODE);
   base::RunLoop().RunUntilIdle();
 }
 
@@ -439,7 +535,7 @@ TEST_F(RendererImplTest, ErrorDuringPlaying) {
   Play();
 
   EXPECT_CALL(callbacks_, OnError(PIPELINE_ERROR_DECODE));
-  audio_error_cb_.Run(PIPELINE_ERROR_DECODE);
+  audio_renderer_client_->OnError(PIPELINE_ERROR_DECODE);
   base::RunLoop().RunUntilIdle();
 }
 
@@ -449,9 +545,9 @@ TEST_F(RendererImplTest, ErrorDuringFlush) {
 
   InSequence s;
   EXPECT_CALL(time_source_, StopTicking());
-  EXPECT_CALL(*audio_renderer_, Flush(_)).WillOnce(DoAll(
-      AudioError(&audio_error_cb_, PIPELINE_ERROR_DECODE),
-      RunClosure<0>()));
+  EXPECT_CALL(*audio_renderer_, Flush(_))
+      .WillOnce(DoAll(SetError(&audio_renderer_client_, PIPELINE_ERROR_DECODE),
+                      RunClosure<0>()));
   EXPECT_CALL(callbacks_, OnError(PIPELINE_ERROR_DECODE));
   EXPECT_CALL(callbacks_, OnFlushed());
   renderer_impl_->Flush(
@@ -465,7 +561,7 @@ TEST_F(RendererImplTest, ErrorAfterFlush) {
   Flush(false);
 
   EXPECT_CALL(callbacks_, OnError(PIPELINE_ERROR_DECODE));
-  audio_error_cb_.Run(PIPELINE_ERROR_DECODE);
+  audio_renderer_client_->OnError(PIPELINE_ERROR_DECODE);
   base::RunLoop().RunUntilIdle();
 }
 
@@ -474,12 +570,10 @@ TEST_F(RendererImplTest, ErrorDuringInitialize) {
   SetAudioRendererInitializeExpectations(PIPELINE_OK);
 
   // Force an audio error to occur during video renderer initialization.
-  EXPECT_CALL(*video_renderer_,
-              Initialize(video_stream_.get(), _, _, _, _, _, _, _, _))
-      .WillOnce(DoAll(AudioError(&audio_error_cb_, PIPELINE_ERROR_DECODE),
-                      SaveArg<4>(&video_buffering_state_cb_),
-                      SaveArg<5>(&video_ended_cb_),
-                      RunCallback<1>(PIPELINE_OK)));
+  EXPECT_CALL(*video_renderer_, Initialize(video_stream_.get(), _, _, _, _))
+      .WillOnce(DoAll(SetError(&audio_renderer_client_, PIPELINE_ERROR_DECODE),
+                      SaveArg<2>(&video_renderer_client_),
+                      RunCallback<4>(PIPELINE_OK)));
 
   InitializeAndExpect(PIPELINE_ERROR_DECODE);
 }
@@ -490,7 +584,7 @@ TEST_F(RendererImplTest, AudioUnderflow) {
 
   // Underflow should occur immediately with a single audio track.
   EXPECT_CALL(time_source_, StopTicking());
-  audio_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  audio_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
 }
 
 TEST_F(RendererImplTest, AudioUnderflowWithVideo) {
@@ -500,7 +594,7 @@ TEST_F(RendererImplTest, AudioUnderflowWithVideo) {
   // Underflow should be immediate when both audio and video are present and
   // audio underflows.
   EXPECT_CALL(time_source_, StopTicking());
-  audio_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  audio_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
 }
 
 TEST_F(RendererImplTest, VideoUnderflow) {
@@ -509,7 +603,7 @@ TEST_F(RendererImplTest, VideoUnderflow) {
 
   // Underflow should occur immediately with a single video track.
   EXPECT_CALL(time_source_, StopTicking());
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
 }
 
 TEST_F(RendererImplTest, VideoUnderflowWithAudio) {
@@ -522,7 +616,7 @@ TEST_F(RendererImplTest, VideoUnderflowWithAudio) {
 
   // Underflow should be delayed when both audio and video are present and video
   // underflows.
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
   Mock::VerifyAndClearExpectations(&time_source_);
 
   EXPECT_CALL(time_source_, StopTicking());
@@ -539,11 +633,11 @@ TEST_F(RendererImplTest, VideoUnderflowWithAudioVideoRecovers) {
 
   // Underflow should be delayed when both audio and video are present and video
   // underflows.
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
   Mock::VerifyAndClearExpectations(&time_source_);
 
   // If video recovers, the underflow should never occur.
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_ENOUGH);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_ENOUGH);
   base::RunLoop().RunUntilIdle();
 }
 
@@ -557,11 +651,11 @@ TEST_F(RendererImplTest, VideoAndAudioUnderflow) {
 
   // Underflow should be delayed when both audio and video are present and video
   // underflows.
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
   Mock::VerifyAndClearExpectations(&time_source_);
 
   EXPECT_CALL(time_source_, StopTicking());
-  audio_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  audio_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
 
   // Nothing else should primed on the message loop.
   base::RunLoop().RunUntilIdle();
@@ -577,8 +671,8 @@ TEST_F(RendererImplTest, VideoUnderflowWithAudioFlush) {
 
   // Simulate the cases where audio underflows and then video underflows.
   EXPECT_CALL(time_source_, StopTicking());
-  audio_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
-  video_buffering_state_cb_.Run(BUFFERING_HAVE_NOTHING);
+  audio_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
+  video_renderer_client_->OnBufferingStateChange(BUFFERING_HAVE_NOTHING);
   Mock::VerifyAndClearExpectations(&time_source_);
 
   // Flush the audio and video renderers, both think they're in an underflow
@@ -597,7 +691,7 @@ TEST_F(RendererImplTest, VideoUnderflowWithAudioFlush) {
   EXPECT_CALL(time_source_, SetMediaTime(kStartTime));
   EXPECT_CALL(*audio_renderer_, StartPlaying())
       .WillOnce(
-          SetBufferingState(&audio_buffering_state_cb_, BUFFERING_HAVE_ENOUGH));
+          SetBufferingState(&audio_renderer_client_, BUFFERING_HAVE_ENOUGH));
   EXPECT_CALL(*video_renderer_, StartPlayingFrom(kStartTime));
   renderer_impl_->StartPlayingFrom(kStartTime);
 
