@@ -62,17 +62,20 @@ bool H264POC::ComputePicOrderCnt(
     return false;
   }
 
-  // TODO(sandersd): Handle |gaps_in_frame_num_value|.
-  if (prev_frame_num_ > 0 && prev_frame_num_ < slice_hdr.frame_num - 1) {
-    DLOG(ERROR) << "Gaps in frame_num are not supported";
-    return false;
-  }
-
   bool mmco5 = HasMMCO5(slice_hdr);
   int32_t max_frame_num = 1 << (sps->log2_max_frame_num_minus4 + 4);
   int32_t max_pic_order_cnt_lsb =
       1 << (sps->log2_max_pic_order_cnt_lsb_minus4 + 4);
 
+  // Check for invalid (including duplicate) |frame_num| values. All cases are
+  // treated as gaps, which is to say that nothing is done. (Gaps don't affect
+  // POC computation.)
+  if (!slice_hdr.idr_pic_flag &&
+      slice_hdr.frame_num != (prev_frame_num_ + 1) % max_frame_num) {
+    if (!sps->gaps_in_frame_num_value_allowed_flag)
+      DLOG(WARNING) << "Invalid gap in frame_num";
+  }
+
   // Based on T-REC-H.264 8.2.1, "Decoding process for picture order
   // count", available from http://www.itu.int/rec/T-REC-H.264.
   //
