@@ -12,8 +12,7 @@ AuthChallengeInfo::AuthChallengeInfo() : is_proxy(false) {
 
 bool AuthChallengeInfo::Equals(const AuthChallengeInfo& that) const {
   return (this->is_proxy == that.is_proxy &&
-          this->challenger.IsSameOriginWith(that.challenger) &&
-          this->scheme == that.scheme &&
+          this->challenger == that.challenger && this->scheme == that.scheme &&
           this->realm == that.realm);
 }
 
