@@ -11,7 +11,7 @@
 #include "base/files/file_util.h"
 #include "base/files/scoped_temp_dir.h"
 #include "base/run_loop.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "net/base/io_buffer.h"
 #include "net/base/net_errors.h"
 #include "net/base/test_completion_callback.h"
@@ -53,7 +53,7 @@ class UploadFileElementReaderTest : public PlatformTest {
   }
 
   std::vector<char> bytes_;
-  scoped_ptr<UploadElementReader> reader_;
+  std::unique_ptr<UploadElementReader> reader_;
   base::ScopedTempDir temp_dir_;
   base::FilePath temp_file_path_;
 };
