@@ -13,7 +13,6 @@
 #include "base/strings/utf_string_conversions.h"
 #include "net/base/host_port_pair.h"
 #include "net/base/io_buffer.h"
-#include "net/base/net_util.h"
 #include "net/base/test_completion_callback.h"
 #include "net/dns/mock_host_resolver.h"
 #include "net/ftp/ftp_request_info.h"
@@ -431,6 +430,34 @@ class FtpSocketDataProviderFileDownload : public FtpSocketDataProvider {
   DISALLOW_COPY_AND_ASSIGN(FtpSocketDataProviderFileDownload);
 };
 
+class FtpSocketDataProviderPathSeparatorsNotUnescaped
+    : public FtpSocketDataProvider {
+ public:
+  FtpSocketDataProviderPathSeparatorsNotUnescaped() {}
+
+  MockWriteResult OnWrite(const std::string& data) override {
+    if (InjectFault())
+      return MockWriteResult(ASYNC, data.length());
+    switch (state()) {
+      case PRE_SIZE:
+        return Verify("SIZE /foo%2f..%2fbar%5c\r\n", data, PRE_CWD,
+                      "213 18\r\n");
+      case PRE_CWD:
+        return Verify("CWD /foo%2f..%2fbar%5c\r\n", data,
+                      use_epsv() ? PRE_RETR_EPSV : PRE_RETR_PASV,
+                      "550 Not a directory\r\n");
+      case PRE_RETR:
+        return Verify("RETR /foo%2f..%2fbar%5c\r\n", data, PRE_QUIT,
+                      "200 OK\r\n");
+      default:
+        return FtpSocketDataProvider::OnWrite(data);
+    }
+  }
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(FtpSocketDataProviderPathSeparatorsNotUnescaped);
+};
+
 class FtpSocketDataProviderFileNotFound : public FtpSocketDataProvider {
  public:
   FtpSocketDataProviderFileNotFound() {
@@ -857,7 +884,7 @@ class FtpNetworkTransactionTest
       MockRead(mock_data.c_str()),
     };
 
-    scoped_ptr<StaticSocketDataProvider> data_socket(
+    std::unique_ptr<StaticSocketDataProvider> data_socket(
         new StaticSocketDataProvider(data_reads, arraysize(data_reads), NULL,
                                      0));
     mock_socket_factory_.AddSocketDataProvider(data_socket.get());
@@ -901,7 +928,7 @@ class FtpNetworkTransactionTest
     ExecuteTransaction(ctrl_socket, request, expected_result);
   }
 
-  scoped_ptr<MockHostResolver> host_resolver_;
+  std::unique_ptr<MockHostResolver> host_resolver_;
   MockClientSocketFactory mock_socket_factory_;
   FtpNetworkTransaction transaction_;
   TestCompletionCallback callback_;
@@ -1310,6 +1337,18 @@ TEST_P(FtpNetworkTransactionTest, DownloadTransactionSpaceInPassword) {
   ExecuteTransaction(&ctrl_socket, "ftp://test:hello%20world@host/file", OK);
 }
 
+// Make sure FtpNetworkTransaction doesn't request paths like
+// "/foo/../bar".  Doing so wouldn't be a security issue, client side, but just
+// doesn't seem like a good idea.
+TEST_P(FtpNetworkTransactionTest,
+       DownloadTransactionPathSeparatorsNotUnescaped) {
+  FtpSocketDataProviderPathSeparatorsNotUnescaped ctrl_socket;
+  ExecuteTransaction(&ctrl_socket, "ftp://host/foo%2f..%2fbar%5c", OK);
+
+  // We pass an artificial value of 18 as a response to the SIZE command.
+  EXPECT_EQ(18, transaction_.GetResponseInfo()->expected_content_size);
+}
+
 TEST_P(FtpNetworkTransactionTest, EvilRestartUser) {
   FtpSocketDataProvider ctrl_socket1;
   ctrl_socket1.InjectFailure(FtpSocketDataProvider::PRE_PASSWD,
