@@ -4,7 +4,6 @@
 
 #include <string>
 
-#include "base/basictypes.h"
 #include "base/strings/string_util.h"
 #include "base/strings/utf_string_conversions.h"
 #include "net/base/net_errors.h"
@@ -12,6 +11,7 @@
 #include "net/http/http_auth_challenge_tokenizer.h"
 #include "net/http/http_auth_handler_digest.h"
 #include "net/http/http_request_info.h"
+#include "net/ssl/ssl_info.h"
 #include "testing/gtest/include/gtest/gtest.h"
 
 namespace net {
@@ -45,17 +45,19 @@ bool RespondToChallenge(HttpAuth::Target target,
   EXPECT_FALSE(challenge.empty());
 
   token->clear();
-  scoped_ptr<HttpAuthHandlerDigest::Factory> factory(
+  std::unique_ptr<HttpAuthHandlerDigest::Factory> factory(
       new HttpAuthHandlerDigest::Factory());
   HttpAuthHandlerDigest::NonceGenerator* nonce_generator =
       new HttpAuthHandlerDigest::FixedNonceGenerator("client_nonce");
   factory->set_nonce_generator(nonce_generator);
-  scoped_ptr<HttpAuthHandler> handler;
+  std::unique_ptr<HttpAuthHandler> handler;
 
   // Create a handler for a particular challenge.
+  SSLInfo null_ssl_info;
   GURL url_origin(target == HttpAuth::AUTH_SERVER ? request_url : proxy_name);
   int rv_create = factory->CreateAuthHandlerFromString(
-    challenge, target, url_origin.GetOrigin(), BoundNetLog(), &handler);
+      challenge, target, null_ssl_info, url_origin.GetOrigin(), BoundNetLog(),
+      &handler);
   if (rv_create != OK || handler.get() == NULL) {
     ADD_FAILURE() << "Unable to create auth handler.";
     return false;
@@ -66,7 +68,7 @@ bool RespondToChallenge(HttpAuth::Target target,
   // completes synchronously. That's why this test can get away with a
   // TestCompletionCallback without an IO thread.
   TestCompletionCallback callback;
-  scoped_ptr<HttpRequestInfo> request(new HttpRequestInfo());
+  std::unique_ptr<HttpRequestInfo> request(new HttpRequestInfo());
   request->url = GURL(request_url);
   AuthCredentials credentials(base::ASCIIToUTF16("foo"),
                               base::ASCIIToUTF16("bar"));
@@ -349,15 +351,14 @@ TEST(HttpAuthHandlerDigestTest, ParseChallenge) {
   };
 
   GURL origin("http://www.example.com");
-  scoped_ptr<HttpAuthHandlerDigest::Factory> factory(
+  std::unique_ptr<HttpAuthHandlerDigest::Factory> factory(
       new HttpAuthHandlerDigest::Factory());
   for (size_t i = 0; i < arraysize(tests); ++i) {
-    scoped_ptr<HttpAuthHandler> handler;
-    int rv = factory->CreateAuthHandlerFromString(tests[i].challenge,
-                                                  HttpAuth::AUTH_SERVER,
-                                                  origin,
-                                                  BoundNetLog(),
-                                                  &handler);
+    SSLInfo null_ssl_info;
+    std::unique_ptr<HttpAuthHandler> handler;
+    int rv = factory->CreateAuthHandlerFromString(
+        tests[i].challenge, HttpAuth::AUTH_SERVER, null_ssl_info, origin,
+        BoundNetLog(), &handler);
     if (tests[i].parsed_success) {
       EXPECT_EQ(OK, rv);
     } else {
@@ -514,15 +515,14 @@ TEST(HttpAuthHandlerDigestTest, AssembleCredentials) {
     }
   };
   GURL origin("http://www.example.com");
-  scoped_ptr<HttpAuthHandlerDigest::Factory> factory(
+  std::unique_ptr<HttpAuthHandlerDigest::Factory> factory(
       new HttpAuthHandlerDigest::Factory());
   for (size_t i = 0; i < arraysize(tests); ++i) {
-    scoped_ptr<HttpAuthHandler> handler;
-    int rv = factory->CreateAuthHandlerFromString(tests[i].challenge,
-                                                  HttpAuth::AUTH_SERVER,
-                                                  origin,
-                                                  BoundNetLog(),
-                                                  &handler);
+    SSLInfo null_ssl_info;
+    std::unique_ptr<HttpAuthHandler> handler;
+    int rv = factory->CreateAuthHandlerFromString(
+        tests[i].challenge, HttpAuth::AUTH_SERVER, null_ssl_info, origin,
+        BoundNetLog(), &handler);
     EXPECT_EQ(OK, rv);
     ASSERT_TRUE(handler != NULL);
 
@@ -542,15 +542,16 @@ TEST(HttpAuthHandlerDigestTest, AssembleCredentials) {
 }
 
 TEST(HttpAuthHandlerDigest, HandleAnotherChallenge) {
-  scoped_ptr<HttpAuthHandlerDigest::Factory> factory(
+  std::unique_ptr<HttpAuthHandlerDigest::Factory> factory(
       new HttpAuthHandlerDigest::Factory());
-  scoped_ptr<HttpAuthHandler> handler;
+  std::unique_ptr<HttpAuthHandler> handler;
   std::string default_challenge =
       "Digest realm=\"Oblivion\", nonce=\"nonce-value\"";
   GURL origin("intranet.google.com");
+  SSLInfo null_ssl_info;
   int rv = factory->CreateAuthHandlerFromString(
-      default_challenge, HttpAuth::AUTH_SERVER, origin, BoundNetLog(),
-      &handler);
+      default_challenge, HttpAuth::AUTH_SERVER, null_ssl_info, origin,
+      BoundNetLog(), &handler);
   EXPECT_EQ(OK, rv);
   ASSERT_TRUE(handler.get() != NULL);
   HttpAuthChallengeTokenizer tok_default(default_challenge.begin(),
