@@ -5,6 +5,7 @@
 #include "net/http/http_cache.h"
 
 #include <algorithm>
+#include <utility>
 
 #include "base/bind.h"
 #include "base/bind_helpers.h"
@@ -13,6 +14,8 @@
 #include "base/files/file_util.h"
 #include "base/format_macros.h"
 #include "base/location.h"
+#include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/memory/ref_counted.h"
 #include "base/metrics/field_trial.h"
 #include "base/metrics/histogram_macros.h"
@@ -22,7 +25,7 @@
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_util.h"
 #include "base/strings/stringprintf.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/threading/worker_pool.h"
 #include "base/time/default_clock.h"
 #include "base/time/time.h"
@@ -32,7 +35,6 @@
 #include "net/base/net_errors.h"
 #include "net/base/upload_data_stream.h"
 #include "net/disk_cache/disk_cache.h"
-#include "net/http/disk_based_cert_cache.h"
 #include "net/http/disk_cache_based_quic_server_info.h"
 #include "net/http/http_cache_transaction.h"
 #include "net/http/http_network_layer.h"
@@ -47,15 +49,6 @@
 #include <unistd.h>
 #endif
 
-namespace {
-
-bool UseCertCache() {
-  return base::FieldTrialList::FindFullName("CertCacheTrial") ==
-         "ExperimentGroup";
-}
-
-}  // namespace
-
 namespace net {
 
 HttpCache::DefaultBackend::DefaultBackend(
@@ -74,13 +67,16 @@ HttpCache::DefaultBackend::DefaultBackend(
 HttpCache::DefaultBackend::~DefaultBackend() {}
 
 // static
-HttpCache::BackendFactory* HttpCache::DefaultBackend::InMemory(int max_bytes) {
-  return new DefaultBackend(MEMORY_CACHE, CACHE_BACKEND_DEFAULT,
-                            base::FilePath(), max_bytes, NULL);
+std::unique_ptr<HttpCache::BackendFactory> HttpCache::DefaultBackend::InMemory(
+    int max_bytes) {
+  return base::WrapUnique(
+      new DefaultBackend(MEMORY_CACHE, CACHE_BACKEND_DEFAULT, base::FilePath(),
+                         max_bytes, nullptr));
 }
 
 int HttpCache::DefaultBackend::CreateBackend(
-    NetLog* net_log, scoped_ptr<disk_cache::Backend>* backend,
+    NetLog* net_log,
+    std::unique_ptr<disk_cache::Backend>* backend,
     const CompletionCallback& callback) {
   DCHECK_GE(max_bytes_, 0);
   return disk_cache::CreateCacheBackend(type_,
@@ -119,7 +115,7 @@ struct HttpCache::PendingOp {
   ~PendingOp() {}
 
   disk_cache::Entry* disk_entry;
-  scoped_ptr<disk_cache::Backend> backend;
+  std::unique_ptr<disk_cache::Backend> backend;
   WorkItem* writer;
   CompletionCallback callback;  // BackendCallback.
   WorkItemList pending_queue;
@@ -216,7 +212,7 @@ class HttpCache::MetadataWriter {
   void SelfDestroy();
   void OnIOComplete(int result);
 
-  scoped_ptr<HttpCache::Transaction> transaction_;
+  std::unique_ptr<HttpCache::Transaction> transaction_;
   bool verified_;
   scoped_refptr<IOBuffer> buf_;
   int buf_len_;
@@ -294,49 +290,38 @@ class HttpCache::QuicServerInfoFactoryAdaptor : public QuicServerInfoFactory {
 };
 
 //-----------------------------------------------------------------------------
-HttpCache::HttpCache(const HttpNetworkSession::Params& params,
-                     BackendFactory* backend_factory)
-    : net_log_(params.net_log),
-      backend_factory_(backend_factory),
-      building_backend_(false),
-      bypass_lock_for_test_(false),
-      fail_conditionalization_for_test_(false),
-      mode_(NORMAL),
-      network_layer_(new HttpNetworkLayer(new HttpNetworkSession(params))),
-      clock_(new base::DefaultClock()),
-      weak_factory_(this) {
-  SetupQuicServerInfoFactory(network_layer_->GetSession());
-}
-
-
-// This call doesn't change the shared |session|'s QuicServerInfoFactory because
-// |session| is shared.
 HttpCache::HttpCache(HttpNetworkSession* session,
-                     BackendFactory* backend_factory)
-    : net_log_(session->net_log()),
-      backend_factory_(backend_factory),
-      building_backend_(false),
-      bypass_lock_for_test_(false),
-      fail_conditionalization_for_test_(false),
-      mode_(NORMAL),
-      network_layer_(new HttpNetworkLayer(session)),
-      clock_(new base::DefaultClock()),
-      weak_factory_(this) {
-}
+                     std::unique_ptr<BackendFactory> backend_factory,
+                     bool set_up_quic_server_info)
+    : HttpCache(base::WrapUnique(new HttpNetworkLayer(session)),
+                std::move(backend_factory),
+                set_up_quic_server_info) {}
 
-HttpCache::HttpCache(HttpTransactionFactory* network_layer,
-                     NetLog* net_log,
-                     BackendFactory* backend_factory)
-    : net_log_(net_log),
-      backend_factory_(backend_factory),
+HttpCache::HttpCache(std::unique_ptr<HttpTransactionFactory> network_layer,
+                     std::unique_ptr<BackendFactory> backend_factory,
+                     bool set_up_quic_server_info)
+    : net_log_(nullptr),
+      backend_factory_(std::move(backend_factory)),
       building_backend_(false),
       bypass_lock_for_test_(false),
       fail_conditionalization_for_test_(false),
       mode_(NORMAL),
-      network_layer_(network_layer),
+      network_layer_(std::move(network_layer)),
       clock_(new base::DefaultClock()),
       weak_factory_(this) {
-  SetupQuicServerInfoFactory(network_layer_->GetSession());
+  HttpNetworkSession* session = network_layer_->GetSession();
+  // Session may be NULL in unittests.
+  // TODO(mmenke): Seems like tests could be changed to provide a session,
+  // rather than having logic only used in unit tests here.
+  if (session) {
+    net_log_ = session->net_log();
+    if (set_up_quic_server_info &&
+        !session->quic_stream_factory()->has_quic_server_info_factory()) {
+      // QuicStreamFactory takes ownership of QuicServerInfoFactoryAdaptor.
+      session->quic_stream_factory()->set_quic_server_info_factory(
+          new QuicServerInfoFactoryAdaptor(this));
+    }
+  }
 }
 
 HttpCache::~HttpCache() {
@@ -361,7 +346,6 @@ HttpCache::~HttpCache() {
 
   // Before deleting pending_ops_, we have to make sure that the disk cache is
   // done with said operations, or it will attempt to use deleted data.
-  cert_cache_.reset();
   disk_cache_.reset();
 
   PendingOpsMap::iterator pending_it = pending_ops_.begin();
@@ -459,7 +443,7 @@ void HttpCache::OnExternalCacheHit(const GURL& url,
 }
 
 int HttpCache::CreateTransaction(RequestPriority priority,
-                                 scoped_ptr<HttpTransaction>* trans) {
+                                 std::unique_ptr<HttpTransaction>* trans) {
   // Do lazy initialization of disk cache if needed.
   if (!disk_cache_.get()) {
     // We don't care about the result.
@@ -485,12 +469,13 @@ HttpNetworkSession* HttpCache::GetSession() {
   return network_layer_->GetSession();
 }
 
-scoped_ptr<HttpTransactionFactory>
+std::unique_ptr<HttpTransactionFactory>
 HttpCache::SetHttpNetworkTransactionFactoryForTesting(
-    scoped_ptr<HttpTransactionFactory> new_network_layer) {
-  scoped_ptr<HttpTransactionFactory> old_network_layer(network_layer_.Pass());
-  network_layer_ = new_network_layer.Pass();
-  return old_network_layer.Pass();
+    std::unique_ptr<HttpTransactionFactory> new_network_layer) {
+  std::unique_ptr<HttpTransactionFactory> old_network_layer(
+      std::move(network_layer_));
+  network_layer_ = std::move(new_network_layer);
+  return old_network_layer;
 }
 
 //-----------------------------------------------------------------------------
@@ -502,8 +487,8 @@ int HttpCache::CreateBackend(disk_cache::Backend** backend,
 
   building_backend_ = true;
 
-  scoped_ptr<WorkItem> item(new WorkItem(WI_CREATE_BACKEND, NULL, callback,
-                                         backend));
+  std::unique_ptr<WorkItem> item(
+      new WorkItem(WI_CREATE_BACKEND, NULL, callback, backend));
 
   // This is the only operation that we can do that is not related to any given
   // entry, so we use an empty key for it.
@@ -996,16 +981,6 @@ bool HttpCache::RemovePendingTransactionFromPendingOp(PendingOp* pending_op,
   return false;
 }
 
-void HttpCache::SetupQuicServerInfoFactory(HttpNetworkSession* session) {
-  if (session &&
-      !session->quic_stream_factory()->has_quic_server_info_factory()) {
-    DCHECK(!quic_server_info_factory_);
-    quic_server_info_factory_.reset(new QuicServerInfoFactoryAdaptor(this));
-    session->quic_stream_factory()->set_quic_server_info_factory(
-        quic_server_info_factory_.get());
-  }
-}
-
 void HttpCache::ProcessPendingQueue(ActiveEntry* entry) {
   // Multiple readers may finish with an entry at once, so we want to batch up
   // calls to OnProcessPendingQueue.  This flag also tells us that we should
@@ -1050,7 +1025,7 @@ void HttpCache::OnIOComplete(int result, PendingOp* pending_op) {
   if (op == WI_CREATE_BACKEND)
     return OnBackendCreated(result, pending_op);
 
-  scoped_ptr<WorkItem> item(pending_op->writer);
+  std::unique_ptr<WorkItem> item(pending_op->writer);
   bool fail_requests = false;
 
   ActiveEntry* entry = NULL;
@@ -1146,7 +1121,7 @@ void HttpCache::OnPendingOpComplete(const base::WeakPtr<HttpCache>& cache,
 }
 
 void HttpCache::OnBackendCreated(int result, PendingOp* pending_op) {
-  scoped_ptr<WorkItem> item(pending_op->writer);
+  std::unique_ptr<WorkItem> item(pending_op->writer);
   WorkItemOperation op = item->operation();
   DCHECK_EQ(WI_CREATE_BACKEND, op);
 
@@ -1159,9 +1134,7 @@ void HttpCache::OnBackendCreated(int result, PendingOp* pending_op) {
     // and the last call clears building_backend_.
     backend_factory_.reset();  // Reclaim memory.
     if (result == OK) {
-      disk_cache_ = pending_op->backend.Pass();
-      if (UseCertCache())
-        cert_cache_.reset(new DiskBasedCertCache(disk_cache_.get()));
+      disk_cache_ = std::move(pending_op->backend);
     }
   }
 
