@@ -5,19 +5,21 @@
 #ifndef NET_HTTP_HTTP_STREAM_FACTORY_IMPL_JOB_H_
 #define NET_HTTP_HTTP_STREAM_FACTORY_IMPL_JOB_H_
 
+#include <memory>
+
 #include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/time/time.h"
 #include "net/base/completion_callback.h"
+#include "net/base/net_export.h"
 #include "net/base/request_priority.h"
+#include "net/http/bidirectional_stream_impl.h"
 #include "net/http/http_auth.h"
 #include "net/http/http_auth_controller.h"
 #include "net/http/http_request_info.h"
 #include "net/http/http_stream_factory_impl.h"
 #include "net/log/net_log.h"
-#include "net/net_features.h"
 #include "net/proxy/proxy_service.h"
 #include "net/quic/quic_stream_factory.h"
 #include "net/socket/client_socket_handle.h"
@@ -28,7 +30,6 @@
 
 namespace net {
 
-class BidirectionalStreamJob;
 class ClientSocketHandle;
 class HttpAuthController;
 class HttpNetworkSession;
@@ -40,32 +41,131 @@ class QuicHttpStream;
 // created for the StreamFactory.
 class HttpStreamFactoryImpl::Job {
  public:
+  // Delegate to report Job's status to Request and HttpStreamFactory.
+  class NET_EXPORT_PRIVATE Delegate {
+   public:
+    virtual ~Delegate() {}
+
+    // Invoked when |job| has an HttpStream ready.
+    virtual void OnStreamReady(Job* job,
+                               const SSLConfig& used_ssl_config,
+                               const ProxyInfo& used_proxy_info) = 0;
+
+    // Invoked when |job| has a BidirectionalStream ready.
+    virtual void OnBidirectionalStreamImplReady(
+        Job* job,
+        const SSLConfig& used_ssl_config,
+        const ProxyInfo& used_proxy_info) = 0;
+
+    // Invoked when |job| has a WebSocketHandshakeStream ready.
+    virtual void OnWebSocketHandshakeStreamReady(
+        Job* job,
+        const SSLConfig& used_ssl_config,
+        const ProxyInfo& used_proxy_info,
+        WebSocketHandshakeStreamBase* stream) = 0;
+
+    // Invoked when |job| fails to create a stream.
+    virtual void OnStreamFailed(Job* job,
+                                int status,
+                                const SSLConfig& used_ssl_config) = 0;
+
+    // Invoked when |job| has a certificate error for the Request.
+    virtual void OnCertificateError(Job* job,
+                                    int status,
+                                    const SSLConfig& used_ssl_config,
+                                    const SSLInfo& ssl_info) = 0;
+
+    // Invoked when |job| has a failure of the CONNECT request through an HTTPS
+    // proxy.
+    virtual void OnHttpsProxyTunnelResponse(
+        Job* job,
+        const HttpResponseInfo& response_info,
+        const SSLConfig& used_ssl_config,
+        const ProxyInfo& used_proxy_info,
+        HttpStream* stream) = 0;
+
+    // Invoked when |job| raises failure for SSL Client Auth.
+    virtual void OnNeedsClientAuth(Job* job,
+                                   const SSLConfig& used_ssl_config,
+                                   SSLCertRequestInfo* cert_info) = 0;
+
+    // Invoked when |job| needs proxy authentication.
+    virtual void OnNeedsProxyAuth(Job* job,
+                                  const HttpResponseInfo& proxy_response,
+                                  const SSLConfig& used_ssl_config,
+                                  const ProxyInfo& used_proxy_info,
+                                  HttpAuthController* auth_controller) = 0;
+
+    // Invoked to notify the Request and Factory of the readiness of new
+    // SPDY session.
+    virtual void OnNewSpdySessionReady(
+        Job* job,
+        const base::WeakPtr<SpdySession>& spdy_session,
+        bool direct) = 0;
+
+    // Invoked when the orphaned |job| finishes.
+    virtual void OnOrphanedJobComplete(const Job* job) = 0;
+
+    // Invoked when the |job| finishes pre-connecting sockets.
+    virtual void OnPreconnectsComplete(Job* job) = 0;
+
+    // Invoked to record connection attempts made by the socket layer to
+    // Request if |job| is associated with Request.
+    virtual void AddConnectionAttemptsToRequest(
+        Job* job,
+        const ConnectionAttempts& attempts) = 0;
+
+    // Called when |job| determines the appropriate |spdy_session_key| for the
+    // Request. Note that this does not mean that SPDY is necessarily supported
+    // for this SpdySessionKey, since we may need to wait for NPN to complete
+    // before knowing if SPDY is available.
+    virtual void SetSpdySessionKey(Job* job,
+                                   const SpdySessionKey& spdy_session_key) = 0;
+
+    // Remove session from the SpdySessionRequestMap.
+    virtual void RemoveRequestFromSpdySessionRequestMapForJob(Job* job) = 0;
+
+    virtual const BoundNetLog* GetNetLog(Job* job) const = 0;
+
+    virtual WebSocketHandshakeStreamBase::CreateHelper*
+    websocket_handshake_stream_create_helper() = 0;
+
+    virtual bool for_websockets() = 0;
+  };
+
   // Constructor for non-alternative Job.
-  Job(HttpStreamFactoryImpl* stream_factory,
+  // Job is owned by |delegate|, hence |delegate| is valid for the
+  // lifetime of the Job.
+  Job(Delegate* delegate,
+      JobType job_type,
       HttpNetworkSession* session,
       const HttpRequestInfo& request_info,
       RequestPriority priority,
       const SSLConfig& server_ssl_config,
       const SSLConfig& proxy_ssl_config,
-      HostPortPair server,
+      HostPortPair destination,
       GURL origin_url,
       NetLog* net_log);
+
   // Constructor for alternative Job.
-  Job(HttpStreamFactoryImpl* stream_factory,
+  // Job is owned by |delegate|, hence |delegate| is valid for the
+  // lifetime of the Job.
+  Job(Delegate* delegate,
+      JobType job_type,
       HttpNetworkSession* session,
       const HttpRequestInfo& request_info,
       RequestPriority priority,
       const SSLConfig& server_ssl_config,
       const SSLConfig& proxy_ssl_config,
-      HostPortPair server,
+      HostPortPair destination,
       GURL origin_url,
       AlternativeService alternative_service,
       NetLog* net_log);
-  ~Job();
+  virtual ~Job();
 
-  // Start initiates the process of creating a new HttpStream. |request| will be
-  // notified upon completion if the Job has not been Orphan()'d.
-  void Start(Request* request);
+  // Start initiates the process of creating a new HttpStream.
+  // |delegate_| will be notified upon completion.
+  virtual void Start(HttpStreamRequest::StreamType stream_type);
 
   // Preconnect will attempt to request |num_streams| sockets from the
   // appropriate ClientSocketPool.
@@ -83,8 +183,9 @@ class HttpStreamFactoryImpl::Job {
   // deleting it.
   void Resume(Job* job, const base::TimeDelta& delay);
 
-  // Used to detach the Job from |request|.
-  void Orphan(const Request* request);
+  // Called to detach |this| Job. May resume the other Job, will disconnect
+  // the socket for |this| Job, and notify |delegate| upon completion.
+  void Orphan();
 
   void SetPriority(RequestPriority priority);
 
@@ -93,26 +194,32 @@ class HttpStreamFactoryImpl::Job {
   NextProto protocol_negotiated() const;
   bool using_spdy() const;
   const BoundNetLog& net_log() const { return net_log_; }
-  bool for_bidirectional() const { return for_bidirectional_; }
+  HttpStreamRequest::StreamType stream_type() const { return stream_type_; }
+
+  std::unique_ptr<HttpStream> ReleaseStream() { return std::move(stream_); }
+
+  void SetStream(HttpStream* http_stream) { stream_.reset(http_stream); }
+
+  std::unique_ptr<BidirectionalStreamImpl> ReleaseBidirectionalStream() {
+    return std::move(bidirectional_stream_impl_);
+  }
 
   const SSLConfig& server_ssl_config() const;
   const SSLConfig& proxy_ssl_config() const;
   const ProxyInfo& proxy_info() const;
 
-  // Indicates whether or not this job is performing a preconnect.
-  bool IsPreconnecting() const;
-
-  // Indicates whether or not this Job has been orphaned by a Request.
-  bool IsOrphaned() const;
-
   // Called to indicate that this job succeeded, and some other jobs
   // will be orphaned.
   void ReportJobSucceededForRequest();
 
   // Marks that the other |job| has completed.
-  void MarkOtherJobComplete(const Job& job);
+  virtual void MarkOtherJobComplete(const Job& job);
+
+  JobType job_type() const { return job_type_; }
 
  private:
+  FRIEND_TEST_ALL_PREFIXES(HttpStreamFactoryImplRequestTest, DelayMainJob);
+
   enum State {
     STATE_START,
     STATE_RESOLVE_PROXY,
@@ -133,6 +240,8 @@ class HttpStreamFactoryImpl::Job {
     // The npn-spdy job will Resume() the http job if, in
     // STATE_INIT_CONNECTION_COMPLETE, it detects an error or does not find an
     // existing SpdySession. In that case, the http and npn-spdy jobs will race.
+    // When QUIC protocol is used by the npn-spdy job, then http job will wait
+    // for |wait_time_| when the http job was resumed.
     STATE_WAIT_FOR_JOB,
     STATE_WAIT_FOR_JOB_COMPLETE,
 
@@ -182,7 +291,7 @@ class HttpStreamFactoryImpl::Job {
     // |spdy_session| should not be used.
     int CreateAvailableSessionFromSocket(
         const SpdySessionKey& key,
-        scoped_ptr<ClientSocketHandle> connection,
+        std::unique_ptr<ClientSocketHandle> connection,
         const BoundNetLog& net_log,
         int certificate_error_code,
         bool is_secure,
@@ -201,8 +310,11 @@ class HttpStreamFactoryImpl::Job {
     const bool is_spdy_alternative_;
   };
 
+  // Resume the |this| job after the specified |wait_time_|.
+  void ResumeAfterDelay();
+
   void OnStreamReadyCallback();
-  void OnBidirectionalStreamJobReadyCallback();
+  void OnBidirectionalStreamImplReadyCallback();
   void OnWebSocketHandshakeStreamReadyCallback();
   // This callback function is called when a new SPDY session is created.
   void OnNewSpdySessionReadyCallback();
@@ -237,10 +349,10 @@ class HttpStreamFactoryImpl::Job {
   int DoRestartTunnelAuth();
   int DoRestartTunnelAuthComplete(int result);
 
-  // Creates a SpdyHttpStream or a BidirectionalStreamJob from the given values
-  // and sets to |stream_| or |bidirectional_stream_job_| respectively. Does
+  // Creates a SpdyHttpStream or a BidirectionalStreamImpl from the given values
+  // and sets to |stream_| or |bidirectional_stream_impl_| respectively. Does
   // nothing if |stream_factory_| is for WebSockets.
-  int SetSpdyHttpStreamOrBidirectionalStreamJob(
+  int SetSpdyHttpStreamOrBidirectionalStreamImpl(
       base::WeakPtr<SpdySession> session,
       bool direct);
 
@@ -256,11 +368,9 @@ class HttpStreamFactoryImpl::Job {
   bool IsSpdyAlternative() const;
   bool IsQuicAlternative() const;
 
-  // Sets several fields of |ssl_config| for |server| based on the proxy info
-  // and other factors.
-  void InitSSLConfig(const HostPortPair& server,
-                     SSLConfig* ssl_config,
-                     bool is_proxy) const;
+  // Sets several fields of |ssl_config| based on the proxy info and other
+  // factors.
+  void InitSSLConfig(SSLConfig* ssl_config, bool is_proxy) const;
 
   // Retrieve SSLInfo from our SSL Socket.
   // This must only be called when we are using an SSLSocket.
@@ -309,11 +419,10 @@ class HttpStreamFactoryImpl::Job {
   // session is found, and OK otherwise.
   static int OnHostResolution(SpdySessionPool* spdy_session_pool,
                               const SpdySessionKey& spdy_session_key,
+                              const GURL& origin_url,
                               const AddressList& addresses,
                               const BoundNetLog& net_log);
 
-  Request* request_;
-
   const HttpRequestInfo request_info_;
   RequestPriority priority_;
   ProxyInfo proxy_info_;
@@ -322,16 +431,15 @@ class HttpStreamFactoryImpl::Job {
   const BoundNetLog net_log_;
 
   CompletionCallback io_callback_;
-  scoped_ptr<ClientSocketHandle> connection_;
+  std::unique_ptr<ClientSocketHandle> connection_;
   HttpNetworkSession* const session_;
-  HttpStreamFactoryImpl* const stream_factory_;
   State next_state_;
   ProxyService::PacRequest* pac_request_;
   SSLInfo ssl_info_;
 
   // The server we are trying to reach, could be that of the origin or of the
-  // alternative service.
-  HostPortPair server_;
+  // alternative service (after applying host mapping rules).
+  HostPortPair destination_;
 
   // The origin url we're trying to reach. This url may be different from the
   // original request when host mapping rules are set-up.
@@ -343,6 +451,11 @@ class HttpStreamFactoryImpl::Job {
   // AlternativeService for the other Job if this is not an alternative Job.
   AlternativeService other_job_alternative_service_;
 
+  // Unowned. |this| job is owned by |delegate_|.
+  Delegate* delegate_;
+
+  JobType job_type_;
+
   // This is the Job we're dependent on. It will notify us if/when it's OK to
   // proceed.
   Job* blocking_job_;
@@ -352,6 +465,8 @@ class HttpStreamFactoryImpl::Job {
   // proceed and then race the two Jobs.
   Job* waiting_job_;
 
+  base::TimeDelta wait_time_;
+
   // True if handling a HTTPS request, or using SPDY with SSL
   bool using_ssl_;
 
@@ -378,11 +493,9 @@ class HttpStreamFactoryImpl::Job {
   // read from the socket until the tunnel is done.
   bool establishing_tunnel_;
 
-  scoped_ptr<HttpStream> stream_;
-  scoped_ptr<WebSocketHandshakeStreamBase> websocket_stream_;
-#if BUILDFLAG(ENABLE_BIDIRECTIONAL_STREAM)
-  scoped_ptr<BidirectionalStreamJob> bidirectional_stream_job_;
-#endif
+  std::unique_ptr<HttpStream> stream_;
+  std::unique_ptr<WebSocketHandshakeStreamBase> websocket_stream_;
+  std::unique_ptr<BidirectionalStreamImpl> bidirectional_stream_impl_;
 
   // True if we negotiated NPN.
   bool was_npn_negotiated_;
@@ -394,7 +507,7 @@ class HttpStreamFactoryImpl::Job {
   // preconnect.
   int num_streams_;
 
-  scoped_ptr<ValidSpdySessionPool> valid_spdy_session_pool_;
+  std::unique_ptr<ValidSpdySessionPool> valid_spdy_session_pool_;
 
   // Initialized when we create a new SpdySession.
   base::WeakPtr<SpdySession> new_spdy_session_;
@@ -407,15 +520,49 @@ class HttpStreamFactoryImpl::Job {
 
   JobStatus job_status_;
   JobStatus other_job_status_;
+  base::TimeTicks job_stream_ready_start_time_;
 
-  // True if BidirectionalStreamJob is requested.
-  bool for_bidirectional_;
+  // Type of stream that is requested.
+  HttpStreamRequest::StreamType stream_type_;
 
   base::WeakPtrFactory<Job> ptr_factory_;
 
   DISALLOW_COPY_AND_ASSIGN(Job);
 };
 
+// Factory for creating Jobs.
+class HttpStreamFactoryImpl::JobFactory {
+ public:
+  virtual ~JobFactory() {}
+
+  // Creates an alternative Job.
+  virtual HttpStreamFactoryImpl::Job* CreateJob(
+      HttpStreamFactoryImpl::Job::Delegate* delegate,
+      HttpStreamFactoryImpl::JobType job_type,
+      HttpNetworkSession* session,
+      const HttpRequestInfo& request_info,
+      RequestPriority priority,
+      const SSLConfig& server_ssl_config,
+      const SSLConfig& proxy_ssl_config,
+      HostPortPair destination,
+      GURL origin_url,
+      AlternativeService alternative_service,
+      NetLog* net_log) = 0;
+
+  // Creates a non-alternative Job.
+  virtual HttpStreamFactoryImpl::Job* CreateJob(
+      HttpStreamFactoryImpl::Job::Delegate* delegate,
+      HttpStreamFactoryImpl::JobType job_type,
+      HttpNetworkSession* session,
+      const HttpRequestInfo& request_info,
+      RequestPriority priority,
+      const SSLConfig& server_ssl_config,
+      const SSLConfig& proxy_ssl_config,
+      HostPortPair destination,
+      GURL origin_url,
+      NetLog* net_log) = 0;
+};
+
 }  // namespace net
 
 #endif  // NET_HTTP_HTTP_STREAM_FACTORY_IMPL_JOB_H_
