@@ -11,16 +11,18 @@
 
 #include "base/bind.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/stringprintf.h"
 #include "base/synchronization/cancellation_flag.h"
 #include "base/synchronization/waitable_event.h"
-#include "base/thread_task_runner_handle.h"
 #include "base/threading/thread.h"
 #include "base/threading/thread_restrictions.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "base/trace_event/trace_event.h"
 #include "net/base/address_list.h"
 #include "net/base/net_errors.h"
-#include "net/base/net_util.h"
+#include "net/base/network_interfaces.h"
 #include "net/dns/host_resolver.h"
 #include "net/proxy/proxy_info.h"
 #include "net/proxy/proxy_resolver_error_observer.h"
@@ -92,12 +94,12 @@ class Job : public base::RefCountedThreadSafe<Job>,
   // |params| is non-owned. It contains the parameters for this Job, and must
   // outlive it.
   Job(const Params* params,
-      scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings);
+      std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings);
 
   // Called from origin thread.
   void StartCreateV8Resolver(
       const scoped_refptr<ProxyResolverScriptData>& script_data,
-      scoped_ptr<ProxyResolverV8>* resolver,
+      std::unique_ptr<ProxyResolverV8>* resolver,
       const CompletionCallback& callback);
 
   // Called from origin thread.
@@ -208,7 +210,7 @@ class Job : public base::RefCountedThreadSafe<Job>,
   // Initialized on origin thread and then accessed from both threads.
   const Params* const params_;
 
-  scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings_;
+  std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings_;
 
   // The callback to run (on the origin thread) when the Job finishes.
   // Should only be accessed from origin thread.
@@ -243,7 +245,7 @@ class Job : public base::RefCountedThreadSafe<Job>,
   // -------------------------------------------------------
 
   scoped_refptr<ProxyResolverScriptData> script_data_;
-  scoped_ptr<ProxyResolverV8>* resolver_out_;
+  std::unique_ptr<ProxyResolverV8>* resolver_out_;
 
   // -------------------------------------------------------
   // State specific to GET_PROXY_FOR_URL.
@@ -301,9 +303,9 @@ class Job : public base::RefCountedThreadSafe<Job>,
 class ProxyResolverV8TracingImpl : public ProxyResolverV8Tracing,
                                    public base::NonThreadSafe {
  public:
-  ProxyResolverV8TracingImpl(scoped_ptr<base::Thread> thread,
-                             scoped_ptr<ProxyResolverV8> resolver,
-                             scoped_ptr<Job::Params> job_params);
+  ProxyResolverV8TracingImpl(std::unique_ptr<base::Thread> thread,
+                             std::unique_ptr<ProxyResolverV8> resolver,
+                             std::unique_ptr<Job::Params> job_params);
 
   ~ProxyResolverV8TracingImpl() override;
 
@@ -312,16 +314,16 @@ class ProxyResolverV8TracingImpl : public ProxyResolverV8Tracing,
                       ProxyInfo* results,
                       const CompletionCallback& callback,
                       ProxyResolver::RequestHandle* request,
-                      scoped_ptr<Bindings> bindings) override;
+                      std::unique_ptr<Bindings> bindings) override;
   void CancelRequest(ProxyResolver::RequestHandle request) override;
   LoadState GetLoadState(ProxyResolver::RequestHandle request) const override;
 
  private:
   // The worker thread on which the ProxyResolverV8 will be run.
-  scoped_ptr<base::Thread> thread_;
-  scoped_ptr<ProxyResolverV8> v8_resolver_;
+  std::unique_ptr<base::Thread> thread_;
+  std::unique_ptr<ProxyResolverV8> v8_resolver_;
 
-  scoped_ptr<Job::Params> job_params_;
+  std::unique_ptr<Job::Params> job_params_;
 
   // The number of outstanding (non-cancelled) jobs.
   int num_outstanding_callbacks_;
@@ -330,11 +332,12 @@ class ProxyResolverV8TracingImpl : public ProxyResolverV8Tracing,
 };
 
 Job::Job(const Job::Params* params,
-         scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings)
+         std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings)
     : origin_runner_(base::ThreadTaskRunnerHandle::Get()),
       params_(params),
       bindings_(std::move(bindings)),
-      event_(true, false),
+      event_(base::WaitableEvent::ResetPolicy::MANUAL,
+             base::WaitableEvent::InitialState::NOT_SIGNALED),
       last_num_dns_(0),
       pending_dns_(NULL) {
   CheckIsOnOriginThread();
@@ -342,7 +345,7 @@ Job::Job(const Job::Params* params,
 
 void Job::StartCreateV8Resolver(
     const scoped_refptr<ProxyResolverScriptData>& script_data,
-    scoped_ptr<ProxyResolverV8>* resolver,
+    std::unique_ptr<ProxyResolverV8>* resolver,
     const CompletionCallback& callback) {
   CheckIsOnOriginThread();
 
@@ -548,11 +551,12 @@ void Job::ExecuteNonBlocking() {
 }
 
 int Job::ExecuteProxyResolver() {
+  TRACE_EVENT0("net", "Job::ExecuteProxyResolver");
   int result = ERR_UNEXPECTED;  // Initialized to silence warnings.
 
   switch (operation_) {
     case CREATE_V8_RESOLVER: {
-      scoped_ptr<ProxyResolverV8> resolver;
+      std::unique_ptr<ProxyResolverV8> resolver;
       result = ProxyResolverV8::Create(script_data_, this, &resolver);
       if (result == OK)
         *resolver_out_ = std::move(resolver);
@@ -915,9 +919,9 @@ void Job::DispatchAlertOrErrorOnOriginThread(bool is_alert,
 }
 
 ProxyResolverV8TracingImpl::ProxyResolverV8TracingImpl(
-    scoped_ptr<base::Thread> thread,
-    scoped_ptr<ProxyResolverV8> resolver,
-    scoped_ptr<Job::Params> job_params)
+    std::unique_ptr<base::Thread> thread,
+    std::unique_ptr<ProxyResolverV8> resolver,
+    std::unique_ptr<Job::Params> job_params)
     : thread_(std::move(thread)),
       v8_resolver_(std::move(resolver)),
       job_params_(std::move(job_params)),
@@ -939,7 +943,7 @@ void ProxyResolverV8TracingImpl::GetProxyForURL(
     ProxyInfo* results,
     const CompletionCallback& callback,
     ProxyResolver::RequestHandle* request,
-    scoped_ptr<Bindings> bindings) {
+    std::unique_ptr<Bindings> bindings) {
   DCHECK(CalledOnValidThread());
   DCHECK(!callback.is_null());
 
@@ -970,10 +974,10 @@ class ProxyResolverV8TracingFactoryImpl : public ProxyResolverV8TracingFactory {
 
   void CreateProxyResolverV8Tracing(
       const scoped_refptr<ProxyResolverScriptData>& pac_script,
-      scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings,
-      scoped_ptr<ProxyResolverV8Tracing>* resolver,
+      std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings,
+      std::unique_ptr<ProxyResolverV8Tracing>* resolver,
       const CompletionCallback& callback,
-      scoped_ptr<ProxyResolverFactory::Request>* request) override;
+      std::unique_ptr<ProxyResolverFactory::Request>* request) override;
 
  private:
   class CreateJob;
@@ -989,9 +993,9 @@ class ProxyResolverV8TracingFactoryImpl::CreateJob
     : public ProxyResolverFactory::Request {
  public:
   CreateJob(ProxyResolverV8TracingFactoryImpl* factory,
-            scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings,
+            std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings,
             const scoped_refptr<ProxyResolverScriptData>& pac_script,
-            scoped_ptr<ProxyResolverV8Tracing>* resolver_out,
+            std::unique_ptr<ProxyResolverV8Tracing>* resolver_out,
             const CompletionCallback& callback)
       : factory_(factory),
         thread_(new base::Thread("Proxy Resolver")),
@@ -1053,11 +1057,11 @@ class ProxyResolverV8TracingFactoryImpl::CreateJob
   }
 
   ProxyResolverV8TracingFactoryImpl* factory_;
-  scoped_ptr<base::Thread> thread_;
-  scoped_ptr<Job::Params> job_params_;
+  std::unique_ptr<base::Thread> thread_;
+  std::unique_ptr<Job::Params> job_params_;
   scoped_refptr<Job> create_resolver_job_;
-  scoped_ptr<ProxyResolverV8> v8_resolver_;
-  scoped_ptr<ProxyResolverV8Tracing>* resolver_out_;
+  std::unique_ptr<ProxyResolverV8> v8_resolver_;
+  std::unique_ptr<ProxyResolverV8Tracing>* resolver_out_;
   const CompletionCallback callback_;
   int num_outstanding_callbacks_;
 
@@ -1068,21 +1072,21 @@ ProxyResolverV8TracingFactoryImpl::ProxyResolverV8TracingFactoryImpl() {
 }
 
 ProxyResolverV8TracingFactoryImpl::~ProxyResolverV8TracingFactoryImpl() {
-  for (auto job : jobs_) {
+  for (auto* job : jobs_) {
     job->FactoryDestroyed();
   }
 }
 
 void ProxyResolverV8TracingFactoryImpl::CreateProxyResolverV8Tracing(
     const scoped_refptr<ProxyResolverScriptData>& pac_script,
-    scoped_ptr<ProxyResolverV8Tracing::Bindings> bindings,
-    scoped_ptr<ProxyResolverV8Tracing>* resolver,
+    std::unique_ptr<ProxyResolverV8Tracing::Bindings> bindings,
+    std::unique_ptr<ProxyResolverV8Tracing>* resolver,
     const CompletionCallback& callback,
-    scoped_ptr<ProxyResolverFactory::Request>* request) {
-  scoped_ptr<CreateJob> job(
+    std::unique_ptr<ProxyResolverFactory::Request>* request) {
+  std::unique_ptr<CreateJob> job(
       new CreateJob(this, std::move(bindings), pac_script, resolver, callback));
   jobs_.insert(job.get());
-  request->reset(job.release());
+  *request = std::move(job);
 }
 
 void ProxyResolverV8TracingFactoryImpl::RemoveJob(
@@ -1094,9 +1098,9 @@ void ProxyResolverV8TracingFactoryImpl::RemoveJob(
 }  // namespace
 
 // static
-scoped_ptr<ProxyResolverV8TracingFactory>
+std::unique_ptr<ProxyResolverV8TracingFactory>
 ProxyResolverV8TracingFactory::Create() {
-  return make_scoped_ptr(new ProxyResolverV8TracingFactoryImpl());
+  return base::WrapUnique(new ProxyResolverV8TracingFactoryImpl());
 }
 
 }  // namespace net
