@@ -5,12 +5,16 @@
 #ifndef NET_PROXY_PROXY_SERVICE_H_
 #define NET_PROXY_PROXY_SERVICE_H_
 
+#include <stddef.h>
+
+#include <memory>
+#include <set>
 #include <string>
 #include <vector>
 
 #include "base/gtest_prod_util.h"
+#include "base/macros.h"
 #include "base/memory/ref_counted.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/synchronization/waitable_event.h"
 #include "base/threading/non_thread_safe.h"
 #include "net/base/completion_callback.h"
@@ -34,7 +38,7 @@ namespace net {
 
 class DhcpProxyScriptFetcher;
 class HostResolver;
-class NetworkDelegate;
+class ProxyDelegate;
 class ProxyResolver;
 class ProxyResolverFactory;
 class ProxyResolverScriptData;
@@ -111,11 +115,10 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
                               base::TimeDelta* next_delay) const = 0;
   };
 
-  // The instance takes ownership of |config_service| and |resolver_factory|.
   // |net_log| is a possibly NULL destination to send log events to. It must
   // remain alive for the lifetime of this ProxyService.
-  ProxyService(ProxyConfigService* config_service,
-               scoped_ptr<ProxyResolverFactory> resolver_factory,
+  ProxyService(std::unique_ptr<ProxyConfigService> config_service,
+               std::unique_ptr<ProxyResolverFactory> resolver_factory,
                NetLog* net_log);
 
   ~ProxyService() override;
@@ -124,6 +127,10 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   // TODO(eroman): consider naming this simply "Request".
   class PacRequest;
 
+  // Determines the appropriate proxy for |url| for a |method| request and
+  // stores the result in |results|. If |method| is empty, the caller can expect
+  // method independent resolution.
+  //
   // Returns ERR_IO_PENDING if the proxy information could not be provided
   // synchronously, to indicate that the result will be available when the
   // callback is run.  The callback is run on the thread that calls
@@ -144,19 +151,21 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   //
   // Profiling information for the request is saved to |net_log| if non-NULL.
   int ResolveProxy(const GURL& url,
+                   const std::string& method,
                    int load_flags,
                    ProxyInfo* results,
                    const CompletionCallback& callback,
                    PacRequest** pac_request,
-                   NetworkDelegate* network_delegate,
+                   ProxyDelegate* proxy_delegate,
                    const BoundNetLog& net_log);
 
   // Returns true if the proxy information could be determined without spawning
   // an asynchronous task.  Otherwise, |result| is unmodified.
   bool TryResolveProxySynchronously(const GURL& raw_url,
+                                    const std::string& method,
                                     int load_flags,
                                     ProxyInfo* result,
-                                    NetworkDelegate* network_delegate,
+                                    ProxyDelegate* proxy_delegate,
                                     const BoundNetLog& net_log);
 
   // This method is called after a failure to connect or resolve a host name.
@@ -174,12 +183,13 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   //
   // Profiling information for the request is saved to |net_log| if non-NULL.
   int ReconsiderProxyAfterError(const GURL& url,
+                                const std::string& method,
                                 int load_flags,
                                 int net_error,
                                 ProxyInfo* results,
                                 const CompletionCallback& callback,
                                 PacRequest** pac_request,
-                                NetworkDelegate* network_delegate,
+                                ProxyDelegate* proxy_delegate,
                                 const BoundNetLog& net_log);
 
   // Explicitly trigger proxy fallback for the given |results| by updating our
@@ -200,10 +210,10 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
 
   // Called to report that the last proxy connection succeeded.  If |proxy_info|
   // has a non empty proxy_retry_info map, the proxies that have been tried (and
-  // failed) for this request will be marked as bad. |network_delegate| will
+  // failed) for this request will be marked as bad. |proxy_delegate| will
   // be notified of any proxy fallbacks.
   void ReportSuccess(const ProxyInfo& proxy_info,
-                     NetworkDelegate* network_delegate);
+                     ProxyDelegate* proxy_delegate);
 
   // Call this method with a non-null |pac_request| to cancel the PAC request.
   void CancelPacRequest(PacRequest* pac_request);
@@ -213,18 +223,18 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
 
   // Sets the ProxyScriptFetcher and DhcpProxyScriptFetcher dependencies. This
   // is needed if the ProxyResolver is of type ProxyResolverWithoutFetch.
-  // ProxyService takes ownership of both objects.
+  // ProxyService takes ownership of proxy_script_fetcher.
   void SetProxyScriptFetchers(
       ProxyScriptFetcher* proxy_script_fetcher,
-      DhcpProxyScriptFetcher* dhcp_proxy_script_fetcher);
+      std::unique_ptr<DhcpProxyScriptFetcher> dhcp_proxy_script_fetcher);
   ProxyScriptFetcher* GetProxyScriptFetcher() const;
 
   // Tells this ProxyService to start using a new ProxyConfigService to
   // retrieve its ProxyConfig from. The new ProxyConfigService will immediately
   // be queried for new config info which will be used for all subsequent
-  // ResolveProxy calls. ProxyService takes ownership of
-  // |new_proxy_config_service|.
-  void ResetConfigService(ProxyConfigService* new_proxy_config_service);
+  // ResolveProxy calls.
+  void ResetConfigService(
+      std::unique_ptr<ProxyConfigService> new_proxy_config_service);
 
   // Returns the last configuration fetched from ProxyConfigService.
   const ProxyConfig& fetched_config() {
@@ -254,36 +264,37 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   // Same as CreateProxyServiceUsingV8ProxyResolver, except it uses system
   // libraries for evaluating the PAC script if available, otherwise skips
   // proxy autoconfig.
-  static ProxyService* CreateUsingSystemProxyResolver(
-      ProxyConfigService* proxy_config_service,
+  static std::unique_ptr<ProxyService> CreateUsingSystemProxyResolver(
+      std::unique_ptr<ProxyConfigService> proxy_config_service,
       size_t num_pac_threads,
       NetLog* net_log);
 
   // Creates a ProxyService without support for proxy autoconfig.
-  static ProxyService* CreateWithoutProxyResolver(
-      ProxyConfigService* proxy_config_service,
+  static std::unique_ptr<ProxyService> CreateWithoutProxyResolver(
+      std::unique_ptr<ProxyConfigService> proxy_config_service,
       NetLog* net_log);
 
   // Convenience methods that creates a proxy service using the
   // specified fixed settings.
-  static ProxyService* CreateFixed(const ProxyConfig& pc);
-  static ProxyService* CreateFixed(const std::string& proxy);
+  static std::unique_ptr<ProxyService> CreateFixed(const ProxyConfig& pc);
+  static std::unique_ptr<ProxyService> CreateFixed(const std::string& proxy);
 
   // Creates a proxy service that uses a DIRECT connection for all requests.
-  static ProxyService* CreateDirect();
+  static std::unique_ptr<ProxyService> CreateDirect();
   // |net_log|'s lifetime must exceed ProxyService.
-  static ProxyService* CreateDirectWithNetLog(NetLog* net_log);
+  static std::unique_ptr<ProxyService> CreateDirectWithNetLog(NetLog* net_log);
 
   // This method is used by tests to create a ProxyService that returns a
   // hardcoded proxy fallback list (|pac_string|) for every URL.
   //
   // |pac_string| is a list of proxy servers, in the format that a PAC script
   // would return it. For example, "PROXY foobar:99; SOCKS fml:2; DIRECT"
-  static ProxyService* CreateFixedFromPacResult(const std::string& pac_string);
+  static std::unique_ptr<ProxyService> CreateFixedFromPacResult(
+      const std::string& pac_string);
 
   // Creates a config service appropriate for this platform that fetches the
   // system proxy settings.
-  static ProxyConfigService* CreateSystemProxyConfigService(
+  static std::unique_ptr<ProxyConfigService> CreateSystemProxyConfigService(
       const scoped_refptr<base::SingleThreadTaskRunner>& io_task_runner,
       const scoped_refptr<base::SingleThreadTaskRunner>& file_task_runner);
 
@@ -299,14 +310,12 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
 
   // This method should only be used by unit tests. Creates an instance
   // of the default internal PacPollPolicy used by ProxyService.
-  static scoped_ptr<PacPollPolicy> CreateDefaultPacPollPolicy();
+  static std::unique_ptr<PacPollPolicy> CreateDefaultPacPollPolicy();
 
   void set_quick_check_enabled(bool value) {
     quick_check_enabled_ = value;
   }
 
-  bool quick_check_enabled() const { return quick_check_enabled_; }
-
   void set_sanitize_url_policy(SanitizeUrlPolicy policy) {
     sanitize_url_policy_ = policy;
   }
@@ -318,11 +327,7 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   class InitProxyResolver;
   class ProxyScriptDeciderPoller;
 
-  // TODO(eroman): change this to a std::set. Note that this requires updating
-  // some tests in proxy_service_unittest.cc such as:
-  //   ProxyServiceTest.InitialPACScriptDownload
-  // which expects requests to finish in the order they were added.
-  typedef std::vector<scoped_refptr<PacRequest> > PendingRequests;
+  typedef std::set<scoped_refptr<PacRequest>> PendingRequests;
 
   enum State {
     STATE_NONE,
@@ -352,18 +357,19 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   // Completing synchronously means we don't need to query ProxyResolver.
   int TryToCompleteSynchronously(const GURL& url,
                                  int load_flags,
-                                 NetworkDelegate* network_delegate,
+                                 ProxyDelegate* proxy_delegate,
                                  ProxyInfo* result);
 
   // Identical to ResolveProxy, except that |callback| is permitted to be null.
   // if |callback.is_null()|, this function becomes a thin wrapper around
   // |TryToCompleteSynchronously|.
   int ResolveProxyHelper(const GURL& url,
+                         const std::string& method,
                          int load_flags,
                          ProxyInfo* results,
                          const CompletionCallback& callback,
                          PacRequest** pac_request,
-                         NetworkDelegate* network_delegate,
+                         ProxyDelegate* proxy_delegate,
                          const BoundNetLog& net_log);
 
   // Cancels all of the requests sent to the ProxyResolver. These will be
@@ -384,8 +390,9 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   // asynchronously). Handles logging the result, and cleaning out
   // bad entries from the results list.
   int DidFinishResolvingProxy(const GURL& url,
+                              const std::string& method,
                               int load_flags,
-                              NetworkDelegate* network_delegate,
+                              ProxyDelegate* proxy_delegate,
                               ProxyInfo* result,
                               int result_code,
                               const BoundNetLog& net_log,
@@ -414,9 +421,9 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
       const ProxyConfig& config,
       ProxyConfigService::ConfigAvailability availability) override;
 
-  scoped_ptr<ProxyConfigService> config_service_;
-  scoped_ptr<ProxyResolverFactory> resolver_factory_;
-  scoped_ptr<ProxyResolver> resolver_;
+  std::unique_ptr<ProxyConfigService> config_service_;
+  std::unique_ptr<ProxyResolverFactory> resolver_factory_;
+  std::unique_ptr<ProxyResolver> resolver_;
 
   // We store the proxy configuration that was last fetched from the
   // ProxyConfigService, as well as the resulting "effective" configuration.
@@ -441,21 +448,21 @@ class NET_EXPORT ProxyService : public NetworkChangeNotifier::IPAddressObserver,
   // The fetcher to use when downloading PAC scripts for the ProxyResolver.
   // This dependency can be NULL if our ProxyResolver has no need for
   // external PAC script fetching.
-  scoped_ptr<ProxyScriptFetcher> proxy_script_fetcher_;
+  std::unique_ptr<ProxyScriptFetcher> proxy_script_fetcher_;
 
   // The fetcher to use when attempting to download the most appropriate PAC
   // script configured in DHCP, if any. Can be NULL if the ProxyResolver has
   // no need for DHCP PAC script fetching.
-  scoped_ptr<DhcpProxyScriptFetcher> dhcp_proxy_script_fetcher_;
+  std::unique_ptr<DhcpProxyScriptFetcher> dhcp_proxy_script_fetcher_;
 
   // Helper to download the PAC script (wpad + custom) and apply fallback rules.
   //
   // Note that the declaration is important here: |proxy_script_fetcher_| and
   // |proxy_resolver_| must outlive |init_proxy_resolver_|.
-  scoped_ptr<InitProxyResolver> init_proxy_resolver_;
+  std::unique_ptr<InitProxyResolver> init_proxy_resolver_;
 
   // Helper to poll the PAC script for changes.
-  scoped_ptr<ProxyScriptDeciderPoller> script_poller_;
+  std::unique_ptr<ProxyScriptDeciderPoller> script_poller_;
 
   State current_state_;
 
