@@ -5,6 +5,10 @@
 #ifndef NET_QUIC_CRYPTO_CHACHA20_POLY1305_DECRYPTER_H_
 #define NET_QUIC_CRYPTO_CHACHA20_POLY1305_DECRYPTER_H_
 
+#include <stddef.h>
+#include <stdint.h>
+
+#include "base/macros.h"
 #include "net/quic/crypto/aead_base_decrypter.h"
 
 namespace net {
@@ -26,17 +30,8 @@ class NET_EXPORT_PRIVATE ChaCha20Poly1305Decrypter : public AeadBaseDecrypter {
   ChaCha20Poly1305Decrypter();
   ~ChaCha20Poly1305Decrypter() override;
 
-  // Returns true if the underlying crypto library supports ChaCha20+Poly1305.
-  static bool IsSupported();
-
-#if !defined(USE_OPENSSL)
- protected:
-  // AeadBaseDecrypter methods:
-  void FillAeadParams(base::StringPiece nonce,
-                      const base::StringPiece& associated_data,
-                      size_t auth_tag_size,
-                      AeadParams* aead_params) const override;
-#endif
+  const char* cipher_name() const override;
+  uint32_t cipher_id() const override;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ChaCha20Poly1305Decrypter);
