@@ -18,10 +18,11 @@ class DefaultCryptoStreamFactory : public QuicCryptoClientStreamFactory {
   QuicCryptoClientStream* CreateQuicCryptoClientStream(
       const QuicServerId& server_id,
       QuicChromiumClientSession* session,
-      scoped_ptr<ProofVerifyContext> proof_verify_context,
+      std::unique_ptr<ProofVerifyContext> proof_verify_context,
       QuicCryptoClientConfig* crypto_config) override {
-    return new QuicCryptoClientStream(
-        server_id, session, proof_verify_context.release(), crypto_config);
+    return new QuicCryptoClientStream(server_id, session,
+                                      proof_verify_context.release(),
+                                      crypto_config, session);
   }
 };
 
