@@ -5,15 +5,18 @@
 #ifndef NET_QUIC_QUIC_CRYPTO_CLIENT_STREAM_FACTORY_H_
 #define NET_QUIC_QUIC_CRYPTO_CLIENT_STREAM_FACTORY_H_
 
+#include <memory>
 #include <string>
 
 #include "net/base/net_export.h"
+#include "net/quic/quic_server_id.h"
 
 namespace net {
 
-class QuicClientSession;
+class ProofVerifyContext;
+class QuicChromiumClientSession;
+class QuicCryptoClientConfig;
 class QuicCryptoClientStream;
-class QuicServerId;
 
 // An interface used to instantiate QuicCryptoClientStream objects. Used to
 // facilitate testing code with mock implementations.
@@ -23,8 +26,11 @@ class NET_EXPORT QuicCryptoClientStreamFactory {
 
   virtual QuicCryptoClientStream* CreateQuicCryptoClientStream(
       const QuicServerId& server_id,
-      QuicClientSession* session,
+      QuicChromiumClientSession* session,
+      std::unique_ptr<ProofVerifyContext> proof_verify_context,
       QuicCryptoClientConfig* crypto_config) = 0;
+
+  static QuicCryptoClientStreamFactory* GetDefaultFactory();
 };
 
 }  // namespace net
