@@ -6,13 +6,12 @@
 
 namespace net {
 
-const QuicPriority QuicWriteBlockedList::kHighestPriority =
-    static_cast<QuicPriority>(net::kHighestPriority);
-const QuicPriority QuicWriteBlockedList::kLowestPriority =
-    static_cast<QuicPriority>(net::kLowestPriority);
-
 QuicWriteBlockedList::QuicWriteBlockedList()
-    : crypto_stream_blocked_(false), headers_stream_blocked_(false) {
+    : last_priority_popped_(0),
+      crypto_stream_blocked_(false),
+      headers_stream_blocked_(false) {
+  memset(batch_write_stream_id_, 0, sizeof(batch_write_stream_id_));
+  memset(bytes_left_for_batch_write_, 0, sizeof(bytes_left_for_batch_write_));
 }
 
 QuicWriteBlockedList::~QuicWriteBlockedList() {}
