@@ -11,18 +11,18 @@ namespace test {
 
 MockQuicDispatcher::MockQuicDispatcher(
     const QuicConfig& config,
-    const QuicCryptoServerConfig& crypto_config,
-    QuicDispatcher::PacketWriterFactory* packet_writer_factory,
-    QuicConnectionHelperInterface* helper)
+    const QuicCryptoServerConfig* crypto_config,
+    std::unique_ptr<QuicConnectionHelperInterface> helper,
+    std::unique_ptr<QuicServerSessionBase::Helper> session_helper,
+    std::unique_ptr<QuicAlarmFactory> alarm_factory)
     : QuicDispatcher(config,
                      crypto_config,
                      QuicSupportedVersions(),
-                     packet_writer_factory,
-                     helper) {
-}
+                     std::move(helper),
+                     std::move(session_helper),
+                     std::move(alarm_factory)) {}
 
-MockQuicDispatcher::~MockQuicDispatcher() {
-}
+MockQuicDispatcher::~MockQuicDispatcher() {}
 
 }  // namespace test
 }  // namespace net
