@@ -12,14 +12,14 @@ namespace test {
 // static
 void QuicConfigPeer::SetReceivedSocketReceiveBuffer(
     QuicConfig* config,
-    uint32 receive_buffer_bytes) {
+    uint32_t receive_buffer_bytes) {
   config->socket_receive_buffer_.SetReceivedValue(receive_buffer_bytes);
 }
 
 // static
 void QuicConfigPeer::SetReceivedInitialStreamFlowControlWindow(
     QuicConfig* config,
-    uint32 window_bytes) {
+    uint32_t window_bytes) {
   config->initial_stream_flow_control_window_bytes_.SetReceivedValue(
       window_bytes);
 }
@@ -27,7 +27,7 @@ void QuicConfigPeer::SetReceivedInitialStreamFlowControlWindow(
 // static
 void QuicConfigPeer::SetReceivedInitialSessionFlowControlWindow(
     QuicConfig* config,
-    uint32 window_bytes) {
+    uint32_t window_bytes) {
   config->initial_session_flow_control_window_bytes_.SetReceivedValue(
       window_bytes);
 }
@@ -41,9 +41,22 @@ void QuicConfigPeer::SetReceivedConnectionOptions(
 
 // static
 void QuicConfigPeer::SetReceivedBytesForConnectionId(QuicConfig* config,
-                                                     uint32 bytes) {
+                                                     uint32_t bytes) {
+  DCHECK(bytes == 0 || bytes == 8);
   config->bytes_for_connection_id_.SetReceivedValue(bytes);
 }
 
+// static
+void QuicConfigPeer::SetReceivedDisableConnectionMigration(QuicConfig* config) {
+  config->connection_migration_disabled_.SetReceivedValue(1);
+}
+
+// static
+void QuicConfigPeer::SetReceivedMaxIncomingDynamicStreams(
+    QuicConfig* config,
+    uint32_t max_streams) {
+  config->max_incoming_dynamic_streams_.SetReceivedValue(max_streams);
+}
+
 }  // namespace test
 }  // namespace net
