@@ -5,7 +5,9 @@
 #include "net/quic/test_tools/quic_test_packet_maker.h"
 
 #include <list>
+#include <utility>
 
+#include "base/memory/ptr_util.h"
 #include "net/quic/quic_framer.h"
 #include "net/quic/quic_http_utils.h"
 #include "net/quic/quic_utils.h"
@@ -19,13 +21,15 @@ namespace test {
 QuicTestPacketMaker::QuicTestPacketMaker(QuicVersion version,
                                          QuicConnectionId connection_id,
                                          MockClock* clock,
-                                         const std::string& host)
+                                         const std::string& host,
+                                         Perspective perspective)
     : version_(version),
       connection_id_(connection_id),
       clock_(clock),
       host_(host),
       spdy_request_framer_(HTTP2),
-      spdy_response_framer_(HTTP2) {}
+      spdy_response_framer_(HTTP2),
+      perspective_(perspective) {}
 
 QuicTestPacketMaker::~QuicTestPacketMaker() {}
 
@@ -33,7 +37,7 @@ void QuicTestPacketMaker::set_hostname(const std::string& host) {
   host_.assign(host);
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakePingPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakePingPacket(
     QuicPacketNumber num,
     bool include_version) {
   QuicPacketHeader header;
@@ -44,17 +48,26 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakePingPacket(
   header.packet_number = num;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
   QuicPingFrame ping;
-  return scoped_ptr<QuicEncryptedPacket>(MakePacket(header, QuicFrame(ping)));
+  return std::unique_ptr<QuicReceivedPacket>(
+      MakePacket(header, QuicFrame(ping)));
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeRstPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeRstPacket(
     QuicPacketNumber num,
     bool include_version,
     QuicStreamId stream_id,
     QuicRstStreamErrorCode error_code) {
+  return MakeRstPacket(num, include_version, stream_id, error_code, 0);
+}
+
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeRstPacket(
+    QuicPacketNumber num,
+    bool include_version,
+    QuicStreamId stream_id,
+    QuicRstStreamErrorCode error_code,
+    size_t bytes_written) {
   QuicPacketHeader header;
   header.public_header.connection_id = connection_id_;
   header.public_header.reset_flag = false;
@@ -63,19 +76,21 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeRstPacket(
   header.packet_number = num;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
-  QuicRstStreamFrame rst(stream_id, error_code, 0);
-  return scoped_ptr<QuicEncryptedPacket>(MakePacket(header, QuicFrame(&rst)));
+  QuicRstStreamFrame rst(stream_id, error_code, bytes_written);
+  DVLOG(1) << "Adding frame: " << QuicFrame(&rst);
+  return std::unique_ptr<QuicReceivedPacket>(
+      MakePacket(header, QuicFrame(&rst)));
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckAndRstPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeAckAndRstPacket(
     QuicPacketNumber num,
     bool include_version,
     QuicStreamId stream_id,
     QuicRstStreamErrorCode error_code,
     QuicPacketNumber largest_received,
-    QuicPacketNumber least_unacked,
+    QuicPacketNumber ack_least_unacked,
+    QuicPacketNumber stop_least_unacked,
     bool send_feedback) {
   QuicPacketHeader header;
   header.public_header.connection_id = connection_id_;
@@ -85,44 +100,50 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckAndRstPacket(
   header.packet_number = num;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
   QuicAckFrame ack(MakeAckFrame(largest_received));
-  ack.delta_time_largest_observed = QuicTime::Delta::Zero();
-  for (QuicPacketNumber i = least_unacked; i <= largest_received; ++i) {
+  ack.ack_delay_time = QuicTime::Delta::Zero();
+  for (QuicPacketNumber i = ack_least_unacked; i <= largest_received; ++i) {
     ack.received_packet_times.push_back(make_pair(i, clock_->Now()));
   }
+  if (largest_received > 0 && version_ > QUIC_VERSION_33) {
+    ack.missing = false;
+    ack.packets.Add(1, largest_received + 1);
+  }
   QuicFrames frames;
   frames.push_back(QuicFrame(&ack));
+  DVLOG(1) << "Adding frame: " << frames[0];
 
   QuicStopWaitingFrame stop_waiting;
-  stop_waiting.least_unacked = least_unacked;
+  stop_waiting.least_unacked = stop_least_unacked;
   frames.push_back(QuicFrame(&stop_waiting));
+  DVLOG(1) << "Adding frame: " << frames[1];
 
   QuicRstStreamFrame rst(stream_id, error_code, 0);
   frames.push_back(QuicFrame(&rst));
+  DVLOG(1) << "Adding frame: " << frames[2];
 
-  QuicFramer framer(SupportedVersions(version_), clock_->Now(),
-                    Perspective::IS_CLIENT);
-  scoped_ptr<QuicPacket> packet(
+  QuicFramer framer(SupportedVersions(version_), clock_->Now(), perspective_);
+  std::unique_ptr<QuicPacket> packet(
       BuildUnsizedDataPacket(&framer, header, frames));
   char buffer[kMaxPacketSize];
-  size_t encrypted_size = framer.EncryptPayload(
-      ENCRYPTION_NONE, header.packet_number, *packet, buffer, kMaxPacketSize);
+  size_t encrypted_size = framer.EncryptPayload(ENCRYPTION_NONE, /*path_id=*/0u,
+                                                header.packet_number, *packet,
+                                                buffer, kMaxPacketSize);
   EXPECT_NE(0u, encrypted_size);
-  QuicEncryptedPacket encrypted(buffer, encrypted_size, false);
-  return scoped_ptr<QuicEncryptedPacket>(encrypted.Clone());
+  QuicReceivedPacket encrypted(buffer, encrypted_size, QuicTime::Zero(), false);
+  return std::unique_ptr<QuicReceivedPacket>(encrypted.Clone());
 }
 
-scoped_ptr<QuicEncryptedPacket>
+std::unique_ptr<QuicReceivedPacket>
 QuicTestPacketMaker::MakeAckAndConnectionClosePacket(
     QuicPacketNumber num,
     bool include_version,
-    QuicTime::Delta delta_time_largest_observed,
+    QuicTime::Delta ack_delay_time,
     QuicPacketNumber largest_received,
     QuicPacketNumber least_unacked,
     QuicErrorCode quic_error,
-    std::string& quic_error_details) {
+    const std::string& quic_error_details) {
   QuicPacketHeader header;
   header.public_header.connection_id = connection_id_;
   header.public_header.reset_flag = false;
@@ -131,13 +152,16 @@ QuicTestPacketMaker::MakeAckAndConnectionClosePacket(
   header.packet_number = num;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
   QuicAckFrame ack(MakeAckFrame(largest_received));
-  ack.delta_time_largest_observed = delta_time_largest_observed;
+  ack.ack_delay_time = ack_delay_time;
   for (QuicPacketNumber i = least_unacked; i <= largest_received; ++i) {
     ack.received_packet_times.push_back(make_pair(i, clock_->Now()));
   }
+  if (largest_received > 0 && version_ > QUIC_VERSION_33) {
+    ack.missing = false;
+    ack.packets.Add(1, largest_received + 1);
+  }
   QuicFrames frames;
   frames.push_back(QuicFrame(&ack));
   DVLOG(1) << "Adding frame: " << frames[0];
@@ -154,20 +178,20 @@ QuicTestPacketMaker::MakeAckAndConnectionClosePacket(
   frames.push_back(QuicFrame(&close));
   DVLOG(1) << "Adding frame: " << frames[2];
 
-  QuicFramer framer(SupportedVersions(version_), clock_->Now(),
-                    Perspective::IS_CLIENT);
-  scoped_ptr<QuicPacket> packet(
+  QuicFramer framer(SupportedVersions(version_), clock_->Now(), perspective_);
+  std::unique_ptr<QuicPacket> packet(
       BuildUnsizedDataPacket(&framer, header, frames));
   char buffer[kMaxPacketSize];
-  size_t encrypted_size = framer.EncryptPayload(
-      ENCRYPTION_NONE, header.packet_number, *packet, buffer, kMaxPacketSize);
+  size_t encrypted_size = framer.EncryptPayload(ENCRYPTION_NONE, /*path_id=*/0u,
+                                                header.packet_number, *packet,
+                                                buffer, kMaxPacketSize);
   EXPECT_NE(0u, encrypted_size);
-  QuicEncryptedPacket encrypted(buffer, encrypted_size, false);
-  return scoped_ptr<QuicEncryptedPacket>(encrypted.Clone());
+  QuicReceivedPacket encrypted(buffer, encrypted_size, clock_->Now(), false);
+  return std::unique_ptr<QuicReceivedPacket>(encrypted.Clone());
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeConnectionClosePacket(
-    QuicPacketNumber num) {
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeConnectionClosePacket(QuicPacketNumber num) {
   QuicPacketHeader header;
   header.public_header.connection_id = connection_id_;
   header.public_header.reset_flag = false;
@@ -176,17 +200,38 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeConnectionClosePacket(
   header.packet_number = num;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
   QuicConnectionCloseFrame close;
   close.error_code = QUIC_CRYPTO_VERSION_NOT_SUPPORTED;
   close.error_details = "Time to panic!";
-  return scoped_ptr<QuicEncryptedPacket>(MakePacket(header, QuicFrame(&close)));
+  return std::unique_ptr<QuicReceivedPacket>(
+      MakePacket(header, QuicFrame(&close)));
+}
+
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeGoAwayPacket(
+    QuicPacketNumber num,
+    QuicErrorCode error_code,
+    std::string reason_phrase) {
+  QuicPacketHeader header;
+  header.public_header.connection_id = connection_id_;
+  header.public_header.reset_flag = false;
+  header.public_header.version_flag = false;
+  header.public_header.packet_number_length = PACKET_1BYTE_PACKET_NUMBER;
+  header.packet_number = num;
+  header.entropy_flag = false;
+  header.fec_flag = false;
+
+  QuicGoAwayFrame goaway;
+  goaway.error_code = error_code;
+  goaway.last_good_stream_id = 0;
+  goaway.reason_phrase = reason_phrase;
+  return std::unique_ptr<QuicReceivedPacket>(
+      MakePacket(header, QuicFrame(&goaway)));
 }
 
 // Sets both least_unacked fields in stop waiting frame and ack frame
 // to be |least_unacked|.
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeAckPacket(
     QuicPacketNumber packet_number,
     QuicPacketNumber largest_received,
     QuicPacketNumber least_unacked,
@@ -195,7 +240,7 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckPacket(
                        least_unacked, send_feedback);
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeAckPacket(
     QuicPacketNumber packet_number,
     QuicPacketNumber largest_received,
     QuicPacketNumber ack_least_unacked,
@@ -209,35 +254,40 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckPacket(
   header.packet_number = packet_number;
   header.entropy_flag = false;
   header.fec_flag = false;
-  header.fec_group = 0;
 
   QuicAckFrame ack(MakeAckFrame(largest_received));
-  ack.delta_time_largest_observed = QuicTime::Delta::Zero();
+  ack.ack_delay_time = QuicTime::Delta::Zero();
   for (QuicPacketNumber i = ack_least_unacked; i <= largest_received; ++i) {
     ack.received_packet_times.push_back(make_pair(i, clock_->Now()));
   }
+  if (largest_received > 0 && version_ > QUIC_VERSION_33) {
+    ack.missing = false;
+    ack.packets.Add(1, largest_received + 1);
+  }
 
-  QuicFramer framer(SupportedVersions(version_), clock_->Now(),
-                    Perspective::IS_CLIENT);
+  QuicFramer framer(SupportedVersions(version_), clock_->Now(), perspective_);
   QuicFrames frames;
-  frames.push_back(QuicFrame(&ack));
+  QuicFrame ack_frame(&ack);
+  DVLOG(1) << "Adding frame: " << ack_frame;
+  frames.push_back(ack_frame);
 
   QuicStopWaitingFrame stop_waiting;
   stop_waiting.least_unacked = stop_least_unacked;
   frames.push_back(QuicFrame(&stop_waiting));
 
-  scoped_ptr<QuicPacket> packet(
+  std::unique_ptr<QuicPacket> packet(
       BuildUnsizedDataPacket(&framer, header, frames));
   char buffer[kMaxPacketSize];
-  size_t encrypted_size = framer.EncryptPayload(
-      ENCRYPTION_NONE, header.packet_number, *packet, buffer, kMaxPacketSize);
+  size_t encrypted_size = framer.EncryptPayload(ENCRYPTION_NONE, /*path_id=*/0u,
+                                                header.packet_number, *packet,
+                                                buffer, kMaxPacketSize);
   EXPECT_NE(0u, encrypted_size);
-  QuicEncryptedPacket encrypted(buffer, encrypted_size, false);
-  return scoped_ptr<QuicEncryptedPacket>(encrypted.Clone());
+  QuicReceivedPacket encrypted(buffer, encrypted_size, clock_->Now(), false);
+  return std::unique_ptr<QuicReceivedPacket>(encrypted.Clone());
 }
 
 // Returns a newly created packet to send kData on stream 1.
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeDataPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeDataPacket(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
     bool should_include_version,
@@ -249,7 +299,34 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeDataPacket(
   return MakePacket(header_, QuicFrame(&frame));
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckAndDataPacket(
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeMultipleDataFramesPacket(
+    QuicPacketNumber packet_number,
+    QuicStreamId stream_id,
+    bool should_include_version,
+    bool fin,
+    QuicStreamOffset offset,
+    const std::vector<std::string>& data_writes) {
+  InitializeHeader(packet_number, should_include_version);
+  QuicFrames data_frames;
+  // QuicFrame takes a raw pointer. Use a std::vector here so we keep
+  // StreamFrames alive until MakeMultipleFramesPacket is done.
+  std::vector<std::unique_ptr<QuicStreamFrame>> stream_frames;
+  for (size_t i = 0; i < data_writes.size(); ++i) {
+    bool is_fin = fin && (i == data_writes.size() - 1);
+    stream_frames.push_back(base::WrapUnique(new QuicStreamFrame(
+        stream_id, is_fin, offset, base::StringPiece(data_writes[i]))));
+    offset += data_writes[i].length();
+  }
+  for (const auto& stream_frame : stream_frames) {
+    QuicFrame quic_frame(stream_frame.get());
+    DVLOG(1) << "Adding frame: " << quic_frame;
+    data_frames.push_back(quic_frame);
+  }
+  return MakeMultipleFramesPacket(header_, data_frames);
+}
+
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakeAckAndDataPacket(
     QuicPacketNumber packet_number,
     bool include_version,
     QuicStreamId stream_id,
@@ -261,10 +338,14 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckAndDataPacket(
   InitializeHeader(packet_number, include_version);
 
   QuicAckFrame ack(MakeAckFrame(largest_received));
-  ack.delta_time_largest_observed = QuicTime::Delta::Zero();
+  ack.ack_delay_time = QuicTime::Delta::Zero();
   for (QuicPacketNumber i = least_unacked; i <= largest_received; ++i) {
     ack.received_packet_times.push_back(make_pair(i, clock_->Now()));
   }
+  if (largest_received > 0 && version_ > QUIC_VERSION_33) {
+    ack.missing = false;
+    ack.packets.Add(1, largest_received + 1);
+  }
   QuicFrames frames;
   frames.push_back(QuicFrame(&ack));
 
@@ -278,59 +359,110 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeAckAndDataPacket(
   return MakeMultipleFramesPacket(header_, frames);
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeRequestHeadersPacket(
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeRequestHeadersAndMultipleDataFramesPacket(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
     bool should_include_version,
     bool fin,
     SpdyPriority priority,
-    const SpdyHeaderBlock& headers,
+    SpdyHeaderBlock headers,
+    size_t* spdy_headers_frame_length,
+    const std::vector<std::string>& data_writes) {
+  InitializeHeader(packet_number, should_include_version);
+  SpdySerializedFrame spdy_frame;
+  if (spdy_request_framer_.protocol_version() == SPDY3) {
+    SpdySynStreamIR syn_stream(stream_id, std::move(headers));
+    syn_stream.set_fin(fin);
+    syn_stream.set_priority(priority);
+    spdy_frame = spdy_request_framer_.SerializeSynStream(syn_stream);
+  } else {
+    SpdyHeadersIR headers_frame(stream_id, std::move(headers));
+    headers_frame.set_fin(fin);
+    headers_frame.set_weight(Spdy3PriorityToHttp2Weight(priority));
+    headers_frame.set_has_priority(true);
+    spdy_frame = spdy_request_framer_.SerializeFrame(headers_frame);
+  }
+  if (spdy_headers_frame_length) {
+    *spdy_headers_frame_length = spdy_frame.size();
+  }
+  QuicStreamFrame frame(
+      kHeadersStreamId, false, 0,
+      base::StringPiece(spdy_frame.data(), spdy_frame.size()));
+
+  QuicFrames frames;
+  frames.push_back(QuicFrame(&frame));
+
+  QuicStreamOffset offset = 0;
+  // QuicFrame takes a raw pointer. Use a std::vector here so we keep
+  // StreamFrames alive until MakeMultipleFramesPacket is done.
+  std::vector<std::unique_ptr<QuicStreamFrame>> stream_frames;
+  for (size_t i = 0; i < data_writes.size(); ++i) {
+    bool is_fin = fin && (i == data_writes.size() - 1);
+    stream_frames.push_back(base::WrapUnique(new QuicStreamFrame(
+        stream_id, is_fin, offset, base::StringPiece(data_writes[i]))));
+    offset += data_writes[i].length();
+  }
+  for (const auto& stream_frame : stream_frames) {
+    QuicFrame quic_frame(stream_frame.get());
+    DVLOG(1) << "Adding frame: " << quic_frame;
+    frames.push_back(quic_frame);
+  }
+  return MakeMultipleFramesPacket(header_, frames);
+}
+
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeRequestHeadersPacket(
+    QuicPacketNumber packet_number,
+    QuicStreamId stream_id,
+    bool should_include_version,
+    bool fin,
+    SpdyPriority priority,
+    SpdyHeaderBlock headers,
     size_t* spdy_headers_frame_length) {
-  return MakeRequestHeadersPacket(packet_number, stream_id,
-                                  should_include_version, fin, priority,
-                                  headers, spdy_headers_frame_length, nullptr);
+  return MakeRequestHeadersPacket(
+      packet_number, stream_id, should_include_version, fin, priority,
+      std::move(headers), spdy_headers_frame_length, nullptr);
 }
 
 // If |offset| is provided, will use the value when creating the packet.
 // Will also update the value after packet creation.
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeRequestHeadersPacket(
-    QuicPacketNumber packet_number,
-    QuicStreamId stream_id,
-    bool should_include_version,
-    bool fin,
-    SpdyPriority priority,
-    const SpdyHeaderBlock& headers,
-    size_t* spdy_headers_frame_length,
-    QuicStreamOffset* offset) {
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeRequestHeadersPacket(QuicPacketNumber packet_number,
+                                              QuicStreamId stream_id,
+                                              bool should_include_version,
+                                              bool fin,
+                                              SpdyPriority priority,
+                                              SpdyHeaderBlock headers,
+                                              size_t* spdy_headers_frame_length,
+                                              QuicStreamOffset* offset) {
   InitializeHeader(packet_number, should_include_version);
-  scoped_ptr<SpdySerializedFrame> spdy_frame;
+  SpdySerializedFrame spdy_frame;
   if (spdy_request_framer_.protocol_version() == SPDY3) {
-    SpdySynStreamIR syn_stream(stream_id);
-    syn_stream.set_header_block(headers);
+    SpdySynStreamIR syn_stream(stream_id, std::move(headers));
     syn_stream.set_fin(fin);
     syn_stream.set_priority(priority);
-    spdy_frame.reset(spdy_request_framer_.SerializeSynStream(syn_stream));
+    spdy_frame = spdy_request_framer_.SerializeSynStream(syn_stream);
   } else {
-    SpdyHeadersIR headers_frame(stream_id);
-    headers_frame.set_header_block(headers);
+    SpdyHeadersIR headers_frame(stream_id, std::move(headers));
     headers_frame.set_fin(fin);
-    headers_frame.set_priority(priority);
+    headers_frame.set_weight(Spdy3PriorityToHttp2Weight(priority));
     headers_frame.set_has_priority(true);
-    spdy_frame.reset(spdy_request_framer_.SerializeFrame(headers_frame));
+    spdy_frame = spdy_request_framer_.SerializeFrame(headers_frame);
   }
   if (spdy_headers_frame_length) {
-    *spdy_headers_frame_length = spdy_frame->size();
+    *spdy_headers_frame_length = spdy_frame.size();
   }
   if (offset != nullptr) {
     QuicStreamFrame frame(
         kHeadersStreamId, false, *offset,
-        base::StringPiece(spdy_frame->data(), spdy_frame->size()));
-    *offset += spdy_frame->size();
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
+    *offset += spdy_frame.size();
     return MakePacket(header_, QuicFrame(&frame));
   } else {
     QuicStreamFrame frame(
         kHeadersStreamId, false, 0,
-        base::StringPiece(spdy_frame->data(), spdy_frame->size()));
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
 
     return MakePacket(header_, QuicFrame(&frame));
   }
@@ -338,85 +470,119 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeRequestHeadersPacket(
 
 // Convenience method for calling MakeRequestHeadersPacket with nullptr for
 // |spdy_headers_frame_length|.
-scoped_ptr<QuicEncryptedPacket>
+std::unique_ptr<QuicReceivedPacket>
 QuicTestPacketMaker::MakeRequestHeadersPacketWithOffsetTracking(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
     bool should_include_version,
     bool fin,
     SpdyPriority priority,
-    const SpdyHeaderBlock& headers,
+    SpdyHeaderBlock headers,
     QuicStreamOffset* offset) {
   return MakeRequestHeadersPacket(packet_number, stream_id,
                                   should_include_version, fin, priority,
-                                  headers, nullptr, offset);
+                                  std::move(headers), nullptr, offset);
 }
 
 // If |offset| is provided, will use the value when creating the packet.
 // Will also update the value after packet creation.
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeResponseHeadersPacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakePushPromisePacket(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
+    QuicStreamId promised_stream_id,
     bool should_include_version,
     bool fin,
-    const SpdyHeaderBlock& headers,
+    SpdyHeaderBlock headers,
     size_t* spdy_headers_frame_length,
     QuicStreamOffset* offset) {
   InitializeHeader(packet_number, should_include_version);
-  scoped_ptr<SpdySerializedFrame> spdy_frame;
-  if (spdy_response_framer_.protocol_version() == SPDY3) {
-    SpdySynReplyIR syn_reply(stream_id);
-    syn_reply.set_header_block(headers);
-    syn_reply.set_fin(fin);
-    spdy_frame.reset(spdy_response_framer_.SerializeSynReply(syn_reply));
-  } else {
-    SpdyHeadersIR headers_frame(stream_id);
-    headers_frame.set_header_block(headers);
-    headers_frame.set_fin(fin);
-    spdy_frame.reset(spdy_response_framer_.SerializeFrame(headers_frame));
-  }
+  SpdySerializedFrame spdy_frame;
+  SpdyPushPromiseIR promise_frame(stream_id, promised_stream_id,
+                                  std::move(headers));
+  promise_frame.set_fin(fin);
+  spdy_frame = spdy_request_framer_.SerializeFrame(promise_frame);
   if (spdy_headers_frame_length) {
-    *spdy_headers_frame_length = spdy_frame->size();
+    *spdy_headers_frame_length = spdy_frame.size();
   }
   if (offset != nullptr) {
     QuicStreamFrame frame(
         kHeadersStreamId, false, *offset,
-        base::StringPiece(spdy_frame->data(), spdy_frame->size()));
-    *offset += spdy_frame->size();
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
+    *offset += spdy_frame.size();
     return MakePacket(header_, QuicFrame(&frame));
   } else {
     QuicStreamFrame frame(
         kHeadersStreamId, false, 0,
-        base::StringPiece(spdy_frame->data(), spdy_frame->size()));
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
     return MakePacket(header_, QuicFrame(&frame));
   }
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeResponseHeadersPacket(
+// If |offset| is provided, will use the value when creating the packet.
+// Will also update the value after packet creation.
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeResponseHeadersPacket(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
     bool should_include_version,
     bool fin,
-    const SpdyHeaderBlock& headers,
+    SpdyHeaderBlock headers,
+    size_t* spdy_headers_frame_length,
+    QuicStreamOffset* offset) {
+  InitializeHeader(packet_number, should_include_version);
+  SpdySerializedFrame spdy_frame;
+  if (spdy_response_framer_.protocol_version() == SPDY3) {
+    SpdySynReplyIR syn_reply(stream_id, std::move(headers));
+    syn_reply.set_fin(fin);
+    spdy_frame = spdy_response_framer_.SerializeSynReply(syn_reply);
+  } else {
+    SpdyHeadersIR headers_frame(stream_id, std::move(headers));
+    headers_frame.set_fin(fin);
+    spdy_frame = spdy_response_framer_.SerializeFrame(headers_frame);
+  }
+  if (spdy_headers_frame_length) {
+    *spdy_headers_frame_length = spdy_frame.size();
+  }
+  if (offset != nullptr) {
+    QuicStreamFrame frame(
+        kHeadersStreamId, false, *offset,
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
+    *offset += spdy_frame.size();
+    return MakePacket(header_, QuicFrame(&frame));
+  } else {
+    QuicStreamFrame frame(
+        kHeadersStreamId, false, 0,
+        base::StringPiece(spdy_frame.data(), spdy_frame.size()));
+    return MakePacket(header_, QuicFrame(&frame));
+  }
+}
+
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeResponseHeadersPacket(
+    QuicPacketNumber packet_number,
+    QuicStreamId stream_id,
+    bool should_include_version,
+    bool fin,
+    SpdyHeaderBlock headers,
     size_t* spdy_headers_frame_length) {
-  return MakeResponseHeadersPacket(packet_number, stream_id,
-                                   should_include_version, fin, headers,
-                                   spdy_headers_frame_length, nullptr);
+  return MakeResponseHeadersPacket(
+      packet_number, stream_id, should_include_version, fin, std::move(headers),
+      spdy_headers_frame_length, nullptr);
 }
 
 // Convenience method for calling MakeResponseHeadersPacket with nullptr for
 // |spdy_headers_frame_length|.
-scoped_ptr<QuicEncryptedPacket>
+std::unique_ptr<QuicReceivedPacket>
 QuicTestPacketMaker::MakeResponseHeadersPacketWithOffsetTracking(
     QuicPacketNumber packet_number,
     QuicStreamId stream_id,
     bool should_include_version,
     bool fin,
-    const SpdyHeaderBlock& headers,
+    SpdyHeaderBlock headers,
     QuicStreamOffset* offset) {
   return MakeResponseHeadersPacket(packet_number, stream_id,
-                                   should_include_version, fin, headers,
-                                   nullptr, offset);
+                                   should_include_version, fin,
+                                   std::move(headers), nullptr, offset);
 }
 
 SpdyHeaderBlock QuicTestPacketMaker::GetRequestHeaders(
@@ -449,7 +615,7 @@ SpdyHeaderBlock QuicTestPacketMaker::GetResponseHeaders(
   return headers;
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakePacket(
+std::unique_ptr<QuicReceivedPacket> QuicTestPacketMaker::MakePacket(
     const QuicPacketHeader& header,
     const QuicFrame& frame) {
   QuicFrames frames;
@@ -457,19 +623,19 @@ scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakePacket(
   return MakeMultipleFramesPacket(header, frames);
 }
 
-scoped_ptr<QuicEncryptedPacket> QuicTestPacketMaker::MakeMultipleFramesPacket(
-    const QuicPacketHeader& header,
-    const QuicFrames& frames) {
-  QuicFramer framer(SupportedVersions(version_), QuicTime::Zero(),
-                    Perspective::IS_CLIENT);
-  scoped_ptr<QuicPacket> packet(
+std::unique_ptr<QuicReceivedPacket>
+QuicTestPacketMaker::MakeMultipleFramesPacket(const QuicPacketHeader& header,
+                                              const QuicFrames& frames) {
+  QuicFramer framer(SupportedVersions(version_), clock_->Now(), perspective_);
+  std::unique_ptr<QuicPacket> packet(
       BuildUnsizedDataPacket(&framer, header, frames));
   char buffer[kMaxPacketSize];
-  size_t encrypted_size = framer.EncryptPayload(
-      ENCRYPTION_NONE, header.packet_number, *packet, buffer, kMaxPacketSize);
+  size_t encrypted_size = framer.EncryptPayload(ENCRYPTION_NONE, /*path_id=*/0u,
+                                                header.packet_number, *packet,
+                                                buffer, kMaxPacketSize);
   EXPECT_NE(0u, encrypted_size);
-  QuicEncryptedPacket encrypted(buffer, encrypted_size, false);
-  return scoped_ptr<QuicEncryptedPacket>(encrypted.Clone());
+  QuicReceivedPacket encrypted(buffer, encrypted_size, clock_->Now(), false);
+  return std::unique_ptr<QuicReceivedPacket>(encrypted.Clone());
 }
 
 void QuicTestPacketMaker::InitializeHeader(QuicPacketNumber packet_number,
@@ -479,7 +645,6 @@ void QuicTestPacketMaker::InitializeHeader(QuicPacketNumber packet_number,
   header_.public_header.version_flag = should_include_version;
   header_.public_header.packet_number_length = PACKET_1BYTE_PACKET_NUMBER;
   header_.packet_number = packet_number;
-  header_.fec_group = 0;
   header_.entropy_flag = false;
   header_.fec_flag = false;
 }
