@@ -4,9 +4,13 @@
 
 #include "net/tools/quic/quic_spdy_client_stream.h"
 
+#include <utility>
+
 #include "base/logging.h"
 #include "base/stl_util.h"
 #include "base/strings/string_number_conversions.h"
+#include "net/quic/quic_alarm.h"
+#include "net/quic/quic_client_promised_info.h"
 #include "net/quic/spdy_utils.h"
 #include "net/spdy/spdy_protocol.h"
 #include "net/tools/quic/quic_client_session.h"
@@ -17,7 +21,6 @@ using std::string;
 using base::StringToInt;
 
 namespace net {
-namespace tools {
 
 QuicSpdyClientStream::QuicSpdyClientStream(QuicStreamId id,
                                            QuicClientSession* session)
@@ -26,7 +29,8 @@ QuicSpdyClientStream::QuicSpdyClientStream(QuicStreamId id,
       response_code_(0),
       header_bytes_read_(0),
       header_bytes_written_(0),
-      allow_bidirectional_data_(false) {}
+      allow_bidirectional_data_(false),
+      session_(session) {}
 
 QuicSpdyClientStream::~QuicSpdyClientStream() {}
 
@@ -44,49 +48,114 @@ void QuicSpdyClientStream::OnInitialHeadersComplete(bool fin,
   QuicSpdyStream::OnInitialHeadersComplete(fin, frame_len);
 
   DCHECK(headers_decompressed());
-  header_bytes_read_ = frame_len;
+  header_bytes_read_ += frame_len;
   if (!SpdyUtils::ParseHeaders(decompressed_headers().data(),
                                decompressed_headers().length(),
                                &content_length_, &response_headers_)) {
+    DLOG(ERROR) << "Failed to parse headers: " << decompressed_headers();
     Reset(QUIC_BAD_APPLICATION_PAYLOAD);
     return;
   }
 
-  string status = response_headers_[":status"].as_string();
-  size_t end = status.find(" ");
-  if (end != string::npos) {
-    status.erase(end);
-  }
-  if (!StringToInt(status, &response_code_)) {
-    // Invalid response code.
+  if (!ParseHeaderStatusCode(response_headers_, &response_code_)) {
+    DLOG(ERROR) << "Received invalid response code: "
+                << response_headers_[":status"].as_string();
     Reset(QUIC_BAD_APPLICATION_PAYLOAD);
     return;
   }
 
   MarkHeadersConsumed(decompressed_headers().length());
+  DVLOG(1) << "headers complete for stream " << id();
+
+  session_->OnInitialHeadersComplete(id(), response_headers_);
 }
 
-void QuicSpdyClientStream::OnTrailingHeadersComplete(bool fin,
-                                                     size_t frame_len) {
-  QuicSpdyStream::OnTrailingHeadersComplete(fin, frame_len);
+void QuicSpdyClientStream::OnInitialHeadersComplete(
+    bool fin,
+    size_t frame_len,
+    const QuicHeaderList& header_list) {
+  QuicSpdyStream::OnInitialHeadersComplete(fin, frame_len, header_list);
 
-  size_t final_byte_offset = 0;
-  if (!SpdyUtils::ParseTrailers(decompressed_trailers().data(),
-                                decompressed_trailers().length(),
-                                &final_byte_offset, &response_trailers_)) {
+  DCHECK(headers_decompressed());
+  header_bytes_read_ += frame_len;
+  if (!SpdyUtils::CopyAndValidateHeaders(header_list, &content_length_,
+                                         &response_headers_)) {
+    DLOG(ERROR) << "Failed to parse header list: " << header_list.DebugString();
     Reset(QUIC_BAD_APPLICATION_PAYLOAD);
     return;
   }
-  MarkTrailersConsumed(decompressed_trailers().length());
 
-  // The data on this stream ends at |final_byte_offset|.
-  DVLOG(1) << "Stream ends at byte offset: " << final_byte_offset
-           << "  currently read: " << stream_bytes_read();
-  OnStreamFrame(
-      QuicStreamFrame(id(), /*fin=*/true, final_byte_offset, StringPiece()));
+  if (!ParseHeaderStatusCode(response_headers_, &response_code_)) {
+    DLOG(ERROR) << "Received invalid response code: "
+                << response_headers_[":status"].as_string();
+    Reset(QUIC_BAD_APPLICATION_PAYLOAD);
+    return;
+  }
+
+  ConsumeHeaderList();
+  DVLOG(1) << "headers complete for stream " << id();
+
+  session_->OnInitialHeadersComplete(id(), response_headers_);
+}
+
+void QuicSpdyClientStream::OnTrailingHeadersComplete(
+    bool fin,
+    size_t frame_len,
+    const QuicHeaderList& header_list) {
+  QuicSpdyStream::OnTrailingHeadersComplete(fin, frame_len, header_list);
+  MarkTrailersConsumed(decompressed_trailers().length());
+}
+
+void QuicSpdyClientStream::OnPromiseHeadersComplete(QuicStreamId promised_id,
+                                                    size_t frame_len) {
+  header_bytes_read_ += frame_len;
+  int64_t content_length = -1;
+  SpdyHeaderBlock promise_headers;
+  if (!SpdyUtils::ParseHeaders(decompressed_headers().data(),
+                               decompressed_headers().length(), &content_length,
+                               &promise_headers)) {
+    DLOG(ERROR) << "Failed to parse promise headers: "
+                << decompressed_headers();
+    Reset(QUIC_BAD_APPLICATION_PAYLOAD);
+    return;
+  }
+  MarkHeadersConsumed(decompressed_headers().length());
+
+  session_->HandlePromised(id(), promised_id, promise_headers);
+  if (visitor() != nullptr) {
+    visitor()->OnPromiseHeadersComplete(promised_id, frame_len);
+  }
+}
+
+void QuicSpdyClientStream::OnPromiseHeaderList(
+    QuicStreamId promised_id,
+    size_t frame_len,
+    const QuicHeaderList& header_list) {
+  header_bytes_read_ += frame_len;
+  int64_t content_length = -1;
+  SpdyHeaderBlock promise_headers;
+  if (!SpdyUtils::CopyAndValidateHeaders(header_list, &content_length,
+                                         &promise_headers)) {
+    DLOG(ERROR) << "Failed to parse promise headers: "
+                << header_list.DebugString();
+    Reset(QUIC_BAD_APPLICATION_PAYLOAD);
+    return;
+  }
+
+  session_->HandlePromised(id(), promised_id, promise_headers);
+  if (visitor() != nullptr) {
+    visitor()->OnPromiseHeadersComplete(promised_id, frame_len);
+  }
 }
 
 void QuicSpdyClientStream::OnDataAvailable() {
+  if (FLAGS_quic_supports_push_promise) {
+    // For push streams, visitor will not be set until the rendezvous
+    // between server promise and client request is complete.
+    if (visitor() == nullptr)
+      return;
+  }
+
   while (HasBytesToRead()) {
     struct iovec iov;
     if (GetReadableRegions(&iov, 1) == 0) {
@@ -98,7 +167,9 @@ void QuicSpdyClientStream::OnDataAvailable() {
     data_.append(static_cast<char*>(iov.iov_base), iov.iov_len);
 
     if (content_length_ >= 0 &&
-        static_cast<int>(data_.size()) > content_length_) {
+        data_.size() > static_cast<uint64_t>(content_length_)) {
+      DLOG(ERROR) << "Invalid content length (" << content_length_
+                  << ") with data of size " << data_.size();
       Reset(QUIC_BAD_APPLICATION_PAYLOAD);
       return;
     }
@@ -111,12 +182,13 @@ void QuicSpdyClientStream::OnDataAvailable() {
   }
 }
 
-size_t QuicSpdyClientStream::SendRequest(const SpdyHeaderBlock& headers,
+size_t QuicSpdyClientStream::SendRequest(SpdyHeaderBlock headers,
                                          StringPiece body,
                                          bool fin) {
   bool send_fin_with_headers = fin && body.empty();
   size_t bytes_sent = body.size();
-  header_bytes_written_ = WriteHeaders(headers, send_fin_with_headers, nullptr);
+  header_bytes_written_ =
+      WriteHeaders(std::move(headers), send_fin_with_headers, nullptr);
   bytes_sent += header_bytes_written_;
 
   if (!body.empty()) {
@@ -126,15 +198,4 @@ size_t QuicSpdyClientStream::SendRequest(const SpdyHeaderBlock& headers,
   return bytes_sent;
 }
 
-void QuicSpdyClientStream::SendBody(const string& data, bool fin) {
-  SendBody(data, fin, nullptr);
-}
-
-void QuicSpdyClientStream::SendBody(const string& data,
-                                    bool fin,
-                                    QuicAckListenerInterface* listener) {
-  WriteOrBufferData(data, fin, listener);
-}
-
-}  // namespace tools
 }  // namespace net
