@@ -4,6 +4,8 @@
 
 #include "net/url_request/url_request_job.h"
 
+#include <utility>
+
 #include "base/bind.h"
 #include "base/compiler_specific.h"
 #include "base/location.h"
@@ -12,8 +14,9 @@
 #include "base/profiler/scoped_tracker.h"
 #include "base/single_thread_task_runner.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/values.h"
 #include "net/base/auth.h"
 #include "net/base/host_port_pair.h"
@@ -22,9 +25,9 @@
 #include "net/base/load_states.h"
 #include "net/base/net_errors.h"
 #include "net/base/network_delegate.h"
-#include "net/base/network_quality_estimator.h"
 #include "net/filter/filter.h"
 #include "net/http/http_response_headers.h"
+#include "net/nqe/network_quality_estimator.h"
 #include "net/url_request/url_request_context.h"
 
 namespace net {
@@ -32,12 +35,13 @@ namespace net {
 namespace {
 
 // Callback for TYPE_URL_REQUEST_FILTERS_SET net-internals event.
-scoped_ptr<base::Value> FiltersSetCallback(
+std::unique_ptr<base::Value> FiltersSetCallback(
     Filter* filter,
     NetLogCaptureMode /* capture_mode */) {
-  scoped_ptr<base::DictionaryValue> event_params(new base::DictionaryValue());
+  std::unique_ptr<base::DictionaryValue> event_params(
+      new base::DictionaryValue());
   event_params->SetString("filters", filter->OrderedFilterList());
-  return event_params.Pass();
+  return std::move(event_params);
 }
 
 std::string ComputeMethodForRedirect(const std::string& method,
@@ -59,6 +63,56 @@ std::string ComputeMethodForRedirect(const std::string& method,
   return method;
 }
 
+// A redirect response can contain a Referrer-Policy header
+// (https://w3c.github.io/webappsec-referrer-policy/). This function
+// checks for a Referrer-Policy header, and parses it if
+// present. Returns the referrer policy that should be used for the
+// request.
+URLRequest::ReferrerPolicy ProcessReferrerPolicyHeaderOnRedirect(
+    URLRequest* request) {
+  URLRequest::ReferrerPolicy new_policy = request->referrer_policy();
+
+  std::string referrer_policy_header;
+  request->GetResponseHeaderByName("Referrer-Policy", &referrer_policy_header);
+  std::vector<std::string> policy_tokens =
+      base::SplitString(referrer_policy_header, ",", base::TRIM_WHITESPACE,
+                        base::SPLIT_WANT_NONEMPTY);
+
+  for (const auto& token : policy_tokens) {
+    if (base::CompareCaseInsensitiveASCII(token, "never") == 0 ||
+        base::CompareCaseInsensitiveASCII(token, "no-referrer") == 0) {
+      new_policy = URLRequest::NO_REFERRER;
+      continue;
+    }
+
+    if (base::CompareCaseInsensitiveASCII(token, "default") == 0 ||
+        base::CompareCaseInsensitiveASCII(token,
+                                          "no-referrer-when-downgrade") == 0) {
+      new_policy =
+          URLRequest::CLEAR_REFERRER_ON_TRANSITION_FROM_SECURE_TO_INSECURE;
+      continue;
+    }
+
+    if (base::CompareCaseInsensitiveASCII(token, "origin") == 0) {
+      new_policy = URLRequest::ORIGIN;
+      continue;
+    }
+
+    if (base::CompareCaseInsensitiveASCII(token, "origin-when-cross-origin") ==
+        0) {
+      new_policy = URLRequest::ORIGIN_ONLY_ON_TRANSITION_CROSS_ORIGIN;
+      continue;
+    }
+
+    if (base::CompareCaseInsensitiveASCII(token, "always") == 0 ||
+        base::CompareCaseInsensitiveASCII(token, "unsafe-url") == 0) {
+      new_policy = URLRequest::NEVER_CLEAR_REFERRER;
+      continue;
+    }
+  }
+  return new_policy;
+}
+
 }  // namespace
 
 URLRequestJob::URLRequestJob(URLRequest* request,
@@ -72,12 +126,20 @@ URLRequestJob::URLRequestJob(URLRequest* request,
       has_handled_response_(false),
       expected_content_size_(-1),
       network_delegate_(network_delegate),
+      last_notified_total_received_bytes_(0),
+      last_notified_total_sent_bytes_(0),
       weak_factory_(this) {
   base::PowerMonitor* power_monitor = base::PowerMonitor::Get();
   if (power_monitor)
     power_monitor->AddObserver(this);
 }
 
+URLRequestJob::~URLRequestJob() {
+  base::PowerMonitor* power_monitor = base::PowerMonitor::Get();
+  if (power_monitor)
+    power_monitor->RemoveObserver(this);
+}
+
 void URLRequestJob::SetUpload(UploadDataStream* upload) {
 }
 
@@ -89,55 +151,55 @@ void URLRequestJob::SetPriority(RequestPriority priority) {
 
 void URLRequestJob::Kill() {
   weak_factory_.InvalidateWeakPtrs();
-  // Make sure the request is notified that we are done.  We assume that the
-  // request took care of setting its error status before calling Kill.
-  if (request_)
-    NotifyCanceled();
+  // Make sure the URLRequest is notified that the job is done.  This assumes
+  // that the URLRequest took care of setting its error status before calling
+  // Kill().
+  // TODO(mmenke):  The URLRequest is currently deleted before this method
+  // invokes its async callback whenever this is called by the URLRequest.
+  // Try to simplify how cancellation works.
+  NotifyCanceled();
 }
 
-void URLRequestJob::DetachRequest() {
-  request_ = NULL;
-}
-
-// This function calls ReadData to get stream data. If a filter exists, passes
-// the data to the attached filter. Then returns the output from filter back to
-// the caller.
+// This function calls ReadRawData to get stream data. If a filter exists, it
+// passes the data to the attached filter. It then returns the output from
+// filter back to the caller.
 bool URLRequestJob::Read(IOBuffer* buf, int buf_size, int *bytes_read) {
-  bool rv = false;
-
   DCHECK_LT(buf_size, 1000000);  // Sanity check.
   DCHECK(buf);
   DCHECK(bytes_read);
-  DCHECK(filtered_read_buffer_.get() == NULL);
+  DCHECK(!filtered_read_buffer_);
   DCHECK_EQ(0, filtered_read_buffer_len_);
 
+  Error error = OK;
   *bytes_read = 0;
 
   // Skip Filter if not present.
-  if (!filter_.get()) {
-    rv = ReadRawDataHelper(buf, buf_size, bytes_read);
+  if (!filter_) {
+    error = ReadRawDataHelper(buf, buf_size, bytes_read);
   } else {
     // Save the caller's buffers while we do IO
     // in the filter's buffers.
     filtered_read_buffer_ = buf;
     filtered_read_buffer_len_ = buf_size;
 
-    if (ReadFilteredData(bytes_read)) {
-      rv = true;   // We have data to return.
+    error = ReadFilteredData(bytes_read);
 
-      // It is fine to call DoneReading even if ReadFilteredData receives 0
-      // bytes from the net, but we avoid making that call if we know for
-      // sure that's the case (ReadRawDataHelper path).
-      if (*bytes_read == 0)
-        DoneReading();
-    } else {
-      rv = false;  // Error, or a new IO is pending.
-    }
+    // Synchronous EOF from the filter.
+    if (error == OK && *bytes_read == 0)
+      DoneReading();
   }
 
-  if (rv && *bytes_read == 0)
-    NotifyDone(URLRequestStatus());
-  return rv;
+  if (error == OK) {
+    // If URLRequestJob read zero bytes, the job is at EOF.
+    if (*bytes_read == 0)
+      NotifyDone(URLRequestStatus());
+  } else if (error == ERR_IO_PENDING) {
+    SetStatus(URLRequestStatus::FromError(ERR_IO_PENDING));
+  } else {
+    NotifyDone(URLRequestStatus::FromError(error));
+    *bytes_read = -1;
+  }
+  return error == OK;
 }
 
 void URLRequestJob::StopCaching() {
@@ -149,7 +211,11 @@ bool URLRequestJob::GetFullRequestHeaders(HttpRequestHeaders* headers) const {
   return false;
 }
 
-int64 URLRequestJob::GetTotalReceivedBytes() const {
+int64_t URLRequestJob::GetTotalReceivedBytes() const {
+  return 0;
+}
+
+int64_t URLRequestJob::GetTotalSentBytes() const {
   return 0;
 }
 
@@ -172,12 +238,16 @@ void URLRequestJob::GetLoadTimingInfo(LoadTimingInfo* load_timing_info) const {
   // Only certain request types return more than just request start times.
 }
 
-bool URLRequestJob::GetResponseCookies(std::vector<std::string>* cookies) {
+bool URLRequestJob::GetRemoteEndpoint(IPEndPoint* endpoint) const {
   return false;
 }
 
-Filter* URLRequestJob::SetupFilter() const {
-  return NULL;
+void URLRequestJob::PopulateNetErrorDetails(NetErrorDetails* details) const {
+  return;
+}
+
+std::unique_ptr<Filter> URLRequestJob::SetupFilter() const {
+  return nullptr;
 }
 
 bool URLRequestJob::IsRedirectResponse(GURL* location,
@@ -227,8 +297,8 @@ void URLRequestJob::CancelAuth() {
   NOTREACHED();
 }
 
-void URLRequestJob::ContinueWithCertificate(
-    X509Certificate* client_cert) {
+void URLRequestJob::ContinueWithCertificate(X509Certificate* client_cert,
+                                            SSLPrivateKey* client_private_key) {
   // The derived class should implement this!
   NOTREACHED();
 }
@@ -247,8 +317,8 @@ void URLRequestJob::FollowDeferredRedirect() {
   // will fail inside FollowRedirect.  The DCHECK above asserts that we called
   // OnReceivedRedirect.
 
-  // It is also possible that FollowRedirect will drop the last reference to
-  // this job, so we need to reset our members before calling it.
+  // It is also possible that FollowRedirect will delete |this|, so not safe to
+  // pass along reference to |deferred_redirect_info_|.
 
   RedirectInfo redirect_info = deferred_redirect_info_;
   deferred_redirect_info_ = RedirectInfo();
@@ -274,7 +344,18 @@ HostPortPair URLRequestJob::GetSocketAddress() const {
 }
 
 void URLRequestJob::OnSuspend() {
-  Kill();
+  // Most errors generated by the Job come as the result of the one current
+  // operation the job is waiting on returning an error. This event is unusual
+  // in that the Job may have another operation ongoing, or the Job may be idle
+  // and waiting on the next call.
+  //
+  // Need to cancel through the request to make sure everything is notified
+  // of the failure (Particularly that the NetworkDelegate, which the Job may be
+  // waiting on, is notified synchronously) and torn down correctly.
+  //
+  // TODO(mmenke): This should probably fail the request with
+  //               NETWORK_IO_SUSPENDED instead.
+  request_->Cancel();
 }
 
 void URLRequestJob::NotifyURLRequestDestroyed() {
@@ -314,71 +395,55 @@ GURL URLRequestJob::ComputeReferrerForRedirect(
 
     case URLRequest::NEVER_CLEAR_REFERRER:
       return original_referrer;
+    case URLRequest::ORIGIN:
+      return original_referrer.GetOrigin();
+    case URLRequest::NO_REFERRER:
+      return GURL();
+    case URLRequest::MAX_REFERRER_POLICY:
+      NOTREACHED();
+      return GURL();
   }
 
   NOTREACHED();
   return GURL();
 }
 
-URLRequestJob::~URLRequestJob() {
-  base::PowerMonitor* power_monitor = base::PowerMonitor::Get();
-  if (power_monitor)
-    power_monitor->RemoveObserver(this);
-}
-
 void URLRequestJob::NotifyCertificateRequested(
     SSLCertRequestInfo* cert_request_info) {
-  if (!request_)
-    return;  // The request was destroyed, so there is no more work to do.
-
   request_->NotifyCertificateRequested(cert_request_info);
 }
 
 void URLRequestJob::NotifySSLCertificateError(const SSLInfo& ssl_info,
                                               bool fatal) {
-  if (!request_)
-    return;  // The request was destroyed, so there is no more work to do.
-
   request_->NotifySSLCertificateError(ssl_info, fatal);
 }
 
 bool URLRequestJob::CanGetCookies(const CookieList& cookie_list) const {
-  if (!request_)
-    return false;  // The request was destroyed, so there is no more work to do.
-
   return request_->CanGetCookies(cookie_list);
 }
 
 bool URLRequestJob::CanSetCookie(const std::string& cookie_line,
                                  CookieOptions* options) const {
-  if (!request_)
-    return false;  // The request was destroyed, so there is no more work to do.
-
   return request_->CanSetCookie(cookie_line, options);
 }
 
 bool URLRequestJob::CanEnablePrivacyMode() const {
-  if (!request_)
-    return false;  // The request was destroyed, so there is no more work to do.
-
   return request_->CanEnablePrivacyMode();
 }
 
 void URLRequestJob::NotifyBeforeNetworkStart(bool* defer) {
-  if (!request_)
-    return;
-
   request_->NotifyBeforeNetworkStart(defer);
 }
 
 void URLRequestJob::NotifyHeadersComplete() {
-  if (!request_ || !request_->has_delegate())
-    return;  // The request was destroyed, so there is no more work to do.
-
   if (has_handled_response_)
     return;
 
-  DCHECK(!request_->status().is_io_pending());
+  // The URLRequest status should still be IO_PENDING, which it was set to
+  // before the URLRequestJob was started.  On error or cancellation, this
+  // method should not be called.
+  DCHECK(request_->status().is_io_pending());
+  SetStatus(URLRequestStatus());
 
   // Initialize to the current time, and let the subclass optionally override
   // the time stamps if it has that information.  The default request_time is
@@ -386,43 +451,42 @@ void URLRequestJob::NotifyHeadersComplete() {
   request_->response_info_.response_time = base::Time::Now();
   GetResponseInfo(&request_->response_info_);
 
-  // When notifying the delegate, the delegate can release the request
-  // (and thus release 'this').  After calling to the delgate, we must
-  // check the request pointer to see if it still exists, and return
-  // immediately if it has been destroyed.  self_preservation ensures our
-  // survival until we can get out of this method.
-  scoped_refptr<URLRequestJob> self_preservation(this);
-
-  if (request_)
-    request_->OnHeadersComplete();
+  MaybeNotifyNetworkBytes();
+  request_->OnHeadersComplete();
 
   GURL new_location;
   int http_status_code;
+
   if (IsRedirectResponse(&new_location, &http_status_code)) {
     // Redirect response bodies are not read. Notify the transaction
     // so it does not treat being stopped as an error.
     DoneReadingRedirectResponse();
 
+    // When notifying the URLRequest::Delegate, it can destroy the request,
+    // which will destroy |this|.  After calling to the URLRequest::Delegate,
+    // pointer must be checked to see if |this| still exists, and if not, the
+    // code must return immediately.
+    base::WeakPtr<URLRequestJob> weak_this(weak_factory_.GetWeakPtr());
+
     RedirectInfo redirect_info =
         ComputeRedirectInfo(new_location, http_status_code);
     bool defer_redirect = false;
     request_->NotifyReceivedRedirect(redirect_info, &defer_redirect);
 
-    // Ensure that the request wasn't detached or destroyed in
-    // NotifyReceivedRedirect
-    if (!request_ || !request_->has_delegate())
+    // Ensure that the request wasn't detached, destroyed, or canceled in
+    // NotifyReceivedRedirect.
+    if (!weak_this || !request_->status().is_success())
       return;
 
-    // If we were not cancelled, then maybe follow the redirect.
-    if (request_->status().is_success()) {
-      if (defer_redirect) {
-        deferred_redirect_info_ = redirect_info;
-      } else {
-        FollowRedirect(redirect_info);
-      }
-      return;
+    if (defer_redirect) {
+      deferred_redirect_info_ = redirect_info;
+    } else {
+      FollowRedirect(redirect_info);
     }
-  } else if (NeedsAuth()) {
+    return;
+  }
+
+  if (NeedsAuth()) {
     scoped_refptr<AuthChallengeInfo> auth_info;
     GetAuthChallengeInfo(&auth_info);
 
@@ -437,7 +501,7 @@ void URLRequestJob::NotifyHeadersComplete() {
 
   has_handled_response_ = true;
   if (request_->status().is_success())
-    filter_.reset(SetupFilter());
+    filter_ = SetupFilter();
 
   if (!filter_.get()) {
     std::string content_length;
@@ -451,16 +515,27 @@ void URLRequestJob::NotifyHeadersComplete() {
   }
 
   request_->NotifyResponseStarted();
+
+  // |this| may be destroyed at this point.
 }
 
-void URLRequestJob::NotifyReadComplete(int bytes_read) {
+void URLRequestJob::ConvertResultToError(int result, Error* error, int* count) {
+  if (result >= 0) {
+    *error = OK;
+    *count = result;
+  } else {
+    *error = static_cast<Error>(result);
+    *count = 0;
+  }
+}
+
+void URLRequestJob::ReadRawDataComplete(int result) {
+  DCHECK(request_->status().is_io_pending());
+
   // TODO(cbentzel): Remove ScopedTracker below once crbug.com/475755 is fixed.
   tracked_objects::ScopedTracker tracking_profile(
       FROM_HERE_WITH_EXPLICIT_FUNCTION(
-          "475755 URLRequestJob::NotifyReadComplete"));
-
-  if (!request_ || !request_->has_delegate())
-    return;  // The request was destroyed, so there is no more work to do.
+          "475755 URLRequestJob::RawReadCompleted"));
 
   // TODO(darin): Bug 1004233. Re-enable this test once all of the chrome
   // unit_tests have been fixed to not trip this.
@@ -470,52 +545,80 @@ void URLRequestJob::NotifyReadComplete(int bytes_read) {
   // The headers should be complete before reads complete
   DCHECK(has_handled_response_);
 
-  OnRawReadComplete(bytes_read);
+  Error error;
+  int bytes_read;
+  ConvertResultToError(result, &error, &bytes_read);
 
-  // Don't notify if we had an error.
-  if (!request_->status().is_success())
-    return;
+  DCHECK_NE(ERR_IO_PENDING, error);
 
-  // When notifying the delegate, the delegate can release the request
-  // (and thus release 'this').  After calling to the delegate, we must
-  // check the request pointer to see if it still exists, and return
-  // immediately if it has been destroyed.  self_preservation ensures our
-  // survival until we can get out of this method.
-  scoped_refptr<URLRequestJob> self_preservation(this);
+  GatherRawReadStats(error, bytes_read);
 
-  if (filter_.get()) {
-    // Tell the filter that it has more data
-    FilteredDataRead(bytes_read);
-
-    // Filter the data.
+  if (filter_.get() && error == OK) {
+    // |bytes_read| being 0 indicates an EOF was received.  ReadFilteredData
+    // can incorrectly return ERR_IO_PENDING when 0 bytes are passed to it, so
+    // just don't call into the filter in that case.
     int filter_bytes_read = 0;
-    if (ReadFilteredData(&filter_bytes_read)) {
-      if (!filter_bytes_read)
-        DoneReading();
-      request_->NotifyReadCompleted(filter_bytes_read);
+    if (bytes_read > 0) {
+      // Tell the filter that it has more data.
+      PushInputToFilter(bytes_read);
+
+      // Filter the data.
+      error = ReadFilteredData(&filter_bytes_read);
     }
+
+    if (error == OK && !filter_bytes_read)
+      DoneReading();
+
+    DVLOG(1) << __FUNCTION__ << "() "
+             << "\"" << request_->url().spec() << "\""
+             << " pre bytes read = " << bytes_read
+             << " pre total = " << prefilter_bytes_read_
+             << " post total = " << postfilter_bytes_read_;
+    bytes_read = filter_bytes_read;
   } else {
-    request_->NotifyReadCompleted(bytes_read);
+    DVLOG(1) << __FUNCTION__ << "() "
+             << "\"" << request_->url().spec() << "\""
+             << " pre bytes read = " << bytes_read
+             << " pre total = " << prefilter_bytes_read_
+             << " post total = " << postfilter_bytes_read_;
   }
-  DVLOG(1) << __FUNCTION__ << "() "
-           << "\"" << (request_ ? request_->url().spec() : "???") << "\""
-           << " pre bytes read = " << bytes_read
-           << " pre total = " << prefilter_bytes_read_
-           << " post total = " << postfilter_bytes_read_;
+
+  // Synchronize the URLRequest state machine with the URLRequestJob state
+  // machine. If this read succeeded, either the request is at EOF and the
+  // URLRequest state machine goes to 'finished', or it is not and the
+  // URLRequest state machine goes to 'success'. If the read failed, the
+  // URLRequest state machine goes directly to 'finished'.  If filtered data is
+  // pending, then there's nothing to do, since the status of the request is
+  // already pending.
+  //
+  // Update the URLRequest's status first, so that NotifyReadCompleted has an
+  // accurate view of the request.
+  if (error == OK && bytes_read > 0) {
+    SetStatus(URLRequestStatus());
+  } else if (error != ERR_IO_PENDING) {
+    NotifyDone(URLRequestStatus::FromError(error));
+  }
+
+  // NotifyReadCompleted should be called after SetStatus or NotifyDone updates
+  // the status.
+  if (error == OK)
+    request_->NotifyReadCompleted(bytes_read);
+
+  // |this| may be destroyed at this point.
 }
 
 void URLRequestJob::NotifyStartError(const URLRequestStatus &status) {
   DCHECK(!has_handled_response_);
-  has_handled_response_ = true;
-  if (request_) {
-    // There may be relevant information in the response info even in the
-    // error case.
-    GetResponseInfo(&request_->response_info_);
+  DCHECK(request_->status().is_io_pending());
 
-    request_->set_status(status);
-    request_->NotifyResponseStarted();
-    // We may have been deleted.
-  }
+  has_handled_response_ = true;
+  // There may be relevant information in the response info even in the
+  // error case.
+  GetResponseInfo(&request_->response_info_);
+
+  SetStatus(status);
+  request_->NotifyResponseStarted();
+  // |this| may have been deleted here.
 }
 
 void URLRequestJob::NotifyDone(const URLRequestStatus &status) {
@@ -528,43 +631,41 @@ void URLRequestJob::NotifyDone(const URLRequestStatus &status) {
   // the response before getting here.
   DCHECK(has_handled_response_ || !status.is_success());
 
-  // As with NotifyReadComplete, we need to take care to notice if we were
-  // destroyed during a delegate callback.
-  if (request_) {
-    request_->set_is_pending(false);
-    // With async IO, it's quite possible to have a few outstanding
-    // requests.  We could receive a request to Cancel, followed shortly
-    // by a successful IO.  For tracking the status(), once there is
-    // an error, we do not change the status back to success.  To
-    // enforce this, only set the status if the job is so far
-    // successful.
-    if (request_->status().is_success()) {
-      if (status.status() == URLRequestStatus::FAILED) {
-        request_->net_log().AddEventWithNetErrorCode(NetLog::TYPE_FAILED,
-                                                     status.error());
-      }
-      request_->set_status(status);
+  request_->set_is_pending(false);
+  // With async IO, it's quite possible to have a few outstanding
+  // requests.  We could receive a request to Cancel, followed shortly
+  // by a successful IO.  For tracking the status(), once there is
+  // an error, we do not change the status back to success.  To
+  // enforce this, only set the status if the job is so far
+  // successful.
+  if (request_->status().is_success()) {
+    if (status.status() == URLRequestStatus::FAILED) {
+      request_->net_log().AddEventWithNetErrorCode(NetLog::TYPE_FAILED,
+                                                   status.error());
     }
+    request_->set_status(status);
+  }
 
-    // If the request succeeded (And wasn't cancelled) and the response code was
-    // 4xx or 5xx, record whether or not the main frame was blank.  This is
-    // intended to be a short-lived histogram, used to figure out how important
-    // fixing http://crbug.com/331745 is.
-    if (request_->status().is_success()) {
-      int response_code = GetResponseCode();
-      if (400 <= response_code && response_code <= 599) {
-        bool page_has_content = (postfilter_bytes_read_ != 0);
-        if (request_->load_flags() & net::LOAD_MAIN_FRAME) {
-          UMA_HISTOGRAM_BOOLEAN("Net.ErrorResponseHasContentMainFrame",
-                                page_has_content);
-        } else {
-          UMA_HISTOGRAM_BOOLEAN("Net.ErrorResponseHasContentNonMainFrame",
-                                page_has_content);
-        }
+  // If the request succeeded (And wasn't cancelled) and the response code was
+  // 4xx or 5xx, record whether or not the main frame was blank.  This is
+  // intended to be a short-lived histogram, used to figure out how important
+  // fixing http://crbug.com/331745 is.
+  if (request_->status().is_success()) {
+    int response_code = GetResponseCode();
+    if (400 <= response_code && response_code <= 599) {
+      bool page_has_content = (postfilter_bytes_read_ != 0);
+      if (request_->load_flags() & net::LOAD_MAIN_FRAME) {
+        UMA_HISTOGRAM_BOOLEAN("Net.ErrorResponseHasContentMainFrame",
+                              page_has_content);
+      } else {
+        UMA_HISTOGRAM_BOOLEAN("Net.ErrorResponseHasContentNonMainFrame",
+                              page_has_content);
       }
     }
   }
 
+  MaybeNotifyNetworkBytes();
+
   // Complete this notification later.  This prevents us from re-entering the
   // delegate if we're done because of a synchronous call.
   base::ThreadTaskRunnerHandle::Get()->PostTask(
@@ -574,9 +675,7 @@ void URLRequestJob::NotifyDone(const URLRequestStatus &status) {
 
 void URLRequestJob::CompleteNotifyDone() {
   // Check if we should notify the delegate that we're done because of an error.
-  if (request_ &&
-      !request_->status().is_success() &&
-      request_->has_delegate()) {
+  if (!request_->status().is_success()) {
     // We report the error differently depending on whether we've called
     // OnResponseStarted yet.
     if (has_handled_response_) {
@@ -609,11 +708,8 @@ void URLRequestJob::OnCallToDelegateComplete() {
   request_->OnCallToDelegateComplete();
 }
 
-bool URLRequestJob::ReadRawData(IOBuffer* buf, int buf_size,
-                                int *bytes_read) {
-  DCHECK(bytes_read);
-  *bytes_read = 0;
-  return true;
+int URLRequestJob::ReadRawData(IOBuffer* buf, int buf_size) {
+  return 0;
 }
 
 void URLRequestJob::DoneReading() {
@@ -623,38 +719,34 @@ void URLRequestJob::DoneReading() {
 void URLRequestJob::DoneReadingRedirectResponse() {
 }
 
-void URLRequestJob::FilteredDataRead(int bytes_read) {
+void URLRequestJob::PushInputToFilter(int bytes_read) {
   DCHECK(filter_);
   filter_->FlushStreamBuffer(bytes_read);
 }
 
-bool URLRequestJob::ReadFilteredData(int* bytes_read) {
+Error URLRequestJob::ReadFilteredData(int* bytes_read) {
   DCHECK(filter_);
   DCHECK(filtered_read_buffer_.get());
   DCHECK_GT(filtered_read_buffer_len_, 0);
   DCHECK_LT(filtered_read_buffer_len_, 1000000);  // Sanity check.
-  DCHECK(!raw_read_buffer_.get());
+  DCHECK(!raw_read_buffer_);
 
   *bytes_read = 0;
-  bool rv = false;
+  Error error = ERR_FAILED;
 
   for (;;) {
     if (is_done())
-      return true;
+      return OK;
 
     if (!filter_needs_more_output_space_ && !filter_->stream_data_len()) {
       // We don't have any raw data to work with, so read from the transaction.
       int filtered_data_read;
-      if (ReadRawDataForFilter(&filtered_data_read)) {
-        if (filtered_data_read > 0) {
-          // Give data to filter.
-          filter_->FlushStreamBuffer(filtered_data_read);
-        } else {
-          return true;  // EOF.
-        }
-      } else {
-        return false;  // IO Pending (or error).
-      }
+      error = ReadRawDataForFilter(&filtered_data_read);
+      // If ReadRawDataForFilter returned some data, fall through to the case
+      // below; otherwise, return early.
+      if (error != OK || filtered_data_read == 0)
+        return error;
+      filter_->FlushStreamBuffer(filtered_data_read);
     }
 
     if ((filter_->stream_data_len() || filter_needs_more_output_space_) &&
@@ -680,7 +772,7 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
           filter_needs_more_output_space_ = false;
           *bytes_read = filtered_data_len;
           postfilter_bytes_read_ += filtered_data_len;
-          rv = true;
+          error = OK;
           break;
         }
         case Filter::FILTER_NEED_MORE_DATA: {
@@ -693,7 +785,7 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
           if (filtered_data_len > 0) {
             *bytes_read = filtered_data_len;
             postfilter_bytes_read_ += filtered_data_len;
-            rv = true;
+            error = OK;
           } else {
             // Read again since we haven't received enough data yet (e.g., we
             // may not have a complete gzip header yet).
@@ -704,29 +796,29 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
         case Filter::FILTER_OK: {
           *bytes_read = filtered_data_len;
           postfilter_bytes_read_ += filtered_data_len;
-          rv = true;
+          error = OK;
           break;
         }
         case Filter::FILTER_ERROR: {
           DVLOG(1) << __FUNCTION__ << "() "
-                   << "\"" << (request_ ? request_->url().spec() : "???")
-                   << "\"" << " Filter Error";
+                   << "\"" << request_->url().spec() << "\""
+                   << " Filter Error";
           filter_needs_more_output_space_ = false;
-          NotifyDone(URLRequestStatus(URLRequestStatus::FAILED,
-                     ERR_CONTENT_DECODING_FAILED));
-          rv = false;
+          error = ERR_CONTENT_DECODING_FAILED;
+          UMA_HISTOGRAM_ENUMERATION("Net.ContentDecodingFailed.FilterType",
+                                    filter_->type(), Filter::FILTER_TYPE_MAX);
           break;
         }
         default: {
           NOTREACHED();
           filter_needs_more_output_space_ = false;
-          rv = false;
+          error = ERR_FAILED;
           break;
         }
       }
 
       // If logging all bytes is enabled, log the filtered bytes read.
-      if (rv && request() && filtered_data_len > 0 &&
+      if (error == OK && filtered_data_len > 0 &&
           request()->net_log().IsCapturing()) {
         request()->net_log().AddByteTransferEvent(
             NetLog::TYPE_URL_REQUEST_JOB_FILTERED_BYTES_READ, filtered_data_len,
@@ -734,18 +826,18 @@ bool URLRequestJob::ReadFilteredData(int* bytes_read) {
       }
     } else {
       // we are done, or there is no data left.
-      rv = true;
+      error = OK;
     }
     break;
   }
 
-  if (rv) {
+  if (error == OK) {
     // When we successfully finished a read, we no longer need to save the
     // caller's buffers. Release our reference.
     filtered_read_buffer_ = NULL;
     filtered_read_buffer_len_ = 0;
   }
-  return rv;
+  return error;
 }
 
 void URLRequestJob::DestroyFilters() {
@@ -753,25 +845,25 @@ void URLRequestJob::DestroyFilters() {
 }
 
 const URLRequestStatus URLRequestJob::GetStatus() {
-  if (request_)
-    return request_->status();
-  // If the request is gone, we must be cancelled.
-  return URLRequestStatus(URLRequestStatus::CANCELED,
-                          ERR_ABORTED);
+  return request_->status();
 }
 
 void URLRequestJob::SetStatus(const URLRequestStatus &status) {
-  if (request_)
-    request_->set_status(status);
+  // An error status should never be replaced by a non-error status by a
+  // URLRequestJob.  URLRequest has some retry paths, but it resets the status
+  // itself, if needed.
+  DCHECK(request_->status().is_io_pending() ||
+         request_->status().is_success() ||
+         (!status.is_success() && !status.is_io_pending()));
+  request_->set_status(status);
 }
 
 void URLRequestJob::SetProxyServer(const HostPortPair& proxy_server) {
   request_->proxy_server_ = proxy_server;
 }
 
-bool URLRequestJob::ReadRawDataForFilter(int* bytes_read) {
-  bool rv = false;
-
+Error URLRequestJob::ReadRawDataForFilter(int* bytes_read) {
+  Error error = ERR_FAILED;
   DCHECK(bytes_read);
   DCHECK(filter_.get());
 
@@ -783,29 +875,28 @@ bool URLRequestJob::ReadRawDataForFilter(int* bytes_read) {
   if (!filter_->stream_data_len() && !is_done()) {
     IOBuffer* stream_buffer = filter_->stream_buffer();
     int stream_buffer_size = filter_->stream_buffer_size();
-    rv = ReadRawDataHelper(stream_buffer, stream_buffer_size, bytes_read);
+    error = ReadRawDataHelper(stream_buffer, stream_buffer_size, bytes_read);
   }
-  return rv;
+  return error;
 }
 
-bool URLRequestJob::ReadRawDataHelper(IOBuffer* buf, int buf_size,
-                                      int* bytes_read) {
-  DCHECK(!request_->status().is_io_pending());
-  DCHECK(raw_read_buffer_.get() == NULL);
+Error URLRequestJob::ReadRawDataHelper(IOBuffer* buf,
+                                       int buf_size,
+                                       int* bytes_read) {
+  DCHECK(!raw_read_buffer_);
 
-  // Keep a pointer to the read buffer, so we have access to it in the
-  // OnRawReadComplete() callback in the event that the read completes
-  // asynchronously.
+  // Keep a pointer to the read buffer, so we have access to it in
+  // GatherRawReadStats() in the event that the read completes asynchronously.
   raw_read_buffer_ = buf;
-  bool rv = ReadRawData(buf, buf_size, bytes_read);
+  Error error;
+  ConvertResultToError(ReadRawData(buf, buf_size), &error, bytes_read);
 
-  if (!request_->status().is_io_pending()) {
-    // If the read completes synchronously, either success or failure,
-    // invoke the OnRawReadComplete callback so we can account for the
-    // completed read.
-    OnRawReadComplete(*bytes_read);
+  if (error != ERR_IO_PENDING) {
+    // If the read completes synchronously, either success or failure, invoke
+    // GatherRawReadStats so we can account for the completed read.
+    GatherRawReadStats(error, *bytes_read);
   }
-  return rv;
+  return error;
 }
 
 void URLRequestJob::FollowRedirect(const RedirectInfo& redirect_info) {
@@ -814,44 +905,60 @@ void URLRequestJob::FollowRedirect(const RedirectInfo& redirect_info) {
     NotifyDone(URLRequestStatus(URLRequestStatus::FAILED, rv));
 }
 
-void URLRequestJob::OnRawReadComplete(int bytes_read) {
-  DCHECK(raw_read_buffer_.get());
-  // If |filter_| is non-NULL, bytes will be logged after it is applied instead.
-  if (!filter_.get() && request() && bytes_read > 0 &&
-      request()->net_log().IsCapturing()) {
+void URLRequestJob::GatherRawReadStats(Error error, int bytes_read) {
+  DCHECK(raw_read_buffer_ || bytes_read == 0);
+  DCHECK_NE(ERR_IO_PENDING, error);
+
+  if (error != OK) {
+    raw_read_buffer_ = nullptr;
+    return;
+  }
+  // If |filter_| is non-NULL, bytes will be logged after it is applied
+  // instead.
+  if (!filter_.get() && bytes_read > 0 && request()->net_log().IsCapturing()) {
     request()->net_log().AddByteTransferEvent(
-        NetLog::TYPE_URL_REQUEST_JOB_BYTES_READ,
-        bytes_read, raw_read_buffer_->data());
+        NetLog::TYPE_URL_REQUEST_JOB_BYTES_READ, bytes_read,
+        raw_read_buffer_->data());
   }
 
   if (bytes_read > 0) {
     RecordBytesRead(bytes_read);
   }
-  raw_read_buffer_ = NULL;
+  raw_read_buffer_ = nullptr;
 }
 
 void URLRequestJob::RecordBytesRead(int bytes_read) {
   DCHECK_GT(bytes_read, 0);
   prefilter_bytes_read_ += bytes_read;
 
-  // Notify NetworkQualityEstimator.
+  // On first read, notify NetworkQualityEstimator that response headers have
+  // been received.
   // TODO(tbansal): Move this to url_request_http_job.cc. This may catch
   // Service Worker jobs twice.
-  if (request_ && request_->context()->network_quality_estimator()) {
-    request_->context()->network_quality_estimator()->NotifyDataReceived(
-        *request_, prefilter_bytes_read_, bytes_read);
+  // If prefilter_bytes_read_ is equal to bytes_read, it indicates this is the
+  // first raw read of the response body. This is used as the signal that
+  // response headers have been received.
+  if (request_->context()->network_quality_estimator() &&
+      prefilter_bytes_read_ == bytes_read) {
+    request_->context()->network_quality_estimator()->NotifyHeadersReceived(
+        *request_);
   }
 
   if (!filter_.get())
     postfilter_bytes_read_ += bytes_read;
   DVLOG(2) << __FUNCTION__ << "() "
-           << "\"" << (request_ ? request_->url().spec() : "???") << "\""
+           << "\"" << request_->url().spec() << "\""
            << " pre bytes read = " << bytes_read
            << " pre total = " << prefilter_bytes_read_
            << " post total = " << postfilter_bytes_read_;
   UpdatePacketReadTimes();  // Facilitate stats recording if it is active.
-  if (network_delegate_)
-    network_delegate_->NotifyRawBytesRead(*request_, bytes_read);
+
+  // Notify observers if any additional network usage has occurred. Note that
+  // the number of received bytes over the network sent by this notification
+  // could be vastly different from |bytes_read|, such as when a large chunk of
+  // network bytes is received before multiple smaller raw reads are performed
+  // on it.
+  MaybeNotifyNetworkBytes();
 }
 
 bool URLRequestJob::FilterHasData() {
@@ -896,13 +1003,51 @@ RedirectInfo URLRequestJob::ComputeRedirectInfo(const GURL& location,
         request_->first_party_for_cookies();
   }
 
+  if (request_->context()->enable_referrer_policy_header()) {
+    redirect_info.new_referrer_policy =
+        ProcessReferrerPolicyHeaderOnRedirect(request_);
+  } else {
+    redirect_info.new_referrer_policy = request_->referrer_policy();
+  }
+
   // Alter the referrer if redirecting cross-origin (especially HTTP->HTTPS).
   redirect_info.new_referrer =
-      ComputeReferrerForRedirect(request_->referrer_policy(),
-                                 request_->referrer(),
-                                 redirect_info.new_url).spec();
+      ComputeReferrerForRedirect(redirect_info.new_referrer_policy,
+                                 request_->referrer(), redirect_info.new_url)
+          .spec();
+
+  std::string include_referer;
+  request_->GetResponseHeaderByName("include-referer-token-binding-id",
+                                    &include_referer);
+  if (include_referer == "true" &&
+      request_->ssl_info().token_binding_negotiated) {
+    redirect_info.referred_token_binding_host = url.host();
+  }
 
   return redirect_info;
 }
 
+void URLRequestJob::MaybeNotifyNetworkBytes() {
+  if (!network_delegate_)
+    return;
+
+  // Report any new received bytes.
+  int64_t total_received_bytes = GetTotalReceivedBytes();
+  DCHECK_GE(total_received_bytes, last_notified_total_received_bytes_);
+  if (total_received_bytes > last_notified_total_received_bytes_) {
+    network_delegate_->NotifyNetworkBytesReceived(
+        request_, total_received_bytes - last_notified_total_received_bytes_);
+  }
+  last_notified_total_received_bytes_ = total_received_bytes;
+
+  // Report any new sent bytes.
+  int64_t total_sent_bytes = GetTotalSentBytes();
+  DCHECK_GE(total_sent_bytes, last_notified_total_sent_bytes_);
+  if (total_sent_bytes > last_notified_total_sent_bytes_) {
+    network_delegate_->NotifyNetworkBytesSent(
+        request_, total_sent_bytes - last_notified_total_sent_bytes_);
+  }
+  last_notified_total_sent_bytes_ = total_sent_bytes;
+}
+
 }  // namespace net
