@@ -8,13 +8,15 @@
 
 #include "net/websockets/websocket_basic_stream.h"
 
+#include <stddef.h>
 #include <stdint.h>
 #include <string.h>  // for memcpy() and memset().
-
 #include <string>
+#include <utility>
+#include <vector>
 
-#include "base/basictypes.h"
 #include "base/big_endian.h"
+#include "base/macros.h"
 #include "net/base/test_completion_callback.h"
 #include "net/log/test_net_log.h"
 #include "net/socket/socket_test_util.h"
@@ -74,7 +76,7 @@ WebSocketMaskingKey GenerateNonNulMaskingKey() { return kNonNulMaskingKey; }
 // Base class for WebSocketBasicStream test fixtures.
 class WebSocketBasicStreamTest : public ::testing::Test {
  protected:
-  scoped_ptr<WebSocketBasicStream> stream_;
+  std::unique_ptr<WebSocketBasicStream> stream_;
   TestNetLog net_log_;
 };
 
@@ -115,24 +117,22 @@ class WebSocketBasicStreamSocketTest : public WebSocketBasicStreamTest {
     stream_.reset();
   }
 
-  scoped_ptr<ClientSocketHandle> MakeTransportSocket(MockRead reads[],
-                                                     size_t reads_count,
-                                                     MockWrite writes[],
-                                                     size_t writes_count) {
+  std::unique_ptr<ClientSocketHandle> MakeTransportSocket(MockRead reads[],
+                                                          size_t reads_count,
+                                                          MockWrite writes[],
+                                                          size_t writes_count) {
     socket_data_.reset(new StrictStaticSocketDataProvider(
         reads, reads_count, writes, writes_count, expect_all_io_to_complete_));
     socket_data_->set_connect_data(MockConnect(SYNCHRONOUS, OK));
     factory_.AddSocketDataProvider(socket_data_.get());
 
-    scoped_ptr<ClientSocketHandle> transport_socket(new ClientSocketHandle);
+    std::unique_ptr<ClientSocketHandle> transport_socket(
+        new ClientSocketHandle);
     scoped_refptr<MockTransportSocketParams> params;
-    transport_socket->Init("a",
-                           params,
-                           MEDIUM,
-                           CompletionCallback(),
-                           &pool_,
-                           bound_net_log_.bound());
-    return transport_socket.Pass();
+    transport_socket->Init(
+        "a", params, MEDIUM, ClientSocketPool::RespectLimits::ENABLED,
+        CompletionCallback(), &pool_, bound_net_log_.bound());
+    return transport_socket;
   }
 
   void SetHttpReadBuffer(const char* data, size_t size) {
@@ -161,11 +161,11 @@ class WebSocketBasicStreamSocketTest : public WebSocketBasicStreamTest {
 
   void CreateNullStream() { CreateStream(NULL, 0, NULL, 0); }
 
-  scoped_ptr<SocketDataProvider> socket_data_;
+  std::unique_ptr<SocketDataProvider> socket_data_;
   MockClientSocketFactory factory_;
   MockTransportClientSocketPool pool_;
   BoundTestNetLog(bound_net_log_);
-  ScopedVector<WebSocketFrame> frames_;
+  std::vector<std::unique_ptr<WebSocketFrame>> frames_;
   TestCompletionCallback cb_;
   scoped_refptr<GrowableIOBuffer> http_read_buffer_;
   std::string sub_protocol_;
@@ -226,7 +226,7 @@ class WebSocketBasicStreamSocketChunkedReadTest
     CreateStream(reads_.get(), number_of_chunks, NULL, 0);
   }
 
-  scoped_ptr<MockRead[]> reads_;
+  std::unique_ptr<MockRead[]> reads_;
 };
 
 // Test fixture for write tests.
@@ -240,7 +240,7 @@ class WebSocketBasicStreamSocketWriteTest
   // Creates a WebSocketFrame with a wire format matching kWriteFrame and adds
   // it to |frames_|.
   void PrepareWriteFrame() {
-    scoped_ptr<WebSocketFrame> frame(
+    std::unique_ptr<WebSocketFrame> frame(
         new WebSocketFrame(WebSocketFrameHeader::kOpCodeText));
     const size_t payload_size =
         kWriteFrameSize - (WebSocketFrameHeader::kBaseHeaderSize +
@@ -253,7 +253,7 @@ class WebSocketBasicStreamSocketWriteTest
     header.final = true;
     header.masked = true;
     header.payload_length = payload_size;
-    frames_.push_back(frame.Pass());
+    frames_.push_back(std::move(frame));
   }
 
   // Creates a stream that expects the listed writes.
@@ -795,11 +795,11 @@ TEST_F(WebSocketBasicStreamSocketChunkedReadTest, OneMegFrame) {
   // This should be equal to the definition of kReadBufferSize in
   // websocket_basic_stream.cc.
   const int kReadBufferSize = 32 * 1024;
-  const uint64 kPayloadSize = 1 << 20;
+  const uint64_t kPayloadSize = 1 << 20;
   const size_t kWireSize = kPayloadSize + kLargeFrameHeaderSize;
   const size_t kExpectedFrameCount =
       (kWireSize + kReadBufferSize - 1) / kReadBufferSize;
-  scoped_ptr<char[]> big_frame(new char[kWireSize]);
+  std::unique_ptr<char[]> big_frame(new char[kWireSize]);
   memcpy(big_frame.get(), "\x81\x7F", 2);
   base::WriteBigEndian(big_frame.get() + 2, kPayloadSize);
   memset(big_frame.get() + kLargeFrameHeaderSize, 'A', kPayloadSize);
@@ -889,14 +889,14 @@ TEST_F(WebSocketBasicStreamSocketWriteTest, WriteNullPong) {
       MockWrite(SYNCHRONOUS, kMaskedEmptyPong, kMaskedEmptyPongSize)};
   CreateWriteOnly(writes);
 
-  scoped_ptr<WebSocketFrame> frame(
+  std::unique_ptr<WebSocketFrame> frame(
       new WebSocketFrame(WebSocketFrameHeader::kOpCodePong));
   WebSocketFrameHeader& header = frame->header;
   header.final = true;
   header.masked = true;
   header.payload_length = 0;
-  ScopedVector<WebSocketFrame> frames;
-  frames.push_back(frame.Pass());
+  std::vector<std::unique_ptr<WebSocketFrame>> frames;
+  frames.push_back(std::move(frame));
   EXPECT_EQ(OK, stream_->WriteFrames(&frames, cb_.callback()));
 }
 
@@ -910,7 +910,7 @@ TEST_F(WebSocketBasicStreamSocketTest, WriteNonNulMask) {
   generator_ = &GenerateNonNulMaskingKey;
   CreateStream(NULL, 0, writes, arraysize(writes));
 
-  scoped_ptr<WebSocketFrame> frame(
+  std::unique_ptr<WebSocketFrame> frame(
       new WebSocketFrame(WebSocketFrameHeader::kOpCodeText));
   const std::string unmasked_payload = "graphics";
   const size_t payload_size = unmasked_payload.size();
@@ -920,7 +920,7 @@ TEST_F(WebSocketBasicStreamSocketTest, WriteNonNulMask) {
   header.final = true;
   header.masked = true;
   header.payload_length = payload_size;
-  frames_.push_back(frame.Pass());
+  frames_.push_back(std::move(frame));
 
   EXPECT_EQ(OK, stream_->WriteFrames(&frames_, cb_.callback()));
 }
