@@ -34,9 +34,8 @@ class ManifestMessageFilter : public IPC::SyncMessageFilter {
   ManifestMessageFilter(base::WaitableEvent* shutdown_event)
       : SyncMessageFilter(shutdown_event,
                           false /* is_channel_send_thread_safe */),
-        connected_event_(
-            true /* manual_reset */, false /* initially_signaled */) {
-  }
+        connected_event_(base::WaitableEvent::ResetPolicy::MANUAL,
+                         base::WaitableEvent::InitialState::NOT_SIGNALED) {}
 
   bool Send(IPC::Message* message) override {
     // Wait until set up is actually done.
@@ -102,11 +101,8 @@ bool ManifestService::OpenResource(const char* file, int* fd) {
   // inside NaClIPCAdapter.
   uint64_t file_token_lo = 0;
   uint64_t file_token_hi = 0;
-  if (!filter_->Send(new PpapiHostMsg_OpenResource(
-          file,
-          &ipc_fd,
-          &file_token_lo,
-          &file_token_hi))) {
+  if (!filter_->Send(new PpapiHostMsg_OpenResource(file, &file_token_lo,
+                                                   &file_token_hi, &ipc_fd))) {
     LOG(ERROR) << "ManifestService::OpenResource failed:" << file;
     *fd = -1;
     return false;
