@@ -10,6 +10,7 @@
 #include <vector>
 
 #include "base/logging.h"
+#include "base/macros.h"
 #include "base/memory/singleton.h"
 #include "base/memory/weak_ptr.h"
 #include "build/build_config.h"
@@ -244,7 +245,7 @@ class ImageDataCache {
   void DidDeleteInstance(PP_Instance instance);
 
  private:
-  friend struct LeakySingletonTraits<ImageDataCache>;
+  friend struct base::LeakySingletonTraits<ImageDataCache>;
 
   // Timer callback to expire entries for the given instance.
   void OnTimer(PP_Instance instance);
@@ -263,8 +264,8 @@ class ImageDataCache {
 
 // static
 ImageDataCache* ImageDataCache::GetInstance() {
-  return Singleton<ImageDataCache,
-                   LeakySingletonTraits<ImageDataCache> >::get();
+  return base::Singleton<ImageDataCache,
+                         base::LeakySingletonTraits<ImageDataCache>>::get();
 }
 
 scoped_refptr<ImageData> ImageDataCache::Get(
@@ -386,16 +387,20 @@ PlatformImageData::~PlatformImageData() {
 
 void* PlatformImageData::Map() {
   if (!mapped_canvas_.get()) {
-    mapped_canvas_.reset(transport_dib_->GetPlatformCanvas(desc_.size.width,
-                                                           desc_.size.height));
+    if (!transport_dib_.get())
+      return NULL;
+
+    const bool is_opaque = false;
+    mapped_canvas_ =
+        sk_sp<SkCanvas>(transport_dib_->GetPlatformCanvas(desc_.size.width,
+                                                          desc_.size.height,
+                                                          is_opaque));
     if (!mapped_canvas_.get())
       return NULL;
   }
-  const SkBitmap& bitmap =
-      skia::GetTopDevice(*mapped_canvas_)->accessBitmap(true);
-
-  bitmap.lockPixels();
-  return bitmap.getAddr(0, 0);
+  SkPixmap pixmap;
+  skia::GetWritablePixels(mapped_canvas_.get(), &pixmap);
+  return pixmap.writable_addr();
 }
 
 void PlatformImageData::Unmap() {
@@ -414,19 +419,7 @@ SkCanvas* PlatformImageData::GetCanvas() {
 
 // static
 ImageHandle PlatformImageData::NullHandle() {
-#if defined(OS_WIN)
-  return NULL;
-#else
   return ImageHandle();
-#endif
-}
-
-ImageHandle PlatformImageData::HandleFromInt(int32_t i) {
-#if defined(OS_WIN)
-    return reinterpret_cast<ImageHandle>(i);
-#else
-    return ImageHandle(i, false);
-#endif
 }
 #endif  // !defined(OS_NACL)
 
