@@ -4,29 +4,16 @@
 
 #include "printing/pdf_metafile_skia.h"
 
-#include <stddef.h>
-#include <stdint.h>
-
-#include "base/containers/hash_tables.h"
-#include "base/files/file_util.h"
-#include "base/metrics/histogram.h"
-#include "base/numerics/safe_conversions.h"
-#include "base/posix/eintr_wrapper.h"
+#include "base/files/file.h"
 #include "base/time/time.h"
 #include "printing/print_settings.h"
-#include "skia/ext/refptr.h"
-#include "third_party/skia/include/core/SkData.h"
 #include "third_party/skia/include/core/SkDocument.h"
 #include "third_party/skia/include/core/SkPictureRecorder.h"
-#include "third_party/skia/include/core/SkRect.h"
-#include "third_party/skia/include/core/SkRefCnt.h"
-#include "third_party/skia/include/core/SkScalar.h"
-#include "third_party/skia/include/core/SkSize.h"
 #include "third_party/skia/include/core/SkStream.h"
-#include "ui/gfx/geometry/point.h"
-#include "ui/gfx/geometry/rect.h"
-#include "ui/gfx/geometry/size.h"
-#include "ui/gfx/geometry/size_conversions.h"
+// Note that headers in third_party/skia/src are fragile.  This is
+// an experimental, fragile, and diagnostic-only document type.
+#include "third_party/skia/src/utils/SkMultiPictureDocument.h"
+#include "ui/gfx/geometry/safe_integer_conversions.h"
 #include "ui/gfx/skia_util.h"
 
 #if defined(OS_MACOSX)
@@ -39,40 +26,76 @@
 
 namespace {
 
-// This struct represents all the data we need to draw and redraw this
-// page into a SkDocument.
-struct Page {
-  Page(const SkSize& page_size, const SkRect& content_area, float scale)
-      : page_size_(page_size),
-        content_area_(content_area),
-        scale_factor_(scale),
-        content_(/*NULL*/) {}
-  SkSize page_size_;
-  SkRect content_area_;
-  float scale_factor_;
-  skia::RefPtr<SkPicture> content_;
-};
-
 bool WriteAssetToBuffer(const SkStreamAsset* asset,
                         void* buffer,
                         size_t size) {
   // Calling duplicate() keeps original asset state unchanged.
-  scoped_ptr<SkStreamAsset> assetCopy(asset->duplicate());
+  std::unique_ptr<SkStreamAsset> assetCopy(asset->duplicate());
   size_t length = assetCopy->getLength();
   if (length > size)
     return false;
   return (length == assetCopy->read(buffer, length));
 }
 
+SkTime::DateTime TimeToSkTime(base::Time time) {
+  base::Time::Exploded exploded;
+  time.UTCExplode(&exploded);
+  SkTime::DateTime skdate;
+  skdate.fTimeZoneMinutes = 0;
+  skdate.fYear = exploded.year;
+  skdate.fMonth = exploded.month;
+  skdate.fDayOfWeek = exploded.day_of_week;
+  skdate.fDay = exploded.day_of_month;
+  skdate.fHour = exploded.hour;
+  skdate.fMinute = exploded.minute;
+  skdate.fSecond = exploded.second;
+  return skdate;
+}
+
+sk_sp<SkDocument> MakePdfDocument(SkWStream* wStream) {
+  SkDocument::PDFMetadata metadata;
+  SkTime::DateTime now = TimeToSkTime(base::Time::Now());
+  metadata.fCreation.fEnabled = true;
+  metadata.fCreation.fDateTime = now;
+  metadata.fModified.fEnabled = true;
+  metadata.fModified.fDateTime = now;
+  const std::string& agent = printing::GetAgent();
+  metadata.fCreator = agent.empty() ? SkString("Chromium")
+                                    : SkString(agent.c_str(), agent.size());
+  return SkDocument::MakePDF(wStream, SK_ScalarDefaultRasterDPI, metadata,
+                             nullptr, false);
+}
+
 }  // namespace
 
 namespace printing {
 
+struct Page {
+  Page(SkSize s, sk_sp<SkPicture> c) : size_(s), content_(std::move(c)) {}
+  Page(Page&& that) : size_(that.size_), content_(std::move(that.content_)) {}
+  Page(const Page&) = default;
+  Page& operator=(const Page&) = default;
+  Page& operator=(Page&& that) {
+    size_ = that.size_;
+    content_ = std::move(that.content_);
+    return *this;
+  }
+  SkSize size_;
+  sk_sp<SkPicture> content_;
+};
+
 struct PdfMetafileSkiaData {
   SkPictureRecorder recorder_;  // Current recording
 
   std::vector<Page> pages_;
-  scoped_ptr<SkStreamAsset> pdf_data_;
+  std::unique_ptr<SkStreamAsset> pdf_data_;
+
+  // The scale factor is used because Blink occasionally calls
+  // SkCanvas::getTotalMatrix() even though the total matrix is not as
+  // meaningful for a vector canvas as for a raster canvas.
+  float scale_factor_;
+  SkSize size_;
+  SkiaDocumentType type_;
 
 #if defined(OS_MACOSX)
   PdfMetafileCg pdf_cg_;
@@ -94,57 +117,58 @@ bool PdfMetafileSkia::InitFromData(const void* src_buffer,
   return true;
 }
 
-bool PdfMetafileSkia::StartPage(const gfx::Size& page_size,
+void PdfMetafileSkia::StartPage(const gfx::Size& page_size,
                                 const gfx::Rect& content_area,
                                 const float& scale_factor) {
-  if (data_->recorder_.getRecordingCanvas())
-    this->FinishPage();
-  DCHECK(!data_->recorder_.getRecordingCanvas());
-  SkSize sk_page_size = gfx::SizeFToSkSize(gfx::SizeF(page_size));
-  data_->pages_.push_back(
-      Page(sk_page_size, gfx::RectToSkRect(content_area), scale_factor));
+  DCHECK_GT(page_size.width(), 0);
+  DCHECK_GT(page_size.height(), 0);
   DCHECK_GT(scale_factor, 0.0f);
+  if (data_->recorder_.getRecordingCanvas())
+    FinishPage();
+  DCHECK(!data_->recorder_.getRecordingCanvas());
+
+  float inverse_scale = 1.0 / scale_factor;
+  SkCanvas* canvas = data_->recorder_.beginRecording(
+      inverse_scale * page_size.width(), inverse_scale * page_size.height());
+  // Recording canvas is owned by the data_->recorder_.  No ref() necessary.
+  if (content_area != gfx::Rect(page_size)) {
+    canvas->scale(inverse_scale, inverse_scale);
+    SkRect sk_content_area = gfx::RectToSkRect(content_area);
+    canvas->clipRect(sk_content_area);
+    canvas->translate(sk_content_area.x(), sk_content_area.y());
+    canvas->scale(scale_factor, scale_factor);
+  }
+
+  data_->size_ = gfx::SizeFToSkSize(gfx::SizeF(page_size));
+  data_->scale_factor_ = scale_factor;
   // We scale the recording canvas's size so that
   // canvas->getTotalMatrix() returns a value that ignores the scale
-  // factor.  We store the scale factor and re-apply it to the PDF
-  // Canvas later.  http://crbug.com/469656
-  // Recording canvas is owned by the data_->recorder_.  No ref() necessary.
-  return !!data_->recorder_.beginRecording(sk_page_size.width() / scale_factor,
-                                           sk_page_size.height() / scale_factor,
-                                           NULL, 0);
+  // factor.  We store the scale factor and re-apply it later.
+  // http://crbug.com/469656
 }
 
-skia::PlatformCanvas* PdfMetafileSkia::GetVectorCanvasForNewPage(
+SkCanvas* PdfMetafileSkia::GetVectorCanvasForNewPage(
     const gfx::Size& page_size,
     const gfx::Rect& content_area,
     const float& scale_factor) {
-  if (!StartPage(page_size, content_area, scale_factor))
-    return nullptr;
+  StartPage(page_size, content_area, scale_factor);
   return data_->recorder_.getRecordingCanvas();
 }
 
 bool PdfMetafileSkia::FinishPage() {
   if (!data_->recorder_.getRecordingCanvas())
     return false;
-  DCHECK(!(data_->pages_.back().content_));
-  data_->pages_.back().content_ =
-      skia::AdoptRef(data_->recorder_.endRecordingAsPicture());
-  return true;
-}
 
-static SkTime::DateTime TimeToSkTime(base::Time time) {
-    base::Time::Exploded exploded;
-    time.UTCExplode(&exploded);
-    SkTime::DateTime skdate;
-    skdate.fTimeZoneMinutes = 0;
-    skdate.fYear = exploded.year;
-    skdate.fMonth = exploded.month;
-    skdate.fDayOfWeek = exploded.day_of_week;
-    skdate.fDay = exploded.day_of_month;
-    skdate.fHour = exploded.hour;
-    skdate.fMinute = exploded.minute;
-    skdate.fSecond = exploded.second;
-    return skdate;
+  sk_sp<SkPicture> pic = data_->recorder_.finishRecordingAsPicture();
+  if (data_->scale_factor_ != 1.0f) {
+    SkCanvas* canvas = data_->recorder_.beginRecording(data_->size_.width(),
+                                                       data_->size_.height());
+    canvas->scale(data_->scale_factor_, data_->scale_factor_);
+    canvas->drawPicture(pic);
+    pic = data_->recorder_.finishRecordingAsPicture();
+  }
+  data_->pages_.emplace_back(data_->size_, std::move(pic));
+  return true;
 }
 
 bool PdfMetafileSkia::FinishDocument() {
@@ -153,31 +177,28 @@ bool PdfMetafileSkia::FinishDocument() {
     return false;
 
   if (data_->recorder_.getRecordingCanvas())
-    this->FinishPage();
+    FinishPage();
 
-  SkDynamicMemoryWStream pdf_stream;
-  skia::RefPtr<SkDocument> pdf_doc =
-      skia::AdoptRef(SkDocument::CreatePDF(&pdf_stream));
-  for (const auto& page : data_->pages_) {
-    SkCanvas* canvas = pdf_doc->beginPage(
-        page.page_size_.width(), page.page_size_.height(), &page.content_area_);
-    // No need to save/restore, since this canvas is not reused after endPage()
-    canvas->scale(page.scale_factor_, page.scale_factor_);
-    canvas->drawPicture(page.content_.get());
-    pdf_doc->endPage();
+  SkDynamicMemoryWStream stream;
+  sk_sp<SkDocument> doc;
+  switch (data_->type_) {
+    case PDF_SKIA_DOCUMENT_TYPE:
+      doc = MakePdfDocument(&stream);
+      break;
+    case MSKP_SKIA_DOCUMENT_TYPE:
+      doc = SkMakeMultiPictureDocument(&stream);
+      break;
   }
-  SkTArray<SkDocument::Attribute> info;
-  const std::string& user_agent = GetAgent();
-  info.emplace_back(SkString("Creator"),
-                    user_agent.empty()
-                        ? SkString("Chromium")
-                        : SkString(user_agent.c_str(), user_agent.size()));
-  SkTime::DateTime now = TimeToSkTime(base::Time::Now());
-  pdf_doc->setMetadata(info, &now, &now);
-  if (!pdf_doc->close())
+
+  for (const Page& page : data_->pages_) {
+    SkCanvas* canvas = doc->beginPage(page.size_.width(), page.size_.height());
+    canvas->drawPicture(page.content_);
+    doc->endPage();
+  }
+  if (!doc->close())
     return false;
 
-  data_->pdf_data_.reset(pdf_stream.detachAsStream());
+  data_->pdf_data_.reset(stream.detachAsStream());
   return true;
 }
 
@@ -197,8 +218,9 @@ bool PdfMetafileSkia::GetData(void* dst_buffer,
 
 gfx::Rect PdfMetafileSkia::GetPageBounds(unsigned int page_number) const {
   if (page_number < data_->pages_.size()) {
-    return gfx::Rect(gfx::ToFlooredSize(
-        gfx::SkSizeToSizeF(data_->pages_[page_number].page_size_)));
+    SkSize size = data_->pages_[page_number].size_;
+    return gfx::Rect(gfx::ToRoundedInt(size.width()),
+                     gfx::ToRoundedInt(size.height()));
   }
   return gfx::Rect();
 }
@@ -209,9 +231,10 @@ unsigned int PdfMetafileSkia::GetPageCount() const {
 
 gfx::NativeDrawingContext PdfMetafileSkia::context() const {
   NOTREACHED();
-  return NULL;
+  return nullptr;
 }
 
+
 #if defined(OS_WIN)
 bool PdfMetafileSkia::Playback(gfx::NativeDrawingContext hdc,
                                const RECT* rect) const {
@@ -255,10 +278,10 @@ bool PdfMetafileSkia::SaveTo(base::File* file) const {
     return false;
 
   // Calling duplicate() keeps original asset state unchanged.
-  scoped_ptr<SkStreamAsset> asset(data_->pdf_data_->duplicate());
+  std::unique_ptr<SkStreamAsset> asset(data_->pdf_data_->duplicate());
 
-  const size_t maximum_buffer_size = 1024 * 1024;
-  std::vector<char> buffer(std::min(maximum_buffer_size, asset->getLength()));
+  const size_t kMaximumBufferSize = 1024 * 1024;
+  std::vector<char> buffer(std::min(kMaximumBufferSize, asset->getLength()));
   do {
     size_t read_size = asset->read(&buffer[0], buffer.size());
     if (read_size == 0)
@@ -273,31 +296,16 @@ bool PdfMetafileSkia::SaveTo(base::File* file) const {
   return true;
 }
 
-#if defined(OS_CHROMEOS) || defined(OS_ANDROID)
-bool PdfMetafileSkia::SaveToFD(const base::FileDescriptor& fd) const {
-  DCHECK_GT(GetDataSize(), 0U);
-
-  if (fd.fd < 0) {
-    DLOG(ERROR) << "Invalid file descriptor!";
-    return false;
-  }
-  base::File file(fd.fd);
-  bool result = SaveTo(&file);
-  DLOG_IF(ERROR, !result) << "Failed to save file with fd " << fd.fd;
-
-  if (!fd.auto_close)
-    file.TakePlatformFile();
-  return result;
-}
-#endif
-
-PdfMetafileSkia::PdfMetafileSkia() : data_(new PdfMetafileSkiaData) {
+PdfMetafileSkia::PdfMetafileSkia(SkiaDocumentType type)
+    : data_(new PdfMetafileSkiaData) {
+  data_->type_ = type;
 }
 
-scoped_ptr<PdfMetafileSkia> PdfMetafileSkia::GetMetafileForCurrentPage() {
+std::unique_ptr<PdfMetafileSkia> PdfMetafileSkia::GetMetafileForCurrentPage(
+    SkiaDocumentType type) {
   // If we only ever need the metafile for the last page, should we
   // only keep a handle on one SkPicture?
-  scoped_ptr<PdfMetafileSkia> metafile(new PdfMetafileSkia);
+  std::unique_ptr<PdfMetafileSkia> metafile(new PdfMetafileSkia(type));
 
   if (data_->pages_.size() == 0)
     return metafile;
@@ -305,9 +313,7 @@ scoped_ptr<PdfMetafileSkia> PdfMetafileSkia::GetMetafileForCurrentPage() {
   if (data_->recorder_.getRecordingCanvas())  // page outstanding
     return metafile;
 
-  const Page& page = data_->pages_.back();
-
-  metafile->data_->pages_.push_back(page);
+  metafile->data_->pages_.push_back(data_->pages_.back());
 
   if (!metafile->FinishDocument())  // Generate PDF.
     metafile.reset();
