@@ -20,11 +20,15 @@
 #include "sql/meta_table.h"
 #include "sql/statement.h"
 #include "sql/test/error_callback_support.h"
-#include "sql/test/scoped_error_ignorer.h"
+#include "sql/test/scoped_error_expecter.h"
 #include "sql/test/test_helpers.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "third_party/sqlite/sqlite3.h"
 
+#if defined(OS_IOS) && defined(USE_SYSTEM_SQLITE)
+#include "base/ios/ios_util.h"
+#endif
+
 namespace sql {
 namespace test {
 
@@ -71,7 +75,7 @@ class ScopedMockTimeSource {
   Connection& db_;
 
   // Saves original source from |db_|.
-  scoped_ptr<TimeSource> save_;
+  std::unique_ptr<TimeSource> save_;
 
   // Current time returned by mock.
   base::TimeTicks current_time_;
@@ -247,7 +251,9 @@ class SQLConnectionTest : public sql::SQLTestBase {
 
   // Handle errors by blowing away the database.
   void RazeErrorCallback(int expected_error, int error, sql::Statement* stmt) {
-    EXPECT_EQ(expected_error, error);
+    // Nothing here needs extended errors at this time.
+    EXPECT_EQ(expected_error, expected_error&0xff);
+    EXPECT_EQ(expected_error, error&0xff);
     db().RazeAndClose();
   }
 };
@@ -356,23 +362,23 @@ TEST_F(SQLConnectionTest, Rollback) {
   EXPECT_TRUE(db().BeginTransaction());
 }
 
-// Test the scoped error ignorer by attempting to insert a duplicate
+// Test the scoped error expecter by attempting to insert a duplicate
 // value into an index.
-TEST_F(SQLConnectionTest, ScopedIgnoreError) {
+TEST_F(SQLConnectionTest, ScopedErrorExpecter) {
   const char* kCreateSql = "CREATE TABLE foo (id INTEGER UNIQUE)";
   ASSERT_TRUE(db().Execute(kCreateSql));
   ASSERT_TRUE(db().Execute("INSERT INTO foo (id) VALUES (12)"));
 
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CONSTRAINT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CONSTRAINT);
     ASSERT_FALSE(db().Execute("INSERT INTO foo (id) VALUES (12)"));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 }
 
 // Test that clients of GetUntrackedStatement() can test corruption-handling
-// with ScopedErrorIgnorer.
+// with ScopedErrorExpecter.
 TEST_F(SQLConnectionTest, ScopedIgnoreUntracked) {
   const char* kCreateSql = "CREATE TABLE foo (id INTEGER UNIQUE)";
   ASSERT_TRUE(db().Execute(kCreateSql));
@@ -385,13 +391,13 @@ TEST_F(SQLConnectionTest, ScopedIgnoreUntracked) {
   ASSERT_TRUE(CorruptSizeInHeaderOfDB());
 
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CORRUPT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CORRUPT);
     ASSERT_TRUE(db().Open(db_path()));
     ASSERT_FALSE(db().DoesTableExist("bar"));
     ASSERT_FALSE(db().DoesTableExist("foo"));
     ASSERT_FALSE(db().DoesColumnExist("foo", "id"));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 }
 
@@ -414,10 +420,10 @@ TEST_F(SQLConnectionTest, ErrorCallback) {
   // Callback is no longer in force due to reset.
   {
     error = SQLITE_OK;
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CONSTRAINT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CONSTRAINT);
     ASSERT_FALSE(db().Execute("INSERT INTO foo (id) VALUES (12)"));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
     EXPECT_EQ(SQLITE_OK, error);
   }
 
@@ -614,19 +620,19 @@ TEST_F(SQLConnectionTest, RazeNOTADB) {
   // SQLite will successfully open the handle, but fail when running PRAGMA
   // statements that access the database.
   {
-    sql::ScopedErrorIgnorer ignore_errors;
+    sql::test::ScopedErrorExpecter expecter;
 
     // Earlier versions of Chromium compiled against SQLite 3.6.7.3, which
     // returned SQLITE_IOERR_SHORT_READ in this case.  Some platforms may still
     // compile against an earlier SQLite via USE_SYSTEM_SQLITE.
-    if (ignore_errors.SQLiteLibVersionNumber() < 3008005) {
-      ignore_errors.IgnoreError(SQLITE_IOERR_SHORT_READ);
+    if (expecter.SQLiteLibVersionNumber() < 3008005) {
+      expecter.ExpectError(SQLITE_IOERR_SHORT_READ);
     } else {
-      ignore_errors.IgnoreError(SQLITE_NOTADB);
+      expecter.ExpectError(SQLITE_NOTADB);
     }
 
     EXPECT_TRUE(db().Open(db_path()));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
   EXPECT_TRUE(db().Raze());
   db().Close();
@@ -649,10 +655,10 @@ TEST_F(SQLConnectionTest, RazeNOTADB2) {
   // SQLITE_NOTADB on pragma statemenets which attempt to read the
   // corrupted header.
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_NOTADB);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_NOTADB);
     EXPECT_TRUE(db().Open(db_path()));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
   EXPECT_TRUE(db().Raze());
   db().Close();
@@ -678,12 +684,12 @@ TEST_F(SQLConnectionTest, RazeCallbackReopen) {
   // Open() will succeed, even though the PRAGMA calls within will
   // fail with SQLITE_CORRUPT, as will this PRAGMA.
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CORRUPT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CORRUPT);
     ASSERT_TRUE(db().Open(db_path()));
     ASSERT_FALSE(db().Execute("PRAGMA auto_vacuum"));
     db().Close();
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 
   db().set_error_callback(base::Bind(&SQLConnectionTest::RazeErrorCallback,
@@ -928,6 +934,19 @@ TEST_F(SQLConnectionTest, Poison) {
   // The existing statement has become invalid.
   ASSERT_FALSE(valid_statement.is_valid());
   ASSERT_FALSE(valid_statement.Step());
+
+  // Test that poisoning the database during a transaction works (with errors).
+  // RazeErrorCallback() poisons the database, the extra COMMIT causes
+  // CommitTransaction() to throw an error while commiting.
+  db().set_error_callback(base::Bind(&SQLConnectionTest::RazeErrorCallback,
+                                     base::Unretained(this),
+                                     SQLITE_ERROR));
+  db().Close();
+  ASSERT_TRUE(db().Open(db_path()));
+  EXPECT_TRUE(db().BeginTransaction());
+  EXPECT_TRUE(db().Execute("INSERT INTO x VALUES ('x')"));
+  EXPECT_TRUE(db().Execute("COMMIT"));
+  EXPECT_FALSE(db().CommitTransaction());
 }
 
 // Test attaching and detaching databases from the connection.
@@ -951,10 +970,10 @@ TEST_F(SQLConnectionTest, Attach) {
   // Attach fails in a transaction.
   EXPECT_TRUE(db().BeginTransaction());
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_ERROR);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_ERROR);
     EXPECT_FALSE(db().AttachDatabase(attach_path, kAttachmentPoint));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 
   // Attach succeeds when the transaction is closed.
@@ -973,11 +992,11 @@ TEST_F(SQLConnectionTest, Attach) {
   // Detach also fails in a transaction.
   EXPECT_TRUE(db().BeginTransaction());
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_ERROR);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_ERROR);
     EXPECT_FALSE(db().DetachDatabase(kAttachmentPoint));
     EXPECT_TRUE(db().IsSQLValid("SELECT count(*) from other.bar"));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 
   // Detach succeeds outside of a transaction.
@@ -996,11 +1015,11 @@ TEST_F(SQLConnectionTest, Basic_QuickIntegrityCheck) {
   ASSERT_TRUE(CorruptSizeInHeaderOfDB());
 
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CORRUPT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CORRUPT);
     ASSERT_TRUE(db().Open(db_path()));
     EXPECT_FALSE(db().QuickIntegrityCheck());
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 }
 
@@ -1018,13 +1037,13 @@ TEST_F(SQLConnectionTest, Basic_FullIntegrityCheck) {
   ASSERT_TRUE(CorruptSizeInHeaderOfDB());
 
   {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_CORRUPT);
+    sql::test::ScopedErrorExpecter expecter;
+    expecter.ExpectError(SQLITE_CORRUPT);
     ASSERT_TRUE(db().Open(db_path()));
     EXPECT_TRUE(db().FullIntegrityCheck(&messages));
     EXPECT_LT(1u, messages.size());
     EXPECT_NE(kOk, messages[0]);
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
+    ASSERT_TRUE(expecter.SawExpectedErrors());
   }
 
   // TODO(shess): CorruptTableOrIndex could be used to produce a
@@ -1193,7 +1212,7 @@ TEST_F(SQLConnectionTest, TimeQuery) {
 
   EXPECT_TRUE(db().Execute("SELECT milliadjust(10)"));
 
-  scoped_ptr<base::HistogramSamples> samples(
+  std::unique_ptr<base::HistogramSamples> samples(
       tester.GetHistogramSamplesSinceCreation(kQueryTime));
   ASSERT_TRUE(samples);
   // 10 for the adjust, 1 for the measurement.
@@ -1231,7 +1250,7 @@ TEST_F(SQLConnectionTest, TimeUpdateAutocommit) {
 
   EXPECT_TRUE(db().Execute("INSERT INTO foo VALUES (10, milliadjust(10))"));
 
-  scoped_ptr<base::HistogramSamples> samples(
+  std::unique_ptr<base::HistogramSamples> samples(
       tester.GetHistogramSamplesSinceCreation(kQueryTime));
   ASSERT_TRUE(samples);
   // 10 for the adjust, 1 for the measurement.
@@ -1283,7 +1302,7 @@ TEST_F(SQLConnectionTest, TimeUpdateTransaction) {
     EXPECT_TRUE(db().CommitTransaction());
   }
 
-  scoped_ptr<base::HistogramSamples> samples(
+  std::unique_ptr<base::HistogramSamples> samples(
       tester.GetHistogramSamplesSinceCreation(kQueryTime));
   ASSERT_TRUE(samples);
   // 10 for insert adjust, 10 for update adjust, 100 for commit adjust, 1 for
@@ -1306,9 +1325,9 @@ TEST_F(SQLConnectionTest, TimeUpdateTransaction) {
 }
 
 TEST_F(SQLConnectionTest, OnMemoryDump) {
-  base::trace_event::ProcessMemoryDump pmd(nullptr);
   base::trace_event::MemoryDumpArgs args = {
       base::trace_event::MemoryDumpLevelOfDetail::DETAILED};
+  base::trace_event::ProcessMemoryDump pmd(nullptr, args);
   ASSERT_TRUE(db().memory_dump_provider_->OnMemoryDump(args, &pmd));
   EXPECT_GE(pmd.allocator_dumps().size(), 1u);
 }
@@ -1420,30 +1439,23 @@ TEST_F(SQLConnectionTest, MmapInitiallyEnabled) {
   }
 }
 
-// Test specific operation of the GetAppropriateMmapSize() helper.
-#if defined(OS_IOS)
-TEST_F(SQLConnectionTest, GetAppropriateMmapSize) {
-  ASSERT_EQ(0UL, db().GetAppropriateMmapSize());
-}
-#else
 TEST_F(SQLConnectionTest, GetAppropriateMmapSize) {
+#if defined(OS_IOS) && defined(USE_SYSTEM_SQLITE)
+  // Mmap is not supported on iOS9.
+  if (!base::ios::IsRunningOnIOS10OrLater()) {
+    ASSERT_EQ(0UL, db().GetAppropriateMmapSize());
+    return;
+  }
+#endif
+
   const size_t kMmapAlot = 25 * 1024 * 1024;
+  int64_t mmap_status = MetaTable::kMmapFailure;
 
   // If there is no meta table (as for a fresh database), assume that everything
-  // should be mapped.
+  // should be mapped, and the status of the meta table is not affected.
   ASSERT_TRUE(!db().DoesTableExist("meta"));
   ASSERT_GT(db().GetAppropriateMmapSize(), kMmapAlot);
-
-  // Getting the status fails if there is an error.  GetAppropriateMmapSize()
-  // should not call GetMmapStatus() if the table does not exist, but this is an
-  // easy error to setup for testing.
-  int64_t mmap_status;
-  {
-    sql::ScopedErrorIgnorer ignore_errors;
-    ignore_errors.IgnoreError(SQLITE_ERROR);
-    ASSERT_FALSE(MetaTable::GetMmapStatus(&db(), &mmap_status));
-    ASSERT_TRUE(ignore_errors.CheckIgnoredErrors());
-  }
+  ASSERT_TRUE(!db().DoesTableExist("meta"));
 
   // When the meta table is first created, it sets up to map everything.
   MetaTable().Init(&db(), 1, 1);
@@ -1471,6 +1483,20 @@ TEST_F(SQLConnectionTest, GetAppropriateMmapSize) {
   ASSERT_TRUE(MetaTable::GetMmapStatus(&db(), &mmap_status));
   ASSERT_EQ(MetaTable::kMmapSuccess, mmap_status);
 }
+
+// To prevent invalid SQL from accidentally shipping to production, prepared
+// statements which fail to compile with SQLITE_ERROR call DLOG(FATAL).  This
+// case cannot be suppressed with an error callback.
+TEST_F(SQLConnectionTest, CompileError) {
+  // DEATH tests not supported on Android or iOS.
+#if !defined(OS_ANDROID) && !defined(OS_IOS)
+  if (DLOG_IS_ON(FATAL)) {
+    db().set_error_callback(base::Bind(&IgnoreErrorCallback));
+    ASSERT_DEATH({
+        db().GetUniqueStatement("SELECT x");
+      }, "SQL compile error no such column: x");
+  }
 #endif
+}
 
 }  // namespace sql
