@@ -440,7 +440,7 @@ def GetFlavor(params):
   return 'linux'
 
 
-def CopyTool(flavor, out_path):
+def CopyTool(flavor, out_path, generator_flags={}):
   """Finds (flock|mac|win)_tool.gyp in the gyp directory and copies it
   to |out_path|."""
   # aix and solaris just need flock emulation. mac and win use more complicated
@@ -460,11 +460,18 @@ def CopyTool(flavor, out_path):
   with open(source_path) as source_file:
     source = source_file.readlines()
 
+  # Set custom header flags.
+  header = '# Generated by gyp. Do not edit.\n'
+  mac_toolchain_dir =  generator_flags.get('mac_toolchain_dir', None)
+  if flavor == 'mac' and mac_toolchain_dir:
+    header += "import os;\nos.environ['DEVELOPER_DIR']='%s'\n" \
+        % mac_toolchain_dir
+
   # Add header and write it out.
   tool_path = os.path.join(out_path, 'gyp-%s-tool' % prefix)
   with open(tool_path, 'w') as tool_file:
     tool_file.write(
-        ''.join([source[0], '# Generated by gyp. Do not edit.\n'] + source[1:]))
+        ''.join([source[0], header] + source[1:]))
 
   # Make file executable.
   os.chmod(tool_path, 0755)
