@@ -162,6 +162,8 @@ class XcodeSettings(object):
     self.spec = spec
 
     self.isIOS = False
+    self.mac_toolchain_dir = None
+    self.header_map_path = None
 
     # Per-target 'xcode_settings' are pushed down into configs earlier by gyp.
     # This means self.xcode_settings[config] always contains all settings
@@ -222,12 +224,20 @@ class XcodeSettings(object):
                                                  default)
     return format == "binary"
 
+  def IsIosFramework(self):
+    return self.spec['type'] == 'shared_library' and self._IsBundle() and \
+        self.isIOS
+
   def _IsBundle(self):
-    return int(self.spec.get('mac_bundle', 0)) != 0 or self._IsXCTest()
+    return int(self.spec.get('mac_bundle', 0)) != 0 or self._IsXCTest() or \
+        self._IsXCUiTest()
 
   def _IsXCTest(self):
     return int(self.spec.get('mac_xctest_bundle', 0)) != 0
 
+  def _IsXCUiTest(self):
+    return int(self.spec.get('mac_xcuitest_bundle', 0)) != 0
+
   def _IsIosAppExtension(self):
     return int(self.spec.get('ios_app_extension', 0)) != 0
 
@@ -306,7 +316,8 @@ class XcodeSettings(object):
     """Returns the qualified path to the bundle's plist file. E.g.
     Chromium.app/Contents/Info.plist. Only valid for bundles."""
     assert self._IsBundle()
-    if self.spec['type'] in ('executable', 'loadable_module'):
+    if self.spec['type'] in ('executable', 'loadable_module') or \
+        self.IsIosFramework():
       return os.path.join(self.GetBundleContentsFolderPath(), 'Info.plist')
     else:
       return os.path.join(self.GetBundleContentsFolderPath(),
@@ -326,6 +337,10 @@ class XcodeSettings(object):
       assert self._IsBundle(), ('ios_watch_app flag requires mac_bundle '
           '(target %s)' % self.spec['target_name'])
       return 'com.apple.product-type.application.watchapp'
+    if self._IsXCUiTest():
+      assert self._IsBundle(), ('mac_xcuitest_bundle flag requires mac_bundle '
+          '(target %s)' % self.spec['target_name'])
+      return 'com.apple.product-type.bundle.ui-testing'
     if self._IsBundle():
       return {
         'executable': 'com.apple.product-type.application',
@@ -489,6 +504,9 @@ class XcodeSettings(object):
     if 'SDKROOT' in self._Settings() and sdk_root:
       cflags.append('-isysroot %s' % sdk_root)
 
+    if self.header_map_path:
+      cflags.append('-I%s' % self.header_map_path)
+
     if self._Test('CLANG_WARN_CONSTANT_CONVERSION', 'YES', default='NO'):
       cflags.append('-Wconstant-conversion')
 
@@ -830,7 +848,8 @@ class XcodeSettings(object):
     ldflags.append('-arch ' + archs[0])
 
     # Xcode adds the product directory by default.
-    ldflags.append('-L' + product_dir)
+    # Rewrite -L. to -L./ to work around http://www.openradar.me/25313838
+    ldflags.append('-L' + (product_dir if product_dir != '.' else './'))
 
     install_name = self.GetInstallName()
     if install_name and self.spec['type'] != 'loadable_module':
@@ -858,13 +877,13 @@ class XcodeSettings(object):
       # extensions and provide loader and main function.
       # These flags reflect the compilation options used by xcode to compile
       # extensions.
-      ldflags.append('-lpkstart')
       if XcodeVersion() < '0900':
+        ldflags.append('-lpkstart')
         ldflags.append(sdk_root +
             '/System/Library/PrivateFrameworks/PlugInKit.framework/PlugInKit')
+      else:
+        ldflags.append('-e _NSExtensionMain')
       ldflags.append('-fapplication-extension')
-      ldflags.append('-Xlinker -rpath '
-          '-Xlinker @executable_path/../../Frameworks')
 
     self._Appendf(ldflags, 'CLANG_CXX_LIBRARY', '-stdlib=%s')
 
@@ -938,7 +957,8 @@ class XcodeSettings(object):
         self._Test('STRIP_INSTALLED_PRODUCT', 'YES', default='NO')):
 
       default_strip_style = 'debugging'
-      if self.spec['type'] == 'loadable_module' and self._IsBundle():
+      if ((self.spec['type'] == 'loadable_module' or self._IsIosAppExtension())
+          and self._IsBundle()):
         default_strip_style = 'non-global'
       elif self.spec['type'] == 'executable':
         default_strip_style = 'all'
@@ -994,13 +1014,24 @@ class XcodeSettings(object):
     be deployed to a device.  This should be run as the very last step of the
     build."""
     if not (self.isIOS and
-        (self.spec['type'] == 'executable' or self._IsXCTest())):
+        (self.spec['type'] == 'executable' or self._IsXCTest()) or
+         self.IsIosFramework()):
       return []
 
+    postbuilds = []
+    product_name = self.GetFullProductName()
     settings = self.xcode_settings[configname]
+
+    # Xcode expects XCTests to be copied into the TEST_HOST dir.
+    if self._IsXCTest():
+      source = os.path.join("${BUILT_PRODUCTS_DIR}", product_name)
+      test_host = os.path.dirname(settings.get('TEST_HOST'));
+      xctest_destination = os.path.join(test_host, 'PlugIns', product_name)
+      postbuilds.extend(['ditto %s %s' % (source, xctest_destination)])
+
     key = self._GetIOSCodeSignIdentityKey(settings)
     if not key:
-      return []
+      return postbuilds
 
     # Warn for any unimplemented signing xcode keys.
     unimpl = ['OTHER_CODE_SIGN_FLAGS']
@@ -1009,12 +1040,41 @@ class XcodeSettings(object):
       print 'Warning: Some codesign keys not implemented, ignoring: %s' % (
           ', '.join(sorted(unimpl)))
 
-    return ['%s code-sign-bundle "%s" "%s" "%s" "%s"' % (
+    if self._IsXCTest():
+      # For device xctests, Xcode copies two extra frameworks into $TEST_HOST.
+      test_host = os.path.dirname(settings.get('TEST_HOST'));
+      frameworks_dir = os.path.join(test_host, 'Frameworks')
+      platform_root = self._XcodePlatformPath(configname)
+      frameworks = \
+          ['Developer/Library/PrivateFrameworks/IDEBundleInjection.framework',
+           'Developer/Library/Frameworks/XCTest.framework']
+      for framework in frameworks:
+        source = os.path.join(platform_root, framework)
+        destination = os.path.join(frameworks_dir, os.path.basename(framework))
+        postbuilds.extend(['ditto %s %s' % (source, destination)])
+
+        # Then re-sign everything with 'preserve=True'
+        postbuilds.extend(['%s code-sign-bundle "%s" "%s" "%s" "%s" %s' % (
+            os.path.join('${TARGET_BUILD_DIR}', 'gyp-mac-tool'), key,
+            settings.get('CODE_SIGN_ENTITLEMENTS', ''),
+            settings.get('PROVISIONING_PROFILE', ''), destination, True)
+        ])
+      plugin_dir = os.path.join(test_host, 'PlugIns')
+      targets = [os.path.join(plugin_dir, product_name), test_host]
+      for target in targets:
+        postbuilds.extend(['%s code-sign-bundle "%s" "%s" "%s" "%s" %s' % (
+            os.path.join('${TARGET_BUILD_DIR}', 'gyp-mac-tool'), key,
+            settings.get('CODE_SIGN_ENTITLEMENTS', ''),
+            settings.get('PROVISIONING_PROFILE', ''), target, True)
+        ])
+
+    postbuilds.extend(['%s code-sign-bundle "%s" "%s" "%s" "%s" %s' % (
         os.path.join('${TARGET_BUILD_DIR}', 'gyp-mac-tool'), key,
-        settings.get('CODE_SIGN_RESOURCE_RULES_PATH', ''),
         settings.get('CODE_SIGN_ENTITLEMENTS', ''),
-        settings.get('PROVISIONING_PROFILE', ''))
-    ]
+        settings.get('PROVISIONING_PROFILE', ''),
+        os.path.join("${BUILT_PRODUCTS_DIR}", product_name), False)
+    ])
+    return postbuilds
 
   def _GetIOSCodeSignIdentityKey(self, settings):
     identity = settings.get('CODE_SIGN_IDENTITY')
@@ -1096,25 +1156,37 @@ class XcodeSettings(object):
       xcode, xcode_build = XcodeVersion()
       cache['DTXcode'] = xcode
       cache['DTXcodeBuild'] = xcode_build
+      compiler = self.xcode_settings[configname].get('GCC_VERSION')
+      if compiler is not None:
+        cache['DTCompiler'] = compiler
 
       sdk_root = self._SdkRoot(configname)
       if not sdk_root:
         sdk_root = self._DefaultSdkRoot()
-      cache['DTSDKName'] = sdk_root
-      if xcode >= '0430':
+      sdk_version = self._GetSdkVersionInfoItem(sdk_root, '--show-sdk-version')
+      cache['DTSDKName'] = sdk_root + (sdk_version or '')
+      if xcode >= '0720':
         cache['DTSDKBuild'] = self._GetSdkVersionInfoItem(
-            sdk_root, '--show-sdk-version')
+            sdk_root, '--show-sdk-build-version')
+      elif xcode >= '0430':
+        cache['DTSDKBuild'] = sdk_version
       else:
         cache['DTSDKBuild'] = cache['BuildMachineOSBuild']
 
       if self.isIOS:
-        cache['DTPlatformName'] = cache['DTSDKName']
+        cache['MinimumOSVersion'] = self.xcode_settings[configname].get(
+            'IPHONEOS_DEPLOYMENT_TARGET')
+        cache['DTPlatformName'] = sdk_root
+        cache['DTPlatformVersion'] = sdk_version
+
         if configname.endswith("iphoneos"):
-          cache['DTPlatformVersion'] = self._GetSdkVersionInfoItem(
-              sdk_root, '--show-sdk-version')
           cache['CFBundleSupportedPlatforms'] = ['iPhoneOS']
+          cache['DTPlatformBuild'] = cache['DTSDKBuild']
         else:
           cache['CFBundleSupportedPlatforms'] = ['iPhoneSimulator']
+          # This is weird, but Xcode sets DTPlatformBuild to an empty field
+          # for simulator builds.
+          cache['DTPlatformBuild'] = ""
       XcodeSettings._plist_cache[configname] = cache
 
     # Include extra plist items that are per-target, not per global
@@ -1357,6 +1429,7 @@ def IsMacBundle(flavor, spec):
   just a single file. Bundle rules do not produce a binary but also package
   resources into that directory."""
   is_mac_bundle = int(spec.get('mac_xctest_bundle', 0)) != 0 or \
+      int(spec.get('mac_xcuitest_bundle', 0)) != 0 or \
       (int(spec.get('mac_bundle', 0)) != 0 and flavor == 'mac')
 
   if is_mac_bundle:
@@ -1487,12 +1560,16 @@ def _GetXcodeEnv(xcode_settings, built_products_dir, srcroot, configuration,
     # written for bundles:
     'TARGET_BUILD_DIR' : built_products_dir,
     'TEMP_DIR' : '${TMPDIR}',
+    'XCODE_VERSION_ACTUAL' : XcodeVersion()[0],
   }
   if xcode_settings.GetPerConfigSetting('SDKROOT', configuration):
     env['SDKROOT'] = xcode_settings._SdkPath(configuration)
   else:
     env['SDKROOT'] = ''
 
+  if xcode_settings.mac_toolchain_dir:
+    env['DEVELOPER_DIR'] = xcode_settings.mac_toolchain_dir
+
   if spec['type'] in (
       'executable', 'static_library', 'shared_library', 'loadable_module'):
     env['EXECUTABLE_NAME'] = xcode_settings.GetExecutableName()
@@ -1635,11 +1712,12 @@ def _AddIOSDeviceConfigurations(targets):
   for target_dict in targets.itervalues():
     toolset = target_dict['toolset']
     configs = target_dict['configurations']
-    for config_name, config_dict in dict(configs).iteritems():
-      iphoneos_config_dict = copy.deepcopy(config_dict)
+    for config_name, simulator_config_dict in dict(configs).iteritems():
+      iphoneos_config_dict = copy.deepcopy(simulator_config_dict)
       configs[config_name + '-iphoneos'] = iphoneos_config_dict
-      configs[config_name + '-iphonesimulator'] = config_dict
+      configs[config_name + '-iphonesimulator'] = simulator_config_dict
       if toolset == 'target':
+        simulator_config_dict['xcode_settings']['SDKROOT'] = 'iphonesimulator'
         iphoneos_config_dict['xcode_settings']['SDKROOT'] = 'iphoneos'
   return targets
 
