@@ -78,36 +78,37 @@ namespace ui {
 
 namespace {
 
-typedef base::hash_map<LONG, AXPlatformNodeWin*> UniqueIdWinMap;
-// Map from each AXPlatformNodeWin's unique id to its instance.
-base::LazyInstance<UniqueIdWinMap> g_unique_id_win_map =
-    LAZY_INSTANCE_INITIALIZER;
-
 typedef base::hash_set<AXPlatformNodeWin*> AXPlatformNodeWinSet;
 // Set of all AXPlatformNodeWin objects that were the target of an
 // alert event.
 base::LazyInstance<AXPlatformNodeWinSet> g_alert_targets =
     LAZY_INSTANCE_INITIALIZER;
 
-LONG GetNextNegativeUniqueIdForWinAccessibility(AXPlatformNodeWin* obj) {
-  static LONG next_unique_id = -1;
-  LONG unique_id = next_unique_id;
-  if (next_unique_id == LONG_MIN)
-    next_unique_id = -1;
-  else
-    next_unique_id--;
-
-  g_unique_id_win_map.Get().insert(std::make_pair(unique_id, obj));
-
-  return unique_id;
-}
-
-void UnregisterNegativeUniqueId(LONG unique_id) {
-  g_unique_id_win_map.Get().erase(unique_id);
-}
+base::LazyInstance<base::ObserverList<IAccessible2UsageObserver>>
+    g_iaccessible2_usage_observer_list = LAZY_INSTANCE_INITIALIZER;
 
 }  // namespace
 
+//
+// IAccessible2UsageObserver
+//
+
+IAccessible2UsageObserver::IAccessible2UsageObserver() {
+}
+
+IAccessible2UsageObserver::~IAccessible2UsageObserver() {
+}
+
+// static
+base::ObserverList<IAccessible2UsageObserver>&
+    GetIAccessible2UsageObserverList() {
+  return g_iaccessible2_usage_observer_list.Get();
+}
+
+//
+// AXPlatformNode::Create
+//
+
 // static
 AXPlatformNode* AXPlatformNode::Create(AXPlatformNodeDelegate* delegate) {
   // Make sure ATL is initialized in this module.
@@ -129,25 +130,33 @@ AXPlatformNode* AXPlatformNode::FromNativeViewAccessible(
   return ax_platform_node.get();
 }
 
-AXPlatformNodeWin::AXPlatformNodeWin()
-    : unique_id_win_(GetNextNegativeUniqueIdForWinAccessibility(this)) {
+//
+// AXPlatformNodeWin
+//
+
+AXPlatformNodeWin::AXPlatformNodeWin() {
 }
 
 AXPlatformNodeWin::~AXPlatformNodeWin() {
-  CHECK(!delegate_);
+}
+
+//
+// AXPlatformNodeBase implementation.
+//
+
+void AXPlatformNodeWin::Dispose() {
+  Release();
+}
+
+void AXPlatformNodeWin::Destroy() {
+  RemoveAlertTarget();
+  AXPlatformNodeBase::Destroy();
 }
 
 //
 // AXPlatformNode implementation.
 //
 
-void AXPlatformNodeWin::Destroy() {
-  delegate_ = nullptr;
-  UnregisterNegativeUniqueId(unique_id_win_);
-  RemoveAlertTarget();
-  Release();
-}
-
 gfx::NativeViewAccessible AXPlatformNodeWin::GetNativeViewAccessible() {
   return this;
 }
@@ -157,11 +166,17 @@ void AXPlatformNodeWin::NotifyAccessibilityEvent(ui::AXEvent event_type) {
   if (!hwnd)
     return;
 
+  // Menu items fire selection events but Windows screen readers work reliably
+  // with focus events. Remap here.
+  if (event_type == ui::AX_EVENT_SELECTION &&
+      GetData().role == ui::AX_ROLE_MENU_ITEM)
+    event_type = ui::AX_EVENT_FOCUS;
+
   int native_event = MSAAEvent(event_type);
   if (native_event < EVENT_MIN)
     return;
 
-  ::NotifyWinEvent(native_event, hwnd, OBJID_CLIENT, unique_id_win_);
+  ::NotifyWinEvent(native_event, hwnd, OBJID_CLIENT, -unique_id_);
 
   // Keep track of objects that are a target of an alert event.
   if (event_type == ui::AX_EVENT_ALERT)
@@ -324,25 +339,28 @@ STDMETHODIMP AXPlatformNodeWin::get_accChild(VARIANT var_child,
     // that want to enumerate all immediate children.
     *disp_child = delegate_->ChildAtIndex(child_id - 1);
     if (!(*disp_child))
-      return E_FAIL;
+      return E_INVALIDARG;
     (*disp_child)->AddRef();
     return S_OK;
   }
 
   if (child_id >= 0)
-    return E_FAIL;
+    return E_INVALIDARG;
 
   // Negative child ids can be used to map to any descendant.
-  UniqueIdWinMap* unique_ids = g_unique_id_win_map.Pointer();
-  auto iter = unique_ids->find(child_id);
-  if (iter != unique_ids->end()) {
-    *disp_child = iter->second;
+  AXPlatformNodeWin* child = static_cast<AXPlatformNodeWin*>(
+      GetFromUniqueId(-child_id));
+  if (child && !IsDescendant(child))
+    child = nullptr;
+
+  if (child) {
+    *disp_child = child;
     (*disp_child)->AddRef();
     return S_OK;
   }
 
   *disp_child = nullptr;
-  return E_FAIL;
+  return E_INVALIDARG;
 }
 
 STDMETHODIMP AXPlatformNodeWin::get_accChildCount(LONG* child_count) {
@@ -498,7 +516,7 @@ STDMETHODIMP AXPlatformNodeWin::get_states(AccessibleStates* states) {
 
 STDMETHODIMP AXPlatformNodeWin::get_uniqueID(LONG* unique_id) {
   COM_OBJECT_VALIDATE_1_ARG(unique_id);
-  *unique_id = unique_id_win_;
+  *unique_id = -unique_id_;
   return S_OK;
 }
 
@@ -876,6 +894,13 @@ STDMETHODIMP AXPlatformNodeWin::scrollSubstringToPoint(
 STDMETHODIMP AXPlatformNodeWin::QueryService(
     REFGUID guidService, REFIID riid, void** object) {
   COM_OBJECT_VALIDATE_1_ARG(object);
+
+  if (riid == IID_IAccessible2) {
+    FOR_EACH_OBSERVER(IAccessible2UsageObserver,
+                      GetIAccessible2UsageObserverList(),
+                      OnIAccessible2Used());
+  }
+
   if (guidService == IID_IAccessible ||
       guidService == IID_IAccessible2 ||
       guidService == IID_IAccessible2_2 ||
@@ -982,8 +1007,6 @@ int AXPlatformNodeWin::MSAAState() {
     msaa_state |= STATE_SYSTEM_EXPANDED;
   if (state & (1 << ui::AX_STATE_FOCUSABLE))
     msaa_state |= STATE_SYSTEM_FOCUSABLE;
-  if (state & (1 << ui::AX_STATE_FOCUSED))
-    msaa_state |= STATE_SYSTEM_FOCUSED;
   if (state & (1 << ui::AX_STATE_HASPOPUP))
     msaa_state |= STATE_SYSTEM_HASPOPUP;
   if (state & (1 << ui::AX_STATE_HOVERED))
@@ -1007,6 +1030,21 @@ int AXPlatformNodeWin::MSAAState() {
   if (state & (1 << ui::AX_STATE_DISABLED))
     msaa_state |= STATE_SYSTEM_UNAVAILABLE;
 
+  gfx::NativeViewAccessible focus = delegate_->GetFocus();
+  if (focus == GetNativeViewAccessible())
+    msaa_state |= STATE_SYSTEM_FOCUSED;
+
+  // On Windows, the "focus" bit should be set on certain containers, like
+  // menu bars, when visible.
+  //
+  // TODO(dmazzoni): this should probably check if focus is actually inside
+  // the menu bar, but we don't currently track focus inside menu pop-ups,
+  // and Chrome only has one menu visible at a time so this works for now.
+  if (GetData().role == ui::AX_ROLE_MENU_BAR &&
+      !(state & (1 << ui::AX_STATE_INVISIBLE))) {
+    msaa_state |= STATE_SYSTEM_FOCUSED;
+  }
+
   return msaa_state;
 }
 
