@@ -7,8 +7,9 @@
 
 #include <stdint.h>
 
+#include <memory>
+
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "ui/aura/aura_export.h"
 #include "ui/aura/window_tree_host.h"
 #include "ui/events/platform/platform_event_dispatcher.h"
@@ -27,13 +28,8 @@ class MouseEvent;
 
 namespace aura {
 
-namespace internal {
-class TouchEventCalibrate;
-}
-
 class AURA_EXPORT WindowTreeHostX11 : public WindowTreeHost,
                                       public ui::PlatformEventDispatcher {
-
  public:
   explicit WindowTreeHostX11(const gfx::Rect& bounds);
   ~WindowTreeHostX11() override;
@@ -95,8 +91,6 @@ class AURA_EXPORT WindowTreeHostX11 : public WindowTreeHost,
   // The bounds of |xwindow_|.
   gfx::Rect bounds_;
 
-  scoped_ptr<internal::TouchEventCalibrate> touch_calibrate_;
-
   ui::X11AtomCache atom_cache_;
 
   DISALLOW_COPY_AND_ASSIGN(WindowTreeHostX11);
