@@ -43,8 +43,6 @@ const char kRTFFormat[] = "rtf";
 const char kBitmapFormat[] = "bitmap";
 const char kWebKitSmartPasteFormat[] = "webkit_smart";
 const char kBookmarkFormat[] = "bookmark";
-const char kMimeTypePepperCustomData[] = "chromium/x-pepper-custom-data";
-const char kMimeTypeWebCustomData[] = "chromium/x-web-custom-data";
 
 class ClipboardMap {
  public:
@@ -116,10 +114,9 @@ void ClipboardMap::CommitToAndroidClipboard() {
     ScopedJavaLocalRef<jstring> str =
         ConvertUTF8ToJavaString(env, map_[kPlainTextFormat].c_str());
     DCHECK(str.obj());
-
     Java_Clipboard_setText(env, clipboard_manager_.obj(), str.obj());
   } else {
-    Java_Clipboard_setText(env, clipboard_manager_.obj(), nullptr);
+    Java_Clipboard_clear(env, clipboard_manager_.obj());
     NOTIMPLEMENTED();
   }
 }
@@ -128,7 +125,7 @@ void ClipboardMap::Clear() {
   JNIEnv* env = AttachCurrentThread();
   base::AutoLock lock(lock_);
   map_.clear();
-  Java_Clipboard_setText(env, clipboard_manager_.obj(), NULL);
+  Java_Clipboard_clear(env, clipboard_manager_.obj());
 }
 
 // Add a key:jstr pair to map, but only if jstr is not null, and also
