@@ -7,6 +7,7 @@
 #include <algorithm>
 #include <cstdlib>
 #include <iterator>
+#include <memory>
 #include <string>
 
 #include "base/command_line.h"
@@ -14,11 +15,11 @@
 #include "base/files/file_util.h"
 #include "base/i18n/file_util_icu.h"
 #include "base/i18n/message_formatter.h"
+#include "base/i18n/number_formatting.h"
 #include "base/i18n/rtl.h"
 #include "base/i18n/string_compare.h"
 #include "base/lazy_instance.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/strings/string_number_conversions.h"
 #include "base/strings/string_split.h"
 #include "base/strings/string_util.h"
@@ -50,7 +51,9 @@ namespace {
 static const char* const kAcceptLanguageList[] = {
   "af",     // Afrikaans
   "am",     // Amharic
+  "an",     // Aragonese
   "ar",     // Arabic
+  "ast",    // Asturian
   "az",     // Azerbaijani
   "be",     // Belarusian
   "bg",     // Bulgarian
@@ -74,6 +77,7 @@ static const char* const kAcceptLanguageList[] = {
   "en-AU",  // English (Australia)
   "en-CA",  // English (Canada)
   "en-GB",  // English (UK)
+  "en-IN",  // English (India)
   "en-NZ",  // English (New Zealand)
   "en-US",  // English (US)
   "en-ZA",  // English (South Africa)
@@ -82,6 +86,10 @@ static const char* const kAcceptLanguageList[] = {
   // Spanish speaking countries?
   "es",     // Spanish
   "es-419", // Spanish (Latin America)
+  "es-AR",  // Spanish (Argentina)
+  "es-ES",  // Spanish (Spain)
+  "es-MX",  // Spanish (Mexico)
+  "es-US",  // Spanish (US)
   "et",     // Estonian
   "eu",     // Basque
   "fa",     // Persian
@@ -178,12 +186,14 @@ static const char* const kAcceptLanguageList[] = {
   "ur",     // Urdu
   "uz",     // Uzbek
   "vi",     // Vietnamese
+  "wa",     // Walloon
   "xh",     // Xhosa
   "yi",     // Yiddish
   "yo",     // Yoruba
   "zh",     // Chinese
-  "zh-CN",  // Chinese (Simplified)
-  "zh-TW",  // Chinese (Traditional)
+  "zh-CN",  // Chinese (China)
+  "zh-HK",  // Chinese (Hong Kong)
+  "zh-TW",  // Chinese (Taiwan)
   "zu",     // Zulu
 };
 
@@ -314,6 +324,8 @@ std::string GetLanguage(const std::string& locale) {
   return std::string(locale, 0, hyphen_pos);
 }
 
+// TOOD(jshin): revamp this function completely to use a more sytematic
+// and generic locale fallback based on ICU/CLDR.
 bool CheckAndResolveLocale(const std::string& locale,
                            std::string* resolved_locale) {
 #if defined(OS_MACOSX) && !defined(TOOLKIT_QT)
@@ -355,12 +367,13 @@ bool CheckAndResolveLocale(const std::string& locale,
         tmp_locale.append("-CN");
       }
     } else if (base::LowerCaseEqualsASCII(lang, "en")) {
-      // Map Australian, Canadian, New Zealand and South African English
-      // to British English for now.
+      // Map Australian, Canadian, Indian, New Zealand and South African
+      // English to British English for now.
       // TODO(jungshik): en-CA may have to change sides once
       // we have OS locale separate from app locale (Chrome's UI language).
       if (base::LowerCaseEqualsASCII(region, "au") ||
           base::LowerCaseEqualsASCII(region, "ca") ||
+          base::LowerCaseEqualsASCII(region, "in") ||
           base::LowerCaseEqualsASCII(region, "nz") ||
           base::LowerCaseEqualsASCII(region, "za")) {
         tmp_locale.append("-GB");
@@ -687,23 +700,21 @@ base::string16 GetStringFUTF16(int message_id,
   // check as the code may simply want to find the placeholders rather than
   // actually replacing them.
   if (!offsets) {
-    std::string utf8_string = base::UTF16ToUTF8(format_string);
-
     // $9 is the highest allowed placeholder.
     for (size_t i = 0; i < 9; ++i) {
       bool placeholder_should_exist = replacements.size() > i;
 
-      std::string placeholder =
-          base::StringPrintf("$%d", static_cast<int>(i + 1));
-      size_t pos = utf8_string.find(placeholder.c_str());
+      base::string16 placeholder = base::ASCIIToUTF16("$");
+      placeholder += (L'1' + i);
+      size_t pos = format_string.find(placeholder);
       if (placeholder_should_exist) {
-        DCHECK_NE(std::string::npos, pos) <<
-            " Didn't find a " << placeholder << " placeholder in " <<
-            utf8_string;
+        DCHECK_NE(std::string::npos, pos) << " Didn't find a " << placeholder
+                                          << " placeholder in "
+                                          << format_string;
       } else {
-        DCHECK_EQ(std::string::npos, pos) <<
-            " Unexpectedly found a " << placeholder << " placeholder in " <<
-            utf8_string;
+        DCHECK_EQ(std::string::npos, pos) << " Unexpectedly found a "
+                                          << placeholder << " placeholder in "
+                                          << format_string;
       }
     }
   }
@@ -818,11 +829,11 @@ base::string16 GetStringFUTF16(int message_id,
 }
 
 base::string16 GetStringFUTF16Int(int message_id, int a) {
-  return GetStringFUTF16(message_id, base::UTF8ToUTF16(base::IntToString(a)));
+  return GetStringFUTF16(message_id, base::FormatNumber(a));
 }
 
 base::string16 GetStringFUTF16Int(int message_id, int64_t a) {
-  return GetStringFUTF16(message_id, base::UTF8ToUTF16(base::Int64ToString(a)));
+  return GetStringFUTF16(message_id, base::FormatNumber(a));
 }
 
 base::string16 GetPluralStringFUTF16(int message_id, int number) {
@@ -853,10 +864,11 @@ void GetAcceptLanguagesForLocale(const std::string& display_locale,
                                  std::vector<std::string>* locale_codes) {
   for (size_t i = 0; i < arraysize(kAcceptLanguageList); ++i) {
     if (!l10n_util::IsLocaleNameTranslated(kAcceptLanguageList[i],
-                                           display_locale))
-      // TODO(jungshik) : Put them at the of the list with language codes
+                                           display_locale)) {
+      // TODO(jungshik) : Put them at the end of the list with language codes
       // enclosed by brackets instead of skipping.
-        continue;
+      continue;
+    }
     locale_codes->push_back(kAcceptLanguageList[i]);
   }
 }
