@@ -23,22 +23,27 @@
 #include "base/strings/string_piece.h"
 #include "base/strings/utf_string_conversions.h"
 #include "base/synchronization/lock.h"
+#include "base/sys_byteorder.h"
 #include "build/build_config.h"
 #include "skia/ext/image_operations.h"
 #include "third_party/skia/include/core/SkBitmap.h"
+#include "third_party/skia/include/core/SkColor.h"
+#include "third_party/zlib/zlib.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/layout.h"
+#include "ui/base/material_design/material_design_controller.h"
 #include "ui/base/resource/data_pack.h"
-#include "ui/base/resource/material_design/material_design_controller.h"
 #include "ui/base/ui_base_paths.h"
 #include "ui/base/ui_base_switches.h"
+#include "ui/base/ui_features.h"
+#include "ui/display/display.h"
+#include "ui/display/screen.h"
 #include "ui/gfx/codec/jpeg_codec.h"
 #include "ui/gfx/codec/png_codec.h"
 #include "ui/gfx/geometry/safe_integer_conversions.h"
 #include "ui/gfx/geometry/size_conversions.h"
 #include "ui/gfx/image/image_skia.h"
 #include "ui/gfx/image/image_skia_source.h"
-#include "ui/gfx/screen.h"
 #include "ui/strings/grit/app_locale_settings.h"
 
 #if defined(OS_ANDROID)
@@ -51,22 +56,13 @@
 #endif
 
 #if defined(OS_WIN)
-#include "ui/gfx/win/dpi.h"
-#endif
-
-#if defined(OS_MACOSX) && !defined(OS_IOS)
-#include "base/mac/mac_util.h"
+#include "ui/display/win/dpi.h"
 #endif
 
 namespace ui {
 
 namespace {
 
-// Font sizes relative to base font.
-const int kSmallFontSizeDelta = -1;
-const int kMediumFontSizeDelta = 3;
-const int kLargeFontSizeDelta = 8;
-
 // PNG-related constants.
 const unsigned char kPngMagic[8] = { 0x89, 'P', 'N', 'G', 13, 10, 26, 10 };
 const size_t kPngChunkMetadataSize = 12;  // length, type, crc32
@@ -111,6 +107,46 @@ base::FilePath GetResourcesPakFilePath(const std::string& pak_name) {
 #endif  // OS_WIN
 }
 
+SkBitmap CreateEmptyBitmap() {
+  SkBitmap bitmap;
+  bitmap.allocN32Pixels(32, 32);
+  bitmap.eraseARGB(255, 255, 255, 0);
+  return bitmap;
+}
+
+// Decodes given gzip input via zlib.
+base::RefCountedBytes* DecodeGzipData(const unsigned char* input_buffer,
+                                      size_t input_size) {
+  z_stream inflateStream;
+  memset(&inflateStream, 0, sizeof(inflateStream));
+  inflateStream.zalloc = Z_NULL;
+  inflateStream.zfree = Z_NULL;
+  inflateStream.opaque = Z_NULL;
+
+  inflateStream.avail_in = input_size;
+  inflateStream.next_in = const_cast<Bytef*>(input_buffer);
+
+  CHECK(input_size >= 4);
+  // Size of output comes from footer of gzip file format, found as the last 4
+  // bytes in the compressed file, which are stored little endian.
+  inflateStream.avail_out = base::ByteSwapToLE32(
+      *reinterpret_cast<const uint32_t*>(&input_buffer[input_size - 4]));
+
+  std::vector<unsigned char> output(inflateStream.avail_out);
+  inflateStream.next_out = reinterpret_cast<Bytef*>(&output[0]);
+
+  CHECK(inflateInit2(&inflateStream, 16) == Z_OK);
+  CHECK(inflate(&inflateStream, Z_FINISH) == Z_STREAM_END);
+  CHECK(inflateEnd(&inflateStream) == Z_OK);
+
+  // Cannot use TakeVector since it puts the RefCounted* into a scoped_refptr,
+  // and callers of this function return a raw pointer (not a scoped_refptr), so
+  // the memory will be deallocated upon exit of the calling function.
+  base::RefCountedBytes* returnVal = new base::RefCountedBytes();
+  returnVal->data().swap(output);
+  return returnVal;
+}
+
 }  // namespace
 
 // An ImageSkiaSource that loads bitmaps for the requested scale factor from
@@ -133,8 +169,16 @@ class ResourceBundle::ResourceBundleImageSource : public gfx::ImageSkiaSource {
     ScaleFactor scale_factor = GetSupportedScaleFactor(scale);
     bool found = rb_->LoadBitmap(resource_id_, &scale_factor,
                                  &image, &fell_back_to_1x);
-    if (!found)
+    if (!found) {
+#if defined(OS_ANDROID)
+      // TODO(oshima): Android unit_tests runs at DSF=3 with 100P assets.
       return gfx::ImageSkiaRep();
+#else
+      NOTREACHED() << "Unable to load image with id " << resource_id_
+                   << ", scale=" << scale;
+      return gfx::ImageSkiaRep(CreateEmptyBitmap(), scale);
+#endif
+    }
 
     // If the resource is in the package with SCALE_FACTOR_NONE, it
     // can be used in any scale factor. The image is maked as "unscaled"
@@ -162,6 +206,29 @@ class ResourceBundle::ResourceBundleImageSource : public gfx::ImageSkiaSource {
   DISALLOW_COPY_AND_ASSIGN(ResourceBundleImageSource);
 };
 
+struct ResourceBundle::FontKey {
+  FontKey(int in_size_delta,
+          gfx::Font::FontStyle in_style,
+          gfx::Font::Weight in_weight)
+      : size_delta(in_size_delta), style(in_style), weight(in_weight) {}
+
+  ~FontKey() {}
+
+  bool operator==(const FontKey& rhs) const {
+    return std::tie(size_delta, style, weight) ==
+           std::tie(rhs.size_delta, rhs.style, rhs.weight);
+  }
+
+  bool operator<(const FontKey& rhs) const {
+    return std::tie(size_delta, style, weight) <
+           std::tie(rhs.size_delta, rhs.style, rhs.weight);
+  }
+
+  int size_delta;
+  gfx::Font::FontStyle style;
+  gfx::Font::Weight weight;
+};
+
 // static
 std::string ResourceBundle::InitSharedInstanceWithLocale(
     const std::string& pref_locale,
@@ -180,7 +247,7 @@ void ResourceBundle::InitSharedInstanceWithPakFileRegion(
     base::File pak_file,
     const base::MemoryMappedFile::Region& region) {
   InitSharedInstance(NULL);
-  scoped_ptr<DataPack> data_pack(new DataPack(SCALE_FACTOR_100P));
+  std::unique_ptr<DataPack> data_pack(new DataPack(SCALE_FACTOR_100P));
   if (!data_pack->LoadFromFileRegion(std::move(pak_file), region)) {
     NOTREACHED() << "failed to load pak file";
     return;
@@ -217,7 +284,7 @@ ResourceBundle& ResourceBundle::GetSharedInstance() {
   return *g_shared_instance_;
 }
 
-#if !defined(OS_ANDROID)
+#if !defined(OS_ANDROID) && !defined(TOOLKIT_QT)
 bool ResourceBundle::LocaleDataPakExists(const std::string& locale) {
   return !GetLocaleFilePath(locale, true).empty();
 }
@@ -256,8 +323,7 @@ void ResourceBundle::AddDataPackFromFileRegion(
     base::File file,
     const base::MemoryMappedFile::Region& region,
     ScaleFactor scale_factor) {
-  scoped_ptr<DataPack> data_pack(
-      new DataPack(scale_factor));
+  std::unique_ptr<DataPack> data_pack(new DataPack(scale_factor));
   if (data_pack->LoadFromFileRegion(std::move(file), region)) {
     AddDataPack(data_pack.release());
   } else {
@@ -297,7 +363,7 @@ base::FilePath ResourceBundle::GetLocaleFilePath(const std::string& app_locale,
 }
 #endif
 
-#if !defined(OS_ANDROID)
+#if !defined(OS_ANDROID) && !defined(TOOLKIT_QT)
 std::string ResourceBundle::LoadLocaleResources(
     const std::string& pref_locale) {
   DCHECK(!locale_resources_data_.get()) << "locale.pak already loaded";
@@ -312,8 +378,7 @@ std::string ResourceBundle::LoadLocaleResources(
     return std::string();
   }
 
-  scoped_ptr<DataPack> data_pack(
-      new DataPack(SCALE_FACTOR_100P));
+  std::unique_ptr<DataPack> data_pack(new DataPack(SCALE_FACTOR_100P));
   if (!data_pack->LoadFromPath(locale_file_path)) {
     UMA_HISTOGRAM_ENUMERATION("ResourceBundle.LoadLocaleResourcesError",
                               logging::GetLastSystemErrorCode(), 16000);
@@ -329,10 +394,11 @@ std::string ResourceBundle::LoadLocaleResources(
 
 void ResourceBundle::LoadTestResources(const base::FilePath& path,
                                        const base::FilePath& locale_path) {
+  is_test_resources_ = true;
   DCHECK(!ui::GetSupportedScaleFactors().empty());
   const ScaleFactor scale_factor(ui::GetSupportedScaleFactors()[0]);
   // Use the given resource pak for both common and localized resources.
-  scoped_ptr<DataPack> data_pack(new DataPack(scale_factor));
+  std::unique_ptr<DataPack> data_pack(new DataPack(scale_factor));
   if (!path.empty() && data_pack->LoadFromPath(path))
     AddDataPack(data_pack.release());
 
@@ -394,12 +460,16 @@ gfx::Image& ResourceBundle::GetImageNamed(int resource_id) {
     DCHECK(!data_packs_.empty()) <<
         "Missing call to SetResourcesDataDLL?";
 
-#if defined(OS_CHROMEOS) || defined(OS_WIN)
-  ui::ScaleFactor scale_factor_to_load = GetMaxScaleFactor();
+#if defined(OS_CHROMEOS)
+    ui::ScaleFactor scale_factor_to_load = GetMaxScaleFactor();
+#elif defined(OS_WIN)
+    ui::ScaleFactor scale_factor_to_load =
+        display::win::GetDPIScale() > 1.25
+            ? GetMaxScaleFactor()
+            : ui::SCALE_FACTOR_100P;
 #else
-  ui::ScaleFactor scale_factor_to_load = ui::SCALE_FACTOR_100P;
+    ui::ScaleFactor scale_factor_to_load = ui::SCALE_FACTOR_100P;
 #endif
-
     // TODO(oshima): Consider reading the image size from png IHDR chunk and
     // skip decoding here and remove #ifdef below.
     // ResourceBundle::GetSharedInstance() is destroyed after the
@@ -428,27 +498,30 @@ gfx::Image& ResourceBundle::GetImageNamed(int resource_id) {
   return images_[resource_id];
 }
 
-gfx::Image& ResourceBundle::GetNativeImageNamed(int resource_id) {
-  return GetNativeImageNamed(resource_id, RTL_DISABLED);
-}
-
-base::RefCountedStaticMemory* ResourceBundle::LoadDataResourceBytes(
+base::RefCountedMemory* ResourceBundle::LoadDataResourceBytes(
     int resource_id) const {
   return LoadDataResourceBytesForScale(resource_id, ui::SCALE_FACTOR_NONE);
 }
 
-base::RefCountedStaticMemory* ResourceBundle::LoadDataResourceBytesForScale(
+base::RefCountedMemory* ResourceBundle::LoadDataResourceBytesForScale(
     int resource_id,
     ScaleFactor scale_factor) const {
-  base::RefCountedStaticMemory* bytes = NULL;
+  base::RefCountedMemory* bytes = NULL;
   if (delegate_)
     bytes = delegate_->LoadDataResourceBytes(resource_id, scale_factor);
 
   if (!bytes) {
     base::StringPiece data =
-        GetRawDataResourceForScale(resource_id, scale_factor);
+        GetRawDataResourceForScaleImpl(resource_id, scale_factor);
     if (!data.empty()) {
-      bytes = new base::RefCountedStaticMemory(data.data(), data.length());
+      if (data.starts_with(CUSTOM_GZIP_HEADER)) {
+        // Jump past special identification byte prepended to header
+        const unsigned char* gzip_start =
+            reinterpret_cast<const unsigned char*>(data.data()) + 1;
+        bytes = DecodeGzipData(gzip_start, data.length() - 1);
+      } else {
+        bytes = new base::RefCountedStaticMemory(data.data(), data.length());
+      }
     }
   }
 
@@ -462,31 +535,13 @@ base::StringPiece ResourceBundle::GetRawDataResource(int resource_id) const {
 base::StringPiece ResourceBundle::GetRawDataResourceForScale(
     int resource_id,
     ScaleFactor scale_factor) const {
-  base::StringPiece data;
-  if (delegate_ &&
-      delegate_->GetRawDataResource(resource_id, scale_factor, &data))
-    return data;
+  base::StringPiece data =
+      GetRawDataResourceForScaleImpl(resource_id, scale_factor);
 
-  if (scale_factor != ui::SCALE_FACTOR_100P) {
-    for (size_t i = 0; i < data_packs_.size(); i++) {
-      if (data_packs_[i]->GetScaleFactor() == scale_factor &&
-          data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
-                                         &data))
-        return data;
-    }
-  }
+  // Do not allow this function to retrieve gzip compressed resources.
+  CHECK(!data.starts_with(CUSTOM_GZIP_HEADER));
 
-  for (size_t i = 0; i < data_packs_.size(); i++) {
-    if ((data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_200P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_300P ||
-         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE) &&
-        data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
-                                       &data))
-      return data;
-  }
-
-  return base::StringPiece();
+  return data;
 }
 
 base::string16 ResourceBundle::GetLocalizedString(int message_id) {
@@ -539,29 +594,75 @@ base::string16 ResourceBundle::GetLocalizedString(int message_id) {
   return msg;
 }
 
-const gfx::FontList& ResourceBundle::GetFontList(FontStyle style) {
-  {
-    base::AutoLock lock_scope(*images_and_fonts_lock_);
-    LoadFontsIfNecessary();
-  }
-  switch (style) {
-    case BoldFont:
-      return *bold_font_list_;
+const gfx::FontList& ResourceBundle::GetFontListWithDelta(
+    int size_delta,
+    gfx::Font::FontStyle style,
+    gfx::Font::Weight weight) {
+  base::AutoLock lock_scope(*images_and_fonts_lock_);
+
+  const FontKey styled_key(size_delta, style, weight);
+
+  auto found = font_cache_.find(styled_key);
+  if (found != font_cache_.end())
+    return found->second;
+
+  const FontKey base_key(0, gfx::Font::NORMAL, gfx::Font::Weight::NORMAL);
+  gfx::FontList& base = font_cache_[base_key];
+  if (styled_key == base_key)
+    return base;
+
+  // Fonts of a given style are derived from the unstyled font of the same size.
+  // Cache the unstyled font by first inserting a default-constructed font list.
+  // Then, derive it for the initial insertion, or use the iterator that points
+  // to the existing entry that the insertion collided with.
+  const FontKey sized_key(size_delta, gfx::Font::NORMAL,
+                          gfx::Font::Weight::NORMAL);
+  auto sized = font_cache_.insert(std::make_pair(sized_key, gfx::FontList()));
+  if (sized.second)
+    sized.first->second = base.DeriveWithSizeDelta(size_delta);
+  if (styled_key == sized_key)
+    return sized.first->second;
+
+  auto styled = font_cache_.insert(std::make_pair(styled_key, gfx::FontList()));
+  DCHECK(styled.second);  // Otherwise font_cache_.find(..) would have found it.
+  styled.first->second = sized.first->second.Derive(
+      0, sized.first->second.GetFontStyle() | style, weight);
+
+  return styled.first->second;
+}
+
+const gfx::Font& ResourceBundle::GetFontWithDelta(int size_delta,
+                                                  gfx::Font::FontStyle style,
+                                                  gfx::Font::Weight weight) {
+  return GetFontListWithDelta(size_delta, style, weight).GetPrimaryFont();
+}
+
+const gfx::FontList& ResourceBundle::GetFontList(FontStyle legacy_style) {
+  gfx::Font::Weight font_weight = gfx::Font::Weight::NORMAL;
+  if (legacy_style == BoldFont || legacy_style == SmallBoldFont ||
+      legacy_style == MediumBoldFont || legacy_style == LargeBoldFont)
+    font_weight = gfx::Font::Weight::BOLD;
+
+  int size_delta = 0;
+  switch (legacy_style) {
     case SmallFont:
-      return *small_font_list_;
-    case MediumFont:
-      return *medium_font_list_;
     case SmallBoldFont:
-      return *small_bold_font_list_;
+      size_delta = kSmallFontDelta;
+      break;
+    case MediumFont:
     case MediumBoldFont:
-      return *medium_bold_font_list_;
+      size_delta = kMediumFontDelta;
+      break;
     case LargeFont:
-      return *large_font_list_;
     case LargeBoldFont:
-      return *large_bold_font_list_;
-    default:
-      return *base_font_list_;
+      size_delta = kLargeFontDelta;
+      break;
+    case BaseFont:
+    case BoldFont:
+      break;
   }
+
+  return GetFontListWithDelta(size_delta, gfx::Font::NORMAL, font_weight);
 }
 
 const gfx::Font& ResourceBundle::GetFont(FontStyle style) {
@@ -571,12 +672,11 @@ const gfx::Font& ResourceBundle::GetFont(FontStyle style) {
 void ResourceBundle::ReloadFonts() {
   base::AutoLock lock_scope(*images_and_fonts_lock_);
   InitDefaultFontList();
-  base_font_list_.reset();
-  LoadFontsIfNecessary();
+  font_cache_.clear();
 }
 
 ScaleFactor ResourceBundle::GetMaxScaleFactor() const {
-#if defined(OS_CHROMEOS) || defined(OS_WIN)
+#if defined(OS_CHROMEOS) || defined(OS_WIN) || defined(OS_LINUX)
   return max_scale_factor_;
 #else
   return GetSupportedScaleFactors().back();
@@ -608,15 +708,15 @@ void ResourceBundle::InitSharedInstance(Delegate* delegate) {
   DCHECK(g_shared_instance_ == NULL) << "ResourceBundle initialized twice";
   g_shared_instance_ = new ResourceBundle(delegate);
   static std::vector<ScaleFactor> supported_scale_factors;
-#if !defined(OS_IOS) && !defined(OS_WIN)
+#if !defined(OS_IOS)
   // On platforms other than iOS, 100P is always a supported scale factor.
   // For Windows we have a separate case in this function.
   supported_scale_factors.push_back(SCALE_FACTOR_100P);
 #endif
 #if defined(OS_ANDROID)
   float display_density;
-  if (gfx::Display::HasForceDeviceScaleFactor()) {
-    display_density = gfx::Display::GetForcedDeviceScaleFactor();
+  if (display::Display::HasForceDeviceScaleFactor()) {
+    display_density = display::Display::GetForcedDeviceScaleFactor();
   } else {
     gfx::DeviceDisplayInfo device_info;
     display_density = device_info.GetDIPScale();
@@ -625,7 +725,7 @@ void ResourceBundle::InitSharedInstance(Delegate* delegate) {
   if (closest != SCALE_FACTOR_100P)
     supported_scale_factors.push_back(closest);
 #elif defined(OS_IOS)
-    gfx::Display display = gfx::Screen::GetNativeScreen()->GetPrimaryDisplay();
+  display::Display display = display::Screen::GetScreen()->GetPrimaryDisplay();
   if (display.device_scale_factor() > 2.0) {
     DCHECK_EQ(3.0, display.device_scale_factor());
     supported_scale_factors.push_back(SCALE_FACTOR_300P);
@@ -635,25 +735,9 @@ void ResourceBundle::InitSharedInstance(Delegate* delegate) {
   } else {
     supported_scale_factors.push_back(SCALE_FACTOR_100P);
   }
-#elif defined(OS_MACOSX)
-  if (base::mac::IsOSLionOrLater())
-    supported_scale_factors.push_back(SCALE_FACTOR_200P);
-#elif defined(OS_CHROMEOS)
-  // TODO(oshima): Include 200P only if the device support 200P
+#elif defined(OS_MACOSX) || defined(OS_CHROMEOS) || defined(OS_LINUX) || \
+    defined(OS_WIN)
   supported_scale_factors.push_back(SCALE_FACTOR_200P);
-#elif defined(OS_LINUX) && defined(ENABLE_HIDPI)
-  supported_scale_factors.push_back(SCALE_FACTOR_200P);
-#elif defined(OS_WIN)
-  bool default_to_100P = true;
-  // On Windows if the dpi scale is greater than 1.25 on high dpi machines
-  // downscaling from 200 percent looks better than scaling up from 100
-  // percent.
-  if (gfx::GetDPIScale() > 1.25) {
-    supported_scale_factors.push_back(SCALE_FACTOR_200P);
-    default_to_100P = false;
-  }
-  if (default_to_100P)
-    supported_scale_factors.push_back(SCALE_FACTOR_100P);
 #endif
   ui::SetSupportedScaleFactors(supported_scale_factors);
 }
@@ -663,6 +747,9 @@ void ResourceBundle::FreeImages() {
 }
 
 void ResourceBundle::LoadChromeResources() {
+// TODO(estade): remove material design specific resources.
+// See crbug.com/613593
+#if defined(OS_MACOSX)
   // The material design data packs contain some of the same asset IDs as in
   // the non-material design data packs. Add these to the ResourceBundle
   // first so that they are searched first when a request for an asset is
@@ -680,6 +767,7 @@ void ResourceBundle::LoadChromeResources() {
           SCALE_FACTOR_200P);
     }
   }
+#endif
 
   // Always load the 1x data pack first as the 2x data pack contains both 1x and
   // 2x images. The 1x data pack only has 1x images, thus passes in an accurate
@@ -712,7 +800,7 @@ void ResourceBundle::AddDataPackFromPathInternal(
   if (pack_path.empty() || !pack_path.IsAbsolute())
     return;
 
-  scoped_ptr<DataPack> data_pack(new DataPack(scale_factor));
+  std::unique_ptr<DataPack> data_pack(new DataPack(scale_factor));
   data_pack->set_has_only_material_design_assets(has_only_material_assets);
   if (data_pack->LoadFromPath(pack_path)) {
     AddDataPack(data_pack.release());
@@ -749,76 +837,6 @@ void ResourceBundle::InitDefaultFontList() {
 #endif
 }
 
-void ResourceBundle::LoadFontsIfNecessary() {
-  images_and_fonts_lock_->AssertAcquired();
-  if (!base_font_list_.get()) {
-    if (delegate_) {
-      base_font_list_ = GetFontListFromDelegate(BaseFont);
-      bold_font_list_ = GetFontListFromDelegate(BoldFont);
-      small_font_list_ = GetFontListFromDelegate(SmallFont);
-      small_bold_font_list_ = GetFontListFromDelegate(SmallBoldFont);
-      medium_font_list_ = GetFontListFromDelegate(MediumFont);
-      medium_bold_font_list_ = GetFontListFromDelegate(MediumBoldFont);
-      large_font_list_ = GetFontListFromDelegate(LargeFont);
-      large_bold_font_list_ = GetFontListFromDelegate(LargeBoldFont);
-    }
-
-    if (!base_font_list_.get())
-      base_font_list_.reset(new gfx::FontList());
-
-    if (!bold_font_list_.get()) {
-      bold_font_list_.reset(new gfx::FontList());
-      *bold_font_list_ = base_font_list_->DeriveWithStyle(
-          base_font_list_->GetFontStyle() | gfx::Font::BOLD);
-    }
-
-    if (!small_font_list_.get()) {
-      small_font_list_.reset(new gfx::FontList());
-      *small_font_list_ =
-          base_font_list_->DeriveWithSizeDelta(kSmallFontSizeDelta);
-    }
-
-    if (!small_bold_font_list_.get()) {
-      small_bold_font_list_.reset(new gfx::FontList());
-      *small_bold_font_list_ = small_font_list_->DeriveWithStyle(
-          small_font_list_->GetFontStyle() | gfx::Font::BOLD);
-    }
-
-    if (!medium_font_list_.get()) {
-      medium_font_list_.reset(new gfx::FontList());
-      *medium_font_list_ =
-          base_font_list_->DeriveWithSizeDelta(kMediumFontSizeDelta);
-    }
-
-    if (!medium_bold_font_list_.get()) {
-      medium_bold_font_list_.reset(new gfx::FontList());
-      *medium_bold_font_list_ = medium_font_list_->DeriveWithStyle(
-          medium_font_list_->GetFontStyle() | gfx::Font::BOLD);
-    }
-
-    if (!large_font_list_.get()) {
-      large_font_list_.reset(new gfx::FontList());
-      *large_font_list_ =
-          base_font_list_->DeriveWithSizeDelta(kLargeFontSizeDelta);
-    }
-
-    if (!large_bold_font_list_.get()) {
-      large_bold_font_list_.reset(new gfx::FontList());
-      *large_bold_font_list_ = large_font_list_->DeriveWithStyle(
-          large_font_list_->GetFontStyle() | gfx::Font::BOLD);
-    }
-  }
-}
-
-scoped_ptr<gfx::FontList> ResourceBundle::GetFontListFromDelegate(
-    FontStyle style) {
-  DCHECK(delegate_);
-  scoped_ptr<gfx::Font> font = delegate_->GetFont(style);
-  if (font.get())
-    return make_scoped_ptr(new gfx::FontList(*font));
-  return nullptr;
-}
-
 bool ResourceBundle::LoadBitmap(const ResourceHandle& data_handle,
                                 int resource_id,
                                 SkBitmap* bitmap,
@@ -835,7 +853,7 @@ bool ResourceBundle::LoadBitmap(const ResourceHandle& data_handle,
 #if !defined(OS_IOS)
   // iOS does not compile or use the JPEG codec.  On other platforms,
   // 99% of our assets are PNGs, however fallback to JPEG.
-  scoped_ptr<SkBitmap> jpeg_bitmap(
+  std::unique_ptr<SkBitmap> jpeg_bitmap(
       gfx::JPEGCodec::Decode(memory->front(), memory->size()));
   if (jpeg_bitmap.get()) {
     bitmap->swap(*jpeg_bitmap.get());
@@ -853,6 +871,7 @@ bool ResourceBundle::LoadBitmap(int resource_id,
                                 SkBitmap* bitmap,
                                 bool* fell_back_to_1x) const {
   DCHECK(fell_back_to_1x);
+  ResourceHandle* data_handle_100_percent = nullptr;
   for (size_t i = 0; i < data_packs_.size(); ++i) {
     if (data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE &&
         LoadBitmap(*data_packs_[i], resource_id, bitmap, fell_back_to_1x)) {
@@ -860,22 +879,64 @@ bool ResourceBundle::LoadBitmap(int resource_id,
       *scale_factor = ui::SCALE_FACTOR_NONE;
       return true;
     }
+    if (data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P)
+      data_handle_100_percent = data_packs_[i];
+
     if (data_packs_[i]->GetScaleFactor() == *scale_factor &&
         LoadBitmap(*data_packs_[i], resource_id, bitmap, fell_back_to_1x)) {
       return true;
     }
   }
+
+  // Unit tests may only have 1x data pack. Allow them to fallback to 1x
+  // resources.
+  if (*scale_factor != ui::SCALE_FACTOR_100P && is_test_resources_ &&
+      data_handle_100_percent &&
+      LoadBitmap(*data_handle_100_percent, resource_id, bitmap,
+                 fell_back_to_1x)) {
+    *fell_back_to_1x = true;
+    return true;
+  }
+
   return false;
 }
 
+base::StringPiece ResourceBundle::GetRawDataResourceForScaleImpl(
+    int resource_id,
+    ScaleFactor scale_factor) const {
+  base::StringPiece data;
+  if (delegate_ &&
+      delegate_->GetRawDataResource(resource_id, scale_factor, &data))
+    return data;
+
+  if (scale_factor != ui::SCALE_FACTOR_100P) {
+    for (size_t i = 0; i < data_packs_.size(); i++) {
+      if (data_packs_[i]->GetScaleFactor() == scale_factor &&
+          data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
+                                         &data))
+        return data;
+    }
+  }
+
+  for (size_t i = 0; i < data_packs_.size(); i++) {
+    if ((data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_100P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_200P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_300P ||
+         data_packs_[i]->GetScaleFactor() == ui::SCALE_FACTOR_NONE) &&
+        data_packs_[i]->GetStringPiece(static_cast<uint16_t>(resource_id),
+                                       &data))
+      return data;
+  }
+
+  return base::StringPiece();
+}
+
 gfx::Image& ResourceBundle::GetEmptyImage() {
   base::AutoLock lock(*images_and_fonts_lock_);
 
   if (empty_image_.IsEmpty()) {
     // The placeholder bitmap is bright red so people notice the problem.
-    SkBitmap bitmap;
-    bitmap.allocN32Pixels(32, 32);
-    bitmap.eraseARGB(255, 255, 0, 0);
+    SkBitmap bitmap = CreateEmptyBitmap();
     empty_image_ = gfx::Image::CreateFrom1xBitmap(bitmap);
   }
   return empty_image_;
