@@ -8,6 +8,7 @@
 #include <stddef.h>
 
 #include <map>
+#include <memory>
 #include <string>
 
 #include "base/containers/hash_tables.h"
@@ -15,7 +16,6 @@
 #include "base/files/memory_mapped_file.h"
 #include "base/gtest_prod_util.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/scoped_vector.h"
 #include "base/strings/string16.h"
 #include "base/strings/string_piece.h"
@@ -31,7 +31,7 @@ class SkBitmap;
 namespace base {
 class File;
 class Lock;
-class RefCountedStaticMemory;
+class RefCountedMemory;
 }
 
 namespace ui {
@@ -43,12 +43,18 @@ class ResourceHandle;
 // such as theme graphics. Every resource is loaded only once.
 class UI_BASE_EXPORT ResourceBundle {
  public:
-  // An enumeration of the various font styles used throughout Chrome.
-  // The following holds true for the font sizes:
-  // Small <= SmallBold <= Base <= Bold <= Medium <= MediumBold <= Large.
+  // Legacy font size deltas. Consider these to be magic numbers. New code
+  // should declare their own size delta constant using an identifier that
+  // imparts some semantic meaning.
+  static const int kSmallFontDelta = -1;
+  static const int kMediumFontDelta = 3;
+  static const int kLargeFontDelta = 8;
+
+  static constexpr const char* CUSTOM_GZIP_HEADER = "\xff\x1f\x8b";
+
+  // Legacy font style mappings. TODO(tapted): Phase these out in favour of
+  // client code providing their own constant with the desired font size delta.
   enum FontStyle {
-    // NOTE: depending upon the locale, using one of the *BoldFont below
-    // may *not* actually result in a bold font.
     SmallFont,
     SmallBoldFont,
     BaseFont,
@@ -59,13 +65,6 @@ class UI_BASE_EXPORT ResourceBundle {
     LargeBoldFont,
   };
 
-  enum ImageRTL {
-    // Images are flipped in RTL locales.
-    RTL_ENABLED,
-    // Images are never flipped.
-    RTL_DISABLED,
-  };
-
   enum LoadResources {
     LOAD_COMMON_RESOURCES,
     DO_NOT_LOAD_COMMON_RESOURCES
@@ -97,11 +96,11 @@ class UI_BASE_EXPORT ResourceBundle {
 
     // Return an image resource or an empty value to attempt retrieval of the
     // default resource.
-    virtual gfx::Image GetNativeImageNamed(int resource_id, ImageRTL rtl) = 0;
+    virtual gfx::Image GetNativeImageNamed(int resource_id) = 0;
 
-    // Return a static memory resource or NULL to attempt retrieval of the
+    // Return a ref counted memory resource or NULL to attempt retrieval of the
     // default resource.
-    virtual base::RefCountedStaticMemory* LoadDataResourceBytes(
+    virtual base::RefCountedMemory* LoadDataResourceBytes(
         int resource_id,
         ScaleFactor scale_factor) = 0;
 
@@ -115,9 +114,6 @@ class UI_BASE_EXPORT ResourceBundle {
     // false to attempt retrieval of the default string.
     virtual bool GetLocalizedString(int message_id, base::string16* value) = 0;
 
-    // Returns a font or NULL to attempt retrieval of the default resource.
-    virtual scoped_ptr<gfx::Font> GetFont(FontStyle style) = 0;
-
    protected:
     virtual ~Delegate() {}
   };
@@ -219,27 +215,22 @@ class UI_BASE_EXPORT ResourceBundle {
   // Note that if the same resource has already been loaded in GetImageNamed(),
   // gfx::Image will perform a conversion, rather than using the native image
   // loading code of ResourceBundle.
-  //
-  // If |rtl| is RTL_ENABLED then the image is flipped in RTL locales.
-  gfx::Image& GetNativeImageNamed(int resource_id, ImageRTL rtl);
-
-  // Same as GetNativeImageNamed() except that RTL is not enabled.
   gfx::Image& GetNativeImageNamed(int resource_id);
 
   // Loads the raw bytes of a scale independent data resource.
-  base::RefCountedStaticMemory* LoadDataResourceBytes(int resource_id) const;
+  base::RefCountedMemory* LoadDataResourceBytes(int resource_id) const;
 
   // Loads the raw bytes of a data resource nearest the scale factor
   // |scale_factor| into |bytes|, without doing any processing or
   // interpretation of the resource. Use ResourceHandle::SCALE_FACTOR_NONE
   // for scale independent image resources (such as wallpaper).
   // Returns NULL if we fail to read the resource.
-  base::RefCountedStaticMemory* LoadDataResourceBytesForScale(
+  base::RefCountedMemory* LoadDataResourceBytesForScale(
       int resource_id,
       ScaleFactor scale_factor) const;
 
   // Return the contents of a scale independent resource in a
-  // StringPiece given the resource id
+  // StringPiece given the resource id.
   base::StringPiece GetRawDataResource(int resource_id) const;
 
   // Return the contents of a resource in a StringPiece given the resource id
@@ -253,10 +244,21 @@ class UI_BASE_EXPORT ResourceBundle {
   // string if the message_id is not found.
   base::string16 GetLocalizedString(int message_id);
 
-  // Returns the font list for the specified style.
-  const gfx::FontList& GetFontList(FontStyle style);
+  // Returns a font list derived from the platform-specific "Base" font list.
+  // The result is always cached and exists for the lifetime of the process.
+  const gfx::FontList& GetFontListWithDelta(
+      int size_delta,
+      gfx::Font::FontStyle style = gfx::Font::NORMAL,
+      gfx::Font::Weight weight = gfx::Font::Weight::NORMAL);
 
-  // Returns the font for the specified style.
+  // Returns the primary font from the FontList given by GetFontListWithDelta().
+  const gfx::Font& GetFontWithDelta(
+      int size_delta,
+      gfx::Font::FontStyle style = gfx::Font::NORMAL,
+      gfx::Font::Weight weight = gfx::Font::Weight::NORMAL);
+
+  // Deprecated. Returns fonts using hard-coded size deltas implied by |style|.
+  const gfx::FontList& GetFontList(FontStyle style);
   const gfx::Font& GetFont(FontStyle style);
 
   // Resets and reloads the cached fonts.  This is useful when the fonts of the
@@ -287,6 +289,12 @@ class UI_BASE_EXPORT ResourceBundle {
   // Returns SCALE_FACTOR_100P if no resource is loaded.
   ScaleFactor GetMaxScaleFactor() const;
 
+#if defined(OS_MACOSX)
+  // Loads Material Design data packs and makes them the first items in
+  // |data_packs_|.
+  void LoadMaterialDesignResources();
+#endif
+
  protected:
   // Returns true if |scale_factor| is supported by this platform.
   static bool IsScaleFactorSupported(ScaleFactor scale_factor);
@@ -299,14 +307,19 @@ class UI_BASE_EXPORT ResourceBundle {
                            CountMaterialDesignDataPacksInResourceBundle);
   FRIEND_TEST_ALL_PREFIXES(ResourceBundleMacImageTest,
                            CheckImageFromMaterialDesign);
+  FRIEND_TEST_ALL_PREFIXES(ChromeBrowserMainMacBrowserTest,
+                           MDResourceAccess);
 
   friend class ResourceBundleMacImageTest;
   friend class ResourceBundleImageTest;
   friend class ResourceBundleTest;
+  friend class ChromeBrowserMainMacBrowserTest;
 
   class ResourceBundleImageSource;
   friend class ResourceBundleImageSource;
 
+  struct FontKey;
+
   typedef base::hash_map<int, base::string16> IdToStringMap;
 
   // Ctor/dtor are private, since we're a singleton.
@@ -356,14 +369,6 @@ class UI_BASE_EXPORT ResourceBundle {
   // Initializes the font description of default gfx::FontList.
   void InitDefaultFontList();
 
-  // Initializes all the gfx::FontList members if they haven't yet been
-  // initialized.
-  void LoadFontsIfNecessary();
-
-  // Returns a FontList or NULL by calling Delegate::GetFont and converting
-  // scoped_ptr<gfx::Font> to scoped_ptr<gfx::FontList>.
-  scoped_ptr<gfx::FontList> GetFontListFromDelegate(FontStyle style);
-
   // Fills the |bitmap| given the data file to look in and the |resource_id|.
   // Returns false if the resource does not exist.
   //
@@ -383,6 +388,15 @@ class UI_BASE_EXPORT ResourceBundle {
                   SkBitmap* bitmap,
                   bool* fell_back_to_1x) const;
 
+  // Loads the raw bytes of a data resource nearest the scale factor
+  // |scale_factor| into |bytes|, without doing any processing or
+  // interpretation of the resource. Use ResourceHandle::SCALE_FACTOR_NONE
+  // for scale independent image resources (such as wallpaper).
+  // Returns NULL if we fail to read the resource.
+  base::StringPiece GetRawDataResourceForScaleImpl(
+      int resource_id,
+      ScaleFactor scale_factor) const;
+
   // Returns true if missing scaled resources should be visually indicated when
   // drawing the fallback (e.g., by tinting the image).
   static bool ShouldHighlightMissingScaledResources();
@@ -411,13 +425,13 @@ class UI_BASE_EXPORT ResourceBundle {
   Delegate* delegate_;
 
   // Protects |images_| and font-related members.
-  scoped_ptr<base::Lock> images_and_fonts_lock_;
+  std::unique_ptr<base::Lock> images_and_fonts_lock_;
 
   // Protects |locale_resources_data_|.
-  scoped_ptr<base::Lock> locale_resources_data_lock_;
+  std::unique_ptr<base::Lock> locale_resources_data_lock_;
 
   // Handles for data sources.
-  scoped_ptr<ResourceHandle> locale_resources_data_;
+  std::unique_ptr<ResourceHandle> locale_resources_data_;
   ScopedVector<ResourceHandle> data_packs_;
 
   // The maximum scale factor currently loaded.
@@ -430,22 +444,18 @@ class UI_BASE_EXPORT ResourceBundle {
 
   gfx::Image empty_image_;
 
-  // The various font lists used. Cached to avoid repeated GDI
-  // creation/destruction.
-  scoped_ptr<gfx::FontList> base_font_list_;
-  scoped_ptr<gfx::FontList> bold_font_list_;
-  scoped_ptr<gfx::FontList> small_font_list_;
-  scoped_ptr<gfx::FontList> small_bold_font_list_;
-  scoped_ptr<gfx::FontList> medium_font_list_;
-  scoped_ptr<gfx::FontList> medium_bold_font_list_;
-  scoped_ptr<gfx::FontList> large_font_list_;
-  scoped_ptr<gfx::FontList> large_bold_font_list_;
-  scoped_ptr<gfx::FontList> web_font_list_;
+  // The various font lists used, as a map from a signed size delta from the
+  // platform base font size, plus style, to the FontList. Cached to avoid
+  // repeated GDI creation/destruction and font derivation.
+  // Must be accessed only while holding |images_and_fonts_lock_|.
+  std::map<FontKey, gfx::FontList> font_cache_;
 
   base::FilePath overridden_pak_path_;
 
   IdToStringMap overridden_locale_strings_;
 
+  bool is_test_resources_ = false;
+
   DISALLOW_COPY_AND_ASSIGN(ResourceBundle);
 };
 
