@@ -109,7 +109,7 @@ class InterpolatedTransformTransition : public LayerAnimationElement {
   void OnAbort(LayerAnimationDelegate* delegate) override {}
 
  private:
-  scoped_ptr<InterpolatedTransform> interpolated_transform_;
+  std::unique_ptr<InterpolatedTransform> interpolated_transform_;
 
   DISALLOW_COPY_AND_ASSIGN(InterpolatedTransformTransition);
 };
@@ -332,7 +332,7 @@ class ThreadedLayerAnimationElement : public LayerAnimationElement {
   }
   ~ThreadedLayerAnimationElement() override {}
 
-  bool IsThreaded() const override { return (duration() != base::TimeDelta()); }
+  bool IsThreaded() const override { return !duration().is_zero(); }
 
  protected:
   explicit ThreadedLayerAnimationElement(const LayerAnimationElement& element)
@@ -344,7 +344,10 @@ class ThreadedLayerAnimationElement : public LayerAnimationElement {
       return false;
 
     if (Started() && IsThreaded()) {
-      delegate->RemoveThreadedAnimation(animation_id());
+      LayerThreadedAnimationDelegate* threaded =
+          delegate->GetThreadedAnimationDelegate();
+      DCHECK(threaded);
+      threaded->RemoveThreadedAnimation(animation_id());
     }
 
     OnEnd(delegate);
@@ -353,7 +356,10 @@ class ThreadedLayerAnimationElement : public LayerAnimationElement {
 
   void OnAbort(LayerAnimationDelegate* delegate) override {
     if (delegate && Started() && IsThreaded()) {
-      delegate->RemoveThreadedAnimation(animation_id());
+      LayerThreadedAnimationDelegate* threaded =
+          delegate->GetThreadedAnimationDelegate();
+      DCHECK(threaded);
+      threaded->RemoveThreadedAnimation(animation_id());
     }
   }
 
@@ -364,14 +370,18 @@ class ThreadedLayerAnimationElement : public LayerAnimationElement {
       return;
     }
     set_effective_start_time(base::TimeTicks());
-    scoped_ptr<cc::Animation> animation = CreateCCAnimation();
+    std::unique_ptr<cc::Animation> animation = CreateCCAnimation();
     animation->set_needs_synchronized_start_time(true);
-    delegate->AddThreadedAnimation(std::move(animation));
+
+    LayerThreadedAnimationDelegate* threaded =
+        delegate->GetThreadedAnimationDelegate();
+    DCHECK(threaded);
+    threaded->AddThreadedAnimation(std::move(animation));
   }
 
   virtual void OnEnd(LayerAnimationDelegate* delegate) = 0;
 
-  virtual scoped_ptr<cc::Animation> CreateCCAnimation() = 0;
+  virtual std::unique_ptr<cc::Animation> CreateCCAnimation() = 0;
 
  private:
   DISALLOW_COPY_AND_ASSIGN(ThreadedLayerAnimationElement);
@@ -407,15 +417,13 @@ class ThreadedOpacityTransition : public ThreadedLayerAnimationElement {
     delegate->SetOpacityFromAnimation(target_);
   }
 
-  scoped_ptr<cc::Animation> CreateCCAnimation() override {
-    scoped_ptr<cc::AnimationCurve> animation_curve(
-        new FloatAnimationCurveAdapter(tween_type(),
-                                       start_,
-                                       target_,
+  std::unique_ptr<cc::Animation> CreateCCAnimation() override {
+    std::unique_ptr<cc::AnimationCurve> animation_curve(
+        new FloatAnimationCurveAdapter(tween_type(), start_, target_,
                                        duration()));
-    scoped_ptr<cc::Animation> animation(
-        cc::Animation::Create(std::move(animation_curve), animation_id(),
-                              animation_group_id(), cc::Animation::OPACITY));
+    std::unique_ptr<cc::Animation> animation(cc::Animation::Create(
+        std::move(animation_curve), animation_id(), animation_group_id(),
+        cc::TargetProperty::OPACITY));
     return animation;
   }
 
@@ -460,15 +468,13 @@ class ThreadedTransformTransition : public ThreadedLayerAnimationElement {
     delegate->SetTransformFromAnimation(target_);
   }
 
-  scoped_ptr<cc::Animation> CreateCCAnimation() override {
-    scoped_ptr<cc::AnimationCurve> animation_curve(
-        new TransformAnimationCurveAdapter(tween_type(),
-                                           start_,
-                                           target_,
+  std::unique_ptr<cc::Animation> CreateCCAnimation() override {
+    std::unique_ptr<cc::AnimationCurve> animation_curve(
+        new TransformAnimationCurveAdapter(tween_type(), start_, target_,
                                            duration()));
-    scoped_ptr<cc::Animation> animation(
-        cc::Animation::Create(std::move(animation_curve), animation_id(),
-                              animation_group_id(), cc::Animation::TRANSFORM));
+    std::unique_ptr<cc::Animation> animation(cc::Animation::Create(
+        std::move(animation_curve), animation_id(), animation_group_id(),
+        cc::TargetProperty::TRANSFORM));
     return animation;
   }
 
@@ -537,10 +543,10 @@ class InverseTransformTransition : public ThreadedLayerAnimationElement {
     delegate->SetTransformFromAnimation(computed_target_transform_);
   }
 
-  scoped_ptr<cc::Animation> CreateCCAnimation() override {
-    scoped_ptr<cc::Animation> animation(
-        cc::Animation::Create(animation_curve_->Clone(), animation_id(),
-                              animation_group_id(), cc::Animation::TRANSFORM));
+  std::unique_ptr<cc::Animation> CreateCCAnimation() override {
+    std::unique_ptr<cc::Animation> animation(cc::Animation::Create(
+        animation_curve_->Clone(), animation_id(), animation_group_id(),
+        cc::TargetProperty::TRANSFORM));
     return animation;
   }
 
@@ -580,7 +586,7 @@ class InverseTransformTransition : public ThreadedLayerAnimationElement {
   const gfx::Transform base_transform_;
   gfx::Transform base_target_;
 
-  scoped_ptr<cc::AnimationCurve> animation_curve_;
+  std::unique_ptr<cc::AnimationCurve> animation_curve_;
 
   const ThreadedTransformTransition* const uninverted_transition_;
 
@@ -732,12 +738,11 @@ void LayerAnimationElement::RequestEffectiveStart(
 
 // static
 LayerAnimationElement::AnimatableProperty
-LayerAnimationElement::ToAnimatableProperty(
-    cc::Animation::TargetProperty property) {
+LayerAnimationElement::ToAnimatableProperty(cc::TargetProperty::Type property) {
   switch (property) {
-    case cc::Animation::TRANSFORM:
+    case cc::TargetProperty::TRANSFORM:
       return TRANSFORM;
-    case cc::Animation::OPACITY:
+    case cc::TargetProperty::OPACITY:
       return OPACITY;
     default:
       NOTREACHED();
