@@ -4,10 +4,17 @@
 
 #include "ui/compositor/layer_animator.h"
 
+#include <stddef.h>
+
+#include <memory>
+
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/trace_event/trace_event.h"
+#include "cc/animation/animation_host.h"
 #include "cc/animation/animation_id_provider.h"
+#include "cc/animation/animation_player.h"
+#include "cc/animation/animation_timeline.h"
+#include "cc/animation/element_animations.h"
 #include "cc/output/begin_frame_args.h"
 #include "ui/compositor/compositor.h"
 #include "ui/compositor/layer.h"
@@ -47,6 +54,8 @@ LayerAnimator::LayerAnimator(base::TimeDelta transition_duration)
       is_started_(false),
       disable_timer_for_test_(false),
       adding_animations_(false) {
+  animation_player_ =
+      cc::AnimationPlayer::Create(cc::AnimationIdProvider::NextPlayerId());
 }
 
 LayerAnimator::~LayerAnimator() {
@@ -56,6 +65,7 @@ LayerAnimator::~LayerAnimator() {
   }
   ClearAnimationsInternal();
   delegate_ = NULL;
+  DCHECK(!animation_player_->animation_timeline());
 }
 
 // static
@@ -75,26 +85,26 @@ LayerAnimator* LayerAnimator::CreateImplicitAnimator() {
 // It is worth noting that SetFoo avoids invoking the usual animation machinery
 // if the transition duration is zero -- in this case we just set the property
 // on the layer animation delegate immediately.
-#define ANIMATED_PROPERTY(type, property, name, member_type, member)  \
-void LayerAnimator::Set##name(type value) {                           \
-  base::TimeDelta duration = GetTransitionDuration();                 \
-  if (duration == base::TimeDelta() && delegate() &&                  \
-      (preemption_strategy_ != ENQUEUE_NEW_ANIMATION)) {              \
-    StopAnimatingProperty(LayerAnimationElement::property);           \
-    delegate()->Set##name##FromAnimation(value);                      \
-    return;                                                           \
-  }                                                                   \
-  scoped_ptr<LayerAnimationElement> element(                          \
-      LayerAnimationElement::Create##name##Element(value, duration)); \
-  element->set_tween_type(tween_type_);                               \
-  StartAnimation(new LayerAnimationSequence(element.release()));      \
-}                                                                     \
-                                                                      \
-member_type LayerAnimator::GetTarget##name() const {                  \
-  LayerAnimationElement::TargetValue target(delegate());              \
-  GetTargetValue(&target);                                            \
-  return target.member;                                               \
-}
+#define ANIMATED_PROPERTY(type, property, name, member_type, member)    \
+  void LayerAnimator::Set##name(type value) {                           \
+    base::TimeDelta duration = GetTransitionDuration();                 \
+    if (duration.is_zero() && delegate() &&                             \
+        (preemption_strategy_ != ENQUEUE_NEW_ANIMATION)) {              \
+      StopAnimatingProperty(LayerAnimationElement::property);           \
+      delegate()->Set##name##FromAnimation(value);                      \
+      return;                                                           \
+    }                                                                   \
+    std::unique_ptr<LayerAnimationElement> element(                     \
+        LayerAnimationElement::Create##name##Element(value, duration)); \
+    element->set_tween_type(tween_type_);                               \
+    StartAnimation(new LayerAnimationSequence(element.release()));      \
+  }                                                                     \
+                                                                        \
+  member_type LayerAnimator::GetTarget##name() const {                  \
+    LayerAnimationElement::TargetValue target(delegate());              \
+    GetTargetValue(&target);                                            \
+    return target.member;                                               \
+  }
 
 ANIMATED_PROPERTY(
     const gfx::Transform&, TRANSFORM, Transform, gfx::Transform, transform);
@@ -115,6 +125,7 @@ void LayerAnimator::SetDelegate(LayerAnimationDelegate* delegate) {
     if (collection)
       collection->StopAnimator(this);
   }
+  SwitchToLayer(delegate ? delegate->GetCcLayer() : nullptr);
   delegate_ = delegate;
   if (delegate_ && is_started_) {
     LayerAnimatorCollection* collection = GetLayerAnimatorCollection();
@@ -123,6 +134,98 @@ void LayerAnimator::SetDelegate(LayerAnimationDelegate* delegate) {
   }
 }
 
+void LayerAnimator::SwitchToLayer(scoped_refptr<cc::Layer> new_layer) {
+  // Release ElementAnimations state for old layer.
+  element_animations_state_ = nullptr;
+
+  if (delegate_)
+    DetachLayerFromAnimationPlayer();
+  if (new_layer)
+    AttachLayerToAnimationPlayer(new_layer->id());
+}
+
+void LayerAnimator::SetCompositor(Compositor* compositor) {
+  DCHECK(compositor);
+
+  cc::AnimationTimeline* timeline = compositor->GetAnimationTimeline();
+  DCHECK(timeline);
+
+  DCHECK(delegate_->GetCcLayer());
+
+  // Register ElementAnimations so it will be picked up by
+  // AnimationHost::RegisterPlayerForLayer via
+  // AnimationHost::GetElementAnimationsForLayerId.
+  if (element_animations_state_) {
+    DCHECK_EQ(element_animations_state_->element_id().primaryId,
+              delegate_->GetCcLayer()->id());
+    timeline->animation_host()->RegisterElementAnimations(
+        element_animations_state_.get());
+  }
+
+  timeline->AttachPlayer(animation_player_);
+
+  AttachLayerToAnimationPlayer(delegate_->GetCcLayer()->id());
+
+  // Release ElementAnimations state.
+  element_animations_state_ = nullptr;
+}
+
+void LayerAnimator::ResetCompositor(Compositor* compositor) {
+  DCHECK(compositor);
+
+  cc::AnimationTimeline* timeline = compositor->GetAnimationTimeline();
+  DCHECK(timeline);
+
+  cc::ElementId element_id(animation_player_->element_id());
+
+  // Store a reference to ElementAnimations (if any)
+  // so it may be picked up in LayerAnimator::SetCompositor.
+  if (element_id) {
+    element_animations_state_ =
+        timeline->animation_host()->GetElementAnimationsForElementId(
+            element_id);
+  }
+
+  DetachLayerFromAnimationPlayer();
+
+  timeline->DetachPlayer(animation_player_);
+}
+
+void LayerAnimator::AttachLayerToAnimationPlayer(int layer_id) {
+  // For ui, layer and element ids are equivalent.
+  cc::ElementId element_id(layer_id, 0);
+  if (!animation_player_->element_id())
+    animation_player_->AttachElement(element_id);
+  else
+    DCHECK_EQ(animation_player_->element_id(), element_id);
+
+  animation_player_->set_animation_delegate(this);
+}
+
+void LayerAnimator::DetachLayerFromAnimationPlayer() {
+  animation_player_->set_animation_delegate(nullptr);
+
+  if (animation_player_->element_id())
+    animation_player_->DetachElement();
+}
+
+void LayerAnimator::AddThreadedAnimation(
+    std::unique_ptr<cc::Animation> animation) {
+  animation_player_->AddAnimation(std::move(animation));
+}
+
+void LayerAnimator::RemoveThreadedAnimation(int animation_id) {
+  animation_player_->RemoveAnimation(animation_id);
+}
+
+bool LayerAnimator::HasPendingThreadedAnimationsForTesting() const {
+  return animation_player_->has_pending_animations_for_testing();
+}
+
+cc::AnimationPlayer* LayerAnimator::GetAnimationPlayerForTesting() const {
+  return animation_player_.get();
+}
+
 void LayerAnimator::StartAnimation(LayerAnimationSequence* animation) {
   scoped_refptr<LayerAnimator> retain(this);
   OnScheduled(animation);
@@ -303,23 +406,26 @@ void LayerAnimator::RemoveObserver(LayerAnimationObserver* observer) {
 }
 
 void LayerAnimator::OnThreadedAnimationStarted(
-    const cc::AnimationEvent& event) {
+    base::TimeTicks monotonic_time,
+    cc::TargetProperty::Type target_property,
+    int group_id) {
   LayerAnimationElement::AnimatableProperty property =
-    LayerAnimationElement::ToAnimatableProperty(event.target_property);
+      LayerAnimationElement::ToAnimatableProperty(target_property);
 
   RunningAnimation* running = GetRunningAnimation(property);
   if (!running)
     return;
   DCHECK(running->is_sequence_alive());
 
-  if (running->sequence()->animation_group_id() != event.group_id)
+  if (running->sequence()->animation_group_id() != group_id)
     return;
 
-  running->sequence()->OnThreadedAnimationStarted(event);
+  running->sequence()->OnThreadedAnimationStarted(monotonic_time,
+                                                  target_property, group_id);
   if (!running->sequence()->waiting_for_group_start())
     return;
 
-  base::TimeTicks start_time = event.monotonic_time;
+  base::TimeTicks start_time = monotonic_time;
 
   running->sequence()->set_waiting_for_group_start(false);
 
@@ -330,7 +436,7 @@ void LayerAnimator::OnThreadedAnimationStarted(
        iter != running_animations_.end(); ++iter) {
     // Ensure that each sequence is only Started once, regardless of the
     // number of sequences in the group that have threaded first elements.
-    if (((*iter).sequence()->animation_group_id() == event.group_id) &&
+    if (((*iter).sequence()->animation_group_id() == group_id) &&
         !(*iter).sequence()->IsFirstElementThreaded() &&
         (*iter).sequence()->waiting_for_group_start()) {
       (*iter).sequence()->set_start_time(start_time);
@@ -348,7 +454,7 @@ void LayerAnimator::AddToCollection(LayerAnimatorCollection* collection) {
 }
 
 void LayerAnimator::RemoveFromCollection(LayerAnimatorCollection* collection) {
-  if (is_animating() && is_started_) {
+  if (is_started_) {
     collection->StopAnimator(this);
     is_started_ = false;
   }
@@ -511,7 +617,7 @@ LayerAnimationSequence* LayerAnimator::RemoveAnimation(
 void LayerAnimator::FinishAnimation(
     LayerAnimationSequence* sequence, bool abort) {
   scoped_refptr<LayerAnimator> retain(this);
-  scoped_ptr<LayerAnimationSequence> removed(RemoveAnimation(sequence));
+  std::unique_ptr<LayerAnimationSequence> removed(RemoveAnimation(sequence));
   if (abort)
     sequence->Abort(delegate());
   else
@@ -535,7 +641,7 @@ void LayerAnimator::FinishAnyAnimationWithZeroDuration() {
     if (running_animations_copy[i].sequence()->IsFinished(
           running_animations_copy[i].sequence()->start_time())) {
       SAFE_INVOKE_VOID(ProgressAnimationToEnd, running_animations_copy[i]);
-      scoped_ptr<LayerAnimationSequence> removed(
+      std::unique_ptr<LayerAnimationSequence> removed(
           SAFE_INVOKE_PTR(RemoveAnimation, running_animations_copy[i]));
     }
   }
@@ -587,7 +693,7 @@ void LayerAnimator::RemoveAllAnimationsWithACommonProperty(
 
     if (running_animations_copy[i].sequence()->HasConflictingProperty(
             sequence->properties())) {
-      scoped_ptr<LayerAnimationSequence> removed(
+      std::unique_ptr<LayerAnimationSequence> removed(
           SAFE_INVOKE_PTR(RemoveAnimation, running_animations_copy[i]));
       if (abort)
         running_animations_copy[i].sequence()->Abort(delegate());
@@ -608,7 +714,7 @@ void LayerAnimator::RemoveAllAnimationsWithACommonProperty(
       continue;
 
     if (sequences[i]->HasConflictingProperty(sequence->properties())) {
-      scoped_ptr<LayerAnimationSequence> removed(
+      std::unique_ptr<LayerAnimationSequence> removed(
           RemoveAnimation(sequences[i].get()));
       if (abort)
         sequences[i]->Abort(delegate());
@@ -825,7 +931,7 @@ void LayerAnimator::ClearAnimationsInternal() {
     if (!SAFE_INVOKE_BOOL(HasAnimation, running_animations_copy[i]))
       continue;
 
-    scoped_ptr<LayerAnimationSequence> removed(
+    std::unique_ptr<LayerAnimationSequence> removed(
         RemoveAnimation(running_animations_copy[i].sequence()));
     if (removed.get())
       removed->Abort(delegate());
@@ -850,11 +956,21 @@ LayerAnimatorCollection* LayerAnimator::GetLayerAnimatorCollection() {
   return delegate_ ? delegate_->GetLayerAnimatorCollection() : NULL;
 }
 
+void LayerAnimator::NotifyAnimationStarted(
+    base::TimeTicks monotonic_time,
+    cc::TargetProperty::Type target_property,
+    int group) {
+  OnThreadedAnimationStarted(monotonic_time, target_property, group);
+}
+
 LayerAnimator::RunningAnimation::RunningAnimation(
     const base::WeakPtr<LayerAnimationSequence>& sequence)
     : sequence_(sequence) {
 }
 
+LayerAnimator::RunningAnimation::RunningAnimation(
+    const RunningAnimation& other) = default;
+
 LayerAnimator::RunningAnimation::~RunningAnimation() { }
 
 }  // namespace ui
