@@ -5,13 +5,14 @@
 #include "build/build_config.h"
 #include "ui/display/display_switches.h"
 
-namespace ui {
 namespace switches {
 
+// Overrides the device scale factor for the browser UI and the contents.
+const char kForceDeviceScaleFactor[] = "force-device-scale-factor";
+
 #if defined(OS_CHROMEOS)
 const char kDisableDisplayColorCalibration[] =
     "disable-display-color-calibration";
 #endif
 
 }  // namespace switches
-}  // namespace ui
