@@ -14,8 +14,9 @@
 #include "base/logging.h"
 #include "base/metrics/histogram.h"
 #include "base/single_thread_task_runner.h"
-#include "base/thread_task_runner_handle.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "base/trace_event/trace_event.h"
+#include "cc/input/main_thread_scrolling_reason.h"
 #include "third_party/WebKit/public/web/WebInputEvent.h"
 #include "ui/events/blink/input_handler_proxy_client.h"
 #include "ui/events/blink/input_scroll_elasticity_controller.h"
@@ -34,6 +35,8 @@ using blink::WebTouchPoint;
 
 namespace {
 
+const int32_t kEventDispositionUndefined = -1;
+
 // Maximum time between a fling event's timestamp and the first |Animate| call
 // for the fling curve to use the fling timestamp as the initial animation time.
 // Two frames allows a minor delay between event creation and the first animate.
@@ -122,28 +125,36 @@ WebGestureEvent ObtainGestureScrollBegin(const WebGestureEvent& event) {
 }
 
 cc::ScrollState CreateScrollStateForGesture(const WebGestureEvent& event) {
+  cc::ScrollStateData scroll_state_data;
   switch (event.type) {
     case WebInputEvent::GestureScrollBegin:
-      return cc::ScrollState(0, 0, event.x, event.y, 0, 0, true, false, false);
+      scroll_state_data.position_x = event.x;
+      scroll_state_data.position_y = event.y;
+      scroll_state_data.is_beginning = true;
+      break;
     case WebInputEvent::GestureFlingStart:
-      return cc::ScrollState(
-          0, 0, event.x, event.y, event.data.flingStart.velocityX,
-          event.data.flingStart.velocityX, true, true, false);
+      scroll_state_data.velocity_x = event.data.flingStart.velocityX;
+      scroll_state_data.velocity_y = event.data.flingStart.velocityY;
+      scroll_state_data.is_in_inertial_phase = true;
+      break;
     case WebInputEvent::GestureScrollUpdate:
-      return cc::ScrollState(-event.data.scrollUpdate.deltaX,
-                             -event.data.scrollUpdate.deltaY, event.x, event.y,
-                             event.data.scrollUpdate.velocityX,
-                             event.data.scrollUpdate.velocityY,
-                             event.data.scrollUpdate.inertial, false, false);
+      scroll_state_data.delta_x = -event.data.scrollUpdate.deltaX;
+      scroll_state_data.delta_y = -event.data.scrollUpdate.deltaY;
+      scroll_state_data.velocity_x = event.data.scrollUpdate.velocityX;
+      scroll_state_data.velocity_y = event.data.scrollUpdate.velocityY;
+      scroll_state_data.is_in_inertial_phase =
+          event.data.scrollUpdate.inertialPhase ==
+          WebGestureEvent::MomentumPhase;
+      break;
     case WebInputEvent::GestureScrollEnd:
-      return cc::ScrollState(0, 0, event.x, event.y, 0, 0, false, false, true);
     case WebInputEvent::GestureFlingCancel:
-      return cc::ScrollState(0, 0, event.x, event.y, 0, 0, false, true, true);
-
+      scroll_state_data.is_ending = true;
+      break;
     default:
       NOTREACHED();
-      return cc::ScrollState(0, 0, 0, 0, 0, 0, false, false, false);
+      break;
   }
+  return cc::ScrollState(scroll_state_data);
 }
 
 void ReportInputEventLatencyUma(const WebInputEvent& event,
@@ -199,6 +210,13 @@ void ReportInputEventLatencyUma(const WebInputEvent& event,
   }
 }
 
+cc::InputHandler::ScrollInputType GestureScrollInputType(
+    blink::WebGestureDevice device) {
+  return device == blink::WebGestureDeviceTouchpad
+             ? cc::InputHandler::WHEEL
+             : cc::InputHandler::TOUCHSCREEN;
+}
+
 }  // namespace
 
 namespace ui {
@@ -220,7 +238,8 @@ InputHandlerProxy::InputHandlerProxy(cc::InputHandler* input_handler,
       disallow_vertical_fling_scroll_(false),
       has_fling_animation_started_(false),
       smooth_scroll_enabled_(false),
-      uma_latency_reporting_enabled_(base::TimeTicks::IsHighResolution()) {
+      uma_latency_reporting_enabled_(base::TimeTicks::IsHighResolution()),
+      touch_start_result_(kEventDispositionUndefined) {
   DCHECK(client);
   input_handler_->BindToClient(this);
   cc::ScrollElasticityHelper* scroll_elasticity_helper =
@@ -254,7 +273,7 @@ InputHandlerProxy::HandleInputEventWithLatencyInfo(
                          TRACE_EVENT_FLAG_FLOW_IN | TRACE_EVENT_FLAG_FLOW_OUT,
                          "step", "HandleInputEventImpl");
 
-  scoped_ptr<cc::SwapPromiseMonitor> latency_info_swap_promise_monitor =
+  std::unique_ptr<cc::SwapPromiseMonitor> latency_info_swap_promise_monitor =
       input_handler_->CreateLatencyInfoSwapPromiseMonitor(latency_info);
   InputHandlerProxy::EventDisposition disposition = HandleInputEvent(event);
   return disposition;
@@ -287,8 +306,9 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleInputEvent(
       const WebGestureEvent& gesture_event =
           static_cast<const WebGestureEvent&>(event);
       if (gesture_event.sourceDevice == blink::WebGestureDeviceTouchpad &&
-          input_handler_->HaveWheelEventHandlersAt(
-              gfx::Point(gesture_event.x, gesture_event.y))) {
+          input_handler_->GetEventListenerProperties(
+              cc::EventListenerClass::kMouseWheel) !=
+              cc::EventListenerProperties::kNone) {
         return DID_NOT_HANDLE;
       } else {
         input_handler_->PinchGestureBegin();
@@ -335,6 +355,12 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleInputEvent(
     case WebInputEvent::TouchStart:
       return HandleTouchStart(static_cast<const WebTouchEvent&>(event));
 
+    case WebInputEvent::TouchMove:
+      return HandleTouchMove(static_cast<const WebTouchEvent&>(event));
+
+    case WebInputEvent::TouchEnd:
+      return HandleTouchEnd(static_cast<const WebTouchEvent&>(event));
+
     case WebInputEvent::MouseMove: {
       const WebMouseEvent& mouse_event =
           static_cast<const WebMouseEvent&>(event);
@@ -359,18 +385,89 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleInputEvent(
   return DID_NOT_HANDLE;
 }
 
-bool InputHandlerProxy::ShouldAnimate(
-    const blink::WebMouseWheelEvent& event) const {
+void InputHandlerProxy::RecordMainThreadScrollingReasons(
+    blink::WebGestureDevice device,
+    uint32_t reasons) {
+  static const char* kGestureHistogramName =
+      "Renderer4.MainThreadGestureScrollReason";
+  static const char* kWheelHistogramName =
+      "Renderer4.MainThreadWheelScrollReason";
+
+  DCHECK(device == blink::WebGestureDeviceTouchpad ||
+         device == blink::WebGestureDeviceTouchscreen);
+
+  if (device != blink::WebGestureDeviceTouchpad &&
+      device != blink::WebGestureDeviceTouchscreen) {
+    return;
+  }
+
+  if (reasons == cc::MainThreadScrollingReason::kNotScrollingOnMain) {
+    if (device == blink::WebGestureDeviceTouchscreen) {
+      UMA_HISTOGRAM_ENUMERATION(
+          kGestureHistogramName,
+          cc::MainThreadScrollingReason::kNotScrollingOnMain,
+          cc::MainThreadScrollingReason::kMainThreadScrollingReasonCount);
+    } else {
+      UMA_HISTOGRAM_ENUMERATION(
+          kWheelHistogramName,
+          cc::MainThreadScrollingReason::kNotScrollingOnMain,
+          cc::MainThreadScrollingReason::kMainThreadScrollingReasonCount);
+    }
+  }
+
+  for (uint32_t i = 0;
+       i < cc::MainThreadScrollingReason::kMainThreadScrollingReasonCount - 1;
+       ++i) {
+    unsigned val = 1 << i;
+    if (reasons & val) {
+      if (device == blink::WebGestureDeviceTouchscreen) {
+        UMA_HISTOGRAM_ENUMERATION(
+            kGestureHistogramName, i + 1,
+            cc::MainThreadScrollingReason::kMainThreadScrollingReasonCount);
+      } else {
+        UMA_HISTOGRAM_ENUMERATION(
+            kWheelHistogramName, i + 1,
+            cc::MainThreadScrollingReason::kMainThreadScrollingReasonCount);
+      }
+    }
+  }
+}
+
+bool InputHandlerProxy::ShouldAnimate(bool has_precise_scroll_deltas) const {
 #if defined(OS_MACOSX)
   // Mac does not smooth scroll wheel events (crbug.com/574283).
   return false;
 #else
-  return smooth_scroll_enabled_ && !event.hasPreciseScrollingDeltas;
+  return smooth_scroll_enabled_ && !has_precise_scroll_deltas;
 #endif
 }
 
 InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
     const WebMouseWheelEvent& wheel_event) {
+  // Only call |CancelCurrentFling()| if a fling was active, as it will
+  // otherwise disrupt an in-progress touch scroll.
+  if (!wheel_event.hasPreciseScrollingDeltas && fling_curve_)
+    CancelCurrentFling();
+
+    cc::EventListenerProperties properties =
+        input_handler_->GetEventListenerProperties(
+            cc::EventListenerClass::kMouseWheel);
+    switch (properties) {
+      case cc::EventListenerProperties::kPassive:
+        return DID_HANDLE_NON_BLOCKING;
+      case cc::EventListenerProperties::kBlockingAndPassive:
+      case cc::EventListenerProperties::kBlocking:
+        return DID_NOT_HANDLE;
+      case cc::EventListenerProperties::kNone:
+        return DROP_EVENT;
+      default:
+        NOTREACHED();
+        return DROP_EVENT;
+    }
+}
+
+InputHandlerProxy::EventDisposition InputHandlerProxy::ScrollByMouseWheel(
+    const WebMouseWheelEvent& wheel_event) {
   InputHandlerProxy::EventDisposition result = DID_NOT_HANDLE;
   cc::InputHandlerScrollResult scroll_result;
 
@@ -388,16 +485,21 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
     // TODO(jamesr): We don't properly handle scroll by page in the compositor
     // thread, so punt it to the main thread. http://crbug.com/236639
     result = DID_NOT_HANDLE;
-  } else if (!wheel_event.canScroll) {
-    // Wheel events with |canScroll| == false will not trigger scrolling,
-    // only event handlers.  Forward to the main thread.
-    result = DID_NOT_HANDLE;
-  } else if (ShouldAnimate(wheel_event)) {
+    RecordMainThreadScrollingReasons(
+        blink::WebGestureDeviceTouchpad,
+        cc::MainThreadScrollingReason::kPageBasedScrolling);
+
+  } else if (ShouldAnimate(wheel_event.hasPreciseScrollingDeltas)) {
     cc::InputHandler::ScrollStatus scroll_status =
         input_handler_->ScrollAnimated(gfx::Point(wheel_event.x, wheel_event.y),
                                        scroll_delta);
-    switch (scroll_status) {
-      case cc::InputHandler::SCROLL_STARTED:
+
+    RecordMainThreadScrollingReasons(
+        blink::WebGestureDeviceTouchpad,
+        scroll_status.main_thread_scrolling_reasons);
+
+    switch (scroll_status.thread) {
+      case cc::InputHandler::SCROLL_ON_IMPL_THREAD:
         result = DID_HANDLE;
         break;
       case cc::InputHandler::SCROLL_IGNORED:
@@ -408,27 +510,39 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
         break;
     }
   } else {
-    cc::ScrollState scroll_state_begin(0, 0, wheel_event.x, wheel_event.y, 0, 0,
-                                       true, false, false);
-    cc::InputHandler::ScrollStatus scroll_status =
-        input_handler_->ScrollBegin(&scroll_state_begin,
-                                    cc::InputHandler::WHEEL);
-    switch (scroll_status) {
-      case cc::InputHandler::SCROLL_STARTED: {
+    cc::ScrollStateData scroll_state_begin_data;
+    scroll_state_begin_data.position_x = wheel_event.x;
+    scroll_state_begin_data.position_y = wheel_event.y;
+    scroll_state_begin_data.is_beginning = true;
+    cc::ScrollState scroll_state_begin(scroll_state_begin_data);
+    cc::InputHandler::ScrollStatus scroll_status = input_handler_->ScrollBegin(
+        &scroll_state_begin, cc::InputHandler::WHEEL);
+
+    RecordMainThreadScrollingReasons(
+        blink::WebGestureDeviceTouchpad,
+        scroll_status.main_thread_scrolling_reasons);
+
+    switch (scroll_status.thread) {
+      case cc::InputHandler::SCROLL_ON_IMPL_THREAD: {
         TRACE_EVENT_INSTANT2("input",
                              "InputHandlerProxy::handle_input wheel scroll",
                              TRACE_EVENT_SCOPE_THREAD, "deltaX",
                              scroll_delta.x(), "deltaY", scroll_delta.y());
-        cc::ScrollState scroll_state_update(scroll_delta.x(), scroll_delta.y(),
-                                            wheel_event.x, wheel_event.y, 0, 0,
-                                            false, false, false);
+
+        cc::ScrollStateData scroll_state_update_data;
+        scroll_state_update_data.delta_x = scroll_delta.x();
+        scroll_state_update_data.delta_y = scroll_delta.y();
+        scroll_state_update_data.position_x = wheel_event.x;
+        scroll_state_update_data.position_y = wheel_event.y;
+        cc::ScrollState scroll_state_update(scroll_state_update_data);
 
         scroll_result = input_handler_->ScrollBy(&scroll_state_update);
         HandleOverscroll(gfx::Point(wheel_event.x, wheel_event.y),
                          scroll_result);
 
-        cc::ScrollState scroll_state_end(0, 0, wheel_event.x, wheel_event.y,
-                                         0, 0, false, false, true);
+        cc::ScrollStateData scroll_state_end_data;
+        scroll_state_end_data.is_ending = true;
+        cc::ScrollState scroll_state_end(scroll_state_end_data);
         input_handler_->ScrollEnd(&scroll_state_end);
 
         result = scroll_result.did_scroll ? DID_HANDLE : DROP_EVENT;
@@ -445,26 +559,8 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleMouseWheel(
       case cc::InputHandler::SCROLL_ON_MAIN_THREAD:
         result = DID_NOT_HANDLE;
         break;
-      case cc::InputHandler::ScrollStatusCount:
-        NOTREACHED();
-        break;
     }
   }
-
-  // Send the event and its disposition to the elasticity controller to update
-  // the over-scroll animation. If the event is to be handled on the main
-  // thread, the event and its disposition will be sent to the elasticity
-  // controller after being handled on the main thread.
-  if (scroll_elasticity_controller_ && result != DID_NOT_HANDLE) {
-    // Note that the call to the elasticity controller is made asynchronously,
-    // to minimize divergence between main thread and impl thread event
-    // handling paths.
-    base::ThreadTaskRunnerHandle::Get()->PostTask(
-        FROM_HERE,
-        base::Bind(&InputScrollElasticityController::ObserveWheelEventAndResult,
-                   scroll_elasticity_controller_->GetWeakPtr(), wheel_event,
-                   scroll_result));
-  }
   return result;
 }
 
@@ -479,33 +575,51 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollBegin(
 #endif
   cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
   cc::InputHandler::ScrollStatus scroll_status;
-  if (gesture_event.data.scrollBegin.targetViewport) {
-    scroll_status = input_handler_->RootScrollBegin(&scroll_state,
-                                                    cc::InputHandler::GESTURE);
+  if (gesture_event.data.scrollBegin.deltaHintUnits ==
+      blink::WebGestureEvent::ScrollUnits::Page) {
+    scroll_status.thread = cc::InputHandler::SCROLL_ON_MAIN_THREAD;
+    scroll_status.main_thread_scrolling_reasons =
+        cc::MainThreadScrollingReason::kContinuingMainThreadScroll;
+  } else if (gesture_event.data.scrollBegin.targetViewport) {
+    scroll_status = input_handler_->RootScrollBegin(
+        &scroll_state, GestureScrollInputType(gesture_event.sourceDevice));
+  } else if (ShouldAnimate(gesture_event.data.scrollBegin.deltaHintUnits !=
+                           blink::WebGestureEvent::ScrollUnits::Pixels)) {
+    gfx::Point scroll_point(gesture_event.x, gesture_event.y);
+    scroll_status = input_handler_->ScrollAnimatedBegin(scroll_point);
   } else {
-    scroll_status =
-        input_handler_->ScrollBegin(&scroll_state, cc::InputHandler::GESTURE);
+    scroll_status = input_handler_->ScrollBegin(
+        &scroll_state, GestureScrollInputType(gesture_event.sourceDevice));
   }
   UMA_HISTOGRAM_ENUMERATION("Renderer4.CompositorScrollHitTestResult",
-                            scroll_status,
-                            cc::InputHandler::ScrollStatusCount);
-  switch (scroll_status) {
-    case cc::InputHandler::SCROLL_STARTED:
+                            scroll_status.thread,
+                            cc::InputHandler::LAST_SCROLL_STATUS + 1);
+
+  RecordMainThreadScrollingReasons(gesture_event.sourceDevice,
+                                   scroll_status.main_thread_scrolling_reasons);
+
+  InputHandlerProxy::EventDisposition result = DID_NOT_HANDLE;
+  switch (scroll_status.thread) {
+    case cc::InputHandler::SCROLL_ON_IMPL_THREAD:
       TRACE_EVENT_INSTANT0("input",
                            "InputHandlerProxy::handle_input gesture scroll",
                            TRACE_EVENT_SCOPE_THREAD);
       gesture_scroll_on_impl_thread_ = true;
-      return DID_HANDLE;
+      result = DID_HANDLE;
+      break;
     case cc::InputHandler::SCROLL_UNKNOWN:
     case cc::InputHandler::SCROLL_ON_MAIN_THREAD:
-      return DID_NOT_HANDLE;
+      result = DID_NOT_HANDLE;
+      break;
     case cc::InputHandler::SCROLL_IGNORED:
-      return DROP_EVENT;
-    case cc::InputHandler::ScrollStatusCount:
-      NOTREACHED();
+      result = DROP_EVENT;
       break;
   }
-  return DID_NOT_HANDLE;
+  if (scroll_elasticity_controller_ && result != DID_NOT_HANDLE)
+    HandleScrollElasticityOverscroll(gesture_event,
+                                     cc::InputHandlerScrollResult());
+
+  return result;
 }
 
 InputHandlerProxy::EventDisposition
@@ -514,14 +628,32 @@ InputHandlerProxy::HandleGestureScrollUpdate(
 #ifndef NDEBUG
   DCHECK(expect_scroll_update_end_);
 #endif
-
   if (!gesture_scroll_on_impl_thread_ && !gesture_pinch_on_impl_thread_)
     return DID_NOT_HANDLE;
 
   cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
+  gfx::Point scroll_point(gesture_event.x, gesture_event.y);
+  gfx::Vector2dF scroll_delta(-gesture_event.data.scrollUpdate.deltaX,
+                              -gesture_event.data.scrollUpdate.deltaY);
+
+  if (ShouldAnimate(gesture_event.data.scrollUpdate.deltaUnits !=
+                    blink::WebGestureEvent::ScrollUnits::Pixels)) {
+    switch (input_handler_->ScrollAnimated(scroll_point, scroll_delta).thread) {
+      case cc::InputHandler::SCROLL_ON_IMPL_THREAD:
+        return DID_HANDLE;
+      case cc::InputHandler::SCROLL_IGNORED:
+        return DROP_EVENT;
+      default:
+        return DID_NOT_HANDLE;
+    }
+  }
   cc::InputHandlerScrollResult scroll_result =
       input_handler_->ScrollBy(&scroll_state);
-  HandleOverscroll(gfx::Point(gesture_event.x, gesture_event.y), scroll_result);
+  HandleOverscroll(scroll_point, scroll_result);
+
+  if (scroll_elasticity_controller_)
+    HandleScrollElasticityOverscroll(gesture_event, scroll_result);
+
   return scroll_result.did_scroll ? DID_HANDLE : DROP_EVENT;
 }
 
@@ -531,10 +663,21 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollEnd(
   DCHECK(expect_scroll_update_end_);
   expect_scroll_update_end_ = false;
 #endif
-  cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
-  input_handler_->ScrollEnd(&scroll_state);
+  if (ShouldAnimate(gesture_event.data.scrollEnd.deltaUnits !=
+                    blink::WebGestureEvent::ScrollUnits::Pixels)) {
+    // Do nothing if the scroll is being animated; the scroll animation will
+    // generate the ScrollEnd when it is done.
+  } else {
+    cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
+    input_handler_->ScrollEnd(&scroll_state);
+  }
   if (!gesture_scroll_on_impl_thread_)
     return DID_NOT_HANDLE;
+
+  if (scroll_elasticity_controller_)
+    HandleScrollElasticityOverscroll(gesture_event,
+                                     cc::InputHandlerScrollResult());
+
   gesture_scroll_on_impl_thread_ = false;
   return DID_HANDLE;
 }
@@ -542,8 +685,9 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureScrollEnd(
 InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureFlingStart(
     const WebGestureEvent& gesture_event) {
   cc::ScrollState scroll_state = CreateScrollStateForGesture(gesture_event);
-  cc::InputHandler::ScrollStatus scroll_status =
-      cc::InputHandler::SCROLL_ON_MAIN_THREAD;
+  cc::InputHandler::ScrollStatus scroll_status;
+  scroll_status.main_thread_scrolling_reasons =
+      cc::MainThreadScrollingReason::kNotScrollingOnMain;
   switch (gesture_event.sourceDevice) {
   case blink::WebGestureDeviceTouchpad:
     if (gesture_event.data.flingStart.targetViewport) {
@@ -555,10 +699,13 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureFlingStart(
     }
     break;
   case blink::WebGestureDeviceTouchscreen:
-    if (!gesture_scroll_on_impl_thread_)
-      scroll_status = cc::InputHandler::SCROLL_ON_MAIN_THREAD;
-    else
+    if (!gesture_scroll_on_impl_thread_) {
+      scroll_status.thread = cc::InputHandler::SCROLL_ON_MAIN_THREAD;
+      scroll_status.main_thread_scrolling_reasons =
+          cc::MainThreadScrollingReason::kContinuingMainThreadScroll;
+    } else {
       scroll_status = input_handler_->FlingScrollBegin();
+    }
     break;
   case blink::WebGestureDeviceUninitialized:
     NOTREACHED();
@@ -569,8 +716,8 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureFlingStart(
   expect_scroll_update_end_ = false;
 #endif
 
-  switch (scroll_status) {
-    case cc::InputHandler::SCROLL_STARTED: {
+  switch (scroll_status.thread) {
+    case cc::InputHandler::SCROLL_ON_IMPL_THREAD: {
       if (gesture_event.sourceDevice == blink::WebGestureDeviceTouchpad) {
         scroll_state.set_is_ending(true);
         input_handler_->ScrollEnd(&scroll_state);
@@ -610,6 +757,7 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureFlingStart(
                            TRACE_EVENT_SCOPE_THREAD);
       gesture_scroll_on_impl_thread_ = false;
       fling_may_be_active_on_main_thread_ = true;
+      client_->DidStartFlinging();
       return DID_NOT_HANDLE;
     }
     case cc::InputHandler::SCROLL_IGNORED: {
@@ -626,27 +774,84 @@ InputHandlerProxy::EventDisposition InputHandlerProxy::HandleGestureFlingStart(
       }
       return DROP_EVENT;
     }
-    case cc::InputHandler::ScrollStatusCount:
-      NOTREACHED();
-      break;
   }
   return DID_NOT_HANDLE;
 }
 
 InputHandlerProxy::EventDisposition InputHandlerProxy::HandleTouchStart(
     const blink::WebTouchEvent& touch_event) {
+  EventDisposition result = DROP_EVENT;
   for (size_t i = 0; i < touch_event.touchesLength; ++i) {
     if (touch_event.touches[i].state != WebTouchPoint::StatePressed)
       continue;
     if (input_handler_->DoTouchEventsBlockScrollAt(
             gfx::Point(touch_event.touches[i].position.x,
                        touch_event.touches[i].position.y))) {
-      // TODO(rbyers): We should consider still sending the touch events to
-      // main asynchronously (crbug.com/455539).
-      return DID_NOT_HANDLE;
+      result = DID_NOT_HANDLE;
+      break;
     }
   }
-  return DROP_EVENT;
+
+  // If |result| is DROP_EVENT it wasn't processed above.
+  if (result == DROP_EVENT) {
+    switch (input_handler_->GetEventListenerProperties(
+        cc::EventListenerClass::kTouchStartOrMove)) {
+      case cc::EventListenerProperties::kPassive:
+        result = DID_HANDLE_NON_BLOCKING;
+        break;
+      case cc::EventListenerProperties::kBlocking:
+        // The touch area rects above already have checked whether it hits
+        // a blocking region. Since it does not the event can be dropped.
+        result = DROP_EVENT;
+        break;
+      case cc::EventListenerProperties::kBlockingAndPassive:
+        // There is at least one passive listener that needs to possibly
+        // be notified so it can't be dropped.
+        result = DID_HANDLE_NON_BLOCKING;
+        break;
+      case cc::EventListenerProperties::kNone:
+        result = DROP_EVENT;
+        break;
+      default:
+        NOTREACHED();
+        result = DROP_EVENT;
+        break;
+    }
+  }
+
+  // Merge |touch_start_result_| and |result| so the result has the highest
+  // priority value according to the sequence; (DROP_EVENT,
+  // DID_HANDLE_NON_BLOCKING, DID_NOT_HANDLE).
+  if (touch_start_result_ == kEventDispositionUndefined ||
+      touch_start_result_ == DROP_EVENT || result == DID_NOT_HANDLE)
+    touch_start_result_ = result;
+
+  // If |result| is still DROP_EVENT look at the touch end handler as
+  // we may not want to discard the entire touch sequence. Note this
+  // code is explicitly after the assignment of the |touch_start_result_|
+  // so the touch moves are not sent to the main thread un-necessarily.
+  if (result == DROP_EVENT &&
+      input_handler_->GetEventListenerProperties(
+          cc::EventListenerClass::kTouchEndOrCancel) !=
+          cc::EventListenerProperties::kNone) {
+    result = DID_HANDLE_NON_BLOCKING;
+  }
+
+  return result;
+}
+
+InputHandlerProxy::EventDisposition InputHandlerProxy::HandleTouchMove(
+    const blink::WebTouchEvent& touch_event) {
+  if (touch_start_result_ != kEventDispositionUndefined)
+    return static_cast<EventDisposition>(touch_start_result_);
+  return DID_NOT_HANDLE;
+}
+
+InputHandlerProxy::EventDisposition InputHandlerProxy::HandleTouchEnd(
+    const blink::WebTouchEvent& touch_event) {
+  if (touch_event.touchesLength == 1)
+    touch_start_result_ = kEventDispositionUndefined;
+  return DID_NOT_HANDLE;
 }
 
 bool InputHandlerProxy::FilterInputEventForFlingBoosting(
@@ -697,7 +902,7 @@ bool InputHandlerProxy::FilterInputEventForFlingBoosting(
               gfx::Point(gesture_event.x, gesture_event.y),
               fling_parameters_.sourceDevice == blink::WebGestureDeviceTouchpad
                   ? cc::InputHandler::NON_BUBBLING_GESTURE
-                  : cc::InputHandler::GESTURE)) {
+                  : cc::InputHandler::TOUCHSCREEN)) {
         CancelCurrentFling();
         return false;
       }
@@ -899,6 +1104,14 @@ void InputHandlerProxy::SynchronouslySetRootScrollOffset(
   input_handler_->SetSynchronousInputHandlerRootScrollOffset(root_offset);
 }
 
+void InputHandlerProxy::SynchronouslyZoomBy(float magnify_delta,
+                                            const gfx::Point& anchor) {
+  DCHECK(synchronous_input_handler_);
+  input_handler_->PinchGestureBegin();
+  input_handler_->PinchGestureUpdate(magnify_delta, anchor);
+  input_handler_->PinchGestureEnd();
+}
+
 void InputHandlerProxy::HandleOverscroll(
     const gfx::Point& causal_event_viewport_point,
     const cc::InputHandlerScrollResult& scroll_result) {
@@ -938,12 +1151,12 @@ bool InputHandlerProxy::CancelCurrentFling() {
 }
 
 bool InputHandlerProxy::CancelCurrentFlingWithoutNotifyingClient() {
-  bool had_fling_animation = fling_curve_;
+  bool had_fling_animation = !!fling_curve_;
   if (had_fling_animation &&
       fling_parameters_.sourceDevice == blink::WebGestureDeviceTouchscreen) {
-    cc::ScrollState scroll_state(0, 0, fling_parameters_.point.x,
-                                 fling_parameters_.point.y, 0, 0, false, true,
-                                 true);
+    cc::ScrollStateData scroll_state_data;
+    scroll_state_data.is_ending = true;
+    cc::ScrollState scroll_state(scroll_state_data);
     input_handler_->ScrollEnd(&scroll_state);
     TRACE_EVENT_ASYNC_END0(
         "input",
@@ -990,25 +1203,47 @@ void InputHandlerProxy::RequestAnimation() {
 
 bool InputHandlerProxy::TouchpadFlingScroll(
     const WebFloatSize& increment) {
-  WebMouseWheelEvent synthetic_wheel;
-  synthetic_wheel.type = WebInputEvent::MouseWheel;
-  synthetic_wheel.timeStampSeconds = InSecondsF(base::TimeTicks::Now());
-  synthetic_wheel.deltaX = increment.width;
-  synthetic_wheel.deltaY = increment.height;
-  synthetic_wheel.hasPreciseScrollingDeltas = true;
-  synthetic_wheel.x = fling_parameters_.point.x;
-  synthetic_wheel.y = fling_parameters_.point.y;
-  synthetic_wheel.globalX = fling_parameters_.globalPoint.x;
-  synthetic_wheel.globalY = fling_parameters_.globalPoint.y;
-  synthetic_wheel.modifiers = fling_parameters_.modifiers;
+  InputHandlerProxy::EventDisposition disposition;
+  cc::EventListenerProperties properties =
+      input_handler_->GetEventListenerProperties(
+          cc::EventListenerClass::kMouseWheel);
+  switch (properties) {
+    case cc::EventListenerProperties::kPassive:
+      disposition = DID_HANDLE_NON_BLOCKING;
+      break;
+    case cc::EventListenerProperties::kBlocking:
+      disposition = DID_NOT_HANDLE;
+      break;
+    case cc::EventListenerProperties::kNone: {
+      WebMouseWheelEvent synthetic_wheel;
+      synthetic_wheel.type = WebInputEvent::MouseWheel;
+      synthetic_wheel.timeStampSeconds = InSecondsF(base::TimeTicks::Now());
+      synthetic_wheel.deltaX = increment.width;
+      synthetic_wheel.deltaY = increment.height;
+      synthetic_wheel.hasPreciseScrollingDeltas = true;
+      synthetic_wheel.x = fling_parameters_.point.x;
+      synthetic_wheel.y = fling_parameters_.point.y;
+      synthetic_wheel.globalX = fling_parameters_.globalPoint.x;
+      synthetic_wheel.globalY = fling_parameters_.globalPoint.y;
+      synthetic_wheel.modifiers = fling_parameters_.modifiers;
+
+      disposition = ScrollByMouseWheel(synthetic_wheel);
+      break;
+    }
+    default:
+      NOTREACHED();
+      return false;
+  }
 
-  InputHandlerProxy::EventDisposition disposition =
-      HandleInputEvent(synthetic_wheel);
   switch (disposition) {
     case DID_HANDLE:
       return true;
     case DROP_EVENT:
       break;
+    case DID_HANDLE_NON_BLOCKING:
+    // TODO(dtapuska): Process the fling on the compositor thread
+    // but post the events to the main thread; for now just pass it to the
+    // main thread.
     case DID_NOT_HANDLE:
       TRACE_EVENT_INSTANT0("input",
                            "InputHandlerProxy::scrollBy::AbortFling",
@@ -1021,6 +1256,7 @@ bool InputHandlerProxy::TouchpadFlingScroll(
       // the subarea but then is flung "under" the pointer.
       client_->TransferActiveWheelFlingAnimation(fling_parameters_);
       fling_may_be_active_on_main_thread_ = true;
+      client_->DidStartFlinging();
       CancelCurrentFlingWithoutNotifyingClient();
       break;
   }
@@ -1063,10 +1299,13 @@ bool InputHandlerProxy::scrollBy(const WebFloatSize& increment,
       break;
     case blink::WebGestureDeviceTouchscreen: {
       clipped_increment = ToClientScrollIncrement(clipped_increment);
-      cc::ScrollState scroll_state(
-          clipped_increment.width, clipped_increment.height,
-          fling_parameters_.point.x, fling_parameters_.point.y,
-          clipped_velocity.width, clipped_velocity.height, false, true, false);
+      cc::ScrollStateData scroll_state_data;
+      scroll_state_data.delta_x = clipped_increment.width;
+      scroll_state_data.delta_y = clipped_increment.height;
+      scroll_state_data.velocity_x = clipped_velocity.width;
+      scroll_state_data.velocity_y = clipped_velocity.height;
+      scroll_state_data.is_in_inertial_phase = true;
+      cc::ScrollState scroll_state(scroll_state_data);
       cc::InputHandlerScrollResult scroll_result =
           input_handler_->ScrollBy(&scroll_state);
       HandleOverscroll(fling_parameters_.point, scroll_result);
@@ -1092,4 +1331,19 @@ bool InputHandlerProxy::scrollBy(const WebFloatSize& increment,
   return did_scroll;
 }
 
+void InputHandlerProxy::HandleScrollElasticityOverscroll(
+    const WebGestureEvent& gesture_event,
+    const cc::InputHandlerScrollResult& scroll_result) {
+  DCHECK(scroll_elasticity_controller_);
+  // Send the event and its disposition to the elasticity controller to update
+  // the over-scroll animation. Note that the call to the elasticity controller
+  // is made asynchronously, to minimize divergence between main thread and
+  // impl thread event handling paths.
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE,
+      base::Bind(&InputScrollElasticityController::ObserveGestureEventAndResult,
+                 scroll_elasticity_controller_->GetWeakPtr(), gesture_event,
+                 scroll_result));
+}
+
 }  // namespace ui
