@@ -8,8 +8,10 @@
 #include <libevdev/libevdev.h>
 #include <linux/input.h>
 
+#include "base/macros.h"
 #include "base/strings/stringprintf.h"
 #include "base/timer/timer.h"
+#include "ui/events/base_event_utils.h"
 #include "ui/events/event.h"
 #include "ui/events/event_utils.h"
 #include "ui/events/keycodes/dom/keycode_converter.h"
@@ -77,9 +79,8 @@ void OnGestureReadyHelper(void* client_data, const Gesture* gesture) {
 }
 
 // Convert gestures timestamp (stime_t) to ui::Event timestamp.
-base::TimeDelta StimeToTimedelta(stime_t timestamp) {
-  return base::TimeDelta::FromMicroseconds(timestamp *
-                                           base::Time::kMicrosecondsPerSecond);
+base::TimeTicks StimeToTimeTicks(stime_t timestamp) {
+  return ui::EventTimeStampFromSeconds(timestamp);
 }
 
 // Number of fingers for scroll gestures.
@@ -270,8 +271,10 @@ void GestureInterpreterLibevdevCros::OnGestureMove(const Gesture* gesture,
 
   cursor_->MoveCursor(gfx::Vector2dF(move->dx, move->dy));
   // TODO(spang): Use move->ordinal_dx, move->ordinal_dy
-  dispatcher_->DispatchMouseMoveEvent(MouseMoveEventParams(
-      id_, cursor_->GetLocation(), StimeToTimedelta(gesture->end_time)));
+  dispatcher_->DispatchMouseMoveEvent(
+      MouseMoveEventParams(id_, cursor_->GetLocation(),
+                           PointerDetails(EventPointerType::POINTER_TYPE_MOUSE),
+                           StimeToTimeTicks(gesture->end_time)));
 }
 
 void GestureInterpreterLibevdevCros::OnGestureScroll(
@@ -288,13 +291,13 @@ void GestureInterpreterLibevdevCros::OnGestureScroll(
   if (is_mouse_) {
     dispatcher_->DispatchMouseWheelEvent(MouseWheelEventParams(
         id_, cursor_->GetLocation(), gfx::Vector2d(scroll->dx, scroll->dy),
-        StimeToTimedelta(gesture->end_time)));
+        StimeToTimeTicks(gesture->end_time)));
   } else {
     dispatcher_->DispatchScrollEvent(ScrollEventParams(
         id_, ET_SCROLL, cursor_->GetLocation(),
         gfx::Vector2dF(scroll->dx, scroll->dy),
         gfx::Vector2dF(scroll->ordinal_dx, scroll->ordinal_dy),
-        kGestureScrollFingerCount, StimeToTimedelta(gesture->end_time)));
+        kGestureScrollFingerCount, StimeToTimeTicks(gesture->end_time)));
   }
 }
 
@@ -338,7 +341,7 @@ void GestureInterpreterLibevdevCros::OnGestureFling(const Gesture* gesture,
   dispatcher_->DispatchScrollEvent(ScrollEventParams(
       id_, type, cursor_->GetLocation(), gfx::Vector2dF(fling->vx, fling->vy),
       gfx::Vector2dF(fling->ordinal_vx, fling->ordinal_vy),
-      kGestureScrollFingerCount, StimeToTimedelta(gesture->end_time)));
+      kGestureScrollFingerCount, StimeToTimeTicks(gesture->end_time)));
 }
 
 void GestureInterpreterLibevdevCros::OnGestureSwipe(const Gesture* gesture,
@@ -357,7 +360,7 @@ void GestureInterpreterLibevdevCros::OnGestureSwipe(const Gesture* gesture,
       id_, ET_SCROLL, cursor_->GetLocation(),
       gfx::Vector2dF(swipe->dx, swipe->dy),
       gfx::Vector2dF(swipe->ordinal_dx, swipe->ordinal_dy),
-      kGestureSwipeFingerCount, StimeToTimedelta(gesture->end_time)));
+      kGestureSwipeFingerCount, StimeToTimeTicks(gesture->end_time)));
 }
 
 void GestureInterpreterLibevdevCros::OnGestureSwipeLift(
@@ -374,18 +377,37 @@ void GestureInterpreterLibevdevCros::OnGestureSwipeLift(
   dispatcher_->DispatchScrollEvent(ScrollEventParams(
       id_, ET_SCROLL_FLING_START, cursor_->GetLocation(),
       gfx::Vector2dF() /* delta */, gfx::Vector2dF() /* ordinal_delta */,
-      kGestureScrollFingerCount, StimeToTimedelta(gesture->end_time)));
+      kGestureScrollFingerCount, StimeToTimeTicks(gesture->end_time)));
 }
 
 void GestureInterpreterLibevdevCros::OnGesturePinch(const Gesture* gesture,
                                                     const GesturePinch* pinch) {
-  DVLOG(3) << base::StringPrintf(
-                  "Gesture Pinch: dz=%f [%f]", pinch->dz, pinch->ordinal_dz);
+  DVLOG(3) << base::StringPrintf("Gesture Pinch: dz=%f [%f] zoom_state=%u",
+                                 pinch->dz, pinch->ordinal_dz,
+                                 pinch->zoom_state);
 
   if (!cursor_)
     return;  // No cursor!
 
-  NOTIMPLEMENTED();
+  EventType type;
+  switch (pinch->zoom_state) {
+    case GESTURES_ZOOM_START:
+      type = ET_GESTURE_PINCH_BEGIN;
+      break;
+    case GESTURES_ZOOM_UPDATE:
+      type = ET_GESTURE_PINCH_UPDATE;
+      break;
+    case GESTURES_ZOOM_END:
+      type = ET_GESTURE_PINCH_END;
+      break;
+    default:
+      LOG(WARNING) << base::StringPrintf("Unrecognized pinch zoom state (%u)",
+                                         pinch->zoom_state);
+      return;
+  }
+  dispatcher_->DispatchPinchEvent(
+      PinchEventParams(id_, type, cursor_->GetLocation(), pinch->dz,
+                       StimeToTimeTicks(gesture->end_time)));
 }
 
 void GestureInterpreterLibevdevCros::OnGestureMetrics(
@@ -420,9 +442,10 @@ void GestureInterpreterLibevdevCros::DispatchMouseButton(unsigned int button,
     return;  // No change.
 
   bool allow_remap = is_mouse_;
-  dispatcher_->DispatchMouseButtonEvent(
-      MouseButtonEventParams(id_, cursor_->GetLocation(), button, down,
-                             allow_remap, StimeToTimedelta(time)));
+  dispatcher_->DispatchMouseButtonEvent(MouseButtonEventParams(
+      id_, cursor_->GetLocation(), button, down, allow_remap,
+      PointerDetails(EventPointerType::POINTER_TYPE_MOUSE),
+      StimeToTimeTicks(time)));
 }
 
 void GestureInterpreterLibevdevCros::DispatchChangedKeys(
@@ -450,7 +473,7 @@ void GestureInterpreterLibevdevCros::DispatchChangedKeys(
       // Dispatch key press or release to keyboard.
       dispatcher_->DispatchKeyEvent(
           KeyEventParams(id_, key, value, false /* suppress_auto_repeat */,
-                         StimeToTimedelta(timestamp)));
+                         StimeToTimeTicks(timestamp)));
     }
   }
 
