@@ -7,13 +7,13 @@
 
 #include <stdint.h>
 
+#include <memory>
 #include <vector>
 
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/strings/string16.h"
 #include "skia/ext/platform_canvas.h"
-#include "skia/ext/refptr.h"
+#include "third_party/skia/include/core/SkRefCnt.h"
 #include "ui/gfx/image/image_skia.h"
 #include "ui/gfx/native_widget_types.h"
 #include "ui/gfx/shadow_value.h"
@@ -25,6 +25,7 @@ class Rect;
 class RectF;
 class FontList;
 class Point;
+class PointF;
 class Size;
 class Transform;
 
@@ -82,14 +83,10 @@ class GFX_EXPORT Canvas {
   // being returned.
   Canvas(const Size& size, float image_scale, bool is_opaque);
 
-  // Constructs a canvas with the size and the image_scale of the provided
-  // |image_rep|, and draws the |image_rep| into it.
-  Canvas(const ImageSkiaRep& image_rep, bool is_opaque);
-
   // Creates a Canvas backed by an |sk_canvas| with |image_scale_|.
   // |sk_canvas| is assumed to be already scaled based on |image_scale|
   // so no additional scaling is applied.
-  Canvas(SkCanvas* sk_canvas, float image_scale);
+  Canvas(sk_sp<SkCanvas> sk_canvas, float image_scale);
 
   virtual ~Canvas();
 
@@ -241,15 +238,22 @@ class GFX_EXPORT Canvas {
 
   // Draws a single pixel line with the specified color.
   void DrawLine(const Point& p1, const Point& p2, SkColor color);
+  void DrawLine(const PointF& p1, const PointF& p2, SkColor color);
 
   // Draws a line with the given |paint| parameters.
   void DrawLine(const Point& p1, const Point& p2, const SkPaint& paint);
+  void DrawLine(const PointF& p1, const PointF& p2, const SkPaint& paint);
 
   // Draws a circle with the given |paint| parameters.
   void DrawCircle(const Point& center_point,
                   int radius,
                   const SkPaint& paint);
 
+  // Draws a circle with the given |paint| parameters.
+  void DrawCircle(const PointF& center_point,
+                  float radius,
+                  const SkPaint& paint);
+
   // Draws the given rectangle with rounded corners of |radius| using the
   // given |paint| parameters. DEPRECATED in favor of the RectF version below.
   // TODO(mgiuca): Remove this (http://crbug.com/553726).
@@ -399,6 +403,18 @@ class GFX_EXPORT Canvas {
                     int w,
                     int h);
 
+  // Helper for TileImageInt().  Initializes |paint| for tiling |image| with the
+  // given parameters.  Returns false if the provided image does not have a
+  // representation for the current scale.
+  bool InitSkPaintForTiling(const ImageSkia& image,
+                            int src_x,
+                            int src_y,
+                            float tile_scale_x,
+                            float tile_scale_y,
+                            int dest_x,
+                            int dest_y,
+                            SkPaint* paint);
+
   // Apply transformation on the canvas.
   void Transform(const Transform& transform);
 
@@ -409,14 +425,12 @@ class GFX_EXPORT Canvas {
                        const Rect& display_rect,
                        int flags);
 
-  skia::PlatformCanvas* platform_canvas() { return owned_canvas_.get(); }
-  SkCanvas* sk_canvas() { return canvas_; }
+  SkCanvas* sk_canvas() { return canvas_.get(); }
   float image_scale() const { return image_scale_; }
 
  private:
-  // Test whether the provided rectangle intersects the current clip rect.
-  bool IntersectsClipRectInt(int x, int y, int w, int h);
-  bool IntersectsClipRect(const Rect& rect);
+  // Tests whether the provided rectangle intersects the current clip rect.
+  bool IntersectsClipRect(const SkRect& rect);
 
   // Helper for the DrawImageInt functions declared above. The
   // |remove_image_scale| parameter indicates if the scale of the |image_rep|
@@ -439,8 +453,7 @@ class GFX_EXPORT Canvas {
   // Canvas::Scale() does not affect |image_scale_|.
   float image_scale_;
 
-  skia::RefPtr<skia::PlatformCanvas> owned_canvas_;
-  SkCanvas* canvas_;
+  sk_sp<SkCanvas> canvas_;
 
   DISALLOW_COPY_AND_ASSIGN(Canvas);
 };
