@@ -2,25 +2,26 @@
 // Use of this source code is governed by a BSD-style license that can be
 // found in the LICENSE file.
 
-#include "ui/gfx/canvas.h"
-
 #include <limits.h>
 #include <stddef.h>
 #include <stdint.h>
 
+#include <memory>
+
 #include "base/i18n/rtl.h"
 #include "base/logging.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/numerics/safe_conversions.h"
 #include "build/build_config.h"
 #include "third_party/skia/include/core/SkBitmap.h"
 #include "third_party/skia/include/core/SkPixmap.h"
+#include "ui/gfx/canvas.h"
 #include "ui/gfx/font_list.h"
 #include "ui/gfx/geometry/insets.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gfx/range/range.h"
 #include "ui/gfx/render_text.h"
 #include "ui/gfx/shadow_value.h"
+#include "ui/gfx/skia_util.h"
 #include "ui/gfx/text_elider.h"
 #include "ui/gfx/text_utils.h"
 
@@ -122,9 +123,9 @@ void UpdateRenderText(const Rect& rect,
 
   render_text->SetColor(color);
   const int font_style = font_list.GetFontStyle();
-  render_text->SetStyle(BOLD, (font_style & Font::BOLD) != 0);
   render_text->SetStyle(ITALIC, (font_style & Font::ITALIC) != 0);
   render_text->SetStyle(UNDERLINE, (font_style & Font::UNDERLINE) != 0);
+  render_text->SetWeight(font_list.GetFontWeight());
 }
 
 }  // namespace
@@ -149,7 +150,7 @@ void Canvas::SizeStringFloat(const base::string16& text,
     ElideRectangleText(text, font_list, *width, INT_MAX, wrap_behavior,
                        &strings);
     Rect rect(base::saturated_cast<int>(*width), INT_MAX);
-    scoped_ptr<RenderText> render_text(RenderText::CreateInstance());
+    std::unique_ptr<RenderText> render_text(RenderText::CreateInstance());
     UpdateRenderText(rect, base::string16(), font_list, flags, 0,
                      render_text.get());
 
@@ -167,7 +168,7 @@ void Canvas::SizeStringFloat(const base::string16& text,
     *width = w;
     *height = h;
   } else {
-    scoped_ptr<RenderText> render_text(RenderText::CreateInstance());
+    std::unique_ptr<RenderText> render_text(RenderText::CreateInstance());
     Rect rect(base::saturated_cast<int>(*width),
               base::saturated_cast<int>(*height));
     base::string16 adjusted_text = text;
@@ -187,7 +188,7 @@ void Canvas::DrawStringRectWithShadows(const base::string16& text,
                                        int line_height,
                                        int flags,
                                        const ShadowValues& shadows) {
-  if (!IntersectsClipRect(text_bounds))
+  if (!IntersectsClipRect(RectToSkRect(text_bounds)))
     return;
 
   Rect clip_rect(text_bounds);
@@ -198,7 +199,7 @@ void Canvas::DrawStringRectWithShadows(const base::string16& text,
 
   Rect rect(text_bounds);
 
-  scoped_ptr<RenderText> render_text(RenderText::CreateInstance());
+  std::unique_ptr<RenderText> render_text(RenderText::CreateInstance());
   render_text->set_shadows(shadows);
 
   if (flags & MULTI_LINE) {
@@ -340,7 +341,7 @@ void Canvas::DrawFadedString(const base::string16& text,
     flags |= TEXT_ALIGN_TO_HEAD;
   flags |= NO_ELLIPSIS;
 
-  scoped_ptr<RenderText> render_text(RenderText::CreateInstance());
+  std::unique_ptr<RenderText> render_text(RenderText::CreateInstance());
   Rect rect = display_rect;
   UpdateRenderText(rect, text, font_list, flags, color, render_text.get());
   render_text->SetElideBehavior(FADE_TAIL);
