@@ -15,7 +15,7 @@
 
 struct ANativeWindow;
 
-namespace gfx {
+namespace gl {
 
 // This class serves as a bridge for native code to call java functions inside
 // android SurfaceTexture class.
@@ -57,6 +57,11 @@ class GL_EXPORT SurfaceTexture
   // by calling ANativeWindow_release().
   ANativeWindow* CreateSurface();
 
+  // Release the SurfaceTexture back buffers.  The SurfaceTexture is no longer
+  // usable after calling this.  Note that this is not called 'Release', like
+  // the android API, because scoped_refptr<> calls that quite a bit.
+  void ReleaseSurfaceTexture();
+
   const base::android::JavaRef<jobject>& j_surface_texture() const {
     return j_surface_texture_;
   }
@@ -77,6 +82,6 @@ class GL_EXPORT SurfaceTexture
   DISALLOW_COPY_AND_ASSIGN(SurfaceTexture);
 };
 
-}  // namespace gfx
+}  // namespace gl
 
 #endif  // UI_GL_ANDROID_SURFACE_TEXTURE_H_
