@@ -10,6 +10,14 @@
 // the X11 headers on linux, which define all kinds of macros that are
 // liable to cause conflicts.
 
+// GL headers may include inttypes.h and so we need to ensure that
+// __STDC_FORMAT_MACROS is defined in order for //base/format_macros.h to
+// function correctly. See comment and #error message in //base/format_macros.h
+// for details.
+#if defined(OS_POSIX) && !defined(__STDC_FORMAT_MACROS)
+#define __STDC_FORMAT_MACROS
+#endif
+
 #include <GL/gl.h>
 #include <GL/glext.h>
 #include <EGL/egl.h>
@@ -28,16 +36,16 @@
 #include <GL/wglext.h>
 #elif defined(OS_MACOSX)
 #include <OpenGL/OpenGL.h>
-#elif defined(USE_X11)
+#elif defined(USE_GLX)
 #include <GL/glx.h>
 #include <GL/glxext.h>
+#endif
 
 // Undefine some macros defined by X headers. This is why this file should only
 // be included in .cc files.
 #undef Bool
 #undef None
 #undef Status
-#endif
 
 
 // GLES2 defines not part of Desktop GL
@@ -127,8 +135,8 @@
 // GL_CHROMIUM_gpu_memory_buffer_image
 #define GL_READ_WRITE_CHROMIUM                           0x78F2
 
-// GL_CHROMIUM_yuv_420_image
-#define GL_RGB_YUV_420_CHROMIUM                          0x78FA
+// GL_CHROMIUM_ycrcb_420_image
+#define GL_RGB_YCRCB_420_CHROMIUM                        0x78FA
 
 // GL_CHROMIUM_ycbcr_422_image
 #define GL_RGB_YCBCR_422_CHROMIUM                        0x78FB
@@ -328,7 +336,7 @@
 #define GL_RG8_EXT 0x822B
 #endif /* GL_EXT_texture_rg */
 
-// This is from NV_path_rendering, but the Mesa GL header is not up-to-date with
+// This is from NV_path_rendering, but the Mesa GL header is not up to date with
 // the most recent
 // version of the extension. This definition could be removed once glext.h
 // r27498 or later is
@@ -368,23 +376,30 @@ typedef void (*OSMESAproc)();
 
 // Forward declare EGL types.
 typedef uint64_t EGLuint64CHROMIUM;
+#ifndef EGL_VERSION_1_5
+typedef intptr_t EGLAttrib;
+#endif
+#ifndef EGL_KHR_stream
+typedef void *EGLStreamKHR;
+typedef uint64_t EGLuint64KHR;
+#endif
 
 #include "gl_bindings_autogen_gl.h"
 #include "gl_bindings_autogen_osmesa.h"
 
-#if defined(OS_WIN)
-#include "gl_bindings_autogen_egl.h"
-#include "gl_bindings_autogen_wgl.h"
-#elif defined(USE_X11)
-#include "gl_bindings_autogen_egl.h"
-#include "gl_bindings_autogen_glx.h"
-#elif defined(USE_OZONE)
-#include "gl_bindings_autogen_egl.h"
-#elif defined(OS_ANDROID)
+#if defined(USE_EGL)
 #include "gl_bindings_autogen_egl.h"
 #endif
 
-namespace gfx {
+#if defined(OS_WIN)
+#include "gl_bindings_autogen_wgl.h"
+#endif
+
+#if defined(USE_GLX)
+#include "gl_bindings_autogen_glx.h"
+#endif
+
+namespace gl {
 
 struct GL_EXPORT DriverGL {
   void InitializeStaticBindings();
@@ -436,7 +451,7 @@ struct GL_EXPORT DriverWGL {
 };
 #endif
 
-#if defined(OS_WIN) || defined(USE_X11) || defined(OS_ANDROID) || defined(USE_OZONE)
+#if defined(USE_EGL)
 struct GL_EXPORT DriverEGL {
   void InitializeStaticBindings();
   void InitializeExtensionBindings();
@@ -452,7 +467,7 @@ struct GL_EXPORT DriverEGL {
 };
 #endif
 
-#if defined(USE_X11)
+#if defined(USE_GLX)
 struct GL_EXPORT DriverGLX {
   void InitializeStaticBindings();
   void InitializeExtensionBindings();
@@ -476,32 +491,21 @@ GL_EXPORT extern OSMESAApi* g_current_osmesa_context;
 GL_EXPORT extern DriverGL g_driver_gl;
 GL_EXPORT extern DriverOSMESA g_driver_osmesa;
 
-#if defined(OS_WIN)
-
-GL_EXPORT extern EGLApi* g_current_egl_context;
-GL_EXPORT extern WGLApi* g_current_wgl_context;
-GL_EXPORT extern DriverEGL g_driver_egl;
-GL_EXPORT extern DriverWGL g_driver_wgl;
-
-#elif defined(USE_X11)
-
-GL_EXPORT extern EGLApi* g_current_egl_context;
-GL_EXPORT extern GLXApi* g_current_glx_context;
-GL_EXPORT extern DriverEGL g_driver_egl;
-GL_EXPORT extern DriverGLX g_driver_glx;
-
-#elif defined(USE_OZONE)
-
+#if defined(USE_EGL)
 GL_EXPORT extern EGLApi* g_current_egl_context;
 GL_EXPORT extern DriverEGL g_driver_egl;
-
-#elif defined(OS_ANDROID)
-
-GL_EXPORT extern EGLApi* g_current_egl_context;
-GL_EXPORT extern DriverEGL g_driver_egl;
-
 #endif
 
-}  // namespace gfx
+#if defined(OS_WIN)
+GL_EXPORT extern WGLApi* g_current_wgl_context;
+GL_EXPORT extern DriverWGL g_driver_wgl;
+#endif
+
+#if defined(USE_GLX)
+GL_EXPORT extern GLXApi* g_current_glx_context;
+GL_EXPORT extern DriverGLX g_driver_glx;
+#endif
+
+}  // namespace gl
 
 #endif  // UI_GL_GL_BINDINGS_H_
