@@ -19,7 +19,7 @@
 #include "ui/gl/gl_osmesa_api_implementation.h"
 #include "ui/gl/gl_switches.h"
 
-namespace gfx {
+namespace gl {
 namespace {
 
 // TODO(piman): it should be Desktop GL marshalling from double to float. Today
@@ -42,8 +42,8 @@ const char kGLLibraryName[] = "libGL.so.1";
 const char kGLESv2LibraryName[] = "libGLESv2.so.2";
 const char kEGLLibraryName[] = "libEGL.so.1";
 
-const char kGLESv2ANGLELibraryName[] = "libGLESv2_ANGLE.so";
-const char kEGLANGLELibraryName[] = "libEGL_ANGLE.so";
+const char kGLESv2ANGLELibraryName[] = "libGLESv2.so";
+const char kEGLANGLELibraryName[] = "libEGL.so";
 
 }  // namespace
 
@@ -148,8 +148,8 @@ bool InitializeStaticGLBindings(GLImplementation implementation) {
 
       // These two functions take single precision float rather than double
       // precision float parameters in GLES.
-      ::gfx::g_driver_gl.fn.glClearDepthFn = MarshalClearDepthToClearDepthf;
-      ::gfx::g_driver_gl.fn.glDepthRangeFn = MarshalDepthRangeToDepthRangef;
+      ::gl::g_driver_gl.fn.glClearDepthFn = MarshalClearDepthToClearDepthf;
+      ::gl::g_driver_gl.fn.glDepthRangeFn = MarshalDepthRangeToDepthRangef;
       break;
     }
     case kGLImplementationMockGL: {
@@ -218,4 +218,4 @@ bool GetGLWindowSystemBindingInfo(GLWindowSystemBindingInfo* info) {
   return false;
 }
 
-}  // namespace gfx
+}  // namespace gl
