@@ -7,18 +7,23 @@
 #include <stddef.h>
 #include <stdint.h>
 
+#include <map>
+#include <memory>
+#include <vector>
+
 #include "base/command_line.h"
 #include "base/logging.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "base/metrics/histogram_macros.h"
 #include "base/strings/string_number_conversions.h"
+#include "base/sys_info.h"
 #include "base/trace_event/trace_event.h"
 #include "build/build_config.h"
 #include "ui/gfx/geometry/rect.h"
 #include "ui/gl/egl_util.h"
 #include "ui/gl/gl_context.h"
+#include "ui/gl/gl_context_egl.h"
 #include "ui/gl/gl_image.h"
 #include "ui/gl/gl_implementation.h"
 #include "ui/gl/gl_surface_stub.h"
@@ -30,18 +35,13 @@
 #include <android/native_window_jni.h>
 #endif
 
-#if defined (USE_OZONE)
-#include "ui/ozone/public/ozone_platform.h"
-#include "ui/ozone/public/surface_factory_ozone.h"
-#endif
-
 #if defined(USE_X11) && !defined(OS_CHROMEOS)
 extern "C" {
 #include <X11/Xlib.h>
 #define Status int
 }
-#include "ui/base/x/x11_util_internal.h"
-#include "ui/gfx/x/x11_switches.h"
+#include "ui/base/x/x11_util_internal.h"  // nogncheck
+#include "ui/gfx/x/x11_switches.h"  // nogncheck
 #endif
 
 #if !defined(EGL_FIXED_SIZE_ANGLE)
@@ -104,7 +104,7 @@ extern "C" {
 
 using ui::GetLastEGLErrorString;
 
-namespace gfx {
+namespace gl {
 
 #if defined(OS_WIN)
 unsigned int NativeViewGLSurfaceEGL::current_swap_generation_ = 0;
@@ -116,7 +116,6 @@ const unsigned int MULTISWAP_FRAME_VSYNC_THRESHOLD = 60;
 
 namespace {
 
-EGLConfig g_config;
 EGLDisplay g_display;
 EGLNativeDisplayType g_native_display;
 
@@ -128,8 +127,7 @@ bool g_egl_surfaceless_context_supported = false;
 bool g_egl_surface_orientation_supported = false;
 bool g_use_direct_composition = false;
 
-class EGLSyncControlVSyncProvider
-    : public gfx::SyncControlVSyncProvider {
+class EGLSyncControlVSyncProvider : public SyncControlVSyncProvider {
  public:
   explicit EGLSyncControlVSyncProvider(EGLSurface surface)
       : SyncControlVSyncProvider(),
@@ -178,10 +176,14 @@ EGLDisplay GetPlatformANGLEDisplay(EGLNativeDisplayType native_display,
   }
 
 #if defined(USE_X11) && !defined(OS_CHROMEOS)
-  Visual* visual;
-  ui::ChooseVisualForWindow(&visual, nullptr);
+  std::string visualid_str =
+      base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
+          switches::kX11VisualID);
+  unsigned int visualid = 0;
+  bool succeed = base::StringToUint(visualid_str, &visualid);
+  DCHECK(succeed);
   display_attribs.push_back(EGL_X11_VISUAL_ID_ANGLE);
-  display_attribs.push_back((EGLint)visual->visualid);
+  display_attribs.push_back((EGLint)visualid);
 #endif
 
   display_attribs.push_back(EGL_NONE);
@@ -197,9 +199,6 @@ EGLDisplay GetDisplayFromType(DisplayType display_type,
     case DEFAULT:
     case SWIFT_SHADER:
       return eglGetDisplay(native_display);
-    case ANGLE_WARP:
-      return GetPlatformANGLEDisplay(native_display,
-                                     EGL_PLATFORM_ANGLE_TYPE_D3D11_ANGLE, true);
     case ANGLE_D3D9:
       return GetPlatformANGLEDisplay(native_display,
                                      EGL_PLATFORM_ANGLE_TYPE_D3D9_ANGLE, false);
@@ -224,8 +223,6 @@ const char* DisplayTypeString(DisplayType display_type) {
       return "Default";
     case SWIFT_SHADER:
       return "SwiftShader";
-    case ANGLE_WARP:
-      return "WARP";
     case ANGLE_D3D9:
       return "D3D9";
     case ANGLE_D3D11:
@@ -240,6 +237,156 @@ const char* DisplayTypeString(DisplayType display_type) {
   }
 }
 
+bool ValidateEglConfig(EGLDisplay display,
+                       const EGLint* config_attribs,
+                       EGLint* num_configs) {
+  if (!eglChooseConfig(display,
+                       config_attribs,
+                       NULL,
+                       0,
+                       num_configs)) {
+    LOG(ERROR) << "eglChooseConfig failed with error "
+               << GetLastEGLErrorString();
+    return false;
+  }
+  if (*num_configs == 0) {
+    LOG(ERROR) << "No suitable EGL configs found.";
+    return false;
+  }
+  return true;
+}
+
+EGLConfig ChooseConfig(GLSurface::Format format) {
+  static std::map<GLSurface::Format, EGLConfig> config_map;
+
+  if (config_map.find(format) != config_map.end()) {
+    return config_map[format];
+  }
+
+  // Choose an EGL configuration.
+  // On X this is only used for PBuffer surfaces.
+  EGLint renderable_type = EGL_OPENGL_ES2_BIT;
+  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kEnableUnsafeES3APIs)) {
+    renderable_type = EGL_OPENGL_ES3_BIT;
+  }
+
+  EGLint buffer_size = 32;
+  EGLint alpha_size = 8;
+
+#if defined(USE_X11) && !defined(OS_CHROMEOS)
+  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kWindowDepth)) {
+    std::string depth =
+        base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
+            switches::kWindowDepth);
+
+    bool succeed = base::StringToInt(depth, &buffer_size);
+    DCHECK(succeed);
+
+    alpha_size = buffer_size == 32 ? 8 : 0;
+  }
+#endif
+
+  EGLint surface_type = (format == GLSurface::SURFACE_SURFACELESS)
+                            ? EGL_DONT_CARE
+                            : EGL_WINDOW_BIT | EGL_PBUFFER_BIT;
+
+  EGLint config_attribs_8888[] = {
+    EGL_BUFFER_SIZE, buffer_size,
+    EGL_ALPHA_SIZE, alpha_size,
+    EGL_BLUE_SIZE, 8,
+    EGL_GREEN_SIZE, 8,
+    EGL_RED_SIZE, 8,
+    EGL_RENDERABLE_TYPE, renderable_type,
+    EGL_SURFACE_TYPE, surface_type,
+    EGL_NONE
+  };
+
+  EGLint* choose_attributes = config_attribs_8888;
+  EGLint config_attribs_565[] = {
+    EGL_BUFFER_SIZE, 16,
+    EGL_BLUE_SIZE, 5,
+    EGL_GREEN_SIZE, 6,
+    EGL_RED_SIZE, 5,
+    EGL_RENDERABLE_TYPE, EGL_OPENGL_ES2_BIT,
+    EGL_SURFACE_TYPE, surface_type,
+    EGL_NONE
+  };
+  if (format == GLSurface::SURFACE_RGB565) {
+    choose_attributes = config_attribs_565;
+  }
+
+  EGLint num_configs;
+  EGLint config_size = 1;
+  EGLConfig config = nullptr;
+  EGLConfig* config_data = &config;
+  // Validate if there are any configs for given attribs.
+  if (!ValidateEglConfig(g_display, choose_attributes, &num_configs)) {
+    return config;
+  }
+
+  std::unique_ptr<EGLConfig[]> matching_configs(new EGLConfig[num_configs]);
+  if (format == GLSurface::SURFACE_RGB565) {
+    config_size = num_configs;
+    config_data = matching_configs.get();
+  }
+
+  if (!eglChooseConfig(g_display, choose_attributes, config_data, config_size,
+                       &num_configs)) {
+    LOG(ERROR) << "eglChooseConfig failed with error "
+               << GetLastEGLErrorString();
+    return config;
+  }
+
+  if (format == GLSurface::SURFACE_RGB565) {
+    // Because of the EGL config sort order, we have to iterate
+    // through all of them (it'll put higher sum(R,G,B) bits
+    // first with the above attribs).
+    bool match_found = false;
+    for (int i = 0; i < num_configs; i++) {
+      EGLint red, green, blue, alpha;
+      // Read the relevant attributes of the EGLConfig.
+      if (eglGetConfigAttrib(g_display, matching_configs[i],
+                             EGL_RED_SIZE, &red) &&
+          eglGetConfigAttrib(g_display, matching_configs[i],
+                             EGL_BLUE_SIZE, &blue) &&
+          eglGetConfigAttrib(g_display, matching_configs[i],
+                             EGL_GREEN_SIZE, &green) &&
+          eglGetConfigAttrib(g_display, matching_configs[i],
+                             EGL_ALPHA_SIZE, &alpha) &&
+          alpha == 0 &&
+          red == 5 &&
+          green == 6 &&
+          blue == 5) {
+        config = matching_configs[i];
+        match_found = true;
+        break;
+      }
+    }
+    if (!match_found) {
+      // To fall back to default 32 bit format, choose with
+      // the right attributes again.
+      if (!ValidateEglConfig(g_display,
+                             config_attribs_8888,
+                             &num_configs)) {
+        return config;
+      }
+      if (!eglChooseConfig(g_display,
+                           config_attribs_8888,
+                           &config,
+                           1,
+                           &num_configs)) {
+        LOG(ERROR) << "eglChooseConfig failed with error "
+                   << GetLastEGLErrorString();
+        return config;
+      }
+    }
+  }
+  config_map[format] = config;
+  return config;
+}
+
 }  // namespace
 
 void GetEGLInitDisplays(bool supports_angle_d3d,
@@ -274,9 +421,6 @@ void GetEGLInitDisplays(bool supports_angle_d3d,
       if (requested_renderer == kANGLEImplementationD3D9Name) {
         init_displays->push_back(ANGLE_D3D9);
       }
-      if (requested_renderer == kANGLEImplementationWARPName) {
-        init_displays->push_back(ANGLE_WARP);
-      }
     }
   }
 
@@ -312,76 +456,6 @@ bool GLSurfaceEGL::InitializeOneOff() {
   if (g_display == EGL_NO_DISPLAY)
     return false;
 
-  // Choose an EGL configuration.
-  // On X this is only used for PBuffer surfaces.
-  EGLint renderable_type = EGL_OPENGL_ES2_BIT;
-  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kEnableUnsafeES3APIs)) {
-    renderable_type = EGL_OPENGL_ES3_BIT;
-  }
-
-  EGLint buffer_size = 32;
-  EGLint alpha_size = 8;
-
-#if defined(USE_X11) && !defined(OS_CHROMEOS)
-  if (base::CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kWindowDepth)) {
-    std::string depth =
-        base::CommandLine::ForCurrentProcess()->GetSwitchValueASCII(
-            switches::kWindowDepth);
-
-    bool succeed = base::StringToInt(depth, &buffer_size);
-    DCHECK(succeed);
-
-    alpha_size = buffer_size == 32 ? 8 : 0;
-  }
-#endif
-
-  const EGLint kConfigAttribs[] = {
-    EGL_BUFFER_SIZE, buffer_size,
-    EGL_ALPHA_SIZE, alpha_size,
-    EGL_BLUE_SIZE, 8,
-    EGL_GREEN_SIZE, 8,
-    EGL_RED_SIZE, 8,
-    EGL_RENDERABLE_TYPE, renderable_type,
-    EGL_SURFACE_TYPE, EGL_WINDOW_BIT | EGL_PBUFFER_BIT,
-    EGL_NONE
-  };
-
-#if defined(USE_OZONE)
-  const EGLint* config_attribs = ui::OzonePlatform::GetInstance()
-                                     ->GetSurfaceFactoryOzone()
-                                     ->GetEGLSurfaceProperties(kConfigAttribs);
-#else
-  const EGLint* config_attribs = kConfigAttribs;
-#endif
-
-  EGLint num_configs;
-  if (!eglChooseConfig(g_display,
-                       config_attribs,
-                       NULL,
-                       0,
-                       &num_configs)) {
-    LOG(ERROR) << "eglChooseConfig failed with error "
-               << GetLastEGLErrorString();
-    return false;
-  }
-
-  if (num_configs == 0) {
-    LOG(ERROR) << "No suitable EGL configs found.";
-    return false;
-  }
-
-  if (!eglChooseConfig(g_display,
-                       config_attribs,
-                       &g_config,
-                       1,
-                       &num_configs)) {
-    LOG(ERROR) << "eglChooseConfig failed with error "
-               << GetLastEGLErrorString();
-    return false;
-  }
-
   g_egl_extensions = eglQueryString(g_display, EGL_EXTENSIONS);
   g_egl_create_context_robustness_supported =
       HasEGLExtension("EGL_EXT_create_context_robustness");
@@ -397,8 +471,8 @@ bool GLSurfaceEGL::InitializeOneOff() {
   g_use_direct_composition =
       HasEGLExtension("EGL_ANGLE_direct_composition") &&
       HasEGLExtension("EGL_ANGLE_flexible_surface_compatibility") &&
-      base::CommandLine::ForCurrentProcess()->HasSwitch(
-          switches::kUseDirectComposition);
+      !base::CommandLine::ForCurrentProcess()->HasSwitch(
+          switches::kDisableDirectComposition);
 
   // TODO(oetuaho@nvidia.com): Surfaceless is disabled on Android as a temporary
   // workaround, since code written for Android WebView takes different paths
@@ -414,9 +488,9 @@ bool GLSurfaceEGL::InitializeOneOff() {
     // EGL_KHR_surfaceless_context is supported but ensure
     // GL_OES_surfaceless_context is also supported. We need a current context
     // to query for supported GL extensions.
-    scoped_refptr<GLSurface> surface = new SurfacelessEGL(Size(1, 1));
-    scoped_refptr<GLContext> context = GLContext::CreateGLContext(
-      NULL, surface.get(), PreferIntegratedGpu);
+    scoped_refptr<GLSurface> surface = new SurfacelessEGL(gfx::Size(1, 1));
+    scoped_refptr<GLContext> context = InitializeGLContext(
+        new GLContextEGL(nullptr), surface.get(), PreferIntegratedGpu);
     if (!context->MakeCurrent(surface.get()))
       g_egl_surfaceless_context_supported = false;
 
@@ -434,10 +508,21 @@ bool GLSurfaceEGL::InitializeOneOff() {
   return true;
 }
 
+GLSurface::Format GLSurfaceEGL::GetFormat() {
+  return format_;
+}
+
 EGLDisplay GLSurfaceEGL::GetDisplay() {
   return g_display;
 }
 
+EGLConfig GLSurfaceEGL::GetConfig() {
+  if (!config_) {
+    config_ = ChooseConfig(format_);
+  }
+  return config_;
+}
+
 EGLDisplay GLSurfaceEGL::GetHardwareDisplay() {
   return g_display;
 }
@@ -526,9 +611,7 @@ EGLDisplay GLSurfaceEGL::InitializeDisplay() {
 
 NativeViewGLSurfaceEGL::NativeViewGLSurfaceEGL(EGLNativeWindowType window)
     : window_(window),
-      config_(NULL),
       size_(1, 1),
-      alpha_(true),
       enable_fixed_size_angle_(false),
       surface_(NULL),
       supports_post_sub_buffer_(false),
@@ -548,12 +631,13 @@ NativeViewGLSurfaceEGL::NativeViewGLSurfaceEGL(EGLNativeWindowType window)
 #endif
 }
 
-bool NativeViewGLSurfaceEGL::Initialize() {
+bool NativeViewGLSurfaceEGL::Initialize(GLSurface::Format format) {
+  format_ = format;
   return Initialize(nullptr);
 }
 
 bool NativeViewGLSurfaceEGL::Initialize(
-    scoped_ptr<VSyncProvider> sync_provider) {
+    std::unique_ptr<gfx::VSyncProvider> sync_provider) {
   DCHECK(!surface_);
 
   if (!GetDisplay()) {
@@ -579,7 +663,7 @@ bool NativeViewGLSurfaceEGL::Initialize(
     egl_window_attributes.push_back(size_.height());
   }
 
-  if (gfx::g_driver_egl.ext.b_EGL_NV_post_sub_buffer) {
+  if (g_driver_egl.ext.b_EGL_NV_post_sub_buffer) {
     egl_window_attributes.push_back(EGL_POST_SUB_BUFFER_SUPPORTED_NV);
     egl_window_attributes.push_back(EGL_TRUE);
   }
@@ -616,7 +700,7 @@ bool NativeViewGLSurfaceEGL::Initialize(
     return false;
   }
 
-  if (gfx::g_driver_egl.ext.b_EGL_NV_post_sub_buffer) {
+  if (g_driver_egl.ext.b_EGL_NV_post_sub_buffer) {
     EGLint surfaceVal;
     EGLBoolean retVal = eglQuerySurface(
         GetDisplay(), surface_, EGL_POST_SUB_BUFFER_SUPPORTED_NV, &surfaceVal);
@@ -644,10 +728,6 @@ void NativeViewGLSurfaceEGL::Destroy() {
   }
 }
 
-EGLConfig NativeViewGLSurfaceEGL::GetConfig() {
-  return g_config;
-}
-
 bool NativeViewGLSurfaceEGL::IsOffscreen() {
   return false;
 }
@@ -722,13 +802,12 @@ gfx::Size NativeViewGLSurfaceEGL::GetSize() {
 bool NativeViewGLSurfaceEGL::Resize(const gfx::Size& size,
                                     float scale_factor,
                                     bool has_alpha) {
-  if (size == GetSize() && has_alpha == alpha_)
+  if (size == GetSize())
     return true;
 
-  alpha_ = has_alpha;
   size_ = size;
 
-  scoped_ptr<ui::ScopedMakeCurrent> scoped_make_current;
+  std::unique_ptr<ui::ScopedMakeCurrent> scoped_make_current;
   GLContext* current_context = GLContext::GetCurrent();
   bool was_current =
       current_context && current_context->IsCurrent(this);
@@ -740,7 +819,7 @@ bool NativeViewGLSurfaceEGL::Resize(const gfx::Size& size,
 
   Destroy();
 
-  if (!Initialize()) {
+  if (!Initialize(format_)) {
     LOG(ERROR) << "Failed to resize window.";
     return false;
   }
@@ -750,7 +829,7 @@ bool NativeViewGLSurfaceEGL::Resize(const gfx::Size& size,
 
 bool NativeViewGLSurfaceEGL::Recreate() {
   Destroy();
-  if (!Initialize()) {
+  if (!Initialize(format_)) {
     LOG(ERROR) << "Failed to create surface.";
     return false;
   }
@@ -769,6 +848,10 @@ bool NativeViewGLSurfaceEGL::FlipsVertically() const {
   return flips_vertically_;
 }
 
+bool NativeViewGLSurfaceEGL::BuffersFlipped() const {
+  return g_use_direct_composition;
+}
+
 gfx::SwapResult NativeViewGLSurfaceEGL::PostSubBuffer(int x,
                                                       int y,
                                                       int width,
@@ -809,15 +892,16 @@ gfx::SwapResult NativeViewGLSurfaceEGL::CommitOverlayPlanes() {
                                          : gfx::SwapResult::SWAP_FAILED;
 }
 
-VSyncProvider* NativeViewGLSurfaceEGL::GetVSyncProvider() {
+gfx::VSyncProvider* NativeViewGLSurfaceEGL::GetVSyncProvider() {
   return vsync_provider_.get();
 }
 
-bool NativeViewGLSurfaceEGL::ScheduleOverlayPlane(int z_order,
-                                                  OverlayTransform transform,
-                                                  gl::GLImage* image,
-                                                  const Rect& bounds_rect,
-                                                  const RectF& crop_rect) {
+bool NativeViewGLSurfaceEGL::ScheduleOverlayPlane(
+    int z_order,
+    gfx::OverlayTransform transform,
+    GLImage* image,
+    const gfx::Rect& bounds_rect,
+    const gfx::RectF& crop_rect) {
 #if !defined(OS_ANDROID)
   NOTIMPLEMENTED();
   return false;
@@ -861,7 +945,21 @@ PbufferGLSurfaceEGL::PbufferGLSurfaceEGL(const gfx::Size& size)
 }
 
 bool PbufferGLSurfaceEGL::Initialize() {
+  GLSurface::Format format = SURFACE_DEFAULT;
+#if defined(OS_ANDROID)
+  // This is to allow context virtualization which requires on- and offscreen
+  // to use a compatible config. We expect the client to request RGB565
+  // onscreen surface also for this to work (with the exception of
+  // fullscreen video).
+  if (base::SysInfo::IsLowEndDevice())
+    format = SURFACE_RGB565;
+#endif
+  return Initialize(format);
+}
+
+bool PbufferGLSurfaceEGL::Initialize(GLSurface::Format format) {
   EGLSurface old_surface = surface_;
+  format_ = format;
 
   EGLDisplay display = GetDisplay();
   if (!display) {
@@ -912,10 +1010,6 @@ void PbufferGLSurfaceEGL::Destroy() {
   }
 }
 
-EGLConfig PbufferGLSurfaceEGL::GetConfig() {
-  return g_config;
-}
-
 bool PbufferGLSurfaceEGL::IsOffscreen() {
   return true;
 }
@@ -935,7 +1029,7 @@ bool PbufferGLSurfaceEGL::Resize(const gfx::Size& size,
   if (size == size_)
     return true;
 
-  scoped_ptr<ui::ScopedMakeCurrent> scoped_make_current;
+  std::unique_ptr<ui::ScopedMakeCurrent> scoped_make_current;
   GLContext* current_context = GLContext::GetCurrent();
   bool was_current =
       current_context && current_context->IsCurrent(this);
@@ -946,7 +1040,7 @@ bool PbufferGLSurfaceEGL::Resize(const gfx::Size& size,
 
   size_ = size;
 
-  if (!Initialize()) {
+  if (!Initialize(format_)) {
     LOG(ERROR) << "Failed to resize pbuffer.";
     return false;
   }
@@ -963,10 +1057,10 @@ void* PbufferGLSurfaceEGL::GetShareHandle() {
   NOTREACHED();
   return NULL;
 #else
-  if (!gfx::g_driver_egl.ext.b_EGL_ANGLE_query_surface_pointer)
+  if (!g_driver_egl.ext.b_EGL_ANGLE_query_surface_pointer)
     return NULL;
 
-  if (!gfx::g_driver_egl.ext.b_EGL_ANGLE_surface_d3d_texture_2d_share_handle)
+  if (!g_driver_egl.ext.b_EGL_ANGLE_surface_d3d_texture_2d_share_handle)
     return NULL;
 
   void* handle;
@@ -987,19 +1081,21 @@ PbufferGLSurfaceEGL::~PbufferGLSurfaceEGL() {
 
 SurfacelessEGL::SurfacelessEGL(const gfx::Size& size)
     : size_(size) {
+  format_ = GLSurface::SURFACE_SURFACELESS;
 }
 
 bool SurfacelessEGL::Initialize() {
+  return Initialize(SURFACE_SURFACELESS);
+}
+
+bool SurfacelessEGL::Initialize(GLSurface::Format format) {
+  format_ = format;
   return true;
 }
 
 void SurfacelessEGL::Destroy() {
 }
 
-EGLConfig SurfacelessEGL::GetConfig() {
-  return g_config;
-}
-
 bool SurfacelessEGL::IsOffscreen() {
   return true;
 }
@@ -1035,4 +1131,4 @@ void* SurfacelessEGL::GetShareHandle() {
 SurfacelessEGL::~SurfacelessEGL() {
 }
 
-}  // namespace gfx
+}  // namespace gl
