@@ -4,10 +4,11 @@
 
 #include "ui/keyboard/keyboard_controller.h"
 
+#include <memory>
+
 #include "base/bind.h"
 #include "base/command_line.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/message_loop/message_loop.h"
 #include "testing/gtest/include/gtest/gtest.h"
 #include "ui/aura/client/focus_client.h"
@@ -96,7 +97,7 @@ class TestKeyboardUI : public KeyboardUI {
   }
 
   // Overridden from KeyboardUI:
-  bool HasKeyboardWindow() const override { return window_; }
+  bool HasKeyboardWindow() const override { return !!window_; }
   bool ShouldWindowOverscroll(aura::Window* window) const override {
     return true;
   }
@@ -115,7 +116,7 @@ class TestKeyboardUI : public KeyboardUI {
   void ResetInsets() override {}
 
  private:
-  scoped_ptr<aura::Window> window_;
+  std::unique_ptr<aura::Window> window_;
   aura::test::TestWindowDelegate delegate_;
   ui::InputMethod* input_method_;
 
@@ -205,11 +206,8 @@ class KeyboardControllerTest : public testing::Test,
 
   void MockRotateScreen() {
     const gfx::Rect root_bounds = root_window()->bounds();
-    controller_->OnWindowBoundsChanged(root_window(), gfx::Rect(),
-                                       gfx::Rect(0,
-                                                 0,
-                                                 root_bounds.height(),
-                                                 root_bounds.width()));
+    root_window()->SetBounds(
+        gfx::Rect(0, 0, root_bounds.height(), root_bounds.width()));
   }
 
  protected:
@@ -250,15 +248,15 @@ class KeyboardControllerTest : public testing::Test,
   }
 
   base::MessageLoopForUI message_loop_;
-  scoped_ptr<aura::test::AuraTestHelper> aura_test_helper_;
-  scoped_ptr<TestFocusController> focus_controller_;
+  std::unique_ptr<aura::test::AuraTestHelper> aura_test_helper_;
+  std::unique_ptr<TestFocusController> focus_controller_;
 
  private:
   int number_of_calls_;
   gfx::Rect notified_bounds_;
   KeyboardUI* ui_;
-  scoped_ptr<KeyboardController> controller_;
-  scoped_ptr<ui::TextInputClient> test_text_input_client_;
+  std::unique_ptr<KeyboardController> controller_;
+  std::unique_ptr<ui::TextInputClient> test_text_input_client_;
   DISALLOW_COPY_AND_ASSIGN(KeyboardControllerTest);
 };
 
@@ -274,7 +272,6 @@ TEST_F(KeyboardControllerTest, KeyboardSize) {
   ASSERT_EQ(gfx::Rect(), initial_bounds);
   VerifyKeyboardWindowSize(container, keyboard);
 
-
   // In FULL_WIDTH mode, attempt to change window width or move window up from
   // the bottom are ignored. Changing window height is supported.
   gfx::Rect expected_bounds(0,
@@ -304,7 +301,6 @@ TEST_F(KeyboardControllerTest, KeyboardSize) {
 TEST_F(KeyboardControllerTest, FloatingKeyboardSize) {
   aura::Window* container(controller()->GetContainerWindow());
   aura::Window* keyboard(ui()->GetKeyboardWindow());
-  gfx::Rect screen_bounds = root_window()->bounds();
   root_window()->AddChild(container);
   controller()->SetKeyboardMode(FLOATING);
   container->AddChild(keyboard);
@@ -316,9 +312,10 @@ TEST_F(KeyboardControllerTest, FloatingKeyboardSize) {
 
 // Tests that tapping/clicking inside the keyboard does not give it focus.
 TEST_F(KeyboardControllerTest, ClickDoesNotFocusKeyboard) {
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   const gfx::Rect& root_bounds = root_window()->bounds();
   aura::test::EventCountDelegate delegate;
-  scoped_ptr<aura::Window> window(new aura::Window(&delegate));
+  std::unique_ptr<aura::Window> window(new aura::Window(&delegate));
   window->Init(ui::LAYER_NOT_DRAWN);
   window->SetBounds(root_bounds);
   root_window()->AddChild(window.get());
@@ -356,9 +353,11 @@ TEST_F(KeyboardControllerTest, ClickDoesNotFocusKeyboard) {
   generator.ClickLeftButton();
   EXPECT_EQ("1 1", delegate.GetMouseButtonCountsAndReset());
   keyboard_container->RemovePreTargetHandler(&observer);
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 TEST_F(KeyboardControllerTest, VisibilityChangeWithTextInputTypeChange) {
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   ui::DummyTextInputClient input_client_0(ui::TEXT_INPUT_TYPE_TEXT);
   ui::DummyTextInputClient input_client_1(ui::TEXT_INPUT_TYPE_TEXT);
   ui::DummyTextInputClient input_client_2(ui::TEXT_INPUT_TYPE_TEXT);
@@ -366,7 +365,7 @@ TEST_F(KeyboardControllerTest, VisibilityChangeWithTextInputTypeChange) {
   ui::DummyTextInputClient no_input_client_1(ui::TEXT_INPUT_TYPE_NONE);
 
   aura::Window* keyboard_container(controller()->GetContainerWindow());
-  scoped_ptr<KeyboardContainerObserver> keyboard_container_observer(
+  std::unique_ptr<KeyboardContainerObserver> keyboard_container_observer(
       new KeyboardContainerObserver(keyboard_container));
   root_window()->AddChild(keyboard_container);
 
@@ -394,6 +393,7 @@ TEST_F(KeyboardControllerTest, VisibilityChangeWithTextInputTypeChange) {
 
   EXPECT_FALSE(WillHideKeyboard());
   EXPECT_TRUE(keyboard_container->IsVisible());
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 // Test to prevent spurious overscroll boxes when changing tabs during keyboard
@@ -423,12 +423,13 @@ TEST_F(KeyboardControllerTest, CheckOverscrollInsetDuringVisibilityChange) {
 // Verify switch to FLOATING mode will reset the overscroll or resize and when
 // in FLOATING mode, overscroll or resize wont be triggered.
 TEST_F(KeyboardControllerTest, FloatingKeyboardDontOverscrollOrResize) {
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   ui::DummyTextInputClient input_client(ui::TEXT_INPUT_TYPE_TEXT);
   ui::DummyTextInputClient no_input_client(ui::TEXT_INPUT_TYPE_NONE);
 
   aura::Window* container(controller()->GetContainerWindow());
   root_window()->AddChild(container);
-  scoped_ptr<KeyboardContainerObserver> keyboard_container_observer(
+  std::unique_ptr<KeyboardContainerObserver> keyboard_container_observer(
       new KeyboardContainerObserver(container));
   gfx::Rect screen_bounds = root_window()->bounds();
   keyboard::SetTouchKeyboardEnabled(true);
@@ -453,6 +454,7 @@ TEST_F(KeyboardControllerTest, FloatingKeyboardDontOverscrollOrResize) {
   // In FLOATING mode, no overscroll or resize should be triggered.
   EXPECT_EQ(3, number_of_calls());
   EXPECT_EQ(gfx::Rect(), controller()->current_keyboard_bounds());
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 // Verify switch to FULL_WIDTH mode will move virtual keyboard to the right
@@ -479,13 +481,14 @@ TEST_F(KeyboardControllerTest, SwitchToFullWidthVirtualKeyboard) {
 }
 
 TEST_F(KeyboardControllerTest, AlwaysVisibleWhenLocked) {
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   ui::DummyTextInputClient input_client_0(ui::TEXT_INPUT_TYPE_TEXT);
   ui::DummyTextInputClient input_client_1(ui::TEXT_INPUT_TYPE_TEXT);
   ui::DummyTextInputClient no_input_client_0(ui::TEXT_INPUT_TYPE_NONE);
   ui::DummyTextInputClient no_input_client_1(ui::TEXT_INPUT_TYPE_NONE);
 
   aura::Window* keyboard_container(controller()->GetContainerWindow());
-  scoped_ptr<KeyboardContainerObserver> keyboard_container_observer(
+  std::unique_ptr<KeyboardContainerObserver> keyboard_container_observer(
       new KeyboardContainerObserver(keyboard_container));
   root_window()->AddChild(keyboard_container);
 
@@ -514,6 +517,7 @@ TEST_F(KeyboardControllerTest, AlwaysVisibleWhenLocked) {
   // Wait for hide keyboard to finish.
   base::MessageLoop::current()->Run();
   EXPECT_FALSE(keyboard_container->IsVisible());
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 class KeyboardControllerAnimationTest : public KeyboardControllerTest {
@@ -553,6 +557,7 @@ class KeyboardControllerAnimationTest : public KeyboardControllerTest {
 // Tests virtual keyboard has correct show and hide animation.
 TEST_F(KeyboardControllerAnimationTest, ContainerAnimation) {
   ui::Layer* layer = keyboard_container()->layer();
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   ShowKeyboard();
 
   // Keyboard container and window should immediately become visible before
@@ -593,12 +598,14 @@ TEST_F(KeyboardControllerAnimationTest, ContainerAnimation) {
   EXPECT_GT(hide_start_opacity, hide_end_opacity);
   EXPECT_EQ(transform, layer->transform());
   EXPECT_EQ(gfx::Rect(), notified_bounds());
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 // Show keyboard during keyboard hide animation should abort the hide animation
 // and the keyboard should animate in.
 // Test for crbug.com/333284.
 TEST_F(KeyboardControllerAnimationTest, ContainerShowWhileHide) {
+  keyboard::SetAccessibilityKeyboardEnabled(true);
   ui::Layer* layer = keyboard_container()->layer();
   ShowKeyboard();
   RunAnimationForLayer(layer);
@@ -611,6 +618,7 @@ TEST_F(KeyboardControllerAnimationTest, ContainerShowWhileHide) {
   EXPECT_TRUE(keyboard_window()->IsVisible());
   EXPECT_EQ(1.0, layer->opacity());
   EXPECT_EQ(gfx::Transform(), layer->transform());
+  keyboard::SetAccessibilityKeyboardEnabled(false);
 }
 
 // Test for crbug.com/568274.
@@ -631,4 +639,22 @@ TEST_F(KeyboardControllerTest, FloatingKeyboardShowOnFirstTap) {
   EXPECT_TRUE(container->IsVisible());
 }
 
+TEST_F(KeyboardControllerTest, DisplayChangeShouldNotifyBoundsChange) {
+  ui::DummyTextInputClient input_client(ui::TEXT_INPUT_TYPE_TEXT);
+
+  aura::Window* container(controller()->GetContainerWindow());
+  root_window()->AddChild(container);
+
+  keyboard::SetTouchKeyboardEnabled(true);
+  controller()->SetKeyboardMode(FULL_WIDTH);
+  SetFocus(&input_client);
+  gfx::Rect new_bounds(0, 0, 1280, 800);
+  ASSERT_NE(new_bounds, root_window()->bounds());
+  EXPECT_EQ(1, number_of_calls());
+  root_window()->SetBounds(new_bounds);
+  EXPECT_EQ(2, number_of_calls());
+  MockRotateScreen();
+  EXPECT_EQ(3, number_of_calls());
+}
+
 }  // namespace keyboard
