@@ -3,6 +3,9 @@
 // found in the LICENSE file.
 
 #include "base/command_line.h"
+#include "base/location.h"
+#include "base/single_thread_task_runner.h"
+#include "base/threading/thread_task_runner_handle.h"
 #include "ui/gfx/animation/slide_animation.h"
 #include "ui/message_center/message_center_style.h"
 #include "ui/message_center/message_center_switches.h"
@@ -191,17 +194,24 @@ void MessageListView::ResetRepositionSession() {
   fixed_height_ = 0;
 }
 
-void MessageListView::ClearAllNotifications(
+void MessageListView::ClearAllClosableNotifications(
     const gfx::Rect& visible_scroll_rect) {
   for (int i = 0; i < child_count(); ++i) {
-    views::View* child = child_at(i);
+    // Safe cast since all views in MessageListView are MessageViews.
+    MessageView* child = (MessageView*)child_at(i);
     if (!child->visible())
       continue;
     if (gfx::IntersectRects(child->bounds(), visible_scroll_rect).IsEmpty())
       continue;
+    if (child->IsPinned())
+      continue;
     clearing_all_views_.push_back(child);
   }
-  DoUpdateIfPossible();
+  if (clearing_all_views_.empty()) {
+    message_center_view()->OnAllNotificationsCleared();
+  } else {
+    DoUpdateIfPossible();
+  }
 }
 
 void MessageListView::OnBoundsAnimatorProgressed(
@@ -449,7 +459,7 @@ void MessageListView::AnimateClearingOneNotification() {
 
   // Schedule to start sliding out next notification after a short delay.
   if (!clearing_all_views_.empty()) {
-    base::MessageLoop::current()->PostDelayedTask(
+    base::ThreadTaskRunnerHandle::Get()->PostDelayedTask(
         FROM_HERE, base::Bind(&MessageListView::AnimateClearingOneNotification,
                               weak_ptr_factory_.GetWeakPtr()),
         base::TimeDelta::FromMilliseconds(
