@@ -13,6 +13,7 @@
 #include "ui/gfx/shadow_value.h"
 #include "ui/message_center/message_center.h"
 #include "ui/message_center/message_center_style.h"
+#include "ui/message_center/views/message_center_controller.h"
 #include "ui/message_center/views/padded_button.h"
 #include "ui/resources/grit/ui_resources.h"
 #include "ui/strings/grit/ui_strings.h"
@@ -26,28 +27,23 @@
 
 namespace {
 
-const int kCloseIconTopPadding = 5;
-const int kCloseIconRightPadding = 5;
+constexpr int kCloseIconTopPadding = 5;
+constexpr int kCloseIconRightPadding = 5;
 
-const int kShadowOffset = 1;
-const int kShadowBlur = 4;
+constexpr int kShadowOffset = 1;
+constexpr int kShadowBlur = 4;
 
 }  // namespace
 
 namespace message_center {
 
-MessageView::MessageView(MessageViewController* controller,
-                         const std::string& notification_id,
-                         const NotifierId& notifier_id,
-                         const gfx::ImageSkia& small_image,
-                         const base::string16& display_source)
+MessageView::MessageView(MessageCenterController* controller,
+                         const Notification& notification)
     : controller_(controller),
-      notification_id_(notification_id),
-      notifier_id_(notifier_id),
-      background_view_(NULL),
-      scroller_(NULL),
-      display_source_(display_source) {
-  SetFocusable(true);
+      notification_id_(notification.id()),
+      notifier_id_(notification.notifier_id()),
+      display_source_(notification.display_source()) {
+  SetFocusBehavior(FocusBehavior::ALWAYS);
 
   // Create the opaque background that's above the view's shadow.
   background_view_ = new views::View();
@@ -56,26 +52,13 @@ MessageView::MessageView(MessageViewController* controller,
   AddChildView(background_view_);
 
   views::ImageView* small_image_view = new views::ImageView();
-  small_image_view->SetImage(small_image);
+  small_image_view->SetImage(notification.small_image().AsImageSkia());
   small_image_view->SetImageSize(gfx::Size(kSmallImageSize, kSmallImageSize));
   // The small image view should be added to view hierarchy by the derived
   // class. This ensures that it is on top of other views.
   small_image_view->set_owned_by_client();
   small_image_view_.reset(small_image_view);
 
-  PaddedButton *close = new PaddedButton(this);
-  close->SetPadding(-kCloseIconRightPadding, kCloseIconTopPadding);
-  close->SetNormalImage(IDR_NOTIFICATION_CLOSE);
-  close->SetHoveredImage(IDR_NOTIFICATION_CLOSE_HOVER);
-  close->SetPressedImage(IDR_NOTIFICATION_CLOSE_PRESSED);
-  close->set_animate_on_state_change(false);
-  close->SetAccessibleName(l10n_util::GetStringUTF16(
-      IDS_MESSAGE_CENTER_CLOSE_NOTIFICATION_BUTTON_ACCESSIBLE_NAME));
-  // The close button should be added to view hierarchy by the derived class.
-  // This ensures that it is on top of other views.
-  close->set_owned_by_client();
-  close_button_.reset(close);
-
   focus_painter_ = views::Painter::CreateSolidFocusPainter(
       kFocusBorderColor, gfx::Insets(0, 1, 3, 2));
 }
@@ -86,6 +69,7 @@ MessageView::~MessageView() {
 void MessageView::UpdateWithNotification(const Notification& notification) {
   small_image_view_->SetImage(notification.small_image().AsImageSkia());
   display_source_ = notification.display_source();
+  CreateOrUpdateCloseButtonView(notification);
 }
 
 // static
@@ -97,18 +81,27 @@ gfx::Insets MessageView::GetShadowInsets() {
 }
 
 void MessageView::CreateShadowBorder() {
-  SetBorder(scoped_ptr<views::Border>(new views::ShadowBorder(
+  SetBorder(std::unique_ptr<views::Border>(new views::ShadowBorder(
       gfx::ShadowValue(gfx::Vector2d(0, kShadowOffset), kShadowBlur,
                        message_center::kShadowColor))));
 }
 
 bool MessageView::IsCloseButtonFocused() {
+  if (!close_button_)
+    return false;
+
   views::FocusManager* focus_manager = GetFocusManager();
-  return focus_manager && focus_manager->GetFocusedView() == close_button();
+  return focus_manager &&
+         focus_manager->GetFocusedView() == close_button_.get();
 }
 
 void MessageView::RequestFocusOnCloseButton() {
-  close_button_->RequestFocus();
+  if (close_button_)
+    close_button_->RequestFocus();
+}
+
+bool MessageView::IsPinned() {
+  return !close_button_;
 }
 
 void MessageView::GetAccessibleState(ui::AXViewState* state) {
@@ -151,8 +144,8 @@ bool MessageView::OnKeyReleased(const ui::KeyEvent& event) {
 }
 
 void MessageView::OnPaint(gfx::Canvas* canvas) {
-  DCHECK_EQ(this, close_button_->parent());
   DCHECK_EQ(this, small_image_view_->parent());
+  DCHECK_EQ(this, close_button_->parent());
   SlideOutView::OnPaint(canvas);
   views::Painter::PaintFocusPainter(this, canvas, focus_painter_.get());
 }
@@ -176,12 +169,14 @@ void MessageView::Layout() {
   background_view_->SetBoundsRect(content_bounds);
 
   // Close button.
-  gfx::Size close_size(close_button_->GetPreferredSize());
-  gfx::Rect close_rect(content_bounds.right() - close_size.width(),
-                       content_bounds.y(),
-                       close_size.width(),
-                       close_size.height());
-  close_button_->SetBoundsRect(close_rect);
+  if (close_button_) {
+    gfx::Rect content_bounds = GetContentsBounds();
+    gfx::Size close_size(close_button_->GetPreferredSize());
+    gfx::Rect close_rect(content_bounds.right() - close_size.width(),
+                         content_bounds.y(), close_size.width(),
+                         close_size.height());
+    close_button_->SetBoundsRect(close_rect);
+  }
 
   gfx::Size small_image_size(small_image_view_->GetPreferredSize());
   gfx::Rect small_image_rect(small_image_size);
@@ -229,8 +224,8 @@ void MessageView::OnGestureEvent(ui::GestureEvent* event) {
 
 void MessageView::ButtonPressed(views::Button* sender,
                                 const ui::Event& event) {
-  if (sender == close_button()) {
-    controller_->RemoveNotification(notification_id_, true);  // By user.
+  if (close_button_ && sender == close_button_.get()) {
+    controller()->RemoveNotification(notification_id(), true);  // By user.
   }
 }
 
@@ -247,4 +242,25 @@ void MessageView::SetDrawBackgroundAsActive(bool active) {
   SchedulePaint();
 }
 
+void MessageView::CreateOrUpdateCloseButtonView(
+    const Notification& notification) {
+  set_slide_out_enabled(!notification.pinned());
+
+  if (!notification.pinned() && !close_button_) {
+    PaddedButton* close = new PaddedButton(this);
+    close->SetPadding(-kCloseIconRightPadding, kCloseIconTopPadding);
+    close->SetNormalImage(IDR_NOTIFICATION_CLOSE);
+    close->SetHoveredImage(IDR_NOTIFICATION_CLOSE_HOVER);
+    close->SetPressedImage(IDR_NOTIFICATION_CLOSE_PRESSED);
+    close->set_animate_on_state_change(false);
+    close->SetAccessibleName(l10n_util::GetStringUTF16(
+        IDS_MESSAGE_CENTER_CLOSE_NOTIFICATION_BUTTON_ACCESSIBLE_NAME));
+    close->set_owned_by_client();
+    AddChildView(close);
+    close_button_.reset(close);
+  } else if (notification.pinned() && close_button_) {
+    close_button_.reset();
+  }
+}
+
 }  // namespace message_center
