@@ -50,25 +50,23 @@
 #include "ui/views/widget/widget.h"
 #include "url/gurl.h"
 
-#if defined(OS_WIN)
-#include "ui/base/win/shell.h"
-#endif
-
 namespace {
 
 // Dimensions.
 const int kProgressBarBottomPadding = 0;
 
 // static
-scoped_ptr<views::Border> MakeEmptyBorder(int top,
-                                          int left,
-                                          int bottom,
-                                          int right) {
+std::unique_ptr<views::Border> MakeEmptyBorder(int top,
+                                               int left,
+                                               int bottom,
+                                               int right) {
   return views::Border::CreateEmptyBorder(top, left, bottom, right);
 }
 
 // static
-scoped_ptr<views::Border> MakeTextBorder(int padding, int top, int bottom) {
+std::unique_ptr<views::Border> MakeTextBorder(int padding,
+                                              int top,
+                                              int bottom) {
   // Split the padding between the top and the bottom, then add the extra space.
   return MakeEmptyBorder(padding / 2 + top,
                          message_center::kTextLeftPadding,
@@ -77,7 +75,7 @@ scoped_ptr<views::Border> MakeTextBorder(int padding, int top, int bottom) {
 }
 
 // static
-scoped_ptr<views::Border> MakeProgressBarBorder(int top, int bottom) {
+std::unique_ptr<views::Border> MakeProgressBarBorder(int top, int bottom) {
   return MakeEmptyBorder(top,
                          message_center::kTextLeftPadding,
                          bottom,
@@ -85,38 +83,12 @@ scoped_ptr<views::Border> MakeProgressBarBorder(int top, int bottom) {
 }
 
 // static
-scoped_ptr<views::Border> MakeSeparatorBorder(int top,
-                                              int left,
-                                              SkColor color) {
+std::unique_ptr<views::Border> MakeSeparatorBorder(int top,
+                                                   int left,
+                                                   SkColor color) {
   return views::Border::CreateSolidSidedBorder(top, left, 0, 0, color);
 }
 
-// static
-// Return true if and only if the image is null or has alpha.
-bool HasAlpha(gfx::ImageSkia& image, views::Widget* widget) {
-  // Determine which bitmap to use.
-  float factor = 1.0f;
-  if (widget)
-    factor = ui::GetScaleFactorForNativeView(widget->GetNativeView());
-
-  // Extract that bitmap's alpha and look for a non-opaque pixel there.
-  SkBitmap bitmap = image.GetRepresentation(factor).sk_bitmap();
-  if (!bitmap.isNull()) {
-    SkBitmap alpha;
-    bitmap.extractAlpha(&alpha);
-    for (int y = 0; y < bitmap.height(); ++y) {
-      for (int x = 0; x < bitmap.width(); ++x) {
-        if (alpha.getColor(x, y) != SK_ColorBLACK) {
-          return true;
-        }
-      }
-    }
-  }
-
-  // If no opaque pixel was found, return false unless the bitmap is empty.
-  return bitmap.isNull();
-}
-
 // ItemView ////////////////////////////////////////////////////////////////////
 
 // ItemViews are responsible for drawing each list notification item's title and
@@ -170,49 +142,6 @@ namespace message_center {
 
 // NotificationView ////////////////////////////////////////////////////////////
 
-// static
-NotificationView* NotificationView::Create(MessageCenterController* controller,
-                                           const Notification& notification,
-                                           bool top_level) {
-  switch (notification.type()) {
-    case NOTIFICATION_TYPE_BASE_FORMAT:
-    case NOTIFICATION_TYPE_IMAGE:
-    case NOTIFICATION_TYPE_MULTIPLE:
-    case NOTIFICATION_TYPE_SIMPLE:
-    case NOTIFICATION_TYPE_PROGRESS:
-      break;
-    default:
-      // If the caller asks for an unrecognized kind of view (entirely possible
-      // if an application is running on an older version of this code that
-      // doesn't have the requested kind of notification template), we'll fall
-      // back to a notification instance that will provide at least basic
-      // functionality.
-      LOG(WARNING) << "Unable to fulfill request for unrecognized "
-                   << "notification type " << notification.type() << ". "
-                   << "Falling back to simple notification type.";
-  }
-
-  // Currently all roads lead to the generic NotificationView.
-  NotificationView* notification_view =
-      new NotificationView(controller, notification);
-
-#if defined(OS_LINUX) && !defined(OS_CHROMEOS)
-  // Don't create shadows for notification toasts on linux wih aura.
-  if (top_level)
-    return notification_view;
-#endif
-
-#if defined(OS_WIN)
-  // Don't create shadows for notifications on Windows under classic theme.
-  if (top_level && !ui::win::IsAeroGlassEnabled()) {
-    return notification_view;
-  }
-#endif  // OS_WIN
-
-  notification_view->CreateShadowBorder();
-  return notification_view;
-}
-
 views::View* NotificationView::TargetForRect(views::View* root,
                                              const gfx::Rect& rect) {
   CHECK_EQ(root, this);
@@ -228,7 +157,8 @@ views::View* NotificationView::TargetForRect(views::View* root,
                                     action_buttons_.end());
   if (settings_button_view_)
     buttons.push_back(settings_button_view_);
-  buttons.push_back(close_button());
+  if (close_button())
+    buttons.push_back(close_button());
 
   for (size_t i = 0; i < buttons.size(); ++i) {
     gfx::Point point_in_child = point;
@@ -268,23 +198,8 @@ void NotificationView::SetAccessibleName(const Notification& notification) {
 
 NotificationView::NotificationView(MessageCenterController* controller,
                                    const Notification& notification)
-    : MessageView(this,
-                  notification.id(),
-                  notification.notifier_id(),
-                  notification.small_image().AsImageSkia(),
-                  notification.display_source()),
-      controller_(controller),
-      clickable_(notification.clickable()),
-      top_view_(NULL),
-      title_view_(NULL),
-      message_view_(NULL),
-      context_message_view_(NULL),
-      settings_button_view_(NULL),
-      icon_view_(NULL),
-      bottom_view_(NULL),
-      image_container_(NULL),
-      image_view_(NULL),
-      progress_bar_view_(NULL) {
+    : MessageView(controller, notification),
+      clickable_(notification.clickable()) {
   // Create the top_view_, which collects into a vertical box all content
   // at the top of the notification (to the right of the icon) except for the
   // close button.
@@ -308,11 +223,11 @@ NotificationView::NotificationView(MessageCenterController* controller,
   // image to overlap the content as needed to provide large enough click and
   // touch areas (<http://crbug.com/168822> and <http://crbug.com/168856>).
   AddChildView(small_image());
-  AddChildView(close_button());
+  CreateOrUpdateCloseButtonView(notification);
   SetAccessibleName(notification);
 
   SetEventTargeter(
-      scoped_ptr<views::ViewTargeter>(new views::ViewTargeter(this)));
+      std::unique_ptr<views::ViewTargeter>(new views::ViewTargeter(this)));
 }
 
 NotificationView::~NotificationView() {
@@ -349,13 +264,16 @@ int NotificationView::GetHeightForWidth(int width) const {
     }
   }
 
-  int content_height = std::max(top_height, kIconSize) + bottom_height;
+  int content_height =
+      std::max(top_height, kNotificationIconSize) + bottom_height;
 
   // Adjust the height to make sure there is at least 16px of space below the
   // icon if there is any space there (<http://crbug.com/232966>).
-  if (content_height > kIconSize)
-    content_height = std::max(content_height,
-                              kIconSize + message_center::kIconBottomPadding);
+  if (content_height > kNotificationIconSize) {
+    content_height =
+        std::max(content_height,
+                 kNotificationIconSize + message_center::kIconBottomPadding);
+  }
 
   return content_height + GetInsets().height();
 }
@@ -379,10 +297,11 @@ void NotificationView::Layout() {
   top_view_->SetBounds(insets.left(), insets.top(), content_width, top_height);
 
   // Icon.
-  icon_view_->SetBounds(insets.left(), insets.top(), kIconSize, kIconSize);
+  icon_view_->SetBounds(insets.left(), insets.top(), kNotificationIconSize,
+                        kNotificationIconSize);
 
   // Settings & Bottom views.
-  int bottom_y = insets.top() + std::max(top_height, kIconSize);
+  int bottom_y = insets.top() + std::max(top_height, kNotificationIconSize);
   int bottom_height = bottom_view_->GetHeightForWidth(content_width);
 
   if (settings_button_view_) {
@@ -409,7 +328,7 @@ void NotificationView::ScrollRectToVisible(const gfx::Rect& rect) {
 }
 
 gfx::NativeCursor NotificationView::GetCursor(const ui::MouseEvent& event) {
-  if (!clickable_ || !controller_->HasClickedListener(notification_id()))
+  if (!clickable_ || !controller()->HasClickedListener(notification_id()))
     return views::View::GetCursor(event);
 
   return views::GetNativeHandCursor();
@@ -433,14 +352,14 @@ void NotificationView::ButtonPressed(views::Button* sender,
   std::string id(notification_id());
 
   if (sender == settings_button_view_) {
-    controller_->ClickOnSettingsButton(id);
+    controller()->ClickOnSettingsButton(id);
     return;
   }
 
   // See if the button pressed was an action button.
   for (size_t i = 0; i < action_buttons_.size(); ++i) {
     if (sender == action_buttons_[i]) {
-      controller_->ClickOnNotificationButton(id, i);
+      controller()->ClickOnNotificationButton(id, i);
       return;
     }
   }
@@ -451,15 +370,6 @@ void NotificationView::ButtonPressed(views::Button* sender,
   MessageView::ButtonPressed(sender, event);
 }
 
-void NotificationView::ClickOnNotification(const std::string& notification_id) {
-  controller_->ClickOnNotification(notification_id);
-}
-
-void NotificationView::RemoveNotification(const std::string& notification_id,
-                                          bool by_user) {
-  controller_->RemoveNotification(notification_id, by_user);
-}
-
 void NotificationView::CreateOrUpdateTitleView(
     const Notification& notification) {
   if (notification.title().empty()) {
@@ -533,12 +443,10 @@ base::string16 NotificationView::FormatContextMessage(
   if (notification.UseOriginAsContextMessage()) {
     const GURL url = notification.origin_url();
     DCHECK(url.is_valid());
-    // TODO(palmer): Find a way to get the Profile's real languages.
-    // crbug.com/496965.
-    return gfx::ElideText(url_formatter::FormatUrlForSecurityDisplayOmitScheme(
-                              url, std::string()),
-                          views::Label().font_list(), kContextMessageViewWidth,
-                          gfx::ELIDE_HEAD);
+    return gfx::ElideText(
+        url_formatter::FormatUrlForSecurityDisplay(
+            url, url_formatter::SchemeDisplay::OMIT_HTTP_AND_HTTPS),
+        views::Label().font_list(), kContextMessageViewWidth, gfx::ELIDE_HEAD);
   }
 
   return gfx::TruncateString(notification.context_message(),
@@ -663,24 +571,20 @@ void NotificationView::CreateOrUpdateListItemViews(
 
 void NotificationView::CreateOrUpdateIconView(
     const Notification& notification) {
+  gfx::Size image_view_size(kNotificationIconSize, kNotificationIconSize);
+
   if (!icon_view_) {
-    icon_view_ = new ProportionalImageView(gfx::Size(kIconSize, kIconSize));
+    icon_view_ = new ProportionalImageView(image_view_size);
     AddChildView(icon_view_);
   }
 
   gfx::ImageSkia icon = notification.icon().AsImageSkia();
-  if (notification.adjust_icon()) {
+  icon_view_->SetImage(icon, icon.size());
+
+  if (notification.draw_icon_background()) {
     icon_view_->set_background(
         views::Background::CreateSolidBackground(kIconBackgroundColor));
-    gfx::Size max_image_size =
-        notification.type() == NOTIFICATION_TYPE_SIMPLE &&
-                (icon.width() < kIconSize || icon.height() < kIconSize ||
-                 HasAlpha(icon, GetWidget()))
-            ? gfx::Size(kLegacyIconSize, kLegacyIconSize)
-            : gfx::Size(kIconSize, kIconSize);
-    icon_view_->SetImage(icon, max_image_size);
   } else {
-    icon_view_->SetImage(icon, icon.size());
     icon_view_->set_background(nullptr);
   }
 }
