@@ -5,8 +5,9 @@
 #include "ui/native_theme/common_theme.h"
 
 #include "base/logging.h"
+#include "base/memory/ptr_util.h"
 #include "third_party/skia/include/core/SkCanvas.h"
-#include "ui/base/resource/material_design/material_design_controller.h"
+#include "ui/base/material_design/material_design_controller.h"
 #include "ui/base/resource/resource_bundle.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/color_palette.h"
@@ -20,33 +21,22 @@ namespace ui {
 
 SkColor GetAuraColor(NativeTheme::ColorId color_id,
                      const NativeTheme* base_theme) {
-  // Shared colors.
-  static const SkColor kTextfieldDefaultBackground = SK_ColorWHITE;
-  static const SkColor kTextfieldSelectionBackgroundFocused =
-      SkColorSetARGB(0x54, 0x60, 0xA8, 0xEB);
-
   // MD colors.
   if (ui::MaterialDesignController::IsModeMaterial()) {
     // Dialogs:
     static const SkColor kDialogBackgroundColorMd = SK_ColorWHITE;
-    // Button:
-    static const SkColor kTextButtonEnabledColorMd =
-        SkColorSetRGB(0x64, 0x64, 0x64);
-    static const SkColor kTextButtonDisabledColorMd =
-        SkColorSetA(kTextButtonEnabledColorMd, 0x80);
+    // Buttons:
+    static const SkColor kButtonEnabledColorMd = gfx::kChromeIconGrey;
     // MenuItem:
     static const SkColor kMenuHighlightBackgroundColorMd =
         SkColorSetARGB(0x14, 0x00, 0x00, 0x00);
     static const SkColor kSelectedMenuItemForegroundColorMd = SK_ColorBLACK;
     // Link:
-    static const SkColor kLinkDisabledColorMd = SK_ColorBLACK;
     static const SkColor kLinkEnabledColorMd = gfx::kGoogleBlue700;
     // Results tables:
     static const SkColor kResultsTableTextMd = SK_ColorBLACK;
     static const SkColor kResultsTableDimmedTextMd =
         SkColorSetRGB(0x64, 0x64, 0x64);
-    static const SkColor kPositiveTextColorMd = SkColorSetRGB(0x0b, 0x80, 0x43);
-    static const SkColor kNegativeTextColorMd = SkColorSetRGB(0xc5, 0x39, 0x29);
 
     switch (color_id) {
       // Dialogs
@@ -54,11 +44,10 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
       case NativeTheme::kColorId_BubbleBackground:
         return kDialogBackgroundColorMd;
 
-      // Button
-      case NativeTheme::kColorId_MdTextButtonEnabledColor:
-        return kTextButtonEnabledColorMd;
-      case NativeTheme::kColorId_MdTextButtonDisabledColor:
-        return kTextButtonDisabledColorMd;
+      // Buttons
+      case NativeTheme::kColorId_ButtonEnabledColor:
+      case NativeTheme::kColorId_ButtonHoverColor:
+        return kButtonEnabledColorMd;
 
       // MenuItem
       case NativeTheme::kColorId_FocusedMenuItemBackgroundColor:
@@ -66,13 +55,17 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
       case NativeTheme::kColorId_SelectedMenuItemForegroundColor:
         return kSelectedMenuItemForegroundColorMd;
       // Link
-      case NativeTheme::kColorId_LinkDisabled:
-        return kLinkDisabledColorMd;
       case NativeTheme::kColorId_LinkEnabled:
       case NativeTheme::kColorId_LinkPressed:
         // Normal and pressed share a color.
         return kLinkEnabledColorMd;
 
+      // FocusableBorder
+      case NativeTheme::kColorId_FocusedBorderColor:
+        return gfx::kGoogleBlue500;
+      case NativeTheme::kColorId_UnfocusedBorderColor:
+        return SkColorSetA(SK_ColorBLACK, 0x66);
+
       // Results Tables
       case NativeTheme::kColorId_ResultsTableHoveredBackground:
         return SkColorSetA(base_theme->GetSystemColor(
@@ -85,9 +78,6 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
       case NativeTheme::kColorId_ResultsTableNormalText:
       case NativeTheme::kColorId_ResultsTableHoveredText:
       case NativeTheme::kColorId_ResultsTableSelectedText:
-      case NativeTheme::kColorId_ResultsTableNormalHeadline:
-      case NativeTheme::kColorId_ResultsTableHoveredHeadline:
-      case NativeTheme::kColorId_ResultsTableSelectedHeadline:
         return kResultsTableTextMd;
       case NativeTheme::kColorId_ResultsTableNormalDimmedText:
       case NativeTheme::kColorId_ResultsTableHoveredDimmedText:
@@ -97,14 +87,6 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
       case NativeTheme::kColorId_ResultsTableHoveredUrl:
       case NativeTheme::kColorId_ResultsTableSelectedUrl:
         return base_theme->GetSystemColor(NativeTheme::kColorId_LinkEnabled);
-      case NativeTheme::kColorId_ResultsTablePositiveText:
-      case NativeTheme::kColorId_ResultsTablePositiveHoveredText:
-      case NativeTheme::kColorId_ResultsTablePositiveSelectedText:
-        return kPositiveTextColorMd;
-      case NativeTheme::kColorId_ResultsTableNegativeText:
-      case NativeTheme::kColorId_ResultsTableNegativeHoveredText:
-      case NativeTheme::kColorId_ResultsTableNegativeSelectedText:
-        return kNegativeTextColorMd;
 
       default:
         break;
@@ -132,6 +114,7 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
   static const SkColor kBlueButtonHoverColor = SK_ColorWHITE;
   static const SkColor kBlueButtonShadowColor = SkColorSetRGB(0x53, 0x8C, 0xEA);
   static const SkColor kCallToActionColor = gfx::kGoogleBlue500;
+  static const SkColor kTextOnCallToActionColor = SK_ColorWHITE;
   // MenuItem:
   static const SkColor kMenuBackgroundColor = SK_ColorWHITE;
   static const SkColor kMenuHighlightBackgroundColor =
@@ -158,8 +141,11 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
   static const SkColor kLinkPressedColor = SK_ColorRED;
   // Textfield:
   static const SkColor kTextfieldDefaultColor = SK_ColorBLACK;
+  static const SkColor kTextfieldDefaultBackground = SK_ColorWHITE;
   static const SkColor kTextfieldReadOnlyColor = SK_ColorDKGRAY;
   static const SkColor kTextfieldReadOnlyBackground = SK_ColorWHITE;
+  static const SkColor kTextfieldSelectionBackgroundFocused =
+      SkColorSetARGB(0x54, 0x60, 0xA8, 0xEB);
   static const SkColor kTextfieldSelectionColor = color_utils::AlphaBlend(
       SK_ColorBLACK, kTextfieldSelectionBackgroundFocused, 0xdd);
   // Tooltip
@@ -263,6 +249,8 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
       return kBlueButtonShadowColor;
     case NativeTheme::kColorId_CallToActionColor:
       return kCallToActionColor;
+    case NativeTheme::kColorId_TextOnCallToActionColor:
+      return kTextOnCallToActionColor;
 
     // MenuItem
     case NativeTheme::kColorId_MenuBorderColor:
@@ -371,13 +359,10 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
     case NativeTheme::kColorId_ResultsTableSelectedText:
       return kResultsTableSelectedText;
     case NativeTheme::kColorId_ResultsTableNormalDimmedText:
-    case NativeTheme::kColorId_ResultsTableNormalHeadline:
       return kResultsTableNormalDimmedText;
     case NativeTheme::kColorId_ResultsTableHoveredDimmedText:
-    case NativeTheme::kColorId_ResultsTableHoveredHeadline:
       return kResultsTableHoveredDimmedText;
     case NativeTheme::kColorId_ResultsTableSelectedDimmedText:
-    case NativeTheme::kColorId_ResultsTableSelectedHeadline:
       return kResultsTableSelectedDimmedText;
     case NativeTheme::kColorId_ResultsTableNormalUrl:
       return kResultsTableNormalUrl;
@@ -405,8 +390,6 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
     case NativeTheme::kColorId_ThrobberLightColor:
       return kThrobberLightColor;
 
-    case NativeTheme::kColorId_MdTextButtonEnabledColor:
-    case NativeTheme::kColorId_MdTextButtonDisabledColor:
     case NativeTheme::kColorId_NumColors:
       break;
   }
@@ -415,32 +398,12 @@ SkColor GetAuraColor(NativeTheme::ColorId color_id,
   return gfx::kPlaceholderColor;
 }
 
-gfx::Size CommonThemeGetPartSize(NativeTheme::Part part,
-                                 NativeTheme::State state,
-                                 const NativeTheme::ExtraParams& extra) {
-  gfx::Size size;
-  switch (part) {
-    case NativeTheme::kComboboxArrow:
-      return ui::ResourceBundle::GetSharedInstance().GetImageNamed(
-          IDR_MENU_DROPARROW).Size();
-
-    default:
-      break;
-  }
-
-  return size;
-}
-
-void CommonThemePaintComboboxArrow(SkCanvas* canvas, const gfx::Rect& rect) {
-  gfx::ImageSkia* arrow = ui::ResourceBundle::GetSharedInstance().
-      GetImageSkiaNamed(IDR_MENU_DROPARROW);
-  CommonThemeCreateCanvas(canvas)->DrawImageInt(*arrow, rect.x(), rect.y());
-}
-
-void CommonThemePaintMenuItemBackground(const NativeTheme *theme,
-                                        SkCanvas* canvas,
-                                        NativeTheme::State state,
-                                        const gfx::Rect &rect) {
+void CommonThemePaintMenuItemBackground(
+    const NativeTheme* theme,
+    SkCanvas* canvas,
+    NativeTheme::State state,
+    const gfx::Rect& rect,
+    const NativeTheme::MenuItemExtraParams& menu_item) {
   SkPaint paint;
   switch (state) {
     case NativeTheme::kNormal:
@@ -456,16 +419,12 @@ void CommonThemePaintMenuItemBackground(const NativeTheme *theme,
       NOTREACHED() << "Invalid state " << state;
       break;
   }
+  if (menu_item.corner_radius > 0) {
+    const SkScalar radius = SkIntToScalar(menu_item.corner_radius);
+    canvas->drawRoundRect(gfx::RectToSkRect(rect), radius, radius, paint);
+    return;
+  }
   canvas->drawRect(gfx::RectToSkRect(rect), paint);
 }
 
-// static
-scoped_ptr<gfx::Canvas> CommonThemeCreateCanvas(SkCanvas* sk_canvas) {
-  // TODO(pkotwicz): Do something better and don't infer device
-  // scale factor from canvas scale.
-  SkMatrix m = sk_canvas->getTotalMatrix();
-  float device_scale = static_cast<float>(SkScalarAbs(m.getScaleX()));
-  return make_scoped_ptr(new gfx::Canvas(sk_canvas, device_scale));
-}
-
 }  // namespace ui
