@@ -4,10 +4,20 @@
 
 #include "ui/native_theme/native_theme.h"
 
+#include <cstring>
+
 #include "ui/native_theme/native_theme_observer.h"
 
 namespace ui {
 
+NativeTheme::ExtraParams::ExtraParams() {
+  memset(this, 0, sizeof(*this));
+}
+
+NativeTheme::ExtraParams::ExtraParams(const ExtraParams& other) {
+  memcpy(this, &other, sizeof(*this));
+}
+
 void NativeTheme::SetScrollbarColors(unsigned inactive_color,
                                      unsigned active_color,
                                      unsigned track_color) {
