@@ -10,7 +10,6 @@
 #include <vector>
 
 #include "ui/ozone/common/gpu/ozone_gpu_message_params.h"
-#include "ui/ozone/ozone_base_export.h"
 
 namespace base {
 class FilePath;
@@ -33,8 +32,7 @@ class FindDisplayById {
   int64_t display_id_;
 };
 
-DisplayMode_Params OZONE_BASE_EXPORT
-GetDisplayModeParams(const DisplayMode& mode);
+DisplayMode_Params GetDisplayModeParams(const DisplayMode& mode);
 DisplaySnapshot_Params GetDisplaySnapshotParams(const DisplaySnapshot& display);
 
 // Create a display using the Ozone command line parameters.
