@@ -5,16 +5,21 @@
 #ifndef UI_OZONE_PLATFORM_DRM_GPU_GBM_SURFACELESS_H_
 #define UI_OZONE_PLATFORM_DRM_GPU_GBM_SURFACELESS_H_
 
+#include <vector>
+
+#include "base/macros.h"
+#include "ui/gfx/native_widget_types.h"
+#include "ui/ozone/platform/drm/gpu/overlay_plane.h"
 #include "ui/ozone/public/surface_ozone_egl.h"
 
 namespace gfx {
 class Size;
-}  // namespace gfx
+}
 
 namespace ui {
 
-class DrmDeviceManager;
-class DrmWindow;
+class DrmWindowProxy;
+class GbmSurfaceFactory;
 
 // In surfaceless mode drawing and displaying happens directly through
 // NativePixmap buffers. CC would call into SurfaceFactoryOzone to allocate the
@@ -22,21 +27,30 @@ class DrmWindow;
 // presentation.
 class GbmSurfaceless : public SurfaceOzoneEGL {
  public:
-  GbmSurfaceless(DrmWindow* window, DrmDeviceManager* drm_device_manager);
+  GbmSurfaceless(std::unique_ptr<DrmWindowProxy> window,
+                 GbmSurfaceFactory* surface_manager);
   ~GbmSurfaceless() override;
 
+  void QueueOverlayPlane(const OverlayPlane& plane);
+
   // SurfaceOzoneEGL:
   intptr_t GetNativeWindow() override;
   bool ResizeNativeWindow(const gfx::Size& viewport_size) override;
   bool OnSwapBuffers() override;
-  bool OnSwapBuffersAsync(const SwapCompletionCallback& callback) override;
-  scoped_ptr<gfx::VSyncProvider> CreateVSyncProvider() override;
+  void OnSwapBuffersAsync(const SwapCompletionCallback& callback) override;
+  std::unique_ptr<gfx::VSyncProvider> CreateVSyncProvider() override;
   bool IsUniversalDisplayLinkDevice() override;
+  void* /* EGLConfig */ GetEGLSurfaceConfig(
+      const EglConfigCallbacks& egl) override;
 
  protected:
-  DrmWindow* window_;
-  DrmDeviceManager* drm_device_manager_;
+  std::unique_ptr<DrmWindowProxy> window_;
 
+  GbmSurfaceFactory* surface_manager_;
+
+  std::vector<OverlayPlane> planes_;
+
+ private:
   DISALLOW_COPY_AND_ASSIGN(GbmSurfaceless);
 };
 
