@@ -6,6 +6,13 @@
 
 namespace ui {
 
+EglConfigCallbacks::EglConfigCallbacks() {}
+
+EglConfigCallbacks::EglConfigCallbacks(const EglConfigCallbacks& other) =
+    default;
+
+EglConfigCallbacks::~EglConfigCallbacks() {}
+
 bool SurfaceOzoneEGL::IsUniversalDisplayLinkDevice() {
   return false;
 }
