@@ -5,12 +5,13 @@
 #ifndef UI_SNAPSHOT_SCREENSHOT_GRABBER_H_
 #define UI_SNAPSHOT_SCREENSHOT_GRABBER_H_
 
+#include <memory>
+
 #include "base/callback.h"
 #include "base/compiler_specific.h"
 #include "base/files/file_path.h"
 #include "base/macros.h"
 #include "base/memory/ref_counted_memory.h"
-#include "base/memory/scoped_ptr.h"
 #include "base/memory/weak_ptr.h"
 #include "base/observer_list.h"
 #include "base/time/time.h"
