@@ -8,11 +8,16 @@
 #include <utility>
 
 #include "build/build_config.h"
+#include "ui/base/default_style.h"
 #include "ui/base/hit_test.h"
 #include "ui/base/l10n/l10n_util.h"
 #include "ui/base/resource/resource_bundle.h"
+#include "ui/compositor/paint_context.h"
+#include "ui/compositor/paint_recorder.h"
+#include "ui/display/display.h"
+#include "ui/display/screen.h"
+#include "ui/gfx/geometry/vector2d.h"
 #include "ui/gfx/path.h"
-#include "ui/gfx/screen.h"
 #include "ui/gfx/skia_util.h"
 #include "ui/native_theme/native_theme.h"
 #include "ui/resources/grit/ui_resources.h"
@@ -20,21 +25,22 @@
 #include "ui/views/bubble/bubble_border.h"
 #include "ui/views/controls/button/label_button.h"
 #include "ui/views/controls/image_view.h"
+#include "ui/views/layout/box_layout.h"
+#include "ui/views/layout/layout_constants.h"
 #include "ui/views/resources/grit/views_resources.h"
 #include "ui/views/widget/widget.h"
 #include "ui/views/widget/widget_delegate.h"
 #include "ui/views/window/client_view.h"
 
+namespace views {
+
 namespace {
 
-// Insets for the title bar views in pixels.
-const int kTitleTopInset = 12;
-const int kTitleLeftInset = 19;
-const int kTitleBottomInset = 12;
-const int kTitleRightInset = 7;
+// Background color of the footnote view.
+const SkColor kFootnoteBackgroundColor = SkColorSetRGB(245, 245, 245);
 
-// The horizontal padding between the title and the icon.
-const int kTitleHorizontalPadding = 5;
+// Color of the top border of the footnote.
+const SkColor kFootnoteBorderColor = SkColorSetRGB(229, 229, 229);
 
 // Get the |vertical| or horizontal amount that |available_bounds| overflows
 // |window_bounds|.
@@ -61,27 +67,28 @@ int GetOffScreenLength(const gfx::Rect& available_bounds,
 
 }  // namespace
 
-namespace views {
-
 // static
 const char BubbleFrameView::kViewClassName[] = "BubbleFrameView";
 
-BubbleFrameView::BubbleFrameView(const gfx::Insets& content_margins)
+BubbleFrameView::BubbleFrameView(const gfx::Insets& title_margins,
+                                 const gfx::Insets& content_margins)
     : bubble_border_(nullptr),
+      title_margins_(title_margins),
       content_margins_(content_margins),
       title_icon_(new views::ImageView()),
       title_(nullptr),
       close_(nullptr),
-      titlebar_extra_view_(nullptr),
+      footnote_container_(nullptr),
       close_button_clicked_(false) {
   AddChildView(title_icon_);
 
   ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
   title_ = new Label(base::string16(),
-                     rb.GetFontList(ui::ResourceBundle::MediumFont));
+                     rb.GetFontListWithDelta(ui::kTitleFontSizeDelta));
   title_->SetHorizontalAlignment(gfx::ALIGN_LEFT);
   title_->set_collapse_when_hidden(true);
   title_->SetVisible(false);
+  title_->SetMultiLine(true);
   AddChildView(title_);
 
   close_ = CreateCloseButton(this);
@@ -91,12 +98,6 @@ BubbleFrameView::BubbleFrameView(const gfx::Insets& content_margins)
 
 BubbleFrameView::~BubbleFrameView() {}
 
-// static
-gfx::Insets BubbleFrameView::GetTitleInsets() {
-  return gfx::Insets(
-      kTitleTopInset, kTitleLeftInset, kTitleBottomInset, kTitleRightInset);
-}
-
 // static
 LabelButton* BubbleFrameView::CreateCloseButton(ButtonListener* listener) {
   ui::ResourceBundle& rb = ui::ResourceBundle::GetSharedInstance();
@@ -118,9 +119,12 @@ LabelButton* BubbleFrameView::CreateCloseButton(ButtonListener* listener) {
 }
 
 gfx::Rect BubbleFrameView::GetBoundsForClientView() const {
-  gfx::Rect client_bounds = GetLocalBounds();
+  gfx::Rect client_bounds = GetContentsBounds();
   client_bounds.Inset(GetInsets());
-  client_bounds.Inset(bubble_border_->GetInsets());
+  if (footnote_container_) {
+    client_bounds.set_height(client_bounds.height() -
+                             footnote_container_->height());
+  }
   return client_bounds;
 }
 
@@ -130,6 +134,20 @@ gfx::Rect BubbleFrameView::GetWindowBoundsForClientBounds(
   return bubble_border_->GetBounds(gfx::Rect(), size);
 }
 
+bool BubbleFrameView::GetClientMask(const gfx::Size& size,
+                                    gfx::Path* path) const {
+  const int radius = bubble_border_->GetBorderCornerRadius();
+  gfx::Insets content_insets = GetInsets();
+  // If the client bounds don't touch the edges, no need to mask.
+  if (std::min({content_insets.top(), content_insets.left(),
+                content_insets.bottom(), content_insets.right()}) > radius) {
+    return false;
+  }
+  gfx::RectF rect((gfx::Rect(size)));
+  path->addRoundRect(gfx::RectFToSkRect(rect), radius, radius);
+  return true;
+}
+
 int BubbleFrameView::NonClientHitTest(const gfx::Point& point) {
   if (!bounds().Contains(point))
     return HTNOWHERE;
@@ -137,8 +155,11 @@ int BubbleFrameView::NonClientHitTest(const gfx::Point& point) {
     return HTCLOSE;
 
   // Allow dialogs to show the system menu and be dragged.
-  if (GetWidget()->widget_delegate()->AsDialogDelegate()) {
-    gfx::Rect sys_rect(0, 0, title_->x(), title_->y());
+  if (GetWidget()->widget_delegate()->AsDialogDelegate() &&
+      !GetWidget()->widget_delegate()->AsBubbleDialogDelegate()) {
+    gfx::Rect bounds(GetContentsBounds());
+    bounds.Inset(title_margins_);
+    gfx::Rect sys_rect(0, 0, bounds.x(), bounds.y());
     sys_rect.set_origin(gfx::Point(GetMirroredXForRect(sys_rect), 0));
     if (sys_rect.Contains(point))
       return HTSYSMENU;
@@ -151,30 +172,42 @@ int BubbleFrameView::NonClientHitTest(const gfx::Point& point) {
 
 void BubbleFrameView::GetWindowMask(const gfx::Size& size,
                                     gfx::Path* window_mask) {
-  // NOTE: this only provides implementations for the types used by dialogs.
-  if ((bubble_border_->arrow() != BubbleBorder::NONE &&
-       bubble_border_->arrow() != BubbleBorder::FLOAT) ||
-      (bubble_border_->shadow() != BubbleBorder::SMALL_SHADOW &&
-       bubble_border_->shadow() != BubbleBorder::NO_SHADOW_OPAQUE_BORDER))
+  if (bubble_border_->shadow() != BubbleBorder::SMALL_SHADOW &&
+      bubble_border_->shadow() != BubbleBorder::NO_SHADOW_OPAQUE_BORDER &&
+      bubble_border_->shadow() != BubbleBorder::NO_ASSETS)
+    return;
+
+  // We don't return a mask for windows with arrows unless they use
+  // BubbleBorder::NO_ASSETS.
+  if (bubble_border_->shadow() != BubbleBorder::NO_ASSETS &&
+      bubble_border_->arrow() != BubbleBorder::NONE &&
+      bubble_border_->arrow() != BubbleBorder::FLOAT)
     return;
 
   // Use a window mask roughly matching the border in the image assets.
-  static const int kBorderStrokeSize = 1;
-  static const SkScalar kCornerRadius = SkIntToScalar(6);
+  const int kBorderStrokeSize =
+      bubble_border_->shadow() == BubbleBorder::NO_ASSETS ? 0 : 1;
+  const SkScalar kCornerRadius =
+      SkIntToScalar(bubble_border_->GetBorderCornerRadius());
   const gfx::Insets border_insets = bubble_border_->GetInsets();
-  SkRect rect = { SkIntToScalar(border_insets.left() - kBorderStrokeSize),
-                  SkIntToScalar(border_insets.top() - kBorderStrokeSize),
-                  SkIntToScalar(size.width() - border_insets.right() +
-                                kBorderStrokeSize),
-                  SkIntToScalar(size.height() - border_insets.bottom() +
-                                kBorderStrokeSize) };
-  if (bubble_border_->shadow() == BubbleBorder::NO_SHADOW_OPAQUE_BORDER) {
+  SkRect rect = {
+      SkIntToScalar(border_insets.left() - kBorderStrokeSize),
+      SkIntToScalar(border_insets.top() - kBorderStrokeSize),
+      SkIntToScalar(size.width() - border_insets.right() + kBorderStrokeSize),
+      SkIntToScalar(size.height() - border_insets.bottom() +
+                    kBorderStrokeSize)};
+
+  if (bubble_border_->shadow() == BubbleBorder::NO_SHADOW_OPAQUE_BORDER ||
+      bubble_border_->shadow() == BubbleBorder::NO_ASSETS) {
     window_mask->addRoundRect(rect, kCornerRadius, kCornerRadius);
   } else {
     static const int kBottomBorderShadowSize = 2;
     rect.fBottom += SkIntToScalar(kBottomBorderShadowSize);
     window_mask->addRect(rect);
   }
+  gfx::Path arrow_path;
+  if (bubble_border_->GetArrowPath(gfx::Rect(size), &arrow_path))
+    window_mask->addPath(arrow_path, 0, 0);
 }
 
 void BubbleFrameView::ResetWindowControls() {
@@ -188,7 +221,6 @@ void BubbleFrameView::UpdateWindowIcon() {
   title_icon_->SetImage(&image);
 }
 
-
 void BubbleFrameView::UpdateWindowTitle() {
   title_->SetText(GetWidget()->widget_delegate()->GetWindowTitle());
   title_->SetVisible(GetWidget()->widget_delegate()->ShouldShowWindowTitle());
@@ -200,13 +232,17 @@ void BubbleFrameView::SetTitleFontList(const gfx::FontList& font_list) {
   title_->SetFontList(font_list);
 }
 
+const char* BubbleFrameView::GetClassName() const {
+  return kViewClassName;
+}
+
 gfx::Insets BubbleFrameView::GetInsets() const {
   gfx::Insets insets = content_margins_;
 
   const int icon_height = title_icon_->GetPreferredSize().height();
   const int label_height = title_->GetPreferredSize().height();
   const bool has_title = icon_height > 0 || label_height > 0;
-  const int title_padding = has_title ? kTitleTopInset + kTitleBottomInset : 0;
+  const int title_padding = has_title ? title_margins_.height() : 0;
   const int title_height = std::max(icon_height, label_height) + title_padding;
   const int close_height = close_->visible() ? close_->height() : 0;
   insets += gfx::Insets(std::max(title_height, close_height), 0, 0, 0);
@@ -251,64 +287,50 @@ gfx::Size BubbleFrameView::GetMaximumSize() const {
 }
 
 void BubbleFrameView::Layout() {
+  // The title margins may not be set, but make sure that's only the case when
+  // there's no title.
+  DCHECK(!title_margins_.IsEmpty() || !title_->visible());
+
   gfx::Rect bounds(GetContentsBounds());
-  bounds.Inset(GetTitleInsets());
+  bounds.Inset(title_margins_);
   if (bounds.IsEmpty())
     return;
 
-  // The close button top inset is actually smaller than the title top inset.
-  close_->SetPosition(gfx::Point(bounds.right() - close_->width(),
-                                 bounds.y() - 5));
+  // The close button is positioned somewhat closer to the edge of the bubble.
+  gfx::Point close_position = GetContentsBounds().top_right();
+  close_position += gfx::Vector2d(-close_->width() - 7, 6);
+  close_->SetPosition(close_position);
 
-  gfx::Size title_icon_size(title_icon_->GetPreferredSize());
-  gfx::Size title_label_size(title_->GetPreferredSize());
+  gfx::Size title_icon_pref_size(title_icon_->GetPreferredSize());
   int padding = 0;
-  if (title_icon_size.width() > 0 && title_label_size.width() > 0)
-    padding = kTitleHorizontalPadding;
-  const int title_height = std::max(title_icon_size.height(),
-                                    title_label_size.height());
+  int title_height = title_icon_pref_size.height();
 
-  const int title_icon_width = std::max(0, close_->x() - bounds.x());
-  title_icon_size.SetToMin(gfx::Size(title_icon_width, title_height));
-  gfx::Rect title_icon_bounds(
-      bounds.x(), bounds.y(), title_icon_size.width(), title_height);
-  title_icon_->SetBoundsRect(title_icon_bounds);
+  if (title_->visible() && !title_->text().empty()) {
+    if (title_icon_pref_size.width() > 0)
+      padding = title_margins_.left();
 
-  const int title_label_x = title_icon_->bounds().right() + padding;
-  const int title_label_width = std::max(0, close_->x() - title_label_x);
-  title_label_size.SetToMin(gfx::Size(title_label_width,
-                                      title_label_size.height()));
-  gfx::Rect title_label_bounds(
-      title_label_x, bounds.y(), title_label_size.width(), title_height);
-  title_->SetBoundsRect(title_label_bounds);
+    const int title_label_x =
+        bounds.x() + title_icon_pref_size.width() + padding;
+    title_->SizeToFit(std::max(1, close_->x() - title_label_x));
+    title_height = std::max(title_height, title_->height());
+    title_->SetPosition(gfx::Point(
+        title_label_x, bounds.y() + (title_height - title_->height()) / 2));
+  }
 
-  bounds.set_width(
-      title_icon_size.width() + title_label_size.width() + padding);
+  title_icon_->SetBounds(bounds.x(), bounds.y(), title_icon_pref_size.width(),
+                         title_height);
+  bounds.set_width(title_->bounds().right() - bounds.x());
   bounds.set_height(title_height);
 
-  if (titlebar_extra_view_) {
-    const int extra_width = close_->x() - bounds.right();
-    gfx::Size size = titlebar_extra_view_->GetPreferredSize();
-    size.SetToMin(gfx::Size(std::max(0, extra_width), size.height()));
-    gfx::Rect titlebar_extra_view_bounds(
-        close_->x() - size.width(),
-        bounds.y(),
-        size.width(),
-        bounds.height());
-    titlebar_extra_view_bounds.Subtract(bounds);
-    titlebar_extra_view_->SetBoundsRect(titlebar_extra_view_bounds);
+  if (footnote_container_) {
+    gfx::Rect local_bounds = GetContentsBounds();
+    int height = footnote_container_->GetHeightForWidth(local_bounds.width());
+    footnote_container_->SetBounds(local_bounds.x(),
+                                   local_bounds.bottom() - height,
+                                   local_bounds.width(), height);
   }
 }
 
-const char* BubbleFrameView::GetClassName() const {
-  return kViewClassName;
-}
-
-void BubbleFrameView::ChildPreferredSizeChanged(View* child) {
-  if (child == titlebar_extra_view_ || child == title_)
-    Layout();
-}
-
 void BubbleFrameView::OnThemeChanged() {
   UpdateWindowTitle();
   ResetWindowControls();
@@ -323,6 +345,19 @@ void BubbleFrameView::OnNativeThemeChanged(const ui::NativeTheme* theme) {
   }
 }
 
+void BubbleFrameView::OnPaint(gfx::Canvas* canvas) {
+  OnPaintBackground(canvas);
+  // Border comes after children.
+}
+
+void BubbleFrameView::PaintChildren(const ui::PaintContext& context) {
+  NonClientFrameView::PaintChildren(context);
+
+  ui::PaintCache paint_cache;
+  ui::PaintRecorder recorder(context, size(), &paint_cache);
+  OnPaintBorder(recorder.canvas());
+}
+
 void BubbleFrameView::ButtonPressed(Button* sender, const ui::Event& event) {
   if (sender == close_) {
     close_button_clicked_ = true;
@@ -330,7 +365,7 @@ void BubbleFrameView::ButtonPressed(Button* sender, const ui::Event& event) {
   }
 }
 
-void BubbleFrameView::SetBubbleBorder(scoped_ptr<BubbleBorder> border) {
+void BubbleFrameView::SetBubbleBorder(std::unique_ptr<BubbleBorder> border) {
   bubble_border_ = border.get();
   SetBorder(std::move(border));
 
@@ -338,11 +373,21 @@ void BubbleFrameView::SetBubbleBorder(scoped_ptr<BubbleBorder> border) {
   set_background(new views::BubbleBackground(bubble_border_));
 }
 
-void BubbleFrameView::SetTitlebarExtraView(View* view) {
-  DCHECK(view);
-  DCHECK(!titlebar_extra_view_);
-  AddChildView(view);
-  titlebar_extra_view_ = view;
+void BubbleFrameView::SetFootnoteView(View* view) {
+  if (!view)
+    return;
+
+  DCHECK(!footnote_container_);
+  footnote_container_ = new views::View();
+  footnote_container_->SetLayoutManager(
+      new BoxLayout(BoxLayout::kVertical, content_margins_.left(),
+                    content_margins_.top(), 0));
+  footnote_container_->set_background(
+      Background::CreateSolidBackground(kFootnoteBackgroundColor));
+  footnote_container_->SetBorder(
+      Border::CreateSolidSidedBorder(1, 0, 0, 0, kFootnoteBorderColor));
+  footnote_container_->AddChildView(view);
+  AddChildView(footnote_container_);
 }
 
 gfx::Rect BubbleFrameView::GetUpdatedWindowBounds(const gfx::Rect& anchor_rect,
@@ -370,9 +415,9 @@ gfx::Rect BubbleFrameView::GetUpdatedWindowBounds(const gfx::Rect& anchor_rect,
 gfx::Rect BubbleFrameView::GetAvailableScreenBounds(
     const gfx::Rect& rect) const {
   // The bubble attempts to fit within the current screen bounds.
-  // TODO(scottmg): Native is wrong. http://crbug.com/133312
-  return gfx::Screen::GetNativeScreen()->GetDisplayNearestPoint(
-      rect.CenterPoint()).work_area();
+  return display::Screen::GetScreen()
+      ->GetDisplayNearestPoint(rect.CenterPoint())
+      .work_area();
 }
 
 bool BubbleFrameView::IsCloseButtonVisible() const {
@@ -453,23 +498,23 @@ void BubbleFrameView::OffsetArrowIfOffScreen(const gfx::Rect& anchor_rect,
 gfx::Size BubbleFrameView::GetSizeForClientSize(
     const gfx::Size& client_size) const {
   // Accommodate the width of the title bar elements.
-  int title_bar_width = GetInsets().width() + border()->GetInsets().width();
+  int title_bar_width = title_margins_.width() + border()->GetInsets().width();
   gfx::Size title_icon_size = title_icon_->GetPreferredSize();
   gfx::Size title_label_size = title_->GetPreferredSize();
-  if (title_icon_size.width() > 0 || title_label_size.width() > 0)
-    title_bar_width += kTitleLeftInset;
   if (title_icon_size.width() > 0 && title_label_size.width() > 0)
-    title_bar_width += kTitleHorizontalPadding;
+    title_bar_width += title_margins_.left();
   title_bar_width += title_icon_size.width();
-  title_bar_width += title_label_size.width();
   if (close_->visible())
     title_bar_width += close_->width() + 1;
-  if (titlebar_extra_view_ != NULL)
-    title_bar_width += titlebar_extra_view_->GetPreferredSize().width();
+
   gfx::Size size(client_size);
+  gfx::Insets client_insets = GetInsets();
+  size.Enlarge(client_insets.width(), client_insets.height());
   size.SetToMax(gfx::Size(title_bar_width, 0));
-  const gfx::Insets insets(GetInsets());
-  size.Enlarge(insets.width(), insets.height());
+
+  if (footnote_container_)
+    size.Enlarge(0, footnote_container_->GetHeightForWidth(size.width()));
+
   return size;
 }
 
