@@ -6,19 +6,20 @@
 
 #include <set>
 
-#include "base/basictypes.h"
+#include "base/macros.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "ui/base/ime/input_method.h"
 #include "ui/base/ime/text_input_client.h"
 #include "ui/base/models/combobox_model.h"
+#include "ui/base/models/menu_model.h"
 #include "ui/events/event.h"
 #include "ui/events/event_constants.h"
 #include "ui/events/event_utils.h"
+#include "ui/events/keycodes/dom/dom_code.h"
 #include "ui/events/keycodes/keyboard_codes.h"
 #include "ui/views/controls/combobox/combobox_listener.h"
-#include "ui/views/controls/menu/menu_runner.h"
-#include "ui/views/controls/menu/menu_runner_handler.h"
-#include "ui/views/test/menu_runner_test_api.h"
+#include "ui/views/test/combobox_test_api.h"
 #include "ui/views/test/views_test_base.h"
 #include "ui/views/widget/widget.h"
 
@@ -26,40 +27,16 @@ using base::ASCIIToUTF16;
 
 namespace views {
 
+using test::ComboboxTestApi;
+
 namespace {
 
-// An dummy implementation of MenuRunnerHandler to check if the dropdown menu is
-// shown or not.
-class TestMenuRunnerHandler : public MenuRunnerHandler {
- public:
-  TestMenuRunnerHandler() : executed_(false) {}
-
-  bool executed() const { return executed_; }
-
-  MenuRunner::RunResult RunMenuAt(Widget* parent,
-                                  MenuButton* button,
-                                  const gfx::Rect& bounds,
-                                  MenuAnchorPosition anchor,
-                                  ui::MenuSourceType source_type,
-                                  int32 types) override {
-    executed_ = true;
-    return MenuRunner::NORMAL_EXIT;
-  }
-
- private:
-  bool executed_;
-
-  DISALLOW_COPY_AND_ASSIGN(TestMenuRunnerHandler);
-};
-
 // A wrapper of Combobox to intercept the result of OnKeyPressed() and
 // OnKeyReleased() methods.
 class TestCombobox : public Combobox {
  public:
-  explicit TestCombobox(ui::ComboboxModel* model)
-      : Combobox(model),
-        key_handled_(false),
-        key_received_(false) {}
+  TestCombobox(ui::ComboboxModel* model, Combobox::Style style)
+      : Combobox(model, style), key_handled_(false), key_received_(false) {}
 
   bool OnKeyPressed(const ui::KeyEvent& e) override {
     key_received_ = true;
@@ -93,10 +70,10 @@ class TestComboboxModel : public ui::ComboboxModel {
   TestComboboxModel() {}
   ~TestComboboxModel() override {}
 
-  static const int kItemCount = 10;
+  enum { kItemCount = 10 };
 
   // ui::ComboboxModel:
-  int GetItemCount() const override { return kItemCount; }
+  int GetItemCount() const override { return item_count_; }
   base::string16 GetItemAt(int index) override {
     if (IsItemSeparatorAt(index)) {
       NOTREACHED();
@@ -122,8 +99,11 @@ class TestComboboxModel : public ui::ComboboxModel {
     separators_ = separators;
   }
 
+  void set_item_count(int item_count) { item_count_ = item_count; }
+
  private:
   std::set<int> separators_;
+  int item_count_ = kItemCount;
 
   DISALLOW_COPY_AND_ASSIGN(TestComboboxModel);
 };
@@ -135,15 +115,21 @@ class VectorComboboxModel : public ui::ComboboxModel {
       : values_(values) {}
   ~VectorComboboxModel() override {}
 
+  void set_default_index(int default_index) { default_index_ = default_index; }
+
   // ui::ComboboxModel:
   int GetItemCount() const override { return (int)values_->size(); }
   base::string16 GetItemAt(int index) override {
     return ASCIIToUTF16(values_->at(index));
   }
   bool IsItemSeparatorAt(int index) override { return false; }
+  int GetDefaultIndex() const override { return default_index_; }
 
  private:
   std::vector<std::string>* values_;
+  int default_index_ = 0;
+
+  DISALLOW_COPY_AND_ASSIGN(VectorComboboxModel);
 };
 
 class EvilListener : public ComboboxListener {
@@ -207,14 +193,15 @@ class ComboboxTest : public ViewsTestBase {
     ViewsTestBase::TearDown();
   }
 
-  void InitCombobox(const std::set<int>* separators) {
+  void InitCombobox(const std::set<int>* separators, Combobox::Style style) {
     model_.reset(new TestComboboxModel());
 
     if (separators)
       model_->SetSeparators(*separators);
 
     ASSERT_FALSE(combobox_);
-    combobox_ = new TestCombobox(model_.get());
+    combobox_ = new TestCombobox(model_.get(), style);
+    test_api_.reset(new ComboboxTestApi(combobox_));
     combobox_->set_id(1);
 
     widget_ = new Widget;
@@ -262,13 +249,17 @@ class ComboboxTest : public ViewsTestBase {
 
   // |combobox_| will be allocated InitCombobox() and then owned by |widget_|.
   TestCombobox* combobox_;
+  std::unique_ptr<ComboboxTestApi> test_api_;
 
   // Combobox does not take ownership of the model, hence it needs to be scoped.
-  scoped_ptr<TestComboboxModel> model_;
+  std::unique_ptr<TestComboboxModel> model_;
+
+ private:
+  DISALLOW_COPY_AND_ASSIGN(ComboboxTest);
 };
 
 TEST_F(ComboboxTest, KeyTest) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
   SendKeyEvent(ui::VKEY_END);
   EXPECT_EQ(combobox_->selected_index() + 1, model_->GetItemCount());
   SendKeyEvent(ui::VKEY_HOME);
@@ -294,7 +285,7 @@ TEST_F(ComboboxTest, DisabilityTest) {
   model_.reset(new TestComboboxModel());
 
   ASSERT_FALSE(combobox_);
-  combobox_ = new TestCombobox(model_.get());
+  combobox_ = new TestCombobox(model_.get(), Combobox::STYLE_NORMAL);
   combobox_->SetEnabled(false);
 
   widget_ = new Widget;
@@ -312,7 +303,7 @@ TEST_F(ComboboxTest, DisabilityTest) {
 TEST_F(ComboboxTest, SkipSeparatorSimple) {
   std::set<int> separators;
   separators.insert(2);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   EXPECT_EQ(0, combobox_->selected_index());
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(1, combobox_->selected_index());
@@ -333,7 +324,7 @@ TEST_F(ComboboxTest, SkipSeparatorSimple) {
 TEST_F(ComboboxTest, SkipSeparatorBeginning) {
   std::set<int> separators;
   separators.insert(0);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   EXPECT_EQ(1, combobox_->selected_index());
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(2, combobox_->selected_index());
@@ -354,7 +345,7 @@ TEST_F(ComboboxTest, SkipSeparatorBeginning) {
 TEST_F(ComboboxTest, SkipSeparatorEnd) {
   std::set<int> separators;
   separators.insert(TestComboboxModel::kItemCount - 1);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   combobox_->SetSelectedIndex(8);
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(8, combobox_->selected_index());
@@ -372,7 +363,7 @@ TEST_F(ComboboxTest, SkipMultipleSeparatorsAtBeginning) {
   separators.insert(0);
   separators.insert(1);
   separators.insert(2);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   EXPECT_EQ(3, combobox_->selected_index());
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(4, combobox_->selected_index());
@@ -396,7 +387,7 @@ TEST_F(ComboboxTest, SkipMultipleAdjacentSeparatorsAtMiddle) {
   separators.insert(4);
   separators.insert(5);
   separators.insert(6);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   combobox_->SetSelectedIndex(3);
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(7, combobox_->selected_index());
@@ -412,7 +403,7 @@ TEST_F(ComboboxTest, SkipMultipleSeparatorsAtEnd) {
   separators.insert(7);
   separators.insert(8);
   separators.insert(9);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   combobox_->SetSelectedIndex(6);
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_EQ(6, combobox_->selected_index());
@@ -433,7 +424,7 @@ TEST_F(ComboboxTest, GetTextForRowTest) {
   separators.insert(0);
   separators.insert(1);
   separators.insert(9);
-  InitCombobox(&separators);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
   for (int i = 0; i < combobox_->GetRowCount(); ++i) {
     if (separators.count(i) != 0) {
       EXPECT_TRUE(combobox_->GetTextForRow(i).empty()) << i;
@@ -446,7 +437,7 @@ TEST_F(ComboboxTest, GetTextForRowTest) {
 
 // Verifies selecting the first matching value (and returning whether found).
 TEST_F(ComboboxTest, SelectValue) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
   ASSERT_EQ(model_->GetDefaultIndex(), combobox_->selected_index());
   EXPECT_TRUE(combobox_->SelectValue(ASCIIToUTF16("PEANUT BUTTER")));
   EXPECT_EQ(0, combobox_->selected_index());
@@ -454,9 +445,11 @@ TEST_F(ComboboxTest, SelectValue) {
   EXPECT_EQ(1, combobox_->selected_index());
   EXPECT_FALSE(combobox_->SelectValue(ASCIIToUTF16("BANANAS")));
   EXPECT_EQ(1, combobox_->selected_index());
+}
 
+TEST_F(ComboboxTest, SelectValueActionStyle) {
   // With the action style, the selected index is always 0.
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
   EXPECT_FALSE(combobox_->SelectValue(ASCIIToUTF16("PEANUT BUTTER")));
   EXPECT_EQ(0, combobox_->selected_index());
   EXPECT_FALSE(combobox_->SelectValue(ASCIIToUTF16("JELLY")));
@@ -466,10 +459,9 @@ TEST_F(ComboboxTest, SelectValue) {
 }
 
 TEST_F(ComboboxTest, SelectIndexActionStyle) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
 
   // With the action style, the selected index is always 0.
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
   combobox_->SetSelectedIndex(1);
   EXPECT_EQ(0, combobox_->selected_index());
   combobox_->SetSelectedIndex(2);
@@ -482,44 +474,41 @@ TEST_F(ComboboxTest, ListenerHandlesDelete) {
   TestComboboxModel model;
 
   // |combobox| will be deleted on change.
-  TestCombobox* combobox = new TestCombobox(&model);
-  scoped_ptr<EvilListener> evil_listener(new EvilListener());
+  TestCombobox* combobox = new TestCombobox(&model, Combobox::STYLE_NORMAL);
+  std::unique_ptr<EvilListener> evil_listener(new EvilListener());
   combobox->set_listener(evil_listener.get());
-  ASSERT_NO_FATAL_FAILURE(combobox->ExecuteCommand(2));
+  ASSERT_NO_FATAL_FAILURE(ComboboxTestApi(combobox).PerformActionAt(2));
   EXPECT_TRUE(evil_listener->deleted());
 
   // With STYLE_ACTION
   // |combobox| will be deleted on change.
-  combobox = new TestCombobox(&model);
+  combobox = new TestCombobox(&model, Combobox::STYLE_ACTION);
   evil_listener.reset(new EvilListener());
   combobox->set_listener(evil_listener.get());
-  combobox->SetStyle(Combobox::STYLE_ACTION);
-  ASSERT_NO_FATAL_FAILURE(combobox->ExecuteCommand(2));
+  ASSERT_NO_FATAL_FAILURE(ComboboxTestApi(combobox).PerformActionAt(2));
   EXPECT_TRUE(evil_listener->deleted());
 }
 
 TEST_F(ComboboxTest, Click) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
 
   combobox_->Layout();
+  int menu_show_count = 0;
+  test_api_->InstallTestMenuRunner(&menu_show_count);
 
   // Click the left side. The menu is shown.
-  TestMenuRunnerHandler* test_menu_runner_handler = new TestMenuRunnerHandler();
-  scoped_ptr<MenuRunnerHandler> menu_runner_handler(test_menu_runner_handler);
-  test::MenuRunnerTestAPI test_api(
-      combobox_->dropdown_list_menu_runner_.get());
-  test_api.SetMenuRunnerHandler(menu_runner_handler.Pass());
+  EXPECT_EQ(0, menu_show_count);
   PerformClick(gfx::Point(combobox_->x() + 1,
                           combobox_->y() + combobox_->height() / 2));
   EXPECT_FALSE(listener.on_perform_action_called());
-  EXPECT_TRUE(test_menu_runner_handler->executed());
+  EXPECT_EQ(1, menu_show_count);
 }
 
 TEST_F(ComboboxTest, ClickButDisabled) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
@@ -528,19 +517,16 @@ TEST_F(ComboboxTest, ClickButDisabled) {
   combobox_->SetEnabled(false);
 
   // Click the left side, but nothing happens since the combobox is disabled.
-  TestMenuRunnerHandler* test_menu_runner_handler = new TestMenuRunnerHandler();
-  scoped_ptr<MenuRunnerHandler> menu_runner_handler(test_menu_runner_handler);
-  test::MenuRunnerTestAPI test_api(
-      combobox_->dropdown_list_menu_runner_.get());
-  test_api.SetMenuRunnerHandler(menu_runner_handler.Pass());
+  int menu_show_count = 0;
+  test_api_->InstallTestMenuRunner(&menu_show_count);
   PerformClick(gfx::Point(combobox_->x() + 1,
                           combobox_->y() + combobox_->height() / 2));
   EXPECT_FALSE(listener.on_perform_action_called());
-  EXPECT_FALSE(test_menu_runner_handler->executed());
+  EXPECT_EQ(0, menu_show_count);
 }
 
 TEST_F(ComboboxTest, NotifyOnClickWithReturnKey) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
@@ -548,16 +534,22 @@ TEST_F(ComboboxTest, NotifyOnClickWithReturnKey) {
   // With STYLE_NORMAL, the click event is ignored.
   SendKeyEvent(ui::VKEY_RETURN);
   EXPECT_FALSE(listener.on_perform_action_called());
+}
+
+TEST_F(ComboboxTest, NotifyOnClickWithReturnKeyActionStyle) {
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
+
+  TestComboboxListener listener;
+  combobox_->set_listener(&listener);
 
   // With STYLE_ACTION, the click event is notified.
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
   SendKeyEvent(ui::VKEY_RETURN);
   EXPECT_TRUE(listener.on_perform_action_called());
   EXPECT_EQ(0, listener.perform_action_index());
 }
 
 TEST_F(ComboboxTest, NotifyOnClickWithSpaceKey) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
@@ -567,9 +559,15 @@ TEST_F(ComboboxTest, NotifyOnClickWithSpaceKey) {
   EXPECT_FALSE(listener.on_perform_action_called());
   SendKeyEventWithType(ui::VKEY_SPACE, ui::ET_KEY_RELEASED);
   EXPECT_FALSE(listener.on_perform_action_called());
+}
+
+TEST_F(ComboboxTest, NotifyOnClickWithSpaceKeyActionStyle) {
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
+
+  TestComboboxListener listener;
+  combobox_->set_listener(&listener);
 
   // With STYLE_ACTION, the click event is notified after releasing.
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
   SendKeyEvent(ui::VKEY_SPACE);
   EXPECT_FALSE(listener.on_perform_action_called());
   SendKeyEventWithType(ui::VKEY_SPACE, ui::ET_KEY_RELEASED);
@@ -578,50 +576,44 @@ TEST_F(ComboboxTest, NotifyOnClickWithSpaceKey) {
 }
 
 TEST_F(ComboboxTest, NotifyOnClickWithMouse) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
 
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
   combobox_->Layout();
 
   // Click the right side (arrow button). The menu is shown.
-  TestMenuRunnerHandler* test_menu_runner_handler = new TestMenuRunnerHandler();
-  scoped_ptr<MenuRunnerHandler> menu_runner_handler(test_menu_runner_handler);
-  scoped_ptr<test::MenuRunnerTestAPI> test_api(
-      new test::MenuRunnerTestAPI(combobox_->dropdown_list_menu_runner_.get()));
-  test_api->SetMenuRunnerHandler(menu_runner_handler.Pass());
-
+  int menu_show_count = 0;
+  test_api_->InstallTestMenuRunner(&menu_show_count);
+  EXPECT_EQ(0, menu_show_count);
   PerformClick(gfx::Point(combobox_->x() + combobox_->width() - 1,
                           combobox_->y() + combobox_->height() / 2));
   EXPECT_FALSE(listener.on_perform_action_called());
-  EXPECT_TRUE(test_menu_runner_handler->executed());
+  EXPECT_EQ(1, menu_show_count);
 
   // Click the left side (text button). The click event is notified.
-  test_menu_runner_handler = new TestMenuRunnerHandler();
-  menu_runner_handler.reset(test_menu_runner_handler);
-  test_api.reset(
-      new test::MenuRunnerTestAPI(combobox_->dropdown_list_menu_runner_.get()));
-  test_api->SetMenuRunnerHandler(menu_runner_handler.Pass());
+  test_api_->InstallTestMenuRunner(&menu_show_count);
   PerformClick(gfx::Point(combobox_->x() + 1,
                           combobox_->y() + combobox_->height() / 2));
   EXPECT_TRUE(listener.on_perform_action_called());
-  EXPECT_FALSE(test_menu_runner_handler->executed());
+  EXPECT_EQ(1, menu_show_count);  // Unchanged.
   EXPECT_EQ(0, listener.perform_action_index());
 }
 
 TEST_F(ComboboxTest, ConsumingPressKeyEvents) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   EXPECT_FALSE(combobox_->OnKeyPressed(
       ui::KeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_RETURN, ui::EF_NONE)));
   EXPECT_FALSE(combobox_->OnKeyPressed(
       ui::KeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_SPACE, ui::EF_NONE)));
+}
 
+TEST_F(ComboboxTest, ConsumingKeyPressEventsActionStyle) {
   // When the combobox's style is STYLE_ACTION, pressing events of a space key
   // or an enter key will be consumed.
-  combobox_->SetStyle(Combobox::STYLE_ACTION);
+  InitCombobox(nullptr, Combobox::STYLE_ACTION);
   EXPECT_TRUE(combobox_->OnKeyPressed(
       ui::KeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_RETURN, ui::EF_NONE)));
   EXPECT_TRUE(combobox_->OnKeyPressed(
@@ -631,7 +623,10 @@ TEST_F(ComboboxTest, ConsumingPressKeyEvents) {
 TEST_F(ComboboxTest, ContentWidth) {
   std::vector<std::string> values;
   VectorComboboxModel model(&values);
-  TestCombobox combobox(&model);
+  TestCombobox combobox(&model, Combobox::STYLE_NORMAL);
+  TestCombobox action_combobox(&model, Combobox::STYLE_ACTION);
+  ComboboxTestApi test_api(&combobox);
+  ComboboxTestApi action_test_api(&action_combobox);
 
   std::string long_item = "this is the long item";
   std::string short_item = "s";
@@ -639,31 +634,76 @@ TEST_F(ComboboxTest, ContentWidth) {
   values.resize(1);
   values[0] = long_item;
   combobox.ModelChanged();
+  action_combobox.ModelChanged();
 
-  const int long_item_width = combobox.content_size_.width();
+  const int long_item_width = test_api.content_size().width();
 
   values[0] = short_item;
   combobox.ModelChanged();
+  action_combobox.ModelChanged();
 
-  const int short_item_width = combobox.content_size_.width();
+  const int short_item_width = test_api.content_size().width();
 
   values.resize(2);
   values[0] = short_item;
   values[1] = long_item;
   combobox.ModelChanged();
+  action_combobox.ModelChanged();
 
   // When the style is STYLE_NORMAL, the width will fit with the longest item.
-  combobox.SetStyle(Combobox::STYLE_NORMAL);
-  EXPECT_EQ(long_item_width, combobox.content_size_.width());
+  EXPECT_EQ(long_item_width, test_api.content_size().width());
 
-  // When the style is STYLE_ACTION, the width will fit with the first items'
+  // When the style is STYLE_ACTION, the width will fit with the selected item's
   // width.
-  combobox.SetStyle(Combobox::STYLE_ACTION);
-  EXPECT_EQ(short_item_width, combobox.content_size_.width());
+  EXPECT_EQ(short_item_width, action_test_api.content_size().width());
+}
+
+// Test that model updates preserve the selected index, so long as it is in
+// range.
+TEST_F(ComboboxTest, ModelChanged) {
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
+
+  EXPECT_EQ(0, combobox_->GetSelectedRow());
+  EXPECT_EQ(10, combobox_->GetRowCount());
+
+  combobox_->SetSelectedIndex(4);
+  EXPECT_EQ(4, combobox_->GetSelectedRow());
+
+  model_->set_item_count(5);
+  combobox_->ModelChanged();
+  EXPECT_EQ(5, combobox_->GetRowCount());
+  EXPECT_EQ(4, combobox_->GetSelectedRow());  // Unchanged.
+
+  model_->set_item_count(4);
+  combobox_->ModelChanged();
+  EXPECT_EQ(4, combobox_->GetRowCount());
+  EXPECT_EQ(0, combobox_->GetSelectedRow());  // Resets.
+
+  // Restore a non-zero selection.
+  combobox_->SetSelectedIndex(2);
+  EXPECT_EQ(2, combobox_->GetSelectedRow());
+
+  // Make the selected index a separator.
+  std::set<int> separators;
+  separators.insert(2);
+  model_->SetSeparators(separators);
+  combobox_->ModelChanged();
+  EXPECT_EQ(4, combobox_->GetRowCount());
+  EXPECT_EQ(0, combobox_->GetSelectedRow());  // Resets.
+
+  // Restore a non-zero selection.
+  combobox_->SetSelectedIndex(1);
+  EXPECT_EQ(1, combobox_->GetSelectedRow());
+
+  // Test an empty model.
+  model_->set_item_count(0);
+  combobox_->ModelChanged();
+  EXPECT_EQ(0, combobox_->GetRowCount());
+  EXPECT_EQ(0, combobox_->GetSelectedRow());  // Resets.
 }
 
 TEST_F(ComboboxTest, TypingPrefixNotifiesListener) {
-  InitCombobox(NULL);
+  InitCombobox(nullptr, Combobox::STYLE_NORMAL);
 
   TestComboboxListener listener;
   combobox_->set_listener(&listener);
@@ -671,13 +711,19 @@ TEST_F(ComboboxTest, TypingPrefixNotifiesListener) {
       widget_->GetInputMethod()->GetTextInputClient();
 
   // Type the first character of the second menu item ("JELLY").
-  input_client->InsertChar('J', ui::EF_NONE);
+  ui::KeyEvent key_event(ui::ET_KEY_PRESSED, ui::VKEY_J, ui::DomCode::US_J, 0,
+                         ui::DomKey::FromCharacter('J'), ui::EventTimeForNow());
+
+  input_client->InsertChar(key_event);
   EXPECT_EQ(1, listener.actions_performed());
   EXPECT_EQ(1, listener.perform_action_index());
 
   // Type the second character of "JELLY", item shouldn't change and
   // OnPerformAction() shouldn't be re-called.
-  input_client->InsertChar('E', ui::EF_NONE);
+  key_event =
+      ui::KeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_E, ui::DomCode::US_E, 0,
+                   ui::DomKey::FromCharacter('E'), ui::EventTimeForNow());
+  input_client->InsertChar(key_event);
   EXPECT_EQ(1, listener.actions_performed());
   EXPECT_EQ(1, listener.perform_action_index());
 
@@ -687,9 +733,68 @@ TEST_F(ComboboxTest, TypingPrefixNotifiesListener) {
 
   // Type the first character of "PEANUT BUTTER", which should change the
   // selected index and perform an action.
-  input_client->InsertChar('P', ui::EF_NONE);
+  key_event =
+      ui::KeyEvent(ui::ET_KEY_PRESSED, ui::VKEY_E, ui::DomCode::US_P, 0,
+                   ui::DomKey::FromCharacter('P'), ui::EventTimeForNow());
+  input_client->InsertChar(key_event);
   EXPECT_EQ(2, listener.actions_performed());
   EXPECT_EQ(2, listener.perform_action_index());
 }
 
+// Test properties on the Combobox menu model.
+TEST_F(ComboboxTest, MenuModel) {
+  const int kSeparatorIndex = 3;
+  std::set<int> separators;
+  separators.insert(kSeparatorIndex);
+  InitCombobox(&separators, Combobox::STYLE_NORMAL);
+
+  ui::MenuModel* menu_model = test_api_->menu_model();
+
+  EXPECT_EQ(TestComboboxModel::kItemCount, menu_model->GetItemCount());
+  EXPECT_EQ(ui::MenuModel::TYPE_SEPARATOR,
+            menu_model->GetTypeAt(kSeparatorIndex));
+
+#if defined(OS_MACOSX)
+  // Comboboxes on Mac should have checkmarks, with the selected item checked,
+  EXPECT_EQ(ui::MenuModel::TYPE_CHECK, menu_model->GetTypeAt(0));
+  EXPECT_EQ(ui::MenuModel::TYPE_CHECK, menu_model->GetTypeAt(1));
+  EXPECT_TRUE(menu_model->IsItemCheckedAt(0));
+  EXPECT_FALSE(menu_model->IsItemCheckedAt(1));
+
+  combobox_->SetSelectedIndex(1);
+  EXPECT_FALSE(menu_model->IsItemCheckedAt(0));
+  EXPECT_TRUE(menu_model->IsItemCheckedAt(1));
+#else
+  EXPECT_EQ(ui::MenuModel::TYPE_COMMAND, menu_model->GetTypeAt(0));
+  EXPECT_EQ(ui::MenuModel::TYPE_COMMAND, menu_model->GetTypeAt(1));
+#endif
+
+  EXPECT_EQ(ASCIIToUTF16("PEANUT BUTTER"), menu_model->GetLabelAt(0));
+  EXPECT_EQ(ASCIIToUTF16("JELLY"), menu_model->GetLabelAt(1));
+
+  EXPECT_TRUE(menu_model->IsVisibleAt(0));
+}
+
+// Check that with STYLE_ACTION, the first item (only) is not shown.
+TEST_F(ComboboxTest, MenuModelActionStyleMac) {
+  const int kSeparatorIndex = 3;
+  std::set<int> separators;
+  separators.insert(kSeparatorIndex);
+  InitCombobox(&separators, Combobox::STYLE_ACTION);
+
+  ui::MenuModel* menu_model = test_api_->menu_model();
+
+  EXPECT_EQ(TestComboboxModel::kItemCount, menu_model->GetItemCount());
+  EXPECT_EQ(ui::MenuModel::TYPE_SEPARATOR,
+            menu_model->GetTypeAt(kSeparatorIndex));
+
+  EXPECT_EQ(ui::MenuModel::TYPE_COMMAND, menu_model->GetTypeAt(0));
+  EXPECT_EQ(ui::MenuModel::TYPE_COMMAND, menu_model->GetTypeAt(1));
+
+  EXPECT_EQ(ASCIIToUTF16("PEANUT BUTTER"), menu_model->GetLabelAt(0));
+  EXPECT_EQ(ASCIIToUTF16("JELLY"), menu_model->GetLabelAt(1));
+  EXPECT_FALSE(menu_model->IsVisibleAt(0));
+  EXPECT_TRUE(menu_model->IsVisibleAt(1));
+}
+
 }  // namespace views
