@@ -5,16 +5,13 @@
 #ifndef UI_VIEWS_CONTROLS_MENU_MENU_MESSAGE_LOOP_AURA_H_
 #define UI_VIEWS_CONTROLS_MENU_MENU_MESSAGE_LOOP_AURA_H_
 
+#include <memory>
+
 #include "base/callback.h"
 #include "base/compiler_specific.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
 #include "ui/views/controls/menu/menu_message_loop.h"
 
-namespace base {
-class MessagePumpDispatcher;
-}
-
 namespace ui {
 class ScopedEventDispatcher;
 }
