@@ -6,6 +6,7 @@
 
 #include <stddef.h>
 
+#include <algorithm>
 #include <map>
 #include <utility>
 
@@ -145,7 +146,8 @@ TableView::TableView(ui::TableModel* model,
     visible_column.column = columns[i];
     visible_columns_.push_back(visible_column);
   }
-  SetFocusable(true);
+  // Always focusable, even on Mac (consistent with NSTableView).
+  SetFocusBehavior(FocusBehavior::ALWAYS);
   SetModel(model);
 }
 
@@ -178,7 +180,7 @@ View* TableView::CreateParentIfNecessary() {
 }
 
 void TableView::SetRowBackgroundPainter(
-    scoped_ptr<TableViewRowBackgroundPainter> painter) {
+    std::unique_ptr<TableViewRowBackgroundPainter> painter) {
   row_background_painter_ = std::move(painter);
 }
 
@@ -659,9 +661,11 @@ void TableView::SortItemsAndUpdateMapping() {
       GroupSortHelper sort_helper(this);
       GetModelIndexToRangeStart(grouper_, RowCount(),
                                 &sort_helper.model_index_to_range_start);
-      std::sort(view_to_model_.begin(), view_to_model_.end(), sort_helper);
+      std::stable_sort(view_to_model_.begin(), view_to_model_.end(),
+                       sort_helper);
     } else {
-      std::sort(view_to_model_.begin(), view_to_model_.end(), SortHelper(this));
+      std::stable_sort(view_to_model_.begin(), view_to_model_.end(),
+                       SortHelper(this));
     }
     for (int i = 0; i < row_count; ++i)
       model_to_view_[view_to_model_[i]] = i;
