@@ -4,14 +4,22 @@
 
 #include "ui/views/controls/textfield/textfield.h"
 
+#include <stddef.h>
+#include <stdint.h>
+
 #include <set>
 #include <string>
 #include <vector>
 
 #include "base/command_line.h"
+#include "base/format_macros.h"
+#include "base/i18n/rtl.h"
+#include "base/macros.h"
 #include "base/pickle.h"
 #include "base/strings/string16.h"
+#include "base/strings/stringprintf.h"
 #include "base/strings/utf_string_conversions.h"
+#include "build/build_config.h"
 #include "ui/accessibility/ax_view_state.h"
 #include "ui/base/clipboard/clipboard.h"
 #include "ui/base/clipboard/scoped_clipboard_writer.h"
@@ -20,7 +28,6 @@
 #include "ui/base/ime/input_method_delegate.h"
 #include "ui/base/ime/input_method_factory.h"
 #include "ui/base/ime/text_input_client.h"
-#include "ui/base/l10n/l10n_util.h"
 #include "ui/base/ui_base_switches.h"
 #include "ui/base/ui_base_switches_util.h"
 #include "ui/events/event.h"
@@ -34,6 +41,7 @@
 #include "ui/views/controls/textfield/textfield_model.h"
 #include "ui/views/controls/textfield/textfield_test_api.h"
 #include "ui/views/focus/focus_manager.h"
+#include "ui/views/style/platform_style.h"
 #include "ui/views/test/test_views_delegate.h"
 #include "ui/views/test/views_test_base.h"
 #include "ui/views/test/widget_test.h"
@@ -45,7 +53,7 @@
 #endif
 
 #if defined(OS_LINUX) && !defined(OS_CHROMEOS)
-#include "ui/events/linux/text_edit_key_bindings_delegate_auralinux.h"
+#include "ui/base/ime/linux/text_edit_key_bindings_delegate_auralinux.h"
 #endif
 
 #if defined(USE_X11)
@@ -70,13 +78,12 @@ class MockInputMethod : public ui::InputMethodBase {
   // Overridden from InputMethod:
   bool OnUntranslatedIMEMessage(const base::NativeEvent& event,
                                 NativeEventResult* result) override;
-  bool DispatchKeyEvent(const ui::KeyEvent& key) override;
+  void DispatchKeyEvent(ui::KeyEvent* key) override;
   void OnTextInputTypeChanged(const ui::TextInputClient* client) override;
   void OnCaretBoundsChanged(const ui::TextInputClient* client) override {}
   void CancelComposition(const ui::TextInputClient* client) override;
   void OnInputLocaleChanged() override {}
   std::string GetInputLocale() override;
-  bool IsActive() override;
   bool IsCandidatePopupOpen() const override;
   void ShowImeIfNeeded() override {}
 
@@ -139,19 +146,33 @@ bool MockInputMethod::OnUntranslatedIMEMessage(const base::NativeEvent& event,
   return false;
 }
 
-bool MockInputMethod::DispatchKeyEvent(const ui::KeyEvent& key) {
+void MockInputMethod::DispatchKeyEvent(ui::KeyEvent* key) {
+// On Mac, emulate InputMethodMac behavior for character events. Composition
+// still needs to be mocked, since it's not possible to generate test events
+// which trigger the appropriate NSResponder action messages for composition.
+#if defined(OS_MACOSX)
+  if (key->is_char()) {
+    ignore_result(DispatchKeyEventPostIME(key));
+    return;
+  }
+#endif
+
   // Checks whether the key event is from EventGenerator on Windows which will
   // generate key event for WM_CHAR.
   // The MockInputMethod will insert char on WM_KEYDOWN so ignore WM_CHAR here.
-  if (key.is_char() && key.HasNativeEvent())
-    return true;
+  if (key->is_char() && key->HasNativeEvent()) {
+    key->SetHandled();
+    return;
+  }
 
   bool handled = !IsTextInputTypeNone() && HasComposition();
   ClearStates();
   if (handled) {
-    DCHECK(!key.is_char());
-    ui::KeyEvent mock_key(ui::ET_KEY_PRESSED, ui::VKEY_PROCESSKEY, key.flags());
-    DispatchKeyEventPostIME(mock_key);
+    DCHECK(!key->is_char());
+    ui::KeyEvent mock_key(ui::ET_KEY_PRESSED,
+                          ui::VKEY_PROCESSKEY,
+                          key->flags());
+    DispatchKeyEventPostIME(&mock_key);
   } else {
     DispatchKeyEventPostIME(key);
   }
@@ -165,15 +186,14 @@ bool MockInputMethod::DispatchKeyEvent(const ui::KeyEvent& key) {
         client->SetCompositionText(composition_);
       else
         client->ClearCompositionText();
-    } else if (key.type() == ui::ET_KEY_PRESSED) {
-      base::char16 ch = key.GetCharacter();
+    } else if (key->type() == ui::ET_KEY_PRESSED) {
+      base::char16 ch = key->GetCharacter();
       if (ch)
-        client->InsertChar(ch, key.flags());
+        client->InsertChar(*key);
     }
   }
 
   ClearComposition();
-  return true;
 }
 
 void MockInputMethod::OnTextInputTypeChanged(
@@ -194,10 +214,6 @@ std::string MockInputMethod::GetInputLocale() {
   return "en-US";
 }
 
-bool MockInputMethod::IsActive() {
-  return true;
-}
-
 bool MockInputMethod::IsCandidatePopupOpen() const {
   return false;
 }
@@ -273,8 +289,8 @@ class TestTextfield : public views::Textfield {
   }
 
   // ui::TextInputClient overrides:
-  void InsertChar(base::char16 ch, int flags) override {
-    views::Textfield::InsertChar(ch, flags);
+  void InsertChar(const ui::KeyEvent& e) override {
+    views::Textfield::InsertChar(e);
 #if defined(OS_MACOSX)
     // On Mac, characters are inserted directly rather than attempting to get a
     // unicode character from the ui::KeyEvent (which isn't always possible).
@@ -300,7 +316,7 @@ class TestTextfield : public views::Textfield {
 class GestureEventForTest : public ui::GestureEvent {
  public:
   GestureEventForTest(int x, int y, ui::GestureEventDetails details)
-      : GestureEvent(x, y, 0, base::TimeDelta(), details) {}
+      : GestureEvent(x, y, 0, base::TimeTicks(), details) {}
 
  private:
   DISALLOW_COPY_AND_ASSIGN(GestureEventForTest);
@@ -320,13 +336,14 @@ class TextfieldDestroyerController : public views::TextfieldController {
   // views::TextfieldController:
   bool HandleKeyEvent(views::Textfield* sender,
                       const ui::KeyEvent& key_event) override {
-    target_->OnBlur();
+    if (target_)
+      target_->OnBlur();
     target_.reset();
     return false;
   }
 
  private:
-  scoped_ptr<views::Textfield> target_;
+  std::unique_ptr<views::Textfield> target_;
 };
 
 base::string16 GetClipboardText(ui::ClipboardType type) {
@@ -433,16 +450,9 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
     widget_->Show();
     textfield_->RequestFocus();
 
-    // On Mac, activation is asynchronous since desktop widgets are used. We
-    // don't want parallel tests to steal active status either, so fake it.
-#if defined(OS_MACOSX) && !defined(USE_AURA)
-    fake_activation_ = test::WidgetTest::FakeWidgetIsActiveAlways();
-#endif
-
     event_generator_.reset(
-        new ui::test::EventGenerator(GetContext(), widget_->GetNativeWindow()));
+        new ui::test::EventGenerator(widget_->GetNativeWindow()));
   }
-
   ui::MenuModel* GetContextMenuModel() {
     test_api_->UpdateContextMenu();
     return test_api_->context_menu_contents();
@@ -457,7 +467,19 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
 #endif
   }
 
+  bool TestingNativeCrOs() const {
+#if defined(OS_CHROMEOS)
+    return true;
+#else
+    return false;
+#endif  // defined(OS_CHROMEOS)
+  }
+
  protected:
+  void SendKeyPress(ui::KeyboardCode key_code, int flags) {
+    event_generator_->PressKey(key_code, flags);
+  }
+
   void SendKeyEvent(ui::KeyboardCode key_code,
                     bool alt,
                     bool shift,
@@ -471,12 +493,13 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
     if (TestingNativeMac())
       std::swap(control, command);
 
-    int flags = (alt ? ui::EF_ALT_DOWN : 0) | (shift ? ui::EF_SHIFT_DOWN : 0) |
+    int flags = (shift ? ui::EF_SHIFT_DOWN : 0) |
                 (control ? ui::EF_CONTROL_DOWN : 0) |
+                (alt ? ui::EF_ALT_DOWN : 0) |
                 (command ? ui::EF_COMMAND_DOWN : 0) |
-                (caps_lock ? ui::EF_CAPS_LOCK_DOWN : 0);
+                (caps_lock ? ui::EF_CAPS_LOCK_ON : 0);
 
-    event_generator_->PressKey(key_code, flags);
+    SendKeyPress(key_code, flags);
   }
 
   void SendKeyEvent(ui::KeyboardCode key_code,
@@ -497,27 +520,37 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
       SendKeyEvent(code);
     } else {
       // For unicode characters, assume they come from IME rather than the
-      // keyboard. So they are dispatched directly to the input method.
+      // keyboard. So they are dispatched directly to the input method. But on
+      // Mac, key events don't pass through InputMethod. Hence they are
+      // dispatched regularly.
       ui::KeyEvent event(ch, ui::VKEY_UNKNOWN, ui::EF_NONE);
-      input_method_->DispatchKeyEvent(event);
+#if defined(OS_MACOSX)
+      event_generator_->Dispatch(&event);
+#else
+      input_method_->DispatchKeyEvent(&event);
+#endif
     }
   }
 
-  // Sends a platform-specific move (and select) to start of line.
+  // Sends a platform-specific move (and select) to the logical start of line.
+  // Eg. this should move (and select) to the right end of line for RTL text.
   void SendHomeEvent(bool shift) {
     if (TestingNativeMac()) {
-      // Use Cmd+Left on native Mac. An RTL-agnostic "end" doesn't have a
-      // default key-binding on Mac.
-      SendKeyEvent(ui::VKEY_LEFT, shift /* shift */, true /* command */);
+      // [NSResponder moveToBeginningOfLine:] is the correct way to do this on
+      // Mac, but that doesn't have a default key binding. Since
+      // views::Textfield doesn't currently support multiple lines, the same
+      // effect can be achieved by Cmd+Up which maps to
+      // [NSResponder moveToBeginningOfDocument:].
+      SendKeyEvent(ui::VKEY_UP, shift /* shift */, true /* command */);
       return;
     }
     SendKeyEvent(ui::VKEY_HOME, shift /* shift */, false /* control */);
   }
 
-  // Sends a platform-specific move (and select) to end of line.
+  // Sends a platform-specific move (and select) to the logical end of line.
   void SendEndEvent(bool shift) {
     if (TestingNativeMac()) {
-      SendKeyEvent(ui::VKEY_RIGHT, shift, true);  // Cmd+Right.
+      SendKeyEvent(ui::VKEY_DOWN, shift, true);  // Cmd+Down.
       return;
     }
     SendKeyEvent(ui::VKEY_END, shift, false);
@@ -625,11 +658,49 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
     EXPECT_TRUE(menu->IsEnabledAt(7 /* SELECT ALL */));
   }
 
+  void PressLeftMouseButton(int extra_flags) {
+    ui::MouseEvent click(ui::ET_MOUSE_PRESSED, mouse_position_, mouse_position_,
+                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
+                         ui::EF_LEFT_MOUSE_BUTTON | extra_flags);
+    textfield_->OnMousePressed(click);
+  }
+
+  void PressLeftMouseButton() {
+    PressLeftMouseButton(0);
+  }
+
+  void ReleaseLeftMouseButton() {
+    ui::MouseEvent release(ui::ET_MOUSE_RELEASED, mouse_position_,
+                           mouse_position_, ui::EventTimeForNow(),
+                           ui::EF_LEFT_MOUSE_BUTTON, ui::EF_LEFT_MOUSE_BUTTON);
+    textfield_->OnMouseReleased(release);
+  }
+
+  void ClickLeftMouseButton(int extra_flags) {
+    PressLeftMouseButton(extra_flags);
+    ReleaseLeftMouseButton();
+  }
+
+  void ClickLeftMouseButton() {
+    ClickLeftMouseButton(0);
+  }
+
+  void DragMouseTo(const gfx::Point& where) {
+    mouse_position_ = where;
+    ui::MouseEvent drag(ui::ET_MOUSE_DRAGGED, where, where,
+                        ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON, 0);
+    textfield_->OnMouseDragged(drag);
+  }
+
+  // Textfield does not listen to OnMouseMoved, so this function does not send
+  // an event when it updates the cursor position.
+  void MoveMouseTo(const gfx::Point& where) { mouse_position_ = where; }
+
   // We need widget to populate wrapper class.
   Widget* widget_;
 
   TestTextfield* textfield_;
-  scoped_ptr<TextfieldTestApi> test_api_;
+  std::unique_ptr<TextfieldTestApi> test_api_;
   TextfieldModel* model_;
 
   // The string from Controller::ContentsChanged callback.
@@ -645,10 +716,10 @@ class TextfieldTest : public ViewsTestBase, public TextfieldController {
   int on_after_user_action_;
 
  private:
+  // Position of the mouse for synthetic mouse events.
+  gfx::Point mouse_position_;
   ui::ClipboardType copied_to_clipboard_;
-  scoped_ptr<test::WidgetTest::FakeActivation> fake_activation_;
-  scoped_ptr<ui::test::EventGenerator> event_generator_;
-
+  std::unique_ptr<ui::test::EventGenerator> event_generator_;
   DISALLOW_COPY_AND_ASSIGN(TextfieldTest);
 };
 
@@ -694,6 +765,38 @@ TEST_F(TextfieldTest, KeyTest) {
     EXPECT_STR_EQ("TexT!1!1", textfield_->text());
 }
 
+TEST_F(TextfieldTest, KeysWithModifiersTest) {
+  InitTextfield();
+  const int ctrl = ui::EF_CONTROL_DOWN;
+  const int alt = ui::EF_ALT_DOWN;
+  const int command = ui::EF_COMMAND_DOWN;
+  const int altgr = ui::EF_ALTGR_DOWN;
+  const int shift = ui::EF_SHIFT_DOWN;
+
+  SendKeyPress(ui::VKEY_T, shift | alt | altgr);
+  SendKeyPress(ui::VKEY_H, alt);
+  SendKeyPress(ui::VKEY_E, altgr);
+  SendKeyPress(ui::VKEY_T, shift);
+  SendKeyPress(ui::VKEY_E, shift | altgr);
+  SendKeyPress(ui::VKEY_X, 0);
+  SendKeyPress(ui::VKEY_T, ctrl);  // This causes transpose on Mac.
+  SendKeyPress(ui::VKEY_1, alt);
+  SendKeyPress(ui::VKEY_2, command);
+  SendKeyPress(ui::VKEY_3, 0);
+  SendKeyPress(ui::VKEY_4, 0);
+  SendKeyPress(ui::VKEY_OEM_PLUS, ctrl);
+  SendKeyPress(ui::VKEY_OEM_PLUS, ctrl | shift);
+  SendKeyPress(ui::VKEY_OEM_MINUS, ctrl);
+  SendKeyPress(ui::VKEY_OEM_MINUS, ctrl | shift);
+
+  if (TestingNativeCrOs())
+    EXPECT_STR_EQ("TeTEx34", textfield_->text());
+  else if (TestingNativeMac())
+    EXPECT_STR_EQ("TheTxE134", textfield_->text());
+  else
+    EXPECT_STR_EQ("TeTEx234", textfield_->text());
+}
+
 TEST_F(TextfieldTest, ControlAndSelectTest) {
   // Insert a test string in a textfield.
   InitTextfield();
@@ -787,6 +890,33 @@ TEST_F(TextfieldTest, InsertionDeletionTest) {
 #endif
 }
 
+// Test that deletion operations behave correctly with an active selection.
+TEST_F(TextfieldTest, DeletionWithSelection) {
+  struct {
+    ui::KeyboardCode key;
+    bool shift;
+  } cases[] = {
+      {ui::VKEY_BACK, false},
+      {ui::VKEY_BACK, true},
+      {ui::VKEY_DELETE, false},
+      {ui::VKEY_DELETE, true},
+  };
+
+  InitTextfield();
+  // [Ctrl] ([Alt] on Mac) + [Delete]/[Backspace] should delete the active
+  // selection, regardless of [Shift].
+  for (size_t i = 0; i < arraysize(cases); ++i) {
+    SCOPED_TRACE(base::StringPrintf("Testing cases[%" PRIuS "]", i));
+    textfield_->SetText(ASCIIToUTF16("one two three"));
+    textfield_->SelectRange(gfx::Range(2, 6));
+    // Make selection as - on|e tw|o three.
+    SendWordEvent(cases[i].key, cases[i].shift);
+    // Verify state is on|o three.
+    EXPECT_STR_EQ("ono three", textfield_->text());
+    EXPECT_EQ(gfx::Range(2), textfield_->GetSelectedRange());
+  }
+}
+
 TEST_F(TextfieldTest, PasswordTest) {
   InitTextfield();
   textfield_->SetTextInputType(ui::TEXT_INPUT_TYPE_PASSWORD);
@@ -825,6 +955,31 @@ TEST_F(TextfieldTest, PasswordTest) {
   EXPECT_STR_EQ("foofoofoo", textfield_->text());
 }
 
+// Check that text insertion works appropriately for password and read-only
+// textfields.
+TEST_F(TextfieldTest, TextInputType_InsertionTest) {
+  InitTextfield();
+  textfield_->SetTextInputType(ui::TEXT_INPUT_TYPE_PASSWORD);
+  EXPECT_EQ(ui::TEXT_INPUT_TYPE_PASSWORD, textfield_->GetTextInputType());
+
+  SendKeyEvent(ui::VKEY_A);
+  SendKeyEvent(kHebrewLetterSamekh);
+  SendKeyEvent(ui::VKEY_B);
+
+  EXPECT_EQ(WideToUTF16(L"a\x05E1"
+                        L"b"),
+            textfield_->text());
+
+  textfield_->SetReadOnly(true);
+  EXPECT_EQ(ui::TEXT_INPUT_TYPE_NONE, textfield_->GetTextInputType());
+  SendKeyEvent(ui::VKEY_C);
+
+  // No text should be inserted for read only textfields.
+  EXPECT_EQ(WideToUTF16(L"a\x05E1"
+                        L"b"),
+            textfield_->text());
+}
+
 TEST_F(TextfieldTest, TextInputType) {
   InitTextfield();
 
@@ -883,7 +1038,7 @@ TEST_F(TextfieldTest, OnKeyPress) {
   EXPECT_TRUE(textfield_->key_handled());
   textfield_->clear();
 
-  // F20, up/down key won't be handled.
+  // F20 key won't be handled.
   SendKeyEvent(ui::VKEY_F20);
 #if defined(OS_MACOSX)
   // On Mac, key combinations that don't map to editing commands are forwarded
@@ -895,14 +1050,24 @@ TEST_F(TextfieldTest, OnKeyPress) {
   EXPECT_FALSE(textfield_->key_handled());
   textfield_->clear();
 
+  // Up/Down keys won't be handled except on Mac where they map to move
+  // commands.
   SendKeyEvent(ui::VKEY_UP);
   EXPECT_TRUE(textfield_->key_received());
+#if defined(OS_MACOSX)
+  EXPECT_TRUE(textfield_->key_handled());
+#else
   EXPECT_FALSE(textfield_->key_handled());
+#endif
   textfield_->clear();
 
   SendKeyEvent(ui::VKEY_DOWN);
   EXPECT_TRUE(textfield_->key_received());
+#if defined(OS_MACOSX)
+  EXPECT_TRUE(textfield_->key_handled());
+#else
   EXPECT_FALSE(textfield_->key_handled());
+#endif
   textfield_->clear();
 }
 
@@ -1034,10 +1199,8 @@ TEST_F(TextfieldTest, FocusTraversalTest) {
   // Test if clicking on textfield view sets the focus.
   widget_->GetFocusManager()->AdvanceFocus(true);
   EXPECT_EQ(3, GetFocusedView()->id());
-  ui::MouseEvent click(ui::ET_MOUSE_PRESSED, gfx::Point(), gfx::Point(),
-                       ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                       ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(click);
+  MoveMouseTo(gfx::Point());
+  ClickLeftMouseButton();
   EXPECT_EQ(1, GetFocusedView()->id());
 
   // Tab/Shift+Tab should also cycle focus, not insert a tab character.
@@ -1086,33 +1249,20 @@ TEST_F(TextfieldTest, ContextMenuDisplayTest) {
 TEST_F(TextfieldTest, DoubleAndTripleClickTest) {
   InitTextfield();
   textfield_->SetText(ASCIIToUTF16("hello world"));
-  ui::MouseEvent click(ui::ET_MOUSE_PRESSED, gfx::Point(), gfx::Point(),
-                       ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                       ui::EF_LEFT_MOUSE_BUTTON);
-  ui::MouseEvent release(ui::ET_MOUSE_RELEASED, gfx::Point(), gfx::Point(),
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  ui::MouseEvent double_click(ui::ET_MOUSE_PRESSED, gfx::Point(), gfx::Point(),
-                              ui::EventTimeForNow(),
-                              ui::EF_LEFT_MOUSE_BUTTON | ui::EF_IS_DOUBLE_CLICK,
-                              ui::EF_LEFT_MOUSE_BUTTON);
 
   // Test for double click.
-  textfield_->OnMousePressed(click);
-  textfield_->OnMouseReleased(release);
+  MoveMouseTo(gfx::Point());
+  ClickLeftMouseButton();
   EXPECT_TRUE(textfield_->GetSelectedText().empty());
-  textfield_->OnMousePressed(double_click);
-  textfield_->OnMouseReleased(release);
+  ClickLeftMouseButton(ui::EF_IS_DOUBLE_CLICK);
   EXPECT_STR_EQ("hello", textfield_->GetSelectedText());
 
   // Test for triple click.
-  textfield_->OnMousePressed(click);
-  textfield_->OnMouseReleased(release);
+  ClickLeftMouseButton();
   EXPECT_STR_EQ("hello world", textfield_->GetSelectedText());
 
   // Another click should reset back to double click.
-  textfield_->OnMousePressed(click);
-  textfield_->OnMouseReleased(release);
+  ClickLeftMouseButton();
   EXPECT_STR_EQ("hello", textfield_->GetSelectedText());
 }
 
@@ -1123,39 +1273,70 @@ TEST_F(TextfieldTest, DragToSelect) {
   const int kEnd = 500;
   gfx::Point start_point(kStart, 0);
   gfx::Point end_point(kEnd, 0);
-  ui::MouseEvent click_a(ui::ET_MOUSE_PRESSED, start_point, start_point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  ui::MouseEvent click_b(ui::ET_MOUSE_PRESSED, end_point, end_point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  ui::MouseEvent drag_left(ui::ET_MOUSE_DRAGGED, gfx::Point(), gfx::Point(),
-                           ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON, 0);
-  ui::MouseEvent drag_right(ui::ET_MOUSE_DRAGGED, end_point, end_point,
-                            ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON, 0);
-  ui::MouseEvent release(ui::ET_MOUSE_RELEASED, end_point, end_point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(click_a);
+
+  MoveMouseTo(start_point);
+  PressLeftMouseButton();
   EXPECT_TRUE(textfield_->GetSelectedText().empty());
+
   // Check that dragging left selects the beginning of the string.
-  textfield_->OnMouseDragged(drag_left);
+  DragMouseTo(gfx::Point());
   base::string16 text_left = textfield_->GetSelectedText();
   EXPECT_STR_EQ("hello", text_left);
+
   // Check that dragging right selects the rest of the string.
-  textfield_->OnMouseDragged(drag_right);
+  DragMouseTo(end_point);
   base::string16 text_right = textfield_->GetSelectedText();
   EXPECT_STR_EQ(" world", text_right);
+
   // Check that releasing in the same location does not alter the selection.
-  textfield_->OnMouseReleased(release);
+  ReleaseLeftMouseButton();
   EXPECT_EQ(text_right, textfield_->GetSelectedText());
+
   // Check that dragging from beyond the text length works too.
-  textfield_->OnMousePressed(click_b);
-  textfield_->OnMouseDragged(drag_left);
-  textfield_->OnMouseReleased(release);
+  MoveMouseTo(end_point);
+  PressLeftMouseButton();
+  DragMouseTo(gfx::Point());
+  ReleaseLeftMouseButton();
   EXPECT_EQ(textfield_->text(), textfield_->GetSelectedText());
 }
 
+// This test checks that dragging above the textfield selects to the beginning
+// and dragging below the textfield selects to the end, but only on platforms
+// where that is the expected behavior.
+TEST_F(TextfieldTest, DragUpOrDownSelectsToEnd) {
+  InitTextfield();
+  textfield_->SetText(ASCIIToUTF16("hello world"));
+  const base::string16 expected_up = base::ASCIIToUTF16(
+      PlatformStyle::kTextfieldDragVerticallyDragsToEnd ? "hello" : "lo");
+  const base::string16 expected_down = base::ASCIIToUTF16(
+      PlatformStyle::kTextfieldDragVerticallyDragsToEnd ? " world" : " w");
+  const int kStartX = GetCursorPositionX(5);
+  const int kDownX = GetCursorPositionX(7);
+  const int kUpX = GetCursorPositionX(3);
+  gfx::Point start_point(kStartX, 0);
+  gfx::Point down_point(kDownX, 500);
+  gfx::Point up_point(kUpX, -500);
+
+  MoveMouseTo(start_point);
+  PressLeftMouseButton();
+  DragMouseTo(up_point);
+  ReleaseLeftMouseButton();
+  EXPECT_EQ(textfield_->GetSelectedText(), expected_up);
+
+  // Click at |up_point|. This is important because drags do not count as clicks
+  // for the purpose of double-click detection, so if this test doesn't click
+  // somewhere other than |start_point| before the code below runs, the second
+  // click at |start_point| will be interpreted as a double-click instead of the
+  // start of a drag.
+  ClickLeftMouseButton();
+
+  MoveMouseTo(start_point);
+  PressLeftMouseButton();
+  DragMouseTo(down_point);
+  ReleaseLeftMouseButton();
+  EXPECT_EQ(textfield_->GetSelectedText(), expected_down);
+}
+
 #if defined(OS_WIN)
 TEST_F(TextfieldTest, DragAndDrop_AcceptDrop) {
   InitTextfield();
@@ -1165,28 +1346,28 @@ TEST_F(TextfieldTest, DragAndDrop_AcceptDrop) {
   base::string16 string(ASCIIToUTF16("string "));
   data.SetString(string);
   int formats = 0;
-  std::set<OSExchangeData::CustomFormat> custom_formats;
+  std::set<ui::Clipboard::FormatType> format_types;
 
   // Ensure that disabled textfields do not accept drops.
   textfield_->SetEnabled(false);
-  EXPECT_FALSE(textfield_->GetDropFormats(&formats, &custom_formats));
+  EXPECT_FALSE(textfield_->GetDropFormats(&formats, &format_types));
   EXPECT_EQ(0, formats);
-  EXPECT_TRUE(custom_formats.empty());
+  EXPECT_TRUE(format_types.empty());
   EXPECT_FALSE(textfield_->CanDrop(data));
   textfield_->SetEnabled(true);
 
   // Ensure that read-only textfields do not accept drops.
   textfield_->SetReadOnly(true);
-  EXPECT_FALSE(textfield_->GetDropFormats(&formats, &custom_formats));
+  EXPECT_FALSE(textfield_->GetDropFormats(&formats, &format_types));
   EXPECT_EQ(0, formats);
-  EXPECT_TRUE(custom_formats.empty());
+  EXPECT_TRUE(format_types.empty());
   EXPECT_FALSE(textfield_->CanDrop(data));
   textfield_->SetReadOnly(false);
 
   // Ensure that enabled and editable textfields do accept drops.
-  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &custom_formats));
+  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &format_types));
   EXPECT_EQ(ui::OSExchangeData::STRING, formats);
-  EXPECT_TRUE(custom_formats.empty());
+  EXPECT_TRUE(format_types.empty());
   EXPECT_TRUE(textfield_->CanDrop(data));
   gfx::Point drop_point(GetCursorPositionX(6), 0);
   ui::DropTargetEvent drop(data, drop_point, drop_point,
@@ -1199,7 +1380,7 @@ TEST_F(TextfieldTest, DragAndDrop_AcceptDrop) {
   // Ensure that textfields do not accept non-OSExchangeData::STRING types.
   ui::OSExchangeData bad_data;
   bad_data.SetFilename(base::FilePath(FILE_PATH_LITERAL("x")));
-  ui::OSExchangeData::CustomFormat fmt = ui::Clipboard::GetBitmapFormatType();
+  ui::Clipboard::FormatType fmt = ui::Clipboard::GetBitmapFormatType();
   bad_data.SetPickledData(fmt, base::Pickle());
   bad_data.SetFileContents(base::FilePath(L"x"), "x");
   bad_data.SetHtml(base::string16(ASCIIToUTF16("x")), GURL("x.org"));
@@ -1238,11 +1419,9 @@ TEST_F(TextfieldTest, DragAndDrop_InitiateDrag) {
   EXPECT_EQ(ui::DragDropTypes::DRAG_NONE,
             textfield_->GetDragOperationsForView(NULL, kStringPoint));
   textfield_->SetTextInputType(ui::TEXT_INPUT_TYPE_TEXT);
+  MoveMouseTo(kStringPoint);
+  PressLeftMouseButton();
   // Ensure that textfields only initiate drag operations inside the selection.
-  ui::MouseEvent press_event(ui::ET_MOUSE_PRESSED, kStringPoint, kStringPoint,
-                             ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                             ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(press_event);
   EXPECT_EQ(ui::DragDropTypes::DRAG_NONE,
             textfield_->GetDragOperationsForView(NULL, gfx::Point()));
   EXPECT_FALSE(textfield_->CanStartDragForView(NULL, gfx::Point(),
@@ -1264,27 +1443,23 @@ TEST_F(TextfieldTest, DragAndDrop_ToTheRight) {
   ui::OSExchangeData data;
   int formats = 0;
   int operations = 0;
-  std::set<OSExchangeData::CustomFormat> custom_formats;
+  std::set<ui::Clipboard::FormatType> format_types;
 
   // Start dragging "ello".
   textfield_->SelectRange(gfx::Range(1, 5));
   gfx::Point point(GetCursorPositionX(3), 0);
-  ui::MouseEvent click_a(ui::ET_MOUSE_PRESSED, point, point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(click_a);
-  EXPECT_TRUE(textfield_->CanStartDragForView(textfield_, click_a.location(),
-                                              gfx::Point()));
-  operations = textfield_->GetDragOperationsForView(textfield_,
-                                                    click_a.location());
+  MoveMouseTo(point);
+  PressLeftMouseButton();
+  EXPECT_TRUE(textfield_->CanStartDragForView(textfield_, point, point));
+  operations = textfield_->GetDragOperationsForView(textfield_, point);
   EXPECT_EQ(ui::DragDropTypes::DRAG_MOVE | ui::DragDropTypes::DRAG_COPY,
             operations);
-  textfield_->WriteDragDataForView(NULL, click_a.location(), &data);
+  textfield_->WriteDragDataForView(nullptr, point, &data);
   EXPECT_TRUE(data.GetString(&string));
   EXPECT_EQ(textfield_->GetSelectedText(), string);
-  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &custom_formats));
+  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &format_types));
   EXPECT_EQ(ui::OSExchangeData::STRING, formats);
-  EXPECT_TRUE(custom_formats.empty());
+  EXPECT_TRUE(format_types.empty());
 
   // Drop "ello" after "w".
   const gfx::Point kDropPoint(GetCursorPositionX(7), 0);
@@ -1318,27 +1493,23 @@ TEST_F(TextfieldTest, DragAndDrop_ToTheLeft) {
   ui::OSExchangeData data;
   int formats = 0;
   int operations = 0;
-  std::set<OSExchangeData::CustomFormat> custom_formats;
+  std::set<ui::Clipboard::FormatType> format_types;
 
   // Start dragging " worl".
   textfield_->SelectRange(gfx::Range(5, 10));
   gfx::Point point(GetCursorPositionX(7), 0);
-  ui::MouseEvent click_a(ui::ET_MOUSE_PRESSED, point, point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(click_a);
-  EXPECT_TRUE(textfield_->CanStartDragForView(textfield_, click_a.location(),
-                                              gfx::Point()));
-  operations = textfield_->GetDragOperationsForView(textfield_,
-                                                    click_a.location());
+  MoveMouseTo(point);
+  PressLeftMouseButton();
+  EXPECT_TRUE(textfield_->CanStartDragForView(textfield_, point, gfx::Point()));
+  operations = textfield_->GetDragOperationsForView(textfield_, point);
   EXPECT_EQ(ui::DragDropTypes::DRAG_MOVE | ui::DragDropTypes::DRAG_COPY,
             operations);
-  textfield_->WriteDragDataForView(NULL, click_a.location(), &data);
+  textfield_->WriteDragDataForView(nullptr, point, &data);
   EXPECT_TRUE(data.GetString(&string));
   EXPECT_EQ(textfield_->GetSelectedText(), string);
-  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &custom_formats));
+  EXPECT_TRUE(textfield_->GetDropFormats(&formats, &format_types));
   EXPECT_EQ(ui::OSExchangeData::STRING, formats);
-  EXPECT_TRUE(custom_formats.empty());
+  EXPECT_TRUE(format_types.empty());
 
   // Drop " worl" after "h".
   EXPECT_TRUE(textfield_->CanDrop(data));
@@ -1371,12 +1542,10 @@ TEST_F(TextfieldTest, DragAndDrop_Canceled) {
   // Start dragging "worl".
   textfield_->SelectRange(gfx::Range(6, 10));
   gfx::Point point(GetCursorPositionX(8), 0);
-  ui::MouseEvent click(ui::ET_MOUSE_PRESSED, point, point,
-                       ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                       ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(click);
+  MoveMouseTo(point);
+  PressLeftMouseButton();
   ui::OSExchangeData data;
-  textfield_->WriteDragDataForView(NULL, click.location(), &data);
+  textfield_->WriteDragDataForView(nullptr, point, &data);
   EXPECT_TRUE(textfield_->CanDrop(data));
   // Drag the text over somewhere valid, outside the current selection.
   gfx::Point drop_point(GetCursorPositionX(2), 0);
@@ -1385,14 +1554,8 @@ TEST_F(TextfieldTest, DragAndDrop_Canceled) {
   EXPECT_EQ(ui::DragDropTypes::DRAG_MOVE, textfield_->OnDragUpdated(drop));
   // "Cancel" the drag, via move and release over the selection, and OnDragDone.
   gfx::Point drag_point(GetCursorPositionX(9), 0);
-  ui::MouseEvent drag(ui::ET_MOUSE_DRAGGED, drag_point, drag_point,
-                      ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON, 0);
-  ui::MouseEvent release(ui::ET_MOUSE_RELEASED, drag_point, drag_point,
-                         ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                         ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMouseDragged(drag);
-  textfield_->OnMouseReleased(release);
-  textfield_->OnDragDone();
+  DragMouseTo(drag_point);
+  ReleaseLeftMouseButton();
   EXPECT_EQ(ASCIIToUTF16("hello world"), textfield_->text());
 }
 
@@ -1728,6 +1891,17 @@ TEST_F(TextfieldTest, CutCopyPaste) {
   EXPECT_STR_EQ("", textfield_->text());
   EXPECT_EQ(ui::CLIPBOARD_TYPE_COPY_PASTE, GetAndResetCopiedToClipboard());
 
+  // Reset clipboard text.
+  SetClipboardText(ui::CLIPBOARD_TYPE_COPY_PASTE, "");
+
+  // Ensure [Shift]+[Delete] is a no-op in case there is no selection.
+  textfield_->SetText(ASCIIToUTF16("123"));
+  textfield_->SelectRange(gfx::Range(0));
+  SendAlternateCut();
+  EXPECT_STR_EQ("", GetClipboardText(ui::CLIPBOARD_TYPE_COPY_PASTE));
+  EXPECT_STR_EQ("123", textfield_->text());
+  EXPECT_EQ(ui::CLIPBOARD_TYPE_LAST, GetAndResetCopiedToClipboard());
+
   // Ensure IDS_APP_COPY copies.
   textfield_->SetText(ASCIIToUTF16("789"));
   textfield_->SelectAll(false);
@@ -1843,7 +2017,7 @@ TEST_F(TextfieldTest, TextCursorDisplayTest) {
 }
 
 TEST_F(TextfieldTest, TextCursorDisplayInRTLTest) {
-  std::string locale = l10n_util::GetApplicationLocale("");
+  std::string locale = base::i18n::GetConfiguredLocale();
   base::i18n::SetICUDefaultLocale("he");
 
   InitTextfield();
@@ -1990,7 +2164,7 @@ TEST_F(TextfieldTest, HitOutsideTextAreaTest) {
 }
 
 TEST_F(TextfieldTest, HitOutsideTextAreaInRTLTest) {
-  std::string locale = l10n_util::GetApplicationLocale("");
+  std::string locale = base::i18n::GetConfiguredLocale();
   base::i18n::SetICUDefaultLocale("he");
 
   InitTextfield();
@@ -2053,7 +2227,7 @@ TEST_F(TextfieldTest, OverflowTest) {
 }
 
 TEST_F(TextfieldTest, OverflowInRTLTest) {
-  std::string locale = l10n_util::GetApplicationLocale("");
+  std::string locale = base::i18n::GetConfiguredLocale();
   base::i18n::SetICUDefaultLocale("he");
 
   InitTextfield();
@@ -2087,11 +2261,11 @@ TEST_F(TextfieldTest, GetCompositionCharacterBoundsTest) {
   InitTextfield();
   ui::CompositionText composition;
   composition.text = UTF8ToUTF16("abc123");
-  const uint32 char_count = static_cast<uint32>(composition.text.length());
+  const uint32_t char_count = static_cast<uint32_t>(composition.text.length());
   ui::TextInputClient* client = textfield_;
 
   // Compare the composition character bounds with surrounding cursor bounds.
-  for (uint32 i = 0; i < char_count; ++i) {
+  for (uint32_t i = 0; i < char_count; ++i) {
     composition.selection = gfx::Range(i);
     client->SetCompositionText(composition);
     gfx::Point cursor_origin = GetCursorBounds().origin();
@@ -2139,8 +2313,7 @@ TEST_F(TextfieldTest, GetCompositionCharacterBounds_ComplexText) {
 
   // Make sure GetCompositionCharacterBounds never fails for index.
   gfx::Rect rects[kUtf16CharsCount];
-  gfx::Rect prev_cursor = GetCursorBounds();
-  for (uint32 i = 0; i < kUtf16CharsCount; ++i)
+  for (uint32_t i = 0; i < kUtf16CharsCount; ++i)
     EXPECT_TRUE(client->GetCompositionCharacterBounds(i, &rects[i]));
 
   // Here we might expect the following results but it actually depends on how
@@ -2166,16 +2339,12 @@ TEST_F(TextfieldTest, KeepInitiallySelectedWord) {
   MouseClick(middle_cursor, 0);
   const gfx::Point middle(middle_cursor.x(),
                           middle_cursor.y() + middle_cursor.height() / 2);
-  ui::MouseEvent press_event(ui::ET_MOUSE_PRESSED, middle, middle,
-                             ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON,
-                             ui::EF_LEFT_MOUSE_BUTTON);
-  textfield_->OnMousePressed(press_event);
+  MoveMouseTo(middle);
+  PressLeftMouseButton();
   EXPECT_EQ(gfx::Range(4, 7), textfield_->GetSelectedRange());
 
   // Drag the mouse to the beginning of the textfield.
-  ui::MouseEvent drag_event(ui::ET_MOUSE_DRAGGED, beginning, beginning,
-                            ui::EventTimeForNow(), ui::EF_LEFT_MOUSE_BUTTON, 0);
-  textfield_->OnMouseDragged(drag_event);
+  DragMouseTo(beginning);
   EXPECT_EQ(gfx::Range(7, 0), textfield_->GetSelectedRange());
 }
 
@@ -2376,14 +2545,6 @@ TEST_F(TextfieldTest, DestroyingTextfieldFromOnKeyEvent) {
 }
 
 class TextfieldTouchSelectionTest : public TextfieldTest {
- public:
-  // TextfieldTest:
-  void SetUp() override {
-    TextfieldTest::SetUp();
-    base::CommandLine::ForCurrentProcess()->AppendSwitch(
-        switches::kEnableTouchEditing);
-  }
-
  protected:
   // Simulates a complete tap.
   void Tap(const gfx::Point& point) {
@@ -2480,11 +2641,11 @@ TEST_F(TextfieldTouchSelectionTest, TouchSelectionInUnfocusableTextfield) {
 
   // Make textfield unfocusable and tap on it. Touch text selection should not
   // get activated.
-  textfield_->SetFocusable(false);
+  textfield_->SetFocusBehavior(View::FocusBehavior::NEVER);
   Tap(touch_point);
   EXPECT_FALSE(textfield_->HasFocus());
   EXPECT_FALSE(test_api_->touch_selection_controller());
-  textfield_->SetFocusable(true);
+  textfield_->SetFocusBehavior(View::FocusBehavior::ALWAYS);
 }
 
 // No touch on desktop Mac. Tracked in http://crbug.com/445520.
