@@ -9,9 +9,10 @@
 #include "base/strings/string_util.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_tree_host.h"
+#include "ui/display/display.h"
+#include "ui/display/screen.h"
 #include "ui/gfx/canvas.h"
 #include "ui/gfx/render_text.h"
-#include "ui/gfx/screen.h"
 #include "ui/gfx/text_elider.h"
 #include "ui/gfx/text_utils.h"
 #include "ui/native_theme/native_theme.h"
@@ -118,7 +119,7 @@ class TooltipAura::TooltipView : public views::View {
     render_text_->SetDisplayRect(gfx::Rect(0, 0, max_text_width, 100000));
   }
 
-  scoped_ptr<gfx::RenderText> render_text_;
+  std::unique_ptr<gfx::RenderText> render_text_;
   int max_width_;
 
   DISALLOW_COPY_AND_ASSIGN(TooltipView);
@@ -138,7 +139,7 @@ void TooltipAura::SetTooltipBounds(const gfx::Point& mouse_pos,
                                    const gfx::Size& tooltip_size) {
   gfx::Rect tooltip_rect(mouse_pos, tooltip_size);
   tooltip_rect.Offset(kCursorOffsetX, kCursorOffsetY);
-  gfx::Screen* screen = gfx::Screen::GetScreenFor(tooltip_window_);
+  display::Screen* screen = display::Screen::GetScreen();
   gfx::Rect display_bounds(screen->GetDisplayNearestPoint(mouse_pos).bounds());
 
   // If tooltip is out of bounds on the x axis, we simply shift it
@@ -165,9 +166,8 @@ void TooltipAura::DestroyWidget() {
   }
 }
 
-int TooltipAura::GetMaxWidth(const gfx::Point& location,
-                             aura::Window* context) const {
-  gfx::Screen* screen = gfx::Screen::GetScreenFor(context);
+int TooltipAura::GetMaxWidth(const gfx::Point& location) const {
+  display::Screen* screen = display::Screen::GetScreen();
   gfx::Rect display_bounds(screen->GetDisplayNearestPoint(location).bounds());
   return std::min(kTooltipMaxWidthPixels, (display_bounds.width() + 1) / 2);
 }
@@ -176,7 +176,7 @@ void TooltipAura::SetText(aura::Window* window,
                           const base::string16& tooltip_text,
                           const gfx::Point& location) {
   tooltip_window_ = window;
-  tooltip_view_->SetMaxWidth(GetMaxWidth(location, window));
+  tooltip_view_->SetMaxWidth(GetMaxWidth(location));
   tooltip_view_->SetText(tooltip_text);
 
   if (!widget_) {
