@@ -20,9 +20,6 @@ const int kPanelHorizMargin = 13;
 // Top or bottom margin.
 const int kPanelVertMargin = 13;
 
-// If some UI has some sub UI. Indent horizontally by the following value.
-const int kPanelHorizIndentation = 24;
-
 // When several controls are aligned vertically, the baseline should be spaced
 // by the following number of pixels.
 const int kPanelVerticalSpacing = 32;
