@@ -12,25 +12,25 @@ TypeConverter<mus::mojom::WindowType, views::Widget::InitParams::Type>::Convert(
     views::Widget::InitParams::Type type) {
   switch (type) {
     case views::Widget::InitParams::TYPE_WINDOW:
-      return mus::mojom::WINDOW_TYPE_WINDOW;
+      return mus::mojom::WindowType::WINDOW;
     case views::Widget::InitParams::TYPE_PANEL:
-      return mus::mojom::WINDOW_TYPE_PANEL;
+      return mus::mojom::WindowType::PANEL;
     case views::Widget::InitParams::TYPE_WINDOW_FRAMELESS:
-      return mus::mojom::WINDOW_TYPE_WINDOW_FRAMELESS;
+      return mus::mojom::WindowType::WINDOW_FRAMELESS;
     case views::Widget::InitParams::TYPE_CONTROL:
-      return mus::mojom::WINDOW_TYPE_CONTROL;
+      return mus::mojom::WindowType::CONTROL;
     case views::Widget::InitParams::TYPE_POPUP:
-      return mus::mojom::WINDOW_TYPE_POPUP;
+      return mus::mojom::WindowType::POPUP;
     case views::Widget::InitParams::TYPE_MENU:
-      return mus::mojom::WINDOW_TYPE_MENU;
+      return mus::mojom::WindowType::MENU;
     case views::Widget::InitParams::TYPE_TOOLTIP:
-      return mus::mojom::WINDOW_TYPE_TOOLTIP;
+      return mus::mojom::WindowType::TOOLTIP;
     case views::Widget::InitParams::TYPE_BUBBLE:
-      return mus::mojom::WINDOW_TYPE_BUBBLE;
+      return mus::mojom::WindowType::BUBBLE;
     case views::Widget::InitParams::TYPE_DRAG:
-      return mus::mojom::WINDOW_TYPE_DRAG;
+      return mus::mojom::WindowType::DRAG;
   }
-  return mus::mojom::WINDOW_TYPE_POPUP;
+  return mus::mojom::WindowType::POPUP;
 }
 
 }  // namespace mojo
