@@ -11,6 +11,8 @@
 
 namespace views {
 
+class LabelButton;
+
 // Skia port of the default button style used for dialogs on Chrome Mac.
 // Originally provided by ConstrainedWindowButton, which used Quartz-backed
 // Cocoa drawing routines.
@@ -19,6 +21,9 @@ class VIEWS_EXPORT DialogButtonBorderMac : public LabelButtonBorder {
   DialogButtonBorderMac();
   ~DialogButtonBorderMac() override;
 
+  // Whether the given |button| should get a highlighted background.
+  static bool ShouldRenderDefault(const LabelButton& button);
+
   // views::Border:
   void Paint(const View& view, gfx::Canvas* canvas) override;
   gfx::Size GetMinimumSize() const override;
