@@ -4,13 +4,14 @@
 
 #include "ui/views/widget/desktop_aura/desktop_drag_drop_client_aurax11.h"
 
+#include <X11/Xatom.h>
 #include <stddef.h>
 #include <stdint.h>
-#include <X11/Xatom.h>
 
 #include "base/event_types.h"
 #include "base/lazy_instance.h"
 #include "base/macros.h"
+#include "base/memory/ptr_util.h"
 #include "base/message_loop/message_loop.h"
 #include "base/metrics/histogram_macros.h"
 #include "third_party/skia/include/core/SkBitmap.h"
@@ -24,11 +25,11 @@
 #include "ui/base/x/selection_utils.h"
 #include "ui/base/x/x11_foreign_window_manager.h"
 #include "ui/base/x/x11_util.h"
+#include "ui/display/screen.h"
 #include "ui/events/event.h"
 #include "ui/events/event_utils.h"
 #include "ui/events/platform/platform_event_source.h"
 #include "ui/gfx/image/image_skia.h"
-#include "ui/gfx/screen.h"
 #include "ui/views/controls/image_view.h"
 #include "ui/views/widget/desktop_aura/desktop_native_cursor_manager.h"
 #include "ui/views/widget/desktop_aura/x11_topmost_window_finder.h"
@@ -127,7 +128,7 @@ const int kRepeatMouseMoveTimeoutMs = 350;
 const uint32_t kMinAlpha = 32;
 
 // |drag_widget_|'s opacity.
-const unsigned char kDragWidgetOpacity = 0xc0;
+const float kDragWidgetOpacity = .75f;
 
 static base::LazyInstance<
     std::map< ::Window, views::DesktopDragDropClientAuraX11*> >::Leaky
@@ -764,11 +765,15 @@ void DesktopDragDropClientAuraX11::OnWindowDestroyed(aura::Window* window) {
 void DesktopDragDropClientAuraX11::OnMouseMovement(
     const gfx::Point& screen_point,
     int flags,
-    base::TimeDelta event_time) {
+    base::TimeTicks event_time) {
   if (drag_widget_.get()) {
+    display::Display display =
+        display::Screen::GetScreen()->GetDisplayNearestWindow(
+            drag_widget_->GetNativeWindow());
+    gfx::Point scaled_point = gfx::ScaleToRoundedPoint(
+        screen_point, 1.f / display.device_scale_factor());
     drag_widget_->SetBounds(
-        gfx::Rect(screen_point - drag_widget_offset_,
-                  drag_widget_->GetWindowBoundsInScreen().size()));
+        gfx::Rect(scaled_point - drag_widget_offset_, drag_image_size_));
     drag_widget_->StackAtTop();
   }
 
@@ -780,7 +785,8 @@ void DesktopDragDropClientAuraX11::OnMouseMovement(
   current_modifier_state_ = flags & kModifiers;
 
   repeat_mouse_move_timer_.Stop();
-  ProcessMouseMove(screen_point, event_time.InMilliseconds());
+  ProcessMouseMove(screen_point,
+                   (event_time - base::TimeTicks()).InMilliseconds());
 }
 
 void DesktopDragDropClientAuraX11::OnMouseReleased() {
@@ -838,9 +844,9 @@ void DesktopDragDropClientAuraX11::OnMoveLoopEnded() {
   end_move_loop_timer_.Stop();
 }
 
-scoped_ptr<X11MoveLoop> DesktopDragDropClientAuraX11::CreateMoveLoop(
+std::unique_ptr<X11MoveLoop> DesktopDragDropClientAuraX11::CreateMoveLoop(
     X11MoveLoopDelegate* delegate) {
-  return make_scoped_ptr(new X11WholeScreenMoveLoop(this));
+  return base::WrapUnique(new X11WholeScreenMoveLoop(this));
 }
 
 XID DesktopDragDropClientAuraX11::FindWindowFor(
@@ -952,8 +958,8 @@ void DesktopDragDropClientAuraX11::EndMoveLoop() {
 
 void DesktopDragDropClientAuraX11::DragTranslate(
     const gfx::Point& root_window_location,
-    scoped_ptr<ui::OSExchangeData>* data,
-    scoped_ptr<ui::DropTargetEvent>* event,
+    std::unique_ptr<ui::OSExchangeData>* data,
+    std::unique_ptr<ui::DropTargetEvent>* event,
     aura::client::DragDropDelegate** delegate) {
   gfx::Point root_location = root_window_location;
   root_window_->GetHost()->ConvertPointFromNativeScreen(&root_location);
@@ -1065,8 +1071,8 @@ void DesktopDragDropClientAuraX11::CompleteXdndPosition(
     ::Window source_window,
     const gfx::Point& screen_point) {
   int drag_operation = ui::DragDropTypes::DRAG_NONE;
-  scoped_ptr<ui::OSExchangeData> data;
-  scoped_ptr<ui::DropTargetEvent> drop_target_event;
+  std::unique_ptr<ui::OSExchangeData> data;
+  std::unique_ptr<ui::DropTargetEvent> drop_target_event;
   DragDropDelegate* delegate = NULL;
   DragTranslate(screen_point, &data, &drop_target_event, &delegate);
   if (delegate)
@@ -1186,7 +1192,7 @@ void DesktopDragDropClientAuraX11::CreateDragWidget(
   params.ownership = Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
   params.accept_events = false;
 
-  gfx::Point location = gfx::Screen::GetNativeScreen()->GetCursorScreenPoint() -
+  gfx::Point location = display::Screen::GetScreen()->GetCursorScreenPoint() -
                         drag_widget_offset_;
   params.bounds = gfx::Rect(location, image.size());
   widget->set_focus_on_creation(false);
@@ -1195,9 +1201,10 @@ void DesktopDragDropClientAuraX11::CreateDragWidget(
   widget->SetOpacity(kDragWidgetOpacity);
   widget->GetNativeWindow()->SetName("DragWindow");
 
+  drag_image_size_ = image.size();
   ImageView* image_view = new ImageView();
   image_view->SetImage(image);
-  image_view->SetBounds(0, 0, image.width(), image.height());
+  image_view->SetBoundsRect(gfx::Rect(drag_image_size_));
   widget->SetContentsView(image_view);
   widget->Show();
   widget->GetNativeWindow()->layer()->SetFillsBoundsOpaquely(false);
