@@ -3,16 +3,21 @@
 // found in the LICENSE file.
 
 #include <map>
+#include <memory>
 #include <vector>
 
 // Include views_test_base.h first because the definition of None in X.h
 // conflicts with the definition of None in gtest-type-util.h
 #include "ui/views/test/views_test_base.h"
 
+#include "base/location.h"
 #include "base/macros.h"
-#include "base/memory/scoped_ptr.h"
+#include "base/memory/ptr_util.h"
 #include "base/run_loop.h"
+#include "base/single_thread_task_runner.h"
 #include "base/strings/utf_string_conversions.h"
+#include "base/threading/thread_task_runner_handle.h"
+#include "ui/aura/test/test_screen.h"
 #include "ui/aura/window.h"
 #include "ui/aura/window_tree_host.h"
 #include "ui/base/dragdrop/os_exchange_data.h"
@@ -117,9 +122,11 @@ class SimpleTestDragDropClient : public DesktopDragDropClientAuraX11 {
   // Returns true if the move loop is running.
   bool IsMoveLoopRunning();
 
+  Widget* drag_widget() { return DesktopDragDropClientAuraX11::drag_widget(); }
+
  private:
   // DesktopDragDropClientAuraX11:
-  scoped_ptr<X11MoveLoop> CreateMoveLoop(
+  std::unique_ptr<X11MoveLoop> CreateMoveLoop(
       X11MoveLoopDelegate* delegate) override;
   XID FindWindowFor(const gfx::Point& screen_point) override;
 
@@ -281,10 +288,10 @@ bool SimpleTestDragDropClient::IsMoveLoopRunning() {
   return loop_->IsRunning();
 }
 
-scoped_ptr<X11MoveLoop> SimpleTestDragDropClient::CreateMoveLoop(
+std::unique_ptr<X11MoveLoop> SimpleTestDragDropClient::CreateMoveLoop(
     X11MoveLoopDelegate* delegate) {
   loop_ = new TestMoveLoop(delegate);
-  return make_scoped_ptr(loop_);
+  return base::WrapUnique(loop_);
 }
 
 XID SimpleTestDragDropClient::FindWindowFor(const gfx::Point& screen_point) {
@@ -384,6 +391,11 @@ class DesktopDragDropClientAuraX11Test : public ViewsTestBase {
   int StartDragAndDrop() {
     ui::OSExchangeData data;
     data.SetString(base::ASCIIToUTF16("Test"));
+    SkBitmap drag_bitmap;
+    drag_bitmap.allocN32Pixels(10, 10);
+    drag_bitmap.eraseARGB(0xFF, 0, 0, 0);
+    gfx::ImageSkia drag_image(gfx::ImageSkia::CreateFrom1xBitmap(drag_bitmap));
+    data.provider().SetDragImage(drag_image, gfx::Vector2d());
 
     return client_->StartDragAndDrop(
         data,
@@ -397,6 +409,7 @@ class DesktopDragDropClientAuraX11Test : public ViewsTestBase {
   // ViewsTestBase:
   void SetUp() override {
     ViewsTestBase::SetUp();
+    views_delegate()->set_use_desktop_native_widgets(true);
 
     // Create widget to initiate the drags.
     widget_.reset(new Widget);
@@ -427,11 +440,11 @@ class DesktopDragDropClientAuraX11Test : public ViewsTestBase {
   }
 
  private:
-  scoped_ptr<TestDragDropClient> client_;
-  scoped_ptr<DesktopNativeCursorManager> cursor_manager_;
+  std::unique_ptr<TestDragDropClient> client_;
+  std::unique_ptr<DesktopNativeCursorManager> cursor_manager_;
 
   // The widget used to initiate drags.
-  scoped_ptr<Widget> widget_;
+  std::unique_ptr<Widget> widget_;
 
   DISALLOW_COPY_AND_ASSIGN(DesktopDragDropClientAuraX11Test);
 };
@@ -523,23 +536,62 @@ void BasicStep3(TestDragDropClient* client, XID toplevel) {
 TEST_F(DesktopDragDropClientAuraX11Test, Basic) {
   XID toplevel = 1;
 
-  base::MessageLoop::current()->PostTask(FROM_HERE,
-                                         base::Bind(&BasicStep2,
-                                         client(),
-                                         toplevel));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&BasicStep2, client(), toplevel));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_COPY, result);
 
   // Do another drag and drop to test that the data is properly cleaned up as a
   // result of the XdndFinished message.
-  base::MessageLoop::current()->PostTask(FROM_HERE,
-                                         base::Bind(&BasicStep3,
-                                         client(),
-                                         toplevel));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&BasicStep3, client(), toplevel));
   result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_COPY, result);
 }
 
+void HighDPIStep(TestDragDropClient* client) {
+  float scale =
+      display::Screen::GetScreen()->GetPrimaryDisplay().device_scale_factor();
+  // Start dragging at 100, 100 in native coordinates.
+  gfx::Point mouse_position_in_screen_pixel(100, 100);
+  client->OnMouseMovement(mouse_position_in_screen_pixel, 0,
+                          ui::EventTimeForNow());
+
+  EXPECT_EQ(gfx::ScaleToFlooredPoint(gfx::Point(100, 100), 1.f / scale),
+            client->drag_widget()->GetWindowBoundsInScreen().origin());
+
+  // Drag the mouse down 200 pixels.
+  mouse_position_in_screen_pixel.Offset(200, 0);
+  client->OnMouseMovement(mouse_position_in_screen_pixel, 0,
+                          ui::EventTimeForNow());
+  EXPECT_EQ(gfx::ScaleToFlooredPoint(gfx::Point(300, 100), 1.f / scale),
+            client->drag_widget()->GetWindowBoundsInScreen().origin());
+
+  client->OnMouseReleased();
+}
+
+TEST_F(DesktopDragDropClientAuraX11Test, HighDPI200) {
+  aura::TestScreen* screen =
+      static_cast<aura::TestScreen*>(display::Screen::GetScreen());
+  screen->SetDeviceScaleFactor(2.0f);
+
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&HighDPIStep, client()));
+  int result = StartDragAndDrop();
+  EXPECT_EQ(ui::DragDropTypes::DRAG_NONE, result);
+}
+
+TEST_F(DesktopDragDropClientAuraX11Test, HighDPI150) {
+  aura::TestScreen* screen =
+      static_cast<aura::TestScreen*>(display::Screen::GetScreen());
+  screen->SetDeviceScaleFactor(1.5f);
+
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&HighDPIStep, client()));
+  int result = StartDragAndDrop();
+  EXPECT_EQ(ui::DragDropTypes::DRAG_NONE, result);
+}
+
 namespace {
 
 void TargetDoesNotRespondStep2(TestDragDropClient* client) {
@@ -568,9 +620,8 @@ void TargetDoesNotRespondStep2(TestDragDropClient* client) {
 // DNDCollectionWindow is an example of an XdndAware target which does not
 // respond to XdndPosition messages at all.
 TEST_F(DesktopDragDropClientAuraX11Test, TargetDoesNotRespond) {
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&TargetDoesNotRespondStep2, client()));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&TargetDoesNotRespondStep2, client()));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_NONE, result);
 }
@@ -614,9 +665,8 @@ void QueuePositionStep2(TestDragDropClient* client) {
 // Test that XdndPosition messages are queued till the pending XdndPosition
 // message is acked via an XdndStatus message.
 TEST_F(DesktopDragDropClientAuraX11Test, QueuePosition) {
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&QueuePositionStep2, client()));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&QueuePositionStep2, client()));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_COPY, result);
 }
@@ -667,9 +717,8 @@ void TargetChangesStep2(TestDragDropClient* client) {
 
 // Test the behavior when the target changes during a drag.
 TEST_F(DesktopDragDropClientAuraX11Test, TargetChanges) {
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&TargetChangesStep2, client()));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&TargetChangesStep2, client()));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_COPY, result);
 }
@@ -737,16 +786,14 @@ void RejectAfterMouseReleaseStep3(TestDragDropClient* client) {
 // Test that the source sends XdndLeave instead of XdndDrop if the drag
 // operation is rejected after the mouse is released.
 TEST_F(DesktopDragDropClientAuraX11Test, RejectAfterMouseRelease) {
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&RejectAfterMouseReleaseStep2, client()));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&RejectAfterMouseReleaseStep2, client()));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_NONE, result);
 
   // Repeat the test but reject the drop in the XdndFinished message instead.
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&RejectAfterMouseReleaseStep3, client()));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&RejectAfterMouseReleaseStep3, client()));
   result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_NONE, result);
 }
@@ -869,11 +916,11 @@ class DesktopDragDropClientAuraX11ChromeSourceTargetTest
   }
 
  private:
-  scoped_ptr<SimpleTestDragDropClient> client_;
-  scoped_ptr<DesktopNativeCursorManager> cursor_manager_;
+  std::unique_ptr<SimpleTestDragDropClient> client_;
+  std::unique_ptr<DesktopNativeCursorManager> cursor_manager_;
 
   // The widget used to initiate drags.
-  scoped_ptr<Widget> widget_;
+  std::unique_ptr<Widget> widget_;
 
   DISALLOW_COPY_AND_ASSIGN(DesktopDragDropClientAuraX11ChromeSourceTargetTest);
 };
@@ -884,7 +931,7 @@ void ChromeSourceTargetStep2(SimpleTestDragDropClient* client,
                              int modifier_flags) {
   EXPECT_TRUE(client->IsMoveLoopRunning());
 
-  scoped_ptr<Widget> target_widget(new Widget);
+  std::unique_ptr<Widget> target_widget(new Widget);
   Widget::InitParams target_params(Widget::InitParams::TYPE_WINDOW_FRAMELESS);
   target_params.ownership = Widget::InitParams::WIDGET_OWNS_NATIVE_WIDGET;
   target_params.native_widget =
@@ -893,8 +940,9 @@ void ChromeSourceTargetStep2(SimpleTestDragDropClient* client,
   target_widget->Init(target_params);
   target_widget->Show();
 
-  TestDragDropDelegate* delegate = new TestDragDropDelegate;
-  aura::client::SetDragDropDelegate(target_widget->GetNativeWindow(), delegate);
+  std::unique_ptr<TestDragDropDelegate> delegate(new TestDragDropDelegate);
+  aura::client::SetDragDropDelegate(target_widget->GetNativeWindow(),
+                                    delegate.get());
 
   client->SetTopmostXWindow(
       target_widget->GetNativeView()->GetHost()->GetAcceleratedWidget());
@@ -944,9 +992,8 @@ void ChromeSourceTargetStep2(SimpleTestDragDropClient* client,
 }  // namespace
 
 TEST_F(DesktopDragDropClientAuraX11ChromeSourceTargetTest, Basic) {
-  base::MessageLoop::current()->PostTask(
-      FROM_HERE,
-      base::Bind(&ChromeSourceTargetStep2, client(), ui::EF_NONE));
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
+      FROM_HERE, base::Bind(&ChromeSourceTargetStep2, client(), ui::EF_NONE));
   int result = StartDragAndDrop();
   EXPECT_EQ(ui::DragDropTypes::DRAG_COPY, result);
 }
@@ -954,7 +1001,7 @@ TEST_F(DesktopDragDropClientAuraX11ChromeSourceTargetTest, Basic) {
 // Test that if 'Ctrl' is pressed during a drag and drop operation, that
 // the aura::client::DragDropDelegate is properly notified.
 TEST_F(DesktopDragDropClientAuraX11ChromeSourceTargetTest, CtrlPressed) {
-  base::MessageLoop::current()->PostTask(
+  base::ThreadTaskRunnerHandle::Get()->PostTask(
       FROM_HERE,
       base::Bind(&ChromeSourceTargetStep2, client(), ui::EF_CONTROL_DOWN));
   int result = StartDragAndDrop();
