@@ -71,7 +71,7 @@ class WEB_DIALOGS_EXPORT WebDialogWebContentsDelegate
   // Weak pointer.  Always an original profile.
   content::BrowserContext* browser_context_;
 
-  scoped_ptr<WebContentsHandler> handler_;
+  std::unique_ptr<WebContentsHandler> handler_;
 
   DISALLOW_COPY_AND_ASSIGN(WebDialogWebContentsDelegate);
 };
