@@ -34,7 +34,6 @@
 #include "ui/gfx/geometry/vector2d.h"
 #include "ui/gfx/geometry/vector3d_f.h"
 #include "ui/gfx/interpolated_transform.h"
-#include "ui/gfx/screen.h"
 #include "ui/wm/core/window_util.h"
 #include "ui/wm/core/wm_core_switches.h"
 #include "ui/wm/public/animation_host.h"
@@ -85,7 +84,7 @@ class HidingWindowAnimationObserverBase : public aura::WindowObserver {
   // activation change does not put the window above the animating
   // layer.
   void DetachAndRecreateLayers() {
-    layer_owner_ = RecreateLayers(window_);
+    layer_owner_ = RecreateLayers(window_, nullptr);
     if (window_->parent()) {
       const aura::Window::Windows& transient_children =
           GetTransientChildren(window_);
@@ -104,6 +103,11 @@ class HidingWindowAnimationObserverBase : public aura::WindowObserver {
             layer_owner_->root(), topmost_transient_child->layer());
       }
     }
+    // Reset the transform for the |window_|. Because the animation may have
+    // changed the transform, when recreating the layers we need to reset the
+    // transform otherwise the recreated layer has the transform installed
+    // for the animation.
+    window_->layer()->SetTransform(gfx::Transform());
   }
 
  protected:
@@ -134,7 +138,7 @@ class HidingWindowAnimationObserverBase : public aura::WindowObserver {
   aura::Window* window_;
 
   // The owner of detached layers.
-  scoped_ptr<ui::LayerTreeOwner> layer_owner_;
+  std::unique_ptr<ui::LayerTreeOwner> layer_owner_;
 
   DISALLOW_COPY_AND_ASSIGN(HidingWindowAnimationObserverBase);
 };
@@ -349,23 +353,22 @@ void AnimateHideWindow_Fade(aura::Window* window) {
 
 ui::LayerAnimationElement* CreateGrowShrinkElement(
     aura::Window* window, bool grow) {
-  scoped_ptr<ui::InterpolatedTransform> scale(new ui::InterpolatedScale(
-      gfx::Point3F(kWindowAnimation_Bounce_Scale,
-                   kWindowAnimation_Bounce_Scale,
-                   1),
-      gfx::Point3F(1, 1, 1)));
-  scoped_ptr<ui::InterpolatedTransform> scale_about_pivot(
+  std::unique_ptr<ui::InterpolatedTransform> scale(
+      new ui::InterpolatedScale(gfx::Point3F(kWindowAnimation_Bounce_Scale,
+                                             kWindowAnimation_Bounce_Scale, 1),
+                                gfx::Point3F(1, 1, 1)));
+  std::unique_ptr<ui::InterpolatedTransform> scale_about_pivot(
       new ui::InterpolatedTransformAboutPivot(
           gfx::Point(window->bounds().width() * 0.5,
                      window->bounds().height() * 0.5),
           scale.release()));
   scale_about_pivot->SetReversed(grow);
-  scoped_ptr<ui::LayerAnimationElement> transition(
+  std::unique_ptr<ui::LayerAnimationElement> transition(
       ui::LayerAnimationElement::CreateInterpolatedTransformElement(
           scale_about_pivot.release(),
           base::TimeDelta::FromMilliseconds(
               kWindowAnimation_Bounce_DurationMS *
-                  kWindowAnimation_Bounce_GrowShrinkDurationPercent / 100)));
+              kWindowAnimation_Bounce_GrowShrinkDurationPercent / 100)));
   transition->set_tween_type(grow ? gfx::Tween::EASE_OUT : gfx::Tween::EASE_IN);
   return transition.release();
 }
@@ -375,7 +378,7 @@ void AnimateBounce(aura::Window* window) {
       window->layer()->GetAnimator());
   scoped_settings.SetPreemptionStrategy(
       ui::LayerAnimator::REPLACE_QUEUED_ANIMATIONS);
-  scoped_ptr<ui::LayerAnimationSequence> sequence(
+  std::unique_ptr<ui::LayerAnimationSequence> sequence(
       new ui::LayerAnimationSequence);
   sequence->AddElement(CreateGrowShrinkElement(window, true));
   sequence->AddElement(ui::LayerAnimationElement::CreatePauseElement(
@@ -433,7 +436,7 @@ void AddLayerAnimationsForRotate(aura::Window* window, bool show) {
         duration * (100 - kWindowAnimation_Rotate_OpacityDurationPercent) / 100,
         ui::LayerAnimationElement::OPACITY);
   }
-  scoped_ptr<ui::LayerAnimationElement> opacity(
+  std::unique_ptr<ui::LayerAnimationElement> opacity(
       ui::LayerAnimationElement::CreateOpacityElement(
           show ? kWindowAnimation_ShowOpacity : kWindowAnimation_HideOpacity,
           duration * kWindowAnimation_Rotate_OpacityDurationPercent / 100));
@@ -447,30 +450,30 @@ void AddLayerAnimationsForRotate(aura::Window* window, bool show) {
   transform.Translate(xcenter, 0);
   transform.ApplyPerspectiveDepth(kWindowAnimation_Rotate_PerspectiveDepth);
   transform.Translate(-xcenter, 0);
-  scoped_ptr<ui::InterpolatedTransform> perspective(
+  std::unique_ptr<ui::InterpolatedTransform> perspective(
       new ui::InterpolatedConstantTransform(transform));
 
-  scoped_ptr<ui::InterpolatedTransform> scale(
+  std::unique_ptr<ui::InterpolatedTransform> scale(
       new ui::InterpolatedScale(1, kWindowAnimation_Rotate_ScaleFactor));
-  scoped_ptr<ui::InterpolatedTransform> scale_about_pivot(
+  std::unique_ptr<ui::InterpolatedTransform> scale_about_pivot(
       new ui::InterpolatedTransformAboutPivot(
           gfx::Point(xcenter, kWindowAnimation_Rotate_TranslateY),
           scale.release()));
 
-  scoped_ptr<ui::InterpolatedTransform> translation(
+  std::unique_ptr<ui::InterpolatedTransform> translation(
       new ui::InterpolatedTranslation(
           gfx::PointF(), gfx::PointF(0, kWindowAnimation_Rotate_TranslateY)));
 
-  scoped_ptr<ui::InterpolatedTransform> rotation(
-      new ui::InterpolatedAxisAngleRotation(
-          gfx::Vector3dF(1, 0, 0), 0, kWindowAnimation_Rotate_DegreesX));
+  std::unique_ptr<ui::InterpolatedTransform> rotation(
+      new ui::InterpolatedAxisAngleRotation(gfx::Vector3dF(1, 0, 0), 0,
+                                            kWindowAnimation_Rotate_DegreesX));
 
   scale_about_pivot->SetChild(perspective.release());
   translation->SetChild(scale_about_pivot.release());
   rotation->SetChild(translation.release());
   rotation->SetReversed(show);
 
-  scoped_ptr<ui::LayerAnimationElement> transition(
+  std::unique_ptr<ui::LayerAnimationElement> transition(
       ui::LayerAnimationElement::CreateInterpolatedTransformElement(
           rotation.release(), duration));
   ui::LayerAnimationSequence* last_sequence =
