@@ -11,6 +11,7 @@ const char kAboutBlankURL[] = "about:blank";
 const char kAboutScheme[] = "about";
 const char kBlobScheme[] = "blob";
 const char kContentScheme[] = "content";
+const char kContentIDScheme[] = "cid";
 const char kDataScheme[] = "data";
 const char kFileScheme[] = "file";
 const char kFileSystemScheme[] = "filesystem";
@@ -25,4 +26,6 @@ const char kWssScheme[] = "wss";
 
 const char kStandardSchemeSeparator[] = "://";
 
+const size_t kMaxURLChars = 2 * 1024 * 1024;
+
 }  // namespace url
