@@ -66,6 +66,7 @@ QQuickWebEngineDownloadItemPrivate::QQuickWebEngineDownloadItemPrivate(QQuickWeb
     , downloadId(-1)
     , downloadState(QQuickWebEngineDownloadItem::DownloadCancelled)
     , savePageFormat(QQuickWebEngineDownloadItem::UnknownSaveFormat)
+    , type(QQuickWebEngineDownloadItem::Attachment)
     , totalBytes(-1)
     , receivedBytes(0)
 {
@@ -184,6 +185,8 @@ quint32 QQuickWebEngineDownloadItem::id() const
             Download is in progress.
     \value  WebEngineDownloadItem.DownloadCompleted
             Download completed successfully.
+    \value  WebEngineDownloadItem.DownloadCancelled
+            Download was cancelled by the user.
     \value  WebEngineDownloadItem.DownloadInterrupted
             Download has been interrupted (by the server or because of lost connectivity).
 */
@@ -299,6 +302,32 @@ void QQuickWebEngineDownloadItem::setSavePageFormat(QQuickWebEngineDownloadItem:
     }
 }
 
+/*!
+    \qmlproperty enumeration WebEngineDownloadItem::type
+    \readonly
+    \since QtWebEngine 1.4
+
+    Describes the requested download's type.
+
+    \value WebEngineDownloadItem.Attachment The web server's response includes a
+           \c Content-Disposition header with the \c attachment directive. If \c Content-Disposition
+           is present in the reply, the web server is indicating that the client should prompt the
+           user to save the content regardless of the content type.
+           See \l {RFC 2616 section 19.5.1} for details.
+    \value WebEngineDownloadItem.DownloadAttribute The user clicked a link with the \c download
+           attribute. See \l {HTML download attribute} for details.
+    \value WebEngineDownloadItem.UserRequested The user initiated the download, for example by
+           selecting a web action.
+    \value WebEngineDownloadItem.SavePage Saving of the current page was requested (for example by
+           the \l{WebEngineView::WebAction}{WebEngineView.SavePage} web action).
+ */
+
+QQuickWebEngineDownloadItem::DownloadType QQuickWebEngineDownloadItem::type() const
+{
+    Q_D(const QQuickWebEngineDownloadItem);
+    return d->type;
+}
+
 QQuickWebEngineDownloadItem::QQuickWebEngineDownloadItem(QQuickWebEngineDownloadItemPrivate *p, QObject *parent)
     : QObject(parent)
     , d_ptr(p)
