@@ -47,6 +47,7 @@
 #include "favicon_manager.h"
 #include "file_picker_controller.h"
 #include "javascript_dialog_controller.h"
+#include "pdfium_document_wrapper_qt.h"
 #include "qwebenginefullscreenrequest.h"
 #include "qwebenginehistory.h"
 #include "qwebenginehistory_p.h"
@@ -79,6 +80,9 @@
 #include <QMenu>
 #include <QMessageBox>
 #include <QMimeData>
+#ifndef QT_NO_PRINTER
+#include <QPrinter>
+#endif
 #include <QStandardPaths>
 #include <QStyle>
 #include <QTimer>
@@ -90,6 +94,89 @@ QT_BEGIN_NAMESPACE
 
 using namespace QtWebEngineCore;
 
+static const int MaxTooltipLength = 1024;
+
+#ifndef QT_NO_PRINTER
+#if defined(ENABLE_PDF)
+static bool printPdfDataOnPrinter(const QByteArray& data, QPrinter& printer)
+{
+    QRect printerPageRect = printer.pageRect();
+    PdfiumDocumentWrapperQt pdfiumWrapper(data.constData(), data.size(), printerPageRect.size());
+
+    int toPage = printer.toPage();
+    int fromPage = printer.fromPage();
+    bool ascendingOrder = true;
+
+    if (fromPage == 0 && toPage == 0) {
+        fromPage = 1;
+        toPage = pdfiumWrapper.pageCount();
+    }
+    fromPage = qMax(1, fromPage);
+    toPage = qMin(pdfiumWrapper.pageCount(), toPage);
+
+    if (printer.pageOrder() == QPrinter::LastPageFirst) {
+        qSwap(fromPage, toPage);
+        ascendingOrder = false;
+    }
+
+    int pageCopies = 1;
+    int documentCopies = 1;
+
+    if (!printer.supportsMultipleCopies())
+        documentCopies = printer.copyCount();
+
+    if (printer.collateCopies()) {
+        pageCopies = documentCopies;
+        documentCopies = 1;
+    }
+
+    QPainter painter;
+    if (!painter.begin(&printer)) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+        qWarning("Failure to print on printer %ls: Could not open printer for painting.", qUtf16Printable(printer.printerName()));
+#else
+        qWarning("Failure to print on printer %s: Could not open printer for painting.", qPrintable(printer.printerName()));
+#endif
+        return false;
+    }
+
+    for (int printedDocuments = 0; printedDocuments < documentCopies; printedDocuments++) {
+        int currentPageIndex = fromPage;
+        while (true) {
+            for (int printedPages = 0; printedPages < pageCopies; printedPages++) {
+                if (printer.printerState() == QPrinter::Aborted
+                        || printer.printerState() == QPrinter::Error)
+                    return false;
+
+                QImage currentImage = pdfiumWrapper.pageAsQImage(currentPageIndex - 1);
+                if (currentImage.isNull())
+                    return false;
+
+                painter.drawImage(printerPageRect, currentImage, currentImage.rect());
+                if (printedPages < pageCopies - 1)
+                    printer.newPage();
+            }
+
+            if (currentPageIndex == toPage)
+                break;
+
+            if (ascendingOrder)
+                currentPageIndex++;
+            else
+                currentPageIndex--;
+
+            printer.newPage();
+        }
+        if (printedDocuments < documentCopies - 1)
+            printer.newPage();
+    }
+    painter.end();
+
+    return true;
+}
+#endif // defined(ENABLE_PDF)
+#endif // QT_NO_PRINTER
+
 static QWebEnginePage::WebWindowType toWindowType(WebContentsAdapterClient::WindowOpenDisposition disposition)
 {
     switch (disposition) {
@@ -140,6 +227,9 @@ QWebEnginePagePrivate::QWebEnginePagePrivate(QWebEngineProfile *_profile)
     , fullscreenMode(false)
     , webChannel(nullptr)
     , webChannelWorldId(QWebEngineScript::MainWorld)
+#ifndef QT_NO_PRINTER
+    , currentPrinter(nullptr)
+#endif
 {
     memset(actions, 0, sizeof(actions));
 }
@@ -152,7 +242,16 @@ QWebEnginePagePrivate::~QWebEnginePagePrivate()
 
 RenderWidgetHostViewQtDelegate *QWebEnginePagePrivate::CreateRenderWidgetHostViewQtDelegate(RenderWidgetHostViewQtDelegateClient *client)
 {
-    return new RenderWidgetHostViewQtDelegateWidget(client);
+    // Set the QWebEngineView as the parent for a popup delegate, so that the new popup window
+    // responds properly to clicks in case the QWebEngineView is inside a modal QDialog. Setting the
+    // parent essentially notifies the OS that the popup window is part of the modal session, and
+    // should allow interaction.
+    // The new delegate will not be deleted by the parent view though, because we unset the parent
+    // when the parent is destroyed. The delegate will be destroyed by Chromium when the popup is
+    // dismissed.
+    // If the delegate is not for a popup, but for a newly created QWebEngineView, the parent is 0
+    // just like before.
+    return new RenderWidgetHostViewQtDelegateWidget(client, this->view);
 }
 
 void QWebEnginePagePrivate::titleChanged(const QString &title)
@@ -378,7 +477,26 @@ void QWebEnginePagePrivate::didFindText(quint64 requestId, int matchCount)
 
 void QWebEnginePagePrivate::didPrintPage(quint64 requestId, const QByteArray &result)
 {
+#if defined(ENABLE_PDF)
+#ifndef QT_NO_PRINTER
+    // If no currentPrinter is set that means that were printing to PDF only.
+    if (!currentPrinter) {
+        m_callbacks.invoke(requestId, result);
+        return;
+    }
+
+    bool printerResult = printPdfDataOnPrinter(result, *currentPrinter);
+
+    m_callbacks.invoke(requestId, printerResult);
+    currentPrinter = nullptr;
+#else // If print support is disabled, only PDF printing is available.
     m_callbacks.invoke(requestId, result);
+#endif // ifndef QT_NO_PRINTER
+#else // defined(ENABLE_PDF)
+    // we should never enter this branch, but just for safe-keeping...
+    Q_UNUSED(result);
+    m_callbacks.invoke(requestId, QByteArray());
+#endif // defined(ENABLE_PDF)
 }
 
 void QWebEnginePagePrivate::passOnFocus(bool reverse)
@@ -480,6 +598,9 @@ void QWebEnginePagePrivate::updateAction(QWebEnginePage::WebAction action) const
     case QWebEnginePage::ReloadAndBypassCache:
         enabled = !isLoading;
         break;
+    case QWebEnginePage::ViewSource:
+        enabled = adapter->canViewSource();
+        break;
     default:
         break;
     }
@@ -495,6 +616,7 @@ void QWebEnginePagePrivate::updateNavigationActions()
     updateAction(QWebEnginePage::Stop);
     updateAction(QWebEnginePage::Reload);
     updateAction(QWebEnginePage::ReloadAndBypassCache);
+    updateAction(QWebEnginePage::ViewSource);
 }
 
 #ifndef QT_NO_ACTION
@@ -778,6 +900,29 @@ void QWebEnginePage::setBackgroundColor(const QColor &color)
     d->adapter->backgroundColorChanged();
 }
 
+/*!
+ * Save the currently loaded web page to disk.
+ *
+ * The web page is saved to \a filePath in the specified \a{format}.
+ *
+ * This is a short cut for the following actions:
+ * \list
+ *   \li Trigger the Save web action.
+ *   \li Accept the next download item and set the specified file path and save format.
+ * \endlist
+ *
+ * This function issues an asynchronous download request for the web page and returns immediately.
+ *
+ * \sa QWebEngineDownloadItem::SavePageFormat
+ * \since 5.8
+ */
+void QWebEnginePage::save(const QString &filePath,
+                          QWebEngineDownloadItem::SavePageFormat format) const
+{
+    Q_D(const QWebEnginePage);
+    d->adapter->save(filePath, format);
+}
+
 /*!
     \property QWebEnginePage::audioMuted
     \brief whether the current page audio is muted.
@@ -963,6 +1108,9 @@ QAction *QWebEnginePage::action(WebAction action) const
     case SavePage:
         text = tr("Save &Page");
         break;
+    case ViewSource:
+        text = tr("&View Page Source");
+        break;
     case NoWebAction:
     case WebActionCount:
         Q_UNREACHABLE();
@@ -1027,122 +1175,122 @@ void QWebEnginePage::triggerAction(WebAction action, bool)
         d->adapter->unselect();
         break;
     case OpenLinkInThisWindow:
-        if (menuData.linkUrl.isValid())
-            setUrl(menuData.linkUrl);
+        if (menuData.linkUrl().isValid())
+            setUrl(menuData.linkUrl());
         break;
     case OpenLinkInNewWindow:
-        if (menuData.linkUrl.isValid()) {
+        if (menuData.linkUrl().isValid()) {
             QWebEnginePage *newPage = createWindow(WebBrowserWindow);
             if (newPage)
-                newPage->setUrl(menuData.linkUrl);
+                newPage->setUrl(menuData.linkUrl());
         }
         break;
     case OpenLinkInNewTab:
-        if (menuData.linkUrl.isValid()) {
+        if (menuData.linkUrl().isValid()) {
             QWebEnginePage *newPage = createWindow(WebBrowserTab);
             if (newPage)
-                newPage->setUrl(menuData.linkUrl);
+                newPage->setUrl(menuData.linkUrl());
         }
         break;
     case OpenLinkInNewBackgroundTab:
-        if (menuData.linkUrl.isValid()) {
+        if (menuData.linkUrl().isValid()) {
             QWebEnginePage *newPage = createWindow(WebBrowserBackgroundTab);
             if (newPage)
-                newPage->setUrl(menuData.linkUrl);
+                newPage->setUrl(menuData.linkUrl());
         }
         break;
     case CopyLinkToClipboard:
-        if (menuData.linkUrl.isValid()) {
-            QString urlString = menuData.linkUrl.toString(QUrl::FullyEncoded);
-            QString title = menuData.linkText.toHtmlEscaped();
+        if (menuData.linkUrl().isValid()) {
+            QString urlString = menuData.linkUrl().toString(QUrl::FullyEncoded);
+            QString title = menuData.linkText().toHtmlEscaped();
             QMimeData *data = new QMimeData();
             data->setText(urlString);
             QString html = QStringLiteral("<a href=\"") + urlString + QStringLiteral("\">") + title + QStringLiteral("</a>");
             data->setHtml(html);
-            data->setUrls(QList<QUrl>() << menuData.linkUrl);
+            data->setUrls(QList<QUrl>() << menuData.linkUrl());
             qApp->clipboard()->setMimeData(data);
         }
         break;
     case DownloadLinkToDisk:
-        if (menuData.linkUrl.isValid())
-            d->adapter->download(menuData.linkUrl, menuData.suggestedFileName);
+        if (menuData.linkUrl().isValid())
+            d->adapter->download(menuData.linkUrl(), menuData.suggestedFileName());
         break;
     case CopyImageToClipboard:
-        if (menuData.hasImageContent &&
-                (menuData.mediaType == WebEngineContextMenuData::MediaTypeImage ||
-                 menuData.mediaType == WebEngineContextMenuData::MediaTypeCanvas))
+        if (menuData.hasImageContent() &&
+                (menuData.mediaType() == WebEngineContextMenuData::MediaTypeImage ||
+                 menuData.mediaType() == WebEngineContextMenuData::MediaTypeCanvas))
         {
-            d->adapter->copyImageAt(menuData.pos);
+            d->adapter->copyImageAt(menuData.position());
         }
         break;
     case CopyImageUrlToClipboard:
-        if (menuData.mediaUrl.isValid() && menuData.mediaType == WebEngineContextMenuData::MediaTypeImage) {
-            QString urlString = menuData.mediaUrl.toString(QUrl::FullyEncoded);
-            QString title = menuData.linkText;
+        if (menuData.mediaUrl().isValid() && menuData.mediaType() == WebEngineContextMenuData::MediaTypeImage) {
+            QString urlString = menuData.mediaUrl().toString(QUrl::FullyEncoded);
+            QString title = menuData.linkText();
             if (!title.isEmpty())
                 title = QStringLiteral(" alt=\"%1\"").arg(title.toHtmlEscaped());
             QMimeData *data = new QMimeData();
             data->setText(urlString);
             QString html = QStringLiteral("<img src=\"") + urlString + QStringLiteral("\"") + title + QStringLiteral("></img>");
             data->setHtml(html);
-            data->setUrls(QList<QUrl>() << menuData.mediaUrl);
+            data->setUrls(QList<QUrl>() << menuData.mediaUrl());
             qApp->clipboard()->setMimeData(data);
         }
         break;
     case DownloadImageToDisk:
     case DownloadMediaToDisk:
-        if (menuData.mediaUrl.isValid())
-            d->adapter->download(menuData.mediaUrl, menuData.suggestedFileName);
+        if (menuData.mediaUrl().isValid())
+            d->adapter->download(menuData.mediaUrl(), menuData.suggestedFileName());
         break;
     case CopyMediaUrlToClipboard:
-        if (menuData.mediaUrl.isValid() &&
-                (menuData.mediaType == WebEngineContextMenuData::MediaTypeAudio ||
-                 menuData.mediaType == WebEngineContextMenuData::MediaTypeVideo))
+        if (menuData.mediaUrl().isValid() &&
+                (menuData.mediaType() == WebEngineContextMenuData::MediaTypeAudio ||
+                 menuData.mediaType() == WebEngineContextMenuData::MediaTypeVideo))
         {
-            QString urlString = menuData.mediaUrl.toString(QUrl::FullyEncoded);
+            QString urlString = menuData.mediaUrl().toString(QUrl::FullyEncoded);
             QMimeData *data = new QMimeData();
             data->setText(urlString);
-            if (menuData.mediaType == WebEngineContextMenuData::MediaTypeAudio)
+            if (menuData.mediaType() == WebEngineContextMenuData::MediaTypeAudio)
                 data->setHtml(QStringLiteral("<audio src=\"") + urlString + QStringLiteral("\"></audio>"));
             else
                 data->setHtml(QStringLiteral("<video src=\"") + urlString + QStringLiteral("\"></video>"));
-            data->setUrls(QList<QUrl>() << menuData.mediaUrl);
+            data->setUrls(QList<QUrl>() << menuData.mediaUrl());
             qApp->clipboard()->setMimeData(data);
         }
         break;
     case ToggleMediaControls:
-        if (menuData.mediaUrl.isValid() && menuData.mediaFlags & WebEngineContextMenuData::MediaCanToggleControls) {
-            bool enable = !(menuData.mediaFlags & WebEngineContextMenuData::MediaControls);
-            d->adapter->executeMediaPlayerActionAt(menuData.pos, WebContentsAdapter::MediaPlayerControls, enable);
+        if (menuData.mediaUrl().isValid() && menuData.mediaFlags() & WebEngineContextMenuData::MediaCanToggleControls) {
+            bool enable = !(menuData.mediaFlags() & WebEngineContextMenuData::MediaControls);
+            d->adapter->executeMediaPlayerActionAt(menuData.position(), WebContentsAdapter::MediaPlayerControls, enable);
         }
         break;
     case ToggleMediaLoop:
-        if (menuData.mediaUrl.isValid() &&
-                (menuData.mediaType == WebEngineContextMenuData::MediaTypeAudio ||
-                 menuData.mediaType == WebEngineContextMenuData::MediaTypeVideo))
+        if (menuData.mediaUrl().isValid() &&
+                (menuData.mediaType() == WebEngineContextMenuData::MediaTypeAudio ||
+                 menuData.mediaType() == WebEngineContextMenuData::MediaTypeVideo))
         {
-            bool enable = !(menuData.mediaFlags & WebEngineContextMenuData::MediaLoop);
-            d->adapter->executeMediaPlayerActionAt(menuData.pos, WebContentsAdapter::MediaPlayerLoop, enable);
+            bool enable = !(menuData.mediaFlags() & WebEngineContextMenuData::MediaLoop);
+            d->adapter->executeMediaPlayerActionAt(menuData.position(), WebContentsAdapter::MediaPlayerLoop, enable);
         }
         break;
     case ToggleMediaPlayPause:
-        if (menuData.mediaUrl.isValid() &&
-                (menuData.mediaType == WebEngineContextMenuData::MediaTypeAudio ||
-                 menuData.mediaType == WebEngineContextMenuData::MediaTypeVideo))
+        if (menuData.mediaUrl().isValid() &&
+                (menuData.mediaType() == WebEngineContextMenuData::MediaTypeAudio ||
+                 menuData.mediaType() == WebEngineContextMenuData::MediaTypeVideo))
         {
-            bool enable = (menuData.mediaFlags & WebEngineContextMenuData::MediaPaused);
-            d->adapter->executeMediaPlayerActionAt(menuData.pos, WebContentsAdapter::MediaPlayerPlay, enable);
+            bool enable = (menuData.mediaFlags() & WebEngineContextMenuData::MediaPaused);
+            d->adapter->executeMediaPlayerActionAt(menuData.position(), WebContentsAdapter::MediaPlayerPlay, enable);
         }
         break;
     case ToggleMediaMute:
-        if (menuData.mediaUrl.isValid() && menuData.mediaFlags & WebEngineContextMenuData::MediaHasAudio) {
+        if (menuData.mediaUrl().isValid() && menuData.mediaFlags() & WebEngineContextMenuData::MediaHasAudio) {
             // Make sure to negate the value, so that toggling actually works.
-            bool enable = !(menuData.mediaFlags & WebEngineContextMenuData::MediaMuted);
-            d->adapter->executeMediaPlayerActionAt(menuData.pos, WebContentsAdapter::MediaPlayerMute, enable);
+            bool enable = !(menuData.mediaFlags() & WebEngineContextMenuData::MediaMuted);
+            d->adapter->executeMediaPlayerActionAt(menuData.position(), WebContentsAdapter::MediaPlayerMute, enable);
         }
         break;
     case InspectElement:
-        d->adapter->inspectElementAt(menuData.pos);
+        d->adapter->inspectElementAt(menuData.position());
         break;
     case ExitFullScreen:
         d->adapter->exitFullScreen();
@@ -1153,6 +1301,17 @@ void QWebEnginePage::triggerAction(WebAction action, bool)
     case SavePage:
         d->adapter->save();
         break;
+    case ViewSource:
+        // This is a workaround to make the ViewSource action working in a context menu.
+        // The WebContentsAdapter::viewSource() method deletes a
+        // RenderWidgetHostViewQtDelegateWidget instance which passes the control to the event
+        // loop. If the QMenu::aboutToHide() signal is connected to the QObject::deleteLater()
+        // slot the QMenu is deleted by the event handler while the ViewSource action is still not
+        // completed. This may lead to a crash. To avoid this the WebContentsAdapter::viewSource()
+        // method is called indirectly via the QTimer::singleShot() function which schedules the
+        // the viewSource() call after the QMenu's destruction.
+        QTimer::singleShot(0, this, [d](){ d->adapter->viewSource(); });
+        break;
     case NoWebAction:
         break;
     case WebActionCount:
@@ -1161,6 +1320,22 @@ void QWebEnginePage::triggerAction(WebAction action, bool)
     }
 }
 
+/*!
+ * \since 5.8
+ * Replace the current misspelled word with \a replacement.
+ *
+ * The current misspelled word can be found in QWebEngineContextMenuData::misspelledWord(),
+ * and suggested replacements in QWebEngineContextMenuData::spellCheckerSuggestions().
+ *
+ * \sa contextMenuData(),
+ */
+
+void QWebEnginePage::replaceMisspelledWord(const QString &replacement)
+{
+    Q_D(QWebEnginePage);
+    d->adapter->replaceMisspelling(replacement);
+}
+
 void QWebEnginePage::findText(const QString &subString, FindFlags options, const QWebEngineCallback<bool> &resultCallback)
 {
     Q_D(QWebEnginePage);
@@ -1197,7 +1372,7 @@ bool QWebEnginePagePrivate::contextMenuRequested(const WebEngineContextMenuData
         return false;
 
     contextData.reset();
-    QContextMenuEvent event(QContextMenuEvent::Mouse, data.pos, view->mapToGlobal(data.pos));
+    QContextMenuEvent event(QContextMenuEvent::Mouse, data.position(), view->mapToGlobal(data.position()));
     switch (view->contextMenuPolicy()) {
     case Qt::PreventContextMenu:
         return false;
@@ -1207,7 +1382,7 @@ bool QWebEnginePagePrivate::contextMenuRequested(const WebEngineContextMenuData
         break;
     case Qt::CustomContextMenu:
         contextData = data;
-        Q_EMIT view->customContextMenuRequested(data.pos);
+        Q_EMIT view->customContextMenuRequested(data.position());
         break;
     case Qt::ActionsContextMenu:
         if (view->actions().count()) {
@@ -1342,6 +1517,16 @@ bool QWebEnginePagePrivate::isEnabled() const
     return true;
 }
 
+void QWebEnginePagePrivate::setToolTip(const QString &toolTipText)
+{
+    if (view) {
+        QString wrappedTip;
+        if (!toolTipText.isEmpty())
+             wrappedTip = QLatin1String("<p>") % toolTipText.toHtmlEscaped().left(MaxTooltipLength) % QLatin1String("</p>");
+        view->setToolTip(wrappedTip);
+    }
+}
+
 QMenu *QWebEnginePage::createStandardContextMenu()
 {
     Q_D(QWebEnginePage);
@@ -1352,13 +1537,25 @@ QMenu *QWebEnginePage::createStandardContextMenu()
     QAction *action = 0;
     const WebEngineContextMenuData &contextMenuData = *d->contextData.d;
 
-    if (!contextMenuData.linkText.isEmpty() && contextMenuData.linkUrl.isValid()) {
+    if (contextMenuData.isEditable() && !contextMenuData.spellCheckerSuggestions().isEmpty()) {
+        QPointer<QWebEnginePage> thisRef(this);
+        for (int i=0; i < contextMenuData.spellCheckerSuggestions().count() && i < 4; i++) {
+            QAction *action = new QAction(menu);
+            QString replacement = contextMenuData.spellCheckerSuggestions().at(i);
+            QObject::connect(action, &QAction::triggered, [thisRef, replacement] { if (thisRef) thisRef->replaceMisspelledWord(replacement); });
+            action->setText(replacement);
+            menu->addAction(action);
+        }
+        menu->addSeparator();
+    }
+
+    if (!contextMenuData.linkText().isEmpty() && contextMenuData.linkUrl().isValid()) {
         action = QWebEnginePage::action(OpenLinkInThisWindow);
         action->setText(tr("Follow Link"));
         menu->addAction(action);
         menu->addAction(QWebEnginePage::action(DownloadLinkToDisk));
     }
-    if (contextMenuData.selectedText.isEmpty()) {
+    if (contextMenuData.selectedText().isEmpty()) {
         action = new QAction(QIcon::fromTheme(QStringLiteral("go-previous")), tr("&Back"), menu);
         connect(action, &QAction::triggered, d->view, &QWebEngineView::back);
         action->setEnabled(d->adapter->canGoBack());
@@ -1372,16 +1569,18 @@ QMenu *QWebEnginePage::createStandardContextMenu()
         action = new QAction(QIcon::fromTheme(QStringLiteral("view-refresh")), tr("&Reload"), menu);
         connect(action, &QAction::triggered, d->view, &QWebEngineView::reload);
         menu->addAction(action);
+
+        menu->addAction(QWebEnginePage::action(ViewSource));
     } else {
         menu->addAction(QWebEnginePage::action(Copy));
         menu->addAction(QWebEnginePage::action(Unselect));
     }
 
-    if (!contextMenuData.linkText.isEmpty() && contextMenuData.linkUrl.isValid()) {
+    if (!contextMenuData.linkText().isEmpty() && contextMenuData.linkUrl().isValid()) {
         menu->addAction(QWebEnginePage::action(CopyLinkToClipboard));
     }
-    if (contextMenuData.mediaUrl.isValid()) {
-        switch (contextMenuData.mediaType) {
+    if (contextMenuData.mediaUrl().isValid()) {
+        switch (contextMenuData.mediaType()) {
         case WebEngineContextMenuData::MediaTypeImage:
             menu->addAction(QWebEnginePage::action(DownloadImageToDisk));
             menu->addAction(QWebEnginePage::action(CopyImageUrlToClipboard));
@@ -1396,15 +1595,15 @@ QMenu *QWebEnginePage::createStandardContextMenu()
             menu->addAction(QWebEnginePage::action(CopyMediaUrlToClipboard));
             menu->addAction(QWebEnginePage::action(ToggleMediaPlayPause));
             menu->addAction(QWebEnginePage::action(ToggleMediaLoop));
-            if (contextMenuData.mediaFlags & WebEngineContextMenuData::MediaHasAudio)
+            if (contextMenuData.mediaFlags() & WebEngineContextMenuData::MediaHasAudio)
                 menu->addAction(QWebEnginePage::action(ToggleMediaMute));
-            if (contextMenuData.mediaFlags & WebEngineContextMenuData::MediaCanToggleControls)
+            if (contextMenuData.mediaFlags() & WebEngineContextMenuData::MediaCanToggleControls)
                 menu->addAction(QWebEnginePage::action(ToggleMediaControls));
             break;
         default:
             break;
         }
-    } else if (contextMenuData.mediaType == WebEngineContextMenuData::MediaTypeCanvas) {
+    } else if (contextMenuData.mediaType() == WebEngineContextMenuData::MediaTypeCanvas) {
         menu->addAction(QWebEnginePage::action(CopyImageToClipboard));
     }
 
@@ -1462,7 +1661,7 @@ static inline QWebEnginePage::FileSelectionMode toPublic(FilePickerController::F
     return static_cast<QWebEnginePage::FileSelectionMode>(mode);
 }
 
-void QWebEnginePagePrivate::runFileChooser(FilePickerController *controller)
+void QWebEnginePagePrivate::runFileChooser(QSharedPointer<FilePickerController> controller)
 {
     Q_Q(QWebEnginePage);
 
@@ -1472,8 +1671,6 @@ void QWebEnginePagePrivate::runFileChooser(FilePickerController *controller)
         controller->accepted(selectedFileNames);
     else
         controller->rejected();
-
-    delete controller;
 }
 
 WebEngineSettings *QWebEnginePagePrivate::webEngineSettings() const
@@ -1747,8 +1944,23 @@ QSizeF QWebEnginePage::contentsSize() const
 */
 void QWebEnginePage::printToPdf(const QString &filePath, const QPageLayout &pageLayout)
 {
+#if defined(ENABLE_PDF)
     Q_D(const QWebEnginePage);
+#ifndef QT_NO_PRINTER
+    if (d->currentPrinter) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+        qWarning("Cannot print to PDF while at the same time printing on printer %ls", qUtf16Printable(d->currentPrinter->printerName()));
+#else
+        qWarning("Cannot print to PDF while at the same time printing on printer %s", qPrintable(d->currentPrinter->printerName()));
+#endif
+        return;
+    }
+#endif
     d->adapter->printToPDF(pageLayout, filePath);
+#else
+    Q_UNUSED(filePath);
+    Q_UNUSED(pageLayout);
+#endif // if defined(ENABLE_PDF)
 }
 
 
@@ -1766,10 +1978,63 @@ void QWebEnginePage::printToPdf(const QString &filePath, const QPageLayout &page
 void QWebEnginePage::printToPdf(const QWebEngineCallback<const QByteArray&> &resultCallback, const QPageLayout &pageLayout)
 {
     Q_D(QWebEnginePage);
+#if defined(ENABLE_PDF)
+#ifndef QT_NO_PRINTER
+    if (d->currentPrinter) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+        qWarning("Cannot print to PDF while at the same time printing on printer %ls", qUtf16Printable(d->currentPrinter->printerName()));
+#else
+        qWarning("Cannot print to PDF while at the same time printing on printer %s", qPrintable(d->currentPrinter->printerName()));
+#endif
+        d->m_callbacks.invokeEmpty(resultCallback);
+        return;
+    }
+#endif // ifndef QT_NO_PRINTER
     quint64 requestId = d->adapter->printToPDFCallbackResult(pageLayout);
     d->m_callbacks.registerCallback(requestId, resultCallback);
+#else // if defined(ENABLE_PDF)
+    Q_UNUSED(pageLayout);
+    d->m_callbacks.invokeEmpty(resultCallback);
+#endif // if defined(ENABLE_PDF)
 }
 
+#ifndef QT_NO_PRINTER
+/*!
+    \fn void QWebEnginePage::print(QPrinter *printer, FunctorOrLambda resultCallback)
+    Renders the current content of the page into a temporary PDF document, then prints it using \a printer.
+
+    The settings for creating and printing the PDF document will be retrieved from the \a printer
+    object.
+    It is the users responsibility to ensure the \a printer remains valid until \a resultCallback
+    has been called.
+
+    The \a resultCallback must take a boolean as parameter. If printing was successful, this
+    boolean will have the value \c true, otherwise, its value will be \c false.
+    \since 5.8
+*/
+void QWebEnginePage::print(QPrinter *printer, const QWebEngineCallback<bool> &resultCallback)
+{
+    Q_D(QWebEnginePage);
+#if defined(ENABLE_PDF)
+    if (d->currentPrinter) {
+#if QT_VERSION >= QT_VERSION_CHECK(5, 7, 0)
+        qWarning("Cannot print page on printer %ls: Already printing on %ls.", qUtf16Printable(printer->printerName()), qUtf16Printable(d->currentPrinter->printerName()));
+#else
+        qWarning("Cannot print page on printer %s: Already printing on %s.", qPrintable(printer->printerName()), qPrintable(d->currentPrinter->printerName()));
+#endif
+        d->m_callbacks.invokeDirectly(resultCallback, false);
+        return;
+    }
+    d->currentPrinter = printer;
+    quint64 requestId = d->adapter->printToPDFCallbackResult(printer->pageLayout(), printer->colorMode() == QPrinter::Color);
+    d->m_callbacks.registerCallback(requestId, resultCallback);
+#else // if defined(ENABLE_PDF)
+    Q_UNUSED(printer);
+    d->m_callbacks.invokeDirectly(resultCallback, false);
+#endif // if defined(ENABLE_PDF)
+}
+#endif // QT_NO_PRINTER
+
 /*!
     \since 5.7
 
