@@ -96,6 +96,11 @@ using QtWebEngineCore::BrowserContextAdapter;
     A QWebEngineUrlSchemeHandler can be registered for a profile by installUrlSchemeHandler()
     to add support for custom URL schemes. Requests for the scheme are then issued to
     QWebEngineUrlSchemeHandler::requestStarted() as QWebEngineUrlRequestJob objects.
+
+    Spellchecking HTML form fields can be enabled per profile by using the setSpellCheckEnabled()
+    method and the current languages used for spellchecking can be set by using the
+    setSpellCheckLanguages() method.
+
 */
 
 /*!
@@ -180,10 +185,12 @@ void QWebEngineProfilePrivate::downloadRequested(DownloadItemInfo &info)
     Q_ASSERT(!m_ongoingDownloads.contains(info.id));
     QWebEngineDownloadItemPrivate *itemPrivate = new QWebEngineDownloadItemPrivate(this, info.url);
     itemPrivate->downloadId = info.id;
-    itemPrivate->downloadState = QWebEngineDownloadItem::DownloadRequested;
+    itemPrivate->downloadState = info.accepted ? QWebEngineDownloadItem::DownloadInProgress
+                                               : QWebEngineDownloadItem::DownloadRequested;
     itemPrivate->downloadPath = info.path;
     itemPrivate->mimeType = info.mimeType;
     itemPrivate->savePageFormat = static_cast<QWebEngineDownloadItem::SavePageFormat>(info.savePageFormat);
+    itemPrivate->type = static_cast<QWebEngineDownloadItem::DownloadType>(info.downloadType);
 
     QWebEngineDownloadItem *download = new QWebEngineDownloadItem(itemPrivate, q);
 
@@ -555,6 +562,87 @@ QWebEngineProfile *QWebEngineProfile::defaultProfile()
     return profile;
 }
 
+/*!
+    \since 5.8
+
+    Sets the current list of \a languages for the spell checker.
+    Each language should match the name of the \c .bdic dictionary.
+    For example, the language \c en-US will load the \c en-US.bdic
+    dictionary file.
+
+    Qt WebEngine checks for the \c qtwebengine_dictionaries subdirectory
+    first in the local directory and if it is not found, in the Qt
+    installation directory.
+
+    On macOS, depending on how Qt WebEngine is configured at build time, there are two possibilities
+    how spellchecking data is found:
+
+    \list
+        \li Hunspell dictionaries (default) - .bdic dictionaries are used, just like on other
+            platforms
+        \li Native dictionaries - the macOS spellchecking APIs are used (which means the results
+            will depend on the installed OS dictionaries)
+    \endlist
+
+    Thus, in the macOS Hunspell case, Qt WebEngine will look in the \e qtwebengine_dictionaries
+    subdirectory located inside the application bundle \c Resources directory, and also in the
+    \c Resources directory located inside the Qt framework bundle.
+
+    To summarize, in case of Hunspell usage, the following paths are considered:
+
+    \list
+        \li QCoreApplication::applicationDirPath()/qtwebengine_dictionaries
+            or QCoreApplication::applicationDirPath()/../Contents/Resources/qtwebengine_dictionaries
+            (on macOS)
+        \li [QLibraryInfo::DataPath]/qtwebengine_dictionaries
+            or path/to/QtWebEngineCore.framework/Resources/qtwebengine_dictionaries (Qt framework
+            bundle on macOS)
+    \endlist
+
+    For more information about how to compile \c .bdic dictionaries, see the
+    \l{WebEngine Widgets Spellchecker Example}{Spellchecker Example}.
+
+*/
+void QWebEngineProfile::setSpellCheckLanguages(const QStringList &languages)
+{
+    Q_D(QWebEngineProfile);
+    d->browserContext()->setSpellCheckLanguages(languages);
+}
+
+/*!
+    \since 5.8
+
+    Returns the list of languages used by the spell checker.
+*/
+QStringList QWebEngineProfile::spellCheckLanguages() const
+{
+    const Q_D(QWebEngineProfile);
+    return d->browserContext()->spellCheckLanguages();
+}
+
+/*!
+    \since 5.8
+
+    Enables spell checker if \a enable is \c true, otherwise disables it.
+    \sa isSpellCheckEnabled()
+ */
+void QWebEngineProfile::setSpellCheckEnabled(bool enable)
+{
+     Q_D(QWebEngineProfile);
+     d->browserContext()->setSpellCheckEnabled(enable);
+}
+/*!
+    \since 5.8
+
+    Returns \c true if the spell checker is enabled; otherwise returns \c false.
+    \sa setSpellCheckEnabled()
+ */
+bool QWebEngineProfile::isSpellCheckEnabled() const
+{
+     const Q_D(QWebEngineProfile);
+     return d->browserContext()->isSpellCheckEnabled();
+}
+
 /*!
     Returns the default settings for all pages in this profile.
 */
