@@ -89,6 +89,13 @@ static WebEngineSettings::Attribute toWebEngineAttribute(QWebEngineSettings::Web
         return WebEngineSettings::AutoLoadIconsForPage;
     case QWebEngineSettings::TouchIconsEnabled:
         return WebEngineSettings::TouchIconsEnabled;
+    case QWebEngineSettings::FocusOnNavigationEnabled:
+        return WebEngineSettings::FocusOnNavigationEnabled;
+    case QWebEngineSettings::PrintElementBackgrounds:
+        return WebEngineSettings::PrintElementBackgrounds;
+    case QWebEngineSettings::AllowRunningInsecureContent:
+        return WebEngineSettings::AllowRunningInsecureContent;
+
     default:
         return WebEngineSettings::UnsupportedInCoreSettings;
     }
